
(in-package :om)

(defclass  fifo ()
   ((head :initform (list nil) :accessor head :initarg :head :type list)
    (tail  :initform nil :accessor tail :initarg :tail :type list) 
    (fifo-length :initform 0 :accessor fifo-length :initarg :fifo-length :type integer) 
    (duration :initform 0 :accessor duration :initarg :duration :type integer)
    (lock  :initform (make-lock 'fifo-lock)  :accessor lock :initarg :lock :type ccl::lock)))

(defmethod initialize-instance ((self fifo) &rest initargs)
  (declare (ignore initargs))
  (call-next-method)
  (setf (slot-value self 'tail) (slot-value self 'head))
  self)

(defmethod fifo-in ((self fifo) (object t))
   (with-lock-grabbed ((lock self))
     (setf (rest (tail self)) (cons object nil)
           (tail self) (rest (tail self)))
     (incf (fifo-length self))
     (incf (duration self) (fifo-obj-duration object))))


(defmethod fifo-out ((self fifo) )
   (with-lock-grabbed ((lock self))
     (if (null (rest (head self)))
       nil
       (let ((object  (first (rest (head self)))))
         (prog1 object
           (setf (rest (head self)) (cddr (head self)))
           (unless (rest (head self)) (setf (tail self) (head self)))
           (decf (fifo-length self) )
           (decf (duration self) (fifo-obj-duration object)))))))


(defmethod fifo-empty ((self fifo))
   (with-lock-grabbed ((lock self))
     (setf (head self) (list nil) (tail self) (head self) (fifo-length self) 0 (duration self) 0)))


(defmethod fifo-size ((self fifo))
   (with-lock-grabbed ((lock self))
     (fifo-length self)))


(defmethod fifo-obj-duration ((self crossevent))
  (duration self))

(defmethod fifo-obj-duration ((self t))
  0)

(defmethod duration ((self fifo))
  (duration self))

