;;; ============================================
;;; Fichier 1-4-Voicings.lisp / 10 octobre 2004
;;; ============================================
;;; NewMarcSubsti.lisp

(in-package :midishare)

;;; 1-1-Steedman.lisp

;;; En general, grilles de 4 mesures sauf le blues de 12 mesures
;;; MARQUEUR /
;;; -> sert a controler la profondeur d'application des regles de division
;;; (accord divisable = encadre par deux /)
;;; Table d'arrangement: unite de duree accord = 1 demi-interbarre (/ X X /)
;;; -> accords = 1 demi-interbarre, ou 2 demi-interbarres = interbarre (interbarre = pseudo-"mesure")

;;; *nbeatsaccord* = nbre de beats par unite de duree grille (DEMI-BARRE)
;;; "Rain" -> beat = 1 demi-interbarre -> *nbeatsaccord* = 1
;;; "Goodbye Pork Pie Hat" -> idem (2 beats par / X / = demi-mesure) -> *nbeatsaccord* = 1

;;; ATTENTION: ne gere pas les enharmonies (Ex: g# et non ab)
;;; -> (c c# d eb e f f# g g# a bb b)



;;; ===========================================
;;; CALCUL AUTOMATIQUE D'UNE TABLE / 29 septembre 2004
;;; ===========================================
;;; Table d'arrangement avec 2 registres:
;;; - registre1 = C-C#-D-Eb-E: renversement avec 3ce en haut
;;; - registre2 = F-F#-G-Ab-A-Bb-B: renversement avec 7eme en haut

;;; Durees des accords = 1 ou 2 X valmin
;;; *nbeatsaccord* = nbre de beats par valmin
;;; valmin = (ancien nb-demi-interbarre) = plus petite duree d'accords substitues
;;; valmin = 1 => duree = 484 ms
;;; valmin = 2 => duree = 968 ms
;;; (la duree est indifferente car dans Max midijoue met a l'echelle en fonction de la duree totale)

;;; il faut un exemplaire des accords suivants:
;;; accord 7, registre1      -> Rain = C#7
;;; accord 7, registre2      -> Rain = G#7, B7
;;; accord Maj, registre1    -> Rain = C#
;;; accord Maj, registre2    -> Rain = F#
;;; accord m7, registre1     -> Rain = G#m7
;;; accord m7, registre2     -> Rain = C#m7

;;; et les stocker en 2 copies pour chaque duree 1 ou 2 X valmin

;;; MAX: captevoicing affiche un accord, et envoie son numero a OM
;;; OM: capte le dernier accord joue, et le met dans la table, avec l'etiquette correspondant au numero

(setf ms::*accord* ())

(in-package :om)


(defmethod ajoute-voicing ((self improvizer) accord)  ;;; ATTENTION: RAJOUTER ACCORD CANAL 14
  (let* ((lastEvent (otext self (maxEtat self)))
         (LMidic (CrossEvt->LMidic lastEvent))
         (LMidi (mapcar #'(lambda (x) (/ x 100)) LMidic))
         (mididata1 (list (list LMidi)
                          (list 0)                                               ;onset
                          (list (make-list (length LMidi) :initial-element (- 484 5))) ;duration (-5ms pour NotOff)
                          (list (make-list (length LMidi) :initial-element 80))  ;velo
                          (list (make-list (length LMidi) :initial-element 0)))) ;canal
         (mididata2 (list (list LMidi)
                          (list 0)                                               ;onset
                          (list (make-list (length LMidi) :initial-element (- 968 5))) ;duration
                          (list (make-list (length LMidi) :initial-element 80))  ;velo
                          (list (make-list (length LMidi) :initial-element 0))))) ;canal
(format t "mididata1 = ~a" mididata1)
    (setf ms::*table-arrangements* (cons (list (list (list accord 1))  ;;; attention: liste d'accords
                                               mididata1) ms::*table-arrangements*))
    (setf ms::*table-arrangements* (cons (list (list (list accord 2)) 
                                               mididata2) ms::*table-arrangements*))
))
    


(in-package :midishare)

;;; VOIR FONCTION GENERIQUE POUR RECEPTIONNER ACCORDS ENVOYES PAR MAX
;;; NewMarc.lisp


;;; format table:
#|
((((c# 7) 2))
         (((1 37 70 63 59 58 53) (37) (65) (37) (68)) (0 242 605 726 847)
          ((968 242 484 968 968 968 968) (484) (233) (242) (121))
          ((3 97 69 69 69 69 69) (97) (69) (97) (69)) ((13 1 0 0 0 0 0) (1) (0) (1) (0))))

|#


;;; 6-Voicings.lisp
;;; (traduc-deg p) = lettre c, c# etc. associee a un midi pith de 0 a 11
;;; (ajoute-chiffre deg v) = couple (deg chif) selon velo v 1 = Maj, 2 = m7, 3 = 7


(defmethod save-voicings-as ()
  (catch-cancel
    (let* ((name (choose-new-file-dialog :prompt "Save Voicing Table As...")))
      (when name
        (WITH-OPEN-FILE (out name :direction :output  :if-does-not-exist :create :if-exists :supersede
                             :external-format :|TEXT|)

          (prin1 '(in-package :ms) out)
          (print `(setf ms::*table-arrangements*
                        ',ms::*table-arrangements*) out))))))

(defmethod load-voicings ()
  (catch-cancel
    (let ((name (choose-file-dialog)))
      (when name
        (WITH-OPEN-FILE (in name :direction :input  :if-does-not-exist :nil)
          (eval (read in)) (eval (read in)))))))

;(length *table-arrangements*)

