;;; ===========================================
;;; Fichier 1-2-Arrange.lisp / 18 mai 2003
;;; ===========================================
;;; Creation des donnees midi piano-bass:
;;; - grille harmo substituee + traduite en donnees midi (table d'arrangement de "Rain")
;;; Utilise 4 oracles en generation, d'apres la grille substituee
;;; (bip quand l'oracle ne trouve pas de beat associe a l'accord: fonction ImprovizeOnHarmGrid)

(in-package :midishare)

(setf *grilletmp*  ())

;;; cht duplique-beat 14/9/04
(defun creer-piano-bass ()
  (setf *grilletmp* (reecriture *grille* *k*))   ;;; *grille* definie dans les fonctions "cree"
  (when *blues?* (setf *grilletmp* (forcer-septieme  *grilletmp*)))
  (setf *piano-bass0* (virtual-boogie *grilletmp* *table-arrangements* *duree-mes*)) ;;; *piano-bass0* -> duree beat = 469 (table Rain)
  (setf *grilletmporacle* (enleve-marque* (duplique-beat *grilletmp* *nbeatsaccord*)))
  (setf *piano-bass1* (genere-oracle *oracle1*))                                     ;;; *piano-bass1*, etc. -> duree beat = *dureebeat*
  (setf *piano-bass2* (genere-oracle *oracle2*))
  (setf *piano-bass3* (genere-oracle *oracle3*)) 
  (setf *piano-bass4* (genere-oracle *oracle4*))
  (format t "~%Oracles contribuant a la generation: ~a"
          (append (if (null *piano-bass1*) () (list '*oracle1*))
                  (if (null *piano-bass2*) () (list '*oracle2*))
                  (if (null *piano-bass3*) () (list '*oracle3*))
                  (if (null *piano-bass4*) () (list '*oracle4*))))
  (list *piano-bass0* *piano-bass1* *piano-bass2* *piano-bass3* *piano-bass4*)   ;;; -> les 2 tempi differents ("Rain"=469, oracle=*dureebeat*) sont synchronises dans Max  
)    


(defun genere-oracle (oracle)
  (let ((res ()))
    (when (> (om::NbEvent? oracle) 0)
      (progn (setf res (om::ImprovizeOnHarmGrid oracle (length *grilletmporacle*) *grilletmporacle*))
             (setf res (om::clone-and-scale-beats res *dureebeat*))
             (setf res (om::beats-check-sustain res *dureebeat* 30))
             (setf res (om::beats->5list res *dureebeat* 0)))
      res)))





;;; determine-durees -> Donne liste de couples (accord nb-interbarre) avec eventuellement marques *
;;; Ex "Rain": (((e 7) 1) ((b m7) 1) (((e *) 7) 1) ((bb m7) 1) ((eb 7) 1) ((g# m7) 2) ((c# 7) 2) ((f#) 2) ((g# 7) 2) ((b 7) 2) ((bb 7) 2))


;; 14/9/04
(defun duplique-beat (grille nbeatsaccord)
  (loop for l in (determine-durees grille)
        append (loop for i from 1 to (* nbeatsaccord (cadr l)) collect (car l))))

(defun enleve-marque* (grille)
  (mapcar #'(lambda (x) (list (creer-accord (sans-marque x) (accord-chiffre x)))) grille))

(setf *blues?* nil)

(defun forcer-septieme (grille)
  (let ((res ()))
    (do ((l grille (cdr l)))
        ((null l) (reverse res))
      (setf res (cons (if (and (listp (car l)) (null (cdr (car l))))  ;;; Maj
                        (append (car l) (list 7))
                        (car l))
                      res)))))

(setq *k* 10)


;;; ========================================================
;;; Arrangement d'une grille par choix de motifs midi piano-bass
;;; associes aux accords:
;;; - 1 bloc, ou
;;; - 2 blocs avec un meme accord
;;; ========================================================

(defun virtual-boogie (grille table-arrangements duree-mes) 
  (let* ((grille-avec-dur (determine-durees grille))
         (liste-blocs (arrange grille-avec-dur () (mapcar #'car table-arrangements) table-arrangements duree-mes))
         (l (met-intermittence liste-blocs)))
    (list (fait-haut l) (fait-onset l) (fait-dur l) (fait-vel l) (fait-canal l))))

;;; Recherche du plus long prefixe de la grille qui correspond a une sequence de la table

(defun arrange (grille pref liste-seq table-arrangements duree-mes)
  (cond ((null grille) (list (creer-bloc (reverse pref) table-arrangements duree-mes)))
        (t (let ((new-seq (elague (car grille) liste-seq)))
             (if (null new-seq)
               (cons (creer-bloc (reverse pref) table-arrangements duree-mes)
                     (arrange grille () (mapcar #'car table-arrangements) table-arrangements duree-mes))
               (arrange (cdr grille) (cons (car grille) pref) new-seq table-arrangements duree-mes))))))

;;; Supprime des mots non-facteurs au fur et a mesure qu'on avance vers le plus long facteur

(defun elague (acc liste-seq)
  (cond ((null liste-seq) ())
        ((test-cond? acc (caar liste-seq))
         (cons (cdr (car liste-seq))
               (elague acc (cdr liste-seq))))
        (t (elague acc (cdr liste-seq)))))

;;; ========================================================
;;; Structure de bloc = donnees midi correspondant a une sequence de la table
;;; 5 listes : haut, onset, dur, vel, canal (utilisees dans OpenMusic)
;;; ========================================================

(defstruct bloc (accords) (haut) (onset) (dur) (vel) (canal) (length))

;;; ========================================================
;;; Cree un bloc vide quand un accord est le meme que le bloc precedent
;;; (aucune donnee midi, sauf accords sur canal OM=13 Max=14)
;;; ========================================================

(defun met-intermittence (liste-blocs)
  (let ((prec (bloc-accords (car liste-blocs))) (res (list (car liste-blocs))))
    (do ((l (cdr liste-blocs) (cdr l)))
        ((null l) (reverse res))
      (setf res (cons (if (equal (bloc-accords (car l)) prec) (vider-bloc (car l)) (car l)) res))
      (setf prec (bloc-accords (car l))))))

(defun vider-bloc (m)
  (setf (bloc-haut m) (vider-midi (bloc-haut m) (bloc-canal m))) 
  (setf (bloc-onset m) (vider-onset (bloc-onset m) (bloc-canal m))) 
  (setf (bloc-dur m) (vider-midi (bloc-dur m) (bloc-canal m))) 
  (setf (bloc-vel m) (vider-midi (bloc-vel m) (bloc-canal m))) 
  (setf (bloc-canal m) (vider-midi (bloc-canal m) (bloc-canal m)))
  m)

(defun vider-midi (l lcan)
  (remove () (mapcar #'(lambda (l1 l1can)
                         (remove () (mapcar #'(lambda (x c) (if (= c 13) x ())) 
                                            l1 l1can)))
                     l lcan)))

(defun vider-onset (lonset lcan)
  (remove () (mapcar #'(lambda (o l1can)
                         (if (member 13 l1can) o ()))
                     lonset lcan)))

;;; ========================================================
;;; Suppression des / en determinant les durees des accords
;;;  -> liste de couples (accord nb-demi-interbarre)
;;; nb-demi-interbarre = 1 ou 2 (interbarre = pseudo-"mesure")
;;; ========================================================

(defun determine-durees (grille)
  (cond ((null grille) ())
        ((eq (car grille) '/) (determine-durees (cdr grille)))
        (t (let ((mesure (decoupe grille)))
             (append (etiquette-durees mesure) 
                     (determine-durees (nthcdr (length mesure) grille)))))))

(defun decoupe (grille)
  (if (or (null grille) (eq (car grille) '/)) () 
      (cons (car grille) (decoupe (cdr grille)))))

(defun etiquette-durees (mesure)
  (cond ((null (cdr mesure)) (list (list (car mesure) 2)))
        ((eq (sans-marque (car mesure)) (sans-marque (cadr mesure)))
         (list (list (cadr mesure) 2)))
        (t (list (list (car mesure) 1) (list (cadr mesure) 1)))))

(defun compter-mesures (l)
  (cond ((or (null l) (and (null (cdr l)) (eq (car l) '/))) 1)
        ((eq (car l) '/) (+ 1 (compter-mesures (cdr l))))
        (t (compter-mesures (cdr l)))))

;;; ========================================================
;;; Choix des donnees midi associees aux accords (1 bloc, ou 
;;; 2 blocs avec un meme accord)
;;; Le choix depend de deux registres de degres:
;;; (c c# d eb e) et (f f# g g# a bb b)
;;; ========================================================

(defun creer-bloc (liste table-arrangements duree-mes) 
  (let* ((res (tirage-motif liste table-arrangements)) (clef (car res)) (cinq-listes (cadr res))
         (m (make-bloc :accords liste
                       :haut (adapte-haut  (car cinq-listes) (cadr cinq-listes) liste clef duree-mes)
                       :onset (cadr cinq-listes)
                       :dur (caddr cinq-listes)
                       :vel (cadddr cinq-listes)
                       :canal (car (cddddr cinq-listes)))))
    (setf (bloc-length m) (calcul-duree-totale m))
    m))

(defun calcul-duree-totale (m)
  (+ (car (reverse (bloc-onset m))) (caar (reverse (bloc-dur m)))))

(defun tirage-motif (clef table-arrangements)
  (tirage-alea (tous-les-assoc clef table-arrangements)))

(defun tous-les-assoc (clef table)
  (cond ((null table) ())
        ((condition? clef (caar table)) 
         (cons (car table) (tous-les-assoc clef (cdr table))))
        (t (tous-les-assoc clef (cdr table)))))

(defun tirage-alea (l) (nth (random (length l)) l))

(defun condition? (clef grille) 
  (cond ((and (null clef) (null grille)) t)
        ((or (null clef) (null grille)) ())
        ((test-cond? (car clef) (car grille))
         (condition? (cdr clef) (cdr grille)))
        (t ())))

(defun test-cond? (acc1 acc2)        ;;; acc = ((deg chiffre) duree) ou (nil duree)
  (and (eq (registre (car acc1)) (registre (car acc2)))
       (eq (cadar acc1) (cadar acc2))   ;;; chiffrage: rien, 7, m7
       (= (cadr acc1) (cadr acc2))))    ;;; duree: 1, 2 demi-mes

(defun registre (acc)
  (let ((deg (sans-marque acc)))
    (case deg ((c c# d eb e) 'registre1) 
              ((f f# g g# a bb b) 'registre2))))

;;; ========================================================
;;; Adaptation des hauteurs choisies aux accords de la grille
;;; ========================================================

(defun fait-haut (liste-blocs) 
  (apply #'append (mapcar #'bloc-haut liste-blocs)))

;;; La sequence memorisee dans la table peut contenir plusieurs demi-mesures
;;; La transposition peut differer d'une demi-mes a la suivante,
;;; donc avec onset, grille, et duree-mes, on controle le changement eventuel de transpo

(defun adapte-haut (liste-haut liste-onset grille clef duree-mes) 
  (boucle-haut liste-haut liste-onset () (mapcar #'degre-sans-marque grille) (mapcar #'degre-sans-marque clef) 0 duree-mes))

(defun degre-sans-marque (acc) (if (null acc) () (sans-marque (car acc))))

(defun boucle-haut (liste-haut liste-onset midi-tmp liste-deg1 liste-deg2 date-demi-mes duree-mes)
  (cond ((null liste-onset) (transpose (reverse midi-tmp) (car liste-deg1) (car liste-deg2)))
        ((< (car liste-onset) (+ date-demi-mes (floor duree-mes 2)))
         (boucle-haut (cdr liste-haut) (cdr liste-onset) (cons (car liste-haut) midi-tmp) liste-deg1 liste-deg2 date-demi-mes duree-mes))
        (t (append (transpose (reverse midi-tmp) (car liste-deg1) (car liste-deg2))
                   (boucle-haut liste-haut liste-onset () liste-deg1 liste-deg2 (+ date-demi-mes (floor duree-mes 2)) duree-mes)))))

;;; deg1 = degre de la grille, deg2 = degre des donnees midi

(defun transpose (data-midi deg1 deg2) (ajoute-liste (int deg2 deg1) data-midi))

(defun ajoute-liste (val liste) 
  (mapcar #'(lambda (x) 
              (if (atom x) (+ val x)
                  (mapcar #'(lambda (x) (+ val x)) x))) liste))

(defun int (deg2 deg1) 
  (let* ((d1 (normalise deg1)) (d2 (normalise deg2))
         (l '(e eb d c# c b bb a g# g f# f)))
    (- (length (cdr (member d1 l))) 
       (length (cdr (member d2 l))))))

(defun normalise (deg)
  (case deg ((eb d#) 'eb) ((c# dd) 'c#) ((bb a#) 'bb) ((g# ab) 'g#) ((f# gb) 'f#) (t deg)))

;;; ========================================================
;;; Preparation des autres donnes midi (onset, dur, vel, can) pour la liste des blocs
;;; ========================================================

(defun fait-onset (liste-blocs) (datation liste-blocs 0))

(defun datation (liste-blocs date)
  (cond ((null liste-blocs) ())
        (t (let ((m (car liste-blocs)))
             (append (ajoute-liste date (bloc-onset m))
                     (datation (cdr liste-blocs) 
                               (+ date (bloc-length m))))))))

(defun fait-dur (liste-blocs) 
  (apply #'append (mapcar #'bloc-dur liste-blocs)))

(defun fait-vel (liste-blocs) 
  (apply #'append (mapcar #'bloc-vel liste-blocs)))

(defun fait-canal (liste-blocs)
  (apply #'append (mapcar #'bloc-canal liste-blocs)))

;;; ========================================================
;;; Table d'arrangement avec 2 registres:
;;; - registre1 = C-C#-D-Eb-E: renversement avec 3ce en haut
;;; - registre2 = F-F#-G-Ab-A-Bb-B: renversement avec 7eme en haut
;;; ========================================================
;;; duree du beat (BPM=124 dans table "Rain") = 469 ms (-> 2 beat par interbarre / X / => beat = demi-interbarre)
;;; 1 demi-interbarre = 469 ms, 2 demi-interbarre = 953 ms 
;;; Attention: ancien solo "Marc-rain" enregistre a BPM=125 (duree beat = 480 ms)

(setq *duree-grille* (* 242 32))
(setq *duree-mes* (* 242 4))        ;;; = 2 beats (469*2), 4 pseudo-"temps"
;;; Grille de 'Rain' voir 1-1Steedman.lisp

;;; TABLE D'ARRANGEMENT DE "RAIN"

(setq *table-arrangements*        ;;; version avec accords sur canal OM=13 (Max=14)
                                  ;;; degre=haut 0-11, chiffre=velo 1=Maj, 2=m7, 3=7
      '(((((b m7) 1) ((e 7) 1))
         (((11 35 66 62 61 57) (47) (4 34 73 56) (46)) (0 242 469 726)
          ((469 242 469 953 953 469) (242) (469 242 469 469) (242))
          ((2 97 69 69 69 69) (97) (3 97 69 69) (97)) ((13 1 0 0 0 0) (1) (13 1 0 0) (1))))
        ((((c# 7) 2))
         (((1 37 70 63 59 58 53) (37) (65) (37) (68)) (0 242 605 726 847)
          ((953 242 469 953 953 953 953) (469) (233) (242) (121))
          ((3 97 69 69 69 69 69) (97) (69) (97) (69)) ((13 1 0 0 0 0 0) (1) (0) (1) (0))))
        ((((g# m7) 2))
         (((8 32 63 59 58 54) (44) (32) (44)) (0 469 726 847)
          ((953 469 953 953 953 953) (242) (121) (121)) ((2 97 69 69 69 69) (97) (97) (97))
          ((13 1 0 0 0 0) (1) (1) (1))))
        ((((g# 7) 2))
         (((8 32 65 62 60 58 54) (44) (32) (44)) (0 242 469 726)   ;;;;(16 258 500 742)
          ((953 242 953 953 953 953 953) (242) (242) (242)) ((3 97 69 69 69 69 69) (97) (97) (97))
          ((13 1 0 0 0 0 0) (1) (1) (1))))
        ((((b 7) 2))
         (((11 35 68 63 61 57) (42) (47) (69) (73)) (0 242 469 605 847)
          ((953 242 469 953 953 953) (242) (242) (250) (121)) ((3 97 69 69 69 69) (97) (97) (69) (69))
          ((13 1 0 0 0 0) (1) (1) (0) (0))))
        ((((f#) 2))
         (((6 30 70 65 61 58 54) (42) (37)) (0 242 726) ((953 242 953 953 953 953 953) (469) (242))
          ((1 97 69 69 69 69 69) (97) (97)) ((13 1 0 0 0 0 0) (1) (1))))
        ((((c# m7) 2))
         (((1 37 63 59 56 52) (37) (37)) (0 242 726) ((953 242 953 953 953 953) (469) (242))
          ((2 97 69 69 69 69) (97) (97)) ((13 1 0 0 0 0) (1) (1))))
        ((((c#) 2))
         (((1 37 63 60 56 53) (37) (37)) (0 242 726) ((953 242 953 953 953 953) (469) (242))
          ((1 97 69 69 69 69) (97) (97)) ((13 1 0 0 0 0) (1) (1))))
        ((((b m7) 1))
         (((11 35 66 62 61 57) (47)) (0 242) ((469 242 469 469 469 469) (242)) ((2 97 69 69 69 69) (97))
          ((13 1 0 0 0 0) (1))))
        ((((b 7) 1))
         (((11 35 68 63 61 57) (47)) (0 242) ((469 242 469 469 469 469) (242)) ((3 97 69 69 69 69) (97))
          ((13 1 0 0 0 0) (1))))
        ((((f#) 1))
         (((6 30 70 65 61 58 54) (42)) (0 242) ((469 242 469 469 469 469 469) (242))
          ((1 97 69 69 69 69 69) (97)) ((13 1 0 0 0 0 0) (1))))
        ((((e m7) 1))
         (((4 40 66 62 59 55)) (0) ((469 469 469 469 469 469)) ((2 97 69 69 69 69)) ((13 1 0 0 0 0))))
        ((((e 7) 1))
         (((4 40 70 66 62 56)) (0) ((469 469 469 469 469 469)) ((3 97 69 69 69 69)) ((13 1 0 0 0 0))))
        ((((e) 1))
         (((4 40 71 68 66 63) (52)) (0 242) ((469 242 469 469 469 469) (242)) ((1 97 69 69 69 69) (97))
          ((13 1 0 0 0 0) (1))))))



;;; TABLE D'ARRANGEMENT DE "GOODBYE PORK PIE HAT"


(setq *table-arrangements*    ;;; pour Goodbye Pork Pie Hat

'(
#|
;;; voicings Lubat 1er octobre 2004
 ((((bb m7) 2)) 
  (((10 46 56 60 65 68 72)) (0) ((968 953 953 953 953 953)) ((2 80 80 80 80 80 80)) ((13 0 0 0 0 0 0))))
 ((((bb m7) 1)) 
  (((10 46 56 60 65 68 72)) (0) ((484 469 469 469 469 469)) ((2 80 80 80 80 80 80)) ((13 0 0 0 0 0 0))))
 ((((f 7) 2)) 
  (((5 41 51 57 63 67 71)) (0) ((968 953 953 953 953 953)) ((3 80 80 80 80 80 80)) ((13 0 0 0 0 0 0))))
 ((((f 7) 1)) 
  (((5 41 51 57 63 67 71)) (0) ((484 469 469 469 469 469)) ((3 80 80 80 80 80 80)) ((13 0 0 0 0 0 0))))
 ((((eb) 2)) 
  (((3 39 50 58 62 67 70)) (0) ((968 953 953 953 953 953)) ((1 80 80 80 80 80 80)) ((13 0 0 0 0 0 0))))
 ((((eb) 1)) 
  (((3 39 50 58 62 67 70)) (0) ((484 469 469 469 469 469)) ((1 80 80 80 80 80 80)) ((13 0 0 0 0 0 0))))
; ((((c# 7) 2))    ;;; sonne tres epais
;  (((1 37 44 51 53 55 59 61 63 65)) (0) ((968 953 953 953 953 953 953 953 953)) ((3 80 80 80 80 80 80 80 80 80)) ((13 0 0 0 0 0 0 0 0 0))))
; ((((c# 7) 1)) 
;  (((1 37 44 51 53 55 59 61 63 65)) (0) ((484 469 469 469 469 469 469 469 469)) ((3 80 80 80 80 80 80 80 80 80)) ((13 0 0 0 0 0 0 0 0 0))))
 ((((c m7) 2)) 
  (((0 48 55 58 62 67 70 74)) (0) ((968 953 953 953 953 953 953)) ((2 80 80 80 80 80 80 80)) ((13 0 0 0 0 0 0 0))))
 ((((c m7) 1)) 
  (((0 48 55 58 62 67 70 74)) (0) ((484 469 469 469 469 469 469)) ((2 80 80 80 80 80 80 80)) ((13 0 0 0 0 0 0 0))))
|#

;;; voicings initiaux     (953=968-10ms pour NoteOff)
 ((((f 7) 2)) 
  (((5 41 51 57 59)) (0) ((968 953 953 953)) ((3 80 80 80 80)) ((13 0 0 0 0))))
 ((((f 7) 1)) 
  (((5 41 51 57 59)) (0) ((484 469 469 469)) ((3 80 80 80 80)) ((13 0 0 0 0))))

; ((((c m7) 2))       ; moche (trop aigu)
;   (((0 48 63 67 74)) (0) ((968 953 953 953)) ((2 80 80 80 80)) ((13 0 0 0 0)))) 
; ((((c m7) 1))
;  (((0 48 63 67 74)) (0) ((484 469 469 469)) ((2 80 80 80 80)) ((13 0 0 0 0))))

 ((((c# 7) 2)) 
  (((1 49 59 63 68)) (0) ((968 953 953 953)) ((3 80 80 80 80)) ((13 0 0 0 0))))
 ((((c# 7) 1)) 
  (((1 49 59 63 68)) (0) ((484 469 469 469)) ((3 80 80 80 80)) ((13 0 0 0 0))))
 ((((eb) 2)) 
  (((3 39 46 55 62 67)) (0) ((968 953 953 953 953)) ((1 80 80 80 80 80)) ((13 0 0 0 0 0))))
 ((((eb) 1)) 
  (((3 39 46 55 62 67)) (0) ((484 469 469 469 469)) ((1 80 80 80 80 80)) ((13 0 0 0 0 0))))
 ((((bb m7) 2)) 
  (((10 46 56 62 67 70)) (0) ((968 953 953 953 953)) ((2 80 80 80 80 80)) ((13 0 0 0 0 0))))
 ((((bb m7) 1)) 
  (((10 46 56 62 67 70)) (0) ((484 469 469 469 469)) ((2 80 80 80 80 80)) ((13 0 0 0 0 0))))
 ((((f#) 2)) 
  (((6 42 53 58 65)) (0) ((968 953 953 953)) ((1 80 80 80 80)) ((13 0 0 0 0))))
 ((((f#) 1)) 
  (((6 42 53 58 65)) (0) ((484 469 469 469)) ((1 80 80 80 80)) ((13 0 0 0 0))))
 ((((f 7) 2)) 
  (((5 41 51 57 65)) (0) ((968 953 953 953)) ((3 80 80 80 80)) ((13 0 0 0 0))))
 ((((f 7) 1)) 
  (((5 41 51 57 65)) (0) ((484 469 469 469)) ((3 80 80 80 80)) ((13 0 0 0 0))))


;;; nouvelles positions 19/10/04:

; ((((c m7) 2))      
;   (((0 36 51 58 63)) (0) ((968 953 953 953)) ((2 80 80 80 80)) ((13 0 0 0 0)))) 
; ((((c m7) 1))
;  (((0 36 51 58 63)) (0) ((484 469 469 469)) ((2 80 80 80 80)) ((13 0 0 0 0))))

; ((((b m7) 2))       
;   (((0 46 56 61 65)) (0) ((968 953 953 953)) ((2 80 80 80 80)) ((13 0 0 0 0)))) 
; ((((b m7) 1))
;  (((0 46 56 61 65)) (0) ((484 469 469 469)) ((2 80 80 80 80)) ((13 0 0 0 0))))

))

; F7  (b5)
      
; 59=si2
; 57=la2
; 51=mib2
; 41=fa1
   
;;; VERSION CONCERT A L'ESTAM (2 octobre 2004)
;;; aucune regle ne genere de m7
;;; bug du Bbm7 joue Bb7+13, qui sonne tres bien debut 2e ligne "Goodbye"
;--------------------------------------------------
;Cm7      C#7      Eb7M    Bbm7    Gb7M     F7

;74=re4   68=sol#3 67=sol3 70=sib3 65=fa3   65=fa3
;67=sol3  63=re#3  62=re3  67=sol3 58=sib2  57=la2
;63=mib3  59=si2   55=sol2 62=re3  53=fa2   51=mib2
;48=do2   49=do#2  46=sib1 56=lab2 42=solb1 41=fa1
;                  39=mib1 46=sib1

;7e?     3ce?             Bb7+13??? 
;--------------------------------------------------

;;; nouvelles positions:

;Cm7      Bbm7(4)

;63=mib3  63=mib3
;58=sib2  61=reb3
;51=mib2  56=lab2
;36=do1   46=sib1


;;; au collectage des voicings, rajouter rythme
;;; sur les accords durant 1/2 mesure (968 ms)







#|  Selectionner, puis Shift-Alt-#

;;; ========================================================
;;; Mixage de 2 chord-seq
;;; ========================================================

(defun mixe-chseq (l1 l2)   ;;; Carlos, 25 fevrier 2003
  (om::mat-trans(sort (om::mat-trans (loop for e in l1
                                           for e2 in l2 collect (append e e2))) '< :key 'second)))
|#