;;; =======================================================
;;; Fichier 1-3-Affiche.lisp / 30 mai 2003
;;; =======================================================

(in-package :midishare)

;;; ========================================================
;;; Affichage d'une grille par lignes de 4 mesures
;;; ========================================================
;;; *nbeatsaccord* = nbre de beats par demi-interbarre
;;;    *nbeatsaccord* = 1 => mesure = / X X / X X /
;;;    *nbeatsaccord* = 2 => mesure = / X X / ("Rain")
;;; determine-durees -> Donne liste de couples (accord nb-demi-interbarre) avec eventuellement marques *
;;; "Rain": (((b m7) 1) (((e *) 7) 1) ((bb m7) 1) ((eb 7) 1) ((g# m7) 2) ((c# 7) 2) ((f#) 2) ((g# 7) 2) ((b 7) 2) ((bb 7) 2))
;;;    *nbeatsaccord* = 1 => (accord 1) = 1 quart-de-mesure (beat) => afficher barre tous les 4
;;;    *nbeatsaccord* = 2 => (accord 1) = 1 demi-mesure (2 beats)  => afficher barre tous les 2

;(affiche-grille *grille*)
; (setf *nbeatsaccord* 1)

(defun affiche-grille (grille) (affiche-accords-durees (determine-durees grille)))

(defun affiche-accords-durees (lacc)
  (let ((nbeats 0))
    (do ((l lacc (cdr l)))
        ((null l) (format t "|") (affiche-bordure *nbeatsaccord*))
      (when (= nbeats 0) (affiche-bordure *nbeatsaccord*) (format t "|"))
      (when (and (> nbeats 0) (= (mod nbeats 16) 0)) 
        (format t "|~%") (affiche-ligne *nbeatsaccord*))
      (when (and (> nbeats 0) (= (mod nbeats 4) 0)) (format t "|"))
      (affiche-accord (car l))
      (setf nbeats (+ nbeats (* (cadar l) *nbeatsaccord*))))))

(defun affiche-ligne (n)
  (if (= n 2)
;    (format t "|__________|__________|~%")
    (format t "|__________|__________|__________|__________|~%")
    (format t "|____________________|____________________|____________________|____________________|~%"))
)

(defun affiche-bordure (n)
  (if (= n 2)
;    (format t "~%_______________________~%")
    (format t "~%_____________________________________________~%")
    (format t "~%_____________________________________________________________________________________~%"))
)

(defun affiche-accord (couple)
  (let ((d (sans-marque (car couple)))
        (c (accord-chiffre (car couple)))
        (k (cadr couple)))
    (affiche-degre d)
    (affiche-chiffre c)
    (affiche-blanc d c)
    (affiche-accordvide k)))

;;; ATTENTION: ne gere pas les enharmonies (Ex: g# et non ab)
;;; -> (c c# d eb e f f# g g# a bb b)

(defun affiche-degre (d)
  (format t (case d (c "C") (c# "C#") (d "D") (eb "Eb") (e "E") (f "F") (f# "F#") 
        (g "G") (g# "G#") (a "A") (bb "Bb") (b "B"))))

(defun affiche-chiffre (c)
  (format t (case c (7 "7") (m7 "m7") (t "  "))))

(defun affiche-blanc (d c)
  (format t " ")
  (when (member d '(c d e f g a b)) (format t " "))
  (when (eq c 7) (format t " ")))

(defun affiche-accordvide (k)
  (when (= k 2) (format t "     ")))


;;; ========================================================
;;; AFFICHAGE DES GRILLES A PARTIR D'UN FICHIER MIDI ENREGISTRE
;;; Recuperation des accords en couples (accord nb-demi-interbarre) 
;;; cf. determine-durees
;;; patche OM "recupere-midi->grille"
;;; ========================================================
;;; accords sur canal OM=13 (Max=14)
;;; velo -> 1=Maj, 2=m7, 3=7 
;;; On fixe *nbeatsaccord*=2, donc demi-interbarre = 2 beats
;;;   (accord 1) = dure 1 demi-mesure
;;;   (accord 2) = dure 1 mesure
;;; (traduc-deg p), (ajoute-chiffre deg v) cf. 6-Voicings.lisp

;;; *nbeatsaccord* = 2 -> accords par demi-mesure
;;; *nbeatsaccord* = 1 -> accords par temps


(defun recupere-accords (lpitch lvelo ldur duree-demimesure nb-mesures)
  (setf *nbeatsaccord* 1)     ;;; demi-interbarre = 2 beats
  (let ((res ()) (grille ()) (k 0))   ;;; k = nbre de demi-mesures
    (do ((l1 lpitch (cdr l1)) (l2 lvelo (cdr l2)) (l3 ldur (cdr l3)))
        ((null l1) (reverse res))
      (let ((d (round (/ (caar l3) duree-demimesure))))
        (setf k (+ k d))
        (when (> k (* 2 nb-mesures))
          (setf k d)
          (setf res (cons (reverse grille) res))
          (setf grille ()))
        (setf grille (cons (list (ajoute-chiffre (traduc-deg (caar l1)) (caar l2)) d) grille))))))

;;; Pour afficher la liste de grilles obtenues:

(defun affiche-recupere (lcouples)
  (let ((n 1))
    (do ((l lcouples (cdr l)))
        ((null l) 'stop)
      (format t "~%Cycle ~a" n)
      (affiche-accords-durees (car l))
      (setf n (+ n 1)))))







