;;; ============================================
;;; Fichier 5-Beats.lisp / 27 fevrier 2003
;;; ============================================
;;; ex 6-Voicings.lisp
;;; Transformation en beats avec labels (accords liste-de-5uplets)
;;; des donnees live venant de Max = liste d'evenements (notes on/off, clocks)
;;; pour l'apprentissage de l'oracle (om::learn-beat-list) dans 2-Omax.lisp
;;; Dans 1-2-Arrange.lisp, l'oracle genere des sequences (om::ImprovizeOnHarmGrid), puis les met au format 5-listes,
;;; puis elles sont envoyees a Max au format 2-coll

(in-package :midishare)

(setf *oracle-courant* ())

;;; ============================================
;;; DONNEES LIVE SEGMENTEES EN BEATS LABELISES 
;;; Conversion Max live (liste events) -> liste beats = liste (accord liste-5-uplets)
;;; Sequence live avec clocks (248) + accords sur canal OM=13 (octet 157) Max=14
;;; ============================================
(defmacro Hstate (HEvent)
  `(first ,HEvent))
(defmacro HOnset (HEvent)
  `(second ,HEvent))
(defmacro HVel (HEvent)
  `(third ,HEvent))
(defmacro EPitch (Event)
  `(third ,Event))
(defmacro EOnset (Event)
  `(first ,Event))
(defmacro EStatus (Event)
  `(second ,Event))
(defmacro EVel (Event)
  `(fourth ,Event))
(defun status->chan (status)
  (- status 144))
(defun noteoffp (event)
  (= (Evel event) 0))
(defun noteonp (event)
  (/= (Evel event) 0))
(defmacro MEPitch (event)
  `(first ,event))

(defmacro MEOnset (event)
  `(second ,event))

(defmacro MEDur (event)
  `(third ,event))

(defmacro MEVel (event)
  `(fourth ,event))

(defmacro MEChannel (event)
  `(fifth ,event))


(defun formate-liste-events->liste-beats (seq-live)             
  (let* ((5u (list-ev->quintuples (copy-tree seq-live)))   ;;; a cause des modifications physiques
         (clocks (first 5u)) (quint (second 5u)))
    (label-accord-beat (set-relative-time-beat clocks (cut-beat-events clocks (quintuples->beats clocks quint))))))


(defun gethashevent (event htable)
  (let ((key (list (EStatus event) (EPitch event))))
    (gethash key htable)))

(defun sethashevent (event htable state)
  (let ((key (list (EStatus event) (EPitch event))))
    (setf (gethash key  htable)
          (list state (EOnset event) (Evel event)))))
  
(defun sethashState (event htable state)
  (let* ((key (list (EStatus event) (EPitch event)))
         (Hevt (gethash key  htable)))
    (setf (gethash key  htable) (cons state (rest Hevt)))))


(defun list-ev->quintuples (listev)
  (let ((htable (make-hash-table :test 'equal)))
    (loop for event in listev
          with current-5
          with collectp = nil          
          do (setf collectp nil)
          if (= (length event) 2) collect event into clocks   ;;; CLOCK = (date 248)
          else
          do (cond
              ((or (null (third event)) (null (fourth event)))
               (print (list "label fautif" event)))
              ((noteonp event)
               (let  ((state (Hstate (gethashevent event htable))))
                 (when (null state) (SetHashevent event htable 'waiting))))
              ;waiting ; ne rien faire
              ((noteoffp event)
               (when (eq  (Hstate (gethashevent event htable)) 'waiting)
                 (let ((hashevt (gethashevent event htable))) 
                   (setf collectp t 
                         current-5 (list (Epitch event) 
                                         (Honset hashevt) 
                                         (- (Eonset event) 
                                            (Honset hashevt)) 
                                         (HVel hashevt) 
                                         (status->chan (EStatus event))))
                   (sethashstate event htable nil)))))
          and if collectp collect current-5 into result end
          finally return (list clocks (sort result #'< :key #'second))          
          )))
                                   

(defun quintuples->beats (clocks quintuples)
  (loop for startbeat in clocks
        for endbeat in (rest clocks)
        collect (loop while quintuples
                      while (< (second (first quintuples)) (first endbeat))
                      collect (pop quintuples)) into result
        finally return (if quintuples (append result (list quintuples)) result)
        ))
  

;; attention modification physique de beat-list !!!!  => si on l'appelle 2 fois, le resultat est different!!!
(defun cut-beat-events (clocks beat-list)   
  (loop for startbeat in clocks
        for endbeat in (rest clocks)
        for cur-beat in beat-list
        for next-beat in (rest beat-list)
        for next-parent on (rest beat-list)
        for new-events = (loop for event in cur-beat
                               for end-event  = (+ (MEOnset event) (MEDur event))
                               if (> end-event (first endbeat))
                               do (setf (MEDur event) (- (first endbeat) (MEOnset event)))
                               and collect (list (MEPitch event) (first endbeat) (- end-event (first endbeat)) 
                                                 (MEvel event) (MeChannel event)))
        if new-events do (setf (car next-parent) (append new-events (car next-parent)))
        finally return beat-list))
        
(defun set-relative-time-beat (clocks beat-list)
  (loop for startbeat in clocks
        for cur-beat in beat-list
        do (loop for event in cur-beat
                 do (setf (MEOnset event) (- (MEOnset event) (first startbeat)))))
  beat-list)

(defun label-accord-beat (beat-list)
  (loop for cur-beat in beat-list
        for lpitch = nil
        for lvel = nil
        for newevents = (loop for event in cur-beat
                              if  (= (MEChannel event) 13)  ;;; ne jamais collecter les chan=13
                              do 
                              (when  (<= (MEDur event) 20) (print (list "label fautif : " event)))
                              (when (> (MEDur event) 20) ;;; suppression reliquats d'accords (<= 20 ms) 
                                   (push (MEPitch event) lpitch) (push (MEVel event) lvel))
                              else collect event)
        for label = (pitch-vel->label (reverse lpitch) (reverse lvel))
        if label collect (list  label newevents)))


;;; ATTENTION: ne gere pas les enharmonies (Ex: g# et non ab)
;;; -> (c c# d eb e f f# g g# a bb b)

(defun pitch-vel->label (lpitch lvel)
  (mapcar #'(lambda (x y) (ajoute-chiffre (traduc-deg x) y)) lpitch lvel))

(defun traduc-deg (p)
  (case (mod p 12) (0 'c) (1 'c#) (2 'd) (3 'eb) (4 'e) (5 'f) (6 'f#) (7 'g) (8 'g#) (9 'a) (10 'bb) (11 'b)))

(defun ajoute-chiffre (deg v)
  (cons deg (case v (1 ()) (2 (list 'm7)) (3 (list 7)))))
 
     
;;; Exemple:            
(setf levts   
'((0 248) (10 157 10 2) (0 155 80 95) (137 155 80 0) (147 155 81 55) (232 155 80 61) (262 155 81 0) (337 155 78 58) (342 155 80 0) (427 155 78 0) (462 157 10 0) 
(462 248) (462 155 80 67) (464 157 3 3) (752 155 78 79) (787 155 80 0) (940 157 3 0) 
(940 248) (947 157 8 2) (1227 155 75 64) (1262 155 78 0) (1367 155 75 0) 
(1422 248) (1902 157 8 0) 
(1902 248) (1902 155 73 58) (1904 157 1 3) (2042 155 73 0) 
(2380 248) (2387 155 73 82) (2635 155 75 60) (2667 155 73 0) (2807 155 75 0) (2862 157 1 0) 
(2862 248) (2867 157 6 1) 
(3342 248) (3820 157 6 0) 
(3820 248) (3827 157 8 3) 
(4302 248) (4782 157 8 0) 
(4782 248) (4787 157 11 3) (4807 157 11 0)))

;resultat de (formate-liste-events->liste-beats levts)
'(
 (((midishare::bb midishare::m7)) ((80 0 137 95 11) (81 147 115 55 11) (80 232 110 61 11) (78 337 90 58 11)))
 (((midishare::eb 7)) ((80 0 325 67 11) (78 290 516 79 11)))
 (((midishare::g# midishare::m7)) ((78 0 322 79 11) (75 287 140 64 11)))
 (((midishare::g# midishare::m7)) nil)
 (((midishare::c# 7)) ((73 0 140 58 11)))
 (((midishare::c# 7)) ((73 7 280 82 11) (75 255 172 60 11)))
 (((midishare::f#)) nil)       ;;; beats vides pendant l'attente du seuil (2000 ms)
 (((midishare::f#)) nil)
 (((midishare::g# 7)) nil)
 (((midishare::g# 7)) nil)
)
                 






