;;; ============================================
;;; Fichier 1-1-Steedman.lisp / 20 fevrier 2003
;;; ============================================

(in-package :midishare)


;;; ========================================================
;;; Reecriture d'une grille en fonction des regles de *grammaire*
;;; ========================================================

#|
(defun reecriture (grille k)
  (let* ((res (tirage-regle grille (mapcar #'car *grammaire*))) 
         (r (car res)) (grille1 (cadr res)))
    (if (or (= k 0) (null r)) 
      (progn ;(format t "~%Grille : ~a" grille) 
             grille) 
      (reecriture grille1 (if (member r '(1-1 1-2)) k (- k 1))))))
|#

(defun tirage-regle (grille liste-num)  ;;; "APPLIQUE" UNE REGLE
  (if (null liste-num) (list () grille) 
      (let* ((r (nth (random (length liste-num)) liste-num)) 
             (grille1 (moteur grille r)))
        (if (null grille1) ;;; APPLICATION IMPOSSIBLE  
            (tirage-regle grille (delete r liste-num))
            (list r grille1)))))

(defun moteur (grille r)
  (let ((liste-pos (detecte grille r)))
    (if (null liste-pos) ()
        (applique grille r (nth (random (length liste-pos)) 
                                liste-pos)))))

(defun detecte (grille regle) (detecte-aux grille regle 1))

(defun detecte-aux (grille r pos)
  (cond ((null grille) ())
        ((prefixe? (partie-gauche r) grille)
         (cons pos (detecte-aux (cdr grille) r (+ pos 1))))
        (t (detecte-aux (cdr grille) r (+ pos 1)))))

(defun applique (grille r pos)     ;;; FAIRE nthcar
  (cond ((null grille) 'erreur)
        ((= pos 1)    (format t" Regle = ~a~%" r)
         (let* ((p1 (partie-gauche r))
                (valeur (fixe-variable p1 grille)))
           (append (actualise valeur (partie-droite r)) 
                   (nthcdr (length p1) grille))))
        (t (cons (car grille) 
                 (applique (cdr grille) r (- pos 1))))))

;;; ========================================================
;;; Un accord est un couple: ex (c m7)
;;; degre (c, c#, d...) et chiffre (7, m7, ...)
;;; Le degre peut etre "marque": (c *) au lieu de c
;;; ========================================================

(defun creer-accord (note chiffre) 
  (cons note (if (null chiffre) () (list chiffre))))
(defun accord-degre (acc) (if (atom acc) () (car acc)))
(defun accord-chiffre (acc) 
  (if (or (atom acc) (null (cdr acc))) () (cadr acc)))
(defun sans-marque (acc) (ote-marque (accord-degre acc))) ;;; DEGRE, PAS ACCORD
(defun ote-marque (deg) (if (atom deg) deg (car deg)))
(defun marque? (acc) 
  (let ((n (accord-degre acc))) (and (listp n) (eq (cadr n) '*))))

;;; ========================================================
;;; Detection d'un membre gauche de regle dans la grille: 
;;; les regles utilisent deux variables x (degre qcq), w (accord qcq non marque)
;;; et des "fonctions" sur les degres (D x), (Stb x), etc.
;;; La fonction fixe-variable determine la valeur de x, et la fonction actualise 
;;; met a jour une sequence en remplacant x par sa valeur, grace a une affectation 
;;; sur une variable globale, et (eval ...) pour les "fonctions" sur les degres
;;; ========================================================

(defun prefixe? (p1 grille)
  (compare (actualise (fixe-variable p1 grille) p1) grille ()))

(defun compare (p1 grille res)
  (cond ((null p1) (reverse res))
        ((or (null grille) 
             (not (correct? (car p1) (car grille)))) ())
        (t (compare (cdr p1) (cdr grille) (cons (car grille) res)))))

(defun correct? (acc1 acc2)
  (or (identique? acc1 acc2) 
      (and (eq acc1 'w) (not (eq acc2 '/)) (not (marque? acc2)))))

(defun identique? (acc1 acc2)
  (or (equal acc1 acc2)
      (and (listp acc1) (listp acc2)
           (eq (accord-chiffre acc1) (accord-chiffre acc2))
           (eq (sans-marque acc1) (sans-marque acc2)))))

(defun fixe-variable (p1 grille)
  (cond ((null p1) ())
        ((or (equal (car p1) '/) 
             (equal (car grille) '/) 
             (not (equal (accord-degre (car p1)) 'x))) 
         (fixe-variable (cdr p1) (cdr grille)))
        (t (accord-degre (car grille)))))

(setq x ())

(defun actualise (valeur p)
  (setf x valeur)
  (mapcar #'(lambda (acc) 
              (let ((n (accord-degre acc)))
                (if (and (listp acc) (or (listp n) (eq n 'x)))
                    (creer-accord (eval n) (accord-chiffre acc))
                    acc)))
          p))


(defun fonction (d int) 
  (let ((l '(c c# d eb e f f# g g# a bb b))) 
    (list (nth int (member d (append l l))) '*)))

(defun D (d) (fonction (ote-marque d) 7))
(defun Stb (d) (fonction (ote-marque d) 1))

;;; ========================================================
;;; La grammaire: chaque regle comporte un numero, suivi des membres gauche et droite
;;; ========================================================


(setf *grammaire*
  '(
;;; regles de division
    (1-1 (/ (x) /) (/ (x) (x) /))
    (1-2 (/ (x 7) /) (/ (x) (x 7) /))
    (1-3 (/ (x m7) /) (/ (x) (x m7) /))

;;; regles de substitution   
    (3a-1 (w (x 7)) (((D x) 7) (x 7)))
    ;(3a-2 (w (x 7)) (((D x) m7) (x 7)))   ; intro m7

    (3b-1 (w (x m7)) (((D x) 7) (x m7)))
    
    (4-1 (((D x) 7) (x 7)) (((Stb x) 7) (x 7)))
    ;(4-2 (((D x) 7) (x)) (((Stb x)) (x)))
    (4-3 (((D x) 7) (x m7)) (((Stb x)) (x m7)))
   
    (3a-11 (w / (x 7)) (((D x) 7) / (x 7)))
    ;(3a-21 (w / (x 7)) (((D x) m7) / (x 7)))
 
    ;(3b-11 (w / (x m7)) (((D x) 7) / (x m7)))
   
    (4-11 (((D x) 7) / (x 7)) (((Stb x) 7) / (x 7)))
    ;(4-21 (((D x) 7) / (x)) (((Stb x)) / (x)))
    ;(4-31 (((D x) 7) / (x m7)) (((Stb x) m7) / (x m7)))
    ))

(defun partie-gauche (r) (cadr (assoc r *grammaire*)))
(defun partie-droite (r) (caddr (assoc r *grammaire*)))


;Regle 1:  x -> x x
;          x7 -> x x7
;          xm7 -> x xm7
;Regle 2:  x -> x Sdx
;          x7 -> x7 Sdx
;          xm7 -> xm7 Sdx
;Regle 3a: w x7 -> Dx7 x7
;          w x7 -> Dxm7 x7
;Regle 3b: w xm7 -> Dx7 xm7
;Regle 4:  Dx7 x7 -> Stbx7 x7
;          Dx7 x -> Stbx x
;          Dx7 xm7 -> Stbxm7 xm7
;Regle 5:  x x x -> x Stxm Mxm
;Regle 6:  x x {Dx, Stxm7, Lxm7} -> x x#7 {Dx, Stxm7, Lxm7}
;          xm xm {Dx, Stxm7, Lxm7} -> xm x#7 {Dx, Stxm7, Lxm7}
