(in-package :cl-user)

(cl:defpackage "NETCLOS"
  (:nicknames "NC")
  (:use "COMMON-LISP" "CCL" "FF")
  )

(in-package :netclos)


(defstruct request message-id)


(defmacro defmessage (name args &key receive-action request-p process-p)
  (if request-p
      (create-request name (fetch-names args) args receive-action process-p)
    (create-operation name (fetch-names args) args  receive-action process-p)))


(defun fetch-names (args)
  (loop for arg in args
      collect (if (listp arg)
		  (first arg)
		arg)))

(defun create-request (name arg-names args body process-p)
  (let* ((sym (gensym "message-id"))
	 (body `(kernel-send *calling-os* 
			     (reply-message :content ,body
					    :message-id ,sym))))
    `(progn 
       ,(create-struct name arg-names :super 'request)
       ,(create-defun name (cons sym arg-names) body process-p) 
       ,(create-pack name (cons 'message-id arg-names) (cons 'message-id args)))))

(defun create-operation  (name arg-names args body process-p)
  `(progn 
     ,(create-struct name arg-names)
     ,(create-defun name arg-names body process-p)
     ,(create-pack name arg-names args)))


(defun create-struct (name args &key super)
  (if super
       `(defstruct (,name 
		    (:constructor ,(intern (format nil "~a-MESSAGE" name)))
		    (:include ,super)) 
	  ,@args)
     `(defstruct (,name 
		  (:constructor ,(intern (format nil "~a-MESSAGE" name)))) 
	,@args)))

(defun create-defun (name arg-names body process-p)
  `(defun ,name ,arg-names ,(if process-p
				`(mp:process-run-function ,(format nil "receiving ~a" name)
							  #'(lambda (*calling-os*)
							      ,body)
							  *calling-os*)
			      body)))
(defvar *pack-forms* ())

(defun create-pack (name arg-names args)
  `(defmethod pack ((message ,name) &optional stream)
     (let ((*pack-forms*))
       (write-string ,(concatenate 'string "("  (write-to-string name) " ") stream)
       ,@(loop for arg in args
	       for arg-name in arg-names
	       for packfunc = (if (and (listp arg) 
				       (eq (second arg) :packing))
				(third arg)
			        'pack) 
	       collect `(,packfunc (,(intern 
				      (format nil "~a-~a" name arg-name)) 
				    message)
				   stream))
       (write-char #\) stream))))


(defmacro defpargeneric (name send-func lambda-list &rest options)
  ;; I wanted to use defgeneric, but you can't add your own keyword arguments then.
  ;; So now there is no source-file-recording and fewer compile-time error-checking.
  
  (let ((class (compute-gf-class  name (second (assoc :generic-function-class 
                                                      options)))))

(print (list "***" class))

    `(progn (when (and *manager* *defremote-p*)
              (loop for space in (spaces *manager*)
                    do (remote-eval space 
                                    '(let ((*defremote-p* nil)) 
                                       (defpargeneric ,name ,send-func ,lambda-list ,@options)))))
            
            (ensure-generic-function 
             ',name 
             :lambda-list ',lambda-list
             :generic-function-class (print (find-class ',class))
             :send-func ,(compute-send-func send-func)
             ,. (when (subtypep class 'sending-gf)
                  `(:method-combination , (find-method-combination #'initialize-instance 'sending ())))
             ,@(compute-options options)))))




   
	
	   



