

(in-package :oa)

(export '(
          *max-buf-size*
          om-make-send-connection
          om-make-receive-connection
          om-send-udp-packet
          om-receive-udp-packet
          om-close-udp-connection
          ) :om-api)

(defvar *max-buf-size* nil)
(setf *max-buf-size* 50000)

;;-----------------------------------------------------------------------------
;; Low-level functions using OpenTransport.
;;-----------------------------------------------------------------------------

;;;(require :Opentransport)


(defun open-opentransport ()
  (#_InitOpentransportInContext #$kInitOTForApplicationMask (ccl::%null-ptr)))

(defun free-opentransport ()
  (#_CloseOpentransportInContext (ccl::%null-ptr)))

(defun new-udp-conn (port)
  (let ((endpoint (open-udp-endpoint)))
    (bind-endpoint endpoint (string2host "localhost") port)
    endpoint))

(defun close-udp-conn (endpoint)
  (#_OTUnbind endpoint)
  (#_OTCloseProvider endpoint))

(defun open-udp-endpoint ()  
  (ccl::with-cstrs ((conf #$kUDPName))   
    (let* ((udp-conf (#_OTCreateConfiguration conf)) 
           (endpoint nil) 
           (err #$kOTNoError)) 
      (ccl::rlet ((errP :osstatus))
        (setq endpoint (#_OTOpenEndpointIncontext udp-conf 0 (ccl::%null-ptr) errP (ccl::%null-ptr))
              err (ccl::pref errP :osstatus)))
      (unless (eql err #$kOTNoError)
        (ccl::ot-error err :create))
      endpoint)))

(defun bind-endpoint (endpoint host port )    
  (ccl::rlet ((errP :osstatus))
    (let* ((bindreq (#_OTAllocInContext endpoint #$T_BIND #$T_ADDR errP (ccl::%null-ptr)))
           (bindret (#_OTAllocInContext endpoint #$T_BIND #$T_ADDR errP (ccl::%null-ptr)))
           (status (cl-user::pref errP :osstatus)))
      (declare (dynamic-extent bindreq bindret))
      (let* ((netbuf (cl-user::pref bindreq :tbind.addr)))
        (declare (dynamic-extent  netbuf))
        (setf (cl-user::pref netbuf :tnetbuf.len) (ccl::record-length :inetaddress))
        (#_OTInitInetAddress (cl-user::pref netbuf :tnetbuf.buf) port host)
        (setq status (#_OTBind endpoint bindreq bindret)))
      ;(format t "binded to port ~a" (cl-user::pref (cl-user::pref (cl-user::pref bindret :tbind.addr) :tnetbuf.buf) :inetaddress.fport))
      (unless (ccl::%null-ptr-p bindreq) (#_OTFree bindreq #$T_BIND))
      (unless (ccl::%null-ptr-p bindret) (#_OTFree bindret #$T_BIND))
      status)))


;;; corrections by ga
;;; la fonction existante : elle est bloquante tant qu'un paquet n'est pas arriv. Mme appele
;;; par un process concurrent elle ralentit les autres processes

(defun wait-receive-packet (endpoint buffer length)
  (ccl::rlet ((unit-data :TUnitdata)
         (addr :TNetBuf)
         (opt :TNetBuf)
         (udata :TNetBuf)
         (source :INetAddress)
         (flag :OTFlags))
    (setf (cl-user::pref addr :TNetBuf.maxlen) (ccl::record-length :INetAddress)
          (cl-user::pref addr :TNetBuf.buf) source
          (cl-user::pref opt :TNetBuf.maxlen) 0 
          (cl-user::pref opt :TNetBuf.buf) (ccl::%null-ptr)
          (cl-user::pref udata :TNetBuf.maxlen) length  
          (cl-user::pref udata :TnetBuf.buf) buffer
          (cl-user::pref unit-data :TUnitData.addr) addr
          (cl-user::pref unit-data :TUnitData.opt) opt
          (cl-user::pref unit-data :TUnitData.udata) udata)
    (loop  
      (when (eql  #$kOTNoError (#_OTRcvUData endpoint unit-data flag)) 
        (return (cl-user::pref (cl-user::pref unit-data :tunitdata.udata) :tnetbuf.len))))))


;;; j'ai rajout une fonction receive-packet-1 qui revient tout de suite avec nil s'il n' ya rien 
;;; ou la taille de buffer s'il y a qqchose. C'est  l'utilisateur de grer sa loop, ou bien
;;; de rendre la main au scheduler s'il l'appelle dans un process.
;;; ca serait biend de l'intgrer  la librairie standard, car pour la compiler il faut OpenTransport etc.

(defun receive-packet (endpoint buffer length)
  (ccl::rlet ((unit-data :TUnitdata)
         (addr :TNetBuf)
         (opt :TNetBuf)
         (udata :TNetBuf)
         (source :INetAddress)
         (flag :OTFlags))
    (setf (cl-user::pref addr :TNetBuf.maxlen) (ccl::record-length :INetAddress)
          (cl-user::pref addr :TNetBuf.buf) source
          (cl-user::pref opt :TNetBuf.maxlen) 0 
          (cl-user::pref opt :TNetBuf.buf) (ccl::%null-ptr)
          (cl-user::pref udata :TNetBuf.maxlen) length  
          (cl-user::pref udata :TnetBuf.buf) buffer
          (cl-user::pref unit-data :TUnitData.addr) addr
          (cl-user::pref unit-data :TUnitData.opt) opt
          (cl-user::pref unit-data :TUnitData.udata) udata)
      (if (eql  #$kOTNoError (#_OTRcvUData endpoint unit-data flag)) 
        (cl-user::pref (cl-user::pref unit-data :tunitdata.udata) :tnetbuf.len)
        nil)))




(defun send-packet (endpoint dest packet length)
  (ccl::rlet ((unit-data :TUnitdata)
              (addr :TNetBuf)
              (opt :TNetBuf)
              (udata :TNetBuf))
    (setf (cl-user::pref addr :TNetBuf.maxlen) (ccl::record-length :INetAddress)
          (cl-user::pref addr :TNetBuf.buf) dest
          (cl-user::pref opt :TNetBuf.maxlen) 0 
          (cl-user::pref opt :TNetBuf.buf) (ccl::%null-ptr)
          (cl-user::pref udata :TNetBuf.len)  length
          (cl-user::pref udata :TnetBuf.buf) packet
          (cl-user::pref unit-data :TUnitData.addr) addr
          (cl-user::pref unit-data :TUnitData.opt) opt
          (cl-user::pref unit-data :TUnitData.udata) udata)
    (#_OTSndUData endpoint unit-data)))


;;-----------------------------------------------------------------------------
; Miscelaneous suppport functions
;;-----------------------------------------------------------------------------

(defun string2host (str)
  (ccl::rlet ((host :InetHost))
    (ccl::with-cstrs ((hs str))
      (#_OTInetStringToHost hs host))
             (cl-user::%get-long host)))



;;-----------------------------------------------------------------------------
; API functions
;;-----------------------------------------------------------------------------


(defclass UDPsocket () 
  ((endpoint :accessor endpoint :initarg :endpoint)
   (host :accessor host :initarg :host)))

(defmethod udpsocket-p ((self t)) nil)
(defmethod udpsocket-p ((self udpsocket)) t)


(defun om-make-send-connection (host port)
  (make-instance 'UDPsocket 
    :host (ccl::make-record :inetaddress
                            :faddresstype #$AF_INET
                            :fhost (string2host (ccl::string host))
                            :fport port)
    :endpoint (new-udp-conn 1910))
  )

(defun om-make-receive-connection (port)
  (new-udp-conn port))

(defun om-send-udp-packet (connection data size)
    (send-packet (endpoint connection) (host connection) data size))

(defun om-receive-udp-packet (connection buffer size)
  (wait-receive-packet connection buffer size))

(defun om-close-udp-connection (connection)
  (if (udpsocket-p connection)
      (close-udp-conn (endpoint connection))
    (close-udp-conn connection)))




