;;; ============================================
;;; Fichier 2-Omax.lisp / 17 fevrier 2003
;;; ============================================
;;; Utilise l'oracle (apprentissage des sequences recues du live)

(in-package :midishare)        ;;; mettre om:: devant les fonctions a utiliser dans Listener

;;; NB: - les fichiers places dans le dossier Patches sont evalues au lancement de OM dans l'ordre alpha
;;;     - ceux d'une librairie le sont si on ouvre un patch contenant une methode de la librairie

;;; ============================================
;;; PRINCIPE DE LA TRANSMISSION MAX - OPEN MUSIC
;;; ============================================
;;; (omax)   => place OM en position de reception (d'apres G. Assayag, 19 decembre 2001)
;;; - Max envoie a OM soit des notes (canaux OM=0-13), soit des donnees (canal OM=14) ou des instructions (canal OM=15) 
;;; (canal OM=13 (Max=14) = accords, OM=12 (Max=13) = enregistrement live)
;;; - OM envoie a Max des seq midi (port 1=iac2) sous forme de couples de chaines d'octets TypeStream de MidiShare:
;;; chaine deltatimes + chaine NoteOn/NoteOff/Clock
;;; ou des donnes (port 2=iac3)
;;; ATTENTION: deltatime = duree entre l'event et le suivant (pas le precedent, i.d. dernier deltatime = 0)

;;; Preferences OM (File/Preferences/MidiShare SetUp: grille de petits carres = ports de 0 a 255)
;;; --------------
;;; port 0: Entree=iac1 (venant de Max: instructions ou notes)
;;; port 1: Sortie=iac2 (vers Max: sequences midi codees en chaines d'octets)
;;; port 2: Sortie=iac3 (vers Max: donnees codees en chaines d'octets)

;;; Objets Max pour communiquer avec OM 
;;; -----------------------------------
;;; - "ommidiout": vers OM, envoie notes (ou clocks-tempo generes par "loop") via Sortie=iac1 
;;; ("noteout iac1" ou "midiout iac1")
;;; - "ommidiin": venant de OM, recoit notes (chaines d'octets) via Entree=iac2
;;; - "omcodeout": vers OM, envoie instructions et donnees codees par des notes via Sortie=iac1 (canaux Max=15, 16) 
;;; (cf. fonction om::omax ci-dessous)
;;; - "omcodein": venant de OM, recoit des donnees via Entree=iac3
;;; NB: mode Play, Alt-click sur boite ("midiin") pour connecter, ou symbole=port (iac1,...) ou c=instrument (Clavinova, SY77...)
;;; ============================================


; open MS Application, and recall to close it at quit time. (G.A. 08/04/03)

(defparameter *refnum* (midiopen "Common Lisp"))


(defun omax-midi-close ()
  (midiclose *refnum*))

(pushnew  #'omax-midi-close *lisp-cleanup-functions*)



(defvar *suite?* t)
(setf *numbeat* 0)
(setf *numbeatinit* 0)

(defun om::omaxstop ()        ;;; pour arreter proprement: (omaxstop) dans Listener1 ou patch
  (setf *suite?* nil)
  (format t "Fin de la session omax, enregistrez l'oracle~%") 'om::fin)


(defun om::set-rec-min (n) (setf *min-rec-dur* n))
(om::set-rec-min 50)

(defun om::omax ()
  (setf *suite?* t)
  (setf *numbeat* 0)
  (MidiConnect *refnum* 0 -1)			; connect MCL to MidiShare
  (MidiConnect 0 *refnum* -1)			; connect MidiShare to MCL
  (MidiFlushEvs *refnum*)			; flush old events in the rcv fifo
  (let ((seq-tmp ()) (date-init (MidiGetTime)) (accord-init ())
        (seq-inst ()) (clock-ev ()) (clock-ev-date ()) 
        (accord-ev ()) (bool-enreg nil) (bool-inst nil)
        (seq-bend-tmp nil))
    (loop while (and *suite?* *midishare*) do 
          (do ((event (MidiGetEv *refnum*) (MidiGetEv *refnum*)))	;read all the events
              ((%null-ptr-p event))		                        ; in the rcv fifo (defaut port 0=iac1)
;(setf clock-ev nil)
            (when (member (type event) (list typeNote typeKeyOn typeClock typePitchBend))
              (when (eq (type event) typeClock) 
                (setf clock-ev event)      ;;; dernier clock recu, nil=>non mesure
                (setf clock-ev-date (date event)) ; l'event est freed en fin de boucle !!!!
                (setf *numbeat* (1+ *numbeat*)))
              
              (when (and (member (type event) (list typeNote typeKeyOn))   ;;; dernier accord NoteOn recu 
                         (= (chan event) 13) ;;; canal OM=13 (Max=14) -> accords de la grille
                         (> (vel event) 0))
                (setf accord-ev (list (pitch event) (vel event))) )           
              (if (and (member (type event) (list typeNote typeKeyOn))
                       (member (chan event) '(14 15)))
                (case (chan event)
                  (15                                    ;;; canal OM=15 (Max=16) -> instructions
                   (case (pitch event)
                     ;------------------------------------------------------------------------------------------
                     (61 (setf bool-enreg t)      ;;;............... debut enregistrement
                      (setf date-init (if (null clock-ev)   
                                        (date event)         ;;; non mesure -> date du code 61
                                        clock-ev-date))      ;;; mesure -> date du dernier clock recu
                      (setf accord-init accord-ev)
                      (setf *numbeatinit* *numbeat*))
                     (62 (setf bool-enreg nil)    ;;;............... fin enregistrement
                      (when (not (null seq-tmp))
                        (setf seq-tmp (flush-note-off (- (date event) date-init) (reverse seq-tmp)))
                        ;(if (null clock-ev)  
                        (if (not *mode-beat*)
                          ;;; CAS IMPRO FREE (sans clock) -> format 2-coll
                          (if (or (null seq-tmp) (and seq-tmp (< (caar (last seq-tmp)) *min-rec-dur*)))
                            (format t "~%Phrase tro courte (~D ms), non enregistree~%" (caar (last seq-tmp)))
                            (progn (format t "~%Recu phrase de ~a evenements, " (length seq-tmp))
                                   (setf seq-tmp (cadr (list-ev->quintuples seq-tmp)))   ;;; !!!!!!!!VERSION FREE!!!!!
                                   
                                   (let ((prevmaxetat (om::maxEtat *oracle-courant*)))
                                     (om::learn-event-list *oracle-courant* (om::mf->crossEvents (list seq-tmp)))
                                     (when *Ofon-mode*
                                       (om::learn-bend-list *oracle-courant* prevmaxetat (reverse seq-bend-tmp))))
                                   
                                   (format t "ajoutee dans l'oracle FREE~%")
                                   )   ) 
                          
                          ;;; CAS HARMONICO-METRIQUE => clock init
                          (progn 
                            (format t "~%Recu phrase de ~a evenements, " (length seq-tmp))
                            ;(print seq-tmp)
                            (setf seq-tmp (cons '(0 248)                     
                                                (cons (list 10 157 (car accord-init) (cadr accord-init));;; accord precedent phrase live
                                                      seq-tmp))) 
                            ;(print seq-tmp)  
                            (setf seq-tmp (formate-liste-events->liste-beats  seq-tmp))     ;;; cf. 6-Voicings.lisp
                            (om::learn-beat-list *oracle-courant* seq-tmp *numbeatinit*)  ;;; AJOUTER NUMERO CLOCK
                            (format t "ajoutee dans l'oracle")))
                        (setf seq-tmp () seq-bend-tmp ())))
                     
                     (82 (setf bool-enreg nil)    ;;;............... fin enregistrement SANS garder seq-tmp
                      (setf seq-tmp ()))
                     
                     ;------------------------------------------------------------------------------------------
                     (80 (setf bool-inst t))                   ;;; debut appel generique de fonction
                     (81 (print "toto")(when (not (null seq-inst))           ;;; fin appel generique de fonction
                           (fonction-generique seq-inst)
                           (setf seq-inst ())
                           (setf bool-inst nil)))
                     ;------------------------------------------------------------------------------------------
                     (58 (setf bool-inst t))                   ;;; debut donnees choix oracle
                     (59 (when (not (null seq-inst))           ;;; fin donnees choix oracle
                           (fonction-choixoracle seq-inst)
                           (setf seq-inst ())
                           (setf bool-inst nil)))
                     ;------------------------------------------------------------------------------------------
                     (60 (fonction-grammaire))
                     (57 (fonction-affichegrille))
                     (63 (setf bool-inst t))                   ;;; debut donnees grammaire
                     (64 (when (not (null seq-inst))           ;;; fin donnees grammaire
                           (fonction-controlegrammaire seq-inst)
                           (setf seq-inst ())
                           (setf bool-inst nil)))
                     ;------------------------------------------------------------------------------------------
                     (69 (format t "Code 69 recu (free impro)~%")
                      (fonction-free)
                      )
                     (70 (setf bool-inst t))                   ;;; debut donnees free (sans clock)
                     (71 (when (not (null seq-inst))           ;;; fin donnees free (sans clock)
                           (fonction-controlefree seq-inst)
                           (setf seq-inst ())
                           (setf bool-inst nil)))
                     ;------------------------------------------------------------------------------------------
                     (50 (setf *numbeat* 0))
                     ))
                  (14                             ;;; canal OM=14 (Max=15) -> donnees pour instructions 
                   (setf seq-inst (cons (pitch event) seq-inst)))
                  ) 

                ;;; EVENT AUTRE QUE FONCTION OU DATA
                (if (eq (type event) typePitchBend)
                  (let ((bendval (+ 8192 (bend event))))
                    (push (list (chan event) bendval) seq-bend-tmp))
                  ;; Ofon pitch bend sur canal 234
                  (when bool-enreg 
;(print (type event))
                    (setf seq-tmp  (cons (cons (- (date event) date-init)
                                               (if (eq (type event) typeClock)
                                                 (list 248)
                                                 (list (+ 144 (chan event)) (pitch event) (vel event))))
                                         seq-tmp))))))
            (MidiFreeEv event)
            )))
  (and *midishare* (MidiConnect 0 *refnum* 0))	
  (and *midishare* (MidiConnect  *refnum* 0 0))
  )


;;; ================================================
;;; Fonctions de l'interface OpenMusic (patch "Omax")
;;; ================================================


(defun switch-prolifere (bool)
  (setf *Prolifere-mode* bool))

(defun switch-ofon (bool)
  (setf *Ofon-mode* bool))

(defun switch-spectral (bool)
  (setf *spectral-mode* bool))

(defun om::switch-voicing (bool)
  (setf *Voicing-mode* bool))

;;; ================================================
;;; FONCTION GENERIQUE D'APPEL DE FONCTIONS OM
;;; ================================================
;;; seq-inst envoye par la boite Max "omfonctionout"
;;; = liste a 3 elements, le 1er est le code OM de la fonction a executer,
;;; les 2 autres sont des parametres eventuels

(defun fonction-generique (seq-inst)
  (case (car seq-inst)
    (1
     (let ((type-morceau (cadr seq-inst)))  ;;; type-morceau (en fait orchestre) = 0 free, 1 beat
       (om::switch-perf-orc type-morceau)
       (setf *mode-beat* (= type-morceau 1))))
    (2
     (let ((num-orc (cadr seq-inst))
           (type-morceau (caddr seq-inst)))  ;;; type-morceau = 0 free, 1 free estrada, 2, 3, ... grille avec beat
       (om::set-perf-orc num-orc type-morceau)))
    (3
     (let ((dureebeat (+ (* 100 (cadr seq-inst)) (caddr seq-inst))))
       (om::set-cur-orc-dureebeat dureebeat)
       (setf *dureebeat* dureebeat)
       ))
    (4         ;;; FONCTION GENERIQUE POUR RECEPTIONNER ACCORDS
     (let ((deg (cadr seq-inst)) (chif (caddr seq-inst)))
       (print (list deg chif))
       (setf *accord* (ajoute-chiffre (traduc-deg deg) chif))         ;cf. 6-Voicings.lisp
       ))   ;;; la mise dans table se fait dans cree-free, car *Voicing-mode* = t
    ))


;;; ============================================
;;; AUTRES FONCTIONS D'APPEL NON GENERIQUES
;;; ============================================

;;; ============================================
;;; Choix de l'oracle pour enregistrement live
;;; ============================================

(defun fonction-choixoracle (seq-inst)
(print seq-inst)
  (setf *oracle-courant*
        (case (car seq-inst)
          (1 *oracle1*) (2 *oracle2*) (3 *oracle3*) (4 *oracle4*)))) 

;;; ============================================
;;; Enrichissement d'une grille harmonique avec oracle
;;; ============================================
; (fichiers 1-1-Steedman.lisp, 1-2-Arrange.lisp)

(defun fonction-grammaire ()
  (let* ((piano-bass (creer-piano-bass))
         (5-listes0 (first piano-bass))      ;;; donnees venant de la table de "Rain" (duree beat = 484 ms)
         (5-listes1 (second piano-bass))     ;;; donnees eventuelles venant de *oracle1* (*dureebeat*), nil si oracle vide
         (5-listes2 (third piano-bass))
         (5-listes3 (fourth piano-bass))
         (5-listes4 (fifth piano-bass)))
    (setf *genere-quoi* 'grammaire)
    (5-listes->max 60 5-listes0)        ;;; 60 = identifiant des sequences calculees par grammaire
    (setf *genere-quoi* 'oracle)
    (5-listes->max 65 5-listes1)        ;;; 65 = identifiant des sequences calculees a partir de *oracle1*
    (5-listes->max 66 5-listes2)        ;;; 66 = identifiant des sequences calculees a partir de *oracle2*
    (5-listes->max 67 5-listes3)        ;;; 67 = identifiant des sequences calculees a partir de *oracle3*
    (5-listes->max 68 5-listes4)))      ;;; 68 = identifiant des sequences calculees a partir de *oracle4*



(defun 5-listes->max (id 5-listes)
  (when (not (null 5-listes))
    (let ((2-coll (formate-5-listes->2-coll 5-listes)))
      (joue-midi-om->max id 2-coll))))

(defun fonction-controlegrammaire (seq-inst)   ;;; cons: les donnees arrivent en ordre inverse
  (setf *k* (nth 0 seq-inst))
 ;(format t "~%Nombre de rgles: ~a" *k*)
)

(defun fonction-affichegrille ()                       ;;; 57 = declenche affichage grille
  (affiche-grille *grilletmp*))

;;; ============================================
;;; Calcul d'une impro free (sans clock) 
;;; ============================================
; (fichier 5-Free.lisp, OracleMarc.lisp)

(defun fonction-free ()
  (let* ((5-listes (creer-free)))
    (setf *genere-quoi* 'free)
    (if (null 5-listes) (format t "Pas d'impro disponible~%")
        (joue-midi-om->max 69 (formate-5-listes->2-coll 5-listes)))))

(defun fonction-controlefree (seq-inst)
  (om::set-start-region *oracle-courant* (list (nth 0 seq-inst) (nth 1 seq-inst)))
  (setf *lengthfree* (nth 2 seq-inst)))



;;; ============================================
;;; FONCTIONS DE TRANSMISSION DE OM VERS MAX
;;; ============================================
;;; FORMAT DES SEQUENCES MIDI POUR MAX

;;; au debut: (127 3 id) id = pour identifier la sequence dans Max, 
;;;           (127 4 duree-totale sur 3 octets)
;;; data = liste d'entiers-octet, 2 types possibles:
;;;      (127 route indice duree1 duree2)
;;;      (127 route indice [noteon midi vel + clock]*)     noteon=144+canal, clock=248
;;;      (127 route indice o1 o2 o3 o4 o5 o6)
;;;      o1 + 127 * o2 = valeur d'un pitch bend
;;;      les 3 pitch-bend successifs encodent : no fichier, date LSB, date MSB
;;; 127 = separe les evenements 
;;; (pas octet de statut, car Max "midiin" traite les octets de statuts de facon speciale)
;;; route = 1 (duree) ou 2 (notes) ou 5 audio-dates
;;; indice = numero d'evenement a partir de 1
;;; duree1, duree2 < 100 pour coder diff-date < 10000 ms  (= duree1 * 100 + duree2)
;;; a la fin: (127 0 127) 




(defun flush-note-off (date-stop liste) (ajoute-off-fin date-stop (enleve-off-init liste)))

(setf e
'((0 248) (30 156 89 22) (209 156 94 25) (418 156 96 33) (775 157 6 0) 
(778 248) (785 157 11 3) (825 156 100 17) (1289 156 89 0) (1300 156 96 0) (1312 156 94 0) (1335 156 100 0) 
(1557 248) (2334 157 11 0) 
(2340 248) (2340 157 3 3) 
(3120 248) ))

;(flush-note-off 5120 e)

(defun enleve-off-init (liste)
  (do ((l liste (cdr l)))  ;;; enleve les NoteOff qui restent au debut de la sequence (et clocks associes)
      ((or (null l) 
           (and (not (null (cddr (car l)))) (not (= 0 (cadddr (car l)))))) l)))

(defun ajoute-off-fin (date-stop liste)
  (let ((deja-off ()) (new-off ()))
    (do ((l (reverse liste) (cdr l)))    ;;; ajoute des NoteOff a la fin pour les notes coupees
        ((null l) (append liste new-off))
      (when (not (null (cddr (car l))))   ;;; clock
        (let ((4uplet (car l)))
          (when (and (> (cadddr 4uplet) 0) (not (member (list (cadr 4uplet) (caddr 4uplet)) deja-off :test #'equal)))
            (setf new-off (cons (list date-stop (cadr 4uplet) (caddr 4uplet) 0) new-off)))
          (when (= (cadddr 4uplet) 0)
            (setf deja-off (cons (list (cadr 4uplet) (caddr 4uplet)) deja-off))))))))

(setf *iac2* 1)   ;;; OM -> Max: midi 
(setf *iac3* 2)   ;;; OM -> Max: data 

(defun joue-data-om->max (id data)
  (envoie-event-om->max *iac3* (list 127 3 id))
  (envoie-event-om->max *iac3* (append (list 127) data (list 127))))


;;; pour Ofon

(defun joue-midi-om->max (id 2-coll)
  (envoie-event-om->max *iac2* (list 127 3 id)) 
  (envoie-event-om->max *iac2* (append (list 127 4) (2-coll-duree-totale 2-coll)))
  (do ((l1 (car 2-coll) (cdr l1)) (l2 (cadr 2-coll) (cdr l2)) (l3 (caddr 2-coll) (cdr l3)))
      ((null l1) (envoie-event-om->max *iac2* (list 127 0 127)))   ;;; pour cloturer la sequence
    (when (not (null (cdr (car l2))))
      (envoie-event-om->max *iac2* (append (list 127 1) (cdr (car l1))))  ;;; pas le numero d'event (pb val > 255)
      (envoie-event-om->max *iac2* (append (list 127 2) (del-127 (cdr (car l2)))))
      (when *Ofon-mode* (envoie-event-om->max *iac2* (append (list 127 5)  (cdr (car l3))))))))


;; new pour Ofon

(defun make-3st-col (notes)
  (loop for noteset in notes
        with noteon = nil
        do 
        (setf noteon nil)
        (loop for vel in (cdddr noteset) by #'cdddr
              if (/= vel 0) do (setf noteon T))
        collect (if noteon ; at least one noteon
                  (let* ((date (pop *Ofon-Audio-dates*))
                         (ldate-127 (date-127 (second date) (third date))))
                    (if (null date)
                      (list (first noteset) 0 0 0 0 0 0)
                      (cons (first noteset)
                            (append  ldate-127
                                     (list (msb-127 (first date))
                                           (lsb-127 (first date)))))))
                  (list (first noteset) 0 0 0 0 0 0))))

;; utilitaires Ofon


(defun build-date (triple)
  (list (first triple)
        (+ (second triple) (* 16384 (third triple)))))


;; entre 0 et 126 (on evite 127)  base 127
(defun lsb-127 (n)
  (mod n 127))

(defun msb-127 (n)
  (floor n 127))

;(lsb-127 236)
;(msb-127 12568) 
  

(defun date-127 (date-lsb date-msb)
  (let* ((date (+ date-lsb (* 16384 date-msb)))
         (date4 (floor date 2048383))
         (mod1 (mod date 2048383))
         (date3 (floor mod1 16129))
         (mod2 (mod mod1 16129))
         (date2 (floor mod2 127))
         (date1 (mod mod2 127)))
    (list date4 date3 date2 date1)))
         
    
(defun date-127-2 (date)
  (let* ((date4 (floor date 2048383))
          (mod1 (mod date 2048383))
          (date3 (floor mod1 16129))
          (mod2 (mod mod1 16129))
          (date2 (floor mod2 127))
          (date1 (mod mod2 127)))
       (list date4 date3 date2 date1)))




(defun del-127 (l) (mapcar #'(lambda (x) (if (= x 127) 126 x)) l))   ;;; 127 = code reserve pour separateur

(defun detripler (duree)
  (list (floor duree 10000) (mod (floor duree 100) 100) (mod duree 100)))

(defun 2-coll-duree-totale (2-coll)
  (case  *genere-quoi*
    ((grammaire free)
      (let ((d (apply #'+ (mapcar #'(lambda (triplet) (+ (* 100 (cadr triplet)) (caddr triplet))) (cdr (reverse (car 2-coll)))))))
        (print (list "**" d)) (detripler d)))
    (oracle (print (detripler
                    (print (* (om::HarmGridLength *oracle-courant*) 
                              (om::RefTempo *oracle-courant*))))))))

    
(defun envoie-event-om->max (num-iac liste)
  (let ((neweventstream (Midi-New-Ev TypeStream)))	; STREAM EVENT = CHAINE D'OCTETS     
    (unless (%null-ptr-p neweventstream)	; if the allocation was succesfull
      (port neweventstream num-iac)			; port 1 (=iac2 midi vers Max), port 2 (=iac3 data vers Max)
      (mettre-contenu neweventstream liste)
      (MidiSendIm *refnum* neweventstream))))	; send the event immediatly

(defun midi-new-ev (evt-type)    ;;; eviter engorgement (cf Tableau de bord MidiShare, Carlos, 16 septembre 2002)
  (let ((e (MidiNewEv evt-type)))
    (when (%null-ptr-p e)
      (midigrowspace 1000)
      (setf e (MidiNewEv evt-type))
      (if (%null-ptr-p e) (error "Midishare n'a plus de memoire")))
    e))
  
(defun mettre-contenu (neweventstream data)
  (do ((l data (cdr l)))
      ((null l) 'stop)
    (MidiAddField neweventstream (car l))))

;;; ============================================
;;; Conversion OM (5 listes) -> Max "midijoue" (2 coll)
;;; ============================================

#|
(defun formate-5-listes->2-coll (5-uplet)
  (let ((haut (car 5-uplet)) (onset (cadr 5-uplet)) (dur (caddr 5-uplet)) (vel (cadddr 5-uplet))
        (can (car (cddddr 5-uplet))) (res ()))
    (setf res (duree->noteoff haut onset dur vel can))
    (setf res (trier-date res))
    (setf res (regroupe-date res))
    (setf res (diff-date res))
    res))

|#

;;; pour Ofon attention le 2-coll devient un 3-col, la 3eme liste etant les dates audio

(defun formate-5-listes->2-coll (5-uplet)
  (let ((haut (car 5-uplet)) (onset (cadr 5-uplet)) (dur (caddr 5-uplet)) (vel (cadddr 5-uplet))
        (can (car (cddddr 5-uplet))) (res ()))
    (setf res (duree->noteoff haut onset dur vel can))
    (setf res (trier-date res))
    (setf res (regroupe-date res))
    (setf res (diff-date res))
    (when *Ofon-mode* (setf res (append res (list (make-3st-col (second res))))))
    res))



(defun duree->noteoff (haut onset dur vel can)
  (let ((res ()))
    (do ((l1 haut (cdr l1)) (l2 onset (cdr l2)) (l3 dur (cdr l3)) (l4 vel (cdr l4)) (l5 can (cdr l5)))
        ((null l1) res)
      (setf res (append res
                        (list-noteon (car l1) (car l2) (car l3) (car l4) (car l5))
                        (list-noteoff (car l1) (car l2) (car l3) (car l4) (car l5)))))))

(defun list-noteon (haut val-onset dur vel can)
  (if (not (listp haut))
    (list (list val-onset (+ 144 can) haut vel))
    (mapcar #'(lambda (h v c) (list val-onset (+ 144 c) h v)) haut vel can)))

(defun list-noteoff (haut val-onset dur vel can)   ;;; Carlos, 16 septembre 2002: enlever 5 msec. aux durees
  (if (not (listp haut))
    (list (list (+ val-onset (- dur 5)) (+ 144 can) haut 0))
    (mapcar #'(lambda (h d c) (list (+ val-onset (- d 5)) (+ 144 c) h 0)) 
            haut dur can)))

(defun trier-date (liste)
  (sort liste #'(lambda (l1 l2) (<= (car l1) (car l2)))))

;;; ============================================
;;; Conversion 
;;; Max live (liste events) -> Max "midijoue" (2 coll)
;;; ============================================

(defun formate-liste-events->2-coll (seq-tmp) (diff-date (regroupe-date seq-tmp)))

(defun regroupe-date (liste)
  (let ((res (list (car liste))))
    (do ((l (cdr liste) (cdr l)))
        ((null l) (reverse res))
      (let ((new-ev (car l)) (prec (car res)))
        (if (= (car new-ev) (car prec))
          (setf res (cons (append prec (cdr new-ev)) (cdr res)))
          (setf res (cons new-ev res)))))))

(defun diff-date (liste)
  (let ((date-diff ()) (ev ()))
    (do ((l liste (cdr l)) (i 1 (+ i 1)))
        ((null l) (list (reverse date-diff) (reverse ev)))
      (let  ((delta-suiv (dedoubler (if (null (cdr l)) 0 (- (caadr l) (caar l))))))
        (setf date-diff (cons (cons i delta-suiv) date-diff))
        (setf ev (cons (cons i (cdar l)) ev))))))

(defun dedoubler (duree)
  (list (floor duree 100) (mod duree 100)))      ;;; entiers < 100 (a cause de 127 utilise comme separateur)









;;; ---------------- ANNEXE --------------------
;;; Fonctions independante de la boucle (omax)
;;; --------------------------------------------


;;; ============================================
;;; FONCTION POUR PATCH "omaxbouclecoll" OPEN MUSIC
;;; ============================================
;;; Pour preparer des sequences midi dans OpenMusic,
;;; ecriture de fichiers textes au format "midijoue" de Max:
;;; (fichiers ecrits dans dossier "Images" de "OM4.0")

(defun 5-listes->fichier-max-midijoue (5-uplet fich)
  (delete-file fich)
  (with-open-file (my-flot fich :direction :output :if-does-not-exist :create :if-exists :supersede)
    (ecrire-max-midijoue 5-uplet my-flot)))

(defun ecrire-max-midijoue (5-uplet flot)
  (let ((2-coll (formate-5-listes->2-coll 5-uplet)))
    (do ((l1 (car 2-coll) (cdr l1)) (l2 (cadr 2-coll) (cdr l2))
         (i 1 (+ i 2)))
        ((null l1) 'stop)
      (ecrire-ligne i 1 (cdr (car l1)) flot)
      (ecrire-ligne (+ i 1) 2 (cdr (car l2)) flot))))

(defun ecrire-ligne (i route l flot)
  (write i :stream flot) (write-char #\, flot) (write-char #\space flot) (write route :stream flot)
  (mapcar #'(lambda (x) (write-char #\space flot) (write x :stream flot)) l)
  (write-char #\; flot) (write-char #\newline flot))



;;; ============================================
;;; Conversion Max "midijoue" (2 coll) -> OM liste de 5-uplets
;;; (format accepte par midi->cross)
;;; ============================================
;;; fonction recuperee de 5-3-Baseimprolz.lisp

(defun 2-coll->liste-5uplets (2-coll)
  (let ((liste-date-ev (date-diff 2-coll)))
    (noteoff->duree liste-date-ev)))

;;; permet de faire aussi cette conversion:
(defun 5listes->liste-5uplets (5listes)
  (2-coll->liste-5uplets (formate-5-listes->2-coll 5listes)))

(setf test-2-coll    ;;; pour tester (debut 2eme phrase d'impro sur "Rain")
'(((1 0 67) (2 0 49) (3 0 46) (4 0 80) (5 0 25) (6 0 45) (7 0 45) (8 0 60) (9 0 30) (10 0 35) (11 0 29) (12 0 23) (13 1 3) (14 0 10) (15 0 60) (16 0 35) (17 0 80) (18 0 35) (19 0 25) (20 0 80)) 
  ((1 248) (2 146 65 36) (3 146 65 0) (4 146 66 35) (5 146 66 0) (6 146 67 34) (7 146 67 0) (8 146 68 32) (9 146 68 0) (10 146 69 60) (11 248) (12 146 69 0) (13 146 70 33) (14 146 70 0) (15 146 71 59) (16 146 71 0) (17 146 72 37) (18 146 73 33 146 72 0) (19 146 73 0) (20 146 74 46)))
)

;;; date-diff retourne une liste de 
;;; (date (144+canal pitch vel)) ou (date (248)) pour les clocks
 
(defun date-diff (2-coll)
  (let ((res ()) (date-courante 0))
    (do ((liste-diff (car 2-coll) (cdr liste-diff)) 
         (liste-ev (cadr 2-coll) (cdr liste-ev)))
        ((null liste-ev) (reverse res))
      (let* ((ev-separes (isole-ev-midi (cdr (car liste-ev))))
             (delta-date (cumuler (cdr (car liste-diff))))
             (ev-dates (mapcar #'(lambda (l) (list date-courante l)) ev-separes)))
        (setf res (append ev-dates res))
        (setf date-courante (+ date-courante delta-date))))))

(defun cumuler (couple-dur) (+ (* 100 (car couple-dur)) (cadr couple-dur))) 

(defun isole-ev-midi (ev)
  (cond ((null ev) ())
        ((and (>= (car ev) 144) (<= (car ev) 159))  ;;; NoteOn
         (cons (list (car ev) (cadr ev) (caddr ev)) (isole-ev-midi (cdddr ev))))
        ((= (car ev) 248)  ;;; Clock
         (cons (list (car ev)) (isole-ev-midi (cdr ev))))
        (t (isole-ev-midi (cdr ev)))))

; (isole-ev-midi '(146 73 33 146 72 0))
(setf liste-date-ev
'((0 (248)) (67 (146 65 36)) (116 (146 65 0)) (162 (146 66 35)) (242 (146 66 0)) (267 (146 67 34)) (312 (146 67 0)) (357 (146 68 32)) (417 (146 68 0)) (447 (146 69 60)) (482 (248)) (511 (146 69 0)) (534 (146 70 33)) (637 (146 70 0)) (647 (146 71 59)) (707 (146 71 0)) (742 (146 72 37)) (822 (146 72 0)) (822 (146 73 33)) (857 (146 73 0)) (882 (146 74 46)))
)

;(noteoff->duree liste-date-ev)

(defun noteoff->duree (liste-date-ev)
  (let ((res ()))
    (do ((l liste-date-ev (cdr l)))
        ((null l) (reverse res))
      (let ((date (car (car l))) (ev (cadr (car l))))
        (when (and (not (null (cdr ev)))      ;;; elimine les clocks 248
                   (not (= 0 (caddr ev))))    ;;; et les NoteOff
          (let* ((canal (- (car ev) 144))
                 (pitch (cadr ev))
                 (date-noteoff (cherche-noteoff canal pitch (cdr l)))   ;;; ATTENTION cherche-noteoff = date, pas duree
                 (dur (if (null date-noteoff) 0 (- date-noteoff date))))
            (when (> dur 0) (setf res (cons (list pitch (car (car l)) dur (caddr ev) canal) res)))))))))

; (cherche-noteoff 2 71 '((647 (146 71 59)) (707 (146 71 0)) (742 (146 72 37)) (822 (146 72 0)) (822 (146 73 33)) (857 (146 73 0)) (882 (146 74 46))))
(defun cherche-noteoff (canal pitch liste-date-ev)
  (cond ((null liste-date-ev) ())
        ((and (not (null (cddr (cadr (car liste-date-ev)))))
              (= pitch (cadr (cadr (car liste-date-ev))))
              (= canal (- (car (cadr (car liste-date-ev))) 144))
              (= 0 (caddr (cadr (car liste-date-ev)))))
         (car (car liste-date-ev)))                       ;;; date correspondant au NoteOff
        (t (cherche-noteoff canal pitch (cdr liste-date-ev)))))

;;; pour tester:
(setf ex-2-coll 
'(((1 0 40) (2 0 10) (3 1 19) (4 0 6) (5 0 25) (6 2 85) (7 4 85) (8 0 0)) 
  ((1 248) (2 145 56 34) (3 145 66 37 145 61 46) (4 145 61 0) (5 145 56 0) (6 145 66 0) (7 248) (8 248))
))

