
(in-package :om)


;;; envoyer sur le port 3000
;(osc-send '((\toto 1 2 3)) "127.0.0.1" 3000)



;;; OSC receive process
;;; waits for messages in the form of
;;;           /lisp func-name arg1 arg2 ...
;;; or        /lisp-string "<S-expr>"
;;; or        /omsend obj-name method-name arg1 arg2 ...
;;; if method-name is a slot name, performs (setf (slot-value obj method-name) arg1) where obj is the object referred
;;; by obj-name. All names are packaged in :om.
;;; 

(in-package :oa)
(defun receive-packet-1 (endpoint buffer length)
  (ccl::rlet ((unit-data :TUnitdata)
         (addr :TNetBuf)
         (opt :TNetBuf)
         (udata :TNetBuf)
         (source :INetAddress)
         (flag :OTFlags))
    (setf (cl-user::pref addr :TNetBuf.maxlen) (ccl::record-length :INetAddress)
          (cl-user::pref addr :TNetBuf.buf) source
          (cl-user::pref opt :TNetBuf.maxlen) 0 
          (cl-user::pref opt :TNetBuf.buf) (ccl::%null-ptr)
          (cl-user::pref udata :TNetBuf.maxlen) length  
          (cl-user::pref udata :TnetBuf.buf) buffer
          (cl-user::pref unit-data :TUnitData.addr) addr
          (cl-user::pref unit-data :TUnitData.opt) opt
          (cl-user::pref unit-data :TUnitData.udata) udata)
      (if (eql  #$kOTNoError (#_OTRcvUData endpoint unit-data flag)) 
        (cl-user::pref (cl-user::pref unit-data :tunitdata.udata) :tnetbuf.len)
        nil)))
(in-package :om)




;;; handles arriving OSC Packets, routes messages to Lisp functions or methods
(defun OSC-listen (endpoint buffer)
   (let ((result nil))
     (loop while t do
           (handler-case
             (let ((bufsize (oa::receive-packet-1 endpoint buffer *max-buf-size*))
                   bundle)
               (if (null bufsize)
                 (process-allow-schedule)
                 (progn
                   (setf bundle (parse-buffer buffer 0 bufsize))
                   (print (list "=====" bundle))
                   (osc-route-bundle bundle)
                   )))
             (error (any)
                    (error any)
                    )))))

(defun osc-route-bundle (bundle)
  (handler-case
    (loop for item in bundle collect
          (cond
           ((stringp (car item)) 
            (cond ((string-equal (car item) "/omsend") (osc-method-invoke item))
                  ((string-equal (car item) "/lisp") (execute-message item))
                  ((string-equal (car item) "/lisp-string") (execute-message-from-string item))
                  (t (format t "~D~" item))))
           ((consp (car item)) (osc-route-bundle item))
           (t nil))
          )
    (error (any) (print "Error trying to route OSC message") (print any) NIL))
  )



(defun osc-method-invoke (invocation)
  (let ((object (symbol-value (intern  (string-upcase  (second invocation)) :om)))
        (method  (intern  (string-upcase  (third invocation)) :om)))
    (if (slot-exists-p  object method)
      (setf (slot-value object method) (fourth invocation))
      (apply  (symbol-function method) object (cdddr invocation)))))



(defclass osc-listener ()
  ((buffer :accessor buffer)
   (endpoint :accessor endpoint)
   (process :accessor process)))


(defun new-osc-listener (port bufsize name)
  (let ((self (mki 'osc-listener)))
    (setf (buffer self) (#_newptrclear bufsize)
          (endpoint self) (om-make-receive-connection port)
          (process self) (cl-user::process-run-function (list :name name) #'OSC-listen  (endpoint self)   (buffer self)))
    self))


(defmethod kill-osc-listener ((self osc-listener))
  (kill-receive (process self)  (endpoint self) (buffer self)))


;;; ============================================================================================================

(defclass MS-Player ()
  ((name :initform "ms-player" :accessor name :initarg :name :type string)
   (refnum :initform -1 :accessor refnum :initarg :refnum :type integer)
   (port :initform 0 :accessor port :initarg :port :type integer)
   (process  :accessor process :initarg :process :type ccl::process)
   (in-fifo :initform  (mki 'fifo) :accessor in-fifo :initarg :in-fifo :type fifo)
   (segdur :initform 500 :accessor segdur :initarg :segdur :type integer)
   (avance :initform 150 :accessor avance :initarg :avance :type integer)
   (start-play :initform nil :accessor start-play :initarg :start-play :type boolean)
   (next-time :initform 0 :accessor next-time :initarg :next-time :type integer)))


(defun MS-Player-process-function (self)
  (let ((segdur 0) (segment nil) (seg-onset 0) (event nil))
    (while t
           (process-wait "wait start" #'start-play self)
           (process-wait "wait nextseg" #'ready-for-next-seg self)
           (setf event (fifo-out (in-fifo self)))
           (if (null event) 
             (process-allow-schedule)
             (progn
               (incf segdur (duration event))
               (push event segment)
               (when (> segdur (segdur self)) ; time to play
                 (setf seg-onset 
                       (max (next-time self) (+ (ms:MidiGetTime) 50)))
                 (loop for evt in (nreverse segment) 
                       with delay = 0
                       do (loop for note in (MidiSet evt)
                                do (send-note-at note (refnum self) (+ seg-onset delay)))
                       sum (duration evt) into delay)
                 (setf (next-time self) (+ seg-onset segdur)
                       segdur 0
                       segment nil)
                 ))))))

(defun new-ms-player (name ms-port feeder)
  (let ((player (make-instance 'MS-player)))
    (setf (refnum player) (ms:midiopen name)
          (name player) name
          (port player) ms-port)
    ;(ms:MidiConnect ms-port (refnum player) -1)
    (ms:MidiConnect (refnum player) ms-port  -1)
    (ms:MidiConnect (refnum player) (1+ ms-port)  -1)
    (ms:MidiFlushEvs (refnum player))
    (setf (in-fifo player) (out-fifo feeder))
    (setf (process player)
          (cl-user::process-run-function (list :name name) #'MS-Player-process-function player))
    player))


(defmethod kill-ms-player ((self MS-player))
  (process-kill (process self))
  (ms:MidiConnect (refnum self) (port self) 0)
  ;(ms:MidiConnect  (port self) (refnum self) 0)
  (ms:midiclose (refnum self))
  )

(defmethod reset-ms-player ((self ms-player))
  (process-reset (process self))
  (fifo-empty (in-fifo self))
  (setf (next-time self) 0)
  (process-preset (process self) #'MS-Player-process-function  self)
  (process-enable (process self))
  )

(defmethod ready-for-next-seg ((self MS-Player))
  (or (zerop (Next-time self))
      (<= (- (Next-time self) (ms:MidiGetTime))
          (avance self))))

(defmethod look-time-ahead ((self ms-player))
  (duration (in-fifo self))
  )


;;; ============================================================================================================

(defclass MS-Listener ()
  ((name :initform "ms-listener" :accessor name :initarg :name :type string)
   (refnum :initform -1 :accessor refnum :initarg :refnum :type integer)
   (port :initform 0 :accessor port :initarg :port :type integer)
   (start-time :initform 0 :accessor start-time :initarg :start-time :type integer)   
   (process  :accessor process :initarg :process :type ccl::process)
   (out-fifo :initform  (mki 'fifo) :accessor out-fifo :initarg :out-fifo :type fifo)))


(defun listen-to-midishare (listener)
   (let ((event) )
     (while t
            (setf event (ms:MidiGetEv (refnum listener)))
            (if (%null-ptr-p event) 
              (process-allow-schedule)
              (case  (ms:type event)
                (#.ms:typeActiveSens)
                ((#.ms:typeKeyOn #.ms:typeKeyOff)
                 ;(ms:MidiSendIM (refnum listener) (ms:MidiCopyEv event))
                 (fifo-in (out-fifo listener)  
                          (new-midi-event (ms:pitch event) (- (ms:date event) (start-time listener)) 
                                          (ms:dur event) (if (eq (ms:type event) #.ms:typeKeyOff) 0 (ms:vel event) )
                                          (1+ (ms:chan event))))
                 (ms:MidiFreeEv event)
                 ))))))

(defun new-ms-listener (name ms-port)
  (let ((listener (make-instance 'MS-Listener)))
    (setf (refnum listener) (ms:midiopen name)
          (name listener) name
          (port listener) ms-port)
    (ms:MidiConnect ms-port (refnum listener) -1)
    (ms:MidiConnect (refnum listener) ms-port  -1)
    (ms:MidiFlushEvs (refnum listener))
    (setf (start-time listener) (ms:MidiGetTime))
    (setf (process listener)
          (cl-user::process-run-function (list :name name) #'listen-to-midishare  listener))
    listener))


(defmethod ms-listener-kill ((self MS-Listener))
  (process-kill (process self))
  (ms:MidiConnect (refnum self) (port self) 0)
  (ms:MidiConnect  (port self) (refnum self) 0)
  (ms:midiclose (refnum self))
  )


(defun ms-appls ()
  (format t "List of MidiShare client applications ~%")
  (dotimes (i (ms:MidiCountAppls))
    (let ((ref (ms:MidiGetIndAppl (1+ i))))
      (format t 
              " ~2d : reference number ~2d, name : '~a' ~%" 
              (1+ i) ref (ms:MidiGetName ref)))))



;;; ============================================================================================================


(defclass  fifo ()
   ((head :initform (list nil) :accessor head :initarg :head :type list)
    (tail  :initform nil :accessor tail :initarg :tail :type list) 
    (fifo-length :initform 0 :accessor fifo-length :initarg :fifo-length :type integer) 
    (duration :initform 0 :accessor duration :initarg :duration :type integer)
    (lock  :initform (make-lock 'fifo-lock)  :accessor lock :initarg :lock :type ccl::lock)))

(defmethod initialize-instance ((self fifo) &rest initargs)
  (declare (ignore initargs))
  (call-next-method)
  (setf (slot-value self 'tail) (slot-value self 'head))
  self)

(defmethod fifo-in ((self fifo) (object t))
   (with-lock-grabbed ((lock self))
     (setf (rest (tail self)) (cons object nil)
           (tail self) (rest (tail self)))
     (incf (fifo-length self))
     (incf (duration self) (fifo-obj-duration object))))


(defmethod fifo-out ((self fifo) )
   (with-lock-grabbed ((lock self))
     (if (null (rest (head self)))
       nil
       (let ((object  (first (rest (head self)))))
         (prog1 object
           (setf (rest (head self)) (cddr (head self)))
           (unless (rest (head self)) (setf (tail self) (head self)))
           (decf (fifo-length self) )
           (decf (duration self) (fifo-obj-duration object)))))))


(defmethod fifo-empty ((self fifo))
   (with-lock-grabbed ((lock self))
     (setf (head self) (list nil) (tail self) (head self) (fifo-length self) 0 (duration self) 0)))


(defmethod fifo-size ((self fifo))
   (with-lock-grabbed ((lock self))
     (fifo-length self)))


(defmethod fifo-obj-duration ((self crossevent))
  (duration self))

(defmethod fifo-obj-duration ((self t))
  0)




;;; ============================================================================================================
;;; Midi basic events are 5-uple
;;; MEPitch, MEOnset, Medur, MEVel, MEChannel
;;; Raw events have no dur and keyon-keyoff is encoded by 0 vel

(defun new-midi-event (pitch onset dur vel chan)
  (list pitch onset dur vel chan))


(defmacro MEPitch (event)
  `(first ,event))

(defmacro MEOnset (event)
  `(second ,event))

(defmacro MEDur (event)
  `(third ,event))

(defmacro MEVel (event)
  `(fourth ,event))

(defmacro MEChannel (event)
  `(fifth ,event))

(defmacro IsKeyOff? (event)
  `(zerop (MEVel ,event)))


;;; Midishare utils


(defun send-note-at (MEnote refnum at)
   (let ((event (ms:MidiNewEv ms:typeNote)))	; ask for a new note event
     (unless (ccl:%null-ptr-p event)	; if the allocation was succesfull
       (ms:chan event (1- (MEChannel MEnote)))			; set the midi channel to 0 (means channel 1)
       (ms:port event 0)			; set the destination port to 0
       (ms:pitch event (MEPitch MEnote))		; set the pitch field
       (ms:vel event  (MEVel MEnote))		; set the velocity field
       (ms:dur  event (MEdur MEnote))		; set the duration field to 1 second
       (ms:MidiSendAt refnum event at))	; send the note at
     ))


#|
(defun send-note-at (MEnote refnum at)
  (let ((event (ms:MidiNewEv ms:typeNote)) copy)	; ask for a new note event
    (unless (ccl:%null-ptr-p event)	; if the allocation was succesfull
      (setf copy (ms:MidiCopyEv event))
      (ms:chan event (1- (MEChannel MEnote)))			; set the midi channel to 0 (means channel 1)
      (ms:port event 0)			; set the destination port to 0
      (ms:pitch event (MEPitch MEnote))		; set the pitch field
      (ms:vel event  (MEVel MEnote))		; set the velocity field
      (ms:dur  event (MEdur MEnote))		; set the duration field to 1 second
      (ms:MidiSendAt refnum event at)
      (ms:port copy 1)			        ; set the destination port to 0
      (ms:MidiSendAt refnum copy at)           ; send the note at
      )	
    ))


|#


;;; ============================================================================================================
;;; A class and process that slices a stream of MidiEvents (5uples) into a list of crossevents
;;; typical call : (new-polySlicer  "Slicer-0" feeder)
;;; where feeder is an object with a slot 'fifo-out, from which the MidiEvents will be popped.


(defclass PolySlicer ()
  ((magnet1 :initform 0 :accessor magnet1 :initarg :magnet1 :type integer)
   (magnet2 :initform 0 :accessor magnet2 :initarg :magnet2 :type integer) 
   (delta-t :initform 50 :accessor delta-t :initarg :delta-t :type integer)
   (running-notes :initform nil :accessor running-notes :initarg :running-notes :type list)
   (process  :accessor process :initarg :process :type ccl::process)
   (in-fifo :initform  (mki 'fifo) :accessor in-fifo :initarg :in-fifo :type fifo)
   (out-fifo :initform  (mki 'fifo) :accessor out-fifo :initarg :out-fifo :type fifo)))

;------ INTERFACE -------------------------------

(defun new-PolySlicer (name feeder)
  (let ((self (make-instance 'PolySlicer)))
    (setf 
     (in-fifo self) (out-fifo feeder)
     (process self)
     (cl-user::process-run-function (list :name name) #'slicer-process-function  self))
    self))


(defmethod kill-slicer ((self PolySlicer))
  (process-kill (process self))
  )

 
;------ INTERFACE -------------------------------

(defun slicer-process-function (self)
  (let ((event) )
    (while t
           (setf event (fifo-out (in-fifo self)))
           (if (null event) 
             (process-allow-schedule)
             (progn
               ;(print (list "eater:" event))
               (eat-note-evt self event))))))

(defmethod eat-note-evt ((self polySlicer) (MEnote list))
  (if (IsKeyOff? MeNote) (eat-note-off self MENote) (eat-note-on self MENote)))

(defmethod eat-note-on ((self polySlicer) (MEnoteon list))
  (cond
   ((in-time-window1 self  MENoteOn)
    (setf (MEOnset MeNoteOn) (magnet1 self))
    (add-to-run-notes self MeNoteOn))
   (t (setf (magnet2 self) (MEOnset MeNoteOn))
      (when (running-notes self) (output-slice1 self))
      (setf (magnet1 self) (magnet2 self))
      (add-to-run-notes self MENoteOn)))
  self)


(defmethod eat-note-off ((self polySlicer) (MEnoteoff list))
   (cond
    ((in-time-window1 self  MENoteOff)
     (setf (MEOnset MeNoteOff) (magnet1 self))
     (remove-from-run-notes self MeNoteOff))
   (t (setf (magnet2 self) (MEOnset MeNoteOff))
      (when (running-notes self) (output-slice1 self))
      (setf (magnet1 self) (magnet2 self))
      (remove-from-run-notes self MENoteOff)))
  self)


(defmethod in-time-window1 ((self polySlicer) (MEnoteon list))
   (<= (magnet1 self) (MEOnset MENoteOn) (+ (magnet1 self) (delta-t self))))


(defmethod add-to-run-notes ((self polySlicer) (MEnoteon list))
  (push MeNoteOn (running-notes self)))

(defmethod remove-from-run-notes ((self polySlicer) (MEnoteoff list))
  (setf (running-notes self)
        (delete-if #'(lambda (note) (and (= (MEPitch note)  (MEPitch MENoteOff))
                                         (= (MEChannel note)  (MEChannel MENoteOff))))
                   (running-notes self)))
  self)

(defmethod output-slice1 ((self polySlicer))
  (let ((cross (mki 'crossevent))
        (dur (- (magnet2 self) (magnet1 self))))
    (setf (duration cross) dur
          (MidiSet cross)
          (loop for note in (running-notes self)
                collect 
                (new-midi-event  
                 (if (< (MEOnset note) (magnet1 self))
                   (- (MEPitch note))
                   (MEPitch note))
                 0 
                 dur
                 (MEVel note) 
                 (MEChannel note)))
          
          )
    ;(format t "~%Slice : ~D ~D~%" (duration cross) (MidiSet cross))
    (fifo-in (out-fifo self) cross)
    ;(pushr cross *crosslist*)
    cross))

;;; ============================================================================================================


(defclass UnSlicer ()
  ((process  :accessor process :initarg :process :type ccl::process)
   (in-fifo :initform  (mki 'fifo) :accessor in-fifo :initarg :in-fifo :type fifo)
   (out-fifo :initform  (mki 'fifo) :accessor out-fifo :initarg :out-fifo :type fifo)))

;------ INTERFACE -------------------------------

(defun new-UnSlicer (name feeder)
  (let ((self (make-instance 'UnSlicer)))
    (setf 
     (in-fifo self) (out-fifo feeder)
     (process self)
     (cl-user::process-run-function (list :name name) #'unslicer-process-function  self))
    self))


(defmethod kill-unslicer ((self unSlicer))
  (process-kill (process self))
  )
;------ INTERFACE -------------------------------


(defun unslicer-process-function (self)
  (loop
    for event = (fifo-out (in-fifo self))
    with new-waiting-list
    for waiting-list = nil then new-waiting-list
    if (null event) do (process-allow-schedule) 
    else do
    (setf event (clone-object event))
    (loop for ME in (midiset event)
          with new-midiset = nil
          initially do (setf new-waiting-list nil)
          do (cond ((plusp (MEPitch ME))
                    (push ME new-waiting-list)
                    (push ME new-midiset))
                   ((minusp (MEPitch ME))
                    (let ((waiting (find ME waiting-list 
                                         :test #'(lambda (m1 m2) 
                                                   (and (= (abs (MEPitch m1)) (abs (MEPitch m2))) 
                                                        (= (MEChannel m1) (MEChannel m2)))))))
                      (cond 
                       (waiting
                        (setf (MEDur waiting) (+ (MEDur waiting) (MEDur ME)))
                        (push waiting new-waiting-list))
                       (t (setf (MEPitch ME) (abs (MEPitch ME)))
                          (push ME new-waiting-list)
                          (push ME new-midiset)))))) ; might be empty if only running notes
          finally (progn (setf (Midiset event) new-midiSet) 
                         (fifo-in (out-fifo self) event)
                         (pushr event *crosslist*))))
  self)

;;; ============================================================================================================

(defclass* cimprovizer (improvizer)
   ((learn-process  :accessor learn-process :initarg :learn-process :type ccl::process)
    (play-process  :accessor play-process :initarg :play-process :type ccl::process)
    (in-fifo :initform  (mki 'fifo) :accessor in-fifo :initarg :in-fifo :type fifo)
    (out-fifo :initform  (mki 'fifo) :accessor out-fifo :initarg :out-fifo :type fifo)
    (lock  :initform (make-lock 'impro-lock)  :accessor lock :initarg :lock :type ccl::lock)
    (start-play :initform nil :accessor start-play :initarg :start-play ) 
    (gindex :initform 0  :accessor gindex :initarg :gindex :type integer)
    (last-learned :initform nil  :accessor last-learned :initarg :last-learned :type crossevent)))


;------ INTERFACE -------------------------------

(defun new-improvizer (name feeder)
  (let ((self (make-instance 'cimprovizer :comparateur 'CompareEvents :lrsMode t)))
    (setf 
     (in-fifo self) (out-fifo feeder)
     (learn-process self)
     (cl-user::process-run-function 
      (list :name (concatenate 'string name "-learn"))
      #'improvizer-learn-process-function  self)
     (play-process self)
     (cl-user::process-run-function 
      (list :name (concatenate 'string name "-play")) 
      #'improvizer-play-process-function  self))
    self))


(defmethod kill-improvizer ((self improvizer))
  (process-kill (learn-process self))
  (process-kill (play-process self))
  )


(defmethod reset-improvizer ((self cimprovizer))
   (process-reset (learn-process self))
   (process-reset (play-process self))
   (setf (vectext self) (make-array 1000 :adjustable t))
   (when (lrsMode self) (setf (veclrs self) (make-array 1000 :adjustable t)))
   (setf (maxetat self) 0)
   (setf (hashtransition self) (make-hash-table :test 'equal)
         (hashsuppl  self) (make-hash-table :test 'equal)
         (hashsuppl->  self) (make-hash-table :test 'equal))
   (setf (suppleance self 0) -1  (gindex self) 0)
   (process-preset (learn-process self) #'improvizer-learn-process-function  self)
   (process-enable (learn-process self))
   (process-preset (play-process self) #'improvizer-play-process-function  self)
   (process-enable (play-process self))
   self)




(defmethod clone-improvizer ((self cimprovizer))
  (process-disable (play-process self))
  (process-disable (learn-process self))
  (while (or (process-active-p (play-process self))
             (process-active-p (learn-process self))))
  (let ((clone (clone self)))
    
    (process-enable (play-process self)) 
    (process-enable (learn-process self)) 
    clone))


(defmethod restore-improvizer ((self cimprovizer) (clone cimprovizer))
   (process-disable (play-process self))
   (process-disable (learn-process self))
   (while (or (process-active-p (play-process self))
              (process-active-p (learn-process self))))
   (fifo-empty (in-fifo self))
   (fifo-empty (out-fifo self))
   (setf (vectext self) (vectext clone)
         (veclrs self) (veclrs clone) 
         (lrsMode self) (lrsMode clone)
         (maxetat self) (maxetat clone)
         (hashtransition self) (hashtransition clone)
         (hashsuppl  self) (hashsuppl  clone)
         (hashsuppl->  self) (hashsuppl->  clone)
         (suppleance self 0) -1  
         (gindex self) 0
         (start-play self) t)
   (process-enable (play-process self))
   (process-enable (learn-process self))
   self)


(defmethod restore-old-improvizer ((self cimprovizer))
  (let ((orchestre (load-orchestre-from)))
    (when (neq orchestre :cancel)
      (let ((clone (aref (improvizer-array orchestre) 0)))
        (process-disable (play-process self))
        (process-disable (learn-process self))
        (while (or (process-active-p (play-process self))
                   (process-active-p (learn-process self))))
        (fifo-empty (in-fifo self))
        (fifo-empty (out-fifo self))
        (setf (vectext self) (vectext clone)
         (veclrs self) (veclrs clone) 
         (lrsMode self) (lrsMode clone)
         (maxetat self) (maxetat clone)
         (hashtransition self) (hashtransition clone)
         (hashsuppl  self) (hashsuppl  clone)
         (hashsuppl->  self) (hashsuppl->  clone)
         (suppleance self 0) -1  
         (gindex self) 0
         (start-play self) t)
        (process-enable (play-process self))
        (process-enable (learn-process self))
        self))))

;(inspect (clone-improvizer *improvizer*))


;------ INTERFACE -------------------------------


(defun improvizer-learn-process-function (self)
  (loop
    for event = (fifo-out (in-fifo self))
    if (null event) do (process-allow-schedule) 
    else do
    (setf *current-improvizer* self) ;; this doesn't work concurrently !
    (with-lock-grabbed ((lock self))
      (setf (last-learned self) event)
      (ajouter-objet self event))))

(defun improvizer-play-process-function (self)
  ;(process-wait "silence" #'start-play self)
  (cimprovize self)
  )
  

(defmethod cImprovize ((self improvizer)  &optional (harmgrid nil))
  (format t  "Region: ~a Suff. law: ~a Best Suff.: ~a Max Cont.: ~a Hindemith : ~a Estrada ~a~%" 
          (start-region self) (list (bwsuffix self) (fwsuffix self)) (bestSuffixMode self) (max-continuity self)
          (useHindemith self) (useEstrada self))
  
  (setf harmgrid (refHarmGrid self))
  
  (when (null (veclrs self)) (setf (bestSuffixMode self) nil)
        (format t "Old Oracle, setting bestSuffix mode to nil~%"))
  
  (loop ;for i from 1 to length
    for label =  (pop harmgrid)
    ;with index = 0
    with previndex = -1
    with index2
    with links
    with suppleances
    with mode = 'continuity
    with tocollect = nil
    
    initially (setf (continuity self) 0)
    do
    (process-wait "silence" #'start-play self)
    (process-wait "empty" #'(lambda (s) (> (maxetat s) 0))  self)
    (if (>= (fifo-size (out-fifo self)) 10)
      (process-allow-schedule)
      (with-lock-grabbed ((lock self))
        (setf previndex (gindex self))
        (cond
         ((and (zerop (gindex self)) (setf index2 (find-beat-label-match self label)))
          (format t "Starting point : ~a ~%" index2)
          (setf tocollect (otext self index2)
                (gindex self) index2))
         (t (if (check-continuity self) 
              (setf mode 'continuity)  
              (if (available-suppleance self (gindex self)) 
                (setf mode 'suppleance)
                (setf mode 'continuity)))
            (when (eq mode 'continuity)
              ;(when (= index (maxetat self)) (setf index 0)
              ;      (format t "zero~%" ))
              (setf links (flink self (gindex self))
                    index2 (choose-factor-link self links label))
              (if index2 
                (progn
                  ;(format t "c : ~a ~%" index2)
                  (setf (gindex self) index2
                        tocollect  (otext self index2)))
                (if (available-suppleance self (gindex self)) 
                  (setf mode 'suppleance)
                  (setf mode 'nothing))))
            (when (eq mode 'suppleance)
              (setf index2 (continuations-by-suppleance self (gindex self) label))
              (if (and index2 (/= index2 previndex))
                (progn 
                  ;(format t "--->s : ~a ~%" index2)
                  (setf (gindex self) index2
                        tocollect (otext self index2) 
                        (continuity self) 0))
                (setf mode 'nothing)))
            (when (eq mode 'nothing)
              (setf index2 (find-beat-label-match self label))
              (if index2 
                (progn (setf tocollect (otext self index2)
                             (gindex self) index2))
                (progn               
                  (print 'empty)
                  (setf (gindex self) 0
                        tocollect nil ;(null-beat)
                        ))))))
        when tocollect do (fifo-in (out-fifo self) tocollect)
        ))))



;;; omax redefs


(defmethod CompareEvents ((Event1 CrossEvent) (Event2 CrossEvent))
   
   (let* ((m1 (delete-duplicates  
               (mapcar #'(lambda (x) (mod (abs (MEPitch x)) 12)) (MidiSet Event1))))
          (m2 (delete-duplicates  
               (mapcar #'(lambda (x) (mod (abs (MEPitch x)) 12)) (MidiSet Event2))))
          (l1 (length m1))
          (l2 (length m2))
          (l3 (length (intersection m1 m2))))
     (or 
      (= l1 l2 l3)
      (>= l3 (/ (+ l1 l2) 3)))))
   
;;; ============================================================================================================
;;; proliferation mode


(defmethod proliferate ((self crossevent))
  (if (= (length (MidiSet self)) 1)
    (proliferate-1-note self)
    (proliferate-chord self)))

(defmethod proliferate-chord ((self crossevent))
  (let* ((LMidic (CrossEvt->LMidic self))
         (registre '(1000 10000))
         (auto (autotransp LMidic registre)) path)
    (when (rest auto) 
      (pop auto)
      (setf path (graph-tour (make-graph auto) (random (length auto))  1 1 2 1)))
    
    (loop for chord in path
          collect (mki 'crossevent 
                       :MidiSet 
                       (loop for ev in chord
                             collect (new-midi-event  (om-round ev 0 100) 0 (duration self) 
                                                      (MEVel (first (MidiSet self))) (MEChannel (first (MidiSet self)))))
                       :duration (duration self)))))

(defmethod proliferate-1-note ((self crossevent)) nil)

  ;(loop for i from 1 to 100 collect (clone-object self)))





;;; ============================================================================================================
(defclass ImproSupervizer ()
  ((process  :accessor process :initarg :process :type ccl::process)
   (ms-player :accessor ms-player)
   (improvizer :accessor improvizer)
   (proliferate :initform nil :accessor proliferate)))


(defun new-impro-supervizer (name msplayer improvizer)
  (let ((self (make-instance 'ImproSupervizer)))
    (setf (ms-player self) msplayer
          (improvizer self) improvizer
          (process self)
          (cl-user::process-run-function 
           (list :name name)  #'ImproSup-process-function self))
    self))

(defun ImproSup-process-function (self)
  (loop
    for timeahead = (look-time-ahead (ms-player self))
    do
    (if (>= timeahead 1000)
      (when (process-active-p (play-process (improvizer self)))
        (process-disable (play-process (improvizer self)))
        ;(print "disabling")
        )
      (when (not (process-active-p (play-process (improvizer self))))
        (process-enable (play-process (improvizer self))) 
        ;(print "enabling")
        ))
    (when (and (proliferate self) (last-learned (Improvizer self)))
      (let ((seq (proliferate  (last-learned (Improvizer self)))))
        (when
          seq  (loop for event in seq do (fifo-in  (in-fifo (improvizer self)) event))
          (setf (start-region (Improvizer self)) '(99 100)))))
    (setf (proliferate self) nil)
    (process-allow-schedule)))

(defmethod kill-supervizer ((self improSupervizer))
  (process-kill (process self))
  )



;;; ============================================================================================================

;;; OSC messages for controlling processes


(defun OMaxInitSession ()
  (print "Initializing Impro Session")
  (setf *crosslist* nil)
  (setf *mslisten* (new-ms-listener "MS-Listen-0" 0))
  (setf *slicer* (new-polySlicer  "Slicer-0" *mslisten*))
  (setf *Improvizer* (new-improvizer "Improvizer-0" *slicer*))
  (setf *unslicer* (new-unslicer "Unslicer-0" *Improvizer*))
  (setf *msplayer* (new-ms-player "MS-Play-0" 0 *unslicer*))
  (setf *supervizer* (new-impro-supervizer "Supervizer" *msplayer* *Improvizer*))
  (print "Initializing OK")
  )


(defun OMaxKillSession ()
  (print "Killing Impro Session")
  (kill-supervizer *supervizer*)
  (ms-listener-kill *mslisten*)
  (kill-slicer *slicer*)
  (kill-improvizer *Improvizer*)
  (kill-unslicer *unslicer*)
  (kill-ms-player *msplayer*)
  (print "Killing  OK")
  (ms-appls)
  )


(defun OMaxStartPlayer ()
  (setf (start-play *msplayer*) t))

(defun OMaxStopPlayer ()
  (setf (start-play *msplayer*) nil))

(defun OMaxStartGeneration ()
  (setf (start-play *improvizer*) t (gindex *improvizer*)  0))

(defun OMaxStopGeneration ()
  (setf (start-play *improvizer*) nil)
  )
 
;(format t "~D events learned~%" (Maxetat *improvizer*))


(defun OMaxResetPlayer ()
  (reset-ms-player *msplayer*))

(defun OMaxResetImprovizer ()
  (reset-improvizer *improvizer*))

(defun OMaxImpRange (min max)
  (setf (start-region *improvizer*) (list min max)))


(defun OMaxCloneImprovizer (numimp)
  (when (<= numimp *nb-clones*)
    (setf (aref *clone-array* (1- numimp))
          (clone-improvizer *improvizer*))))

(defun OMaxWakeClone (numimp)
  (when (and (<= numimp *nb-clones*)
             (aref *clone-array* (1- numimp)))
      (restore-improvizer *improvizer* (aref *clone-array* (1- numimp)))))

(defun OMaxRestoreOld ()
  (restore-old-improvizer  *improvizer*))

(defun OMaxProliferate ()
  (setf (proliferate *supervizer*) T))


;;; Initialisations
(defun OMaxOn ()
  
  (setf *nb-clones* 5)
  (setf *clone-array* (make-array *nb-clones* :initial-element nil))
  (setf osclistener (new-osc-listener 3005 *max-buf-size* "osc-listener-3005")))




;;;  la fin

(defun OMaxOff()
  (kill-osc-listener osclistener)
  (process-enable (process osclistener)))



#|

;;; ============================================================================================================


;;;  a penser : midifreeevt

(setf orc (load-orchestre-from))
(inspect orc)

(OMaxInitSession)
(OMaxKillSession)



(progn
  (setf *crosslist* nil)
  (setf msp (new-ms-listener "MS-Listen-0" 0))
  (setf slicer (new-polySlicer  "Slicer-0" msp))
  (setf Improvizer (new-improvizer "Improvizer-0" slicer))
  (setf unslicer (new-unslicer "Unslicer-0" improvizer))
  (setf player (new-ms-player "MS-Play-0" 0 unslicer))

  )


(reset-learn improvizer)

(setf (start-play *improvizer*) t)
(setf (start-play *improvizer*) nil)

(setf (start-play *msplayer*) t)
(setf (start-play *msplayer*) nil)


(reset-ms-player player)

(progn
(ms-listener-kill msp)
(kill-slicer slicer)
(kill-unslicer unslicer)
(kill-improvizer improvizer)
(kill-ms-player player))



(inspect msp)
(inspect slicer)
(inspect unslicer)
(inspect *improvizer*)
(inspect *msplayer*)
(inspect *clone-array*)

(process-reset (process slicer))
(process-preset (process slicer) #'slicer-process-function slicer)
(process-enable (process slicer))



(process-kill (first *all-processes*))


(
(loop for i from 1 to (maxetat improvizer) 
if (null (midiset (oText improvizer i))) do (prin t (list i "midiset null")))

(ms-appls)

(ms:Midiclose 5)

(loop for i from 4 to 5 do (ms:Midiclose i))


(setf *MsOrigTime* (ms:MidiGetTime))

(start-play improvizer)

(funcall #'start-play improvizer)


|#



#|
(setf ff (mki 'fifo))

(inspect ff)
(fifo-in ff 100)

(fifo-out ff)

(setf xx (cl-user::process-run-function 
           (list :name "foo")  #'foofoo ))

(defun foofoo () (loop for i = 0 then (mod (+ i 1) 1000000) if (zerop i) do (print 'o)))

(process-disable xx)
(process-enable xx)
(process-active-p xx)
(process-kill xx)



|#

#|

(defclass test-class () ((slot1) (slot2)))
(defmethod activate ((self test-class))
  (print "activating method for object *object*"))
(defun test () (print *object*))

(setf *object* (mki 'test-class))
(slot-value *object* 'slot1)

|#




#|

(time 
(without-interrupts(loop for i from 1 to 500 do
(osc-send (loop for i from 1 to 25 collect '(\toto 1 2 3)) "127.0.0.1" 3000))))

; 95 msecs 10 paquets de 10

(setf seq (loop for i from 1 to 400 collect '(\toto 1 2 3 4 5 6)))
(time 
(without-interrupts  (osc-send seq "127.0.0.1" 3000)))




(setf chord (mki 'chord))
(lmidic chord )
(funcall 'lmidic chord)

(setf (slot-value   chord (symbol-function 'lmidic)) '(99))
(slot-exists-p chord 'lmidic)

(intern  "foo")
(intern  (string-upcase  "foo") :om)

(function car)

(inspect chord)

(apply 'list chord '(1 2 3))
|#




#|

;;; a note-on note-off decoder

(defclass PolySlicer ()
  ((waiting-noteOn :initform nil :accessor waiting-noteOn :initarg :waiting-noteOn :type list)
   (note-list :initform nil :accessor note-list :initarg :note-list :type list)
))


(defmethod SendNoteEvent ((self polySlicer) (ME list))
  (if (IsKeyOff? ME)
    (SendNoteOff self ME)
    (SendNoteOn self ME)))

(defmethod SendNOteOn ((self PolySlicer) (ME list))
  (add-noteOn self ME)
  
)


(defmethod SendNOteOff ((self PolySlicer) (MENoteOff list))
  (let ((MENoteOn (Find-NoteOn self MENoteOff)))
    (when MENoteOn
      (add-MidiNote self
                    (new-midi-event (MEPitch MENoteOn) (MEOnset MENoteOn) 
                                    (- (MEOnset MeNoteOff) (MEOnset MeNoteOn))
                                    (MEVel MeNoteOn) (MEChannel MeNoteOn))))
    self))


(defmethod add-noteOn ((self PolySlicer) (ME list))
  (push ME (waiting-noteOn self)))


(defmethod find-noteOn ((self PolySlicer) (ME list))
  (find ME (waiting-noteOn self) :test #'(lambda (x y) (= (MEPitch x) (MEPitch y)))))


(defmethod add-MidiNote ((self PolySlicer) (note list))
  (push note (note-list self))
  (setf (note-list self)
        (sort (note-list self) #'< :key #'(lambda (note) (MEOnset note))))
  (print 
  (note-list self)))
  
|#

#|

(defun om-start-scheduler (symbol)
  (setf ccl::*eventhook* (list #'check-scheduler-mcl)))

;------
(defun om-stop-scheduler ()
  (setq ccl::*eventhook* nil))

|#