;OpenMusic
;
;Copyright (C) 1997, 1998, 1999, 2000 by IRCAM-Centre Georges Pompidou, Paris, France.
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
;
;Authors: Gerard Assayag and Augusto Agon


;DocFile
;This file defines the meta-object Library. Libraries are OMpackages load dinamicly.
;Last Modifications :
;18/10/97 first date.
;DocFile
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :om)

(defvar *current-lib* nil "This variable containts the current library that is being loaded.")

;--------------------
;The Lib Class in OM
;--------------------

#|
(defclass OMLib (OMpackage) 
   ((lib-pathname :initform nil :initarg :lib-pathname :accessor lib-pathname)
    (loaded? :initform nil  :accessor loaded?))
   (:documentation "The class for the OM Library  metaobjects.
One OMlib is a collection of classes and generic functions loaded dinamiclly.#enddoc#
#seealso# (ombasicobject ompackage) #seealso#
#lib-pathname# The pathname of the librarie folder. #lib-pathname#
#loaded?# T is the LIbrary is already loaded. #loaded?#"))
|#


;--------------------------------------------------
;Method redefinition of OMPackage
;--------------------------------------------------

(defmethod get-object-insp-name ((self OMLib)) "Library")

(defmethod get-class-icon ((self OMLib)) 'lib-icon-frame)

(defmethod OpenEditorframe :before ((self OMLib))
   "When you double click this method load the lib if it is not loaded."
   (unless (loaded? self)
     (load-om-lib self)))
;--------------------------------------------------
;Other methods
;--------------------------------------------------

;;; returns the folder of the library's icons
(defmethod resource-lib-file ((self OMLib))
   (make-pathname :directory (append (pathname-directory (lib-pathname self)) (list "Resources" "icon"))))

(defmethod load-om-lib ((self OMLib))
  "Load the librarie 'self'."
  (handler-bind ((error #'(lambda (c)
                            (hide-message-win)
                            (when *msg-error-label-on*
                              (om-message-dialog (string+ "Error while loading " (name self) " " 
                                                          (om-report-condition c))
                                                 :size (om-make-point 300 200))
                              (clear-after-error self)
                              (om-abort)))))
    (unless (loaded? self)
      (om-with-cursor *om-wait-cursor* 
        (let ((*current-lib* self))
          (def-icon-var self (resource-lib-file self))
          (show-message-win (string+ "Loading " (name self) " library...")) 
          (load (lib-pathname self))
          (hide-message-win)
          (setf (loaded? self) t)
          (lib2menu self))))))

(defmethod load-om-lib-without-menu ((self OMLib))
   "Load the librarie 'self' at starting, because the WorkSpace needs it."
   (unless (loaded? self)
     (om-with-cursor *om-wait-cursor* 
     (let ((*current-lib* self))
       (def-icon-var self (resource-lib-file self))
       (load (lib-pathname self))
       (setf (loaded? self) t)))))

(defmethod omlib-p ((self OMLib)) t)
(defmethod omlib-p ((self t)) nil)

;--------------------------------------------------
;Tools
;--------------------------------------------------


;----Builder


(defun omNG-make-new-lib (name)
  "Make an instance of the class OMLib."
  (let ((libname (if (pathnamep name)
                   (car (last (pathname-directory name)))
                   name))
        (libdir (if (pathnamep name)
                  (pathname-directory name)
                  (append (pathname-directory (OMroot "UserLibrary;")) (list name)))))
  (make-instance 'OMLib 
    :name (string-until-space libname) 
    :lib-pathname (make-pathname 
                   :directory libdir
                   :name (string-until-space libname) 
                   :type "lisp")
    :icon 170)))


;----Method definition
;See defmethod* the differences are
;  -Set the flag lib-fun-p of the generic function to the name of the librarie
;  -If the icon is in the resource fork file of the lib store icon as (IconID OMLiB)
(defmacro defmethod! (name &rest args)
   `(let ((themethod (defmethod* ,name ,.args)))
      (when (and *current-lib* (not (lib-fun-p (fdefinition ',name))))
        (if (listp (icon (fdefinition ',name)))
          (setf (icon (fdefinition ',name)) (car (icon (fdefinition ',name))))
          (setf (icon (fdefinition ',name)) 
                (list (icon (fdefinition ',name)) *current-lib*)))
        (setf (lib-fun-p (fdefinition ',name)) (string-until-space (name *current-lib*))))
      themethod))

 ;----Class definition
;See defclass* the differences are
;  -Set the flag lib-fun-p of the generic function to the name of the librarie
;  -If the icon is in the resource fork file of the lib store icon as (IconID OMLiB)
(defmacro defclass! (name superclass slots &rest class-options)
   `(let ((newclass (defclass* ,name ,superclass ,slots ,.class-options)))
      (when (and *current-lib* (not (lib-class-p (find-class ',name))))
        (if (listp (icon (find-class ',name)))
          (setf (icon (find-class ',name)) (car (icon (find-class ',name))))
          (setf (icon (find-class ',name)) 
                (list (icon (find-class ',name)) *current-lib*)))
        (setf (lib-class-p (find-class ',name)) (string-until-space (name *current-lib*))))
      newclass))

;----Generic function definition
;   See defgeneric* the differences are
;  -Set the flag lib-fun-p of the generic function to the name of the librarie
;  -If the icon is in the resource fork file of the lib store icon as (IconID OMLiB)
(defmacro defgeneric! (name lambda-list &rest options-and-methods &environment env)
   `(let ((thegenfun (defgeneric* ,name ,lambda-list ,.options-and-methods)))
      (when (and *current-lib* (not (lib-fun-p (fdefinition ',name))))
        (if (listp (icon (fdefinition ',name)))
          (setf (icon (fdefinition ',name)) (car (icon (fdefinition ',name))))
          (setf (icon (fdefinition ',name)) 
                (list (icon (fdefinition ',name)) *current-lib*)))
        (setf (lib-fun-p (fdefinition ',name)) (string-until-space (name *current-lib*))))
      themethod))

 
;Used when you have two or more more libraries with the same name, but different version.
(defun choise-a-lib (Libs name)
   (let* ((dialog (om-make-window 'om-dialog
                    :position :centered
                    :size (om-make-point 230 175)
                    :window-title "Multiple versions of lib"
                    :font *om-default-font1*
                    :close nil
                    :bg-color (om-make-color 0.874 0.874 0.874)))
            (tabla-ws (om-make-dialog-item 'om-single-item-list (om-make-point 60 50) (om-make-point 120 80) "Untitled"
                                                                        :range (loop for lib in LIbs collect (name-of-directory (lib-pathname LIb)))
                                                                        :on-double-click #'(lambda (win itemlist) 
                                                                                                             (let ((selected-index (om-get-selected-item-index itemlist)))
                                                                                                               (om-return-from-modal-dialog dialog (nth selected-index Libs))))
                                                                      ;;;;:cell-size (om-make-point 130 14)
                                                                      ;;;;:selection-type :single
                                                                      ;;;;:table-hscrollp nil
                                                                      ;;;;:table-sequence 
                                                                      ))
     )
     (om-add-subviews dialog tabla-ws 
                   (om-make-dialog-item 'om-static-text (om-make-point 10 23) (om-make-point 200 16) 
                                     (string+ "Choose a version of " name))
                   (om-make-dialog-item 'om-button (om-make-point 85 140) (om-make-point 72 25) "cancel" 
                                     :di-action (om-dialog-item-act item (declare (ignore item))
                                        (om-return-from-modal-dialog dialog nil))))
     (om-modal-dialog dialog)))

;Called when you start OM if the workspace needs the libraries in the list 'lib'
(defun load-lib-for-first (list)
  (without-interrupts 
   (loop for lib in list do
         (unless (member lib *skip-libs* :test 'string-equal)
           (let ((thelib (exist-lib-p lib)))
             (if thelib
               (load-om-lib-without-menu thelib)
               (progn
                 (dialog-message (string+ "This workspace can not be loaded because i don't find the library " lib))
                 (om-quit)
                 )))))))

;Return the OMLib instance with name 'name' if there two or more call the choiose-a-lib function.
(defun exist-lib-p (name)
   (let (libs)
     (loop for lib in (subpackages *library-package*) do
           (when (string-equal (string-until-space (name lib)) name)
             (push lib libs)))
     (when libs
       (if (= 1 (length libs)) (car libs)
           (choise-a-lib libs name)))))

; Resolve a little problem with the lib situation
; Libraries name must not include the version
; Situation2.1 is a bad name you must name it as situation 2.1
(defun situationcorrige (lib)
 (if (and (> (length lib) 10) (string-equal (subseq lib 0 11) "OMsituation"))
    "OMsituation" lib))


;Make a list of OMLib instance from directories in the 'CL:UserLibrary;' directory"
(defun load-om-libs ()
  (let ((elements (om-directory (OMroot  "UserLibrary;") 
                                :files nil :directories t)))
    (mapc #'(lambda (x)
                      (AddPackage2Pack (omNG-make-new-lib (car (last (pathname-directory x)))) *library-package*))
;;;              (omNG-add-element *library-package* 
;;;                                (omNG-protect-object (omNG-make-new-lib (car (last (pathname-directory x)))))))
          elements)))


(defun add-one-lib (pathname)
  (let ((libname (string-until-space (car (last (pathname-directory pathname))))))
    (if (probe-file (print (make-pathname :directory (pathname-directory pathname)
                                   :name libname
                                   :type "lisp")))
      (AddPackage2Pack (omNG-make-new-lib pathname) *library-package*)
      (om-message-dialog "This library does not contains initfile!" :window-title "Error!"))
    (show-libs)))
    
(defun show-libs ()
  (unless (editorframe *om-package-tree*)
    (openobjecteditor *om-package-tree*))
  (loop for sub in (get-subframes (editorframe *om-package-tree*))
        do (when (equal (object sub) *library-package*)
             (when (open? (triangle sub))
               (open/close-triangle (triangle sub) nil))
             (open/close-triangle (triangle sub) t)))
  )


;Init the subpackages of the current library from a list of the form :
; ("sub-pack-name" subpacke-lists class-list function-list class-alias-list)
(defun fill-library (list)
  (loop for item in list do
        (add-new-packages item *current-lib* nil 170 t)))


;Mount functions, classes and sub-package of 'lib' to the menubar.
(defun lib2menu (lib)
   (let ((menufun (om-new-menu (name lib)))
         (menuclass (if (or (subpackages lib) (classes lib)) (om-new-menu (name lib)))))
     (when (and menufun (om-find-menu-item *patch-menu-classes* "Libraries"))
       (om-add-menu-items (om-find-menu-item *patch-menu-functions* "Libraries") t menufun)
       (when menuclass
         (om-add-menu-items (om-find-menu-item *patch-menu-classes* "Libraries") t menuclass))
       (packages2menu menufun menuclass lib))))


(defun require-library (nameORlist)
   (let ((list (list! nameORlist)))
     (loop for lib in list do
           (let ((thelib (exist-lib-p (situationcorrige lib))))
             (if thelib
               (unless (loaded? thelib)
                 (load-om-lib thelib))
               (progn
                 (dialog-message (string+ "This patch can not be loaded because i don't find the library " lib))
                 (om-abort)))))))

(defun get-library-from-name (name)
   (let (rep)
     (loop for lib in (subpackages *library-package*)
           while (not rep) do
           (when (string-equal (string-until-space (name lib)) name)
             (setf rep lib)))
     rep))

