;OpenMusic
;
;Copyright (C) 1997, 1998, 1999, 2000 by IRCAM-Centre Georges Pompidou, Paris, France.
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
;
;Authors: Gerard Assayag and Augusto Agon

;DocFile
;This file defines the OMWorkspace meta-object, it is not a computable objects in OM.
;The Workspace implements an idea of persistence, that is you expect to find your workspace as you 
;left it at the end of the previous session. 
;Last Modifications :
;18/10/97 first date.
;DocFile
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(in-package :om)

(defvar *current-workSpace* nil "The WorkSpace used in the current session.")
(defvar *om-workSpace-win* nil "The WorkSpace's editor window is always opened, this window is kept in this variable.")

(defvar *workSpace-win-size* nil "The WorkSpace's editor window size")
(defvar *workSpace-win-pos* nil "The WorkSpace's editor window position")

;-----------------
;The WorkSpace in OM
;-----------------
#|
(defclass OMWorkSpace (OMPersistantFolder) 
   ((elements :initform nil :accessor elements)
    (packages :initform nil :accessor packages))
   (:documentation "The class of the OM Workspace. #enddoc#
#seealso# (OMFolder OMPackage) #seealso#
#elements# A list of objects contained in the Folder i.e. patches, maquettes or  folders. #elements#
#packages# Point to the main package of the session. #packages#"))
|#

;--------------------------------------------------
;Method redefinition of OMpersistantObject
;--------------------------------------------------

(defmethod get-elements ((self OMWorkSpace)) 
   "WorkSpace's elements are folders patches or maquettes." (elements self))

(defmethod get-class-icon ((self OMWorkSpace)) 'ws-icon-frame)

(defmethod get-object-insp-name ((self OMWorkSpace)) "WorkSpace")

(defmethod presentation ((self OMWorkSpace))
  (if (and (wsparams self) (integerp (car (wsparams self))))
    (car (wsparams self)) 0))

(defmethod set-presentation ((self OMWorkSpace) val)
  (when (wsparams self)
    (setf (nth 0 (wsparams self)) val)))

;--------------------------------------------------
;Other Methods
;--------------------------------------------------
;For workspaces the elements are not in the pathname, but in "pathname:elements:" directory.
(defmethod elements-pathname ((self OMWorkSpace)) 
   (make-pathname :directory  (append (pathname-directory (mypathname self)) (list "elements"))))


(defmethod InitWorkSpace ((self OMWorkSpace))
   "Load the globals folder, the package folder, the preferences file and all elements of 'self'."
   (declare (special *om-package-tree* *package-user*))
   (om-with-load-verbose nil
      (om-with-compiler-warnings nil
     (om-select-window *presentation*)
     (intiglobalslist)
     (init-globalsfolder)
     (setf  *current-workSpace* self)
     (load (make-pathname :directory  (pathname-directory (mypathname self)) :name "userpackage" :type "lisp"))
     (load-package-from-folders)
     (load (make-pathname :directory  (pathname-directory (mypathname self)) :name "preferences" :type "lisp"))
     (load-elements self)
     (setf (packages *current-workSpace*) (list *om-package-tree* *om-globalsfolder*))
     (setf (mypathname *om-package-tree*) (mypathname *current-workspace*))
     (ShowObjectEditor *current-workSpace*)
     (om-hide-window *presentation*)
     )))


(defmethod load-elements ((self OMWorkSpace))
   "Make instances for all elements in 'self', but their are not yet loaded."
   (let ((j -1)
         (elements (om-directory (make-pathname :directory (append (pathname-directory (mypathname self))
                                                                   (list "elements"))) :files t :directories t)))
     (setf (elements self) (remove nil (mapcar #'(lambda (x)
                                                   (ws-load-element x (incf j))) elements) :test 'equal))))

(defmethod get-editor-class ((self OMWorkSpace)) 'WorkSpaceEditor)


(defmethod ShowObjectEditor ((self OMWorkSpace))
  "WorkSpace's editor is open automaticly by this method."
  (declare (special *om-workSpace-win*))
   (let ((i -1) (j 0))
     (setf *workSpace-win-size* (if (fboundp 'om-get-workspace-initsize) 
                                                   (om-get-workspace-initsize)
                                                 (om-make-point 300 500)))
     (setf *workSpace-win-pos* (if (fboundp 'om-get-workspace-initpos) 
                                                   (om-get-workspace-initpos)
                                                 (om-make-point 0 50)))
    (setf *om-workSpace-win*
          (make-editor-window (get-editor-class self) *current-workSpace* (name self) nil 
                                                   :winsize *workSpace-win-size*             
                                                   :winpos *workSpace-win-pos*             
                                                   :close-p nil))
    (setf (editorFrame self) (panel *om-workSpace-win*))
        
    (mapc #'(lambda (elem)
              (add-icon-finder (make-icon-from-object  elem (om-point-h (get-icon-pos  elem)) (om-point-v (get-icon-pos elem)) 1 (incf i)) 
                               (panel *om-workSpace-win*))) (elements *current-workSpace*))
    (mapc #'(lambda (elem)
              (add-icon-finder (make-icon-from-object  elem (+ 5 (* 70 (incf j))) (- (h *om-workSpace-win*) 120) 1 (incf i)) 
                               (panel *om-workSpace-win*))) (packages *current-workSpace*))
    (setf (mypathname *om-globalsfolder*) (make-pathname 
                                                                               :directory (append (pathname-directory (mypathname self)) (list "globals"))))
    
    ;;; TEST
    (om-add-about-menu "About OpenMusic..." 
                                             #'(lambda () (om-select-window *presentation*)) 
                                             *om-workSpace-win*)
    
    (om-invalidate-view (editor *om-workSpace-win*))
    
    *om-workSpace-win*))


(defmethod save-all-persistants ((self omworkspace))
   "This method is called by the WorkSpace at the end of the OM session."
   (loop for item in (elements self) do
         (save-all-persistants item)))

;--------------------------------------------------
;Tools
;--------------------------------------------------

;----Builder
(defun make-new-WorkSpace (name)
   "Make an instance of the OMWorkspace class."
   (let ((new-ws (make-instance 'OMWorkSpace
                   :icon 203
                   :name (name-of-directory name)
                   :mypathname name)))
     new-ws))


;----Init session

;Choise the workspace path
(defmethod init-remote/local ((name t))
   (om-beep)
   (show-workspaces-dialog))

(defmethod init-remote/local ((name string))
   (init-om-sesion (OMRoot (string+ "WorkSpaces;" name ";"))))

(defmethod init-remote/local ((name list))
   (let ((string (car name)))
     (cond
      ((or (stringp string) (directory-pathname-p string))
       (init-om-sesion string))
      (t (om-beep)
         (show-workspaces-dialog)))))

(defmethod init-remote/local ((name pathname))
  (init-om-sesion name))


;Init the WorkSpace specified by pathname.
(defun init-OM-sesion (pathname)
   (declare (special *patch-menu-functions* *patch-menu-classes* *om-package-tree*))
   (init-om-package)                    
   (load-om-libs)
   (workspace-from-name pathname)                   
   (initWorkSpace *current-workSpace*)
   (packages2menu *patch-menu-functions* *patch-menu-classes* *om-package-tree*)
   (int-scorepal-buttons)
   
   (set-ompref 'prev-ws (mypathname *current-workSpace*))
   (save-omprefs)
   )


;This function load all files in the PATCHES folder.
(defun Load-modif-patches ()
  (om-with-redefinition-warnings nil
    (om-with-compiler-warnings nil
      (om-with-load-verbose nil
        (when (probe-file (OMRoot "patches;"))
          (mapc #'(lambda (file)
                    (load file :verbose nil)) 
                (sort (append (om-directory  (OMRoot "patches;") :type "lisp" :files t :directories nil)
                              (om-directory  (OMRoot "patches;") :type *om-compiled-type* :files t :directories nil)) 'string< :key 'pathname-name)))
        
        ))))


(defun get-preferences-version (name)
  (let (rep)
    (WITH-OPEN-FILE (in (make-pathname :directory (pathname-directory name) :name  "preferences" :type "lisp")
                         :direction :input 
                         :if-does-not-exist nil) 
       (let ((char (read-char in)))
         (when (equal char #\;)
           (setf rep (read-line in)))))
    rep))


(defvar *ws-params* nil)

(defun workspace-from-name (name)
  (when (probe-file (make-pathname :directory (pathname-directory name) :name  "modifs" :type "lisp"))
    (load (make-pathname :directory (pathname-directory name) :name  "modifs" :type "lisp")))
  (unless (probe-file (make-pathname :directory (pathname-directory name)))
    (create-directory (make-pathname :directory (pathname-directory name)) :if-exists nil))
  (unless (probe-file (make-pathname :directory (pathname-directory name) :name  "preferences" :type "lisp"))
    (WITH-OPEN-FILE (out (make-pathname :directory (pathname-directory name) :name  "preferences" :type "lisp")
                         :direction :output 
                         :if-does-not-exist :create :if-exists :supersede) 
      (write-line (format nil ";~D" *om-version*) out)
      (prin1 '(in-package :om) out)))
  (unless (probe-file (make-pathname :directory (append (pathname-directory name) (list "elements"))))
    (create-directory (make-pathname :directory (append (pathname-directory name) (list "elements"))) :if-exists nil))
  (unless (probe-file (make-pathname :directory (append (pathname-directory name) (list "globals"))))
    (create-directory (make-pathname :directory (append (pathname-directory name) (list "globals"))) :if-exists nil))
  (unless (probe-file (make-pathname :directory (append (pathname-directory name) (list "user"))))
    (create-directory (make-pathname :directory (append (pathname-directory name) (list "user"))) :if-exists nil))
  ;;; --- new for resources
  (unless (probe-file (make-pathname :directory (append (pathname-directory name) (list "resources"))))
    (create-directory (make-pathname :directory (append (pathname-directory name) (list "resources"))) :if-exists nil))
  (unless (probe-file (make-pathname :directory (append (pathname-directory name) (list "resources" "icon"))))
    (create-directory (make-pathname :directory (append (pathname-directory name) (list "resources" "icon"))) :if-exists nil))
  (unless (probe-file (make-pathname :directory (append (pathname-directory name) (list "resources" "pict"))))
    (create-directory (make-pathname :directory (append (pathname-directory name) (list "resources" "pict"))) :if-exists nil))
  ;;; ----
  (unless (probe-file (make-pathname :directory (pathname-directory name) :name  "userpackage" :type "lisp"))
    (WITH-OPEN-FILE (out (make-pathname :directory (pathname-directory name) :name  "userpackage" :type "lisp") :direction :output 
                         :if-does-not-exist :create :if-exists :supersede) 
      (prin1 '(in-package :om) out)
      (prin1 '(setf *package-user* (add-new-packages (list "user" nil nil nil nil)  *om-package-tree*)) out)
      (prin1 '(init-user-pathname) out)))
  (unless (get-preferences-version name)
    (WITH-OPEN-FILE (out (make-pathname :directory (pathname-directory name) :name  "preferences" :type "lisp")
                         :direction :output 
                         :if-does-not-exist :create :if-exists :supersede) 
      (write-line (format nil ";~D" *om-version*) out)
      (prin1 '(in-package :om) out)))
  (load (make-pathname :directory (pathname-directory name) :name  "preferences" :type "lisp"))
  
  ;;;test apres...
  ;;;(restore-preferences)
  
  (setf *current-workSpace* (make-new-WorkSpace name))
  ;new by AAA
  (when (probe-file (make-pathname :directory (pathname-directory name) :name  "wsparams" :type "lisp"))
    (load (make-pathname :directory (pathname-directory name) :name  "wsparams" :type "lisp")))
  (when *ws-params*
    (setf (wsparams *current-workSpace*) *ws-params*))
  ;;; new
  (user-init-funcs)
  
  (restore-preferences)
  
  )

;;;;;;;;;;;;;;;;;
(defvar *user-init-funcs* "les fonctions qui sont chargees apres initialisation du workspace")
(setf *user-init-funcs* nil)
(defun add-init-user-func (func-name)
  (unless (member func-name *user-init-funcs* :test 'equal)
      (push func-name *user-init-funcs*)))

(defun user-init-funcs ()
   (mapc #.(lambda (x) (funcall x)) (reverse *user-init-funcs*)))
;;;;;;;;;;;;;;;;;


;=============End of a session===================

(defmethod save-all-persistants ((self t))
   "This method is called by the WorkSpace at the end of the OM session."
   t)


(defun Save-ws-ws-params ()
  (when (and *current-workSpace* *om-workSpace-win*)
    (let ((path (make-pathname :directory (pathname-directory (mypathname *current-workSpace*)) :name  "wsparams" :type "lisp")))
      (delete-file-protection path)
      (WITH-OPEN-FILE (out path :direction :output 
                           :if-does-not-exist :create :if-exists :supersede) 
        (prin1 '(in-package :om) out)
        (prin1 `(setf  *ws-params* (list ,(omng-save (presentation *current-workSpace*))
                                         ,(omng-save (om-view-position *om-workSpace-win*))
                                         ,(omng-save (om-view-size *om-workSpace-win*)))) out))
      )))

(defun SaveWorkSpace ()
   (when *current-workSpace*
     (ws-save-globals)
     (ws-save-user-package)))

(defun Save-before-Quit ()
  (Save-ws-ws-params)
  (when (and *current-workSpace* (om-y-or-n-dialog "Do you want to save your workspace?"))
    (show-message-win "Saving workspace...")
    (SaveWorkSpace)
    (setf *save-apply-all* nil)
    (omNG-save-ws *current-workSpace*)
    (setf *save-apply-all* nil)
    (hide-message-win)))


(om-add-exit-cleanup-func 'Save-before-Quit)


;=============Generation of new WorkSpace in Disk

(defun Make-new-WS-folder ()
   (let ((wspath (OMRoot "WorkSpaces;")))
     (create-directory wspath)))


(defun Make-one-new-WS-folder (name)
   (let ((wspath (make-pathname :directory  (append (pathname-directory (OMRoot "WorkSpaces;")) (list name ))))
         (*package* (find-package :om)))
     (create-directory wspath :if-exists nil)
     (create-directory (make-pathname :directory  (append (pathname-directory wspath) (list "elements" ))) :if-exists nil)
     (create-directory (make-pathname :directory  (append (pathname-directory wspath) (list "globals" ))) :if-exists nil)
     (create-directory (make-pathname :directory  (append (pathname-directory wspath) (list "user" ))) :if-exists nil)
     ;;; --- new for resources
     (create-directory (make-pathname :directory  (append (pathname-directory wspath) (list "resources" ))) :if-exists nil)
     (create-directory (make-pathname :directory  (append (pathname-directory wspath) (list "resources" "icon"))) :if-exists nil)
     (create-directory (make-pathname :directory  (append (pathname-directory wspath) (list "resources" "pict"))) :if-exists nil)
     ;;; ---
     (let ((thepath (make-pathname :directory (pathname-directory wspath) :name  "preferences" :type "lisp")))
       (delete-file-protection thepath)
       (WITH-OPEN-FILE (out thepath :direction :output 
                            :if-does-not-exist :create :if-exists :supersede) 
         (write-line (format nil ";~D" *om-version*) out)
         (prin1 '(in-package :om) out))
       (setf thepath (make-pathname :directory (pathname-directory wspath) :name  "userpackage" :type "lisp"))
       (delete-file-protection thepath)
       (WITH-OPEN-FILE (out thepath :direction :output 
                            :if-does-not-exist :create :if-exists :supersede) 
         (prin1 '(in-package :om) out)
         (prin1 '(setf *package-user* (add-new-packages (list "user" nil nil nil nil)  *om-package-tree*)) out)
         (prin1 '(init-user-pathname) out)))))






;;;==================================================
;; OM PREFS

(defvar *omprefs* nil)

(defun ompref-file ()
  (let* ((userpref (om-user-pref-folder)))
    (make-pathname
     :device (pathname-device userpref)
     :directory (append (pathname-directory userpref) (list "OpenMusic" (format nil "~f" *om-version*)))
     :name "OMPrefs" :type "lisp")))

(defmethod save-omprefs ()
  (let ((path (ompref-file)))
    (delete-file-protection path)
    (WITH-OPEN-FILE (out path :direction :output 
                         :if-does-not-exist :create :if-exists :supersede) 
      (write-line (format nil ";~D" *om-version*) out)
      (prin1 '(in-package :om) out)
      (prin1 `(setf *omprefs* ,(omng-save *omprefs*)) out))
    t))

(defun get-ompref (key)
  (cadr (find key *omprefs* :test 'equal :key 'car)))

(defun set-ompref (key val)
  (let ((pos (position key *omprefs* :test 'equal :key 'car)))
    (if pos 
      (setf (nth pos *omprefs*) (list key val))
      (setf *omprefs* (append *omprefs* (list (list key val)))))))
