;OpenMusic
;
;Copyright (C) 1997, 1998, 1999, 2000 by IRCAM-Centre Georges Pompidou, Paris, France.
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
;
;Authors: Gerard Assayag and Augusto Agon


;DocFile
;Maquette Editor definition.
;Last Modifications :
;18/10/97 first date.
;DocFile
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; VERIF OK a part ";;;"

(in-package :om)

;====================================================
;PALETTE FOR MAQUETTES
;====================================================
(omg-defclass maq-palette (Playing-Palette) 
   ((pushed-b :initform 0 :accessor pushed-b))
   (:default-initargs :source-name "maq-pal"))


(defmethod palette-act ((self maq-palette) x)
   (unless (call-next-method)
     (let ((scroller (panel (editor-assoc self))))
       (case x
         (11 (init-coor-system scroller) 
          ;(om-invalidate-view scroller t)
          (om-invalidate-view (rulerx scroller) t)
          (om-invalidate-view (rulery scroller) t))
         (12 (setf (grille-p scroller) (not (grille-p scroller)))
          (om-invalidate-view scroller t))  
         (13 (let ((new-color (om-choose-color-dialog :color (om-get-bg-color scroller))))
               (when new-color
                 (change-colorframe scroller new-color))))
         (14 (eval-maquette scroller))
         (otherwise
          (when x
            (setf (mode scroller) X)
            (setf (pushed-b self) X)
            (if (= x 9)
              (setf (cursor-p (panel (editor-assoc self))) t)
              (when (or (= x 7) (= x 8) (= x 10))
                (setf (cursor-p (panel (editor-assoc self))) nil))
              )
            (om-invalidate-view self t)
            ))))))


 
;================================================================
;THE MAQUETTE WINDOW
;================================================================

;-------------------------------------------------
; X RULER
;-------------------------------------------------
(omg-defclass maq-ruler (ruler) 
  ((x-step :accessor x-step :initarg :x-step :initform 1)))


(defmethod change-x-params  ((self maq-ruler) list)
   (let* ((container (panel (om-view-container self))))
     (setf (x-step self) (first list))
     (setf (xparam (params (object container))) list)
     ))

(defmethod om-view-click-handler ((self maq-ruler) where) 
   (if (om-option-key-p)
     (add-new-marquer self where)
     (let* ((container (panel (om-view-container self)))
            (frames (get-subframes container)))
       (call-next-method)
       (make-move-after container frames))))

(defmethod add-new-marquer  ((self maq-ruler) where)
  (let* ((x (om-point-h (pixel2point (assoc-view self) where))))
    (omg-add-element (assoc-view self) 
                     (make-frame-from-callobj (omNG-make-new-marker (om-make-point (om-point-h where) (- (h (assoc-view self)) 26))
                                                                                                                      (mk-unique-name (assoc-view self) "marker")   x)))))




;-------------------------------------------------
;;; VERTICAL RULER
;-------------------------------------------------
(omg-defclass maq-y-ruler (ruler) 
   ((y-step :accessor y-step :initarg :y-step :initform 1)
    (force :accessor force :initarg :force :initform t)))

(defmethod om-view-doubleclick-handler ((self maq-y-ruler) where)
     (let ((changes (edit-y-params self)))
       (when (consp changes)
           (change-y-params self changes)
           (om-invalidate-view self t))))

;;; y params = step , force-t-ou-nil
(defmethod change-y-params  ((self maq-y-ruler) list)
   (let* ((container (panel (om-view-container self))))
     (setf (y-step self) (first list))
     (setf (force self) (second list))
     (setf (yparam (params (object container))) list)))

(defmethod edit-y-params  ((self maq-y-ruler))
  (let ((dialog (om-make-window 'om-dialog
                                :window-title "Y Settings"
                                :position :centered
                                :size (om-make-point 290 190)
                                :maximize nil :resizable nil
                                :font *om-default-font4*
                                :bg-color (om-make-color 0.623 0.623 0.623)))
        (steplabel (om-make-dialog-item 'om-static-text (om-make-point 20 20) (om-make-point 80 20) "Y Step"
                                        :bg-color (om-make-color 0.623 0.623 0.623)))
        (stepdi (om-make-dialog-item 'om-editable-text (om-make-point 100 20) (om-get-text-edit-size (om-make-point 40 16)) 
                                     (num2string (y-step self))))
        
        (flabel (om-make-dialog-item 'om-static-text (om-make-point 20 60) (om-make-point 80 20) "Magnetic"
                                     :bg-color (om-make-color 0.623 0.623 0.623)))
        (fdi (om-make-dialog-item 'om-check-box (om-make-point 100 60) (om-make-point 20 20) ""
                                  :checked-p (force self)
                                  :bg-color (om-make-color 0.623 0.623 0.623)))
        )
    
    (om-add-subviews dialog steplabel stepdi flabel fdi 
                     (om-make-dialog-item 'om-button (om-make-point 171 162) (om-make-point 72 24) "cancel" 
                                          :di-action (om-dialog-item-act item
                                                       (declare (ignore item))
                                                       (om-return-from-modal-dialog dialog nil)))
                     (om-make-dialog-item 'om-button (om-make-point 71 162) (om-make-point 72 24) "OK" 
                                          :di-action (om-dialog-item-act item
                                                       (declare (ignore item))
                                                       (let ((step (read-from-string (om-dialog-item-text stepdi))))
                                                         (if (integerp step)
                                                           (om-return-from-modal-dialog dialog 
                                                                                        (list step (om-checked-p fdi)))
                                                           (progn (om-message-dialog "Bad Step Value !!")
                                                                  (om-return-from-modal-dialog dialog nil)))))
                                          :default-button t))
    (om-modal-dialog dialog)))

;-------------------------------------------------
;This is the optional metric ruler
;-------------------------------------------------
(omg-defclass metric-ruler (om-view-drop ruler-metric) ())

(defmethod om-view-drag-hilite-p ((self metric-ruler)) nil)



(defmethod om-view-click-handler ((self metric-ruler) where)
  (cond
   ((om-option-key-p) (add-new-marquer self where))
   (t  (let* ((container (panel (om-view-container self)))
              (frames (get-subframes container)))
         (call-next-method)
         (om-invalidate-view (rulerx container) t)
         (make-move-after container frames)   ;;; ?? pourquoi ??
         ))))

(defmethod om-view-doubleclick-handler  ((self metric-ruler) where)
  (if (om-option-key-p) (add-new-marquer self where)
      (let ((changes (edit-metric-ruler self)))
        (when (consp changes)
          (change-params self changes)
          (om-invalidate-view self t)))))
   
   
(defmethod add-new-marquer  ((self metric-ruler) where)
  (let* ((x (om-point-h (pixel2point (assoc-view self) where))))
    (omg-add-element (assoc-view self) 
                     (make-frame-from-callobj (omNG-make-new-marker (om-make-point (om-point-h where) (- (h (assoc-view self)) 26))
                                                                    (mk-unique-name (assoc-view self) "marker")   x)))))



;;===========================================

(omg-defclass change-tempo-item (om-view)
  ((temponum :initform 4 :initarg :temponum :accessor temponum)))

(defmethod om-draw-contents ((self change-tempo-item))
  (om-with-focused-view self
                        (om-with-font (om-get-font self)
                                      (om-draw-string 5 (- (h self) 2) (get-stringfrom-num (temponum self))))))

(defmethod om-view-click-handler ((self change-tempo-item) pos)
  (declare (ignore pos))
  (update-tempo-item self))


(defmethod update-tempo-item ((self change-tempo-item))
   (let ((numtempo (temponum self))
          (strin nil))
     (loop while (not strin) do
           (setf numtempo (if (zerop (mod (+ numtempo 1) 17)) 1 (mod (+ numtempo 1) 17)))
           (setf strin (get-stringfrom-num numtempo))
           )
     (setf (temponum self)  numtempo)
     (om-invalidate-view self t)))

(defmethod edit-metric-ruler  ((self metric-ruler))
  (let ((dialog (om-make-window 'om-dialog
                                :window-title "Metric settings"
                                :position :centered
                                :size (om-make-point 290 190)
                                :maximize nil :resizable nil
                                :font *om-default-font4*
                                :bg-color (om-make-color 0.623 0.623 0.623)))
        (metric (om-make-dialog-item 'om-editable-text (om-make-point 3 68) (om-get-text-edit-size (om-make-point 260 16)) 
                                     (num2string (meslist self))))
        (tempo1 (om-make-view 'change-tempo-item :position (om-make-point 140 10) :size (om-make-point 22 30)
                              :font (om-make-font *heads-font* 24)
                              :temponum (first (tempo self))
                              ))
        (tempo2 (om-make-dialog-item 'om-editable-text (om-make-point 180 15) (om-get-text-edit-size (om-make-point 30 16)) 
                                     (num2string (second (tempo self))) ))
        (maxsub (om-make-dialog-item 'om-editable-text (om-make-point 140 100) (om-get-text-edit-size (om-make-point 20 16)) 
                                     (num2string (maxsub self))  ))
        (loop-p (om-make-dialog-item 'om-radio-button (om-make-point 3 135) (om-make-point 115 16) "loop measures"
                                     :checked-p (loop-mes-p self)
                                     :bg-color (om-make-color 0.623 0.623 0.623)
                                     :radio-button-cluster 'loop))
        (last-p (om-make-dialog-item 'om-radio-button  (om-make-point 143 135) (om-make-point 115 16) "last measure"
                                     :checked-p (not (loop-mes-p self))
                                     :bg-color (om-make-color 0.623 0.623 0.623)
                                     :radio-button-cluster 'loop)))
    (om-add-subviews dialog metric maxsub loop-p last-p tempo1 tempo2
                     (om-make-dialog-item 'om-static-text (om-make-point 3 9) (om-make-point 100 16) "Tempo"
                                          :bg-color (om-make-color 0.623 0.623 0.623))
                     (om-make-dialog-item 'om-static-text (om-make-point 3 41) (om-make-point 100 16) "Metric" 
                                          :bg-color (om-make-color 0.623 0.623 0.623))
                     (om-make-dialog-item 'om-static-text (om-make-point 3 100) (om-make-point 100 16) "Max Subdiv (u)"
                                          :bg-color (om-make-color 0.623 0.623 0.623))
                     
                     (om-make-dialog-item 'om-button (om-make-point 171 162) (om-make-point 72 24) "cancel" 
                                          :di-action (om-dialog-item-act item
                                                       (declare (ignore item))
                                                       (om-return-from-modal-dialog dialog nil)))
                     (om-make-dialog-item 'om-button (om-make-point 71 162) (om-make-point 72 24) "OK" 
                                          :di-action (om-dialog-item-act item
                                                       (declare (ignore item))
                                                       (om-return-from-modal-dialog dialog (list (list (temponum tempo1) 
                                                                                                       (read-from-string (om-dialog-item-text tempo2)))
                                                                                                 (read-from-string (om-dialog-item-text metric))
                                                                                                 (read-from-string (om-dialog-item-text maxsub))
                                                                                                 (om-checked-p loop-p))))
                                          :default-button t))
    (om-modal-dialog dialog)))
  
(defmethod change-params  ((self metric-ruler) list)
   (let* ((container (panel (om-view-container self))))
     (setf (metricparam (params (object container))) list)
     (setf (tempo self) (first list))
     (setf (maxsub self) (third list))
     (setf (loop-mes-p self) (fourth list))
     (setf (meslist self) (second list))))

;D&D

(defmethod om-drag-selection-p ((self metric-ruler) mouse-position)
   (declare (ignore mouse-position)) nil)

(defmethod om-drag-receive ((view metric-ruler) 
                            (dragged-view t))
   nil)


         

;-------------------------------------------------
;EDITOR
;-------------------------------------------------

(omg-defclass MaquetteEditor (patchEditor) 
   ((rulermetric :initform nil :accessor rulermetric)
    (patchview :accessor patchview :initform nil :initarg :patchview)))

;;;=========================================
(omg-defclass drop-patch-view (om-transparent-view om-view-drop) 
  ((selected :accessor selected :initform nil :initarg :selected)))

(defmethod om-view-drag-hilite-p ((self drop-patch-view)) t)

(defmethod om-drag-enter-view ((self drop-patch-view))
  (om-with-focused-view self
    (om-with-fg-color self *azulito*
      (om-fill-rect 3 3 (- (w self) 6) (- (h self) 6))) 
    ))

(defmethod om-drag-leave-view ((self drop-patch-view))
  (om-invalidate-view self t))

(defmethod om-drag-receive ((view drop-patch-view) 
                            (dragged-view t)) 
  (declare (ignore data-size item-reference))
  (let ((fun (if (icon-finder-p (dragged-view *OM-drag&drop-handler*))
               (object (dragged-view *OM-drag&drop-handler*))
               (reference (object (dragged-view *OM-drag&drop-handler*))))))
    (cond 
     ((and (patch-p fun) (not (maquette-p fun)))
      (set-eval-func (object (om-view-container view)) fun)
      t)
     ((and (symbolp fun) (fboundp fun))
      (set-eval-func (object (om-view-container view)) (if (omgenfun-p (fdefinition fun)) 
                                                         (fdefinition fun)
                                                         (omNG-make-new-lispfun fun)))
      t)
     (t nil))
    ))

(defmethod om-draw-contents ((self drop-patch-view))
  (let ((evalf (eval-func (object (om-view-container self)))))
  (when evalf
    (om-with-focused-view self
      (om-draw-icon (cadr (get&corrige-icon (icon evalf))) self (om-make-point 2 2) (om-make-point 20 20) (selected self))
      ))))

(defmethod om-view-click-handler ((self drop-patch-view) pos)
  (mapc #'(lambda (control) 
            (omG-unselect control)) (get-actives (panel (om-view-container self))))
  (setf (selected self) t)
  (om-invalidate-view self t))

(defmethod om-view-doubleclick-handler ((self drop-patch-view) pos)
  (when (eval-func (object (om-view-container self)))
    (openobjecteditor (reference (eval-func (object (om-view-container self)))))
  ))

(defmethod om-view-click-handler ((self maquetteeditor) pos)
  (setf (selected (patchview self)) nil)
  (om-invalidate-view (patchview self) t)
  (call-next-method))
;;;=========================================


(defmethod get-editor-panel-class ((self MaquetteEditor))  'maquettePanel)

(defmethod metaobj-scrollbars-params ((self maquetteEditor))  '(nil nil))

(defmethod om-set-view-size ((self MaquetteEditor) size)
   (declare (ignore h v))
   (call-next-method)
   (update-subviews self))

(defmethod set-clipboard ((self MaquetteEditor) val)
   (let* ((container (panel self)))
     (setf (scroll-scrap-ma container) val)))

(defmethod get-clipboard ((self MaquetteEditor))
   (let* ((container (panel self)))
     (scroll-scrap-ma container)))  

(defmethod maq-x-ruler-class ((self MaquetteEditor)) 'maq-ruler)
(defmethod maq-y-ruler-class ((self MaquetteEditor)) 'maq-y-ruler)
(defmethod maq-metric-ruler-class ((self MaquetteEditor)) 'metric-ruler)

(defmethod initialize-instance :after ((self MaquetteEditor) &rest l)
  (declare (ignore l))
  (let* ((panel (panel self))
         (rulery (om-make-view (maq-y-ruler-class self)
                               :owner self
                               :axe 'y
                               :assoc-view panel
                               :position (om-make-point 0 0) 
                               :size (om-make-point 25 (- (h self) 25))
                               :y-step (first (yparam (params (object panel))))))
         (rulerx (om-make-view (maq-x-ruler-class self) 
                               :owner self
                               :axe 'x
                               :zoom 1000
                               :minzoom 0.001
                               :assoc-view panel
                               :position (om-make-point 25 (- (h self) 25)) 
                               :size (om-make-point (- (w self) 25) 25)
                               :x-step (first (xparam (params (object panel))))
                               )))
    
    (setf (rulerx panel) rulerx)
    (setf (rulery panel) rulery)
    (om-set-view-position panel (om-make-point 25 0))
    (om-set-view-size panel (om-make-point (- (w self) 25) (- (h self) 25)))
    (when (string-equal (maq-show-ruler (object self)) "on")
      (add-ruler-metric self))
    (om-add-subviews self (setf (patchview self) (om-make-view 'drop-patch-view
                                                               :position (om-make-point 0 0)
                                                               :size (om-make-point 24 24)
                                                               )))
    (add-more-information panel)
    (om-invalidate-view (panel self))))



(defmethod add-ruler-metric ((self MaquetteEditor))
  (om-beep-msg "The music project is not loaded"))


(defmethod delete-ruler-metric ((self MaquetteEditor))
   (om-remove-subviews self (rulermetric self))
   (setf (rulermetric self) nil)
   (maq-show-ruler (object self) "off")
   (update-all-offsets self)
   (update-subviews self)
   )

(defmethod update-all-offsets ((self MaquetteEditor))
   (let* ((container (panel self))
          (themaquette (object container))
          (metric-pa (metricparam (params themaquette))))
     (loop for frame in (get-subframes container) do
           (let ((tempobj (object frame)))
             (when (rulermetric self)
               (setf (slot-value tempobj 'offset)
                     (u2ms (ms2u (offset tempobj) (first metric-pa) (third metric-pa))
                           (first metric-pa) (third metric-pa))))))))


(defmethod give-editor-list-range ((self MaquetteEditor))
   (let ((subframes (get-subframes (panel self))))
     (setf subframes
           (loop for item in subframes
                 when (tempframe-p item) collect item))
     (cond
      ((null subframes) (list 0 10000 0 127))
      (t (loop for item in subframes
               maximize (+ (* (strech-fact (object item)) (extend (object item))) (offset (object item))) into x
               maximize (posy (object item)) into y
               minimize (- (posy (object item)) (sizey (object item))) into y0
               finally (return (list 0 (round (* x 1.1)) (- y0 (round (* 0.1 (- y y0)))) (+ y (round (* 0.1 (- y y0)))))))))))

(defmethod editor-has-palette-p ((self MaquetteEditor)) 'maq-palette)

(defmethod mode ((self MaquetteEditor))
   (mode (panel self)))

(defmethod update-subviews ((self MaquetteEditor))
  (let ((scroller (panel self)))
    (om-set-view-size (rulerx scroller) (om-make-point (- (w self) 25) 25))
    (om-set-view-position (rulerx scroller) (om-make-point 25 (- (h self) 25)))
    (if (not (rulermetric self))
       (progn
         (om-set-view-position (rulery scroller) (om-make-point 0 0))
         (om-set-view-size (rulery scroller) (om-make-point 25 (- (h self) 25)))
         (om-set-view-position scroller (om-make-point 25 0))
         (om-set-view-size scroller (om-make-point (- (w self) 25) (- (h self) 25))))
       (progn
         (om-set-view-position (rulery scroller) (om-make-point 0 25))
         (om-set-view-size (rulermetric self) (om-make-point (- (w self) 25) 25))
         (om-set-view-size (rulery scroller) (om-make-point 25 (- (h self) 50)))
         (om-set-view-position scroller (om-make-point 25 25))
         (om-set-view-size scroller (om-make-point (- (w self) 25) (- (h self) 50)))))
    (when (patchview self) (om-set-view-position (patchview self) (om-make-point 0 (- (h self) 24))))
    (update-size+pos-frames scroller)
    (make-move-after scroller (get-subframes scroller))
    (om-invalidate-view self t)))

;--------------------------
;PANEL
;--------------------------

(omg-defclass MaquettePanel (patchPanel view-with-ruler-xy cursor-play-view-mixin) 
   ((mode :initform 7 :accessor mode)
    (selected-mark-lines :initform nil :accessor selected-mark-lines)
    (scroll-scrap-ma :initform nil :allocation :class :accessor scroll-scrap-ma))
   (:documentation "This is the class for editors of Maquette meta objects.
 Elements of these editors are tempobjframe instances.#enddoc#
#seealso# (OMMaquette tempobjframe temporalbox) #seealso#")
   )

(defmethod delete-general :after ((self maquettepanel))
  (when (selected (patchview (editor self)))
    (set-eval-func (object (editor self)) nil)
    (om-invalidate-view (patchview (editor self)) t)))

(defmethod add-window-buttons ((self MaquettePanel))
  "There are not buttons in a maquette it has a palette." nil)


(defmethod maq-panel-set-color ((self  MaquettePanel) color)
   (om-set-bg-color self color)
   (om-set-bg-color (rulerx self) color)
   (om-set-bg-color (rulery self) color)
   (om-set-bg-color (om-view-container self) color)
   (when (rulermetric (editor self))
     (om-set-bg-color (rulermetric (editor self)) color)))

(defmethod add-more-information ((self  MaquettePanel))
   (let ((ranges (range (params (object self)))))
     (setf (rangex self) (list (first ranges) (second ranges)))
     (setf (rangey self) (list (third ranges) (fourth ranges)))
     
     (set-ranges self  (rangex self) (rangey self))
     (let ((color (maq-color (params (object self)))))
       (when color
           (maq-panel-set-color self color)
         ))))

(defmethod save-ranges ((self  MaquettePanel))
   (setf (range (params (object self))) (list (car (rangex self)) (second (rangex self))
                                              (car (rangey self)) (second (rangey self)))))

(defmethod set-ranges ((self  MaquettePanel) rangex rangey)
   (declare (ignore rangex rangey))
   (call-next-method)
   (save-ranges self))

(defmethod show-con? ((self MaquettePanel))
   (show-conect (params (object self))))

(defmethod delta-scroll ((self MaquettePanel))
   "Return a point with the scroll positions"
   (om-make-point 0 0))

;-------------------------------
;MCL methods
;-------------------------------

(defmethod om-add-subviews :before ((self MaquettePanel) &rest subviews)
   (mapc #'(lambda (elem) (init-size&position elem self)) subviews))

(defmethod om-view-cursor ((self MaquettePanel))
   (cond
    ((om-control-key-p) 
     *om-contex-cursor*)
    (t (case (mode self)
           (8 *om-loupe-cursor*)
           (10 *om-hand-cursor*)
           (otherwise  (if (cursor-p self) 
                                     *om-i-beam-cursor*
                                   *om-arrow-cursor*
                                   ))))))

(defmethod control-actives ((view MaquettePanel) where)
   (when (text-view (editor view))
     (exit-from-dialog (text-view (editor view)) (om-dialog-item-text (text-view (editor view)))))
   (cond
    ((= (mode view) 8) (zoom-system view where)
     (om-invalidate-view (rulerx view) t)
     (om-invalidate-view (rulery view) t))
    ((= (mode view) 10) (scroll-system view where))
    ((= (mode view) 9) (new-interval-cursor view where))
    ((= (mode view) 7)
     (cond
      ((and (show-con? view) (click-in-connection view where))
       (mapc #'(lambda (control) 
                 (omG-unselect control)) (get-actives  view))
       (when (selected-mark-lines view)
         (draw-mark-lines view nil)
         (setf (selected-mark-lines view) nil)
         (draw-mark-lines view)))
      ((and (show-con? view) (click-in-marker-line view where))
       (mapc #'(lambda (control) 
                 (omG-unselect control)) (get-actives  view))
       (mapc #'(lambda (control) 
                 (deactivate-connect control)) (get-connections view)))
      (t (let* ((float (om-subtract-points  (om-mouse-position view) where)))
           (mapc #'(lambda (control) 
                     (deactivate-connect control)) (get-connections view))
           (when (selected-mark-lines view)
             (draw-mark-lines view nil)
             (setf (selected-mark-lines view) nil)
             (draw-mark-lines view))
           (cond 
            ((om-command-key-p)
             (let ((user-rect (om-pts-to-rect where (grow-gray-rect  where float view nil))))
               (if (om-shift-key-p)
                   (make-maq-tempobj view where (om-rect-bottomright user-rect))
                 (make-tempobj view where (om-rect-bottomright user-rect)))))
            (t (let ((user-rect (om-pts-to-rect where (grow-gray-rect  where float view nil)))
                       scratch-rect-i i-rect)
                 (dolist (item (get-subframes view))
                    (setf i-rect (om-pts-to-rect (om-view-position item) (om-add-points (om-view-position item) (om-view-size item))))
                    (setf scratch-rect-i (om-sect-rect user-rect i-rect))
                    (unless (om-rect-empty scratch-rect-i)
                      (omG-select item))))))))))))

;-------------------------------
;EVENTS
;-------------------------------

(defmethod change-colorframe ((self MaquettePanel) newcolor)
   "If there are selected boxes set their frame color to 'newcolor', else set the back color of 'self' to newcolor."
   (let ((frames (get-actives  self)))
     (if frames
       (loop for item in frames do
             (change-colorframe item newcolor))
       (progn
         (maq-panel-set-color self newcolor)
         (setf (maq-color (params (object self))) newcolor)))))

(defmethod move-and-not-action ((self MaquettePanel) (cible MaquettePanel)) t)
(defmethod move-and-not-action ((self MaquettePanel) (cible t)) nil)
(defmethod move-and-not-action ((self t) (cible MaquettePanel)) nil)

(defmethod get-colorframe ((view MaquettePanel))
   (maq-color (params (object view))))

(defmethod make-tempobj ((self MaquettePanel) x y)
   "Add a new empty temporalbox to 'self'. This method is called when you make ALT+CLICK+DRAG in 'self'."
   (let* ((thename (mk-unique-name self "tempobj"))
          (new-patch (make-instance 'OMPatchAbs 
                       :name thename :icon 210))
          (pixsizex (max 20 (- (om-point-h y) (om-point-h x))))
          (pixsizey (max 10 (- (om-point-v y) (om-point-v x))))
          (maqpos (get-offset/posy-from-pixel self  (om-make-point (om-point-h x) (om-point-v x))))
          (y-size (pixel2norme self 'y  pixsizey))
          (tempobj (omNG-make-tempobj new-patch maqpos thename))
          new-frame)
         ;; *
     (add-temp-boxes new-patch)
     (setf (slot-value tempobj 'extend) (pixel2norme  self 'x pixsizex))
     (setf (slot-value tempobj 'sizey)  y-size)
     (setf new-frame (make-frame-from-callobj tempobj))
     (omG-add-element self new-frame)))

;; ***
(defun get-absmaqclass () 'OMMaqAbs)

;; ***
(defmethod get-maq-obj-name ((class t)) "maquette")
(defmethod get-maq-obj-name ((class (eql 'ommaqabs))) "internal-maq")

;; ***
(defmethod make-maq-tempobj ((self MaquettePanel) x y)
   "Add a new empty temporal maq to 'self'. This method is called when you make ALT+CLICK+DRAG in 'self'."
   (let* ((class (get-absmaqclass))
          (thename (mk-unique-name self (get-maq-obj-name class)))
          (new-patch (make-instance class 
                       :name thename :icon 265))
          (pixsizex (max 20 (- (om-point-h y) (om-point-h x))))
          (pixsizey (max 10 (- (om-point-v y) (om-point-v x))))
          (maqpos (get-offset/posy-from-pixel self  (om-make-point (om-point-h x) (om-point-v x))))
          (y-size (pixel2norme self 'y  pixsizey))
          (tempobj (omNG-make-tempobj new-patch maqpos thename))
          new-frame)
     (setf (slot-value tempobj 'extend) (pixel2norme  self 'x pixsizex))
     (setf (slot-value tempobj 'sizey)  y-size)
     (setf new-frame (make-frame-from-callobj tempobj))
     (omG-add-element self new-frame)))
       


(defmethod maquette-extra-eval ((self ommaquette)) 
  (setf (value self) 
        (let ((boxestoplay (loop for b in (boxes self) when (and (boxtempobj-p b) (not (mute b))) collect b)))
          (list (cons-copy-maquette-object self boxestoplay)))))

(defmethod eval-maquette ((self MaquettePanel))
   "Eval all boxes in 'self'."
   (loop for item in (get-objects-to-value (get-subframes self)) do
         (eval+redraw item))
   (maquette-extra-eval (object (editor self)))
   (clear-ev-once self))


(defun get-objects-to-value (list)
   (let (rep)
     (mapc #'(lambda (frame)
               (when (and (not (markerframe-p frame)) (non-connected frame list))
                 (push frame rep))) list) 
     rep))


(defmethod omg-remove-element ((self MaquettePanel) frame)
   "Frame can be a tempobjframe or a marker."
   (let ((x0 (x frame))
         (xf (x+w frame)))
     (call-next-method)
     (cond 
      ((markerframe-p frame)
       (om-invalidate-corners self (om-make-point (- x0 2) 0) (om-make-point (+ x0 2) (h self)) t))
      (t
       (let* ((mark-list (get-maquette-markers self))
              (finmarker (is-marketed (object frame) 1 mark-list))
              (startmarker (is-marketed (object frame) 0 mark-list)))
         (when startmarker
           (del-mark (object frame) startmarker 0))
         (when finmarker
           (del-mark (object frame) finmarker 1))
         (om-invalidate-corners self (om-make-point (- x0 2) 0) (om-make-point (+ xf 2) (h self)) t))))))
 
;-------------------------------------------
;DRAW
;-------------------------------------------
(defvar *scrolling-ruler* nil)

(defmethod priority-connecctions ((self MaquettePanel)) t)
 
(defmethod om-draw-contents :before ((self MaquettePanel))
   (when (thepict (pictu (object self)))
     (let* ((pictsize (om-get-picture-size (thepict (pictu (object self)))))
            (x (om-point-h pictsize))
            (y (om-point-v pictsize))
            (sizex (ceiling (w self) x))
            (sizey (ceiling (h self) y)))
       (loop for i from 0 to sizex do
             (loop for j from 0 to sizey do
                   (om-draw-picture self (thepict (pictu (object self))) (om-make-point (* i x) (* j y)) 
                                    (om-make-point x y)))))))

(defmethod om-draw-contents ((self MaquettePanel))
   (unless *scrolling-ruler*
     (when (show-con? self) (draw-mark-lines self))
     (when (grille-p self)
       (draw-axis self)
       (if (rulermetric (editor self))
         (draw-grille-metric self (rangex self))
         (draw-grille self)))
     (when (cursor-p self)
       (draw-interval-cursor self))
     ;;; a boucle quand on ajoute le ruler metric...
     ;;;(when (rulermetric (editor self))
     ;;;  (om-invalidate-view (rulermetric (editor self)) t))
     (call-next-method)))


(defmethod change-view-ranges ((self t)) t)

(defmethod change-view-ranges ((self MaquettePanel))
   (update-size+pos-frames self))


(defmethod update-view-of-ruler  ((self MaquettePanel) )
   (setf *scrolling-ruler* t)
   (update-size+pos-frames self)
   (setf *scrolling-ruler* nil)
   ;(erase-view self)
   (save-ranges self)
   (om-invalidate-view self t)
   (when (rulermetric (editor self)) (om-invalidate-view (rulermetric (editor self)) t))
)
  
(defmethod update-size+pos-frames ((self MaquettePanel)) 
   (let* ((frames (get-subframes self)))
     (loop for frame in frames do
           (init-size&position frame self))))

(defmethod get-string-nom  ((self MaquettePanel)  num axe)
   (if (equal axe 'y)
     (call-next-method)
     (cond
      ((integerp (/ num 1000)) (format () "~D" (round num 1000)))
      ((integerp (/ num 100)) (format () "~,1F" (/ num 1000)))
      (t (format () "~,2F" (/ num 1000))))))

(defmethod scroll-system ((self MaquettePanel) where)
   (declare (ignore where))
   (let* ((old-mouse (om-mouse-position self))
          (initmouse old-mouse)
          (initrangex (rangex self))
          (initrangey (rangey self))
          (old-grille (grille-p self))
          (frames (get-subframes self))
          (new-mouse old-mouse) deltax deltay)
     ;;;(rlet ((event :eventRecord))
     (setf (grille-p self) nil)
     (loop while (om-mouse-down-p) do
           (setq new-mouse (om-mouse-position self))
           (unless (om-points-equal-p old-mouse new-mouse)
             (draw-items-frames self frames)
             (setf deltax (pixel2norme self 'x (- (om-point-h initmouse) (om-point-h new-mouse))))
             (setf deltay (pixel2norme self 'y (- (om-point-v new-mouse) (om-point-v initmouse))))
             (if (minusp (+ (first initrangex) deltax))
               (setf (rangex self) (list 0 (- (second initrangex) (first initrangex))))
               (setf (rangex self) (list (+ (first initrangex) deltax)
                                         (+ (second initrangex) deltax))))
             (setf (rangey self) (list (+ (first initrangey) deltay)
                                       (+ (second initrangey) deltay)))
             (when (rulermetric (editor self))
               (om-redraw-view (rulermetric (editor self))))
             (draw-items-frames self frames)
             (om-redraw-view (rulerx self))
             (om-redraw-view (rulery self))
             (setq old-mouse new-mouse))
               ;;;(get-next-event event nil 0 1)
               (om-get-next-event)
               )
     (setf (grille-p self) old-grille)
     (setf deltax (pixel2norme self 'x (- (om-point-h initmouse) (om-point-h new-mouse))))
     (setf deltay (pixel2norme self 'y (- (om-point-v new-mouse) (om-point-v initmouse))))
     (if (minusp (+ (first initrangex) deltax))
       (setf (rangex self) (list 0 (- (second initrangex) (first initrangex))))
       (setf (rangex self) (list (+ (first initrangex) deltax)
                                 (+ (second initrangex) deltax))))
     (setf (rangey self) (list (+ (first initrangey) deltay)
                               (+ (second initrangey) deltay)))
     (save-ranges self)
     (change-view-ranges self)
     (om-invalidate-view self t)
     ;;;)
     ))

(defun draw-items-frames (cont items)
  (om-with-focused-view cont
    (om-with-pen (cont :mode :xor :line :gray-line)
      (loop for item in items do
            (let ((maqpos (get-offset/posy-in-pixel item cont)))
              (om-draw-rect (om-point-h maqpos) (om-point-v maqpos)  (w item) (h item)))))))


  
;----------------------------------------------------
;Play from palette
;----------------------------------------------------

(defmethod handle-key-event ((self MaquettePanel) char) 
   (case char
     (#\SPACE (if (Idle-p *general-player*)
                (palette-act (palette *global-palette*) 0)
                (palette-act (palette *global-palette*) 1)))
     (#\x (mute-boxes self))
     (#\z (lock-boxes self))
     (#\a (mapc 'internalize-patch (get-actives self)))
     (#\w (align-boxes self))
     (#\q (memesize-boxes self))
      (otherwise (call-next-method))))


;======
(defmethod mute-boxes ((self MaquettePanel))
  (let ((boxes (get-actives self)))
    (loop for box in boxes do 
          (setf (mute (object box)) (not (mute (object box))))
          (om-invalidate-view box))
    ))


(defmethod lock-boxes ((self MaquettePanel))
  (let ((boxes (get-actives self)))
    (loop for box in boxes do 
          (setf (lock (object box)) (not (lock (object box))))
          (om-invalidate-view box))
    )) 

;======
(defmethod align-boxes ((self MaquettePanel))
   (let ((boxes (get-actives self))
          (min 1000000)
          refobj tempobjs)
     (setf tempobjs (loop for box in boxes do 
                                    (when (< (offset (object box)) min) 
                                        (setf refobj (object box))
                                        (setf min (offset (object box))))
                                 collect (object box)))
     (loop for obj in tempobjs do (setf (posy obj) (posy refobj)))
     ))

(defmethod memesize-boxes ((self MaquettePanel))
   (let ((boxes (get-actives self))
          (min 1000000)
          refobj tempobjs)
     (setf tempobjs (loop for box in boxes do 
                                    (when (< (offset (object box)) min) 
                                        (setf refobj (object box))
                                        (setf min (offset (object box))))
                                 collect (object box)))
     (loop for obj in tempobjs do (setf (sizey obj) (sizey refobj)))
     ))

;;; play maquette : construit le maquette-obj (en copiant les objs contenus)
(defmethod cons-play-maquette-object ((self OMMaquette) objs)
  (cons-copy-maquette-object self objs))

(defmethod get-obj-to-play ((self MaquettePanel))
  (if (and (eval-func (object self)) (value (object self)))
    (if *microplay* 
      (list (value (object self)))
      (list (value (object self)) :approx 8))
    (let ((boxestoplay (loop for b in (boxes (object self)) when (and (boxtempobj-p b) (not (mute b))) collect b)))
      (list (cons-play-maquette-object (object self) boxestoplay)))
    ))

(defmethod selection-to-play-? ((self MaquettePanel))
   (or (cursor-p self) (get-actives self)))
                   
(defmethod get-selection-to-play ((self MaquettePanel))
   (cond
    ((cursor-p  self)
     (let ((interval (cursor-interval self)))
       (values (list (if (and (eval-func (object self)) (value (object self)))
                       (list (value (object self)))
                       (let ((boxestoplay (loop for b in (boxes (object self)) when (and (boxtempobj-p b) (not (mute b))) collect b)))
                         (list (cons-play-maquette-object (object self) boxestoplay)))
                       )
                     :interval interval)
               (first interval)
               (second interval))))
    ((get-actives self)
     (let* ((minim (loop for item in (get-actives self)
                         minimize (offset (object item))))
            (maxim (loop for item in (get-actives self)
                         maximize (+ (offset (object item)) (extend (object item))))))
       (values (list (if (and (eval-func (object self)) (value (object self)))
                       (list (value (object self)))
                       (let ((boxestoplay (loop for b in (get-actives self) when (and (boxtempobj-p (object b)) (not (mute (object b)))) collect (object b))))
                         (list (cons-play-maquette-object (object self) boxestoplay)))
                       ) 
                     :interval (list minim maxim))
               minim
               maxim)))))



;-------------------Markers

(defmethod delete-general ((self MaquettePanel))
   "Remove boxes markers lines or connecctions from 'self'."
   (let ((markers (selected-mark-lines self)))
     (if markers
       (progn
         (draw-mark-lines self nil)
         (loop for item in markers do
               (setf (tempojlist (car item))
                     (remove (second item) (tempojlist (car item)) :test 'equal :key 'second)))
         (setf (selected-mark-lines self) nil)
         (draw-mark-lines self))
       (call-next-method))))

(defmethod update-after-mark  ((self MaquettePanel)) 
  (loop for item in (get-maquette-markers self) do
        (loop for box in (tempojlist (object item)) do
              (let ((frame (car (frames (second box)))))
                (cond
                 ((= 1 (first box))
                  (om-set-view-size frame (om-make-point (- (x item) (x frame)) (h frame)))
                  (om-invalidate-view frame t))
                 ((= 0 (first box))
                  (update-movement frame (om-make-point (x item) (y frame)))))))))
                 
;;; modifie jb
(defmethod draw-mark-lines ((self MaquettePanel) &optional (mode t))
   (om-with-focused-view self
     (loop for item in (get-maquette-markers self) do
           (loop for frame in (tempojlist (object item)) do
                 (let* ((source (car (frames (second frame))))
                        (x1 (if (zerop (first frame)) (x source) (x+w source)))
                        (y1 (+ (y source) (h source)))
                        (x2 (x item))
                        (y2 (y item))
                        (size 2)
                        modepat)
                   (if (null mode) 
                     (progn
                       (om-with-pen (self :size size)
                         (om-erase-line (om-make-point x1 y1) (om-make-point x2 y2))
                         ))
                     (progn 
                       (cond
                        ;;;((null mode) (setf modepat :white-line))
                        ((member (list (object item) (second frame)) (selected-mark-lines self) :test 'equal)
                         (setf modepat :black-line))
                        (t (setf modepat :gray-line)))
                       (om-with-pen (self :line modepat :size size)
                         (om-draw-line x1 y1 x2 y2)))))))))
                                               
(defmethod get-maquette-markers ((self MaquettePanel))
   "Return a list of the markers in 'self'."
   (let* (rep)
     (mapc #'(lambda (item)
               (if (markerframe-p item)
                 (push item rep))) (om-subviews self))
     rep))

(defmethod click-in-marker-line ((self MaquettePanel) where)
   "T if where is on a marker connection."
   (let ((markers (get-maquette-markers self))
         (x0 (om-point-h where))
         (y0 (om-point-v where))
         rep)
     (loop for marker in  markers
           while (not rep) do
           (when (and (>= x0 (- (x marker) 2)) (<= x0 (+ (x marker) 2)))
             (loop for box in (tempojlist (object marker))
                   while (not rep) do
                   (let* ((temobj (second box))
                          (temframe (car (frames temobj))))
                     (when (>= y0 (y+h temframe))
                       (setf rep (list (object marker) temobj)))))))
     (when rep
       (draw-mark-lines self nil)
       (cond 
        ((om-shift-key-p)
         (if (member rep (selected-mark-lines self) :test 'equal)
           (setf (selected-mark-lines self) (remove rep (selected-mark-lines self) :test 'equal))
           (push rep (selected-mark-lines self))))
        (t (setf (selected-mark-lines self) (list rep))))
       (draw-mark-lines self))
     rep))


;========================================================
;========================================================
;========================================================
;========================================================
;Marker Class. It is not so good I think that we can make better
;========================================================
;========================================================
;========================================================
;========================================================



(defclass temp-marker (omboxcall)
   ((tempojlist :initform nil :accessor tempojlist)
    (offset :initform nil :initarg :offset :accessor offset))
   (:documentation "This is the class for red flag boxes into a maquette.
#seealso# (OMMaquette temporalbox markerframe)#seealso#
#tempojlist# This slot store a list of pairs (begin/end , temporal box)  where the second of each element are
boxes attached to the flag, the car of each element say if the box is attached at its begin or its end.#tempojlist#
#offset# Contains the x position of the box (in ms).#offset#"))


;------Predicats
(defmethod marker-p ((self temp-marker)) t)
(defmethod marker-p ((self t)) nil)


(defmethod (setf frame-position) ((pos integer) (self temp-marker))
  (setf (frame-position self) (om-correct-point pos )))
;-------Info
(defmethod get-frame-class ((self temp-marker)) 'markerframe)
(defmethod numouts ((self temp-marker)) 0)
(defmethod get-frame-name ((self temp-marker)) nil)
(defmethod do-add-one-input ((self temp-marker)) (om-beep))
(defmethod get-icon-box-class ((self temp-marker)) 'bandera)

;-------Constructor
(defun omNG-make-new-marker (posi name offset)
   (let ((rep (make-instance 'temp-marker 
                :name name
                :reference nil 
                :icon 143
                :offset offset
                :inputs nil)))
     (setf (frame-position rep) (borne-position posi))
     rep))

(defmethod make-frame-from-callobj ((self temp-marker))
   "Cons a simple frame for a temp-marker instance."
   (let* ((module (om-make-view 'markerframe 
                                 :position (frame-position self) 
                                 :size  (om-make-point 8 10)
                                 :bg-color (om-make-color 1 0 0)
                                 :object self)))
     (om-add-subviews module (setf (iconView module)
                                (om-make-view 'bandera
                                                             :iconID (icon self)
                                                             :help-spec "Click + Shift key to connect to a temporal box"
                                                             :size (om-make-point 8 10)
                                                             :bg-color (om-make-color 0 1 0)
                                                             :position (om-make-point 0 0))))
     (setf (name module) (name self))
     (setf (frames self) (list module))
     module))

;-----------Edition

(defmethod OpenEditorframe ((self temp-marker)) nil)

(defmethod change-offset ((self temp-marker) offset)
   "Move 'self' to the new position 'offset'. If 'self' has a simple frame active the graphic update is made too."
   (if (and (integerp offset) (not (minusp offset)))
     (if (car (frames self))
       (let* ((container (om-view-container (car (frames self))))
              (sys-etat (get-system-etat container))
              (new-pos (point2pixel container  (om-make-big-point offset 0) sys-etat)))
         (OMGMoveObject (car (frames self)) new-pos)
         (setf (offset self) offset))
       (setf (offset self) offset))
     (om-beep-msg "incorrect offset value")))

(defmethod add-mark ((self TemporalBox) (marker temp-marker) typesource)
   "Push the pair ('typesource' 'self') in the tempojlist slot of 'marker' (connect)."
   (push (list typesource self) (tempojlist marker)))
  
(defmethod del-mark ((self TemporalBox) (marker temp-marker) typesource )
   "Remove the pair ('typesource' 'self') from the tempojlist slot of 'marker' (unconnect)."
   (setf (tempojlist marker) (remove (list typesource self) (tempojlist marker) :test 'equal)))


;------------TOOLS
(defun copy-markers (listsource listtarget)
   (loop for  marker in listsource
         for i = 0 then (incf i) do
         (when (equal (class-name (class-of marker)) 'temp-marker)
           (loop for item in (tempojlist marker) do
                 (push (list (first item)
                             (nth (position (second item) listsource :test 'equal) listtarget))
                       (tempojlist (nth i listtarget)))))))


(defun mk-markers-list (list)
   (let (rep)
     (loop for marker in list
           for i from 0 to (length list) do
           (when (equal (class-name (class-of marker)) 'temp-marker)
             (loop for item in (tempojlist marker) do
                   (push (list i (first item) (position (second item) list :test 'equal))
                         rep))))
     rep))

(defun remk-markers (list boxes)
   (loop for item in list do
         (push (list (second item)
                     (nth (third item) boxes))
               (tempojlist (nth (first item) boxes)))))
      
;-------------------------------------------
;Simple frame
;-------------------------------------------

;------------Icon class
(omg-defclass bandera (icon-box) ())

(defmethod om-drag-selection-p ((self bandera) mouse-position)
   (declare (ignore mouse-position))
   (not (or (om-control-key-p) (om-shift-key-p))))

(defmethod om-view-cursor :around ((self bandera))
   *om-arrow-cursor*)

;------------simple frame class
(omg-defclass markerframe (boxframe) ())

;;;(defmethod om-view-click-handler ((self markerframe) where)
;;;   (when (show-con? (om-view-container self))
;;;     (cond ((om-shift-key-p) (drag-out-line self where))
;;;           ((om-command-key-p) (om-set-help t))
;;;           ((om-control-key-p)))))
(defmethod om-view-click-handler ((self markerframe) where)
  (OMG-select self) 
   (when (show-con? (om-view-container self))
     (cond ((om-shift-key-p) (drag-out-line self where))
                 (t nil))))

;;; provisoire !!!
(defmethod om-view-click-handler ((self bandera) where)
  (om-view-click-handler (om-view-container self) where))


;;; modif :draw on panel
(defmethod drag-out-line ((view markerframe) where)
   (let* ((win (panel (om-view-window view)))
          (last-mp (om-mouse-position win)) ctrl)
     (setq where (om-mouse-position win))
     
     (om-with-focused-view win
                         (om-with-pen (win :mode :xor :size 4 :line :gray-line)
                                        (loop while (om-mouse-down-p)  do
                                                  (let ((mp (om-mouse-position win)))
                                                    (when (not (om-points-equal-p mp last-mp))
                                                        (om-draw-line (om-point-h where) (om-point-v where) (om-point-h last-mp) (om-point-v last-mp))
                                                        (setq last-mp mp)  
                                                        (om-draw-line (om-point-h where) (om-point-v where) (om-point-h last-mp) (om-point-v last-mp))
                                                        ))
                                                  (om-get-next-event))
                                        
                                        (om-draw-line (om-point-h where) (om-point-v where) (om-point-h last-mp) (om-point-v last-mp))
                                        )
   )
     (setf ctrl (om-find-view-containing-point win last-mp))
     (om-invalidate-view view t)
     (connect-box view ctrl)))

(defmethod size-offset-frame-update ((self markerframe) container)
  (declare (ignore container))
  (om-beep-msg "Error"))

(defmethod markerframe-p ((self markerframe)) t)
(defmethod markerframe-p ((self t)) nil)

(defmethod init-size&position ((self markerframe) container)
   (let ((sysetat (get-system-etat container)))
     (om-set-view-position self (om-make-point (om-point-h (point2pixel container (om-make-big-point (offset (object self)) 0)
                                                               sysetat)) (- (h container) 10)))))

(defmethod OMGMoveObject ((self markerframe) new-position)
   "Move 'self' to 'new-position'"
   (if (< (om-point-h new-position) 0)
     (om-beep-msg "Object must start after 0"))
   (let ((cantmove? (can-move-marker? self new-position)))
     (if cantmove?
       (progn
         (om-beep-msg "The selected temporal object does not allow this movement")
         (omg-select (car (frames cantmove?))))
       (progn
         (mapc #'(lambda (conection)
                   (draw-connection conection nil)) (connections self))
         (draw-mark-lines (om-view-container self) nil)
         (setf new-position (om-make-point (om-point-h new-position) (- (h (om-view-container self)) 10)))
         (om-set-view-position self new-position)
         (setf (frame-position (object self)) (borne-position new-position))
         (setf (offset (object self)) (om-point-h (pixel2point (om-view-container self) (om-make-point (x self) 0))))
         (update-after-mark (om-view-container self))
         ;;(draw-mark-lines (om-view-container self))
         ;; ***
         (om-invalidate-view (om-view-container self))
         ))))

(defmethod can-move-marker? ((self markerframe) new-position)
   (let* ((marker (object self))
          (attached (tempojlist marker))
          (mark-list (get-maquette-markers (om-view-container self)))
          (offset (om-point-h (pixel2point (om-view-container self) new-position)))
          rep)
     (loop for item in attached 
           while (not rep) do
           (let* ((temobj (second item))
                  (startmarker (is-marketed temobj 0 mark-list))
                  (finmarker (is-marketed temobj 1 mark-list)))
             (cond
              ((and startmarker (not (equal startmarker marker)))
               (if (<= offset (offset startmarker))
                 (setf rep temobj)))
              ((and (equal startmarker marker) finmarker)
               (if (>= offset (offset finmarker))
                 (setf rep temobj)))
              ((and finmarker (not (equal finmarker marker)))
               (if (>= offset (offset finmarker))
                 (setf rep temobj)))
              ((and (equal finmarker marker) startmarker)
               (if (<= offset (offset startmarker))
                 (setf rep temobj)))
              ((equal finmarker marker)
               (unless (allow-strech-p (soft-get-mus-ob temobj) 0)
                 (setf rep temobj))
               (if (<= (om-point-h new-position) (x (car (frames temobj))))
                 (setf rep temobj))))))
     rep))


(defmethod init-size&position ((self t) container)
  (declare (ignore container)))





