;OpenMusic
;
;Copyright (C) 1997, 1998, 1999, 2000 by IRCAM-Centre Georges Pompidou, Paris, France.
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
;
;Authors: Gerard Assayag and Augusto Agon


;DocFile
;Dialog to define get and set slots Methods.
;Last Modifications :
;18/10/97 first date.
;DocFile
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(in-package :om)

(defun open-dialog-slots (class)
   (let* ((slots (get-initargs-of-class  (class-name class)))
          (thedialog (om-make-window 'om-dialog 
                                                        :position :centered
                                                        :window-title "Slots functions"
                                                        :size (om-make-point (+ 5 (* 50 (length slots))) 130)
                                                        :bg-color *om-light-gray-color*
                                                        :resizable nil
                                                        :maximize nil
                                     )))
     (loop for item in slots
           for i = 0 then (incf i) do
             (let* ((slot-name (first item))
                     (slot-type (second item))
                     (thetype (string slot-type)) thetype?)
               (when (setf thetype? (find-if 
                                                #'(lambda (item) (string-equal (name item) thetype))
                                                (list+ *Basic-Lisp-Types* *OM-class-list*)))
                   (om-add-subviews thedialog
                                               (om-make-view 'icon-view 
                                                                       :position (om-make-point (+ 15 (* i 50)) 50) 
                                                                       :size (om-make-point 24 24) 
                                                                       :iconID (icon thetype?))))
               (om-add-subviews thedialog
                                           (om-make-dialog-item 'om-button (om-make-point (+ 4 (* i 50)) 10) (om-make-point 50 25) "set" 
                                                                             :di-action  (om-dialog-item-act x (declare (ignore x))
                                                                                                                             (open-set-slots thedialog class slot-name slot-type ))))
               (om-add-subviews thedialog
                                           (om-make-dialog-item 'om-static-text (om-make-point (+ 4 (* i 50)) 75) (om-make-point 50 25) 
                                                                             (string-downcase (string (first item)))
                                                                             :bg-color *om-light-gray-color*))
               (om-add-subviews thedialog
                                           (om-make-dialog-item 'om-button (om-make-point (+ 4 (* i 50)) 100) (om-make-point 50 25) "get" 
                                                                             :di-action  (om-dialog-item-act x (declare (ignore x))
                                                                                                                             (open-get-slots thedialog class slot-name))))))
     
     (om-modal-dialog thedialog)))


(defun open-get-slots (dialog class slot-name)
   (let ((fun-name (internp (string+ "&get" (string slot-name) "-" (string (class-name class))) 
                            (symbol-package (class-name class))))
         thegetmethod)
     (unless (fboundp fun-name)
       (let (newmethod input1 out1 set1)
         (setf newmethod (eval `(defmethod* ,fun-name ((self ,(class-name class))) nil)))
         (setf input1 (make-new-typed-input "self" (class-name class) 0 (om-make-point 15 15)))
         (setf out1 (make-new-output "out" 0 (om-make-point 15 250)))
         (setf set1 (omNG-make-new-boxcall (fdefinition 'get-slot) (om-make-point 30 150) "get"))
         (setf (class-method-p newmethod) 'get)
         (push newmethod (internal-met class))
         (setf (graph-fun newmethod) nil)
         (omng-add-element newmethod input1)
         (omng-add-element newmethod out1)
         (omng-add-element newmethod set1)
         (omNG-connect input1 0  set1 0 nil)
         (omNG-connect set1 0  out1 0 nil)
         (setf (value (nth 1 (inputs set1))) slot-name)))
     (setf thegetmethod (car (generic-function-methods  (fdefinition fun-name) )))
     (when thegetmethod
       (openObjecteditor thegetmethod))
     (om-return-from-modal-dialog dialog ())))

(defun open-set-slots (dialog class slot-name type)
   (let ((fun-name (internp (string+ "@set" (string slot-name) "-" (string (class-name class))) 
                            (symbol-package (class-name class))))
         thesetmethod)
     (unless (fboundp fun-name)
       (let (newmethod input1 input2 out1 set1)
         (setf newmethod (eval `(defmethod* ,fun-name ((self ,(class-name class)) (val ,type)) nil)))
         (setf input1 (make-new-typed-input "self" (class-name class) 0 (om-make-point 15 15)))
         (setf input2 (make-new-typed-input "val" type 1 (om-make-point 150 15)))
         (setf out1 (make-new-output "out" 0 (om-make-point 15 250)))
         (setf set1 (omNG-make-new-boxcall (fdefinition 'set-slot) (om-make-point 30 150) "set"))
         (setf (graph-fun newmethod) nil)
         (omng-add-element newmethod input1)
         (omng-add-element newmethod input2)
         (setf (value (nth 1 (inputs set1))) slot-name)
         (omng-add-element newmethod out1)
         (omng-add-element newmethod set1)
         (omNG-connect input1 0  set1 0 nil)
         (omNG-connect input2 0  set1 2 nil)
         (omNG-connect set1 0  out1 0 nil)
         (setf (class-method-p newmethod) 'set)
         (push newmethod (internal-met class))))
     (setf thesetmethod (car (generic-function-methods  (fdefinition fun-name))))
     (when thesetmethod
       (openObjecteditor thesetmethod))
     (om-return-from-modal-dialog dialog ())))

(defun getsetfunname (str)
   (let ((index (search (format nil "-") str)))
     (if index (read-from-string (subseq str 4 index)) 
         (error "bad set-get slot name"))))



                    
                   
            
  