;OpenMusic
; 
;Copyright (C) 1997, 1998, 1999, 2000 by IRCAM-Centre Georges Pompidou, Paris, France.
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
;
;Authors: Gerard Assayag and Augusto Agon


;DocFile
;Window, editor and panel abstract classes are defined in this file.
;Last Modifications :
;18/10/97  first date
;DocFile

;;; Notes :


;;; - ccl::window-close-nicely definition commente
;;; - drag-receive-dropped-flavor definition commente


(in-package :om)


;------------------------------------------------------------

;==============
;=== WINDOW ===
;==============

;;; obj est rajoute pour associer un objet a une fenetre
;;; pour les pb de menubar en multiplateforme
(omg-defclass EditorWindow (om-window) 
   ((editor :initform nil  :accessor editor)
    (obj :initform nil  :initarg :obj :accessor obj))
   )

(defmethod omG-make-new-icon-window ((self EditorWindow) &optional type)
   "Called from New menu item type say if it create a patch a maquettte or a folder."
   (editor-make-new-icon-window (editor self) type))

(defmethod om-minimum-size ((window editorwindow))
  (editor-minimum-size (editor window)))

(defmethod editor-minimum-size ((self t)) nil)


(defmethod do-undo ((self t)) nil)

(defmethod undo ((self EditorWindow))  
   (do-undo (editor self)))

(defmethod omG-change-presentation ((self EditorWindow) presentation)
   "Show icons in 'self' by name by type or normal."
  (without-interrupts 
   (editor-change-presentation (editor self) presentation)
   (om-window-resized self (om-view-size self))
   ;; test :
   (om-invalidate-view self)
   ))

;;; regarder MCL peut etre a appeler dans chaque methode de events...
;;;(defmethod window-event :after ((self EditorWindow))
;;;(defmethod om-window-event :after ((self EditorWindow))

;;; C'ETAIT POUR LES TOOL TIPS
;;;(defmethod  om-view-mouse-moved-handler :after ((self EditorWindow) position)
;;;   (declare (ignore position))
;;;   (editor-event-after (editor self)))

(defmethod alias-window ((self EditorWindow))
   "Make aliases of the selected icons." 
   (alias-editor (editor self)))

(defmethod om-view-activate :after ((self EditorWindow))
   ;;; le menu est gere directement dans l'api
   ;;;(update-themenubar self)
   (change-palette self)
   )

(defmethod get-info-window ((self EditorWindow))
   (let ((controls (get-actives (panel (om-front-window)))))
     (loop for item in controls do
               (show-info-window 
                (get-obj-for-info item)))))

(defmethod window-save ((self EditorWindow))
   (editor-save (editor self)))

(defmethod copy ((self EditorWindow))
   (or (editor-copy (editor self)) (call-next-method)))

(defmethod paste ((self EditorWindow)) 
   (unless (editor-paste (editor self)) (call-next-method)))

(defmethod duplicate-window ((self t))
   "Copy and paste the selected icons." nil)

(defmethod duplicate-window ((self EditorWindow))
   "Copy and paste the selected icons."
   (when (copy self) (paste self)))

(defmethod cut ((self EditorWindow)) (editor-cut (editor self)))

(defmethod clear ((self EditorWindow)) (editor-clear (editor self)))

(defmethod select-all ((self EditorWindow)) (editor-select-all (editor self)))

(defmethod get-clipboard ((self EditorWindow)) nil)

(defmethod get-editor-class ((self EditorWindow)) nil)

(defmethod window ((self EditorWindow)) self)

(defmethod panel ((self EditorWindow))
   (panel (editor self)))

(defmethod get-win-ed-size ((self t)) (om-make-point 335 275))

(defmethod get-win-ed-pos ((self t)) (om-make-point 10 40))


(defmethod om-window-class-menubar ((self EditorWindow))
    
   (let ((editor (editor self))
          menubar firstitem)
     
     ;;; quand une fenetre est creee (om-make-window) elle n'a pas forcement d'editor au debut
     (when (and (not editor) (obj self))
        (setf editor (make-instance (get-editor-class (obj self)) :ref nil :object (obj self) )))
     
     (update-themenubar editor)
     
    (when editor (setf menubar (get-menubar editor))
        (setf firstitem (om-get-menu-item *window-menu-file* 0))
        (unless (string-equal (om-menu-item-title firstitem) "Close")
           (om-remove-menu-items *window-menu-file* firstitem))
        (change-new-menu *window-menu-file* (get-new-menu editor)))
     menubar))



(defmethod make-editor-window ((class t) object name ref &key 
                               winsize winpos (close-p t) (winshow t) (resize t) (retain-scroll nil)
                               (wintype nil))

   (let* ((sizewin (or winsize (get-win-ed-size object)))
          (poswin (or winpos (get-win-ed-pos object)))
          ;;;(ontop (equal wintype :topmost))
          (win (om-make-window 'EditorWindow 
                               :window-title name
                               :position poswin 
                               :close close-p
                               ;;;   :window-type :document-with-zoom
                               ;;;:topmost ontop
                               :resizable resize
                               :maximize resize
                               :window-show nil
                               :size sizewin
                               :obj object))
          (editor (om-make-view class
                                :ref ref
                                :owner win
                                :object object
                                :position (om-make-point 0 0)
                                :size (om-interior-size win)
                                ))
          panel)
     
     (setf (editor win) editor)
     
     ;; mac
     ;;(update-themenubar win)
     ;;; only for win et linux 
     ;;(om-add-menu-to-win win (get-editor-menu editor))
     
     (om-set-view-size editor (om-interior-size win))
     
     (when winshow
         (om-select-window win))
     win))



;;;(defmethod window-zoom-event-handler ((self EditorWindow) h )
;;;  (declare (ignore h ))
;;;  (without-interrupts
;;;   (call-next-method)
;;;   (om-set-view-size (editor self) (view-size self))))
(defmethod om-window-resized ((self EditorWindow) size)
  (declare (ignore size))
   (when (editor self)
       (without-interrupts 
        (call-next-method)
        (om-set-view-size (editor self) (om-interior-size self))
        )))


(defmethod om-set-view-size ((self EditorWindow) size) 
   (declare (ignore size))
   (call-next-method)  
   (without-interrupts 
    (om-set-view-size (editor self) (om-interior-size self))
    ))

(defmethod om-set-interior-size ((self EditorWindow) size) 
   (declare (ignore size))
   (call-next-method)  
   (without-interrupts 
    (om-set-view-size (editor self) (om-interior-size self))
))

(defmethod om-view-key-handler ((self EditorWindow) char)
  (unless (call-next-method)
    (handle-key-event (editor self) char)))

(defmethod handle-key-event ((self t) char) nil)

(defmethod om-window-close-event :before ((self EditorWindow))
  (when (editor self)
    (close-editor-before (editor self))))

(defmethod om-window-close-event :after ((self EditorWindow)) 
   (when (editor self)
       (loop for ed in (attached-editors (editor self)) do
                 (om-close-window ed))
       (close-editorFrame (editor self))
       (close-editor-after (editor self))
       ))

   

;;; !!!! 
(defmethod window-open-palette ((win EditorWindow))
  (unless (and (palette *global-palette*) 
               (equal (editor-assoc (palette *global-palette*)) (editor win)))
    ;;(print (list "editoropenpalette" win (editor win) 
    ;;             (if (palette *global-palette*) 
    ;;               (equal (editor-assoc (palette *global-palette*)) (editor win)) 
    ;;               "pas de palette")))
    (call-next-method)
    (Editor-Open-Palette (editor win))
    )
   )


(defmethod om-view-key-handler :around ((self EditorWindow) char)
   (if (key-event-around (editor self) char)
     (call-next-method)))

;;;(defmethod window-null-event-handler ((self EditorWindow) position)
;;;   (call-next-method)
;;;   (editor-null-event-handler (editor self)))
(defmethod om-window-mouse-moved-handler ((self EditorWindow) position)
   (call-next-method)
   (editor-null-event-handler (editor self)))

(defmethod workspace-window-p ((self EditorWindow))
   (equal *current-workSpace* (and (editor self) (object (editor self) ))))

(defmethod workspace-window-p ((self t)) nil)

;=============
;EDITOR
;=============

(omg-defclass EditorView (OMSimpleFrame) 
   ((selected-p :initform nil :accessor selected-p)
    (mini-editor-p :initform nil :initarg :mini-editor-p :accessor mini-editor-p)
    (panel :initform nil :initarg :panel :accessor panel)
    (text-view :initform nil :accessor text-view)
    (ref :initform nil  :initarg :ref :accessor ref)
    (attached-editors :initform nil :accessor attached-editors)
    (undo :initform nil :accessor undo)))

(defmethod Editor-Open-Palette  ((self EditorView))
  (when (editor-has-palette-p self)
    (setf (palette *global-palette*)
      (om-make-window (editor-has-palette-p self)
                      :editor-assoc self
                      :window-title "General Palette"
                      :position (pos-palette *global-palette*)
                      :size (om-make-point 25 25)
                      :close nil :resizable nil))
    ;(om-select-window (window self))
    (init-palette (palette *global-palette*))
    ))


(defmethod init-palette ((self t)) nil)


(defmethod EditorView-p ((self EditorView)) t)
(defmethod EditorView-p ((self t)) nil)

(defmethod InternalEditor-p ((self EditorView)) 
   (EditorView-p (ref self)))


(defmethod window ((self EditorView)) (om-view-window self))

(defmethod editor ((self EditorView)) self)


;Events from the window pour enlever les tooltips...  mettre dans l'api?
;;;(defmethod editor-event-after ((self EditorView)) 
;;;   (let ((mouse (om-mouse-position self)))
;;;     (when (and (help-on?) (not (om-points-equal-p mouse *mouse-help-position*)))
;;;       (om-set-help nil))
;;;     (setf *mouse-help-position* mouse)))


(defmethod close-editor-before ((self EditorView))  
   (when (palette *global-palette*)
   (om-close-window (palette *global-palette*))))

(defmethod close-editor-after ((self EditorView))
  nil)

(defmethod editor-copy ((self EditorView))
   "Copy in the clipboard the selected icons in 'self'."
   t)

(defmethod editor-paste ((self EditorView))
   "Paste from the clipboard to 'self'."
   t)


#|
(defmethod editor-copy ((self EditorView))
   "Copy in the clipboard the selected icons in 'self'."
   nil)

(defmethod editor-paste ((self EditorView))
   "Paste from the clipboard to 'self'."
   nil)

|#
(defmethod editor-change-presentation ((self EditorView) presentation)
   "Show icons in 'self' by name by type or normal." 
   nil)

(defmethod editor-make-new-icon-window ((self EditorView) &optional type)
   "Called from New menu item type say if it create a patch a maquettte or a folder."
   nil)

(defmethod editor-cut ((self EditorView))
   "Cut the selected icons in 'self'."
   nil)

(defmethod editor-clear ((self EditorView))
   "Clear the selected icons in 'self'."
   nil)

(defmethod editor-select-all ((self EditorView))
   "Select all icons in 'self'."
   nil)

(defmethod editor-save ((self EditorView))
   "Save the object associated to the selected icons in 'self'."
   nil)





;===============

(defmethod get-defaults-edit-param ((self EditorView)) 
   (default-edition-params (object self)))

(defmethod om-view-click-handler ((self EditorView) where)
   (declare (ignore where))
   (if (and (mini-editor-p self) (not (selected-p self)))
     (progn (setf (selected-p self) t) (om-invalidate-view self t))
     (call-next-method)))

(defmethod (setf selected-p) (selected-p (self EditorView))
   "Set the slot selected-p and show the palette if selected-p equal True"
   (setf (slot-value self 'selected-p) selected-p)
   (if selected-p
     (Editor-Open-Palette self)
     (when (palette *global-palette*)
    (om-close-window (palette *global-palette*)))))

(defmethod om-draw-contents ((self EditorView))
   (call-next-method))

(defmethod editor-has-palette-p ((self EditorView)) nil)

(defmethod get-pos-in-object ((self EditorView) where)
  (om-add-points (om-view-position self) where))


(defmethod om-set-view-size ((self editorView) size)
   (declare (ignore size))
   (call-next-method)
   (update-subviews self))

(defmethod handle-key-event ((self EditorView) char)
   (if (text-view self)
       (om-view-key-handler (text-view self) char)
     (handle-key-event (panel self) char)))


(defmethod editor-null-event-handler ((self EditorView)) nil)

(defmethod key-event-around ((self EditorView) char) 
t)

(defmethod update-editor-after-eval ((self EditorView) val)
  (setf (object self) val)
  (om-invalidate-view self t))


(defmethod close-editorFrame ((self EditorView))
  (cond ((null (ref self)) nil)
        ((EditorView-p (ref self))
         (setf (attached-editors (ref self)) 
               (remove self (attached-editors (ref self)) :test 'equal))
          (om-invalidate-view (ref self) t))
         ((slot-initform-p (ref self))
          (change-initform-ed (ref self) (object self)))
         ((ominstance-p (ref self))
          (setf (Editorframe (ref self)) nil))
         ((boxtempobj-p (ref self))
          (setf (Editorframe (ref self)) nil)
          (when (frames (ref self))
            (om-invalidate-view (car (frames (ref self))))))
         ((is-boxpatch-p  (ref self))
          (setf (Editorframe (ref self)) nil)
          (when (frames (ref self))
            (om-invalidate-view (car (frames (ref self))))))))

(defmethod editor-params-edition ((self EditorView))
   (cond ((or (ominstance-p (ref self)) (is-boxpatch-p  (ref self)))
          (edition-params (ref self)))
         ((EditorView-p (ref self)) (get-defaults-edit-param (ref self))) ;Mirar si es la misma clase de editor
         (t (get-defaults-edit-param self))))

(defmethod get-edit-param ((self EditorView) param)
   ;;; renvoie sur la liste d'assoc
   (get-edit-param (editor-params-edition self) param)
   )

(defmethod get-edit-param ((self list) param)
   (cdr (assoc param self)))


(defmethod set-edit-param ((self EditorView) param val)
   (when (editor-params-edition self)
     (rplacd (assoc param (editor-params-edition self)) val)))

(defmethod update-subviews ((self EditorView)) nil)

(defmethod calcule-page-size ((view EditorView) page-size compute-doc-size)
  (declare (ignore page-size compute-doc-size))
  (values 1 1 1))

(defmethod close-attached-editors ((self EditorView))
   (loop for ed in (attached-editors self) do
         (om-close-window ed)))

(defmethod update-panel ((self EditorView) &optional (updateref nil))
   (when updateref
     (report-modifications self))
   (om-invalidate-view self t)
   )


(defmethod report-modifications ((self EditorView))
  (cond ((null (ref self)) nil)
        ((and (or (is-boxpatch-p (ref self)) (boxtempobj-p (ref self))) (car (frames (ref self))))
         (when (lock-button (car (frames (ref self))))
           (remove-lock-button (car (frames (ref self)))))
         (add-lock-button (car (frames (ref self))))
         (when (showpict (ref self))
           (if (boxtempobj-p (ref self))
             (update-miniview (car (frames (ref self))) (car (value (ref self))))
             (update-miniview (iconview (car (frames (ref self)))) (value (ref self))))))
        ((EditorView-p (ref self))
         (change-in-int-editor (panel (ref self)) (panel self) (object self) (object self)))
        ;;; new sheet
        ((ref self)
         (change-in-int-editor (ref self) (panel self) (object self) (object self))
         )))


(defmethod change-in-int-editor ((self t) (internal t) newobj lastobj) t)






