;OpenMusic
;
;Copyright (C) 1997, 1998, 1999, 2000 by IRCAM-Centre Georges Pompidou, Paris, France.
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
;
;Authors: Gerard Assayag and Augusto Agon


;DocFile
;Class hierarchical tree editor.
;Last Modifications :
;18/10/97 first date.
;DocFile
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(in-package :om)

(defun open-new-RelationTree (object name elements)
   (let* (newwindow)
     (setf newwindow 
           (make-editor-window 'classTreeEditor object name nil 
                               :winsize (get-win-size object) 
                               :winpos (get-win-position object)))
     (mapc #'(lambda (elem)
               (let ((newframe (make-frame-from-callobj elem)))
                 (om-add-subviews (panel newwindow) newframe)
                 (add-subview-extra newframe))) elements)
     (mapc #'(lambda (elem)
               (update-graphic-connections elem elements)) (get-subframes (panel newwindow)))
     (add-window-buttons (panel newwindow))
     newwindow))

;---------------------------------------------

(omg-defclass classTreeEditor (relationEditor)  ()
   (:documentation "xx"))

(defmethod get-editor-panel-class ((self classTreeEditor))  'classTreePanel)

(defmethod om-view-close-event ((self classTreeEditor)) nil)
(defmethod get-clipboard ((self classTreeEditor)) nil)



(defmethod editor-make-new-icon-window ((self classTreeEditor) &optional type)
   (declare (ignore type))
   (if (protected-p (object (panel self)))
     (dialog-message "This package is protected, you can not add classes to it!")
     (let* ((scrollframe (panel self))
            new-object new-frame dial string iconID doc)
       (setf dial (get-classname-dialog))
       (when dial
         (setf string (first dial))
         (setf iconID (third dial))
         (setf doc (second dial))
         (when (string-equal doc "")
           (setf doc "non documentation"))
         (when (and string (not (string-equal "" string)))
           (if (exist-class-p string) 
             (progn
               (dialog-message (string+ "The class " string " is already defined!"))
               (om-abort))
             (progn
               (setf new-object (eval `(defclass* ,(interne string ) () () (:icon ,(car (list! iconID))) (:documentation ,doc))))
               (when (listp iconID)
                 (icon-for-user-package new-object (second iconID)))
               (setf new-frame (omNG-add-element (object scrollframe) new-object))
               (om-add-subviews scrollframe (make-frame-from-callobj new-frame)))))))))


;--------------
;PANEL
;--------------

(omg-defclass classTreePanel (relationPanel) ()
   (:documentation "This is the class for hierarchical class tree editors.
Boxes refer to classes and connections to inheritance.#enddoc#
#seealso# (OMBoxClass classboxframe) #seealso#"))

;This methods works only in patches
(defmethod clear-ev-once ((self classTreePanel)))
(defmethod make-undefined-box ((self classTreePanel) x) (declare (ignore x)))
(defmethod make-delete-before ((self classTreePanel) dragged cible) (declare (ignore dragged cible)))


;-------------------------
(defmethod omg-remove-element ((self classTreePanel) frame)
   "We can remove only alias from 'self' if it is the case we redefine classes inheriting from the alias class."
   (let ((subclasses (get-subclasses-list (object frame) (mapcar #'(lambda (fra)
                                                                     (object fra)) (get-subframes self)))))
     (loop for item in subclasses do
           (let ((classbox (car (frames (first item)))))
             (remove-connection classbox  (second item))
             (unconnect-class (first item) (second item))
             (redraw-frame classbox)))
     (omng-remove-element (object self) (object frame))
     (om-remove-subviews self frame)))


(defmethod handle-key-event ((self classTreeEditor) char) 
   (let ((container (panel self)))
     (case char
       (:om-key-delete (let ((actives (get-actives container))
                       (connections (get-actives-connections container)))
                   (loop for item in actives do
                         (when (equal (class-name (class-of item)) 'aliasBoxframe)
                           (omg-remove-element container item)))
                   (mapc #'(lambda (connec)
                             (let ((thebox (thebox connec)))
                               (if (protected-p (find-class (get-reference (object thebox))))
                                 (om-beep-msg (string+ "The class " (string (get-reference (object thebox))) "  is protected"))
                                 (progn
                                   (remove-connection thebox (index connec))
                                   (unconnect-class (object thebox) (index connec))
                                   (redraw-frame thebox))))) connections)))
       (:om-key-up (mapc #'(lambda (item) (move-frame-delta item 0)) (get-actives container))
        (make-move-after container (get-actives container)))
       (:om-key-down (mapc #'(lambda (item) (move-frame-delta item 1)) (get-actives container))
        (make-move-after container (get-actives container)))
       (:om-key-left  (mapc #'(lambda (item) (move-frame-delta item 3)) (get-actives container))
        (make-move-after container (get-actives container)))
       (:om-key-right  (mapc #'(lambda (item) (move-frame-delta item 2)) (get-actives container))
        (make-move-after container (get-actives container)))
       (#\D  (om-invalidate-view self t))
       (otherwise (om-beep)))))


(defmethod om-view-cursor ((self classTreePanel))
   *om-arrow-cursor*)

