;OpenMusic
;
;Copyright (C) 1997, 1998, 1999, 2000 by IRCAM-Centre Georges Pompidou, Paris, France.
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
;
;Authors: Gerard Assayag and Augusto Agon


;DocFile
;Editor for metaobjects without connections i.e. folders generic functions, workspaces, etc.
;Last Modifications :
;18/10/97 first date.
;DocFile
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :om)


;--------------------------
;EDITOR
;----------------------


(omg-defclass nonrelationEditor (metaobj-Editor)  
   ((presentation :initform 0 :initarg :presentation :accessor presentation))
   (:documentation "xx"))

(defmethod get-editor-panel-class ((self nonrelationEditor))  'nonrelationPanel)


;---------------------------

;;;(defvar *mouse-help-position* 0)
;In order to show help ballons
;;; --> api


(defmethod big-icon-p ((self nonrelationEditor))
   (= (presentation self) 0))

(defmethod set-clipboard ((self nonrelationEditor) copies)
   (let* ((container (panel self)))
     (setf (scroll-scrap container) `(list ,@copies))))


(defmethod get-clipboard ((self nonrelationEditor))
   (let* ((container (panel self)))
     (scroll-scrap container))) 

(defmethod editor-copy ((self nonrelationEditor))
   "Copy in the clipboard the selected icons in 'self'."
   (let* ((container (panel self))
          (subframes (get-actives container))
          (cannotcopy nil) copies)
     (if (text-view self)
       (copy-command (text-view self))
       (progn
         (loop for item in subframes
               while (not cannotcopy) do
               (setf cannotcopy (no-allow-copy-p (object item))))
         (if cannotcopy
           (not (dialog-message (string+ "Sorry i can't  copy " cannotcopy)))
           (om-with-cursor *om-wait-cursor* 
             (setf copies (mapcar #'(lambda (frame)
                                      (omNG-copy (object frame))) subframes))
             (set-clipboard self copies))))) t))


(defmethod om-paste ((self t) elem)
  (omNG-add-element self elem))

(defmethod editor-paste ((self nonrelationEditor))
   "Paste from the clipboard to 'self'."
   (let* ((container (panel self)))
     (if (text-view self)
       (paste-command (text-view self))
       (when (get-clipboard self)
         (let ((new-frames (eval (get-clipboard self)))
               (bx 10) (i -1))
           (loop for item in (get-actives container) do
                 (omG-unselect item))
           (mapcar #'(lambda (object)
                       (setf (name object) (mk-unique-name container (name object)))
                       (let ((new-frame (make-icon-from-object  object (+ 5 bx (* 50 (incf i))) 10 1 1)))
                         (unless (big-icon-p self)
                           (let* ((frames (get-subframes container))
                                  (newcol (findcol container frames new-frame)))
                             (setf (col new-frame) newcol)))
                         (add-icon-finder new-frame container)
                         (om-paste (object container) object)
                         (omG-select new-frame))) new-frames)))) t))


;save the size and the position of the editor.
(defmethod close-editor-before ((self nonrelationEditor))
  (call-next-method)
  (set-win-size (object (panel self)) (om-interior-size (window self)))
  (set-win-position (object (panel self)) (om-view-position (window self))))

(defmethod alias-editor ((self nonrelationEditor))
   "Make aliases of the selected icons."  nil)

(defmethod editor-change-presentation ((self nonrelationEditor) presentation)
   "Show icons in 'self' by name by type or normal."
   (unless (= presentation (presentation self))
     (change-frame-presentation (panel self) presentation) t))

(defmethod editor-make-new-icon-window ((self nonrelationEditor) &optional type)
   "Called from New menu item type say if it create a patch a maquettte or a folder."
   (let ((scrollframe (panel self))
         new-object new-frame)
     (cond 
      ((equal type 'p) (setf new-object (omNG-make-new-patch (mk-unique-name scrollframe "Patch") (om-make-point 10 10))))
      ((equal type 'm) (setf new-object (omNG-make-new-maquette (mk-unique-name scrollframe "Maquette") (om-make-point 10 10))))
      ((equal type 'f) (setf new-object (omNG-make-new-folder (mk-unique-name scrollframe "NewFolder") (om-make-point 10 10))))
      (t (setf new-object (omNG-make-new-persistant type scrollframe (om-make-point 10 10)))))
     (when new-object
       (setf new-frame (make-icon-from-object new-object 10 10 1 1))
       (omg-add-element scrollframe new-frame)
       ;;; new
       (set-field-size scrollframe)
       )))

;;; objets persistants supplementaires...
(defmethod omNG-make-new-persistant ((type t) frame pos) nil)


(defmethod editor-cut ((self nonrelationEditor))
   "Cut the selected icons in 'self'."
   (let* ((container (panel self))
          (subframes (get-actives container)))
     (if (text-view self)
       (cut (text-view self))
       (when subframes
         (editor-copy self)
         (dolist (frame subframes)
           (omg-remove-element container frame))
         (setf *ready-question-p* nil)))))

;---------- Copy and paste

(defmethod editor-clear ((self nonrelationEditor))
   "Clear the selected icons in 'self'."
   (let* ((container (panel self))
          (subframes (get-actives container)))
     (when subframes
       (dolist (frame subframes)
         (omg-remove-element container frame))
       (setf *ready-question-p* nil))))

(defmethod editor-save ((self nonrelationEditor))
   "Save the object associated to the selected icons in 'self'."
   (let* ((container (panel self))
          (subframes (get-actives container)))
     (om-with-cursor *om-wait-cursor*
       (loop for item in subframes do
             (omNG-save (object item) nil)))))

(defmethod editor-select-all ((self nonrelationEditor))
   "Select all icons in 'self'."
   (let* ((subframes (get-subframes (panel self))))
     (mapc #'(lambda (icon)
               (omG-select icon)) subframes)))

;-------------------------------------
;PANEL
;-------------------------------------

(omg-defclass nonrelationPanel (metaobj-Panel)
   ((scroll-scrap :initform nil :allocation :class :accessor scroll-scrap))
   (:documentation "Editorframes of meta objects where there are not connections beetwen their elements 
are instances of this class. (i.e. Folders, WorkSpaces, Classes, Generic functions, etc.) .#enddoc#
#seealso# (classpanel folderpannel instancepanel workspacepanel genericfunpanel) #seealso#
#scroll-scrap# This slot store, if exists, an expression used for copy and paste propose.#scroll-scrap#"))

;-------------------Protocole------------------------------
(defgeneric omg-remove-element (editor frame)  
   (:documentation "Remove graphicly the subview 'frame' from the scroller 'editor' and call omng-remove-element
with the objects respectly associeted."))

(defgeneric omG-add-element (editor frame)  
   (:documentation "Add graphicly the subview 'frame' to the scroller 'editor' and call omNG-add-element
with the objects respectly associeted."))

;-------------------------
(defmethod omg-remove-element ((self nonrelationPanel) frame)
   "Remove 'frame' from 'self', this function call the 'omng-remove-element' function."
   (let ((object-container (object self)))
     (if (protected-p (object frame))
       (dialog-message "This object is protected, you can not erase it")
       (progn
         (when (and (not (big-icon-p (editor self)))
                    (> (fil frame) 1))
          (setf object-container (object (get-parent frame self))))
         (when (EditorFrame (object frame))
           (om-close-window (window (EditorFrame (object frame)))))
         (omng-remove-element object-container (object frame))
          (delete-icon-finder frame self)
          ))))
  
(defmethod omG-add-element ((self nonrelationPanel) (frame OMFrame))
   "Add 'frame' to 'self'."
   (unless (big-icon-p (editor self))
     (let* ((frames (get-subframes self))
            (newcol (findcol self frames frame)))
       (setf (col frame) newcol)))
   (omNG-add-element (object self) (object frame))
   (add-icon-finder frame self))

(defmethod omG-add-element ((self nonrelationPanel) frame)
   "See generic function's documentation."
   (om-add-subviews self frame)
   ;;; new
   (om-invalidate-view self t)
   (set-field-size self)
   ;;;
)


;-----------------------------------
;Other methods
;-----------------------------------

(defmethod allow-move-element ((self nonrelationPanel))
   "You can move icons in 'self' if it is in normal icon mode."
   (big-icon-p (editor self)))

(defmethod get-subframes ((self nonrelationPanel))
   "Return a list with the icons (simpleframes) subviews of 'self'."
   (let* (rep)
     (mapc #'(lambda (icon)
               (if (icon-finder-p icon)
                 (push icon rep))) (om-subviews self))
     (sort rep #'(lambda(x y) (or (< (col x) (col y))
                                                       (and (= (col x) (col y)) (> (fil x) (fil y))))))))

(defmethod get-actives ((self nonrelationPanel))
   "Return a list with the selected icons (simpleframes) subviews of 'self'."
   (let* (rep)
     (mapc #'(lambda (icon)
               (if (and (icon-finder-p icon)
                        (active-mode icon)) 
                 (push icon rep))) (om-subviews self))
     (reverse rep)))

;SORT by type or by name
(defmethod sort-subframes ((self nonrelationPanel) elements)
   (case (presentation (om-view-container self))
     (0 elements)
     (1  (sort elements #'string-lessp :key #'name))
     (2  (sort elements #'string-lessp :key #'get-object-insp-name))))

;------------------------------------------------------------------
;The next methods are used to show icons by name or by type

;Called when you change the presentation (i.e. by name, by icon, by type).
(defmethod change-frame-presentation ((self nonrelationPanel) presentation)
   (let* ((icons (get-subframes self)))
     (unless (= (presentation (om-view-container self)) 0)
       (mapc #'(lambda (icon)
                 (when (> (fil icon) 1)
                   (delete-icon-finder icon self))
                 (remove-triangle icon self)) icons))
     (setf (presentation (om-view-container self)) presentation)
     (case presentation
       (0 (mapc #'(lambda (icon)
                    (set-size icon self)) (get-subframes self)))
       (otherwise (let ((i 1))
                    (mapc #'(lambda (icon)
                              (setf (col icon) i)
                              (setf (fil icon) 1)
                              (set-size icon self)
                              (incf i)) (sort-subframes self (get-subframes self))))
                  (set-triangles self)))))


;add triangles views to open or close folders.
(defmethod set-triangles ((self nonrelationPanel))
   (let* ((icons (get-subframes self)))
     (mapc #'(lambda (icon)
               (when (container-p icon)
                 (let ((triangle (om-make-view 'triangle-icon
                                   :position (om-make-point 5 (y icon))
                                   :size (om-make-point 11 11)
                                   :icon-finder icon
                                   :iconID 164)))
                   (setf (open? triangle) nil)
                   (om-add-subviews self triangle)
                   (setf (triangle icon) triangle)))
               (add-extras-infos icon)) icons)))

;if the view is in list mode this method find the  horizontal position 
;of the new element in the view
(defmethod findcol ((self nonRelationPanel) list new)
   (if list
     (let ((i (col (car list))))
       (loop while list do
             (let ((frame (pop list)))
               (when (= (fil frame) (fil new))
                 (case (presentation (om-view-container self))
                   (1 (if (string-lessp  (string (name new)) (string (name frame)))
                        (progn
                          (setf list nil)
                          (setf i (- (col frame) 1)))))
                   (2 (if (string-not-greaterp (get-object-insp-name new) (get-object-insp-name frame))
                        (progn
                          (setf list nil)
                          (setf i (- (col frame) 1)))))))
               (incf i)))
       i) 1))

;update the position of the icons in self
(defmethod up-from ((self nonRelationPanel) pos)
   (let* ((controls (get-subframes self))
          (final-list (subseq controls  pos)))
     (mapc #'(lambda (icon)
               (change-icon-position icon (fil icon) (- (col icon) 1))) final-list)))



;------------------------------------------
;creating Windows and scrollers 
;------------------------------------------

(defun open-new-nonrelationFrame (object name elements )
   (let* ((i 0) newwindow)
     (setf newwindow (make-editor-window (get-editor-class object)
                                         object name nil 
                                         :winsize (get-win-size object)
                                         :winpos (get-win-position object)))
     (mapc #'(lambda (elem)
               (add-icon-finder (make-icon-from-object elem  (om-point-h (get-icon-pos elem))  (om-point-v (get-icon-pos elem))   1 (+ i 1))
                                (panel newwindow))
                       (incf i)) (sort-subframes (panel newwindow) elements))
     (set-field-size (panel newwindow))
     newwindow))


(defmethod add-more-information ((self nonRelationPanel)) nil)

  



;************************************************************************
;Specific window and scrollers
;************************************************************************




;-----------------
;WorkSpace's Editor
;-----------------

;---------------------------------------------

(omg-defclass workSpaceEditor (nonRelationEditor)  ()
   (:documentation "This is the class of the workspace's window.#enddoc#
#seealso# (OmWorkspace WorkSpaceScroller) #seealso#"))

(defmethod get-editor-panel-class ((self workSpaceEditor))  'workSpacePanel)

(defmethod editor-save ((self workSpaceEditor)) 
  (show-message-win "Saving workspace...")
  (setf *save-apply-all* nil)
  (omG-save-etat)
  (setf *save-apply-all* nil)
  (hide-message-win))

(defmethod presentation ((self workSpaceEditor))
  (presentation (object self)))

(defmethod (setf presentation) ((val integer) (self workSpaceEditor))
  (set-presentation (object self) val))


;--------------------------
;PANEL
;--------------------------

(omg-defclass workSpacePanel (nonRelationPanel) () 
   (:documentation "The editorframe of the WorkSpace is an instance of this class.
Workspace Panels contain icons of patches, maquettes and folders
(patch-icon-frame, maquette-icon-frame and folder-icon-frame instances).#enddoc#
#seealso# (OmWorkspace patch-icon-frame maquette-icon-frame folder-icon-frame) #seealso#"))
 

;SAVE METHOD FrOM MENU
(defun omG-save-etat ()
   "Save all objects in the WorkSpace."
   (om-with-cursor *om-wait-cursor*
                                  (SaveWorkSpace)
                                  ;(save-all-persistants *current-workSpace*)
                                  (omNG-save-ws *current-workSpace*)))

(defmethod set-panel-color ((self workSpacePanel))
  (om-set-bg-color self *ws-color*))


;-----------------
;Folder's Editor
;-----------------


(omg-defclass folderEditor (nonRelationEditor)  ()
   (:documentation "This is the class of the folder's window.#enddoc#
#seealso# (Omfolder FolderPanel) #seealso#"))

(defmethod get-editor-panel-class ((self folderEditor))  'folderPanel)

(defmethod editor-save ((self folderEditor)) 
  (show-message-win "Saving folder...")
  (setf *save-apply-all* nil)
  (omNG-save-ws (object self))
  (setf *save-apply-all* nil)
  (hide-message-win))


;--------------------------
;PANEL
;--------------------------

(omg-defclass folderPanel (nonRelationPanel) ()
   (:documentation "This is the class for folder'editors.
Elements in these editors are patch-icon-frame maquette-icon-frame or folder-icon-frame instances.#enddoc#
#seealso# (OMFolder patch-icon-frame maquette-icon-frame folder-icon-frame) #seealso#"))



(defmethod set-panel-color ((self folderPanel))
  (om-set-bg-color self (om-make-color 0.9 0.9 0.9)))

;-----------------
;Global Folder's Editor
;-----------------


(omg-defclass GlobalsfolderEditor (nonRelationEditor)  ()
   (:documentation "This is the class of the Globalsfolder's window.#enddoc#
#seealso# (OmGlobalsfolder GlobalsFolderPanel) #seealso#"))

(defmethod get-editor-panel-class ((self  GlobalsfolderEditor))  'GlobalsfolderPanel)

(defmethod set-clipboard ((self  GlobalsfolderEditor) copies)
   (let* ((container (panel self)))
     (setf (scroll-scrap-glo container) `(list ,@copies))))

(defmethod get-clipboard ((self  GlobalsfolderEditor))
   (let* ((container (panel self)))
     (scroll-scrap-glo container)))

(defmethod get-win-size ((self OMGlobalsFolder))
   (om-make-point 300 200))

;--------------------------
;PANEL
;--------------------------

(omg-defclass GlobalsfolderPanel (nonRelationPanel)
   ((scroll-scrap-glo :initform nil :allocation :class :accessor scroll-scrap-glo))
   (:documentation "This is the class for OMGlobalsFolder'editors. 
Elements in these editors are instance-icon-frame instances.#enddoc#
#seealso# (OMglobalsFolder instance-icon-frame) #seealso#
#scroll-scrap-glo# Used to distinguish for the OMfolder scrap. #scroll-scrap-glo#"))

(defmethod omG-new-var ((self GlobalsfolderPanel))
   "Make a new global instance of the class store."
   (when (find-class 'store nil)
     (let* ((name (mk-unique-name self "variable"))  
            (obj (omNG-make-new-instance (make-instance 'store)  name (om-make-point 22 22)))
            new-frame)
       (setf new-frame (make-icon-from-object  obj 22 22 1 1))
       (omg-add-element self new-frame)
       (omng-save obj) t))) 

;-----------------
;Class's Editor
;-----------------


(defmethod get-win-size ((self OMClass))
   (om-make-point 400 200))
;---------------------------------------------

(omg-defclass classEditor (nonRelationEditor)  
   ((initform-editor :initform nil :accessor initform-editor))
   (:documentation "xx")
   (:default-initargs  :presentation 1))

(defmethod get-editor-panel-class ((self classEditor))  'classPanel)

(defmethod get-clipboard ((self classEditor)) nil)

(defmethod editor-make-new-icon-window ((self classEditor) &optional type)
   "Add a slot of type 'type' to the class associeted to the self's scroller and redefine it."
   (declare (ignore type))
   (if (protected-p (object (panel self)))
     (dialog-message "This object is protected, you can not add slots")
     (let* ((scrollframe (panel self))
            new-object new-frame)
       (setf new-object (omNG-make-new-slot  (class-name (object scrollframe))   
                                             (mk-unique-name  scrollframe "slot") 't ':instance t t ))
       (setf new-frame (make-icon-from-object  new-object 10 10 1 1))
       (unless (big-icon-p self)
         (let* ((frames (get-subframes scrollframe))
                (newcol (+ 1 (length frames))))
           (setf (col new-frame) newcol)))
       (add-icon-finder new-frame scrollframe)
       (omNG-add-element (object scrollframe) new-object))))


(defmethod close-editor-before ((self classEditor))  
   (call-next-method)
   (if (initform-editor self)
     (om-close-window (initform-editor self))))


;---------------------------
;PANEL
;---------------------------

(omg-defclass classPanel (nonRelationPanel) ()
   (:documentation "This is the editor class for a OMClass. Elements of these editors are slots simple frames.#enddoc#
#seealso# (OMClass slot-icon-frame) #seealso# "))


(defmethod omg-remove-element ((self classPanel) frame)
   "When you remove one slot from the editor it is necessary to redefine the class."
   (let ((object-container (object self)))
     (if (protected-p (object frame))
       (dialog-message "This object is protected, I can not erase it")
       (progn
         (when (and (not (big-icon-p (editor self)))
                    (> (fil frame) 1))
           (setf object-container (object (get-parent frame self))))
         (when (EditorFrame (object frame))
           (om-close-window (om-view-container (EditorFrame (object frame)))))
         (omng-remove-element  object-container (object frame))
         (delete-icon-finder frame self)))))


;Classes are showed allways by list.

(defmethod sort-subframes ((self classPanel) elements) elements)
(defmethod change-frame-presentation ((self classPanel) presentation)
   (declare (ignore presentation)) nil)

(defmethod re-sort-slots ((self classPanel))
   (let ((i 0)) 
     (loop for item in (om-subviews self) do
           (om-remove-subviews self item))
     (mapc #'(lambda (elem)
               (add-icon-finder (make-icon-from-object elem  (om-point-h (get-icon-pos elem))  (om-point-v (get-icon-pos elem))   1 (+ i 1))
                                self)
               (incf i)) (sort-subframes self (get-elements (object self))))
     (add-titles self)
     ))

(defmethod add-titles ((self classPanel))
  (om-add-subviews self 
                   (om-make-dialog-item 'om-static-text (om-make-point 30 3) (om-make-point 50 12) "Slots"  :font (om-make-font "Chicago" 12 :style '(:bold)) )                   
                   (om-make-dialog-item 'om-static-text (om-make-point 142 3) (om-make-point 50 12) "Show"  :font (om-make-font "Chicago" 12 :style '(:bold)))
                   (om-make-dialog-item 'om-static-text (om-make-point 195 3) (om-make-point 70 12) "Allocation"  :font (om-make-font "Chicago" 12 :style '(:bold)))
                   (om-make-dialog-item 'om-static-text (om-make-point 280 3) (om-make-point 100 12) "Default value"  :font (om-make-font "Chicago" 12 :style '(:bold)))))




(defmethod om-view-cursor ((self classPanel))
   *om-arrow-cursor*)



;-----------------
;Generic function's Editor
;-----------------



(omg-defclass GenericFunEditor (nonRelationEditor)  ()
   (:documentation "This is the class of the GenericFun's window.#enddoc#
#seealso# (OmGenericFun GenericFunPanel) #seealso#"))

(defmethod get-editor-panel-class ((self GenericFunEditor))  'GenericFunPanel)

(defmethod editor-make-new-icon-window ((self GenericFunEditor) &optional type)
   "Open an editor to const a new method for the genric function 'self'."
   (declare (ignore type))
   (let ((scrollframe (panel self)))
     (make-new-method (object scrollframe))))

(defmethod get-clipboard ((self GenericFunEditor)) nil)

(defmethod editor-change-presentation ((self GenericFunEditor) presentation)
   (declare (ignore presentation)) nil)


;---------------------------
;PANEL
;---------------------------
(omg-defclass GenericFunPanel (nonRelationPanel) ()
   (:documentation "This is the class for editor of Generic Functions meta objects.
 Elements of these editors are icon-method instances.#enddoc#
#seealso# (OMgenericFunction icon-method) #seealso#"))

(defmethod allow-move-element ((self GenericFunPanel))
   "You can not move the method'icons in a generic function editor." nil)

(defmethod get-subframes ((self GenericFunPanel))
   "Get a list of method's icons (instances of the icon-method class)." 
   (let* (rep)
     (mapc #'(lambda (icon)
               (if (icon-method-p icon)
                 (push icon rep))) (om-subviews self))
     rep))

(defmethod get-actives ((self GenericFunPanel))
   "Get a list of selected method's icons (instances of the icon-method class)."
   (declare (ignore scroll))
   (let* (rep)
     (mapc #'(lambda (icon)
               (if (and (icon-method-p icon)
                        (active-mode icon)) 
                 (push icon rep))) (om-subviews self))
     (reverse rep)))



;----------------
;Init methods and read/write slots methods for an OMCLASS
;----------------



(omg-defclass InternalMethEditor (GenericFunEditor)  ()
   (:documentation "This is the class of the InternalMeth's window.#enddoc#
#seealso# (OmInternalMeth InternalMethScroller) #seealso#"))

(defmethod get-editor-panel-class ((self InternalMethEditor))  'InternalMethPanel)


;---------------
;PANEL
;---------------
(omg-defclass InternalMethPanel (GenericFunPanel) ()
   (:documentation "This is the class for the editor of init methods and read/write slots methods associated to an OMClass.#enddoc#
#seealso# (OMClass) #seealso#"))


;;;;-----------------------------------
;;;; events
;;;;-----------------------------------

(defmethod om-view-click-handler ((view nonrelationPanel) where)
  (let ((clicked (call-next-method)))
    (when (eq clicked view) 
      (do-click-event-handler view where))
    clicked))

(defmethod do-click-event-handler ((view nonrelationPanel) where)
   (unless (om-shift-key-p) 
     (mapc #'(lambda (control) 
               (omG-unselect control)) (get-actives view)))
   (if (om-control-key-p)
     (menu-item-context view where)
     (control-actives view where)))


;Click on the scroller not in a subview
(defmethod control-actives ((view nonrelationPanel) where)
  (when (text-view (editor view))
    (exit-from-dialog (text-view (editor view)) (om-dialog-item-text (text-view (editor view)))))
  (let (user-rect scratch-rect-i scratch-rect-n i-rect n-rect)
    (setf user-rect (om-pts-to-rect where (grow-gray-rect where (om-make-point 0 0) view nil)))
    (dolist (item (get-subframes view))
      (let ((posIconx (om-add-points (om-view-position item) (om-view-position (iconView item))))
            (posNamex (om-add-points (om-view-position item) (om-view-position (nameView item)))))
        (setf i-rect (om-pts-to-rect posIconx (om-add-points posIconx (om-view-size (iconView item)))))
        (setf scratch-rect-i (om-sect-rect user-rect i-rect))
        (setf n-rect (om-pts-to-rect posNamex (om-add-points posNamex (om-view-size (nameView item)))))
        (setf scratch-rect-n (om-sect-rect user-rect n-rect))
        (unless (and (om-rect-empty scratch-rect-i) (om-rect-empty scratch-rect-n))
          (omG-select item))))))


(defmethod om-view-cursor ((self GenericFunPanel))
   *om-arrow-cursor*
   )

(defmethod om-view-cursor ((self nonrelationPanel))
   (if (om-control-key-p) *om-contex-cursor* *om-arrow-cursor*)
)

(defmethod om-view-cursor ((self GenericFunPanel))
   *om-arrow-cursor*)



