;OpenMusic
;
;Copyright (C) 1997, 1998, 1999, 2000 by IRCAM-Centre Georges Pompidou, Paris, France.
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
;
;Authors: Gerard Assayag and Augusto Agon


;DocFile
;Classes for box's inputs are defined in this file.
;Last Modifications :
;18/10/97 first date.
;DocFile



(in-package :om)

;Non graphic class, this class could be an OMBasicObject
;But i find that is simpler that
(defclass input-funbox () 
   ((doc-string :initform "" :initarg :doc-string  :accessor doc-string)
    (value :initform nil :initarg :value :accessor value)
    (name :initform nil :initarg :name :accessor name)
    (connected? :initform nil :accessor connected?)))


;-------Value and code generation
(defmethod get-icon-input ((self input-funbox)) 185)

(defmethod omNG-box-value ((self input-funbox) &optional numout)
   (declare (ignore numout))
   (if (connected? self)
     (omNG-box-value (first (connected? self)) (second (connected? self)))
     (value self)))
   
(defmethod gen-code ((self input-funbox) numout)
  (declare (ignore numout))
  (let ((theinput (connected? self))
        (value (value self)))
    (if theinput
      (gen-code (first theinput) (second theinput))
      (gen-code value 0))))

(defmethod popup-input-menu ((self input-funbox) container)
   (let ((menulist (thepopup self))
           menu itemlist default)
     (loop for item in menulist 
            for i = 0 then (+ i 1) do
               (let ((title (car item))
                       (arg (second item)))
                 (if (equal (eval arg) (value self)) (setf default i))
                   (push (om-new-leafmenu title #'(lambda ()
                                                    (setf (value self) (eval arg))))
                               itemlist)))
     (setf menu (om-create-menu 'pair-pop-up-menu (reverse itemlist)))
     (when default (om-set-menu-default-item menu default))
     (om-open-pop-up-menu menu container)
     ))

   
;-----------------
;FRAMES
;-----------------

;This is the little bubble icon for inputs
;this class could be a simpleframe but i find that is sampler that
(omg-defclass input-funboxframe (icon-view) 
  ((object :initform nil :initarg :object :accessor object)))


;---------Inits
(defmethod input? ((self input-funboxframe)) t)
(defmethod input? ((self t)) nil)

(defmethod initialize-instance :after ((self input-funboxframe) &key controls)
  (declare (ignore controls))
  (setf (iconID self) (frame-icon-input self))
  (get&corrige-icon (iconID self)))

(defmethod frame-icon-input ((self input-funboxframe))
  (get-icon-input (object self)))

;-------Events
;Show only

;;; pour montrer ou pas les valeurs des inputs
(defvar *show-input-vals* t)

;;; new : textenterview container = panel 
(defmethod om-view-mouse-enter-handler ((self input-funboxframe))
   (unless (or (and (connected? (object self)) (not (keyword-input-p (object self)))) (not *show-input-vals*)) 
     (without-interrupts
      (when (and self (om-view-container self))
        (let* ((thetext (format () "~S" (value (object self))))
              (panel (om-view-container (om-view-container self)))
              (container (editor panel)))
          (when (text-view container)
            (exit-from-dialog (text-view container) (om-dialog-item-text (text-view container))))
          (setf (text-view container)
             (om-make-dialog-item 'text-enter-view
                                                       (om-add-points (om-view-position (om-view-container self)) (om-make-point (x self) -15))
                                                       (om-make-point (get-name-size thetext) 12)
                                                       thetext
                                                       :container panel
                                                       :font *om-default-font2*)))))))

;;;new : text-view is on the panel
(defmethod om-view-mouse-leave-handler ((self input-funboxframe))
   (without-interrupts 
    (let* ((container (editor (om-view-container (om-view-container self)))))
      (when (and container (text-view container) (equal (class-name (class-of (text-view container))) 'text-enter-view))
        (om-remove-subviews (panel container) (text-view container))
        (setf (text-view container) nil))))
   )


;;;new : text-view is on the panel
(defmethod om-view-click-handler ((self input-funboxframe) where)
  (declare (ignore where))
  (let* ((panel (om-view-container (om-view-container self)))
         (container (editor panel)))
     
     (cond
      ((om-command-key-p) (om-show-tooltip self 1))
      ((om-option-key-p) (when (connected? (object self))
                        (disconnect-box (om-view-container self) self)))
      
      ((menu-input-p (object self))
       (popup-input-menu (object self) panel)
       )
      ((and (om-shift-key-p) (not (maquette-p (object container)))) 
            (let* ((new-obj (omNG-make-new-boxcall (car *Basic-Lisp-Types*)
                                                   (om-make-point (+ (x (om-view-container self)) (x self))
                                                                  (- (y (om-view-container self)) 30))
                                                   (mk-unique-name panel "aux")))
                   new-frame)
              (setf (value new-obj) (value (object self)))
              (setf (thestring new-obj) (format () "~S" (value new-obj)))
              (setf (frame-size new-obj) (om-make-point (get-name-size (thestring new-obj)) 22))
              (setf new-frame (make-frame-from-callobj new-obj))
              (omG-add-element (om-view-container (om-view-container self)) new-frame)
              (connect-box (car (outframes new-frame)) self)
              (open-ttybox (iconview new-frame))))
       (t (unless (and (connected? (object self)) (not (keyword-input-p (object self))))
            (when (text-view container)
              (om-remove-subviews panel (text-view container))
              (setf (text-view container) nil))
            (let ((thetext (format () "~S" (value (object self)))))
              (setf (text-view container) (om-make-dialog-item 'edit-text-enter-view
                                                               ;(pos-panel2editor panel 
                                                               (om-add-points (om-view-position (om-view-container self)) 
                                                                              (om-make-point (x self) -15))  
                                                               (om-get-text-edit-size (om-make-point (get-name-size thetext) 12))
                                                               thetext
                                                               :allow-returns t
                                                               :di-selected-p t
                                                               :object self
                                                               :container panel
                                                               :font *om-default-font2*))))))))
  
;-------

(omg-defclass temp-funboxframe (input-funboxframe) 
  ())

(defmethod frame-icon-input ((self temp-funboxframe))
  227)




;------------------------------------------------------
;frame for class inputs, change the icon
;------------------------------------------------------


(omg-defclass input-classframe (icon-view) 
  ((object :initform nil :initarg :object :accessor object)))

(defmethod initialize-instance :after ((self input-classframe) &key controls)
   (declare (ignore controls))
   (setf (iconID self) 184))

;--------------------------------------------------------------------
;Text showed when mouse in input (non edit)
;--------------------------------------------------------------------
(omg-defclass text-enter-view (om-static-text) ())

(defmethod om-draw-contents ((self text-enter-view))
   (call-next-method)
   (om-draw-view-outline self))

;--------------------------------------------------------------------
; Edit input value
;--------------------------------------------------------------------

(omg-defclass edit-text-enter-view (om-editable-text) 
  ((object :initform nil :initarg :object :accessor object)))


(defmethod om-view-key-handler ((self edit-text-enter-view) char)
   (case char
      (:om-key-enter (exit-from-dialog self (om-dialog-item-text self)))
      (:om-key-return (if *provisoire-flag* (call-next-method) 
                               (exit-from-dialog self (om-dialog-item-text self))))
      (otherwise (call-next-method))
      ))


(defmethod om-dialog-item-action ((self edit-text-enter-view))
   (om-set-view-size self (om-make-point (max (w self) (+ 10 (get-name-size (om-dialog-item-text self)))) (h self)))
   )


(defmethod copy ((self text-enter-view)) nil)
(defmethod paste ((self text-enter-view)) nil)

;;;new : text-view is on the panel
(defmethod exit-from-dialog ((self edit-text-enter-view) newtext)
   (handler-bind ((error #'(lambda (c) (declare (ignore c)) 
                            (setf (text-view (editor (om-view-container self))) nil)
                            (om-remove-subviews (panel (editor (om-view-container self))) self)
                            (om-beep)
                            (om-abort))))
     (let ((*package* (find-package :om))
           (fun-input (object (object self))))
       (when (keyword-input-p fun-input)
         (set-new-keyword fun-input newtext))
       (setf (value fun-input) (read-from-string newtext))
       (setf (text-view (editor (om-view-container self))) nil)
       (om-remove-subviews (panel (editor (om-view-container self))) self))))

;-------------------------------------------

(defclass input-funmenu (input-funbox) 
  ((thepopup :initform nil :initarg :thepopup :accessor thepopup)))

(defmethod menu-input-p ((self input-funmenu)) t)
(defmethod menu-input-p ((self t)) nil) 

(omg-defclass in-pop-up-menu (om-pop-up-menu)
  ((in-value :initform 0 :initarg :in-value :accessor in-value)
   (items :initform nil :initarg :items :accessor items)
   (originallist :initform nil :initarg :originallist :accessor originallist)))

(defun change-val (pu val)
  (setf (in-value pu) (eval val)))


;-------------------------------------------

(defclass input-keyword (input-funbox) 
   ((box-ref :initform nil :initarg :box-ref :accessor box-ref)))

(defmethod get-icon-input ((self input-keyword)) 227)

(defmethod keyword-input-p ((self input-keyword)) t)
(defmethod keyword-input-p ((self t)) nil) 

(defmethod set-new-keyword ((self input-keyword) newtext)
  (let ((keywords (get-keywords-from-box (box-ref self))) pos)
       (unless (member (symbol-name (read-from-string newtext)) keywords :test 'equal :key 'symbol-name)
         (format () "~D is not a good keyword. Allowed keywords are : ~S"  newtext keywords)
         (error "error"))
       (setf pos (position (read-from-string newtext) (inputs (box-ref self)) :test 'equal :key 'value))
       (when pos
         (format () "~D is already used by the ~D input in the box." newtext (incf pos))
         (error "error"))))

(defmethod eval-keyword-in ((self input-keyword))
   (declare (ignore numout))
   (list (value self) 
         (if (connected? self)
           (omNG-box-value (first (connected? self)) (second (connected? self)))
           (eval-not-connected self))))
   
(defmethod gen-code-keyword-in ((self input-keyword))
   (declare (ignore numout))
   (let ((theinput (connected? self))
         (value (value self)))
     (list value
           (if theinput
             (gen-code (first theinput) (second theinput))
             (code-not-connected self)))))

(defmethod eval-not-connected ((self input-keyword))
   (om-beep-msg (format nil "WARNING keyword ~D is not connected, NIL is used as default value." (string (value self)))))
   
(defmethod code-not-connected ((self input-keyword))
   (om-beep-msg (format nil "WARNING keyword ~D is not connected, NIL is used as default value." (string (value self)))))


;==============================
;Subclass this class to edit static-text

(omg-defclass click-and-edit-text (om-static-text) 
   ((mini-editor :initform nil :accessor mini-editor)))

;------------Events
(defmethod om-view-click-handler ((self click-and-edit-text) where)
   (declare (ignore where))
   (open-mini-editor self))

(defmethod do-exit-function ((self click-and-edit-text) container newtext)
   (when newtext
     (om-set-dialog-item-text self newtext)
     (om-add-subviews container self)))

;------------Edition
(defmethod open-mini-editor ((self click-and-edit-text))
   (let ((container (om-view-container self)))
     (setf (mini-editor self) (om-make-dialog-item 'edit-click-and-edit 
                                                                                       (om-view-position self) (om-view-size self)
                                                                                       (om-dialog-item-text self)
                                                                                       
                                                                                       :allow-returns t
                                                                                       :object self
                                                                                       :container container
                                                                                       :font *om-default-font2*))
     (om-remove-subviews container self)))

(omg-defclass edit-click-and-edit (edit-text-enter-view) 
   ((object :initform nil :initarg :object :accessor object)))

;;;new : text-view is on the panel
(defmethod exit-from-dialog ((self edit-click-and-edit) newtext)
   (handler-bind ((error #'(lambda (c) (declare (ignore c)) 
                            (setf (text-view (editor (om-view-container self))) nil)
                            (om-remove-subviews (panel (editor (om-view-container self))) self)
                            (om-beep)
                            (om-abort))))
     (let ((*package* (find-package :om))
           (original (object self)))
       (do-exit-function original container newtext) 
       (setf (mini-editor original) nil)
       (om-remove-subviews (om-view-container self) self))))
 



 




