;OpenMusic
;
;Copyright (C) 1997, 1998, 1999, 2000 by IRCAM-Centre Georges Pompidou, Paris, France.
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
;
;Authors: Gerard Assayag and Augusto Agon


;DocFile
;This file defines a simple frame class for temporal boxes in a maquette.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Last Modifications :
;18/10/97 first date.
;DocFile

(in-package :om)

(defvar *shad-white*)
(defvar *shad-black*)
(defvar *shad-gray*)

(setf *shad-white* *om-white-color*)  
(setf *shad-black* *om-black-color*)  
(setf *shad-gray* *om-gray-color*)

(defmethod draw-frame-shadows ((self om-view) )
   (let ((y1 (h self)) 
         (x1 (w self)))
    (om-with-fg-color self *shad-gray*
           (om-draw-line 1 (- y1 2) (- x1 2) (- y1 2))
           (om-draw-line (- x1 2) (- y1 2) (- x1 2) 0))
         (om-with-fg-color self *shad-white*
           (om-draw-line 0 0 x1 0)
           (om-draw-line 0 0 0 y1))
         (om-with-fg-color self *shad-black*
           (om-draw-line 0 (- y1 1) x1 (- y1 1))
           (om-draw-line (- x1 1) y1 (- x1 1) 0))))

;=========================================================
;TEMPORAL OBJECTS FRAME
;=========================================================
;===========================
;INPUTS
;===========================

(omg-defclass input-tempobj-frame (input-funboxframe) ())

(defmethod initialize-instance :after ((self input-tempobj-frame) &key controls)
  (declare (ignore controls))
  (setf (iconID self) 146))

;;; new :text-view container = panel
(defmethod om-view-click-handler ((self input-tempobj-frame) where)
  (declare (ignore where))
  (cond
   ;;;((om-control-key-p) (om-set-help t))
   ((om-option-key-p) (when (connected? (object self))
                        (disconnect-box (om-view-container self) self)))
   ((menu-input-p (object self))
    (om-set-view-position (thepopup (object self))
                          (om-make-point (+ (x (om-view-container self)) (x self))
                                         (- (y (om-view-container self)) 15)))
    (om-add-subviews (om-view-container (om-view-container self)) (thepopup (object self)))
    (om-view-click-handler (thepopup (object self)) (om-make-point 5 5))
    (om-remove-subviews (om-view-container (om-view-container self)) (thepopup (object self)))
    (setf (value (object self)) (in-value (thepopup (object self)))))
   (t (let* ((panel (om-view-container (om-view-container self)))
             (container (editor panel)))
        (unless (connected? (object self))
          (when (text-view container)
            (om-remove-subviews panel (text-view container))
            (setf (text-view container) nil))
          (let ((thetext (format () "~S" (value (object self)))))
            (setf (text-view container) (om-make-dialog-item 'edit-text-enter-view
                                                             ;(pos-panel2editor panel
                                                             (om-make-point (+ (x (om-view-container self)) (x self))
                                                                            (- (y (om-view-container self)) 20))
                                                             (om-get-text-edit-size (om-make-point (get-name-size thetext) 12))
                                                             thetext
                                                             
                                                             :allow-returns t
                                                             :object self
                                                             :container panel
                                                             :font *om-default-font2*))))))))


;===========================
;OUTPUTS
;===========================
(omg-defclass outtempobj (outfleche) ())

(defmethod initialize-instance :after ((self outtempobj) &key controls)
  (declare (ignore controls))
  (setf (iconID self) 401))

(defmethod om-view-click-handler ((self outtempobj) where)
   (if (show-con? (om-view-container self))
     (unless (om-shift-key-p) 
       (call-next-method))
     (click-in-temp-box (om-view-container self) (om-add-points (om-view-position self) where))))

(defmethod om-view-doubleclick-handler ((self outtempobj) where)
   (if (show-con? (om-view-container self)) (om-view-click-handler self where)
       (doubleclick-in-temp-box (om-view-container self) (om-add-points (om-view-position self) where))))

;===========================
;FRAME
;===========================
(omg-defclass tempobjframe (om-view-drag boxframe om-view-drop) 
   ((mode :initform 'normal :accessor mode)
    (resize :initform nil :accessor resize)
    (minipict :initform nil :accessor minipict))
   (:documentation "Simple frame for temporalbox boxes in maquettes. #enddoc#
#seealso# (OMBoxEditCall) #seealso#"))

(defmethod close-frame ((box tempobjframe))
  "Close all editors and kill pictures if 'box' is in pict mode."
  (when (reference (object box))
    (when (and (EditorFrame (reference (object box))) 
               (not (have-persistant-patch? (object box))))
      (om-close-window (window (EditorFrame (reference (object box)))))))
  (when (EditorFrame (object box))
    (om-close-window (window (EditorFrame (object box)))))
  (when (minipict box) 
    (om-kill-picture (minipict box))
    (setf (minipict box) nil))
  (setf (frames (object box)) nil))


(defmethod change-edit-mode ((self tempobjframe))
   "Show or hide pict for temporal boxes."
   (if (showpict (object self))
     (progn
       (setf (showpict (object self)) nil)
       (when (minipict self) 
         (om-kill-picture (minipict self))
         (setf (minipict self) nil))
       (om-invalidate-view self t))
     (progn
       (setf (showpict (object self)) t)
       (update-miniview self (soft-get-mus-ob (object self))))))

(defmethod add-box-resize ((self tempobjframe))
   (om-add-subviews self
                 (setf (resize self)
                       (om-make-view 'c-resize-box
                         :size (om-make-point 8 8)
                         :position (om-make-point (- (w self) 8) (- (h self) 8))))))
   
;-------DRAW ------

(defmethod get-impulsion-pict ((self t)) *impulsion-pict*)

(defmethod om-draw-contents ((self tempobjframe))
   (if (zerop (extend (object self)))
     (om-draw-picture self (get-impulsion-pict (car (value (object self)))) (om-make-point 0 0) (om-make-point (w self) (h self)))
     (if (showpict (object self))
       (progn 
         ;(om-with-focused-view self (om-with-fg-color self *om-white-color* (om-fill-rect 0 0 (w self) (h self))))
         (draw-editor-mode (get-obj-to-draw (object self)) self))
       (let* ((durinx (norme2pixel (om-view-container self) 'x (extend (object self)))))
         (om-with-focused-view self
           (if (has-picture-p self)
             (progn
               (om-draw-picture self (thepict (pictu (object self))) (om-make-point 0 0) (om-make-point (w self) (h self)))
               (draw-carre self t))
             (progn
               (om-with-fg-color self (colorframe (object self))
                 (draw-frame self (object self) 0 0  durinx (h self)))
               (if (active-mode self)
                 (draw-carre self t)
                 (draw-frame-shadows self))))
           (om-draw-string 4 (- (h self) 5) (name (object self)))
           ))))
   (when (mute (object self)) 
     (let ((iconparams (get&corrige-icon 341))
           (size (om-make-point 16 16)))       ;;(om-make-point (round (w self) 4) (round (h self) 4))))
       (om-draw-icon (second iconparams) self (om-make-point 
                                               (- (round (w self) 2) (round (om-point-h size) 2)) 
                                               (- (round (h self) 2) (om-point-v size)))
                                               size)
                     ))
   (when (lock (object self)) 
     (let ((iconparams (get&corrige-icon 340))
           (size (om-make-point 16 16)))       ;;(om-make-point (round (w self) 4) (round (h self) 4))))
       (om-draw-icon (second iconparams) self (om-make-point 
                                               (- (round (w self) 2) (round (om-point-h size) 2)) 
                                               (round (h self) 2))
                                               size)
                     ))
   
   
   (if (maquette-p (reference (object self)))
     (let ((iconparams (get&corrige-icon (if (om-maquette-abs-p (reference (object self))) 265 182)))
           (size (om-make-point 10 10)))
       (om-draw-icon (second iconparams) self (om-make-point (- (w self) 16) 2) size)
       )
     (if (patch-p (reference (object self)))
       (let ((iconparams (get&corrige-icon (if (abspatch-p (reference (object self))) 210 183)))
             (size (om-make-point 10 10)))
         (om-draw-icon (second iconparams) self (om-make-point (- (w self) 16) 2) size)
         )))
   
  (when (show-con? self)
    (call-next-method)))

(defmethod om-draw-contents ((self outtempobj))
  (when (show-con? (om-view-container self))
    (call-next-method)))

(defmethod om-draw-contents ((self input-tempobj-frame))
  (when (show-con? (om-view-container self))
    (call-next-method)))


(defmethod draw-before-box ((self tempobjframe)))
(defmethod draw-after-box  ((self tempobjframe)))
(defmethod centre-icon     ((self tempobjframe)))
(defmethod has-picture-p   ((self tempobjframe))
   (thepict (pictu (object self))))

(defmethod draw-frame ((self tempobjframe) obj x0 y0 x1 y1)

   (om-fill-rect x0 y0 x1 y1)
   ;;; new : dessiner le reste du tempobj
   (om-with-pen (self :pattern *om-gray-pattern*)
                             (om-fill-rect x1 0 (- (w self) x1) (h self)))
   
   )


(defmethod box-draw-connections ((self tempobjframe) val)
   (declare (ignore val))
   (when (show-con? self) (call-next-method)))

(defmethod add-subview-extra ((self tempobjframe))
   (when (showpict (object self))
     (init-miniview self (soft-get-mus-ob (object self)))))

(defmethod move-frame-delta ((self tempobjframe) dir) 
   (if (om-option-key-p)
     (move-miniview  self dir) 
     (let (unite pixnum)
       (if (or (= dir 0) (= dir 1))
         (setf unite (max 1 (norme2pixel  (om-view-container self) 'y 1)))
         (setf unite (max 1 (norme2pixel  (om-view-container self) 'x 1))))
       (setf pixnum (* (+ unite 1) (if (om-shift-key-p) 10 1)))
       (case dir
         (0 (omGMoveObject self (om-subtract-points (om-view-position self) (om-make-point 0 pixnum))))
         (1 (omGMoveObject self (om-add-points (om-view-position self) (om-make-point 0 pixnum))))
         (2 (omGMoveObject self (om-add-points (om-view-position self) (om-make-point pixnum 0))))
         (3 (omGMoveObject self (om-subtract-points (om-view-position self) (om-make-point pixnum 0))))))))

;---------Events
(defmethod om-view-cursor ((self tempobjframe))
   (if (om-control-key-p)
       *om-contex-cursor*
     (case (mode (om-view-container self))
        (8 *om-loupe-cursor*)
        (10 *om-cross-cursor*)
        (otherwise *om-arrow-cursor*))))


(defmethod do-click-inbox ((self tempobjframe) where)
   (call-next-method)
   (cond ((= (mode (om-view-container self)) 10) (scroll-miniview self))
         ((om-control-key-p) (menu-item-context self where))))

;;; dbleclick
(defmethod click-in-temp-box ((self tempobjframe) where)
   (cond ((= (mode (om-view-container self)) 10) (scroll-miniview self))
         (t (toggle-icon-active-mode self)
            (if (om-control-key-p) (menu-item-context self where)))))

(defmethod doubleclick-in-temp-box ((self tempobjframe) where)
   (cond ((= (mode (om-view-container self)) 10) (scroll-miniview self))
         ;;;((om-command-key-p) (om-set-help t))
         (t (OpenObjectEditor (object self)))
         ))

(defmethod eval-box ((self tempobjframe)) 
   (eval+redraw self)
   (format t "OM->~S~%" (nth 0 (value (object self)))))

(defmethod reinit-size ((self tempobjframe))
  (set-mini-param self 'deltapict (om-make-point 0 0))
  (if (maquette-p (reference (object self)))
    (setf (extend (object self)) (get-obj-dur (reference (object self))))
    (when (allowed-in-maq-p (nth 0 (value (object self))))
      (setf (extend (object self)) (get-obj-dur (nth 0 (value (object self))))))
    )
  (unless (=  (strech-fact (object self)) 1)
    (let* ((container (om-view-container self)))
      (setf (slot-value (object self) 'strech-fact) 1)
      (om-set-view-size self (om-make-point  (norme2pixel container 'x  (extend (object self)))
                                             (norme2pixel container 'y  (sizey (object self)))))
      (make-move-after (om-view-container self) (list self))))
  (om-invalidate-view self t))




     

;----- STRECH ------------------


(defmethod change-boxframe-size ((view tempobjframe))
  (let* ((panel (om-view-container view))
         (mark-list (get-maquette-markers panel))
         (pos0 (om-view-position view))
         (posn (om-view-size view))
         (new-position (grow-gray-rect  pos0 posn panel nil)))
    (setf new-position  (first (denormalize-points panel 
                                                   (normalize-points panel 
                                                                     (list (om-subtract-points new-position  pos0))))))
    
    (if (is-marketed (object view) 1 mark-list)
        (om-set-view-size view (om-make-point (w view) (om-point-v new-position)))
      (when (setf new-position (allow-new-size view new-position))
        (when (> (om-point-h new-position) 0)
          (om-set-view-size view new-position)
          (if (zerop (extend (object view)))
              (setf (slot-value (object view) 'strech-fact) 1000)
            (setf (slot-value (object view) 'strech-fact)
              (/ (om-point-h new-position) (norme2pixel panel 'x (extend (object view))))))
          (draw-mark-lines panel nil)
          (update-after-mark panel)
          (draw-mark-lines panel)
          ;;; !!!
          (when (and (allow-strech-p (soft-get-mus-ob (object view)) 1) (showpict (object view)))
            (update-miniview view (soft-get-mus-ob (object view))))
          (when (patch-temp-p (reference (object view)))
            (setf (mode view) 'changed)))))
    (om-invalidate-view view t)
    (make-move-after panel (list view))))



(defmethod allow-new-size ((self tempobjframe) new-pos)
   (let (rep)
     (if (and (allow-strech-p (soft-get-mus-ob (object self)) 0) (not (om-shift-key-p)))
       (if (and (> (om-point-h new-pos) 0) (> (om-point-v new-pos) 0))
         (setf rep new-pos))
       (if (> (om-point-v new-pos) 0)
         (setf rep (om-make-point (w self) (om-point-v new-pos)))))
     rep))

(defmethod om-set-view-size :after ((self tempobjframe) size) 
   (declare (ignore size))
   (if (not (zerop (extend (object self))))
     (let ((numins (length (inputframes self)))
           (numouts (length (outframes self))))
       (when (om-view-container self)
         (setf (slot-value (object self) 'sizey) (pixel2norme (om-view-container self) 'y (h self)))) 
       (om-set-view-position (resize self) (om-make-point (- (w self) 5) (- (h self) 8)))
       (loop for input in (inputframes self)
             for i from 1 to numins do
             (om-set-view-position input (om-make-point (- (* i  (round (w self) (+ numins 1))) 6) 0)))
       (loop for output in (outframes self)
             for i from 1 to numouts do
             (om-set-view-position output (om-make-point (- (* i (round (w self) (+ numouts 1))) 6) 
                                                   (- (h self) 8)))))))


;--------------------------------
;MOVE
;--------------------------------

(defmethod OMGMoveObject ((self tempobjframe) new-position)
   (when (< (om-point-h new-position) 0)
       ;;;(om-beep-msg "Object must start after 0")
       (setf new-position (om-make-point 0 (om-point-v new-position))))
       (let* ((container (om-view-container self))
            (mark-list (get-maquette-markers container)))
       (draw-mark-lines container nil)
       (if (is-marketed (object self) 0 mark-list)
         (MoveTempObject self (om-make-point (x self) (om-point-v new-position)))
         (update-movement self new-position))
       (update-after-mark container)
       ;; ***
       ;;(draw-mark-lines container)
       ;(om-invalidate-view container)
       (omng-MoveObject (object self) new-position)))
    
(defmethod MoveTempObject ((self tempobjframe) new-position)
   (let ((maqpos))
     (mapc #'(lambda (conection)
               (draw-connection conection nil)) (connections self))
     (om-change-view-position self new-position)
     (setf maqpos (get-offset/posy-from-pixel (om-view-container self) new-position)) 
     (setf (slot-value (object self) 'offset) (om-point-h maqpos))
     (setf (slot-value (object self) 'posy)  (om-point-v maqpos))
     (om-change-view-position self (get-offset/posy-in-pixel self (om-view-container self)))))

(defmethod update-movement ((self tempobjframe) new-position)
   (let* ((container (om-view-container self))
          (mark-list (get-maquette-markers container))
          (finmarker (is-marketed (object self) 1 mark-list)))
     (if (and finmarker (not (allow-strech-p (soft-get-mus-ob (object self)) 0)))
       (om-beep-msg "Sorry i can not strech this object")
       (unless (and finmarker (>= (om-point-h new-position) (x+w self)))
         (MoveTempObject self new-position)
         (when finmarker 
           (setf (slot-value (object self) 'strech-fact)
                 (/ (- (x (car (frames finmarker))) (x self)) (norme2pixel container 'x (extend (object self)))))
           (om-set-view-size self (om-make-point (- (x (car (frames finmarker))) (x self)) (h self))))
         (when (patch-temp-p (reference (object self)))
           (setf (mode self) 'changed))
         (make-move-after container (list self)))))
   (om-invalidate-view self t))


;--------------------------------
;Size and position update
;--------------------------------

(defmethod size-offset-frame-update ((self tempobjframe) container)
   (if (zerop (extend (object self)))
     (om-set-view-size self (om-make-point 10 (norme2pixel container 'y (sizey (object self)))))
     (om-set-view-size self (om-make-point  (norme2pixel container 'x (extend (object self)))
                                      (norme2pixel container 'y (sizey (object self))))))
   (om-set-view-position self (get-offset/posy-in-pixel self container))
   (when (showpict (object self))
     (update-miniview self (soft-get-mus-ob (object self))))
   (setf (mode self) 'normal))

(defmethod init-size&position ((self tempobjframe) container)
  (let* ((obj (object self))
         (y-size (norme2pixel container 'y (sizey obj))))
    (cond
     ((zerop (extend obj))
      (om-set-view-position self (get-offset/posy-in-pixel self container))
      (om-set-view-size self (om-make-point 10 y-size))
      (let* ((mark-list (get-maquette-markers container))
             (finmarker (is-marketed (object self) 1 mark-list)))
        (when finmarker
          (del-mark (object self) finmarker 1))))
     
     (t (om-set-view-position self (get-offset/posy-in-pixel self container))
        (om-set-view-size self (om-make-point  (norme2pixel container 'x (* (extend (object self))
                                                                      (strech-fact (object self))))
                                         y-size))
        (when (and (allow-strech-p (soft-get-mus-ob (object self)) 1) (showpict (object self)))
         (update-miniview self (soft-get-mus-ob (object self))))
        ))))


(defmethod get-mini-param ((self tempobjframe) param)
   (cdr (assoc param (edition-params (object self)))))

(defmethod set-mini-param ((self tempobjframe) param val)
   (rplacd (assoc param (edition-params (object self))) val))

;------------------------LOCK
(defmethod add-lock-button ((self tempobjframe) &optional (icon 167))
   "Mode lambda 'l' and refernce 'o' are forbidden for temporal boxes;"
   (setf (lock-button self)  (om-make-view 'lock-button
                               :IconID icon
                               :size (om-make-point 10 10)
                               :position (om-make-point 0 0)
                               :owner self
                               :action #'(lambda (item)
                                           (case (iconID item)
                                             (168 (setf (iconID item) 167)
                                              (setf (allow-lock (object self)) "x"))
                                             (167 (setf (iconID item) 168)
                                              (setf (allow-lock (object self)) "&")))
                                           (om-draw-contents item))))
   (setf (allow-lock (object self)) (get-str-lock icon)))

(defmethod remove-lock-button ((self tempobjframe))
   (om-remove-subviews self (lock-button self))
   (setf (lock-button self) nil)
   (setf (allow-lock (object self)) nil))

(defmethod get-mini-param ((self tempobjframe) param)
   (cdr (assoc param (edition-params (object self)))))


(defmethod set-mini-param ((self tempobjframe) param val)
   (rplacd (assoc param (edition-params (object self))) val))
  
;--------------------------------
;EVAL
;--------------------------------

(defmethod eval+redraw ((self tempobjframe))
   (omNG-box-value (object self) 0)
   (update-after-mark (om-view-container self))
  )

(defmethod update-after-evaluation ((self tempobjframe))
   (let* ((themusobj (nth 0 (value (object self))))
          (container (om-view-container self)))
     (setf (mode self) 'normal)
     (if (allowed-in-maq-p themusobj)
       (if (Maquette-p (reference (object self)))
         (setf (slot-value (object self) 'extend) (get-obj-dur (reference (object self))))
         (setf (slot-value (object self) 'extend) (get-obj-dur themusobj)))
       (setf (slot-value (object self) 'extend) (pixel2norme container  'x (om-point-h (om-view-size self)))))
     (if (= (slot-value (object self) 'extend) 0)
       (om-set-view-size self (om-make-point 4 (om-point-v (om-view-size self))))
       (when (= 1 (strech-fact (object self)))
         (om-set-view-size self (om-make-point (norme2pixel container 'x (extend (object self)))
                                         (om-point-v (om-view-size self))))))
     (om-invalidate-view self t)))

(defmethod non-connected ((self tempobjframe) list)
   (let ((rep t))
     (loop for item in list 
           while list do
           (when (is-connected? (object self) (object item))
             (setf rep nil)
             (setf list nil))) 
     rep))

(defmethod non-connected ((self inframe) list) t)
(defmethod non-connected ((self outframe) list) t)
(defmethod update-after-evaluation ((self inframe)) nil)
(defmethod update-after-evaluation ((self outframe)) nil)



(defun normalize-points (container point-list)
   (loop for point in  point-list
         collect (pixel2point container  point)))

(defun denormalize-points (container point-list)
   (let ((sys-etat (get-system-etat container)))
     (loop for point in  point-list
           collect (point2pixel container point sys-etat))))

(defmethod class-of-connection ((self tempobjframe)) 'maq-connection)

(defmethod get-connection-lines ((self tempobjframe) i)
   (let* ((container (om-view-container self))
            (ctrl (nth i (inputframes self)))
            (connection (connected? (object ctrl)))
            (boxsource (first connection))
            possource sizesource
            (x-self (x self))
            (y-self (y self))
            (in-xs (x ctrl))
            x1 y1 xi yi)
     (if (car (frames boxsource))
         (progn
            (setf possource (om-view-position (car (frames boxsource))))
            (setf sizesource (om-view-size (car (frames boxsource))))
            (setq x1 (round (+ (om-point-h possource)  
                                              (- (* (+ (second connection) 1) (/ (om-point-h sizesource) (+ (numouts boxsource) 1))) 2))))
            (setq y1 (- (y+h (car (frames boxsource))) 2))
            (setq xi (+ x-self in-xs 4))
            (setq yi  y-self)
            (normalize-points container (list (om-make-point x1 y1)
                                                                         (om-make-point xi yi))))
       nil)
     ))

;-------D&D------

(defmethod get-relative-position ((self tempobjframe)) 
   (om-make-big-point (slot-value (object self) 'offset) (posy (object self))))

(defmethod om-drag-selection-p ((self tempobjframe) mouse-position)
   (declare (ignore mouse-position))
   (and (not (om-control-key-p)) (= (mode (om-view-container self)) 7)))

(defmethod make-drag-region ((self tempobjframe) region x0 y0 view)
   (declare (ignore view))
   (om-set-rect-region region (- (x self) x0) (- (y self) y0) (- (x+w self) x0) (- (y+h self) y0))
   region)



;;;------------

(defmethod internalize-patch ((self tempobjframe))
  ;(inspect self))
   "A blue patch becomes a red patch."
   (when (or (equal (type-of (reference (object self))) 'OMPatch)
             (equal (type-of (reference (object self))) 'OMMaquette))
     (let* ((container (om-view-container self))
            (object (object self))
            newpatch newbox
            frame conec-to-me)
       
       (setf newpatch (if (maquette-p (reference object))
                        (maq2abs (reference object))
                        (patch2abs (reference object))))
       (setf newbox (omNG-make-tempobj newpatch
                                       (om-view-position self)
                                       (mk-unique-name container (name self))))
       
       ;(setf (frame-position newbox) (borne-position (frame-position object)))
       (setf (offset newbox) (offset object))
       (setf (posy newbox) (posy object))
       
       (setf (extend newbox) (extend object))
       (setf (strech-fact newbox) (strech-fact object))
       
       (setf (sizey newbox) (sizey object))
       ;(setf (frame-name newbox) (frame-name object))
       (setf (allow-lock newbox) (allow-lock object))
       (setf (value newbox) (eval (omNG-copy (value object))))
       (setf (inputs newbox) (eval (omNG-copy (inputs object))))
       (loop for input in (inputs object)
             for in in (inputs newbox) do
             (setf (connected? in) (connected? input)))
       (setf conec-to-me (get-conect-to-me object))
       (loop for item in conec-to-me do
             (change-conections  object item newbox))
       (omg-remove-element container self)
       (setf frame (make-frame-from-callobj newbox))
       (omg-add-element  container frame)
       (compile-patch newpatch)
       (update-graphic-connections frame (get-elements (object container))))))


(defmethod externalize ((self tempobjframe) newpatch) 
   "A red patch becomes a blue patch."
  (when (or 
         (equal (type-of (reference (object self))) 'OMPatchAbs)
         (equal (type-of (reference (object self))) 'OMMaqAbs))
    (let* ((container (om-view-container self))
           (object (object self))
           (newbox (omNG-make-tempobj newpatch
                                          (om-view-position self)
                                          (name self)))
           frame conec-to-me)
      ;(setf (frame-position newbox) (borne-position (frame-position object)))
      ;(setf (frame-size newbox) (frame-size object))
      
      (setf (offset newbox) (offset object))
      
      (setf (posy newbox) (posy object))
      (setf (sizey newbox) (sizey object))
      
      (setf (strech-fact newbox) (strech-fact object))
      (setf (extend newbox) (extend object))
      
      (setf (frame-name newbox) (frame-name object))
      (setf (allow-lock newbox) (allow-lock object))
      (setf (value newbox) (eval (omNG-copy (value object))))
      (setf (inputs newbox) (eval (omNG-copy (inputs object))))
      (loop for input in (inputs object)
            for in in (inputs newbox) do
            (setf (connected? in) (connected? input)))
      (setf conec-to-me (get-conect-to-me object))
      (loop for item in conec-to-me do
            (change-conections  object item newbox))
      (setf frame (make-frame-from-callobj newbox))
      (omg-remove-element  container self)
      (compile-patch newpatch)
      (omg-add-element  container frame)
      (update-graphic-connections frame (get-elements (object container)))
      (omng-save newpatch)
)))


;===========================
;FRAME
;===========================


(defmethod get-offset/posy-in-pixel (tempobj container)
  (let ((sys-etat (get-system-etat container)))
    (point2pixel container (get-relative-position tempobj) sys-etat)))


(defmethod get-offset/posy-from-pixel ((container t) pointpixel)
  (let* ((maqpos (pixel2point container  pointpixel))
         (posy (om-point-v maqpos))
         (newoffset (om-point-h maqpos))
         (themaquette (object container))
         (metricpar (metricparam (params themaquette))))
    (when (rulermetric (editor container))
      (setf newoffset (u2ms (ms2u newoffset (first metricpar) (third metricpar))
                                        (first metricpar) (third metricpar))))
    (when (force (rulery container))
        (setf posy (calcule-y-pos posy (y-step (rulery container)))))
    (om-make-big-point newoffset posy)))

(defun calcule-y-pos (y step)
   (* step (round y step)))

(defun ms2u (ms tempo min)
  (round (ms2noir ms tempo) (/ 4 min)))

(defun ms2noir (ms tempo)
  (* ms (/ (* (second tempo) (/ 4 (first tempo))) 60000)))

(defun u2ms (u tempo min)
  (round (* (u2noir u min) (/  60000 (* (second tempo) (/ 4 (first tempo)))))))

(defun u2noir (u min)
  (* u (/ 4 min)))

;=================================


(defmethod get-colorframe ((self tempobjframe))
  (colorframe (object self)))

(defmethod change-colorframe ((self tempobjframe) newcolor)
   "set the frame color of 'self' to 'newcolor'."
   (setf (colorframe (object self)) newcolor)
   (om-invalidate-view self t))


(defmethod tempframe-p ((self tempobjframe)) t)
(defmethod tempframe-p ((self t)) nil)


(defmethod connect-box ((self markerframe) (ctrl tempobjframe))
   (let* ((framesource ctrl)
          (boxsource (object framesource))
          (mark-list (get-maquette-markers (om-view-container framesource)))
          (finmarker (is-marketed boxsource 1 mark-list))
          (startmarker (is-marketed boxsource 0 mark-list))
          (allow-mark t)
          (pos (om-point-h (om-mouse-position ctrl)))
          (s-or-e (if (< pos (round (w ctrl) 2)) 0 1)))
     (cond
      ((= s-or-e 0)
       (if finmarker
         (if (not (allow-strech-p (soft-get-mus-ob boxsource) 0))
           (om-beep-msg "Sorry this object can not be streched")
           (if (< (slot-value (object self) 'offset) (slot-value finmarker 'offset))
             (progn
               (when startmarker
                 (del-mark boxsource startmarker 0))
               (add-mark boxsource (object self) 0))
             (progn
               (setf allow-mark nil)
               (om-beep-msg "End of temporal object must be after start"))))
         (progn
           (when startmarker
             (del-mark boxsource startmarker 0))
           (add-mark boxsource (object self) 0))))
      ((= s-or-e 1)
       (if startmarker
         (if (not (allow-strech-p (soft-get-mus-ob boxsource) 0))
           (om-beep-msg "Sorry this object can not be stretched")
           (if (> (slot-value (object self) 'offset) (slot-value startmarker 'offset))
             (progn
               (when finmarker
                 (del-mark boxsource finmarker 1))
               (add-mark boxsource (object self) 1))
             (progn
               (setf allow-mark nil)
               (om-beep-msg "End of temporal object must be after start"))))
         (if (> (- (slot-value (object self) 'offset) (extend boxsource)) 0)
           (progn
             (when finmarker
               (del-mark boxsource finmarker 1))
             (MoveTempObject framesource  (om-make-point (+ (x framesource) (- (x self) (x+w framesource))) (y framesource))) 
             (add-mark boxsource (object self) 1))
           (progn
             (setf allow-mark nil)
             (om-beep-msg  "A temporal object must  start after 0"))))))
     (when allow-mark
       (draw-mark-lines (om-view-container framesource) nil)
       (update-after-mark (om-view-container framesource))
       (draw-mark-lines (om-view-container framesource)))))



