;OpenMusic
;
;Copyright (C) 1997, 1998, 1999, 2000 by IRCAM-Centre Georges Pompidou, Paris, France.
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
;
;Authors: Gerard Assayag and Augusto Agon

;DocFile
;The miniview is the view in a factory that shows graphicly the instance.
;Class and methods for miniviews are defined in this file
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Last Modifications :
;18/10/97 first date.
;DocFile

(in-package :om)

;================================================
;Miniview for boxes of editors
;================================================
(omg-defclass miniview (om-view-drag om-view) 
   ((selected-p :initform nil :accessor selected-p)
    (minipict :initform nil :accessor minipict)))

;===================DRAG&DROP=============================

(defmethod om-drag-selection-p ((self miniview) mouse-position)
   (declare (ignore mouse-position))
   (not (om-control-key-p)))

(defmethod get-drag-object ((self miniview)) (om-view-container self))

(defmethod get-pos-in-object ((self miniview) where)
   (om-add-points (om-view-position self) where))

;======================EVENTS==========================

(defmethod om-view-click-handler ((self miniview) where)
  (when (equal self (call-next-method))  ;;; new for click in lock-button
    (cond  ((mini-view-dialog-item? self) (call-next-method))
           (t 
            (if (om-command-key-p) (om-show-tooltip self 1)
                (progn (toggle-icon-active-mode (om-view-container self))  
                       (when (om-control-key-p) 
                         (menu-item-context (om-view-container self) where))))))))
 
(defmethod om-view-doubleclick-handler ((self miniview) where)
   (when (equal self (call-next-method))  ;;; new for click in lock-button
   (cond  ((mini-view-dialog-item? self) (call-next-method))
          (t (OpenObjectEditor (object (om-view-container self))))
          )))

(defmethod draw-only-select ((self om-view)) 
   (without-interrupts 
   (om-invalidate-corners self (om-make-point 1 1) (om-make-point (w self) 3))
   (om-invalidate-corners self (om-make-point 1 1) (om-make-point 3 (- (h self) 1)))
   (om-invalidate-corners self (om-make-point 1 (- (h self) 3)) (om-make-point (- (w self) 1) (- (h self) 1)))
    (om-invalidate-corners self (om-make-point (- (w self) 3) 1) (om-make-point (- (w self) 1) (- (h self) 1)))))

(defmethod get-fonde-pict ((self t)) *boxedit-pict*)

(defmethod mini-view-dialog-item? ((self miniview)) nil)

(defmethod om-draw-contents ((self miniview))
  (if (mini-view-dialog-item? self) 
     (call-next-method)
     (om-with-focused-view self
       (if (showpict (object (om-view-container self)))
           (progn
             (om-erase-rect-content 0 0 (w self) (h self))
             (draw-mini-view self (value (object (om-view-container self))))
              )
         (let* ((icon (icon (reference (object (om-view-container self)))))
                size posi xi yi iconhdlr)
           (om-draw-picture self (get-fonde-pict (value (object (om-view-container self)))) 
                                            (om-make-point 0 0) (om-make-point (w self) (h self)))
            (setf iconhdlr (second (get&corrige-icon icon)))
            (setf xi (om-pict-width iconhdlr)
                  yi (om-pict-height iconhdlr))
            (setf size (om-make-point xi yi))
            (setf posi (om-make-point (- (round (w self) 2) (round xi 2)) (- (round (h self) 2) (round yi 2))))
            (om-draw-icon iconhdlr self posi size)
            (let ((namef (show-name-p (om-view-container self))))
              (when namef
                (om-draw-string 4 (- (h self) 5) (name (object (om-view-container self)))))
              )
            ))
            (om-draw-rect-outline 0 0 (w self) (h self) (if (selected-p self) 3 1)))))


(defmethod move-miniview ((self t) dir)
   (case dir
     (0 (set-mini-param self 'deltapict (om-add-points (get-mini-param self 'deltapict) (om-make-point 0 -1))))
     (1 (set-mini-param self 'deltapict (om-add-points (get-mini-param self 'deltapict) (om-make-point 0 1))))
     (2 (set-mini-param self 'deltapict (om-add-points (get-mini-param self 'deltapict) (om-make-point 1 0))))
     (3 (set-mini-param self 'deltapict (om-add-points (get-mini-param self 'deltapict) (om-make-point -1 0)))))
   (om-invalidate-view self t))

(defmethod scroll-miniview ((self t))
   (let* ((old-mouse (om-mouse-position self))
          (first-mouse (om-mouse-position self))
          (new-mouse old-mouse)
          (inx (om-point-h (get-mini-param self 'deltapict)))
          (iny (om-point-v (get-mini-param self 'deltapict))))
     (loop while (om-mouse-down-p) do
           (setq new-mouse (om-mouse-position self))
           (unless (om-points-equal-p old-mouse new-mouse)
             (let* ((offx (- (om-point-h new-mouse) (om-point-h first-mouse)))
                    (offy (- (om-point-v first-mouse) (om-point-v new-mouse))))
               (set-mini-param self 'deltapict (om-make-point (+ inx offx) (- iny offy)))
               ;(erase+draw self)
               (om-invalidate-view self)
               (setq old-mouse new-mouse)))
               (om-get-next-event))))

(defmethod update-miniview ((self t) (type t))  (om-invalidate-view self t))
(defmethod init-miniview   ((self t) (val t ))   (update-miniview self val))


(defmethod name-values-list ((self t))
   (mapcar #'(lambda (slot)
               (let* ((slot-name (internp (name slot) (slot-package slot))))
                 (list (name  slot) (eval `(,slot-name ,self))))) 
           (get-all-slots-instances (type-of self))))


(defmethod draw-mini-view  ((self t) (value t)) 
   (draw-obj-in-rect value 0 (w self) 0  (h self) (view-get-ed-params self) self)
   )

(defmethod draw-obj-in-rect ((self t) x x1 y y1 edparams view)
   (declare (ignore edparams))
   (om-with-focused-view view
     (if (omclass-p (class-of (class-of self)))
       (let* ((thelist (name-values-list self))
              (length (length thelist)))
         (loop for item in thelist
               for i = 0 then (+ i 1)
               while (< (+ (* i 13) 13 y) y1) do
               (om-draw-string (+ x 2) (+ (* i 12) 13 y) (format nil "~D : ~D" (car item) (second item)))
               (unless (= i (- length 1))
                 (om-with-fg-color view *om-black-color*   
                   (om-draw-line x (+ (* i 13) 13 y) x1 (+ (* i 13) 13 y))))))
       (om-draw-string (+ x 2) (+ y (round (- y1 y) 2) 4) (format nil "~D"  self)))))

(defmethod print-mini-view  ((self t) (value t) tl br)
  (draw-obj-in-rect value (om-point-h tl) (om-point-h br) (om-point-v tl)  (om-point-v br) (view-get-ed-params self) self))

(defmethod get-boxsize      ((self t))    (om-make-point 70 50))

(defvar *size-miniview* 18)

(defmethod get-mini-param ((self t) param)
   (cdr (assoc param (edition-params (object (om-view-container self))))))

(defmethod set-mini-param ((self t) param val)
   (rplacd (assoc param (edition-params (object (om-view-container self)))) val))

(defmethod initx ((self t))
   (om-point-h (get-mini-param self 'deltapict))) 

(defmethod inity ((self t))
   (om-point-v (get-mini-param self 'deltapict)))


(defmethod view-get-ed-params ((self t))
   (edition-params (object (om-view-container self))))

(defmethod view-get-ed-params ((self tempobjframe))
  (default-edition-params (car (value (object self))))
  ;(edition-params (object self))
  )


;===============================draw-in maquettes=======================================

  
;======================================================
;USED IN MAQUETTES
;======================================================
(defmethod update-miniview ((self tempobjframe) (type t))  (call-next-method))



(defun draw-carre (view &optional onlysel)
  (unless (and onlysel (and (iconview view) (not (selected-p (iconview view)))))
    (om-with-focused-view view
                          (if (and (iconview view) (selected-p (iconview view))) 
                              (om-draw-view-outline view 2)
                            (om-draw-view-outline view 1)))))

(defmethod draw-editor-mode ((self t) view) 
   (draw-obj-in-rect self 0 (w view) 0  (h view) (view-get-ed-params view) view))

;---Maquette

(defvar *obje-color-list* nil)
(setf *obje-color-list* (list (om-make-color 0.525 0.769 0.549) 
                                               (om-make-color 0.51 0.49 0.788) 
                                               (om-make-color 0.851 0.851 0.475)
                                               (om-make-color 0.671 0.768 0.749)
                                               (om-make-color 0.812 1.0 1.0)
                                               (om-make-color 0.773 0.969 0.729)
                                               (om-make-color 0.812 1.0 1.0)))


(defmethod max-y-item ((self ommaquette))
  (let (min max)
    (loop for item in (boxes self) do
          (when (boxtempobj-p item) 
            (when (or (not max) (> (posy item) max)) (setf max (posy item)))
            (when (or (not min) (< (- (posy item) (sizey item)) min)) (setf min (- (posy item) (sizey item))))))
    (list max min)))

(defmethod draw-editor-mode ((self ommaquette) view)
  (let ((delta (+ 5 (if (eval-func self) (round (h view) 4) 0))))
    (om-with-focused-view view
      (om-with-fg-color nil (colorframe (object view)) 
        (let ((durmaq (get-obj-dur self))
              (rangey (max-y-item self)))
          (om-fill-rect 0 0 (w view) (h view))
        (loop for item in (boxes self) do
              (when (boxtempobj-p item)
                (draw-rect-in-rect item  (w view) (- (h view) delta)  0 5 durmaq (first rangey) (second rangey) 1 view))))
        
        (setf delta (- delta 10))
        (when (and (eval-func self) (value self))
          (om-with-fg-color nil *om-white-color*
            (om-fill-rect 0 (- (h view) delta) (w view) delta))
          (unless (maq-obj-p (value self))
            (when (minipict view)
              (om-draw-picture view (minipict view) (om-make-point 0 (- (h view) delta)) (om-make-point (w view) delta)))
            )
          )
        )
      (draw-carre view nil))))

(defmethod update-miniview ((self t) (value ommaquette)) 
  (when (minipict self) (om-kill-picture (minipict self))
        (setf (minipict self) nil))
  (when (and (eval-func value) (value value) (not (maq-obj-p (value value))))
    (setf (minipict self)  (cons-maq-mini-pict (car (list! (value value))) self 20 (om-make-point (om-width self) 40))))
  (om-invalidate-view self t))

(defmethod cons-maq-mini-obj ((self t) (view t) sizefont size)
  (cons-mini-obj self view sizefont size))


(defmethod cons-maq-mini-pict ((self simple-container) frame fontsize size)
  (let ((newfont (om-make-font *heads-font* fontsize)))
    (om-record-pict newfont size
      (cons-maq-mini-obj self frame fontsize size))))
 

(defun draw-rect-in-rect (item w h  x0 y0 durmaq maxy miny i view)
  (let* ((rangey (- maxy miny))
         (rx (+ x0 (round (* w (slot-value item 'offset)) durmaq)))
         (ry (+ y0 (- h (round (* h (- (posy item) miny)) rangey))))
         (rw (round (* w (extend item) (* (strech-fact item))) durmaq))
         (rh (round (* h (sizey item)) rangey)))
    (om-with-fg-color nil (colorframe item) 
      (om-fill-rect  rx ry rw rh))
    
    (om-with-fg-color nil *om-black-color* 
      (om-draw-rect  rx ry rw rh))
    
    (when (maquette-p (reference item))
      (let* ((maq (reference item))
             (durmaq1 (get-obj-dur maq))
             (maxy1 (max-y-item maq)))
        (loop for it in (boxes maq) do
              (when (boxtempobj-p it)
                (draw-rect-in-rect it rw rh rx ry durmaq1 (first maxy1) (second maxy1) (+ i 1) view)))
        ))))



