;OpenMusic
;
;Copyright (C) 1997, 1998, 1999, 2000 by IRCAM-Centre Georges Pompidou, Paris, France.
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
;
;Authors: Gerard Assayag and Augusto Agon


;DocFile
;Icon interface for OMBasicObjects
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Last Modifications :
;18/10/97 first date.
;DocFile

(in-package :om)

;===================
; Pckge browser object
;===================
(omg-defclass pckbrowser-icon-frame (icon-finder) ())

(omg-defclass pckbrowser-icon (icon-finder-icon) ())


(defmethod open-icon-scroller ((self pckbrowser-icon-frame))
   (let* ((panel (om-view-container self))
            (editor (om-view-container panel))
            (actives (get-actives panel))
            (title-item nil))
     (setf (current-scroll editor) (position panel (scroll-list editor) :test 'equal))     
     
     (remove-scrolls editor)
     
     (om-with-delayed-redraw editor

      (let ((new-scroll (make-new-scroll self)))
            (when new-scroll
                (om-add-subviews editor new-scroll)
                ;;; jb new 
                (setf title-item (make-panel-title self))
                (setf (titles editor) (concatenate 'list (titles editor) (list title-item)))
                (when title-item (om-add-subviews (panel-title editor) title-item))
                ;;; 
                (setf (current-scroll editor) (+ 1 (current-scroll editor)))
                (setf (scroll-list editor) (list+ (scroll-list editor) (list new-scroll)))
                (show-elements (car actives) new-scroll)
                ))
      (UpDateScrollers editor)
     ) 
     ))

(defmethod om-view-click-handler ((self pckbrowser-icon) where)
   (let* ((icn-finder (icon-finder self))
            (pan (om-view-container icn-finder))
            (ed (om-view-container pan)))
     (toggle-icon-active-mode icn-finder)
     (if (equal (class-of pan) (find-class 'packagepanel))
     (cond ((om-control-key-p)
                  (setf (current-scroll ed) (position pan (scroll-list ed) :test 'equal))
                  (remove-scrolls ed)
                  (UpDateScrollers ed)
                  (menu-item-context icn-finder where))
               ((and (om-option-key-p) (active-mode icn-finder))
                (open-icon-scroller icn-finder))
                 (t nil))
       )))


;============================
;GenFun
;============================
(omg-defclass genfun-icon-frame (pckbrowser-icon-frame) ()
   (:documentation "The class of simple frames for OMGenericFunction meta objects.#enddoc#
#seealso# (OMGenericFunction) #seealso#"))

(omg-defclass genfun-finder-icon (pckbrowser-icon) ())
(defmethod get-class-icon-icon ((self OMGenericFunction)) 'genfun-finder-icon)

(defmethod make-new-scroll ((self genfun-icon-frame))
   "Open a new editor containing simple method frames."
   (let ((thescroll (om-make-view 'GenericFunPanel
                                                           :size (om-make-point 500 500)
                                                           :position (om-make-point 0 0)
                                                           :scrollbars t
                                                           :retain-scrollbars t
                                                           ;;; :draw-scroller-outline nil
                                                           ))
           )
     (setf (object thescroll) (object self))
     thescroll))


;;; the title of the panel created with make-new-scroll
(defmethod make-panel-title ((self genfun-icon-frame)) 
   (om-make-dialog-item 'om-static-text (om-make-point 10 4) (om-make-point 160 36)
                                             (string+  (format nil "Methods of function~%") (name (object self)))
                                     :font *om-default-font2b*
                                     :bg-color  *om-window-def-color*
                                     )
   )


(defmethod show-elements ((self genfun-icon-frame) container)
   "Show the methods of 'self' as simple frames."
   (let ((i -1))
     (mapc #'(lambda (elem)
               (om-add-subviews container (make-icon-from-method elem (om-make-point 10 (+ 10 (* (incf i) 50))))))
         (get-elements  (object self)))))


;============================
;INstances
;============================
(omg-defclass instance-icon-frame (icon-finder) ()
   (:documentation "The class of simple frames for OMInstance meta objects.#enddoc#
#seealso# (OMInstance) #seealso#"))

(defmethod initialize-instance :after ((self instance-icon-frame) &key controls)
   (declare (ignore controls))
   (setf (change-name-p self) t))

(defmethod OMGMoveObject :after ((self instance-icon-frame) new-position)
  (when (big-icon-p (editor (om-view-container  self)))
    (set-icon-pos (object self) (borne-position new-position))))


;============================
;SLOT
;============================
(omg-defclass slot-icon-frame (pckbrowser-icon-frame) ()
   (:documentation "The class of simple frames for OMSlot meta objects.#enddoc#
#seealso# (OMSlot) #seealso#"))

(omg-defclass slot-finder-icon (pckbrowser-icon) ())
(defmethod get-class-icon-icon ((self OMSlot)) 'slot-finder-icon)


(defmethod make-extra-info ((self slot-icon-frame) (object OMSlot))
   (setf (change-name-p self) (not (protected-p (find-class (classname object))))))

(defmethod open-icon-win ((self slot-icon-frame) where)
   "No editor for 'self'."
   (declare (ignore where))
   (om-beep) nil)


(defun change-allocation-from-frame (frame alloc)
   (let ((newslot (omG-change-allocation (object frame) alloc)))
     (when newslot
       (setf (object frame) newslot))))


(defun initformfromslot (slot)
   (let* ((classname (classname slot))
          (initorallocval (if (equal (alloc slot) ':instance) (theinitform slot)
                              (slot-value (make-instance classname) 
                                          (internp (name slot)  (symbol-package classname))))))
     (valued-val initorallocval)))
          
(defmethod add-extra-subviews ((self slot-icon-frame))
  (let* ((val (initformfromslot (object self)))
         (instORclass-item (om-make-dialog-item 'om-static-text (om-make-point 10 10) (om-make-point 50 14)
                                                (string-downcase (string (alloc (object self)))) 
                                                :font *om-default-font1*))
         
         (thearg (om-make-dialog-item 'om-check-box
                                      (om-make-point 10 10) (om-make-point 14 12) "" 
                                      :di-action (om-dialog-item-act item
                                                          (omG-change-initarg (object self) (om-checked-p item)))
                                      :checked-p (io-p (object self))
                                      :enable (and (not (protected-p (object self)))
                                                   (not (equal (alloc (object self)) :class)))))
         
         (instORclass-but (om-make-view 'button-icon
                                        :iconID 149
                                        :position (om-make-point 140 14)
                                        :size (om-make-point 20 10)
                                        :action #'(lambda (item)
                                                    (if (not (protected-p (object self)))
                                                        (progn
                                                          (cons-pair-pop-menu instORclass-item 'change-allocation-from-frame self
                                                                              '(("instance" :instance) ("class" :class)) (alloc (object self))
                                                                              (view-position-win item) (om-view-size item))
                                                          (om-enable-dialog-item thearg (and (not (protected-p (object self)))
                                                                                             (not (equal (alloc (object self)) :class))))
                                                          (re-sort-slots (om-view-container self)))
                                                      (om-beep-msg "protected class")))))
         
         (theinitform 
          (if (omclass-p (class-of (class-of val)))
              (om-make-view 'initform-button 
                            :position (om-make-point 10 10) 
                            :size (om-make-point 16 16) 
                            :iconID (icon (class-of val))
                            :help-spec "double click to edit initform <command> to see"
                            :object self
                            :val val)
            (om-make-dialog-item 'initform-ttybox 
                                 (om-make-point 10 10) (om-make-point (min 200 (+ 5 (get-name-size (format () "~S" val))))  20) 
                                 (format () "~S" val) 
                                 :font *om-default-font2*
                                 :help-spec "double click to edit initform <command> to see"
                                 :object self :show-rect-p nil))))
    (if (equal (alloc (object self)) :class)
        (om-set-part-color instORclass-item :BODY *om-light-gray-color*))
    (setf (newviews self) (list thearg instORclass-but instORclass-item theinitform ))))



(defmethod add-extras-infos ((self  slot-icon-frame))
   (if (and (null (big-icon-p (editor (om-view-container  self))))
            (equal (type-of (editor (om-view-container  self))) 'classeditor))
     (let ((x (+ (x self) 100))
           (y (y self)))
       (if (equal (alloc (object self)) :class)
         (om-set-part-color (nameView  self) :BODY *om-light-gray-color*))
       (mapcar #'(lambda (new-view deltax)
                   (om-set-view-position new-view (om-make-point (incf x deltax) y))
                   
                   (om-add-subviews (panel (om-view-container   self)) new-view)) 
         (newviews self) '(20 40 25 75)))))

(defmethod change-sub-views-position ((self slot-icon-frame))
   (add-extras-infos self))

(defmethod omG-rename ((self slot-icon-frame) new-name)
   "Some names are not corrects (i.e. car)."
   (let ((oldname (name (object self)))
         (newslot (omG-rename (object self) new-name)))
     (if newslot
       (progn
         (setf (object self) newslot)
         (om-set-dialog-item-text (nameView self) new-name)
         (om-set-view-size (nameView self) (om-make-point (get-name-size new-name) 12))
         (om-add-subviews self (nameView self))
         (set-size self (om-view-container  self))
         (om-set-view-position (nameView self) (get-position-name (nameView self)))
         (setf (name self) new-name)
         (object self))
       (progn
         (om-beep-msg "Bad slot name")
         (setf (name (object self)) new-name)
         (omG-rename (object self) oldname) nil))))

;----------------------------------------------
;INITFORM FOR SLOTS
;----------------------------------------------

(omg-defclass initform-button (icon-view om-view-drop) 
   ((object :initform nil :initarg :object :accessor object)
    (val :initform nil :initarg :val :accessor val)
    (editorframe :initform nil :accessor editorframe)))

(defmethod slot-initform-p ((self initform-button)) t)
(defmethod slot-initform-p ((self t)) nil)

;------------Events
(defmethod om-view-click-handler ((self initform-button) where)
   (declare (ignore where)))
   ;;;(if (command-key-p) (om-set-help (not (om-help-on?)))))

(defmethod om-view-doubleclick-handler ((self initform-button) where)
   (declare (ignore where))
   (open-initform-editor self))


;------------Edition
(defmethod open-initform-editor ((self initform-button))
  (let ((container (editor (om-view-container self))))
    (if (protected-p (object container))
        (om-beep-msg "protected object")
      (if (initform-editor container)
          (om-beep-msg "Another initform editor is open, close it and try again")
        (if (Class-has-editor-p (val self))
            (setf (initform-editor container) (make-editor-window (get-editor-class (val self)) (val self) 
                                                                  (format nil "Initform for ~D slot" (name (object self)))
                                                                  self))
          (setf (initform-editor container) (open-instance-editor self)))))))


(defmethod open-instance-editor ((self initform-button))
  (let* ((ominstance (omNG-make-new-instance (val self) (name (object self))))
          (thewindow (make-editor-window 'InstanceEditor
                                        ominstance 
                                        (format nil "Initform for ~D slot" (name (object self))) self 
                                        :winpos (om-make-point 150 100)
                                        :winsize (om-make-point 250 280)))
         (slot-boxes (slots-inst-boxes ominstance (val self))))
    (setf (presentation (editor thewindow)) 0)
    (mapc #'(lambda (frame)
              (omG-add-element (panel (editor thewindow)) frame)) slot-boxes)
    (editor thewindow)))


(defmethod initform-from-val ((self t)) self)
(defmethod initform-from-val ((self ominstance)) (instance self))

(defmethod change-initform-ed ((self initform-button) newval)
  (handler-bind ((error #'(lambda (c) (declare (ignore c)) 
                            (om-beep)
                            (om-abort))))
    (let ((container (editor (om-view-container  self)))
          (*package* (find-package :om))
          (slot (object (object self))))
      (omG-change-initform slot (initform-from-val newval))
      (setf (initform-editor container) nil))))



;------------D&D
(defmethod om-drag-selection-p ((self initform-button) mouse-position)
  (declare (ignore mouse-position))
  nil)

;;; formerly drag-tracking-enter-view
(defmethod om-drag-enter-view ((self initform-button))
   (setf (selected-p self) t)
   (om-draw-contents self))

;;; formerly drag-tracking-leave-view
(defmethod om-drag-leave-view ((self initform-button)) 
  (setf (selected-p self) nil)
  (om-draw-contents self))

;============================
;Maquette
;============================
(omg-defclass maquette-icon-frame (icon-finder) ()
   (:documentation "The class of simple frames for OMMaquette meta objects.#enddoc#
#seealso# (OMMaquette) #seealso#"))

(defmethod initialize-instance :after ((self  maquette-icon-frame) &key controls)
   (declare (ignore controls))
   (setf (change-name-p self) t))

(defmethod OMGMoveObject :after ((self  maquette-icon-frame) new-position)
  (when (big-icon-p (editor (om-view-container self)))
    (set-icon-pos (object self) (borne-position new-position))))

(defmethod om-draw-contents :after ((self maquette-icon-frame))
   (when (< (omversion (object self)) 5)
     (om-with-focused-view (iconview self)
      (om-with-fg-color nil *om-red-color*
        (om-fill-rect 0 0 (w self) (h self))))))


;============================
;PATCH
;============================
(omg-defclass patch-icon-frame (icon-finder) ()
   (:documentation "The class of simple frames for OMPatch meta objects.#enddoc#
#seealso# (OMPatch) #seealso#"))

(defmethod initialize-instance :after ((self patch-icon-frame) &key controls)
   (declare (ignore controls))
   (setf (change-name-p self) t))

(defmethod OMGMoveObject :after ((self patch-icon-frame) new-position)
   (when (big-icon-p (editor (om-view-container self)))
     (set-icon-pos (object self) (borne-position new-position))))

(defmethod om-draw-contents :after ((self patch-icon-frame))
  (when (< (omversion (object self)) 5)
    (om-with-focused-view (iconview self)
      (om-with-fg-color nil *om-red-color*
        (om-fill-rect 0 0 (w self) (h self))))))

;============================
;CLASSES
;============================
(omg-defclass class-icon-frame (pckbrowser-icon-frame) ()
   (:documentation "The class of simple frames for OMClass meta objects.#enddoc#
#seealso# (OMClass) #seealso#"))

(omg-defclass class-finder-icon (pckbrowser-icon) ())
(defmethod get-class-icon-icon ((self OMClass)) 'class-finder-icon)

(defmethod initialize-instance :after ((self class-icon-frame) &key controls)
  (declare (ignore controls))
  (setf (container-p self) t))


(defmethod make-new-scroll ((self class-icon-frame))
   "When you click+alt on 'self' it open a new scroll with the internal method of the class."
   (let ((theclass (object self)))
     (if (protected-p theclass)
       (not (dialog-message "This class is protected, no access to internal methods"))
       (let ((thescroll (om-make-view 'InternalMethPanel
                                                               :size (om-make-point 500 500)
                                                               :position (om-make-point 0 0)
                                                               :scrollbars t
                                                               ;;;  :draw-scroller-outline nil
                                                               ))
              )
         (setf (object thescroll) (object self))
         thescroll))))


;;; the title of the panel created with make-new-scroll
(defmethod make-panel-title ((self class-icon-frame)) 
   (om-make-dialog-item 'om-static-text (om-make-point 10 4) (om-make-point 160 36)
                                     (string+  "Internal method of class " (name (object self)))
                                     :font *om-default-font2b*
                                     :bg-color  *om-window-def-color*
                                     )
   )


(defmethod show-elements ((self class-icon-frame) container)
   "Show the internal methods of 'self'."
   (let ((i -1))
     (mapc #'(lambda (elem)
               (om-add-subviews container (make-icon-from-method elem (om-make-point 10 (+ 10 (* (incf i) 50))))))
           (internal-met  (object self)))))


(defmethod open-icon-win ((self class-icon-frame) where)
   "Called when you double-click on a class icon."
   (declare (ignore where))
   (when (and (not (big-icon-p (editor (om-view-container  self))))
              (triangle self)
              (open? (triangle self)))
     (setf (iconID (triangle self)) 164)
     (setf (open? (triangle self)) nil)
     ;;;(om-draw-contents (triangle self))
     (om-invalidate-view (triangle self))
     (delete-sub-icons self (om-view-container  self)))
   (unless (EditorFrame (object self))
     (setf (EditorFrame  (object self)) (OpenEditorframe (object self))))
   (cond
    ((om-option-key-p)
     (when (equal (class-name (class-of (EditorFrame (object self)))) 'classPanel)
       (om-close-window (window (Editorframe (object self))))
       (setf (EditorFrame  (object self)) (OpenEditorframe (object self))))
     (om-select-window  (window (EditorFrame  (object self)))))
    (t 
     (when (equal (class-name (class-of (EditorFrame (object self)))) 'GenFunScroller)
       (om-close-window (window (Editorframe (object self))))
       (setf (EditorFrame  (object self)) (OpenEditorframe (object self))))
     (om-select-window  (window (EditorFrame  (object self)))))))
 

;============================
;FOLDER
;============================

(omg-defclass folder-icon-frame (icon-finder) ()
   (:documentation "The class of simple frames for OMFolder meta objects.#enddoc#
#seealso# (OMFolder) #seealso#"))

(defmethod initialize-instance :after ((self folder-icon-frame) &key controls)
   (declare (ignore controls))
   (setf (container-p self) t)
   (setf (change-name-p self) t))


(defmethod OMGMoveObject :after ((self folder-icon-frame) new-position)
   (when (big-icon-p (editor (om-view-container  self)))
     (set-icon-pos (object self) (borne-position new-position))))


;--------------------------
(omg-defclass globals-folder-icon (folder-icon-frame) ()
   (:documentation "The class of simple frames for OMGlobalsFolder meta objects.#enddoc#
#seealso# (OMglobalsFolder) #seealso#"))

(defmethod initialize-instance :after ((self globals-folder-icon) &key controls)
   (declare (ignore controls))
   (setf (container-p self) nil)
   (setf (change-name-p self) nil))

    
;============================
;PACKAGES
;============================
(omg-defclass package-icon-frame (pckbrowser-icon-frame) ()
   (:documentation "The class of simple frames for OMpackage meta objects.#enddoc#
#seealso# (OMpackage) #seealso#"))

(omg-defclass pckicon-finder-icon (pckbrowser-icon) ())
(defmethod get-class-icon-icon ((self OMPackage)) 'pckicon-finder-icon)


(defmethod initialize-instance :after ((self package-icon-frame) &key controls) 
   (declare (ignore controls))
   (setf (container-p self) (not (equal (object self) *om-package-tree*))))

(defmethod make-extra-info ((self package-icon-frame) (object OMPackage))
   (setf (change-name-p self) (not (or (equal object *package-user*) (protected-p object)))))

(defmethod open-icon-win ((self package-icon-frame) where)
   "Double click on 'self' open a subpackage editor or a class hierarchical tree editor"
   (if (null (father (object self)))
     (OpenObjectEditor (object self))
     (progn
       (if (< (om-point-v where) 10)
         (openclasses (object self))
         (OpenObjectEditor (object self)))
       (when (and (not (big-icon-p (editor (om-view-container self))))
                  (triangle self)
                  (open? (triangle self)))
         (setf (iconID (triangle self)) 164)
         (setf (open? (triangle self)) nil)
         ;;;(om-draw-contents (triangle self))
         (om-invalidate-view (triangle self))
         (delete-sub-icons self (om-view-container self))))))

(defmethod make-new-scroll ((self package-icon-frame)) 
   "Create a Package panel to put in a package browser."
   (let ((thescroll (om-make-view 'PackagePanel
                                                           :size (om-make-point 500 500)
                                                           :position (om-make-point 0 0)
                                                           :scrollbars t
                                                           :retain-scrollbars t
                                                           ;;; :draw-scroller-outline nil
                                                           ))
           )
     (setf (object thescroll) (object self))
     thescroll))

;;; the title of the panel created with make-new-scroll
(defmethod make-panel-title ((self package-icon-frame)) 
   (om-make-dialog-item 'om-static-text (om-make-point 10 4) (om-make-point 160 32)
                                     (string+  (format nil "Functions of package~%") (name (object self)))
                                     :font *om-default-font2b*
                                     :bg-color *om-window-def-color*)
   )


(defmethod show-elements ((self package-icon-frame) container)
   "Show the generic functions of 'self'."
   (let ((i 0))
     (mapc #'(lambda (elem)
               (add-icon-finder (make-icon-from-object elem 10 10 1 (incf i)) container)) 
         (get-functions (object self)))))


;-------------------------------------
;;; :around
(defmethod om-view-cursor :around ((self pckicon-finder-icon))
  (let ((pos-in-icon (om-mouse-position self)))
    (when pos-in-icon
     (if (father (object (icon-finder self)))
         (if (< (om-point-v pos-in-icon) 10) *om-tree-cursor* *om-pack-cursor*)
       (call-next-method)))))

;============================
;OMLIB
;============================
(omg-defclass lib-icon-frame (package-icon-frame) ()
  (:documentation "The class of simple frames for OMLib meta objects.#enddoc#
#seealso# (OMLib) #seealso#"))

;============================
;LispFun
;============================
(omg-defclass lispfun-icon-frame (icon-finder) ()
   (:documentation "The class of simple frames for OMLispFun meta objects.#enddoc#
#seealso# (OMLispFun) #seealso#"))

;============================
;TYPES
;============================
(omg-defclass type-icon-frame (icon-finder) ()
   (:documentation "The class of simple frames for OMBasicType meta objects.#enddoc#
#seealso# (OMBasicType) #seealso#"))


;============================
;OMWorkSpace NOT USED
;============================
(omg-defclass ws-icon-frame (icon-finder) ())

(defmethod initialize-instance :after ((self ws-icon-frame) &key controls)
   (declare (ignore controls))
   (setf (container-p self) nil))







