;OpenMusic
;
;Copyright (C) 1997, 1998, 1999, 2000 by IRCAM-Centre Georges Pompidou, Paris, France.
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
;
;Authors: Gerard Assayag and Augusto Agon


;DocFile
;Icon interface for OMMethods
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Last Modifications :
;18/10/97 first date.
;DocFile

(in-package :om)

(omg-defclass icon-method (OMSimpleFrame) 
   ((iconview :initform nil :accessor iconview)
    (nameview :initform nil :accessor nameview))
   (:documentation "This is the class used to visulisate OMMethods meta objects as a simple frame.
patches, folders, maquettes in the workspace are icon-finder instances.
in difference of OMContainerFrame this class is not graphic independent, 
so this class inherite from view. #enddoc#
#seealso# (OMMethod method-icon-view method-name-text-dialog ) #seealso#
#nameView# The name subview of the icon-method, it is a method-icon-view instance.#nameView#
#iconView# The icon subview of the icon-method, it is a method-name-text-dialog instance.#iconView#"))

(defmethod icon-method-p ((self icon-method)) t)
(defmethod icon-method-p ((self t)) nil)

(defmethod get-obj-for-info ((self icon-method)) (object self))

(defmethod fil ((self icon-method)) 0)

(defmethod delete-icon-finder ((self icon-method) container)
   (om-remove-subviews container self))

(defmethod omG-select :before ((self icon-method)) 
   "Select all subviews of 'self'."
  (when (not (active-mode self)) 
    (mapc #'(lambda (item)
              (setf (selected-p item) t)) (om-subviews self))))

(defmethod omG-unselect :before ((self icon-method)) 
   "Unselect all subviews of 'self'."
   (when (active-mode self)
     (mapc #'(lambda (item)
               (setf (selected-p item) nil)) (om-subviews self))))

(defmethod OMGMoveObject ((self icon-method) new-position)
   "I think that this method is never called;"
   (om-set-view-position self (borne-position new-position)))

;-------Drag and drop methods
(defmethod make-drag-region ((self icon-method) region x0 y0 view)
   (let* ((reg1 (om-new-region))
          (reg2 (om-new-region))
          (name (nameView self))
          (icon (iconView self))
          (x (- (x self) x0 (x view)))
          (y (- (y self) y0 (y view))))
     (om-set-rect-region reg1 (+ x (x icon)) (+ y (y icon)) (+ x (x+w icon)) (+ y (y+h icon)))
     (om-set-rect-region reg2  (+ x (x name))  (+ y  (y name)) (+ x (x+w name)) (+ y  (y+h name)))
     (setf region (om-union-region reg1 reg2))
     (om-dispose-region reg1)
     (om-dispose-region reg2))
   region)

;--------------------------
;icon view
;--------------------------

(omg-defclass method-icon-view (icon-view om-view-drag) ())

(defmethod get-drag-object ((self method-icon-view)) (om-view-container self))

(defmethod om-drag-selection-p ((self method-icon-view) mouse-position)
  (declare (ignore mouse-position)) 
t)



;;; formerly drag-tracking-enter-view
(defmethod om-drag-enter-view ((view method-icon-view))
  (drag-enter-view (om-view-container view)))

;;; formerly drag-tracking-leave-view
(defmethod om-drag-leave-view ((view method-icon-view)) 
  (drag-leave-view (om-view-container view)))

(defmethod get-pos-in-object ((self method-icon-view) where)
  (om-add-points (om-view-position self) where))

(defmethod om-view-click-handler ((self method-icon-view) where)
  (declare (ignore where))
    (call-next-method)
   (toggle-icon-active-mode (om-view-container self)))

(defmethod om-view-doubleclick-handler ((self method-icon-view) where)
    (declare (ignore where))
  (OpenObjectEditor (object  (om-view-container self))))


;--------------------------
;icon name
;--------------------------

(omg-defclass method-name-text-dialog (om-static-text-drag)  ())

(defmethod om-drag-selection-p ((self method-name-text-dialog) mouse-position)
  (declare (ignore mouse-position))
   t)

(defmethod get-drag-object ((self method-name-text-dialog)) (om-view-container self))

;;; formerly drag-tracking-enter-view
(defmethod om-drag-enter-view ((view method-name-text-dialog))
  (drag-enter-view (om-view-container view)))

;;; formerly drag-tracking-leave-view
(defmethod om-drag-leave-view ((view method-name-text-dialog)) 
  (drag-leave-view (om-view-container view)))


(defmethod get-pos-in-object ((self method-name-text-dialog) where)
  (om-add-points (om-view-position self) where))


;--------------

(defun make-icon-from-method (method position)
  (let* ((thespe (method-specializers method))
         (numtypes (length thespe))
         (index 0)
         (quali (method-qualifiers method))
         input-views module sizex name sizename)
    (setf name (if quali (string+  (string-downcase (string (car quali))) 
                                      ":" (string-downcase (name method)))
                   (string-downcase (name method))))
    (setf sizename (get-name-size name))
    (setf sizex (apply #'max (list 24 (* 25 numtypes) sizename)))
    (loop while thespe do
          (let ((thetype (string (class-name (pop thespe)))) thetype?)
            (when (setf thetype? (find-if 
                                   #'(lambda (item) (string-equal (name item) thetype))
                                  (list+ *Basic-Lisp-Types* *OM-class-list*)))
              (setf index (+ index 1))
            (push (om-make-view 'method-icon-view 
                    :position (om-make-point (- (* index  (round sizex (+ numtypes 1))) 8) 1) 
                    :size (om-make-point 16 16) 
                    :iconID (icon thetype?)) input-views))))
    
    (setq module
          (om-make-view 'icon-method
            :position position 
            :size  (om-make-point sizex 44)
            :object  method
            :name name))
    (setq input-views (nreverse input-views))
    (loop for input-f in input-views do (om-add-subviews module input-f))
    (om-add-subviews module (setf (iconview module)
                               (om-make-view 'method-icon-view
                                 :iconID (icon (fdefinition (method-name method)))
                                 :size (om-make-point 16 16)
                                 :position (om-make-point (- (round (w module) 2) 8) 16))))
    (om-add-subviews module (setf (nameview module)
                                                           (om-make-dialog-item 'method-name-text-dialog
                                                                                                     (om-make-point (- (round (w module) 2) (round sizename 2)) 34 )
                                                                                                     (om-make-point sizename 12)
                                                                                                     name 
                                                                                                     :font *om-default-font1b*
                                                                                                     )))
    
    module))
