;;;=== IMPORT / EXPORT ===
;;; OM objects from/to files


(in-package :om)


(defun import-file (container)
  (catch-cancel
    (let ((pos (om-mouse-position container))
          (path (om-choose-file-dialog :prompt "import file... ":button-string "import ")))
      (when path
        (handler-bind ((error #'(lambda (c) 
                                  (om-message-dialog (format nil "Error while importing ~D ~%   ~D ~%"
                                                             (namestring path) 
                                                             (om-report-condition c)
                                                             )
                                                     :size (om-make-point 300 300))
                                  (om-abort)
                                  )))
          
          
          (let ((type (file-type path)))
            (cond
             ((equal :patc type) 
              (make-new-file container path 'omNG-make-new-patch pos))
             ((equal :maqt type)
              (make-new-file container path 'omNG-make-new-maquette pos))
             (t (let ((mkfunc (mkobjfromtype type)))
                  (if mkfunc 
                    (make-new-file container path mkfunc pos)
                    (om-beep-msg "This file isn't a patch or a maquette")))))
            ))))))


(defmethod mkobjfromtype (type) nil)



(defun import-folder (container)
  (catch-cancel
     (let ((pos (om-mouse-position container))
        (name (om-choose-directory-dialog)))
    (when name
      (make-new-folder container name pos)))))

(defmethod export-file ((self OMPatch))
   (catch-cancel
     (let* ((name (om-choose-new-file-dialog :name (name self)    
                   :prompt (string+ "Save this " (get-object-insp-name self) " as.."))))
       (when name
         (unless (pathname-type name)
           (setf name (pathname (string+ (namestring name) "." (obj-file-extension self)))))
         (do-copy-to-finder self name)))))

(defmethod export-file ((self t)) t)

(defmethod export-file ((self OMFolder))
   (catch-cancel
     (let* ((name (om-choose-new-directory-dialog)))
       (when name
           (folder2finder self name)))))



;;;=== from persistant to finder ===

;;; !!! marche pas stream-reader stream-writer sur Allegro
(defun my-copy-file (from-file to-file &aux char)
  (delete-file-protection to-file)
  (with-open-file (from-stream from-file)
    (with-open-file (to-stream to-file :direction :output :if-exists :supersede :if-does-not-exist :create)
      (multiple-value-bind (reader reader-arg)
                           (stream-reader from-stream)
        (multiple-value-bind (writer writer-arg)
                             (stream-writer to-stream)
          (loop (unless (setq char (funcall reader reader-arg))
                  (return))
                    (funcall writer writer-arg char)))))))

(defmethod folder2finder ((self OMFolder) path)
  (create-directory path)
  (loop for item in (get-elements self) do
        (cond
         ((folder-p item) 
          (folder2finder item (make-pathname :directory (append (pathname-directory path) (list (name item))))))
         (t (om-copy-file (mypathname item) (make-pathname :directory (pathname-directory path)
                                                        :name (name item)))))))
  
(defmethod do-copy-to-finder ((self OMBasicObject) path)
  (if (om-shift-key-p)
    (my-copy-file (mypathname self) path)
    (progn
      (om-copy-file (mypathname self) path)
      (unless (saved? self)
        (let ((oldpat (mypathname self)))
          (setf (mypathname self) path)
          (omng-save self)
          (setf (saved? self) nil)
          (setf (mypathname self) oldpat))))) t)

(defmethod do-copy-to-finder ((self OMPackage) path)
   (let ((depend? (depending-p self)))
     (if depend?
       (om-beep-msg (string+ depend? " Try dragging the package " (name (father self))))
       (let ((unicpath (OMfile2folder path)))
         (loop while (probe-file unicpath)
               for i = 0 then (+ i 1) do
               (setf unicpath (OMfile2folder path i)))
       (send2finder  self unicpath)))))

(defmethod send2finder ((self OMPackage) thepath)
   (unless (probe-file thepath) (create-file thepath))
   (loop for class in (classes self) do
         (omNG-save class thepath))
   (loop for met in (direct-pack-methods self) do
         (omNG-save met thepath))
   (loop for pack in (subpackages self) do
         (send2finder pack (make-pathname :directory (append (pathname-directory thepath) (list (name pack)))))))





;;;=== from perform drop ===

;;; semble plus utilisee...
(defun make-new-file-little (self pathfile fun pos)
   (declare (ignore pos))
   (if (EditorFrame (object self))
     (make-new-file (EditorFrame (object self)) pathfile fun (om-make-point 22 22))
     (if (name-exist-p (object self) (real-pathname pathfile))
       (om-beep-msg "This name already exists !!!")
       (let* ((name (real-pathname pathfile))
              (newpatch (funcall fun name (om-make-point 22 22))))
         (omg-add-element-in-icon self newpatch)
         (setf (loaded? newpatch) nil)
         (copy-file-sp pathfile (mypathname newpatch)) t))))

(defun make-new-file (self pathfile fun pos)
   (if (name-exist-p (object self) (real-pathname-name pathfile))
     (om-beep-msg "This name already exists !!!")
     (let* ((name (real-pathname-name pathfile))
            (newpatch (funcall fun name pos))
            (new-frame (make-icon-from-object  newpatch (om-point-h pos) (om-point-v pos) 1 1)))
       (when (or (not (file-has-comments pathfile)) (file-has-old-comments pathfile))
                      (setf (omversion newpatch) 4.9))
       (omg-add-element self new-frame)
       (setf (loaded? newpatch) nil)
       (copy-file-sp pathfile (mypathname newpatch))
       t)))

(defun make-new-folder-little (self pathfile pos)
   (declare (ignore pos))
   (if (name-exist-p (object self) (name-of-directory pathfile))
     (om-beep-msg "This name already exists !!!")
     (if (EditorFrame (object self))
       (make-new-folder (EditorFrame (object self)) pathfile (om-make-point 22 22))
       (let* ((name (name-of-directory pathfile))
              (newfolder (omNG-make-new-folder name (om-make-point 22 22))))
         (omg-add-element-in-icon self newfolder)
         (copy-subfiles newfolder (om-directory  pathfile :files t :directories t))))))

(defun make-new-folder (self pathfile pos)
  
  (if (name-exist-p (object self) (name-of-directory pathfile))
    (om-beep-msg "This name already exists !!!")
    (let* ((name (name-of-directory pathfile))
            (newfolder (omNG-make-new-folder name pos))
            (new-frame (make-icon-from-object  newfolder (om-point-h pos) (om-point-v pos) 1 1)))
      (show-message-win "Importing files...")
      (omg-add-element self new-frame)
      (copy-subfiles newfolder (om-directory  pathfile :files t :directories t))
      (hide-message-win)
      t)))

(defun copy-subfiles (folder list)
  (loop for unpath in list
        for i = 0 then (+ i 1) do 
        (handler-bind ((error #'(lambda (c) 
                                  (om-message-dialog (format nil "Error while importing ~D ~D ~%Please cleanup the folder before importing it..."
                                                             (name folder) 
                                                             (om-report-condition c)
                                                             )
                                                     :size (om-make-point 300 300))
                                  (om-abort)
                                  )))
          (let ((item unpath))
            (when (om-persistant-p item) 
              (let ((wsparams (get-init-wsparams item))
                    (doc "non documentation") newobj)
                (setf newicon (seventh wsparams))
                (setf doc (str-with-nl (sixth wsparams)))
                (setf wsparams (subseq wsparams 2 5))
                (if (directoryp item)
                  (progn
                    (setf newobj (omNG-make-new-folder (name-of-directory item) (om-make-point 20 20)))
                    (omNG-add-element folder newobj)
                    (copy-subfiles newobj (om-directory  item :files t :directories t)))         
                  (let ((pathtype (file-type item))
                        (name (real-pathname-name item)))
                    (cond
                     ((equal pathtype :PATC)
                      (setf newobj (omNG-make-new-patch name (om-make-point 20 20)))
                      (when (or (not (file-has-comments item)) (file-has-old-comments item))
                        (setf (omversion newobj) 4.9))
                      (omNG-add-element folder newobj)
                      (setf (loaded? newobj) nil)
                      (copy-file-sp item (mypathname newobj)))
                     ((equal pathtype :MAQT)
                      (setf newobj (omNG-make-new-maquette name (om-make-point 20 20)))
                      (when (or (not (file-has-comments item)) (file-has-old-comments item))
                        (setf (omversion newobj) 4.9))
                      (omNG-add-element folder newobj)
                      (setf (loaded? newobj) nil)
                      (copy-file-sp item (mypathname newobj)))
                     ((equal pathtype :PROJ))
                     ((equal pathtype :APLI))
                     (t nil))
                    ))
                (when newobj
                  ;(setf (wsparams newobj) wsparams)
                  (setf (wsparams newobj) (loop for i in wsparams collect (eval i)))
                  (set-doc newobj doc))
                )
              )
            ))))


;;; PLUS UTILISEE ...
(defmethod new-package-from-folder ((self ompackage) pathfile) 
  (setf *loading-classes* nil
        *loading-methods* nil
        *loading-initmet* nil)
  (mapc #'(lambda (newpath) 
            (fill-package-from-path newpath self))
        (om-directory  pathfile :files t :directories t))
  
  (let* ((*package* (find-package :om))
         (classes *loading-classes*)
         (methods *loading-methods*)
         (init-class-met *loading-initmet*)
         err smethods badclasses)
    (setf err (detect-class-redefinition classes))
    (if err
      (om-beep-msg (format nil "This classes are already defined :~%~{~D~}
you can not import this package." err))
      (progn  
        (setf badclasses (eval-initial-classes classes))
        (setf err (detect-genfun-redefinition methods))
        (setf err (remove-duplicates err :test 'equal))
        (when err
          (om-beep-msg (format nil "WARNING These generic functions are already defined: ~%~{~D ~}" 
                            (mapcar #'(lambda (elem) (string (eval elem))) err))))
        
        (setf smethods (mapcar #'(lambda (elem) (let ((new-semimethod (eval (second elem))))
                                                  (when (ommethod-p new-semimethod) 
                                                    (setf (mypathname new-semimethod) (first elem))
                                                    new-semimethod))) methods))
        (mapc #'(lambda (elem) (when elem (define-really-method elem))) smethods)
        (initial-methods-for-classes init-class-met badclasses)
        (mapc #'(lambda (newpath) 
                  (put-in-packpackages newpath self badclasses))
              (om-directory  pathfile :files t :directories t))
        t))))

;;; PLUS UTILISEE ...
(defun deep-copy-package (path-source path-target)
   (if (directory-pathname-p path-source)
     (let ((newpath (OMfile2folder (make-pathname :directory  (pathname-directory path-target) :name  (name-of-directory path-source))))
           (directories (om-directory  path-source :files t :directories t)))
       (create-directory newpath :if-exists nil)
       (loop for item in directories do
             (deep-copy-package item newpath)))
     (let ((pathtype (file-type path-source))
           (name (real-pathname-name path-source)))
       (if (or (equal pathtype :CLAS) (equal pathtype :METH))
           (copy-file-sp path-source (make-pathname :directory (pathname-directory path-target) :name name))))))


;==================To FINDER=============
(defmethod external-object-p ((self OMFolder)) t)
(defmethod external-object-p ((self OMglobalsFolder)) nil)

(defmethod external-object-p ((self OMInstance)) t)
(defmethod external-object-p ((self OMPatch)) t)
(defmethod external-object-p ((self OMPackage)) (and (father self) (not (kernel-p self))))

(defmethod external-object-p ((self OMClass)) (mypathname self))
(defmethod external-object-p ((self OMMethod)) (mypathname self))



(defmethod external-object-p ((self OMBoxEditCall)) t)
(defmethod om-filetype ((self OMBoxEditCall)) :|INST|)


(defmethod external-object-p ((self OMBoxInstance)) t)
(defmethod om-filetype ((self OMBoxInstance)) :|INST|)

(defmethod external-object-p ((self TemporalBox)) t)
(defmethod om-filetype ((self TemporalBox)) :|INST|)

(defmethod om-filetype ((self OMPatch)) :|PATC|)
(defmethod om-filetype ((self OMMaquette)) :|MAQT|)
(defmethod om-filetype ((self OMInstance)) :|INST|)
(defmethod om-filetype ((self OMFolder)) :|fold|)
(defmethod om-filetype ((self OMPackage)) :|fold|)
(defmethod om-filetype ((self OMClass)) :|CLAS|)
(defmethod om-filetype ((self OMMethod)) :|METH|)

