;;===========================================================================
;OM API 
;Multiplatform API for OpenMusic
;Macintosh version (Digitool Macintosh Common Lisp - MCL)
;
;Copyright (C) 2004 IRCAM-Centre Georges Pompidou, Paris, France.
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
;
;Authors: Augusto Agon and Jean Bresson
;;===========================================================================

;;===========================================================================
;DocFile
; Misc. TOOLS (buffers, pointers, print, etc...)
;;===========================================================================

(in-package :om-api)
;;;========
;;; export :
;;;========
(export '(
                quoted-form-p
                special-form-p
                lambda-expression-p
                om-report-condition
          om-read-list-from-string
          
                om-make-buffer
                om-copy-buffer
                om-buffer-delete
                om-buffer-insert
                om-buffer-text
                om-buffer-char
                om-buffer-size
                om-buffer-substring
                om-lines-in-buffer
                om-buffer-line-start   
                om-buffer-line-end
                om-buffer-skip-fwd-wsp&comments
                om-buffer-insert-file
                om-buffer-write-file

          om-make-pointer
          om-free-pointer
          om-read-ptr
          om-write-ptr

                while
                integer-to-string
                copy-command
                paste-command
                
                om-random-value
                
                om-show-documentation
                om-edit-definition

                om-page-setup
                om-print-window
                
                om-closure-function
                ) :om-api)

;----------------------
; LISP TOOLS FEATURES
;----------------------

(defun quoted-form-p (form)
  (ccl::quoted-form-p form))

(defun lambda-expression-p (form)
   (ccl::lambda-expression-p form))

(defmacro while (&body body)
  `(ccl::while ,. body))


(defun integer-to-string (int)
   (ccl::%integer-to-string int))

(defmethod om-report-condition ((c condition))
  (ccl::report-condition c nil))

(defun om-read-list-from-string (string)
  (ccl::read-list-from-string str))

;=================
; copy/paste 

(defmethod copy-command ((self t))
   (copy self))

(defmethod paste-command ((self t))
  (paste self))


;=================
;safe random
(defun om-random-value (num)
  (if (< num 0)
    (- (random (- num)))
    (random num)))



;=======================
; find and edit source code 
;=======================

(defun om-edit-definition (symbol)
  (edit-definition symbol))


;;;==== POINTER ======

;(defmacro om-make-pointer (size &optional (clear nil))
;  (if clear
;      `(#_newptr ,size)
;    `(#_newptrclear ,size)))
(defun om-make-pointer (size &optional (clear nil))
  (if clear
    (#_newptr size)
    (#_newptrclear size)))



(defun om-free-pointer (ptr) 
  (#_DisposePtr ptr))

;;; a completer pour les autres types
(defun om-write-ptr (ptr pos type value)
  (cond 
   ((equal :unsigned-byte type)
    (%put-byte ptr (coerce value 'unsigned-byte) pos))
   ((equal :single-float type)
    (%put-single-float ptr (coerce value 'single-float) pos))
   ((equal :double-float type)
    (%put-double-float ptr (coerce value 'single-float) pos))
  (t nil)))

(defun om-read-ptr (ptr pos type)
  (cond 
   ((equal :unsigned-byte type)
    (%get-byte ptr pos))
   ((equal :single-float type)
    (%get-single-float ptr pos))
   ((equal :double-float type)
    (%get-double-float ptr pos))
   ((equal :string type)
    (%get-cstring ptr pos))
  (t nil)))

;;;==== TEXT BUFFER ====


(defun om-make-buffer ()
  (make-buffer))

(defun copy-buffer (buffer)
   (let ((newbuffer (om-make-buffer))
           (size (- (om-buffer-size buffer) 1)))
     (loop for i from 0 to size do
               (om-buffer-insert newbuffer (om-buffer-char buffer i) i))
     newbuffer)
   )

(defun fill-buffer (target source)
  (let ((size (- (buffer-size target) 1))
        (size1 (- (buffer-size source) 1)))
    (buffer-delete target 0 size)
    (loop for i from 0 to size1 do
          (buffer-insert target  (buffer-char source i) i))))


(defun om-copy-buffer (buffer)
   (copy-buffer buffer))


(defun om-buffer-delete (textbuffer start &optional end)
   (buffer-delete textbuffer start end))
     
(defun om-buffer-insert (textbuffer string &optional (position nil))
   (buffer-insert textbuffer string  position))


(defun om-buffer-text (textbuffer)
  (buffer-substring textbuffer 0))

;;; peut etre a ajouter.. ?
;;;(defun om-buffer-exp (textbuffer pos)
;;;  (buffer-current-sexp textbuffer pos))


(defun om-buffer-char (textbuffer &optional (i 0))
   (buffer-char textbuffer i))

(defun om-buffer-substring (textbuffer from &optional to)   
  (buffer-substring textbuffer from to))

(defun om-buffer-size (textbuffer)
   (buffer-size textbuffer))


(defun om-lines-in-buffer (buffer)
   (lines-in-buffer buffer))

(defun om-buffer-line-start (buffer &optional (start 0))
   (buffer-line-start buffer start))

(defun om-buffer-line-end (buffer &optional end)
   (buffer-line-end buffer end))


(defun om-buffer-skip-fwd-wsp&comments (buffer start end)
   (buffer-skip-fwd-wsp&comments buffer start end))
   
(defun om-buffer-insert-file (buffer path &optional position)
   (buffer-insert-file buffer path  position))

(defun om-buffer-write-file (buffer path &key if-exists)
   (om-buffer-write-file buffer path :if-exists if-exists))


;;;==================
;;; PRINT
;;;==================

(defun om-page-setup ()
  (print "no printing..."))

(defun om-print-window (win)
  (declare (ignore win))
  (print "no printing..."))

;======for situation

(defun om-closure-function (fun)
  (ccl::closure-function fun))