
(cl:defpackage "OM-API"
  (:nicknames "OA")
   (:use "COMMON-LISP" "CCL")
   )

(defpackage "OpenMusic"
  (:use "COMMON-LISP" "CCL" "OA")
  (:import-from "CCL")
  (:nicknames "OM"))

(defpackage "MIDISHARE"
  (:nicknames "MS"))


(in-package :om)


(defvar *om-version* 5.11 "Contains the current version of OM")

(defun compile&load (file &optional (verbose t))
   (let ((lisp-file 
          (if (pathname-type file) file (concatenate 'string file ".lisp")))
         (fasl-file (compile-file-pathname file)))
     (when (or (not (probe-file fasl-file))
               (not (file-write-date lisp-file))
               (not (file-write-date fasl-file))
               (> (file-write-date lisp-file) (file-write-date fasl-file)))
       (compile-file file :verbose verbose))
     (load file :verbose verbose)))


(defmethod x->dx ((self list))
  (loop for x in self
        for y in (rest self)
        collect (- y x)) )

(defmethod dx->x ((start number) (list list))
  (cons start (loop for dx in list
                    sum dx into thesum
                    collect (+ start thesum)) ))

(defmethod nth-random ((list list))
  (nth (random (length list)) list))

;------------TIMES-----------------

(defmethod om* ((self number) (num number)) (* self num))

(defmethod om* ((self number) (num list))  
  (mapcar #'(lambda (input)
              (om* self input)) num))

(defmethod om* ((self list) (num number))   
  (mapcar #'(lambda (input)
              (om*  input num)) self))

(defmethod om* ((self list) (num list))
  (mapcar #'(lambda (input1 input2)
              (om* input1 input2)) self num))

;------------process-----------------

(defun om-kill-process (process)
  (ccl::process-disable process)
  (ccl::process-kill process))


;------------cloning and saving -----------------

(defclass ompoint () ((pt :accessor pt :initarg :pt :initform (make-point 0 0))))


(defun om-make-point (x y) 
  (make-instance 'ompoint :pt (make-point x y)))






 
(in-package :ccl)

;; To load Frameworks on MacOSX
;;===============================


(export '(*__CFStringMakeConstantString-slep*
          load-framework-bundle
          get-fun-addr
          lookup-function-in-framework
          ) :ccl)

(defvar *__CFStringMakeConstantString-slep*
  (ccl::get-slep "__CFStringMakeConstantString"))

(defun CFSTR (string)
  (with-cstrs ((cstr string))
    (ccl::ff-call-slep *__CFStringMakeConstantString-slep*
                       :address cstr 
                       :address)))

   
(defun create-frameworks-url ()
  (rlet ((fsref :fsref))
    (let* ((err (#_FSFindFolder #$kOnAppropriateDisk #$kFrameworksFolderType #$true fsref)))
      (declare (type (signed-byte 16) err))
      (if (eql #$noErr err)
        (let* ((url (#_CFURLCreateFromFSRef (%null-ptr) fsref)))
          (if (%null-ptr-p url)
            (error "Failed to create URL")
            url))
        (error "Couldn't find system Frameworks folder")))))

(ccl::defloadvar *frameworks-url* nil)

(defun frameworks-url ()
  (or *frameworks-url*
      (setq *frameworks-url* (create-frameworks-url))))


;;;=== FRAMEWORKS NOT IN SYSTEM ... =====

(defun create-user-frameworks-url ()
  (let* ((path "/Library/Frameworks")
     (url
      (with-cfstrs ((cfstr-path path))
           (#_CFURLCreateWithFileSystemPath (%null-ptr) cfstr-path #$kCFURLPOSIXPathStyle #$true))))
    (if (%null-ptr-p url)
      (error "Failed to create URL")
      url)))


 (ccl::defloadvar *user-frameworks-url* nil)

(defun user-frameworks-url ()
  (or *user-frameworks-url*
      (setq *userframeworks-url* (create-user-frameworks-url))))

(defun load-user-framework-bundle (framework-name)
  (let* ((bundle-url 
          (#_CFURLCreateCopyAppendingPathComponent
           (%null-ptr)
           (user-frameworks-url)    ; file:///Library/Frameworks/
           (CFSTR framework-name)
           #$false)))
    (if (%null-ptr-p bundle-url)
      (error "Can't create URL for ~s in user frameworks folder" 
             framework-name)
      (let* ((bundle (#_CFBundleCreate (%null-ptr) bundle-url)))
        bundle))))

;;; =========================================

;(load-user-framework-bundle "OSC.framework")

(defun load-framework-bundle (framework-name)
  (let* ((bundle-folder "/System/Library/Frameworks/")
         (bundle-url 
          (#_CFURLCreateCopyAppendingPathComponent
           (%null-ptr)
           (frameworks-url)    ; file:///System/Library/Frameworks/
           (CFSTR framework-name)
           #$false)))
    (if (%null-ptr-p bundle-url)
      (error "Can't create URL for ~s in system frameworks folder" 
             framework-name)
      (let* ((bundle (#_CFBundleCreate (%null-ptr) bundle-url)))
        (when (%null-ptr-p bundle) 
          (setf bundle (load-user-framework-bundle framework-name))
          (setf bundle-folder "/Library/Frameworks/"))
        (if (%null-ptr-p bundle)
          (error "Can't create bundle for ~s" framework-name)
          (if (eql #$false (#_CFBundleLoadExecutable bundle))
            (error "Couldn't load bundle library for ~s" framework-name)
            bundle))))))



(defun lookup-function-in-framework (symbol-name bundle)
  (let* ((addr (#_CFBundleGetFunctionPointerForName bundle (CFSTR symbol-name))))
    (if (%null-ptr-p addr)
      (error "Couldn't resolve address of foreign function ~s" symbol-name)
      ;; This may be a little confusing: MCL uses fixnums (whose low 2 bits are
      ;; zero) to represent function addresses (whose low 2 bits are zero ...)
      ;; Shove the pointer in a buffer; fetch a signed 32-bit integer, shift it
      ;; right 2 bits ... voila.
      (rlet ((buf :long))
        (setf (%get-ptr buf) addr)
        (ash (%get-signed-long buf) -2)))))

(defmacro get-fun-addr (name framework)
  `(lookup-function-in-framework ,name ,framework))

     
 
     
  





