(in-package :nc)

(export '(agent NewAgent DefAgentMessage kill disable reset pause resume actor-loop task clean-up awake-p) :nc)

(defclass actor () 
  ((process  :accessor process)
   (state :initform :awake :accessor state :initarg :state)
   ))

(defclass reactor (active-object)
  ((process :accessor process)))

(defclass agent (actor reactor)
  (
   (inport :accessor inport :initform nil)
   ))


;;; message macro

(defmacro DefAgentMessage (name when lambda-list &body body)
  `(progn
     (defpargeneric ,name ,when ,(loop for arg in lambda-list
                                      collect (if (listp arg) (first arg) arg)))
     (defmethod ,name ,lambda-list ,. body)))

(defmacro NewAgent (agent-class) `(make-instance ,agent-class))

;;; changes to the active object scheduler
;;; we do not create processes anymore

(defmethod schedule ((sched standard-scheduler) obj message)
  (enqueue (mail-queue obj) message)
  (unless (active-p obj)
    (setf (active-p obj) t)
    ))

(defmethod unschedule ((sched standard-scheduler) obj)
  (setf (active-p obj) nil))


;;; actor methods

(defmacro awake-p (agent)
  `(eq (state ,agent) :awake))

(defmacro set-state-asleep  (agent)
  `(setf (state ,agent) :asleep))

(defmacro set-state-awake  (agent)
  `(setf (state ,agent) :awake))


(defmethod initialize-instance :after ((self actor) &rest initargs)
  (declare (ignore initargs))
  (setf (process self) 
        (process-run-function (symbol-name (class-name (class-of self)))
                              #'actor-loop self)))
(defmethod kill ((obj actor))
  (process-kill (process obj)))


(defmethod task ((self actor))
  (format *stdout*  "Agent running : ~a~%~%" self) (sleep 2))


#|
(defmethod actor-loop ((self actor))
  (loop 
    do
    (loop while (eq (state self) :awake)
          do (funcall #'task self))
    (setf (state self) :asleep)
    (process-wait "actor-asleep" #'(lambda (agent) (eq (state agent) :awake)) self)))
|#

;;; run messages then run task

#|
(defmethod actor-loop ((self actor))
  (loop 
    do
    (active-loop self)
    (if (awake-p self) 
      (funcall #'task self)
      (process-allow-schedule))))
|#

(defvar *sleep* .015)
(defvar *save-sleep* .015)


(defmethod actor-loop ((self actor))
  (loop 
    do
    (active-loop self)
    (if (awake-p self) 
      (progn (funcall #'task self) 
        (sleep *sleep*)
        )
      (sleep 0.1)
      )
    ))



;;; agent methods


#|

(defmethod reset  ((obj agent))
  (AgentReset obj))

(defvar *xxx* 1000)

(DefAgentMessage AgentReset :now ((obj agent))
  ; clean reset : terminates the current actor-loop iteration before
  ; resetting
  ;(pause obj)
  (reset-listen obj)
  (clean-up obj)
  ;(format *stdout* "Prepare to reset ~%")
  (process-reset (process obj))
  (process-preset (process obj) #'actor-loop obj)
  (process-enable (process obj))
  ;(resume obj)
  )

|#


(defmethod clean-up ((self agent)) )


(defmethod kill ((self agent))  
  (process-kill (process self))        ; kill actor process
  (when (string= (process-whostate (process self)) "Suspended") (process-enable (process self)))
  (unschedule (scheduler self) self)
  (disconnect-all self))  ; unschedule reactor 

(defmethod disable ((self agent))  
  (process-disable (process self)))        

(DefAgentMessage pause :now ((self agent))
  (set-state-asleep self))

(DefAgentMessage resume :now ((self agent))
  (set-state-awake self))




;;; some operations cannot be direct messags to agents
;;; they are mediated by god


(defclass god (agent) ())

(defvar *god* nil)

(DefAgentMessage AgentReset :past ((agent-god agent) (agent-creature agent))
  (reset-listen agent-creature)
  (clean-up agent-creature)
  ;(format *stdout* "Prepare to reset ~%")
  (process-reset (process agent-creature))
  (process-preset (process agent-creature) #'actor-loop agent-creature)
  (process-enable (process agent-creature))
  (format *stdout* "~a has been reset ~%" (symbol-name (type-of agent-creature )))
  )

(defmethod reset ((self agent))
  (AgentReset *god* self))

(defmethod task ((self god))
  (sleep 0.090)
   )

#|


(setf aa (make-instance 'actor))
(reset aa)
(kill aa)
(process-kill (first ccl::*all-processes*))

(setf ag (make-instance 'agent))
(setf ag2 (make-instance 'agent))
(kill ag)
(kill ag2)

(pause ag)
(pause ag2)
(resume ag)
(resume ag)
(resume ag2)
(reset ag)

|#
