;;===========================================================================
;OM API 
;Multiplatform API for OpenMusic
;Macintosh version (Digitool Macintosh Common Lisp - MCL)
;
;Copyright (C) 2004 IRCAM-Centre Georges Pompidou, Paris, France.
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
;
;Authors: Augusto Agon and Jean Bresson
;;===========================================================================

;;===========================================================================
;DocFile
; DIALOG ITEMS CLASSES AND FUNCTIONS
;;===========================================================================

(in-package :om-api)
;;;========
;;; export :
;;;========
(export '(
                om-make-dialog-item
                om-dialog-item-act
                om-dialog-item-action
                om-dialog-item-action-function
                om-set-dialog-item-action-function
                om-funcall-dialog-item-action
                
                om-dialog-item-text
                om-set-dialog-item-text
                om-enable-dialog-item
                om-dialog-item-enabled
                
                om-static-text
                om-text-view
 
                om-editable-text
                om-get-text-edit-size
                om-get-text-edit-pos
                
                om-text-edit-view
                om-scroll-text-edit-view
                om-make-edit-view
                
                om-button
                om-check-box
                om-radio-button
                
                om-checked-p
                om-set-check-box
                
                om-single-item-list
                om-multi-item-list
                om-pop-up-dialog-item
                om-set-item-list
                om-get-item-list
                om-get-selected-item-index
                om-select-item-index
                om-unselect-item-index
                om-get-selected-item
                
                om-slider
                om-slider-value
                om-set-slider-value
                
                ) :om-api)

;;;=====================
;;;ABSTRACT
;;;=====================



(defclass om-standard-dialog-item (om-graphic-object) 
  ((di-action :accessor di-action :initform nil :initarg :di-action)
   )
  (:default-initargs 
   :callback-type :item 
   :callback 'om-dialog-item-action))

;;;==========
;;; GENERAL API CALLS
;;;==========

(defmethod om-subviews ((self om-standard-dialog-item)) nil)

(defmethod om-invalidate-view ((self om-standard-dialog-item) &optional (erase t))
  (capi::redisplay self))

(defmethod om-redraw-view ((self om-standard-dialog-item))
 (capi::redisplay self))




;;;==========
;;; ACTION
;;;==========

(defmacro om-dialog-item-act (var &body body)
  `#'(lambda (,var) ,@body))

(defmethod om-dialog-item-action ((self om-standard-dialog-item))  
  (when (di-action self)
    (funcall (di-action self) self)))

(defmethod om-set-dialog-item-action-function ((self om-standard-dialog-item) action)
   (setf (di-action self) action))

(defmethod om-dialog-item-action-function ((self om-standard-dialog-item))
   (di-action self))

(defmethod om-funcall-dialog-item-action ((self om-standard-dialog-item))
    (when (di-action self)
      (funcall (di-action self) self)))

(defmethod om-enable-dialog-item ((self om-standard-dialog-item) t-or-nil) nil)

(defmethod om-dialog-item-enabled ((self om-standard-dialog-item)) t)

;;;==========
;;; ATTRIBUTES
;;;==========

(defmethod om-dialog-item-text ((self om-standard-dialog-item))
  (capi::item-text self))

(defmethod om-set-dialog-item-text ((self om-standard-dialog-item) text)
  (setf (capi::item-text self) text))

(defmethod om-view-position ((self om-standard-dialog-item)) 
   (if  (interface-visible-p self) 
       (let ((point (multiple-value-list (pinboard-pane-position self))))
         (om-make-point (first point) (second point)))
    (om-make-point (vx self) (vy self))))

(defmethod om-set-view-position ((self om-standard-dialog-item) pos-point) 
    (if  (interface-visible-p self) 
        (apply-in-pane-process self 
                               (lambda () 
                                 (setf (pinboard-pane-position self) (values (om-point-h pos-point) (om-point-v pos-point))))))
    (setf (vx self) (om-point-h pos-point) (vy self) (om-point-v pos-point)) )

(defmethod om-view-size ((self om-standard-dialog-item))
  (if  (interface-visible-p self) 
      (let ((point (multiple-value-list (pinboard-pane-size self))))
        (om-make-point (first point) (second point)))
    (om-make-point (vw self) (vh self))))

(defmethod om-set-view-size ((self om-standard-dialog-item) size-point)
  (setf (vw self) (om-point-h size-point) (vh self) (om-point-v size-point))
  (when (interface-visible-p self) 
    (set-hint-table self (list :default-width (om-point-h size-point) :defalut-height (om-point-v size-point)
                               :external-min-width (om-point-h size-point) :external-min-height (om-point-v size-point)
                               :external-max-width (om-point-h size-point) :external-max-height (om-point-v size-point)
                               ))
    (setf (pinboard-pane-size self) (values (om-point-h size-point) (om-point-v size-point)))
    (om-set-view-position self (om-make-point (vx self) (vy self)))
    )
  (di-after-settings self)
  )

(defmethod om-create-callback ((self om-standard-dialog-item))
  (set-hint-table self (list :default-x (vx self) :default-y (vy self) :default-width (vw self) :defalut-height (vh self))))


;=== CONSTRUCTOR ===
(defun dialog-item-scrollbar-h (scroll)
  (or (equal scroll :h) (equal scroll t)))

(defun dialog-item-scrollbar-v (scroll)
  (or (equal scroll :v) (equal scroll t)))

(defun om-make-dialog-item (class position size text &rest attributes &key
                                  container font bg-color fg-color (enable t)
                                  (allow-returns nil) (checked-p nil) (default-button nil) (focus nil) range
                                  (scrollbars nil)
                                  (direction :horizontal) (tick-side :default) (radio-button-cluster 0)
                                  &allow-other-keys)
  (let ((di (apply 'make-instance 
                   (append (list class
                           :default-x (om-point-h position)
                           :default-y (om-point-v position)
                           :default-width (om-point-h size)
                           :external-min-width (om-point-h size)
                           :external-max-width (om-point-h size)
                           :visible-max-width nil
                           :default-height (om-point-v size)
                           :visible-max-height nil
                           ;:external-min-height (om-point-v size)
                           ;:external-max-height (om-point-v size)
                           :width nil :height nil
                           :text text
                           :font font
                           :enabled enable
                           :background (or bg-color :transparent)
                           :foreground fg-color
                           :selected checked-p
                           :allows-newline-p allow-returns
                           :default-p default-button
                           :items range
                           :start (or (first range) 0)
                           :end (or (second range) 100)
                           :orientation direction
                           :start-point tick-side
                         ;  :popup-callback #'om-dialog-item-action
                           :compact t
                           :internal-border 0
                           :visible-border :outline
                           :TITLE-ADJUST t
                           :compact t
                           :horizontal-scroll (dialog-item-scrollbar-h scrollbars)
                           :vertical-scroll (dialog-item-scrollbar-v scrollbars)
                           ;:button-group radio-button-cluster
                           ;:title text
                           :allow-other-keys t)
                   attributes))))
    (if (special-bg di) (om-set-bg-color di (special-bg di)))
    (setf (vx di) (om-point-h position)
          (vy di) (om-point-v position)
          (vw di) (om-point-h size) 
          (vh di) (om-point-v size))
    (when container  (om-add-subviews container di))
    (di-after-settings di)
    (when focus (capi::set-pane-focus di))
    di))

(defmethod special-bg (di) nil)
(defmethod di-after-settings (di) nil)


;;;========================== 
;;;DIALOG ITEMS CLASSES
;;;==========================  

;--------om-static-text

;(defclass om-static-text (om-standard-dialog-item title-pane) 
;  ((real-text :initform nil :initarg :real-text :accessor real-text)))

;(defmethod om-dialog-item-text ((self om-static-text))
;  (or (real-text self) (capi::title-pane-text self)))

;(defmethod om-set-dialog-item-text ((self om-static-text) text)
;  (setf (real-text self) text)
;  (if (interface-visible-p self)
;      (setf (capi::title-pane-text self) (make-wrapped-text (capi::wrap-text-for-pane self text)))      
;    (let ((p 0))
;      (setf (capi::simple-pane-font *record-view*) (or (capi::simple-pane-font self) (gp::make-font-description :family "Arial" :size 12)))
;      (setf p (position (om-width self) (gp::compute-char-extents *record-view* text) :test '<))
;      (setf (capi::title-pane-text self) (if p (make-wrapped-text (capi::wrap-text text (max 1 (- p 1)))) text))
;      )))


(defclass om-static-text (om-view) 
  ((text :initform nil :initarg :text :accessor text)
   (wrapped-text :initform nil :initarg :wrapped-text :accessor wrapped-text)
   (dx :initform 1 :initarg :dx :accessor dx)
   (dy :initform 1 :initarg :dy :accessor dy)
   (il :initform 1 :initarg :il :accessor il)
   (lineh :initform 5 :initarg :lineh :accessor lineh)
   (di-action :accessor di-action :initform nil :initarg :di-action)))

(defmethod om-dialog-item-text ((self om-static-text))
   (text self))

(defun text-wrap-pix (text-str font width-pix)
  (setf (capi::simple-pane-font *record-view*) font)
  (let ((p (position width-pix (gp::compute-char-extents *record-view* text-str) :test '<)))
    (if p (capi::wrap-text text-str (max 1 (- p 1))) (list text-str))))

(defmethod om-set-dialog-item-text ((self om-static-text) text)
  (setf (text self) text)
  (let ((p 0)
        (font (or (capi::simple-pane-font self) (gp::make-font-description :family "Arial" :size 12))))
    (setf (lineh self) (om-string-h font))
    (if (interface-visible-p self)
        (setf (wrapped-text self) (capi::wrap-text-for-pane self text))
      (setf (wrapped-text self) (text-wrap-pix text font (om-width self))))
    ))

(defmethod om-set-view-size ((self om-static-text) size-point)
  (setf (vw self) (om-point-h size-point) (vh self) (om-point-v size-point))
  (when (interface-visible-p self) 
    (set-hint-table self (list :default-width (om-point-h size-point) :defalut-height (om-point-v size-point)
                               :external-min-width (om-point-h size-point) :external-min-height (om-point-v size-point)
                               :external-max-width (om-point-h size-point) :external-max-height (om-point-v size-point)
                               ))
    (setf (pinboard-pane-size self) (values (om-point-h size-point) (om-point-v size-point))))
  (di-after-settings self))

(defmethod om-draw-contents-callback ((self om-static-text) x y w h)
  (loop for line in (or (wrapped-text self) (list (text self)))
        for posy = 0 then (+ posy 1) do
        (gp:draw-string self line (dx self) (round (+ (lineh self) (* posy (lineh self) (il self)))))
        )
  (when (equal (om-component-border self) :line)
    (gp::draw-rectangle self x y (- w 1) (- h 1))))

(defun make-wrapped-text (list)
  (concatenate 'string (car list)
               (reduce #'(lambda (res str) (concatenate 'string res (string #\Linefeed) str)) 
                       (cdr list) :initial-value "")))

(defmethod di-after-settings ((self om-static-text))
  (om-set-dialog-item-text self (om-dialog-item-text self)))

(defmethod om-enable-dialog-item ((self om-static-text) t-or-nil)
  (setf (capi::simple-pane-enabled self) t-or-nil))

(defmethod om-dialog-item-enabled ((self om-static-text)) 
  (capi::simple-pane-enabled self))



(defclass om-text-view (om-standard-dialog-item display-pane) ())

(defmethod om-dialog-item-text ((self om-text-view))
 (capi::display-pane-text self))

(defmethod om-set-dialog-item-text ((self om-text-view) text)
  (setf (capi::display-pane-text self) text))

(defmethod om-enable-dialog-item ((self om-static-text) t-or-nil)
  (setf (capi::simple-pane-enabled self) t-or-nil))

(defmethod om-dialog-item-enabled ((self om-static-text)) 
  (capi::simple-pane-enabled self))


;;; ACTIONS from om-standard-dialog-item

(defmethod om-dialog-item-action ((self om-static-text))  
  (when (di-action self)
    (funcall (di-action self) self)))

(defmethod om-set-dialog-item-action-function ((self om-static-text) action)
   (setf (di-action self) action))

(defmethod om-dialog-item-action-function ((self om-static-text))
   (di-action self))

(defmethod om-funcall-dialog-item-action ((self om-static-text))
    (when (di-action self)
      (funcall (di-action self) self)))


;--------om-editable-text

(defclass om-editable-text (om-standard-dialog-item text-input-pane) ()
  (:default-initargs :navigation-callback 'text-edit-special-action
                     :change-callback 'text-edit-changed-action))

;;; :allows-newline-p
;;; :compact
;;; :multi-line-p
;;; :editing-callback

(defmethod special-bg ((self om-editable-text)) *om-white-color*)

(defun text-edit-special-action (self action)
  (cond ((equal action :enter) (om-dialog-item-action self))
        ((equal action :return) (if (capi::text-input-allows-newline-p self)
                                    (let ((rec (capi::clipboard self)))
                                      (capi::set-clipboard self (string #\Newline))
                                      (capi::text-input-pane-paste self)
                                      (capi::set-clipboard self rec))
                                  (om-dialog-item-action self)))
        ))
 
(defun text-edit-changed-action (text self win position)
  (unless (string-equal text "")
    (om-view-key-handler self (elt text (max 0 (- position 1))))))


(defmethod om-dialog-item-text ((self om-editable-text))
 (capi::text-input-pane-text self))

(defmethod om-set-dialog-item-text ((self om-editable-text) text)
 (setf (capi::text-input-pane-text self) text))

(defun om-get-text-edit-size (init-size) 
    (om-add-points init-size (om-make-point 14 16)))

(defun om-get-text-edit-pos (init-pos)
  (om-subtract-points init-pos (om-make-point 2 14)))
      

(defmethod om-enable-dialog-item ((self om-editable-text) t-or-nil)
  (setf (capi::text-input-pane-enabled self) t-or-nil))

(defmethod om-dialog-item-enabled ((self om-editable-text)) 
  (capi::text-input-pane-enabled self))


(defmethod om-copy-command ((self om-editable-text))
  ;(capi::set-clipboard self (om-dialog-item-text self))
  (capi::text-input-pane-copy self))

(defmethod om-paste-command ((self om-editable-text))
  ;(om-set-dialog-item-text self (capi::clipboard self))
  (capi::text-input-pane-paste self))

(defmethod om-cut-command ((self om-editable-text))
  (capi::text-input-pane-cut self))




;--------om-text-edit-view

(defclass om-text-edit-view (om-standard-dialog-item multi-line-text-input-pane) ()
  (:default-initargs ;;; :navigation-callback 'text-edit-special-action 
   :change-callback 'text-edit-changed-action
    :external-min-height nil
    :external-max-height nil))
;;; navigation callback does not work


(defmethod special-bg ((self om-text-edit-view)) *om-white-color*)

(defmethod om-dialog-item-text ((self om-text-edit-view))
 (capi::text-input-pane-text self))

(defmethod om-set-dialog-item-text ((self om-text-edit-view) text)
 (setf (capi::text-input-pane-text self) text))

(defmethod om-enable-dialog-item ((self om-text-edit-view) t-or-nil)
  (setf (capi::text-input-pane-enabled self) t-or-nil))

(defmethod om-dialog-item-enabled ((self om-text-edit-view)) 
  (capi::text-input-pane-enabled self))


(defmethod om-copy-command ((self om-text-edit-view))
  ;(capi::set-clipboard self (om-dialog-item-text self))
  (capi::text-input-pane-copy self))

(defmethod om-paste-command ((self om-text-edit-view))
  ;(om-set-dialog-item-text self (capi::clipboard self))
  (capi::text-input-pane-paste self))

(defmethod om-cut-command ((self om-text-edit-view))
  (capi::text-input-pane-cut self))


;--------om-button

(defclass om-button (om-standard-dialog-item push-button) ())

(defmethod om-dialog-item-text ((self om-button))
 (capi::item-text self))

(defmethod om-set-dialog-item-text ((self om-button) text)
 (setf (capi::item-text self) text))

(defmethod om-enable-dialog-item ((self om-button) t-or-nil)
  (setf (capi::button-enabled self) t-or-nil))

(defmethod om-dialog-item-enabled ((self om-button)) 
  (capi::button-enabled self))

;--------om-check-box

(defclass om-check-box (om-standard-dialog-item check-button) ()
  (:default-initargs 
   :selection-callback 'om-dialog-item-action
   :retract-callback  'om-dialog-item-action))

(defmethod om-checked-p ((self om-check-box)) (button-selected self))

(defmethod om-set-check-box  ((self om-check-box) check?)
  (capi:apply-in-pane-process self 
                              #'(lambda () (setf (button-selected self) check?))))


;--------om-radio-button

(defclass om-radio-button (om-standard-dialog-item radio-button) ())

(defmethod om-radio-button-p ((self om-radio-button))  t)
(defmethod om-radio-button-p ((self t))  nil)

(defmethod om-dialog-item-action ((self om-radio-button))  
  (let ((container (element-parent self)))
    (when container
      (let ((elems (capi::pane-children container)))
        (loop for item in elems do
              (when (and (om-radio-button-p item) (not (equal item self)))
                (om-set-check-box item nil)))))
  (call-next-method)))

(defmethod om-checked-p ((self om-radio-button)) (button-selected self))

(defmethod om-set-check-box  ((self om-radio-button) check?)
  (setf (button-selected self) check?))



;--------om-item-list abstract
(defclass om-item-list (om-standard-dialog-item list-panel) ()
  (:default-initargs 
   :callback-type '(:collection)
   :selection-callback 'om-dialog-item-action
   :action-callback 'double-click-on-list
   :test-function 'string-equal))

(defmethod special-bg ((self om-item-list)) *om-white-color*)

(defun vector-col-to-list (v)
  (loop for i from 0 to (- (length v) 1) collect (elt v i)))

(defmethod om-set-item-list ((self om-item-list) names)
  (setf (capi::collection-items self) names))

(defmethod om-get-item-list ((self om-item-list))
  (vector-col-to-list (capi::collection-items self )))

(defmethod om-get-selected-item-index ((self om-item-list))
  (capi::choice-selection self))

(defmethod om-set-selected-item-index ((self om-item-list) ind)
  (setf (choice-selection self) ind))

(defmethod double-click-on-list ((self om-item-list)) nil)

;--------om-single-item-list  

(defclass om-single-item-list (om-item-list) ()
  (:default-initargs :interaction :single-selection))

(defmethod om-select-item-index ((self om-single-item-list) i)
 (setf (choice-selection self) i))

(defmethod om-unselect-item-index ((self om-single-item-list) i)
  (when (= (choice-selection self) i) (setf (choice-selection self) nil)))

(defmethod om-get-selected-item ((self om-single-item-list)) 
  (choice-selected-item self))

(defmethod om-set-selected-item ((self om-single-item-list) item) 
  (setf (choice-selected-item self) item))



;--------om-multi-item-list
 
(defclass om-multi-item-list (om-item-list) ()
 (:default-initargs 
   :interaction :extended-selection))

;;; :multiple-selection
(defmethod om-select-item-index ((self om-multi-item-list) i)
  (setf (choice-selection self) (union (choice-selection self) (if (listp i) i (list i)))))

(defmethod om-unselect-item-index ((self om-multi-item-list) i)
  (setf (choice-selection self) (remove i (choice-selection self))))

(defmethod om-get-selected-item ((self om-multi-item-list)) 
  (choice-selected-items self))

(defmethod om-set-selected-item ((self om-multi-item-list) items) 
  (setf (choice-selected-items self) (if (listp items) items (list items))))



;--------om-slider 

(defclass om-slider (om-standard-dialog-item slider) ()
  (:default-initargs 
   :callback 'om-slider-item-action
   :show-value-p  t))
 

(defmethod om-slider-value ((self om-slider))
  (range-slug-start self))

(defmethod om-set-slider-value ((self om-slider) value)
   (setf (range-slug-start self) value))

(defmethod om-slider-item-action ((self om-standard-dialog-item) value type)
  (when (di-action self)
    (funcall (di-action self) self)))



;--------om-pop-up-dialog-item

(defclass om-pop-up-dialog-item (om-standard-dialog-item option-pane) 
  ((value :initform nil :initarg :value :accessor value))
  (:default-initargs 
   :callback-type '(:collection)
   :selection-callback 'om-dialog-item-action))   

(defmethod initialize-instance :after ((self om-pop-up-dialog-item) &rest l)
  (when (value self)
    (let ((pos (position (value self) (vector-col-to-list (capi::collection-items self )) :test 'string-equal)))
      (when pos
        (setf (choice-selection self) pos)))))

(defmethod om-dialog-item-action ((self om-pop-up-dialog-item))  
  (when (di-action self)
    (funcall (di-action self) self)))

(defmethod set-dialog-item-action-function ((self om-pop-up-dialog-item) f)
  (declare (ignore f))
  (when f
    (loop for item in (menu-items self) do
          (set-menu-item-action-function item #'(lambda () (funcall f self))))))

(defmethod om-get-item-list ((self om-pop-up-dialog-item)) 
  (vector-col-to-list (collection-items self)))

(defmethod om-enable-dialog-item ((self om-pop-up-dialog-item) t-or-nil)
  (setf (option-pane-enabled self) t-or-nil))

(defmethod om-dialog-item-enabled ((self om-pop-up-dialog-item))
  (option-pane-enabled self))

;;; !!!!
(defmethod om-get-selected-item ((self om-pop-up-dialog-item))
  (nth (choice-selection self) (om-get-item-list self)))

(defmethod om-get-selected-item-index ((self om-pop-up-dialog-item))
  (choice-selection self))

(defmethod om-set-selected-item-index ((self om-pop-up-dialog-item) str)
  (let ((pos (position str (om-get-item-list self) :test 'string-equal)))
    (when pos (setf (choice-selection self) pos))))


(defmethod om-select-item-index ((self om-pop-up-dialog-item) index)
 (setf (choice-selection self) index))
 



