;;-----------------------------------------------------------------------------
;; Moscou
;;
;; Basic OpenSoundControl (OSC) for OpenMusic. Based on early work done by
;; Carlos Agon.
;;
;; Author : Carl Seleborg
;; Date : 2004
;; Version : 1.0
;; Copyright Ircam 2004
;;-----------------------------------------------------------------------------

(in-package :oa)



;;-----------------------------------------------------------------------------
;; Entry points to the OSC framework. These are mostly the same functions
;; as can be found in the osc_client.c/h files distributed by CNMAT.
;;
;; List of predefined types to use with the ppc-ff-call function ...
; CCL::*MACTYPE-NAME=>PPC-FF-CALL-TYPE*
;;-----------------------------------------------------------------------------



(defvar *osc-framework* nil)

(defun osc-framework ()
    (or *osc-framework*
        (setq *osc-framework*
              (ccl::load-framework-bundle "OSC.framework")))
    (oa::open-opentransport))
  
(defun close-osc-framework ()
  (setq *osc-framework* nil)
  (oa::free-opentransport))
  
;(eval-when (:load-toplevel :execute)
  (ccl::def-load-pointers startup-osc () (osc-framework))
;)

(setf CCL:*lisp-cleanup-functions* 
      (append (list #'close-osc-framework) CCL:*lisp-cleanup-functions*))







(cl:defpackage "OSC"
  (:nicknames "OSC")
   (:use common-lisp ccl om-api))

(in-package :osc)


(ccl::defrecord OSCTT 
  (seconds :unsigned-long)
  (fraction :unsigned-long))

;(defmacro OSCTT_CurrentTime ()
;  `(ccl::ppc-ff-call (ccl::get-fun-addr "OSCTT_CurrentTime" oa::*osc-framework*) :address))

;(defun OSCTT_CurrentTime ()
;  (let ((retfloat (ccl::ppc-ff-call (ccl::get-fun-addr "OSCTT_CurrentTime" oa::*osc-framework*) '(:record 'OSCTT))))
;    (print retfloat)
;    )
;)



(defmacro OSC_initBuffer (ref size ptr)
    `(ccl::ppc-ff-call (ccl::get-fun-addr "OSC_initBuffer" oa::*osc-framework*)
                       :address ,ref :signed-halfword ,size :address ,ptr :signed-halfword))

(defmacro OSC_resetBuffer (ref)
    `(ccl::ppc-ff-call (ccl::get-fun-addr "OSC_resetBuffer" oa::*osc-framework*)
                       :address ,ref :signed-halfword))

(defmacro OSC_openBundle (ref tt_ipart tt_dpart)
  `(ccl::ppc-ff-call (ccl::get-fun-addr "OSC_openBundle" oa::*osc-framework*)
                     :address ,ref :unsigned-fullword ,tt_ipart :unsigned-fullword ,tt_dpart :signed-fullword))

(defmacro OSC_closeBundle (ref )
  `(ccl::ppc-ff-call (ccl::get-fun-addr "OSC_closeBundle" oa::*osc-framework*)
                     :address ,ref :signed-halfword))

(defmacro OSC_closeAllBundles (ref )
  `(ccl::ppc-ff-call (ccl::get-fun-addr "OSC_closeAllBundles" oa::*osc-framework*)
                     :address ,ref :signed-halfword))

(defmacro OSC_writeAddressAndTypes (ref name types)
  `(ccl::ppc-ff-call (ccl::get-fun-addr "OSC_writeAddressAndTypes" oa::*osc-framework*)
                     :address ,ref :address ,name :address ,types :signed-fullword))

(defmacro OSC_writeFloatArg (ref arg)
  `(ccl::ppc-ff-call (ccl::get-fun-addr "OSC_writeFloatArg" oa::*osc-framework*)
                     :address ,ref :double-float ,arg :signed-halfword))

(defmacro OSC_writeFloatArgs (ref numFloats args)
  `(ccl::ppc-ff-call (ccl::get-fun-addr "OSC_writeFloatArgs" oa::*osc-framework*)
                     :address ,ref :signed-halfword ,numFloats :address ,args :signed-halfword))

(defmacro OSC_writeIntArg (ref arg)
  `(ccl::ppc-ff-call (ccl::get-fun-addr "OSC_writeIntArg" oa::*osc-framework*)
                     :address ,ref :signed-halfword ,arg :signed-halfword))

(defmacro OSC_writeStringArg (ref arg)
  `(ccl::ppc-ff-call (ccl::get-fun-addr "OSC_writeStringArg" oa::*osc-framework*)
                     :address ,ref :address ,arg :signed-halfword))

(defmacro OSC_getPacket (ref)
  `(ccl::ppc-ff-call (ccl::get-fun-addr "OSC_getPacket" oa::*osc-framework*)
                     :address ,ref :address))

(defmacro OSC_packetSize (ref)
  `(ccl::ppc-ff-call (ccl::get-fun-addr "OSC_packetSize" oa::*osc-framework*)
                     :address ,ref :signed-halfword))

(defun OSC_assert (err str)
  (assert(= err 0)  nil
         (format t "openSoundControl error n.~S ~A ~S ~%" err str 
                 (ccl::find-opentransport-errno-string err))))





