;;===========================================================================
;OM API 
;Multiplatform API for OpenMusic
;Macintosh version (Digitool Macintosh Common Lisp - MCL)
;
;Copyright (C) 2004 IRCAM-Centre Georges Pompidou, Paris, France.
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
;
;Authors: Jean Bresson and Augusto Agon
;;===========================================================================

;;===========================================================================
;DocFile
;  SDIF FUNCTIONS
;;===========================================================================

(in-package :om-api)


(export '(
              sdif-init
              sdif-kill
              sdif-open-file
              sdif-null-ptr-p
              sdif-close-file
              sdif-check-file
              sdif-to-text
              sdif-get-pos
              sdif-set-pos
              sdif-get-signature
              sdif-signature-to-string
              sdif-string-to-signature
              sdif-matrixtype-get-column-name
              sdif-new-value-in-table
              sdif-put-ids-table
              sdif-Matrix-Insert-Tail-Def
              sdif-Frame-Insert-Component
              Sdif-String-Append
              ) :om-api)

;;; INIT/CLOSE SDIF

(defun sdif-null-ptr-p (sdifptr)
   (cffi:null-pointer-p sdifptr))

(defun sdif-init (string)
  (cffi:with-foreign-string (str string)
    (sdif::SdifGenInit str))
   (setf *sdiflib* t))



(defun sdif-kill ()
   (sdif::SdifGenKill)
   (setf *sdiflib* nil))



;;; auto sdif init... enlever et mettre dans om
(defvar *sdiflib* nil)


(defun sdif-autoinit ()
  (sdif::init-sdif-framework)
  (when sdif::*sdif-framework*
    (sdif-init "")))

(om-add-init-func 'sdif-autoinit)


;;; SDIF FILES
(defun get-file-mode (symb)
   (cond ((equal symb :eUnknownFileMode) 0)
            ((equal symb :eWriteFile) 1)
            ((equal symb :eReadFile) 2)
            ((equal symb :eReadWriteFile) 3)
            ((equal symb :ePredefinedTypes) 4)
            (t 0)))

;;; mode = un entier ou un symbole 'eReadFile, 'eWriteFile, ...
(defun sdif-open-file (filename mode)
   (let ((open-mode (if (integerp mode) mode (get-file-mode mode)))
         rep)
     (cffi:with-foreign-string (str filename)
       (setf rep (sdif::SdifFOpen str open-mode)))
     rep))

(defun sdif-close-file (file)
  (sdif::SdifFClose file))

(defun sdif-check-file (filename)
  (let (rep)
    (cffi:with-foreign-string (str filename)
      (setf rep (sdif::SdifCheckFileFormat str)))
    (not (zerop rep))))


(defun sdif-to-text (sdiff outfilename)
   (when (and sdiff outfilename)
     (cffi:with-foreign-string (str (oa::om-path2cmdpath outfilename))
       (sdif::SdifToText sdiff str))
     ))


(defun sdif-get-pos (ptr)
  (let ((thelong (cffi::%foreign-alloc 8))
        rep)
    (sdif::SdiffGetPos ptr thelong)
    (setf rep (cffi::mem-ref thelong :unsigned-long))
    (cffi:foreign-free thelong)
    rep))

(defun sdif-set-pos (ptr thelong)
  (let ((longptr (cffi::%foreign-alloc 8)))
    (cffi::mem-set thelong longptr :unsigned-long)
    (sdif::SdiffSetPos ptr longptr)
    (cffi::foreign-free longptr)
    t))


;;; FUNCTIONS
(defun sdif-get-signature (sdiff)
   (let ((nbcharread (cffi::%foreign-alloc 4))
         rep)
     (cffi::mem-set 0 sdiff :long)
     (setf rep (sdif::SdifFGetSignature sdiff nbcharread))
     (cffi::foreign-free nbcharread)
     rep))


;;;; ?? c'est bon?
(defun sdif-signature-to-string (sdifsignature)
   (cffi::foreign-string-to-lisp (sdif::SdifSignatureToString sdifsignature)))

(defun sdif-string-to-signature (str)
   ;(with-cstrs ((strsign str))
   ;  (sdif::SdifStringToSignature strsign))
   (read-from-string (concatenate 'string ":" str))
   )

(defun sdif-matrixtype-get-column-name (mtype num) 
  (cffi::foreign-string-to-lisp (sdif::SdifMatrixTypeGetColumnName mtype num)))

(defun sdif-new-value-in-table (table name value)
   (cffi:with-foreign-string (strname name)
       (cffi:with-foreign-string  (strvalue value)
     (sdif::SdifNameValuesLPutCurrNVT table strname strvalue ))))

(defun sdif-put-ids-table (table id name tree)
  (cffi:with-foreign-string (strname name)
       (cffi:with-foreign-string  (strtree tree)
         (sdif::SdifStreamIDTablePutSID table id strname strtree))))

(defun sdif-Matrix-Insert-Tail-Def (matrixtype name)
  (cffi:with-foreign-string (strname name)
    (sdif::SdifMatrixTypeInsertTailColumnDef matrixtype strname )))

(defun sdif-Frame-Insert-Component (frametype sign name)
  (cffi:with-foreign-string (strname name)
    (sdif::SdifFrameTypePutComponent frametype sign strname )))


(defun Sdif-String-Append (ptr str)
   (let (rep)
     (cffi:with-foreign-string (strp str)
       (setf rep (sdif::SdifStringAppend ptr strp)))
    rep))


;;;============================
;;; TESTS

;(Sdif-Init "")
;(Sdif-Kill)
;(setf filepath (om::om-path2cmdpath (om::om-choose-file-dialog))) 
;(setf filepath (namestring filepath))
;(setf sdiffile (Sdif-Open-file filepath :eReadFile))
;(sdif-close-file sdiffile)
;(sdif-check-file filepath)
;(sdif::sdifFreadgeneralheader sdiffile)
;(sdif::sdifFReadAllASCIIChunks sdiffile)  
;(sdif::sdifFReadFrameHeader sdiffile)
;(sdif::SdifFSkipFrameData sdiffile)
;(setf sign (sdif::SdifFCurrSignature sdiffile))
;(setf sign (sdif::SdifFCurrMatrixSignature sdiffile))
;(setf sign (sdif::SdifFCurrFrameSignature sdiffile))
;(sdif-Signature-to-string sign)
;(setf bytesread (ff::make-foreign-pointer :size 2 :type :int))
;(setf posptr (ff::make-foreign-pointer :size 8 :type :int))
;(sdif::SdifFGetSignature sdiffile bytesread)
;(sdif::sdiffcurrtime sdiffile)
;(sdif::sdiffcurrdatatype sdiffile)
;(sdif::sdiffcurrID sdiffile)
;(sdif::sdiffcurrnbmatrix sdiffile)
;(sdif::sdiffreadmatrixheader sdiffile)
;(sdif::sdiffcurrnbcol sdiffile)
;(sdif::sdiffcurrnbrow sdiffile)
;(sdif::sdiffskipmatrixdata sdiffile) 
;(sdif::sdiffgetsignature sdiffile bytesread) 
;(sdif::sdiffgetpos sdiffile posptr)
;(sys::memref-int (ff::foreign-pointer-address posptr) 0 0 :unsigned-long)

;(defun signature (sdiff ptr)
;   (print (sdif::sdifSignaturetostring (sdif::SdifFCurrSignature sdiff)))
;   (sdif::sdifFReadFrameHeader sdiff)
;   (sdif::sdifFSkipFrameData sdiff)
;   (sdif::SdifFGetSignature sdiffile ptr))