;;===========================================================================
;OM API 
;Multiplatform API for OpenMusic
;Macintosh version (Digitool Macintosh Common Lisp - MCL)
;
;Copyright (C) 2004 IRCAM-Centre Georges Pompidou, Paris, France.
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
;
;Authors: Jean Bresson and Augusto Agon
;;===========================================================================

;;===========================================================================
;DocFile
; SDIF Library binding
;;===========================================================================


(in-package :cl-user)

(defpackage "SDIF-PACKAGE"
  (:nicknames "SDIF")
   (:use common-lisp))

(in-package :sdif)

;;;==============================
;;; CHARGEMENT

(defvar *sdif-framework* nil)


(defvar *sdif-pathname* "/Library/Frameworks/SDIF.framework/SDIF")

(defun init-sdif-framework ()
  (setf *sdif-framework* (if (probe-file  *sdif-pathname*)
                             (progn (cffi:load-foreign-library  *sdif-pathname*)
                               t))))

;(init-sdif-framework)

(cffi:define-foreign-type sdif-name () ':pointer)
;;;============================================
;;; SDIF GENERAL

(cffi:defcfun  ("SdifGenInit" SdifGenInit) :void (PredefinedTypesFile sdif-name) )

(cffi:defcfun  ("SdifGenKill" SdifGenKill) :void )

(cffi:defcfun  ("SdifPrintVersion" SdifPrintVersion) :void )

(cffi:defcfun  ("SdifenableErrorOutput" SdifenableErrorOutput) :void )

(cffi:defcfun  ("SdifDisableErrorOutput" SdifDisableErrorOutput) :void )


;;;============================================
;;; SDIF FILE

(cffi:defcfun  ( "SdifFOpen" SdifFOpen) :pointer  (name sdif-name) (mode :unsigned-int ))

(cffi:defcfun  ( "SdifFClose" SdifFClose) :void (sdifF :pointer))

(cffi:defcfun  ( "SdifCheckFileFormat" SdifCheckFileFormat) :int  (name sdif-name))

(cffi:defcfun  ( "SdifToText" SdifToText) :unsigned-int (sdifF :pointer) (textStreamName sdif-name))

(cffi:defcfun  ( "SdifFGetPos" SdifFGetPos) :int  (pointer :pointer) (pos :pointer))

(cffi:defcfun  ( "SdifFSetPos" SdifFSetPos) :int ( pointer :pointer) (pos :pointer))

;;;============================================
;;; READ SDIF

(cffi:defcfun  ( "SdifFGetSignature" SdifFGetSignature) :int  ( sdifF :pointer) (nbCharRead :pointer) )

(cffi:defcfun  ("SdifFReadGeneralHeader" SdifFReadGeneralHeader) :unsigned-int (sdifF :pointer))

(cffi:defcfun  ( "SdifFReadAllASCIIChunks" SdifFReadAllASCIIChunks) :unsigned-int  (sdifF :pointer) )

(cffi:defcfun  ( "SdifFReadMatrixHeader" SdifFReadMatrixHeader) :unsigned-int  (sdifF :pointer) )

(cffi:defcfun  ( "SdifFReadOneRow" SdifFReadOneRow) :unsigned-int  (sdifF :pointer) )

;;; new
(cffi:defcfun  ( "SdifFSkipOneRow" SdifFSkipOneRow) :unsigned-int  (sdifF :pointer) )

(cffi:defcfun  ( "SdifFReadFrameHeader" SdifFReadFrameHeader) :unsigned-int  (sdifF :pointer) )


(cffi:defcfun  ( "SdifFSkipMatrix" SdifFSkipMatrix) :unsigned-int  (sdifF :pointer) )

;;; new F dans le nom...
(cffi:defcfun  ( "SdifFSkipMatrixData" SdifFSkipMatrixData) :unsigned-int  (sdifF :pointer) )

;;; new F dans le nom...
(cffi:defcfun  ("SdifFSkipFrameData" SdifFSkipFrameData) :unsigned-int  (sdifF :pointer) )

(cffi:defcfun  ( "SdifFReadPadding" SdifFReadPadding) :unsigned-int  (sdifF :pointer) (Padding :unsigned-int))

;;; new 
(cffi:defcfun  ("SdifFSkip" SdifFSkip) :unsigned-int  (sdifF :pointer) (bytes :unsigned-int ))

(cffi:defcfun  ( "SdifFReadAndIgnore" SdifFReadAndIgnore) :unsigned-int  (sdifF :pointer) (bytes :unsigned-int ))


;;;============================================
;;; SDIF CURR
;;; remplac unsigned-fullword par address pour les renvois de signature.. a tester

;;; new
(cffi:defcfun  ( "SdifSignatureToString" SdifSignatureToString) :pointer (Signature :pointer))


(cffi:defcfun  ( "SdifFCurrSignature" SdifFCurrSignature)  :pointer (SdifF :pointer))

(cffi:defcfun  ( "SdifFCleanCurrSignature" SdifFCleanCurrSignature)  :pointer (SdifF :pointer))

(cffi:defcfun  ( "SdifFCurrFrameSignature" SdifFCurrFrameSignature)  :pointer (SdifF :pointer))

(cffi:defcfun  ( "SdifFCurrMatrixSignature" SdifFCurrMatrixSignature)  :pointer (SdifF :pointer))

(cffi:defcfun  ( "SdifFCurrTime" SdifFCurrTime) :double (SdifF :pointer))

(cffi:defcfun  ( "SdifFCurrID" SdifFCurrID) :unsigned-int  (SdifF :pointer))

;;; !!! en vrai a retourne pas un int mais SdifDataTypeET ...
(cffi:defcfun  ( "SdifFCurrDataType" SdifFCurrDataType)  :unsigned-int  (SdifF :pointer))

(cffi:defcfun  ( "SdifFCurrNbMatrix" SdifFCurrNbMatrix)  :unsigned-int  (SdifF :pointer))

(cffi:defcfun  ( "SdifFCurrNbCol" SdifFCurrNbCol) :unsigned-int  (SdifF :pointer))

(cffi:defcfun  ( "SdifFCurrNbRow" SdifFCurrNbRow) :unsigned-int  (SdifF :pointer))

(cffi:defcfun  ( "SdifFCurrOneRowCol" SdifFCurrOneRowCol) :double (SdifF :pointer) (numcol :unsigned-int))


;;;============================================
;;; CREATE SDIF

(cffi:defcfun  ( "SdifFSetCurrFrameHeader" SdifFSetCurrFrameHeader) :pointer (SdifF :pointer) (Signature :unsigned-int) (Size :unsigned-int)
                       (NbMatrix :unsigned-int) (NumID :unsigned-int) (Time :double))

(cffi:defcfun  ( "SdifFSetCurrMatrixHeader" SdifFSetCurrMatrixHeader) :pointer (SdifF :pointer) (Signature :unsigned-int) 
               (DataType :unsigned-int) 
               (NbRow :unsigned-int))

(cffi:defcfun  ( "SdifFSetCurrOneRow" SdifFSetCurrOneRow) :pointer (SdifF :pointer) (values :pointer))

(cffi:defcfun  ( "SdifFSetCurrOneRowCol" SdifFSetCurrOneRowCol) :pointer (SdifF :pointer) (numcol :unsigned-int) (value :double))

;;;============================================
;;; WRITE SDIF

(cffi:defcfun  ( "SdifFWriteGeneralHeader" SdifFWriteGeneralHeader) :unsigned-int  (sdifF :pointer) )

(cffi:defcfun  ( "SdifFWriteAllASCIIChunks" SdifFWriteAllASCIIChunks) :unsigned-int  (sdifF :pointer) )

(cffi:defcfun  ( "SdifFWriteFrameHeader" SdifFWriteFrameHeader) :unsigned-int  (sdifF :pointer) )

(cffi:defcfun  ( "SdifUpdateChunkSize" SdifUpdateChunkSize) :void (sdifF :pointer) (chunk-size :unsigned-int))

;;; new
(cffi:defcfun  ( "SdifUpdateFrameHeader" SdifUpdateFrameHeader) :int
                       (sdifF :pointer)  (chunksize :unsigned-int) (nummat :int)) 

(cffi:defcfun  ( "SdifFWriteMatrixHeader" SdifFWriteMatrixHeader) :unsigned-int  (SdifF :pointer))

(cffi:defcfun  ( "SdifFWriteOneRow" SdifFWriteOneRow) :unsigned-int  (SdifF :pointer))

(cffi:defcfun  ( "SdifFWriteMatrixData" SdifFWriteMatrixData) :unsigned-int  (SdifF :pointer) (data :pointer))

(cffi:defcfun  ( "SdifFWriteMatrix" SdifFWriteMatrix) :unsigned-int (SdifF :pointer) (Signature :unsigned-int) (DataType :unsigned-int)
               (NbRow :unsigned-int) (NbCol :unsigned-int) (data :pointer))

(cffi:defcfun  ( "SdifFWritePadding" SdifFWritePadding) :unsigned-int  (sdifF :pointer) (padding :unsigned-int) )


;;;=================================
;;; ID TABLE

(cffi:defcfun  ( "SdifStreamIDTablePutSID" SdifStreamIDTablePutSID) :pointer (SDITable :pointer) (NumID :unsigned-int) (Source sdif-name) (TreeWay sdif-name))

(cffi:defcfun  ( "SdifFStreamIDTable" SdifFStreamIDTable) :pointer (file :pointer) )


;;;=================================
;;; NVT
(cffi:defcfun  ( "SdifCreateNameValuesL" SdifCreateNameValuesL) :pointer  (hashsize :unsigned-int))

;;; new
(cffi:defcfun  ( "SdifKillNameValuesL" SdifKillNameValuesL) :void (NVL :pointer))

(cffi:defcfun  ("SdifNameValuesLNewTable" SdifNameValuesLNewTable) :pointer (SdifNameValuesLT :pointer) (StreamID :unsigned-int))

(cffi:defcfun  ( "SdifFNameValueList" SdifFNameValueList) :pointer (file :pointer) )


;;; new
(cffi:defcfun  ( "SdifNameValuesLSetCurrNVT" SdifNameValuesLSetCurrNVT) :pointer  (NVLT :pointer) (numNVT :unsigned-int))

(cffi:defcfun  ("SdifNameValuesLPutCurrNVT" SdifNameValuesLPutCurrNVT)  :pointer  (SdifNameValuesLT :pointer) (name sdif-name) (value sdif-name))


;;;=================================
;;;TYPES TABLE



(cffi:defcfun  ("SdifFGetMatrixTypesTable" SdifFGetMatrixTypesTable)  :pointer  (file :pointer))

(cffi:defcfun  ( "SdifFGetFrameTypesTable" SdifFGetFrameTypesTable) :pointer  (file :pointer))

(cffi:defcfun  ( "SdifCreateMatrixType" SdifCreateMatrixType) :pointer (Signature :unsigned-int) (SdifMatrixTypeT :pointer))

(cffi:defcfun  ( "SdifPutMatrixType" SdifPutMatrixType) :void (mtable :pointer) (SdifMatrixTypeT :pointer))

(cffi:defcfun  ( "SdifMatrixTypeInsertTailColumnDef" SdifMatrixTypeInsertTailColumnDef) :pointer (matrix :pointer) (str sdif-name))

(cffi:defcfun  ( "SdifCreateFrameType" SdifCreateFrameType) :pointer (Signature :unsigned-int) (SdifMatrixTypeT :pointer))

(cffi:defcfun  ("SdifPutFrameType" SdifPutFrameType) :void (ftable :pointer) (SdifFrameTypeT :pointer))
(cffi:defcfun  ("SdifFrameTypePutComponent" SdifFrameTypePutComponent) :pointer (frame :pointer) (sign :unsigned-int) (str sdif-name))

;; new
(cffi:defcfun  ( "SdifMatrixTypeGetColumnName" SdifMatrixTypeGetColumnName) :pointer (MatrixType :pointer) (num :int))

;;; new
(cffi:defcfun  ( "SdifTestMatrixType" SdifTestMatrixType) :pointer (sdifF :pointer) (signature :unsigned-int))

;;; new
(cffi:defcfun  ("SdifTestFrameType" SdifTestFrameType) :pointer (sdifF :pointer) (signature :unsigned-int))

;;;=================================
;;; STRING

(cffi:defcfun  ( "SdifStringNew" SdifStringNew) :pointer)

(cffi:defcfun  ( "SdifStringFree" SdifStringFree) :void (str :pointer))

(cffi:defcfun  ("SdifStringAppend" SdifStringAppend) :int (str :pointer) (str1 :pointer))

(cffi:defcfun  ( "SdifStringGetC" SdifStringGetC) :int (str :pointer))

(cffi:defcfun  ( "SdifFGetAllTypefromSdifString" SdifFGetAllTypefromSdifString) :unsigned-int (file :pointer) (str :pointer))
  

;;; new
(cffi:defcfun  ("SdifStringToSignature" SdifStringToSignature) :pointer (str :pointer))


