;;===========================================================================
;OM API 
;Multiplatform API for OpenMusic
;Macintosh version (Digitool Macintosh Common Lisp - MCL)
;
;Copyright (C) 2004 IRCAM-Centre Georges Pompidou, Paris, France.
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
;
;Authors: Augusto Agon and Jean Bresson
;;===========================================================================

;;===========================================================================
;DocFile
; PICTURES, ICONS, CURSORS
;;===========================================================================


(in-package :om-api)

;;;==========
;;; export :
;;;==========

(export '(
                *om-icon-folder*
                *om-pict-folder*
                *om-cursor-folder*
                *om-icon-type*
                *om-pict-type*
                *om-cursor-type*
                om-set-default-resource-folder
                
                om-pict-p
                om-load-pixmap
                om-store-picture
                om-get-picture-loaded
                om-kill-picture
                om-pict-width
                om-pict-height
                om-get-picture-size
                om-draw-picture
                om-record-pict
                
                om-load-icon
                om-get-all-icons-id
                om-draw-icon
                
                om-make-cursor
                om-view-cursor
                om-with-cursor
                *om-wait-cursor*
                *om-arrow-cursor*
                *om-verti-size-cursor* 
                *om-horiz-size-cursor*  
                *om-resize-cursor* 
                *om-i-beam-cursor*   
                *om-cross-cursor* 
                *om-hand-cursor*
                
                om-with-open-resource

                om-draw-picture-part
                om-save-as-bmp

                ; *record-view*

                ) :om-api)
;;;================


(defvar *om-icon-folder* nil)
(defvar *om-pict-folder* nil)
(defvar *om-cursor-folder* nil)

(defvar *om-icon-type* "tif")
(defvar *om-pict-type* "tif")
(defvar *om-cursor-type* "tif")


(defun om-set-default-resource-folder (resource-type path)
   (case resource-type
      (:pict (setf *om-pict-folder* path))
      (:icon (setf *om-icon-folder* path))
      (:curs (setf *om-cursor-folder* path))
      ))

(defvar *pict-win* nil)
(defun init-def-rsrc ()
   (om-set-default-resource-folder :pict (om::OMRoot "resources;pict;"))
   (om-set-default-resource-folder :icon (om::OMRoot "resources;icon;"))
   (om-set-default-resource-folder :curs (om::OMRoot "resources;curs;"))
  ; (setf *pict-win* (display (om-make-window 'om-window :position (om-make-point 0 0) :size (om-make-point 1000 1000) :window-show nil)))
   )

(om-add-init-func 'init-def-rsrc)



(defun om-pict-p (pathname)
   (let ((type (pathname-type pathname)))
      (string-equal *om-pict-type* type) ))

;;; returns path for icon number in directory folder 
(defun om-icon-path (icon folder)
  (make-pathname :directory (pathname-directory folder) :name (format nil "~D" icon) :type *om-icon-type*))

(defun icon-string-name (icon)
  (format nil "~D" icon))

;;; tests if a kernel resource exists for icon number
(defun om-probe-icon-resource (icon &optional folder)
  (probe-file (om-icon-path icon (if folder folder *om-icon-folder*))))

; loads creates a system handle for the icon number ID if exists
; and returns the icon (icon can be freed with kill-picture)
(defun om-load-icon (Id &optional folder)
  (let ((handle nil)
        (icnfolder (if folder folder *om-icon-folder*)))
  (when (om-probe-icon-resource id icnfolder)
      (setf handle (om-load-pixmap (icon-string-name id) *om-icon-type* icnfolder t)))
    handle))

; returns the list of all icon numbers in the kernel resources
(defun om-get-all-icons-id (&optional path)
  (let ((iconspath (if path path *om-icon-folder*))
        (icn-list nil))
    (loop for iconfile in (om-directory iconspath) do
          (unless (equal "" (pathname-name iconfile))
            (let ((icn (read-from-string (pathname-name iconfile))))
              (if (integerp icn) (push icn icn-list)))))
    (reverse icn-list)))


;;;;; ==

(defun true-load-pixmap (path)
  (gp::read-external-image path)); :transparent-c


; load a pixmap and set a mask if mask-p and returns the pixmap (without store)
; name = string    -    folder = pathname
(defun om-load-pixmap (name type &optional folder (mask-p nil))
  (let* ((path (if folder folder *om-pict-folder*))
         (file (make-pathname :directory (pathname-directory path) :name name :type type))
         (pixmap nil))
    (setf pixmap (if (probe-file file)  (true-load-pixmap file)))
    pixmap))



(defvar *om-pict-table* nil)

(defun find-pixmap (name)
  (let ((pos (position name *om-pict-table* :key 'car :test 'string-equal)))
    (when pos (second (nth pos *om-pict-table*)))))

(defun push-pixmap (name hdl)
  (push (list name hdl) *om-pict-table*))

(defun remove-pixmap (pict)
  (let ((pos (position pict *om-pict-table*  :test 'equal)))
    (when pos (setf *om-pict-table* (remove (nth pos *om-pict-table*) *om-pict-table* :test 'equal)))))

;;; POSSIBLE AUSSI DANS CAPI AVEC gp:register-image-translation et gp:load-image

; name = symbol avec prefixe   
(defun om-store-picture (pict name)
   (unless (find-pixmap name)
      (push-pixmap name pict)))

; name = symbol avec prefixe
(defun om-get-picture-loaded (name)
   (find-pixmap name))    

; kills a pixmap handler (created with om-load-icon or om-load-picture)
(defun om-kill-picture (picture)
  t)
;;; capi::free-image ??


; draw a simple pict with a mask if it has one

(defmethod om-draw-picture (view pict &optional (topleft (om-make-point 0 0)) size)
 (when pict
   (let* ((port (om-get-view view))
         (image (gp::load-image port pict)))
    (if size
      (gp::draw-image port image (om-point-h topleft) (om-point-v topleft) 
                      :from-width (gp::image-width image) :from-height  (gp:image-height image) 
                      :to-width  (om-point-h size) :to-height (om-point-v size))
      (gp::draw-image port image (om-point-h topleft) (om-point-v topleft))))))

(defmethod om-get-picture-size (picture)
 (let ((image (gp::load-image *record-view* picture))
       rep)
   (setf rep (om-make-point (gp::image-width image) (gp::image-height image)))
   (gp::free-image *record-view* image)
   rep))

(defmethod om-pict-width (picture)
  (let ((image (gp::load-image *record-view* picture))
        rep)
    (setf rep (gp::image-width image))
    (gp::free-image *record-view* image)
    rep))

(defmethod om-pict-height (picture)
  (let ((image (gp::load-image *record-view* picture))
        rep)
    (setf rep (gp::image-height image))
    (gp::free-image *record-view* image)
    rep))

; for passing the picture from one object to another
(defun om-copy-pict-handler (picture)
   picture)

;=======drawing a part of picture


(defun om-draw-picture-part (view pict srctopleft srcsize desttopleft destsize)
  (let* ((port (om-get-view view))
         (image (gp::load-image port pict))) 
    (when (and image port)
      (gp::draw-image port image (om-point-h desttopleft) (om-point-v desttopleft) 
                      :to-width  (om-point-h destsize)  :to-height (om-point-v destsize)
                      :from-x (om-point-h srctopleft) :from-y (om-point-v srctopleft) 
                      :from-width (om-point-h srcsize)  :from-height  (om-point-v srcsize))
      )))

;===========


; draw an icon pict with a special drawing if selected = t

(defun om-draw-icon (pict view topleft size  &optional selected)
  (when pict
    (let ((image (gp::load-image view pict)))
      (if size
          (gp::draw-image view image (om-point-h topleft) (om-point-v topleft) 
                          :from-width (gp::image-width image) :from-height (gp:image-height image) 
                          :to-width (om-point-h size) :to-height (om-point-v size))
        (gp::draw-image view image (om-point-h topleft) (om-point-v topleft)))
      
      (when selected 
          (gp:draw-rectangle view (om-point-h topleft) (om-point-v topleft) (om-point-h size) (om-point-v size) 
                             :filled t :foreground (color::make-rgb (color::color-red *om-select-color*)
                                                                      (color::color-green *om-select-color*)
                                                                      (color::color-blue *om-select-color*)
                                                                      0.2))
        ;;; crer un PIXMAP (depth=1) pour dessiner un masque transparent...
        ;(gp::with-graphics-state (view :foreground :red)
        ;(gp::draw-image view image (om-point-h topleft) (om-point-v topleft) :global-alpha 0.5) )
        ))))




; executes body drawing on an invisible pict which is returned

(defclass internal-picture ()
  ((themetafile :initform nil :initarg :themetafile :accessor themetafile)
   (size-x :initform 0 :initarg :size-x :accessor size-x)
   (size-y :initform 0 :initarg :size-y :accessor size-y)))

(defmethod om-get-view ((self CAPI::METAFILE-PORT)) self)

(defmethod om-draw-picture (view (pict internal-picture) &optional (topleft (om-make-point 0 0)) size)
  (when (themetafile pict)
    (let* ((port (om-get-view view)))
      (draw-metafile port (themetafile pict) (om-point-h topleft) (om-point-v topleft)   (om-point-h size)  (om-point-v size)))))

(defmethod om-get-picture-size ((picture internal-picture))
  (om-make-point (size-x picture) (size-y picture)))

(defmethod om-pict-width ((picture internal-picture))
  (size-x picture))

(defmethod om-pict-height ((picture internal-picture))
  (size-y picture))

(defmacro om-record-pict (font size &body body)
  `(let ((metafile (capi:with-internal-metafile  
                       (port :pane nil  :bounds (list 0 0 (om-point-h ,size) (om-point-v ,size)))
                     (let ((*curstream* port))
                       (om-with-font ,font
                                     ,@body)))))
     (make-instance 'internal-picture
                    :themetafile metafile
                    :size-x (om-point-h ,size)
                    :size-y (om-point-v ,size))))


;;;====================================================================
;;;CURSORS
;;;====================================================================


(defun om-make-cursor (name &optional (click-pos (om-make-point 0 0)))
  (declare (ignore click-pos))
    (let ((cursorpath (make-pathname :directory (pathname-directory *om-cursor-folder*) :name  name :type *om-cursor-type*))
          (cursor nil))
    (when (probe-file cursorpath)
	(setf cursor (load-cursor (list (list :cocoa cursorpath :x-hot (om-point-h click-pos) :y-hot (om-point-v click-pos))))))
    cursor))

(defun om-get-cursor (name)
  t)


(defvar *om-wait-cursor* nil)
(defvar *om-arrow-cursor* nil)
(defvar *om-verti-size-cursor* nil)  
(defvar *om-horiz-size-cursor* nil)  
(defvar *om-resize-cursor* nil)  
(defvar *om-i-beam-cursor* nil)  
(defvar *om-cross-cursor* nil)
(defvar *om-hand-cursor* nil)

;;; init built-in cursors
(defun om-init-cursors ()
  (setf *om-wait-cursor* t);*WATCH-CURSOR*)
  (setf *om-arrow-cursor* t);*ARROW-CURSOR*)
  (setf *om-horiz-size-cursor* (om-make-cursor "cursor-ruler" (om-make-point 8 8)))
  (setf *om-verti-size-cursor* (om-make-cursor "staff-cursor" (om-make-point 8 8))) 
  (setf *om-resize-cursor* (om-make-cursor "resize-cursor" (om-make-point 8 8)))  
  (setf *om-i-beam-cursor* t);*I-BEAM-CURSOR* )  
  (setf *om-cross-cursor* (om-make-cursor "croix" (om-make-point 8 8)))  
  (setf *om-hand-cursor* (om-make-cursor "hand-cursor" (om-make-point 8 8)))
  )

(om-add-init-func 'om-init-cursors)

(defmacro om-with-cursor (cursor &body body)
  `(let () ,@body))


