
;;===========================================================================
; DocFile
; LISTENER
;;===========================================================================
(in-package :om-api)

;;;===================
;;; export :
;;;===================
(export '( 
          om-listener
          om-make-new-listener
          *om-stream*
          ) :om-api)

;;;=============================
;;; LISTENER
;;;=============================

(defvar *om-app* nil)

(defvar *om-listener* nil)
(defvar *om-stream* nil)

;; (defclass om-listener-pane (capi:listener-pane capi:collector-pane) ())

(defun init-listener ()
  (setq *om-stream* (capi:collector-pane-stream (make-instance 'capi:collector-pane))))

; (om-api-add-init-func 'init-listener)

(defvar *om-prompt* "OM->")

(let ((lispworks::*HANDLE-WARN-ON-REDEFINITION* nil))
 (defun print (something &optional stream)
   (declare (ignore stream))
   (format *om-stream* "~D ~D~%" *om-prompt* something)
   something)
)

(defclass om-listener (capi::interface) ())
(defclass om-listener-in-pane (capi::listener-pane) ())
(defclass om-listener-out-pane (capi::collector-pane) ())

(defun make-om-listener (&key title x y width height initial-lambda)
  (or (and oa::*om-listener* (capi::find-interface 'om-listener))
      (progn
        (init-listener)
        (setf oa::*om-listener* 
              (make-instance 'om-listener
                             :layout (make-instance 'capi:column-layout
                                                    :description 
                                                    (list 
                                                     (make-instance 'om-listener-in-pane
                                                                    :echo-area nil
                                                                    :create-callback (if initial-lambda
                                                                                         (lambda (window) 
                                                                                           (declare (ignore window)) 
                                                                                           (capi:execute-with-interface *om-listener* initial-lambda))
                                                                                       (lambda (window)
                                                                                         (declare (ignore window))
                                                                                         (capi:execute-with-interface *om-listener* (lambda () (in-package :oa))))))
                                                     :divider
                                                     (make-instance 'om-listener-out-pane :stream *om-stream*)))
                             :title (or title "OM Listener")
                             :best-x (or x 100)
                             :best-y (or y 200)
                             :best-width (or width 450) :best-height (or height 280)
                             :destroy-callback (lambda (window) (setf oa::*om-listener* nil))
                             ))
        (setf (capi::interface-menu-bar-items oa::*om-listener*)
              (internal-window-class-menubar oa::*om-listener*))
        (capi::display oa::*om-listener*)
        )))

(defmethod internal-window-class-menubar ((self om-listener)) 
  (append (list (make-instance 'capi::menu :title "File"
                       :items 
                       (append (list (make-instance 'capi::menu-item :title "Close Listener"
                                                    :callback-type :interface
                                                    :callback 'quit-interface
                                                    :accelerator #\w))
                               
                               (if (handler-case (find-class 'oa::om-text-editor) (error () nil))
                                   (list 
                                    (make-instance 'capi::menu-item :title "New..."
                                                   :callback-type :interface
                                                   :callback 'listener-new-text-editor
                                                   :accelerator #\n
                                                   :enabled-function 'file-operations-enabled)
                                    (make-instance 'capi::menu-item :title "Open..."
                                                   :callback-type :interface
                                                   :callback 'listener-new-text-file
                                                   :accelerator #\o
                                                   :enabled-function 'file-operations-enabled)))))
                (make-instance 'capi::menu :title "Edit"
                               :items (list (make-instance 'capi::menu-item :title "Copy"
                                                           :callback-type :interface
                                                           :callback 'listener-copy
                                                           :accelerator #\c)
                                            (make-instance 'capi::menu-item :title "Cut"
                                                           :callback-type :interface
                                                           :callback 'listener-cut
                                                           :accelerator #\x)
                                            (make-instance 'capi::menu-item :title "Paste"
                                                           :callback-type :interface
                                                           :callback 'listener-paste
                                                           :accelerator #\v))))
          (om-window-class-menubar oa::*om-listener*)))






;;; redefinie dans OM
(defmethod om-window-class-menubar ((self om-listener)) ())


(defun listener-new-text-file (listenerwin &optional path)
  (om-open-new-text-file path))

(defun listener-new-text-editor (listenerwin)
  (om-open-new-text-editor nil))

(defun listener-paste (listenerwin)
  (let* ((pane (find-pane-with-focus (pane-layout listenerwin)))
         (buffer (editor-pane-buffer pane)))
    (call-editor pane (list 'editor::insert-cut-buffer-command buffer))))

(defun listener-copy (listenerwin)
  (let* ((pane (find-pane-with-focus (pane-layout listenerwin)))
         (buffer (editor-pane-buffer pane)))
    (call-editor pane (list 'editor::copy-to-cut-buffer-command buffer))))

(defun listener-cut (listenerwin)
  (let* ((pane (find-pane-with-focus (pane-layout listenerwin)))
         (buffer (editor-pane-buffer pane)))
    (call-editor pane (list 'editor::copy-to-cut-buffer-command buffer))
    (call-editor pane (list 'editor::kill-region-command buffer))))

(defun om-make-new-listener (&key title x y width height initial-lambda)
  (make-om-listener :title title :x x :y y :width width :height height :initial-lambda initial-lambda))
