;;===========================================================================
;OM API 
;Multiplatform API for OpenMusic
;Macintosh version (Digitool Macintosh Common Lisp - MCL)
;
;Copyright (C) 2004 IRCAM-Centre Georges Pompidou, Paris, France.
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
;
;Authors: Augusto Agon and Jean Bresson
;;===========================================================================

;;===========================================================================
;DocFile
; SYSTEM UTILITIES (File management, foreign calls, scheduler, etc...)
;;===========================================================================
(in-package :om-api)

;;;===================
;;; export :
;;;===================
(export '(
          om-add-init-func
          om-init-funcall
          om-add-exit-cleanup-func
          *om-os*
          om-get-next-event
          catch-cancel
          om-confirmed-quit
          om-abort
          om-eval-enqueue
          
          om-with-redefinitions
          om-redefinition-warnings
          om-set-redefinition-warnings
          om-with-redefinition-warnings
          om-compiler-warnings
          om-set-compiler-warnings
          om-with-compiler-warnings
          om-load-verbose
          om-set-load-verbose
          om-with-load-verbose
          
          om-start-scheduler
          om-stop-scheduler
          
          *om-separator*
          *om-compiled-type*
          directoryp
          directory-pathname-p
          create-directory
          create-file
          om-copy-file
          stream-eofp
          om-deep-delete-file
          om-load-file
          om-directory
          om-set-file-format
          om-read-line
          om-correct-line
          om-namestring
          *om-root*
          
          om-get-user-name 
          om-user-home 
          om-user-pref-folder
          om-path2cmdpath
          om-cmd-line
          om-run-application
          om-find-process
          om-select-process
          om-run-process
          om-kill-process

          om-persistant-p
          om-set-mac-file-type
          om-without-interrupts
          
          ) :om-api)



(defvar *om-os* :MAC)


(defmacro catch-cancel (&body body)
  `(catch :cancel ,@body))

;;;==== INIT AND EXIT FUNCTIONS ====
;------

(defvar *init-func-list* nil)

(defun om-add-init-func (func-name)
   (unless (member func-name *init-func-list* :test 'equal)
      (push func-name *init-func-list*)))

(defun om-init-funcall ()
   (mapc #'(lambda (x) (funcall x)) (reverse *init-func-list*)))

;-------
(defvar *om-cleanup-forms* nil)

(defun om-add-exit-cleanup-func (func-name &optional last?)  
(unless (member func-name *om-cleanup-forms* :test 'equal)
  (if last?
     (push func-name  *om-cleanup-forms*)
     (setf  *om-cleanup-forms* (append  *om-cleanup-forms* (list func-name)))
     )))


;------
;;; fini.. ?
(defun om-get-next-event () nil)

;------
(defun om-confirmed-quit () 
  (lispworks::quit))

;------
(defun om-abort () (abort))

;------
(defun om-eval-enqueue (form) 
  (eval form)) 

;------

(defun om-start-scheduler (symbol)
   (setq *scheduler-timer*  (mp:make-named-timer 'om-scheduler   'om::check-scheduler nil ))
   (mp:schedule-timer *scheduler-timer* 10  0.01))

(defun om-stop-scheduler ()
    (mp:unschedule-timer *scheduler-timer*))

;------
(defmacro om-with-redefinitions (&body body)
  `(let ((lispworks::*HANDLE-WARN-ON-REDEFINITION* nil)) ,@body))

;------
(defun om-redefinition-warnings () lispworks::*HANDLE-WARN-ON-REDEFINITION*)

;------
(defun om-set-redefinition-warnings (t-or-nil) 
  (setf lispworks::*HANDLE-WARN-ON-REDEFINITION* t-or-nil))

;------
(defmacro om-with-redefinition-warnings (t-or-nil &body body)
  `(let ((lispworks::*HANDLE-WARN-ON-REDEFINITION* nil)) ,@body))

;------
(defun om-compiler-warnings () t)

;------
(defun om-set-compiler-warnings (t-or-nil) t)

;------
(defmacro om-with-compiler-warnings (t-or-nil &body body)
  `(let () ,@body))

;------
(defun om-load-verbose () *load-verbose* )

;------
(defun om-set-load-verbose (t-or-nil) (setf *load-verbose* t-or-nil))

;------
(defmacro om-with-load-verbose (t-or-nil &body body)
   `(let ((*load-verbose* ,t-or-nil)) ,@body))


;;;=============================
;;; FILES
;;;=============================

(defvar *om-separator* "/")

(defvar *om-compiled-type* (if (member :X86 *features*) "xfasl" "nfasl"))

(defvar *om-root* (truename "CL:"))


;------
;;; ajoute un butlast
(defun om-root-init ()
  (setf *om-root* (truename "cl:"))
  ;(when nil; (member :om *features*)
  ;  (setf *om-root* (make-pathname :directory 
  ;                                 (butlast (butlast (butlast (butlast  (pathname-directory (truename  (PATHNAME-LOCATION (LISP-IMAGE-NAME)))))))))))
)

(om-add-init-func 'om-root-init)

;------

(defun create-file (pathname)
  (with-open-file (file pathname :direction :output :if-does-not-exist :create) ))

(defun create-folder (pathname &key (if-exists nil))
   (ENSURE-DIRECTORIES-EXIST pathname ))

(defun create-directory (pathname &key (if-exists nil))
   (ENSURE-DIRECTORIES-EXIST pathname ))


;------
(defun om-copy-file (sourcepath targetpath &key (if-exists :supersede))
  (when (probe-file targetpath)
    (delete-file targetpath))
  (system::copy-file sourcepath targetpath) ;;:if-exists if-exists
  )

;------
(defun deep-delete-file (path)
  (if (system::directory-pathname-p path)
    (let ((directories (om-directory  path :directories t :files t)))
      (loop for item in directories do
            (deep-delete-file item))
      (delete-directory path))
     (delete-file path)))

(defun om-deep-delete-file (path)
  (deep-delete-file path))


;------
(defun om-load-file (path)
  (load path))

;------

(defvar *volume-sys* nil)

(defun init-volume-var ()
   t)
  
;(eval-when (:load-toplevel :execute)
;  (ccl::def-load-pointers load-volume-var () (init-volume-var)))

(defun real-pathname-name1 (path)
  (let ((type (pathname-type path)))
    (if (or (equal type :UNSPECIFIC) (not type))
      (om-namestring (pathname-name path))
      (om::string+ (om-namestring (pathname-name path)) "." type)
      )))

(defun path2unixpath (path)
   path)



(defun om-path2cmdpath (path) 
   (namestring path))
  
(defun corrige-path-space (name) name)

;------
(defun om-namestring (path)
  (namestring path))

(defun OM-directory (path &key (type nil) (directories t) (files t) (resolve-aliases nil))
 (let ((rep (directory (namestring path) :link-transparency resolve-aliases)))
   (when (not files)
   (setf rep (loop for item in rep when (directoryp item) collect item)))
   (when (not directories)
     (setf rep (loop for item in rep when (not (directoryp item)) collect item)))
   (when type
     (setf rep (loop for item in rep when (equal (pathname-type item) type) collect item)))
   rep))
              

(defun om-set-file-format (filename)
  t)


(defun om-read-line (file)
  (read-line file nil 'eof))


(defun om-correct-line (line &optional stream)
  line)

(defun om-quit ()  
  (om-confirmed-quit))



(defun stream-eofp (s)
  (not (stream::stream-check-eof-no-hang s)))



;;;===================
;;; system

(defun om-get-user-name ()
  (system::get-user-name))

(defun om-user-home ()
 (USER-HOMEDIR-PATHNAME))

(defun om-user-pref-folder ()
  (let* ((userhome (om-user-home)))
    (make-pathname
     :device (pathname-device userhome)
    :directory (append (pathname-directory userhome) (list "Library" "Preferences")))))

;;;===================
;;;; external apps 



(defun om-cmd-line (str &optional (redirect-output nil))
   (when redirect-output
     (sys:run-shell-command "open -a Console")
     (setf str (concatenate 'string str " >/dev/console")))
   (sys:run-shell-command str))

#|
;;; doit retourner un ID !!
(defun om-run-application (path &optional afterfun)
  (if path
    (ccl::launch-application path)
    (when afterfun (funcall afterfun))
    ))

;???? 
(defun om-run-application (path &optional afterfun)
  (when path
    (let ((ID (ccl::launch-application path)))
      (when afterfun (funcall afterfun))
      ID)
    ))

(defun om-find-process (id)
  (ccl::find-process id))

(defun om-select-process (id)
  (ccl::select-process id))
|#

;;;===================================
;;; multi-processing


(defmacro om-run-process (name func &rest args)
   `(mp:process-run-function  ,name '(:priority 10) ',func ,.args))


(defun om-kill-process (process)
   (mp:process-kill process))

(defun directoryp (p)
  (system::directory-pathname-p p))

(defun om-persistant-p (path)
  (or (system::directory-pathname-p path) 
      (string-equal (pathname-type path) "omp" )
      (string-equal (pathname-type path) "omm" )
      (string-equal (pathname-type path) "omc" )
      (string-equal (pathname-type path) "omi" )
      (string-equal (pathname-type path) "ome" )
      (string-equal (pathname-type path) "ml2" )))

(defun om-set-mac-file-type (file type) t)

(defmacro om-without-interrupts (&body body)
  `(mp::without-interrupts ,.body))




