;;===========================================================================
;OM API 
;Multiplatform API for OpenMusic
;Macintosh version (Digitool Macintosh Common Lisp - MCL)
;
;Copyright (C) 2004 IRCAM-Centre Georges Pompidou, Paris, France.
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
;
;Authors: Augusto Agon and Jean Bresson
;;===========================================================================

;;===========================================================================
;DocFile
; WINDOWS AND VIEWS 
;;===========================================================================

(in-package :om-api)

;;;=====================
;;; export :
;;;=====================
(export '(
          omg-defclass     
          om-view
          om-make-view
          om-width
          om-height
          om-view-size
          om-view-position
          om-set-view-size
          om-set-view-position
          om-change-view-position
          om-set-bg-color
          om-get-bg-color
          om-set-part-color
          om-get-font
          om-set-font
          om-get-view-name
          om-view-window
          om-view-container
          om-subviews
          om-add-subviews
          om-remove-subviews
          om-map-subviews
          om-view-set-help
          om-transparent-view
          om-scroller
          om-field-size
          om-set-field-size
          om-scroll-position
          om-set-scroll-position
          om-h-scroll-position
          om-v-scroll-position
          om-window
          om-make-window
          om-window-title
          om-set-window-title
          om-interior-size
          om-set-interior-size
          om-def-higher-level-win
          om-front-window
          om-get-all-windows
          om-fullscreen-window
          om-dialog
          om-modal-dialog
          om-return-from-modal-dialog
          
          om-windoid
          om-no-border-win

          om-window-close-event
          om-close-window
          om-window-open-p
          om-select-window
          om-hide-window
          om-window-activate
          om-window-resized
          om-minimum-size

          om-view-contains-point-p
          om-find-view-containing-point
          om-convert-coordinates
          om-view-origin

          ) :om-api)
;;;=====================

;;;=======================
;;; GENERAL GRAPHIC OBJECT
;;;=======================
;;; EQUIVALENT SIMPLE-PANE IN LISPWORK/CAPI
;;; manages general callbacks and behaviors

;;; GTK+ Compatibility
;;; used to instanciate om-graphic-object subclasses
(defmacro omg-defclass  (Name Superclass Slots &rest Class-Options)
  `(defclass ,name ,superclass ,slots ,.class-options))

(defclass om-graphic-object (om-movable-view) 
  ((vcontainer :initform nil :initarg :vcontainer :accessor vcontainer)
   (vsubviews :initform nil :initarg :vsubviews :accessor vsubviews)
   (vx :initform 0 :initarg :vx :accessor vx)
   (vy :initform 0 :initarg :vy :accessor vy)
   (vw :initform 32 :initarg :vw :accessor vw)
   (vh :initform 32 :initarg :vh :accessor vh)
   (help-spec :initform nil :initarg :help-spec :accessor help-spec)
   (initialized-p :initform nil :accessor initialized-p))
  (:default-initargs 
 :display-callback 'om-draw-contents-callback
 :input-model '( (:post-menu om-context-menu-callback)
                 ((:button-1 :press :shift :hyper :meta) om-clic-callback '(t t t))
                 ((:button-1 :press :shift :hyper) om-clic-callback '(t t nil))
                 ((:button-1 :press :shift :meta) om-clic-callback '(t  nil t))
                 ((:button-1 :press :meta :hyper) om-clic-callback '(nil t t))
                 ((:button-1 :press :shift ) om-clic-callback '(t nil nil))
                 ((:button-1 :press :hyper) om-clic-callback '(nil t nil))
                 ((:button-1 :press :meta) om-clic-callback '(nil nil t))
                 ((:button-1 :press) om-clic-callback '(nil nil nil))
                 
                 ((:button-1 :motion :shift :hyper :meta)  om-clic-motion-callback '(t t t))
                 ((:button-1 :motion :shift :hyper)  om-clic-motion-callback '(t t nil))
                 ((:button-1 :motion :shift :meta)  om-clic-motion-callback '(t  nil t))
                 ((:button-1 :motion :meta :hyper)  om-clic-motion-callback '(nil t t))
                 ((:button-1 :motion :shift)  om-clic-motion-callback '(t nil nil))
                 ((:button-1 :motion :hyper)  om-clic-motion-callback '(nil t nil))
                 ((:button-1 :motion :meta)  om-clic-motion-callback '(nil nil t))
                 ((:button-1 :motion )  om-clic-motion-callback '(nil nil nil))
               
                 (:motion  om-motion-callback)
                 
                 ((:button-1 :release :shift :hyper :meta)  om-clic-release-callback '(t t t))
                 ((:button-1 :release :shift :hyper)  om-clic-release-callback '(t t nil))
                 ((:button-1 :release :shift :meta)  om-clic-release-callback '(t  nil t))
                 ((:button-1 :release :meta :hyper)  om-clic-release-callback '(nil t t))
                 ((:button-1 :release :shift)  om-clic-release-callback '(t nil nil))
                 ((:button-1 :release :hyper)  om-clic-release-callback '(nil t nil))
                 ((:button-1 :release :meta)  om-clic-release-callback '(nil nil t))
                 ((:button-1 :release )  om-clic-release-callback '(nil nil nil))

                 ((:button-1 :second-press) om-double-clic-callback)
                ;;; ((:button-3 :press) om-right-clic-callback)
                
                (:character om-char-callback))
 :activate-callback 'om-activate-callback
 :create-callback 'om-create-callback
 :geometry-change-callback 'om-resize-callback
 :destroy-callback 'om-destroy-callback))


 

(defmethod om-draw-contents-callback (self x y w h))
(defmethod om-resize-callback (self x y w h))
(defmethod om-clic-callback (self x y type))
(defmethod om-clic-release-callback (self x y type))
(defmethod om-motion-callback (self x y))
(defmethod om-double-clic-callback (self x y))
;;;(defmethod om-right-clic-callback (self x y))
(defmethod om-char-callback (self x y c))
(defmethod om-activate-callback (self activatep))
(defmethod om-create-callback (self))
(defmethod om-destroy-callback (self))
(defmethod om-clic-motion-callback (self x y type))
(defmethod om-context-menu-callback (self x y))

(defun maybe-call-update (graphic-obj)
  (declare (special *delayed-redraw*))
  (when (and (initialized-p graphic-obj) (not *delayed-redraw*)) ;; (interface-visible-p graphic-obj)
    (update-for-subviews-changes graphic-obj nil)
    ))

(defmethod update-for-subviews-changes ((self om-graphic-object) &optional (recursive nil)) nil)

(defmethod om-add-subviews ((self om-graphic-object) &rest subviews)
  (loop for item in subviews do
	(setf (vcontainer item) self))
  (setf (vsubviews self) (append (vsubviews self) subviews))
  (maybe-call-update self))

(defmethod om-remove-subviews ((self om-graphic-object) &rest subviews)
  (loop for item in subviews do 
        (setf (vcontainer item) nil)
        (setf (vsubviews self) (remove item (vsubviews self))))
  (maybe-call-update self)
  ; (loop for item in subviews do (setf (element-parent item) nil)
  )

;;; different pour om-window
(defmethod om-get-view ((self om-graphic-object)) self)

(defmethod om-subviews ((self om-graphic-object)) 
  (if (interface-visible-p self)
      (capi::layout-description (om-get-view self))
    (vsubviews self)
    ))


(defmethod container-skip-layout ((self t)) self)

(defmethod om-view-container ((self om-graphic-object))
 (container-skip-layout (or (vcontainer self)  (capi::element-parent self))))


(defmethod om-view-window ((self om-graphic-object))
  (capi::top-level-interface self))

(defmethod om-map-subviews ((self om-graphic-object) function &optional type)
  (loop for item in (om-subviews self) collect
        (funcall function item)))

(defmethod om-width ((item om-graphic-object)) (om-point-h (om-view-size item)))
(defmethod om-height ((item om-graphic-object)) (om-point-v (om-view-size item)))

(defmethod om-resize-callback ((self om-graphic-object) x y w h)
  (setf (vx self) x (vy self) y (vw self) w (vh self) h))

(defmethod om-get-view-name ((self om-graphic-object))
  (capi-object-name self))

(defmethod om-set-bg-color ((self om-graphic-object) color) 
  (setf (simple-pane-background (om-get-view self)) color))
      
(defmethod om-get-bg-color ((self om-graphic-object))
  (simple-pane-background (om-get-view self)))

(defmethod om-set-part-color ((self om-graphic-object) part color)
  (cond
   ((equal part :text) (setf (simple-pane-foreground (om-get-view self)) color))
   ((equal part :body) (setf (simple-pane-background (om-get-view self)) color))))

(defmethod om-get-font ((self om-graphic-object))
  (let ((font (capi::simple-pane-font self)))
    (if (gp::font-description-p font) font
      (gp::font-description font))))

(defmethod om-set-font ((self om-graphic-object) font) 
  (setf (simple-pane-font self) font))

;;;=============================================
;;; OM-ABSTRACT-WINDOW
;;; Superclass of all windows
;;;=============================================
(defclass om-abstract-window (om-graphic-object interface) ())

(defmethod om-view-container ((self om-abstract-window)) nil)

(defmethod om-get-view ((self om-abstract-window))  (pane-layout self))

(defmethod om-window-title ((self om-abstract-window))
  (interface-title self))

(defmethod om-set-window-title ((self om-abstract-window) (title string))
  (setf (interface-title self) title))

(defmethod om-view-position ((self om-abstract-window)) 
  (if (interface-visible-p self)
      (let ((point (interface-geometry self)))
        (om-make-point (first point) (second point)))
    (om-make-point (vx self) (vy self))))

(defmethod om-set-view-position ((self om-abstract-window) pos-point) 
  (when (interface-visible-p self)
      (execute-with-interface self 
                              'set-top-level-interface-geometry 
                              self 
                              :x (om-point-h pos-point)
                              :y (om-point-v pos-point)))
  (setf (vx self) (om-point-h pos-point))
  (setf (vy self) (om-point-v pos-point))
  )

(defmethod om-view-size ((self om-abstract-window))
  (if (interface-visible-p self)
      (let ((point (interface-geometry self)))
        (om-make-point (third point) (fourth point)))
    (om-make-point (vw self) (vh self))))

(defmethod om-set-view-size ((self om-abstract-window) size-point) 
  (let ((w (om-point-h size-point))
        (h (om-point-v size-point)))
    (when (interface-visible-p self)
      (execute-with-interface self 
                              'set-top-level-interface-geometry 
                              self 
                              :width w
                              :height h))
    (setf (vw self) w
          (vh self) h)))

(defmethod om-interior-size ((self om-abstract-window))
  (om-view-size self))

(defmethod om-set-interior-size ((self om-abstract-window) size)
  (om-set-view-size self size))

(defmethod om-resize-callback ((self om-abstract-window) x y w h)
  (call-next-method)
  (om-window-resized self (om-make-point w h)))

(defmethod om-window-resized ((self om-abstract-window) size)
  (declare (ignore self size)) nil)

(defmethod om-minimum-size ((self om-abstract-window))
  (declare (ignore self)) nil)

(defmethod capi::calculate-constraints ((self om-abstract-window))
  (call-next-method)
  (let ((msize (om-minimum-size self)))
    (when msize
      (with-geometry self
        (setf capi:%min-width% (om-point-h msize))
        (setf capi:%min-height% (om-point-v msize))
        ))))

(defmethod om-fullscreen-window ((self om-abstract-window))
  ;;; pour l'instant, mximize...  faire en fullscreen
  (execute-with-interface self 
                          'set-top-level-interface-geometry 
                          self 
                          :width (capi::screen-width (capi:convert-to-screen self))
                          :height (capi::screen-height (capi:convert-to-screen self))
                          :x 0 :y 0))


(defun om-front-window () 
 (capi:screen-active-interface (capi:convert-to-screen)))

(defun om-get-all-windows (class)
  (capi::collect-interfaces class))
;;; KEYWORDS: :screen :any :sort-by :visible

(defmethod interface-match-p ((self om-abstract-window) &rest initargs  &key name)
  (string-equal (capi::capi-object-name self) name))

(defmethod om-select-window ((self om-abstract-window)) 
  (when (and (window-dialog-p self) (not (initialized-p self)))
      (display self))
  (when (initialized-p self)
    (capi::find-interface (type-of self) :name (capi::capi-object-name self)))
  (om-activate-callback self t))



(defun om-hide-window (win) 
  (capi::execute-with-interface win 
                                #'(lambda (x) (setf (top-level-interface-display-state x) :hidden))
                                win))

;;; Explicit call to close window
(defmethod om-close-window ((win t))
  (quit-interface win))

;;; Called by CAPI when destroying the window
(defmethod om-destroy-callback  ((self om-abstract-window))
  (om-window-close-event self)
  (setf (initialized-p self) nil))

;;; OM event handler
(defmethod om-window-close-event ((self t)) nil)




;;; appel  la cration de la fenetre
;;; (pas a l'apparition)
(defmethod om-create-callback ((self om-abstract-window))
  ;; (set-hint-table self (list :default-x (vx self) :default-y (vy self) :default-width (vw self) :default-height (vh self)))
  ;; (loop for item in (vsubviews self) do
  ;;      (om-add-subviews thewin item))
  (setf (initialized-p self) t)
  t)


(defun om-window-open-p (window) (initialized-p window))

;;; Called by CAPI when activating the window
(defmethod om-activate-callback ((self om-abstract-window) activatep)
  (when activatep
    (om-window-activate self)))

;;; OM event handler
(defmethod om-window-activate ((self om-abstract-window)) t)
 
(defmethod interface-display :after ((self om-abstract-window)) 
  ;;(print (list "display" (om-window-title self)))
  (update-for-subviews-changes self t)
)

(defmethod update-for-subviews-changes ((self om-abstract-window) &optional (recursive nil))
  (apply-in-pane-process (pane-layout self) (lambda (x) (setf (capi:layout-description (pane-layout x)) (vsubviews x))) self)
  (when recursive (mapc 'update-for-subviews-changes (vsubviews self))))


(defmacro om-make-window (class &rest attributes 
			  &key (position (om-make-point 200 200)) (size (om-make-point 500 400))
			  name (window-title "om-window") owner 
			  (font *om-default-font2*) (bg-color *om-white-color*)
			  (window-show t) subviews 
			  (resizable t)	(close t)
			  (minimize t) (maximize t)
			  ;;;textbuffer
			  ;;;ask-for-save 
			  &allow-other-keys)  
  `(let* ((winparent (or ,owner (capi:convert-to-screen)))
          (winname (or ,name (string (gensym))))
          (w (om-point-h ,size))
          (h (om-point-v ,size))
          (x (if (equal ,position :centered) (round (- (* (capi::screen-width winparent) 0.5) (* w 0.5))) (om-point-h ,position)))
          (y (if (equal ,position :centered) (round (- (* (capi::screen-height winparent) 0.5) (* h 0.5))) (om-point-v ,position)))
          (style (if ,minimize 
                     (get-window-styles-from-class ,class)
                 (append (get-window-styles-from-class ,class) (list :never-iconic))))
          layout thewin)
     (setf thewin (make-instance ,class
                                 :title ,window-title :name winname
                                 :x x :y y :width w :height h
                                 :parent winparent
                                 :internal-border 0 :external-border 0
                                 :display-state (if ,window-show :normal :hidden)
                                 ;:window-styles '(:internal-borderless)
                                 :internal-min-height 20 :internal-min-width 40
                                 :window-styles style
                                 :font ,font
                                 ;;; OM-GRAPHIC-OBJECT SLOTS
                                 :vx x :vy y :vw w :vh h
                                 :allow-other-keys t
                                 ,.attributes
                                 ))     
     (when (setf layout (make-window-layout thewin ,bg-color)) (setf (pane-layout thewin) layout))
     (when ,subviews (mapc (lambda (sv) (om-add-subviews thewin sv)) ,subviews))
     (when (or (not ,resizable) (window-dialog-p thewin))
       (set-not-resizable thewin w h))
     (unless (window-dialog-p thewin) (display thewin))
     thewin))

(defmethod set-not-resizable ((self t) &optional w h)
  (let ((width (or w (om-point-h (om-view-size self))))
        (height (or h (om-point-v (om-view-size self)))))
    (set-hint-table self (list :external-min-width w :external-max-width w 
                               :external-min-height h :external-max-height h))))
  

(defmethod get-window-styles-from-class ((class t))
  (cond 
   ((subtypep class 'om-windoid) '(:internal-borderless :always-on-top :shadowed 
                                   :no-geometry-animation :toolbox :textured-background
                                   :ignores-keyboard-input)) ; :borderless
   ((subtypep class 'om-no-border-win) '(:internal-borderless :always-on-top :borderless :shadowed))
   ((subtypep class 'om-window) '(:internal-borderless))
   ((subtypep class 'om-dialog) '(:textured-background :internal-borderless))
   ))

(defmethod window-dialog-p ((self t) ) nil)
    
;;;=====================
 
(defclass window-layout (om-graphic-object pinboard-layout) ())

(defmethod make-window-layout ((self om-abstract-window) &optional color)
  (make-instance 'window-layout :internal-border nil :visible-border nil 
                 :background (or color *om-white-color*)))

(defmethod container-skip-layout ((self window-layout) )
   (capi::top-level-interface self))

(defmethod om-char-callback ((self window-layout) x y char)
  (om-char-callback (capi::element-parent self) x y char)) 

(defmethod om-draw-contents-callback ((self window-layout) x y w h)
  (om-draw-contents-callback (capi::top-level-interface self) x y w h))

(defmethod om-clic-callback ((self window-layout) x y type)
  (om-clic-callback (capi::top-level-interface self) x y type))

(defmethod om-clic-motion-callback ((self window-layout) x y type)
  (om-clic-motion-callback (capi::top-level-interface self) x y type))

(defmethod om-view-double-clic-callback ((self window-layout) x y)
  (om-view-double-clic-callback (capi::top-level-interface self) x y))

(defmethod om-clic-release-callback ((self window-layout) x y type)
  (om-clic-release-callback (capi::top-level-interface self) x y type))

(defmethod om-motion-callback ((self window-layout) x y) 
  (om-motion-callback (capi::top-level-interface self) x y))

;;;(defmethod om-init-motion ((self window-layout) x y) 
;;;  (om-init-motion (capi::top-level-interface self) x y))


;;;=============================================
;;; OM-WINDOW
;;; A simple window with a panel able to host subviews
;;;=============================================
(defclass om-window (om-abstract-window) ()
  ;(:default-initargs :window-styles '(:internal-borderless))
)

(defmethod om-interior-size ((self om-window)) (call-next-method))

(defmethod om-set-interior-size ((self om-window) size) (call-next-method))



;;;====================
;;; DIALOG
;;; Pour mettre des dialog-items
;;; Des interfaces modales, etc.
;;;====================
(defclass om-dialog (om-abstract-window) ()
; (:default-initargs
;   :window-styles '(:textured-background :internal-borderless))
)

(defmethod window-dialog-p ((self om-dialog) )   t)

(defmethod make-window-layout ((self om-dialog) &optional color)
  (make-instance 'pinboard-layout :internal-border nil :visible-border nil 
                 :background :transparent))

(defun om-modal-dialog (dialog &optional (owner nil))
  (update-for-subviews-changes dialog t)
  (capi::display-dialog dialog :owner (or owner (capi:convert-to-screen)) :position-relative-to :owner))

(defun om-return-from-modal-dialog (dialog val) 
  (capi::exit-dialog val))


;;;====================
;;; PALETTE
;;; Always on top, petite barre de titre
;;;====================
(defclass om-windoid (om-window) 
  ((accept-key-evt :accessor accept-key-evt :initarg :accept-key-evt :initform nil))
  ;(:default-initargs
  ; :window-styles '(:internal-borderless :always-on-top :borderless :shadowed))
  )

(defmethod make-window-layout ((self om-windoid) &optional color)
  (make-instance 'window-layout :internal-border nil :visible-border nil 
                 :background :transparent))

(defmethod om-set-view-size ((self om-windoid) size-point) 
  (let ((w (om-point-h size-point))
        (h (om-point-v size-point)))
    (set-not-resizable self w h)
    (call-next-method)))

; (defmethod set-not-resizable ((self om-windoid) &optional w h) (call-next-method))
; (om-make-window 'om-windoid :resizable nil)


;;;====================
;;; NO BORDER
;;;====================
(defclass om-no-border-win (om-window) ()
 ;(:default-initargs
 ;  :window-styles '(:internal-borderless :borderless :shadowed))
 )


(defmethod om-fullscreen-window ((self om-no-border-win)) (call-next-method))

;;;======================
;;; VIEW
;;; General graphic pane
;;;======================

(defclass om-view (om-graphic-object pinboard-layout) ())

;(defmethod initialize-instance  ((self om-view) &rest l)
;  (call-next-method)
;  (when (owner self) (om-add-subviews (owner self) self))
;  self)

(defmethod update-for-subviews-changes ((self om-view) &optional (recursive nil))
  ; (print (list "update" self (vsubviews self)))
  ;(apply-in-pane-process self (lambda () (setf (capi:layout-description self) (vsubviews self))))
  (apply-in-pane-process self (lambda () 
                                (setf (capi:layout-description self) (vsubviews self))))
  (when recursive (mapc 'update-for-subviews-changes (vsubviews self))))

(defmacro om-make-view (class
			&rest attributes
                        &key 
                        (position (om-make-point 0 0))
			(size (om-make-point 32 32))
			owner 
			subviews 
			name 
			font 
			bg-color
                        scrollbars 
			retain-scrollbars 
			field-size 
                        &allow-other-keys)
  `(let*  ((x (om-point-h ,position))
           (y  (om-point-v ,position))
           (w (om-point-h ,size))
           (h (om-point-v ,size))
           (view (make-instance ,class
                               :name ,name
                               ;:x x :y y 
                               :width nil :height nil
                               :default-x x :default-y y :default-width w :default-height h
                               :visible-min-width nil :visible-min-height nil
                               :fit-size-to-children nil
                               :horizontal-scroll (or (equal ,scrollbars t) (equal ,scrollbars :h))
                               :vertical-scroll (or (equal ,scrollbars t) (equal ,scrollbars :v))
                               :scroll-width (if ,field-size (om-point-h ,field-size) 0)
                               :scroll-height (if ,field-size (om-point-v ,field-size) 0)
                               :vx x :vy y :vw w :vh h
                               :vcontainer ,owner
                               :allow-other-keys t
                               ,.attributes
                               )))
     (when ,bg-color (setf (simple-pane-background view) ,bg-color))
     (when ,owner (om-add-subviews ,owner view))
     (when ,subviews (mapc (lambda (sv) (om-add-subviews view sv)) ,subviews))
     view))

(defmethod om-create-callback ((self om-view))
  ;(set-hint-table self (list :default-x (vx self) :default-y (vy self) :default-width (vw self) :default-height (vh self)))
  ;(capi::apply-in-pane-process self (lambda () 
  ;                                    (setf (capi:layout-description self) 
  ;                                          (vsubviews self))))
  ;(print (list "create view" self (vsubviews self)))
  (update-for-subviews-changes self nil)
  (setf (initialized-p self) t))

(defmethod om-set-view-position ((self om-view) pos-point) 
;  (if (interface-visible-p self)
  (capi:apply-in-pane-process self 
                              #'(lambda ()
                                  (setf (pinboard-pane-position self) (values (om-point-h pos-point) (om-point-v pos-point))) 
                                  ))  
;   (set-hint-table self (list :default-x (om-point-h pos-point) :default-x (om-point-v pos-point))))
  (setf (vx self) (om-point-h pos-point))
  (setf (vy self) (om-point-v pos-point)))

(defmethod om-change-view-position ((item om-view) pos-point) 
   (om-set-view-position item pos-point))

(defmethod om-view-position ((self om-view)) 
  (if (interface-visible-p self)
      (multiple-value-bind (x y) (pinboard-pane-position self) (om-make-point x y))
    (om-make-point (vx self) (vy self))))

(defmethod om-set-view-size ((self om-view) size-point) 
;  (if (interface-visible-p self)
      (capi:apply-in-pane-process self 
                                  #'(lambda ()
                                      (setf (pinboard-pane-size self) (values (om-point-h size-point) (om-point-v size-point)))
                                      ))
;    (set-hint-table self (list :default-width (om-point-h size-point) :default-height (om-point-v size-point))))
  (setf (vw self) (om-point-h size-point))
  (setf (vh self) (om-point-v size-point)))

(defmethod om-view-size ((self om-view)) 
  (if (interface-visible-p self)
      (multiple-value-bind (x y) (pinboard-pane-size self) (om-make-point x y))
    (om-make-point (vw self) (vh self))))

(defmethod om-interior-size ((self om-view))
  (om-view-size self))

;;;======================
;;; OM-TRANSPARENT-VIEW
;;;======================
(defclass om-transparent-view (om-view) ()
  (:default-initargs :background :transparent))

;;;======================
;;; OM-SCROLLER
;;;======================
(defclass om-scroller (om-view) 
   ((scrollbars :initform t :initarg :scrollbars :accessor scrollbars)
    (retain-scrollbars :initform nil :initarg :retain-scrollbars :accessor retain-scrollbars)
    ; (field-size :initform nil :initarg :field-size :accessor field-size)
    ))

(defmethod om-view-size ((self om-scroller)) 
  (call-next-method))

(defmethod om-set-view-size ((self om-scroller) size-point) 
  (capi:apply-in-pane-process self 
                                  #'(lambda ()
                                      ;(set-hint-table self (list ;:width (om-point-h size-point) 
                                      ;                           :external-min-width (om-point-h size-point) 
                                      ;                           :external-max-width (om-point-h size-point) 
                                      ;                           ;:height (om-point-v size-point) 
                                      ;                           :external-min-height (om-point-v size-point) 
                                      ;                           :external-max-height (om-point-v size-point)
                                      ;                           ))
                                      (setf (pinboard-pane-size self) (values (om-point-h size-point) (om-point-v size-point)))
                                     ))
  (setf (vw self) (om-point-h size-point))
  (setf (vh self) (om-point-v size-point)))

(defmethod scroller-p ((self om-scroller)) t)
(defmethod scroller-p ((self t)) nil)

(defmethod om-field-size ((self om-scroller))
  (om-make-point (capi::get-vertical-scroll-parameters self :max-range) 
                 (capi::get-horizontal-scroll-parameters self :max-range)))

(defmethod om-set-field-size ((self om-scroller) size)
  (capi:apply-in-pane-process self 
                              #'(lambda ()
                                  (capi::set-horizontal-scroll-parameters self :min-range 0 :max-range (om-point-h size))
                                  (capi::set-vertical-scroll-parameters self :min-range 0 :max-range (om-point-v size))
                                  ))
  ;;; (om-update-scroll-bar-limits self)
  t)

;;; NOT USED 
;;; (defmethod om-update-scroll-bar-limits ((self om-scroller)) t)


(defmethod om-scroll-position ((self om-scroller))
  (om-make-point (om-h-scroll-position self )
                 (om-v-scroll-position self )))

(defmethod om-set-scroll-position ((self om-scroller) pos)
  (capi::scroll self :pan :move (list (om-point-h pos) (om-point-v pos))))

(defmethod om-h-scroll-position ((self om-scroller))
  (capi::get-horizontal-scroll-parameters self :slug-position))

(defmethod om-v-scroll-position ((self om-scroller))
  (capi::get-vertical-scroll-parameters self :slug-position))

;;; au cas ou...
(defmethod om-h-scroll-position ((self om-view)) 0)
(defmethod om-v-scroll-position ((self om-view)) 0)
(defmethod om-scroll-position ((self om-view)) (om-make-point 0 0))


(defmethod update-for-subviews-changes ((self om-scroller) &optional (recursive nil))
  (if (initialized-p self)
      (capi::apply-in-pane-process self (lambda () 
                                        (let ((v (capi::get-vertical-scroll-parameters self :max-range))
                                              (h (capi::get-horizontal-scroll-parameters self :max-range))
                                              (x (capi::get-horizontal-scroll-parameters self :slug-position))
                                              (y (capi::get-vertical-scroll-parameters self :slug-position)))
                                          (setf (capi:layout-description self) (vsubviews self))
                                          (capi::set-vertical-scroll-parameters self :min-range 0 :max-range v)
                                          (capi::set-horizontal-scroll-parameters self :min-range 0 :max-range h)
                                          (capi::scroll self :pan :move (list x y))
                                          )))
    ;(call-next-method)
    ;(when recursive (mapc 'update-for-subviews-changes (vsubviews self)))
    ))





;;;======================
;;; TOOLS
;;;======================
(defmethod om-view-contains-point-p ((view om-graphic-object) point)
(let ((x (om-point-h point))
      (y (om-point-v point))
      (brx  (om-width view))
      (bry  (om-height view))
      (rx  (om-point-h (om-view-position view)))
      (ry  (om-point-v (om-view-position view))))
(and (> x rx) (> y ry) (< x (+ brx rx)) (< y (+ bry ry))))
)

(defun om-find-view-containing-point (view point)
  (if view 
      (let ((subviews (om-subviews view)))
	(do ((i (- (length subviews) 1) (- i 1)))
            ((< i 0))
          (let ((subview (nth i subviews)))
            (when (om-view-contains-point-p subview point)
	      (return-from om-find-view-containing-point
                (progn
		  (om-find-view-containing-point subview (om-convert-coordinates point view subview)))
                ))))
        view)))


(defun om-convert-coordinates (point view1 view2)
(om-add-points point
               (om-subtract-points (om-view-origin view2)
				   (om-view-origin view1))))

(defun om-view-origin (view)
   (let ((container (om-view-container view)))
     (if container
       (let ((position (om-view-position view))
             (container-origin (om-view-origin container)))
         (if position
	     (om-subtract-points container-origin position)
           container-origin))
       (om-make-point 0 0))))


;;; (om-subviews *om-workspace-win*)
;;; (om-find-view-containing-point om::*om-workspace-win* (om-make-point 30 30))


(defun find-pane-with-focus (layout)
  (capi:map-pane-descendant-children 
   layout
   #'(lambda (p)
       (when (capi:pane-has-focus-p p)
         (return-from find-pane-with-focus p)))))
