;OpenMusic
;
;Copyright (C) 1997, 1998, 1999, 2000 by IRCAM-Centre Georges Pompidou, Paris, France.
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
;
;Authors: Gerard Assayag and Augusto Agon


;DocFile
;A class to inspect graphic objects is defined here.
;Last Modifications :
;18/10/97 first date.
;DocFile

(in-package :om)

(defun apply-win (win fun &rest args)
  (let ((controls (get-actives  (panel win))))
    (loop for item in controls do
          (eval `(funcall ',fun ,item ,.args)))))

(defmethod get-object-insp-name ((self t))
  "Get a string to describe the class" "????")

(defmethod get-info-window ((self t)) (om-beep))

(defmethod get-obj-for-info ((self icon-finder))
  (object self))

(defmethod get-obj-for-info ((self boxframe))
  (object self))


;==============================================

(omg-defclass info-window (om-dialog)
   ((sujet :initform nil :initarg :sujet :accessor sujet)
    (doc-item :initform nil :initarg :doc-item :accessor doc-item)))

;;; direct dans l'api
;;;(defmethod om-view-activate :after ((self info-window))
;;;   (update-themenubar self))

(defmethod get-new-menu ((self info-window)) *null-new-menu-item*)
(defmethod get-menubar ((self info-window))  *simplest-menubar*)

(defmethod om-window-close-event :after ((self info-window))
  ;(uddate-close (sujet self) self)
  (setf (infowin (sujet self)) nil)
)

(defmethod uddate-close ((self t) win)
  (set-doc self (om-dialog-item-text (doc-item win))))

(defmethod show-info-window ((self OMBasicObject))
  (if (infowin self)
    (om-select-window (infowin self))
    (let* ((info (get-om-properties self))
           (dialog (om-make-window 'info-window
                                   :position (om-make-point 30 50)
                                   :sujet self
                                   :size (car info)
                                   :resizable nil :minimize nil :maximize nil
                                   :window-title (string+ (name self) " Info")
                                   :font *om-default-font1*
                                   :bg-color *om-window-def-color*)))
      (eval `(om-add-subviews ,dialog ,.(cdr info)))
      (setf (doc-item dialog) (cadr info))
      (setf (infowin self) dialog)
      (om-select-window dialog)
      )
    ))

(defmethod show-info-window ((self OMFuncallableBasicObject))
  (if (infowin self)
    (om-select-window (infowin self))
    (let* ((info (get-om-properties self))
           (dialog (om-make-window 'info-window
                                   :position (om-make-point 30 50)
                                   :sujet self
                                   :size (car info)
                                   :resizable nil :minimize nil :maximize nil
                                   :window-title (string+ (name self) " Info")
                                   :font *om-default-font1*
                                   :bg-color *om-window-def-color*)))
      (eval `(om-add-subviews ,dialog ,.(cdr info)))
      (setf (doc-item dialog) (cadr info))
      (setf (infowin self) dialog)
      (om-select-window dialog)
      )
    ))

;-----------------Some special items used by the inspector-----------------


(defmethod active-icon ((self t) &optional (taille (om-make-point 260 250)))
   (list taille 
         (om-make-dialog-item 'om-text-edit-view (om-make-point 15 100) (om-make-point 225 100)
                              (get-documentation self) 
                              :scrollbars :v 
                              )
         (om-make-view 'button-icon
           :iconID (icon self)
           :position (om-make-point 30 5)
           :bg-color *om-window-def-color*
           :size (om-make-point 32 32)
                                      :action  #'(lambda (item) 
                                                          (omg-change-icon self nil)
                                                          (setf (iconID item) (icon self))))
          (om-make-dialog-item 'om-static-text (om-make-point 120 16) (om-make-point 300 16)
                                                     (name self)
                                                     :bg-color *om-window-def-color*
                                                     :font *om-default-font2b*)
          (om-make-dialog-item 'om-static-text (om-make-point 15 50) (om-make-point 60 18) 
                                                     "Type : "  :bg-color *om-window-def-color*)
          (om-make-dialog-item 'om-static-text (om-make-point 70 50) (om-make-point 72 18)  (get-object-insp-name self)
                                                     :bg-color *om-window-def-color*
                                                     )
          (om-make-dialog-item 'om-static-text (om-make-point 15 80) (om-make-point 130 18)
                                                     "Documentation : " :bg-color *om-window-def-color*
)))

(defmethod unactive-icon ((self t) &optional (taille (om-make-point 250 500)))
   (list taille 
         (om-make-dialog-item 'om-static-text (om-make-point 15 100) (om-make-point 220 200) 
                                                    (string-until-CR (get-documentation self))
                                                   :font *om-default-font1*
                                                   :bg-color *om-window-def-color*)
         (om-make-view 'button-icon
           :iconID (icon self)
           :position (om-make-point 30 5)
           :size (om-make-point 32 32)
           :bg-color *om-window-def-color*
           :action  #'(lambda (item) (declare (ignore item)) t))
          (om-make-dialog-item 'om-static-text (om-make-point 120 16) (om-make-point 225 16) 
                                                    (name self)
                                                    :bg-color *om-window-def-color*
                                                    :font *om-default-font2b*)
          (om-make-dialog-item 'om-static-text (om-make-point 15 50) (om-make-point 60 18) "Type : "
                                                    :bg-color *om-window-def-color*)
         (om-make-dialog-item 'om-static-text (om-make-point 70 50) (om-make-point 140 18) (get-object-insp-name self)
                                                   :bg-color *om-window-def-color*)
          (om-make-dialog-item 'om-static-text (om-make-point 15 80) (om-make-point 130 18) 
                                                    "Documentation : " :bg-color *om-window-def-color*)))

(defmethod get-om-properties ((self t)))

(defmethod get-om-properties ((self ombasicobject))
   (if (protected-p self)
     (unactive-icon self)
     (active-icon self)))

(defmethod get-om-properties ((self omfuncallablebasicobject))
   (if (protected-p self)
     (unactive-icon self)
     (active-icon self)))

(defmethod get-om-properties ((self ompackage))
   (if (protected-p self)
     (unactive-icon self)
     (call-next-method)))

(defmethod get-om-properties ((self omglobalsfolder))
   (unactive-icon self))

(defvar *om-patch-lispbox-disable* " a variable to set to nil to avoid the check box action to be called")
(setf *om-patch-lispbox-disable* nil)

(defmethod get-om-properties ((self ompatch))
   (if (equal (type-of self) 'ompatch)
     (list+ (call-next-method)
            (list (om-make-dialog-item 'om-check-box 
                                       (om-make-point 140 50) (om-make-point 225 18) "Lisp Code"
                                       :di-action (om-dialog-item-act item
                                                    (unless *om-patch-lispbox-disable*
                                                      (if   (om-y-or-n-dialog (format nil "Switching from lisp code to graphic program~%or from garphic program to lisp code~%will erase the current patch contents.~%Are you sure ?"))
                                                        (progn
                                                          (when (editorframe self)
                                                            (om-close-window (window (editorframe self))))
                                                          (setf (lisp-exp-p self) (not (lisp-exp-p self)))
                                                          (when (om-checked-p item)
                                                            (edit-new-lambda-expression self)))
                                                        (progn
                                                          (setf *om-patch-lispbox-disable* t)
                                                          (om-set-check-box item (not (om-checked-p item)))
                                                          (setf *om-patch-lispbox-disable* nil))
                                                        )))
                                       :checked-p (lisp-exp-p self)
                                       :bg-color *om-window-def-color*)))
     (call-next-method)))


(defmethod get-om-properties ((self OMBasictype))
   (unactive-icon self))

(defmethod get-om-properties ((self OMMethod))
   (unactive-icon self))

(defmethod get-om-properties ((self OMInstance))
   (unactive-icon self (om-make-point 250 150)))

(defmethod get-om-properties ((self OMBox))
   (unactive-icon self (om-make-point 250 150)))

(defmethod get-om-properties ((self OMboxclass))
   (let ((class (find-class (reference self))))
     (if (protected-p class)
       (unactive-icon class)
       (active-icon class))))

(defmethod get-om-properties ((self OMboxalias))
   (get-om-properties (reference self)))

(defmethod get-om-properties ((self OMboxtypecall))
   (list (om-make-point 250 100) 
         (om-make-dialog-item 'om-static-text (om-make-point 15 40) (om-make-point 220 50) 
                           "This box containts a constant value"
                                                    :font *om-default-font1*
                                                   :bg-color *om-window-def-color*)
          (om-make-dialog-item 'om-static-text (om-make-point 15 20) (om-make-point 130 12) "Documentation : "
                                                     
                                                    :bg-color *om-window-def-color*)))

(defmethod get-om-properties ((self OMslot))
   (let ((class (find-class (classname self))))
     (if (protected-p class)
       (unactive-icon self)
       (active-icon self ))))


(defmethod get-om-properties ((self omboxeditcall))
  (append (call-next-method)
          (list (om-make-dialog-item 'om-editable-text (om-make-point 120 16) (om-make-point 80 20)
                                     (name self)
                                     :di-action (om-dialog-item-act item
                                                  (setf (name self) (om-dialog-item-text item)))))))

(defmethod uddate-close ((self omboxeditcall) win)
  (when (car (frames self)) (om-invalidate-view (car (frames self)))))


(omg-defclass info+offset-window (info-window)
  ((posx :initform nil :initarg :posx :accessor posx)
   (changes :initform nil  :accessor changes)))


(defmethod uddate-close ((self TemporalBox) win)
  (let ((items (posx win))
        (changs (remove-duplicates (changes win)))
        (oldextend (extend self)))
    (loop for item in changs do
          (case item
            (0  (setf (offset self) (read-info-metric? self (read-from-string (om-dialog-item-text (first items))))))
            (1 (setf (posy self) (read-from-string (om-dialog-item-text (second items)))))
            (2 (setf (strech-fact self) (/ (read-from-string (om-dialog-item-text (third items))) oldextend)))
            (3 (setf (sizey self) (read-from-string (om-dialog-item-text (fourth items)))))
            (4 (setf (name self) (om-dialog-item-text (fifth items)))
             (when (car (frames self)) (om-invalidate-view (car (frames self)))))
            ))))

(defmethod pos-info-metric? ((self TemporalBox) val)
  (if (string-equal (maq-show-ruler (mycontainer self)) "off") val
      (let* ((themaq (mycontainer self))
             (params (metricparam (params themaq))))
        (ms2listmetric val (first params) (second params) (third params) (fourth params)))))


(defmethod read-info-metric? ((self TemporalBox) val)
  (if (listp val)
    (let* ((themaq (mycontainer self))
           (params (metricparam (params themaq))))
      (listmetric2ms val (first params) (second params) (third params) (fourth params)))
    val))

(defmethod show-info-window ((self TemporalBox))
  (if (infowin self)
    (om-select-window (infowin self))
    
    (let* ((info (get-om-properties self))
           (sizex (om-make-dialog-item 'om-editable-text (om-make-point 60 150) (om-make-point 45 20) 
                                       (format () "~D" (if (strech-fact self)
                                                         (round (* (extend self) (strech-fact self)))
                                                         (extend self)))
                                       :di-action (om-dialog-item-act item
                                                    (push 2 (changes (om-view-container item)))) :font *om-default-font1*))
           (posy (om-make-dialog-item 'om-editable-text (om-make-point 180 150) (om-make-point 45 20) (format () "~D" (posy self))
                                      :di-action (om-dialog-item-act item
                                                   (push 1 (changes (om-view-container item)))) :font *om-default-font1*))
           (posx (om-make-dialog-item 'om-editable-text (om-make-point 60 185) (om-make-point 45 20) (format () "~D" (pos-info-metric? self (offset self)))
                                      :di-action (om-dialog-item-act item
                                                   (push 0 (changes (om-view-container item)))) :font *om-default-font1*))
           (sizey (om-make-dialog-item 'om-editable-text (om-make-point 180 185) (om-make-point 45 20) (format () "~D" (sizey self))
                                       :di-action (om-dialog-item-act item
                                                    (push 3 (changes (om-view-container item)))) :font *om-default-font1*))
           (name (om-make-dialog-item 'om-editable-text (om-make-point 120 16) (om-make-point 80 20) (name self) 
                                                    :di-action (om-dialog-item-act item
                                                                 (push 4 (changes (om-view-container item))))
                                                    :font *om-default-font2b*))
           (dialog (om-make-window 'info+offset-window
                                   :position (om-make-point 30 50)
                                   :sujet self
                                   :posx (list posx posy sizex sizey name)
                                   :size (car info)
                                   :window-title (string+ (name self) " Info")
                                   :resizable nil :minimize nil :maximize nil
                    :font *om-default-font1*
                    :bg-color *om-window-def-color*)))
      (eval `(om-add-subviews ,dialog ,.(cdr info) ,.(posx dialog)))
      (setf (infowin self) dialog)
      (om-select-window dialog)
      )
    ))



(defmethod get-om-properties ((self TemporalBox))
   (list (om-make-point 250 250)
         (om-make-dialog-item 'om-static-text (om-make-point 15 100) (om-make-point 220 50) 
                           (string-until-CR (get-documentation self))
                                                    :font *om-default-font1*
                                                   :bg-color *om-window-def-color*)
          (om-make-dialog-item 'om-static-text (om-make-point 15 16) (om-make-point 70 16) "Name : " 
                                                    :bg-color *om-window-def-color*)
          ;;;(om-make-dialog-item 'om-static-text (om-make-point 120 16) (om-make-point 85 16) (name self) 
          ;;;                                          :bg-color *om-window-def-color* :font *om-default-font2b*)
          (om-make-dialog-item 'om-static-text (om-make-point 15 50) (om-make-point 60 16) "Type : " 
                                                    :bg-color *om-window-def-color*)
         (om-make-dialog-item 'om-static-text (om-make-point 70 50) (om-make-point 140 16) (get-object-insp-name self) 
                                                   :font *om-default-font2* :bg-color *om-window-def-color*)
         (om-make-dialog-item 'om-static-text (om-make-point 5 151) (om-make-point 45 16) "Dur : "  :bg-color *om-window-def-color*)
         (om-make-dialog-item 'om-static-text (om-make-point 125 151) (om-make-point 45 16) "Pos y : "  :bg-color *om-window-def-color*)
         (om-make-dialog-item 'om-static-text (om-make-point 5 187) (om-make-point 45 16) "Offset : "  :bg-color *om-window-def-color*)
         (om-make-dialog-item 'om-static-text (om-make-point 125 187) (om-make-point 70 16) "Size y : "  :bg-color *om-window-def-color*)
         (om-make-dialog-item 'om-static-text (om-make-point 15 80) (om-make-point 130 16) "Documentation : "  :bg-color *om-window-def-color*)))



(defmethod get-om-properties ((self temp-marker))
   (list (om-make-point 250 240)
         (om-make-dialog-item 'om-static-text (om-make-point 15 100) (om-make-point 220 60) 
                           "You can connect this marker (Shift-key + Drag) to the end or the offset of a temporal box"
                                                    :font *om-default-font1*
                                                   :bg-color *om-window-def-color*)
          (om-make-dialog-item 'om-static-text (om-make-point 15 16) (om-make-point 70 12) "Name : " 
                                                    :bg-color *om-window-def-color*)
          (om-make-dialog-item 'om-static-text (om-make-point 120 16) (om-make-point 225 12) (name self)  
                                                    :font *om-default-font2b*
                                                    :bg-color *om-window-def-color*)
          (om-make-dialog-item 'om-static-text (om-make-point 15 50) (om-make-point 60 12) "Type : " 
                                                    :bg-color *om-window-def-color*)
         (om-make-dialog-item 'om-static-text (om-make-point 70 50) (om-make-point 140 12) "Marker" 
                                                   :font *om-default-font2*
                                                   :bg-color *om-window-def-color*)
          (om-make-dialog-item 'om-static-text (om-make-point 15 180) (om-make-point 45 12) "Offset : " 
                                                    :bg-color *om-window-def-color*)
          (om-make-dialog-item 'om-static-text (om-make-point 15 80) (om-make-point 130 12) "Documentation : " 
                                                    :bg-color *om-window-def-color*)))



(defmethod show-info-window ((self temp-marker))
  (if (infowin self)
    (om-select-window (infowin self))
    
    (let* ((info (get-om-properties self))
           (posx (om-make-dialog-item 'om-editable-text (om-make-point 80 176) (om-make-point 45 20) (format () "~D" (offset self))
                                      :di-action (om-dialog-item-act item
                                                   (push 0 (changes (om-view-container item))))))
           (dialog (om-make-window 'info+offset-window
                                   :position (om-make-point 30 50)
                                   :sujet self
                                   :posx (list posx)
                                   :size (car info)
                                   :window-title (string+ (name self) " Info")
                                   :font *om-default-font2*
                                   :bg-color *om-window-def-color*)))
      (eval `(om-add-subviews ,dialog ,.(cdr info) ,.(posx dialog)))
      (setf (infowin self) dialog)
      (om-select-window dialog)
      )
    ))

(defmethod uddate-close ((self temp-marker) win)
  (let ((items (posx win))
        (changs (remove-duplicates (changes win))))
    (loop for item in changs do
          (case item
            (0  (change-offset self (read-from-string (om-dialog-item-text (first items)))))))))

