;OpenMusic
;
;Copyright (C) 1997, 1998, 1999, 2000 by IRCAM-Centre Georges Pompidou, Paris, France.
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
;
;Authors: Gerard Assayag and Augusto Agon


;DocFile
;Preference editor is defined in this file.
;Last Modifications :
;18/10/97 first date.
;DocFile

(in-package :om)

;Kept for compatibility
(defvar *current-pref* nil)

(defclass ompreferences ()
   ((midipref :initform nil :initarg :midipref :accessor midipref)
    (muspref :initform nil :initarg :muspref :accessor muspref)
    (genpref :initform nil :initarg :genpref :accessor genpref)
    (quanti :initform nil :initarg :quanti :accessor quanti)))

;New var
(defvar *saved-pref* nil)

(defmethod get-default-scor-params ((self t)) nil)
(defvar *pref-item-list* nil)



;=================================================
;PREFERENCES MODULE CLASS
;=================================================

(omg-defclass pref-icon-view (choise-icon-button) 
   ((defvals :initform nil :initarg :defvals :accessor defvals))
   (:default-initargs :action #'(lambda (item) (update-pref-but item))))

(defmethod save-pref ((iconid number) (item pref-icon-view))
   (list (iconID item) (omng-save (defvals item))))


(defmethod update-pref-but ((item pref-icon-view))
   (let* ((win (editor (om-view-window item)))
          (old-select (get-pref-by-icon  (selected-icon win)))
          (numpref (if (listp (iconID item)) (car (iconID item)) (iconID item)))
          newscroll)
     (unless (= numpref (selected-icon win))
       (setf newscroll (make-new-pref-scroll win numpref))
       (setf (selected-p old-select)  nil)
       (om-invalidate-view old-select t)
       (om-remove-subviews win (secondscroll win))
       (om-add-subviews win newscroll)
       (setf (secondscroll win) newscroll)
       (setf (selected-icon win) numpref)
       (setf (selected-p item) t)
       (om-invalidate-view item t)
       (om-set-interior-size (om-view-window item) (om-view-size win)))))

(defun get-pref-by-icon (x &optional preflist)
   (unless preflist
     (setf preflist *pref-item-list*))
   (find-if #'(lambda (item) (if (listp (iconID item))
                               (= (car (iconID item)) x)   
                               (= (iconID item) x))) preflist))

(defmethod corrige-preference-list ((iconID t) list) list)

(defvar *restore-defaults* nil)

(defmethod restore-preferences ()
   (when *saved-pref*
     (setf *restore-defaults* nil)
     (loop for item in *saved-pref* do
           (let ((modulepref (get-pref-by-icon (car item))))
             (when modulepref
               (setf (defvals modulepref) (corrige-preference-list (car item) (eval (second item))))
               (put-preferences (car item)))))
     (when (and *restore-defaults*
                (om-y-or-n-dialog 
                 (let ((str (format nil "User preferences - ~%The following folders could not be found:~%")))
                   (loop for f in *restore-defaults* do (setf str (string+ str "- " f)))
                   (setf str (string+ str (format nil "~%Default locations were restored. Save new locations?"))))))
       (save-preferences))
     (setf *restore-defaults* nil)))


(defmethod put-all-preferences ()
   (loop for item in *pref-item-list* do
         (put-preferences (iconID item))))


(defmethod put-default-preferences ()
  (loop for item in *pref-item-list* do
         (setf (defvals item) (get-def-vals (iconID item)))
         (put-preferences (iconID item))))

(defmethod save-preferences ()
  (let ((path (make-pathname :directory (pathname-directory (mypathname *current-workSpace*)) :name "preferences" :type "lisp")))
    (delete-file-protection path)
    (WITH-OPEN-FILE (out path :direction :output 
                         :if-does-not-exist :create :if-exists :supersede) 
      (write-line (format nil ";~D" *om-version*) out)
      (prin1 '(in-package :om) out)
      (prin1 `(setf *saved-pref* ',(remove nil
                                           (loop for item in *pref-item-list*
                                                  collect (save-pref (iconid item) item)))) out))))


(defun clone-pref ()
   (loop for item in *pref-item-list*
         collect (om-make-view (type-of item)
                   :iconID (iconID item)
                   :size (om-view-size item)
                   :position (om-view-position item)
                   :defvals (copy-list (defvals item)))))


;=================================================
;PREFERENCES WINDOW and EDITOR CLASS
;=================================================

(defvar *pref-window* nil)
(defvar *pref-window-size* nil)
(setf *pref-window-size* (om-make-point 410 350))


;(setf *pref-window* nil)


;EDITOR
(omg-defclass preferenceEditor (metaobj-editor)  
   ((selected-icon :initform 214 :initarg :selected-icon :accessor selected-icon)
    (secondscroll :initform nil :initarg :secondscroll :accessor secondscroll)))

;PANEL
(omg-defclass Preferencepanel (metaobj-panel) ()
   ;;; (:default-initargs :scrollbars :v :retain-scrollbars t)
   )

(defmethod metaobj-scrollbars-params ((self preferenceEditor))  '(:v t))

(defmethod close-editor-before ((self preferenceEditor))  
   (setf (object (panel self))  nil))

(defmethod om-draw-contents :after ((self preferenceEditor))  
   (om-with-focused-view self
     (om-draw-line 66 (- (h self) 50) 
                (w self) (- (h self) 50))))

(defmethod get-pref-scroll-size ((win preferenceEditor))
   (om-make-point (- (w win) 66) (- (h win) 50)))

(defmethod om-set-view-size ((self preferenceEditor) size)
   (declare (ignore size))
   (call-next-method)
   (when (secondscroll self) 
       (om-set-view-size (secondscroll self) (om-make-point (- (w self) 66) (- (h self) 50)))
       (om-set-field-size (secondscroll self) (om-make-point (- (w self) 66) (- (h self) 50))))
   (when (panel self) 
       ;(om-set-view-size (panel self) (om-make-point 50 (h self)))
       (om-set-view-size (panel self) (om-make-point 65 (h self)))
       )
   )


(defmethod get-editor-panel-class ((self preferenceEditor))  'preferencepanel)

(defmethod get-editor-class ((self list))  (get-editor-class (car self)))
(defmethod get-editor-class ((self pref-icon-view))  'preferenceeditor)


(defun make-preference-win ()
   (let* (b-posy
            (elements *pref-item-list*) newwindow secondscroll)
     (setf newwindow (make-editor-window 'preferenceEditor (clone-pref) "preferences" nil 
                                                                               :winsize *pref-window-size* 
                                                                               :winpos (om-make-point 100 50) :close-p nil :resize nil))
     (setf b-posy (- (om-point-v (om-interior-size newwindow)) 40))
     (om-set-font newwindow *om-default-font2*)
     (setf (selected-p (car elements)) t)
     (mapc #'(lambda (elem) (om-add-subviews (panel newwindow) elem)) elements)
     (setf secondscroll (make-new-pref-scroll (editor newwindow) 214))
     (om-add-subviews (editor newwindow) secondscroll)
     (setf (secondscroll (editor newwindow)) secondscroll)

     (om-add-subviews (editor newwindow) 
                      (om-make-dialog-item 'om-button (om-make-point 75 b-posy) (om-make-point 70 22) "OK" 
                                           :di-action (om-dialog-item-act  item
                                                        (setf *pref-item-list* (object (editor (om-view-window item))))
                                                        (put-all-preferences)
                                                        (om-close-window (om-view-window item))
                                                        )
                                           :default-button t)
                      
                      (om-make-dialog-item 'om-button (om-make-point 160 b-posy) (om-make-point 70 22) "cancel" 
                                           :di-action (om-dialog-item-act  item
                                                        (om-close-window (om-view-window item))
                          ))
                      (om-make-dialog-item 'om-button (om-make-point 245 b-posy) (om-make-point 70 22) "defaults" 
                                           :di-action (om-dialog-item-act item
                                                        (put-default-preferences)
                          (om-close-window (om-view-window item))
                          ))
                      (om-make-dialog-item 'om-button (om-make-point 330 b-posy) (om-make-point 70 22) "save" 
                                           :di-action (om-dialog-item-act item
                                                        (setf *pref-item-list* (object (editor (om-view-window item))))
                                                        (put-all-preferences)
                                                        (save-preferences)
                                                        (om-close-window (om-view-window item))
                                                        )))
     ;(om-set-interior-size newwindow *pref-window-size*)
     newwindow))

(defun omG-make-preference-dialog ()
   (if (and *pref-window* (om-window-open-p *pref-window*))
     (om-select-window *pref-window*)
     (setf *pref-window* (make-preference-win))))




;=================================================
;DEFAULT PREFERENCES MODULE  icon 214
;=================================================

(defvar *composer-name* nil)
(defvar *delete-file* t)
(defvar *comment-style* nil)

(defvar *msg-error-label-on* t)

(setf *composer-name* "Guarigocha")
;(setf *delete-file* nil)
(setf *comment-style* *om-default-font2*)

(setf *msg-error-label-on* t)

(defvar *comment-color* nil)
(defvar *ws-color* nil)

(defun init-om-pref-color ()
  (setf *comment-color* *om-black-color*)
  (setf *ws-color* *workspace-color*))

(om-add-init-func 'init-om-pref-color) 

(defun init-preferences ()
   (setf *pref-item-list* nil)
   (setf *pref-window* nil)
   (pushr 
    (om-make-view 'pref-icon-view :iconID 214 :size (om-make-point 40 40) :position (om-make-point 5 15)
        :defvals (list nil t *om-default-font2* (om-make-color 0 0 0) "Guarigocha" *workspace-color*)) 
    *pref-item-list*)
   )


(om-add-init-func 'init-preferences)


(defmethod put-preferences ((iconID (eql 214)))
   (let ((modulepref (get-pref-by-icon iconID)))
     ;(setf *delete-file*           (first (defvals modulepref)))
     (setf *msg-error-label-on*    (second (defvals modulepref)))
     (setf *comment-style*         (eval (third (defvals modulepref))))
     (setf *comment-color*         (eval (fourth (defvals modulepref))))
     (setf *composer-name*         (or (fifth (defvals modulepref)) "Guarigocha"))
     (setf *ws-color*         (sixth (defvals modulepref)))
     (when *om-workspace-win* (om-set-bg-color (panel *om-workspace-win*) *ws-color*))
     ))


(defmethod put-preferences ((iconID list))
   (put-preferences (car iconID)))


(defmethod corrige-preference-list ((iconID (eql 214)) list)
  ;;;(when (stringp (car (nth 2 list)))
  ;;;  (setf (nth 2 list) `(om-make-font ,(first (nth 2 list)) ,(second (nth 2 list)))))
   (let (rep)
     (loop for i = 0 then (+ i 1)
           for newitem in (get-def-vals iconID) do
           (if (> (length list) 0)
             (push (pop list) rep)
             (push newitem rep)))
     (reverse rep)))


(defmethod get-def-vals ((iconID (eql 214)))
   (list nil t *om-default-font2* (om-make-color 0 0 0) "Guarigocha" *workspace-color*))


(defmethod get-def-vals ((iconID list))
   (get-def-vals (car iconID)))


(defmethod make-new-pref-scroll ((win preferenceEditor) (num (eql 214)))
  (let ((thescroll (om-make-view 'PreferencePanel
                                 :size (get-pref-scroll-size win) 
                                 :position (om-make-point 66 0)
                                 :font *om-default-font2* 
                                 :scrollbars :v 
                                 :retain-scrollbars t
                                 ))
        (modulepref (get-pref-by-icon num (object win)))
        (test-comment (om-make-dialog-item 'om-static-text (om-make-point 180 180) (om-make-point 200 40) "Comment..."
                                           :font *comment-style*
                                           :fg-color *comment-color*))
        (test-wscolor (om-make-view 'om-view :position (om-make-point 180 220) :size (om-make-point 40 20) 
                                    :bg-color *ws-color*))
        )
    (om-add-subviews thescroll 
                     (om-make-dialog-item 'om-check-box (om-make-point 5 40) (om-make-point 180 15) "Message Error handle" 
                                          :di-action (om-dialog-item-act item 
                                                       (setf (nth 1 (defvals modulepref)) (om-checked-p item)))
                                          :font *om-default-font2*
                                          :checked-p *msg-error-label-on*)
                     #|
(om-make-dialog-item 'om-check-box (om-make-point 5 15) (om-make-point 180 15) "Special file writer" 
                                                                                   :di-action (om-dialog-item-act item
                                                                                                (setf (nth 0 (defvals modulepref)) (om-checked-p item)))
                                                 :font *om-default-font2*
                                                                                   :checked-p *delete-file*)
|#
                     (om-make-dialog-item 'om-static-text (om-make-point 5 104) (om-make-point 90 24) "User Name"
                                          :font *om-default-font2*) 
                     (om-make-dialog-item 'om-editable-text (om-make-point 100 100) (om-get-text-edit-size (om-make-point 200 15)) *composer-name* 
                                          :di-action (om-dialog-item-act item 
                                                       (setf (nth 4 (defvals modulepref)) (om-dialog-item-text item)))
                                          :font *om-default-font2*
                                          )
                     (om-make-dialog-item 'om-button (om-make-point 5 160) (om-make-point 125 24) "Comment Style"
                                          :di-action (om-dialog-item-act item
                                                       (declare (ignore button))
                                                       (let* ((font (om-choose-font-dialog :font *comment-style*)))
                                                         (when font
                                                           (setf (nth 2 (defvals modulepref)) font)
                                                           (om-set-font test-comment font)))))
                     (om-make-dialog-item 'om-button (om-make-point 5 190) (om-make-point 125 24) "Comment Color"
                                          :di-action (om-dialog-item-act item
                                                       (declare (ignore button))
                                                       (let* ((newcolor (om-choose-color-dialog)))
                                                         (when newcolor
                                                           (setf (nth 3 (defvals modulepref)) newcolor)
                                                           (om-set-part-color test-comment :text newcolor)))))
                     
                     test-comment
                     
                     (om-make-dialog-item 'om-button (om-make-point 5 220) (om-make-point 125 24) "WorkspaceColor"
                                          :di-action (om-dialog-item-act item
                                                       (declare (ignore button))
                                                       (let* ((newcolor (om-choose-color-dialog :color (eval (nth 5 (defvals modulepref))))))
                                                         (when newcolor
                                                           (setf (nth 5 (defvals modulepref)) newcolor)
                                                           (om-set-bg-color test-wscolor newcolor)
                                                           ))))
                     test-wscolor
                     
                     )
    thescroll))
















