;OpenMusic
;
;Copyright (C) 1997, 1998, 1999, 2000 by IRCAM-Centre Georges Pompidou, Paris, France.
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
;
;Authors: Gerard Assayag and Augusto Agon


;DocFile
;Patch editor definition.
;Last Modifications :
;18/10/97 first date.
;DocFile
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(in-package :om)

;---------------------------------------------------------------------
;EDITOR
;---------------------------------------------------------------------

(omg-defclass patchEditor (relationEditor)  ()
   (:documentation "This is the class for windows containing a patchPanel, so windows for Patch's editors.#enddoc#
#seealso# (patchPanel OMPatch) #seealso#"))


(defmethod get-editor-panel-class ((self patchEditor))  'patchPanel)

(defmethod key-event-around ((self patchEditor) char) 
   (let ((container (panel self)))
     (if (text-view self)
       t
       (case char
         (#\y (select-a-picture container) nil)
         (otherwise t)))))


(defmethod editor-make-new-icon-window ((self patchEditor) &optional type)
  (cond 
   ((equal type 'f) 
    (let ((diag (get-funname-dialog)) string iconid doc)
      (when diag
        (without-interrupts
         (setf string (first diag))
         (setf doc (second diag))
         (setf iconid (third diag))
         (when (string-equal doc "")
           (setf doc "non documentation"))
         (when (and string (not (string-equal "" string)))
           (if (fboundp (interne string))
             (dialog-message "This function exists already")
             (make-new-genfunwin self string iconid doc)))))))))

;---------------------------------------------------------------------
;PANEL
;---------------------------------------------------------------------

(omg-defclass patchPanel (relationPanel) 
   ()
   (:documentation "Patch editors are instance of this class.
Elements of patchPanels are instace of the boxframe class.#enddoc#
#seealso# (OMPatch boxframe) #seealso#"))

(defmethod patchpanel-p ((self t)) nil)
(defmethod patchpanel-p ((self patchpanel)) t)

(defmethod find-the-view-of-patch ((self patchPanel))
   (let (rep)
     (loop for box in (get-subframes self)
           while (not rep) do
           (when (and (has-mini-pict? (object box))
                      (view-of-patch (object box)))
             (setf rep box)))
     rep))

(defmethod find-the-box-view-of-patch ((self OMPatch))
   (let (rep)
     (loop for box in (boxes self)
           while (not rep) do
           (when (and (has-mini-pict? box)
                      (view-of-patch box))
             (setf rep box)))
     rep))


(defmethod om-view-click-handler :before ((self patchPanel) where)
   (declare (ignore where))
   (modify-patch self))

(defmethod om-view-click-handler ((view patchPanel) where)
  (if *adding-a-box*
      ;;; a box is being added ...
    (progn
      (when (equal (window view) (second *adding-a-box*))
        (let* ((window (second *adding-a-box*))
               (item (first *adding-a-box*))
               (thepatch (panel window))
               (newbox (omNG-make-new-boxcall item (om-make-point 22 22) (mk-unique-name thepatch (name item))))
               pospanel pos)
          (if *new-obj-initial-pos*
              ;;; CASE 1 : a box is added from context-menu 
              (progn
                (setf (frame-position newbox) *new-obj-initial-pos*)
                (omG-add-element thepatch (make-frame-from-callobj newbox))
                )
            ;;; CASE 2 : a box is added from window menu 
            (progn (setf pospanel (om-mouse-position thepatch)
                         pos (om-mouse-position window))
              (when (om-point-in-rect-p pos (om-pts-to-rect (om-make-point 0 0) (om-view-size window)))
                (setf (frame-position newbox) (borne-position pospanel))
                (omG-add-element thepatch (make-frame-from-callobj newbox)))
              ))))
      (setf *adding-a-box* nil)
      (setf *new-obj-initial-pos* nil)
      )
    (call-next-method)))



(setf *patchhelp1* '((lrud "Move Selected Boxes")
                     (shift+lrud "Move Faster")
                     (del "Delete Selected Boxes")
                     (p-s "Play/Stop Selected Boxes")
                     (b "Show/Hide Button in Boxes")
                     (a "Make a Patch Abstraction")
                     (v "Eval Selected Boxes")
                     (alt+lr "Add/Remove Optional Inputs")
                     (k-kk "Add/Remove Keyword Inputs")))

(setf *patchhelp2* '((c "Change Connection Color")
                     (d "Show Doc")
                     (e "Edit Lisp Source Code")
                     (dd "reDraw All")
                     (i "Reinitialize Selection")
                     (m "Show/Hide Mini View")
                     (mm "Show/Hide Mini View for All Boxes")
                     (h "Show this Help window")
                     ))


(defmethod handle-key-event ((self patchPanel) char) 
   (case char
      (:om-key-delete (delete-general self))
      (#\D  (om-invalidate-view self t))
      (#\d  (mapc 'show-big-doc (get-actives self)))
      (#\c  (color-connections self))
      (#\e (mapc 'show-fun-code (get-actives self)))
      (#\v (om-eval-enqueue 
              `(progn
                (mapc 'eval-box ',(get-actives self))
                (clear-ev-once ,self))))
      (#\h  (show-help-window "Patch Help" (list *patchhelp1* *patchhelp2*)))
      (#\k (mapc 'add-keywords (get-actives self)))
      (#\K (mapc 'erase-keywords (get-actives self)))
      (#\i (mapc 'reinit-size (get-actives self)) (reinit-connections self))
      (#\r (mapc 'update-doc (get-actives self)))
      (#\b (mapc 'add-rem-lock-button (get-actives self)))
      (#\a (mapc 'internalize-patch (get-actives self)))
      (#\p (play-from-box (get-actives self)))
      (#\t (mapc 'show-online-tutorial (get-actives self)))
      (#\s (Stop-Player *general-player*))
      (#\m (mapc 'change-edit-mode (get-actives self)))
      (#\M (change-edit-mode-all (get-subframes self)))
      (:om-key-up (mapc #'(lambda (item) (move-frame-delta item 0)) (get-actives self))
                            (make-move-after self (get-actives self)))
      (:om-key-down (mapc #'(lambda (item) (move-frame-delta item 1)) (get-actives self))
                                     (make-move-after self (get-actives self)))
      (:om-key-left
       (let ((actives (get-actives self)))
         (if (om-option-key-p)
             (mapc #'(lambda (item) (delete-one-input item)) actives)
           (progn
              (mapc #'(lambda (item) (move-frame-delta item 3)) actives)
              (make-move-after self actives)))))
      (:om-key-right 
       (let ((actives (get-actives self)))
         (if (om-option-key-p)
             (mapc #'(lambda (item) (add-one-input item)) actives)
           (progn
              (mapc #'(lambda (item) (move-frame-delta item 2)) actives)
              (make-move-after self actives)))))
     (otherwise (om-beep))))



;--------------Shortcuts
;Open the little view allowing create a box in 'self' by name, 'x' is the position of the view.
(defmethod make-undefined-box ((self patchPanel) x)
   (let* ((thename (mk-unique-name self "undefined"))
            (new-box (omNG-make-new-boxcall 'undefined x thename))
            (new-frame (make-frame-from-callobj new-box)))
     (omG-add-element self new-frame)
     (open-ttybox (iconview new-frame))
     ))

(defmethod delete-general :before ((self patchPanel))
  (setf (undo (editor self)) (append (list 'remove)
                                   (loop for box in (get-actives self) collect
                                         (object box))))
  )

(defmethod delete-general ((self patchPanel))
   "Remove the selected boxes from 'self'."
  (let ((actives (get-actives self))
        (connections (get-actives-connections self)))
    (real-make-delete-before self actives)
    (mapc #'(lambda (connec) 
              (let* ((thebox (thebox connec))
                     (thein (nth (index connec) (inputs (object thebox)))))
                (remove-connection thebox (index connec))
                (setf (connected? thein) nil))) connections)
    (mapc #'(lambda (box) 
              (omg-remove-element self box)) actives)
    (om-invalidate-view self)))


(defmethod color-connections ((self patchPanel))
   "Set a new color to the selected connections in 'self'."
   (let ((connections (get-actives-connections self)))
     (mapc #'(lambda (connec) 
               (new-color-connection connec)) connections)))

(defmethod color-comments ((self patchPanel))
   "Set a new color to the selected comment boxes in 'self'."
   (let ((active-comments (loop for frame in (get-actives self)
                                when (commentframep frame) collect frame)) newcolor)
     (when active-comments
       (setf newcolor (om-choose-color-dialog))
       (when newcolor
         (mapc #'(lambda (frame) 
                   (comment-new-color frame newcolor)) active-comments)))))

(defmethod font-comments ((self patchPanel))
   "Set a new font to the selected comment boxes in 'self'."
   (let ((active-comments (loop for frame in (get-actives self)
                                when (commentframep frame) collect frame)) 
         newlist)
     (when active-comments
       (setf newfont (om-choose-font-dialog :font (textstyle (object (car active-comments)))))
       (when newfont
         (mapc #'(lambda (frame) 
                   (comment-new-style frame newfont)) active-comments)))))


(defmethod reinit-connections ((self patchPanel))
   "Set the color to black and the edited points to nil for selected connections in 'self'."
   (let ((connections (get-actives-connections self))
         correct)
     (mapc #'(lambda (self) 
               (setf (ccolor self) 0)
               (when (nth 3 (connected? (object (nth (index self) (inputframes (thebox self))))))
                 (setf (nth 3 (connected? (object (nth (index self) (inputframes (thebox self)))))) 0))
               (setf (nth 2 (connected? (nth (index self) (inputs (object (thebox self)))))) nil)
               (push (thebox self) correct)) connections)
     (mapc #'(lambda (box) 
               (redraw-connections box)) (remove-duplicates correct :test 'equal))))


;------------------------------

(defmethod clear-ev-once ((self patchPanel))
   "After one evaluation this methods set the ev-once flag of all boxes in ev-once mode to nil."
   (mapc #'(lambda (box)
             (clear-ev-once (object box))) (get-subframes self)))

(defmethod save-connections ((self patchPanel) (source patchPanel) list)
   "Save the connections beetwen boses of 'self' as a list."
   (declare (ignore source))
   (mk-connection-list list))

(defmethod remake-draggeds-connections ((self patchPanel) (source patchPanel) listobj list) 
   "Used to remake connections when you drag and drop a subset of boxes from 'self' to 'source'."
   (when (move-and-not-action source self)
     (let (boxes)
       (loop for item in listobj do
             (loop for input in (inputs item) do
                   (setf (connected? input) nil)))
       (remk-connections listobj (loop for con in list collect (load-connection con)))
       (setf boxes (get-elements (object self)))
       (mapc #'(lambda (elem)
                 (update-graphic-connections elem boxes)) (mapcar #'(lambda (x) (car (frames x))) listobj)))))

(defmethod modify-patch ((self patchPanel))
   "Set differents flag ON to say that the patch was modified."
   (setf (compiled? (object self)) nil)
   (setf (saved? (object self)) nil)
   (when (direct-window-p self)
     (om-set-window-title (window self) (string+ "^" (name (object self))))))

(defmethod add-window-buttons ((self patchPanel))
   "Add the input and output buttons at the top-button in 'self'."
   (om-add-subviews self (om-make-view 'button-icon
                                            :iconID 156
                                            :position (om-make-point 5 5)
                                            :size (om-make-point 24 24)
                                            :action
                                            #'(lambda(item) (declare (ignore item)) (add-output self nil))
                                            )
                 (om-make-view 'button-icon
                   :iconID 157
                   :position (om-make-point 30 5)
                   :size (om-make-point 24 24)
                    :action #'(lambda(item) (declare (ignore item)) (add-input self nil))
                    )))


(defmethod add-input ((self patchpanel) position)
  (let* ((boxes (get-subframes self)) 
         (i (length (list+ (find-class-boxes boxes 'selfInFrame) (find-class-boxes boxes 'InFrame))))
         (pos (or position (om-make-point (+ 5 (* i 50)) 45))))
    (omG-add-element self
                     (make-frame-from-callobj 
                      (make-new-patch-input (mk-unique-name self "input") i pos)))
    (set-field-size self)
    t))

(defmethod add-output ((self patchpanel) position)
  (let* ((boxes (get-subframes self)) 
         (i (length (list+ (find-class-boxes boxes 'tempOutFrame) (find-class-boxes boxes 'outFrame))))
         (pos (or position (om-make-point (+ 5 (* i 50)) 240))))
    (omG-add-element self
                     (make-frame-from-callobj 
                      (make-new-output (mk-unique-name self "output") i pos)))
    (set-field-size self)
    ))


;--------------------------------
;Tools
;--------------------------------

(defun mk-connection-list (list)
   "Save the connection beetwen boxes in 'list' as a list.
Elements of the list are list as (source-position source-output target-position target-input connection-points connection-color)."
  (let (rep)
    (loop for item in list
          for i from 0 to (length list) do
          (let ((inputs (inputs item))
                (j 0))
            (mapc #'(lambda (input)
                      (when (connected? input)
                        (let ((posi (position (first (connected? input)) list :test 'equal)))
                          (when posi
                            (push (list posi (second (connected? input)) i j (om-save-point-list (third (connected? input))) (fourth (connected? input))) rep))))
                      (incf j))
                  inputs)))
    (reverse rep)))


(defun remk-connections (listobj list)
   "Make connections beetwen box of 'listobj' connections are specified by 'list'."
   (loop for item in list do
         (when (and (nth (first item) listobj) (nth (third item) listobj))
           (omNG-connect (nth (first item) listobj)
                         (second item)
                         (nth (third item) listobj)
                         (fourth item)
                         (fifth item)
                         (sixth item)))))

(defun find-box-with-name (list name)
   "Return the box in 'list' named 'name'."
   (find-if #'(lambda (box) (string-equal (name box) name)) list))


(defun change-edit-mode-all (boxes)
   "Set the minipict mode of ALL editors ON or OFF."
   (let (firstval curval)
     (loop for item in boxes do
           (when (has-mini-pict? (object item))
             (unless firstval
               (setf firstval t
                     curval (showpict (object item))))
             (when (equal  curval (showpict (object item)))
               (change-edit-mode item))))))

;==========================================================================
;NEXT METHODS WERE ADDED FOR PICT MANAGEMENT
;==========================================================================


(defmethod om-draw-contents :before ((self patchPanel))
   (om-with-focused-view self
     (loop for item in (pictu-list (object self)) do
               (draw-pict-patch item self)))
   ) 

(defmethod select-a-picture ((self patchPanel))
   (let ((list (pictu-list (object self)))
         selected)
     (when list
       (loop for item in list
             for i = 0 then (+ i 1)
             while (not selected) do
             (when (selected-p item)
               (setf (selected-p item) nil)
               (setf selected i)))
       (cond
        ((null selected) (setf selected 0))
        ((= selected (- (length list) 1)) (setf selected 0))
        (t (setf selected (+ selected 1))))
       (setf (selected-p (nth selected list)) t)
       (om-invalidate-view self t))))

(defmethod get-selected-picts ((self patchPanel))
   (let ((list (pictu-list (object self))))
         (loop for item in list
               when (selected-p item) collect item)))
             
(defvar *pict-mov-size* nil)
(defvar *pict-mov-delta* nil)
(defmethod om-view-click-handler :around ((self patchPanel) where)
   (let ((sel (car (get-selected-picts self))))
     (om-with-focused-view self 
     (if sel
         (let ((pictsize (pict-size sel))
               (x0 (om-point-h (pict-pos sel)))
               (y0 (om-point-v (pict-pos sel))))
           (let ((r (om-make-rect x0 y0 (+ x0 (om-point-h pictsize)) (+ y0 (om-point-v pictsize)))))
             (if (om-point-in-rect-p where r) 
                 (if (om-control-key-p)
                     (popup-pict-menu-context sel self)  ;pour retourner en bote a faire
                   (progn
                     (setf *pict-mov-size* pictsize)
                     (setf *pict-mov-delta* (om-make-point (- (om-point-h where) x0) (- (om-point-v where) y0)))
                     (om-init-motion-functions self 'make-move-bg-pict 'release-move-bg-pict)
                     (om-new-movable-object self x0 y0 (om-point-h pictsize) (om-point-v pictsize) 'om-selection-rectangle)))
               (progn
                 (setf (selected-p sel) nil)
                 (om-invalidate-view self t)
                 (call-next-method)))))
       (call-next-method)))))

(defmethod make-move-bg-pict ((Self patchPanel) pos)
   (om-update-movable-object self (- (om-point-h pos) (om-point-h *pict-mov-delta*))
                                (- (om-point-v pos) (om-point-v *pict-mov-delta*))
                                (om-point-h *pict-mov-size*) (om-point-v *pict-mov-size*)))

(defmethod release-move-bg-pict ((Self relationPanel) pos)
  (om-erase-movable-object self)
   (let ((sel (car (get-selected-picts self))))
     (setf (pict-pos sel) (om-make-point (- (om-point-h pos) (om-point-h *pict-mov-delta*))
                                         (- (om-point-v pos) (om-point-v *pict-mov-delta*))))
     (om-invalidate-view self t)))


(defmethod delete-general :around ((self patchPanel))
   (let ((sel (car (get-selected-picts self))))
     (if sel
       (progn
         (setf (pictu-list (object self)) (remove sel (pictu-list (object self)) :test 'equal))
         ;;;(om-kill-picture (picthdler sel))
         (om-invalidate-view self t))
       (call-next-method))))




;---------------------------------------------------------------------------
; PATCH EDIT WIN
;---------------------------------------------------------------------------

(omg-defclass patch-lambda-exp-window (om-text-edit-window) 
   ((patchref :initform nil :initarg :patchref :accessor patchref))
   (:documentation "Special fred editor, used when the patch is defined by a lambda expression.#enddoc#
#seealso# (OMPatch) #seealso#
#patchref# The patch being edited. #patchref#"))


;;; ATTENTION : MARCHE PAS
(defmethod om-view-key-handler ((self patch-lambda-exp-window) char)
   (declare (ignore char))
   (setf (compiled? (patchref self)) nil)
   (setf (saved? (patchref self)) nil)
   (call-next-method))



(defmethod om-window-close-event ((self patch-lambda-exp-window))
  (handler-bind ((error #'(lambda (c) (declare (ignore c))
                            (om-select-window self)
                            (let ((rep  (om-y-or-n-dialog "Patch Definition Error. 
   Do you want to close the window?
   If yes no modification will be make to the patch")))
                              (when rep
                                (setf *abort-definition* t)
                                (om-close-window self))
                              (om-abort)))))
    (unless *abort-definition*
      (let* ((expression (om-get-lisp-expression self))
               )
        (unless (lambda-expression-p expression)
          (om-beep-msg "Error! this is not a lambda expression. Lambda expressions are of the form '(lambda <param-list> <body>)")
          (error "error"))
        (unless (equal (lisp-exp-p (patchref self)) expression)
          (setf (lisp-exp-p (patchref self)) expression)
          (eval `(defun ,(intern (string (code (patchref self))) :om)
                        ,.(cdr (lisp-exp-p (patchref self)))))
          
          (loop for item in (attached-objs (patchref self)) do
                (update-from-reference item)))))
    (setf (compiled? (patchref self)) t)
    (setf (editorframe (patchref self)) nil)
    (setf *abort-definition* nil)
    (call-next-method)))


(defmethod compile-without-close ((self patch-lambda-exp-window))
  (let* ((expression (read-from-string (om-get-lisp-expression self)))
           )
    (unless (lambda-expression-p expression)
      (om-beep-msg "Error! Expression in Patch definition is not a lambda expression. 
Lambda expression are the form '(lambda <param-list> <body>")
      (om-abort))
    (unless (equal (lisp-exp-p (patchref self)) expression)
      (setf (lisp-exp-p (patchref self)) expression)
      (eval `(defun ,(intern (string (code (patchref self))) :om)
                    ,.(cdr (lisp-exp-p (patchref self)))))
      (loop for item in (attached-objs (patchref self)) do
            (update-from-reference item)))
    (setf (compiled? (patchref self)) t)))

(defmethod edit-new-lambda-expression ((self ompatch))
   (setf (editorframe self)
      (om-make-window 'patch-lambda-exp-window
                                          :size (om-make-point 300 200)
                                          :patchref self
                                          :window-title (string+ "l-expression for " (name self))))
   ;(buffer-insert (fred-buffer (editorframe self)) (format nil " ~S" '(lambda () (om-beep))))
   (om-set-text (editorframe self) (format nil " ~S" '(lambda () (om-beep))))
   (setf (saved? self) nil)
   (om-invalidate-view (editorframe self) t))

(defmethod edit-old-lambda-expression ((self ompatch))
   (let ((editor (om-make-window 'patch-lambda-exp-window
                                                            :patchref self
                                                            :size (om-make-point 300 200)
                                                            :window-title (string+ "l-expression for " (name self)))))
     (om-set-text editor (format nil " ~S" (lisp-exp-p self)))
     (om-invalidate-view editor t)
     editor))



(defmethod om-view-cursor ((self patchPanel))
  (if (and *adding-a-box* (equal (window self) (second *adding-a-box*)))
    *om-box-cursor* 
    (call-next-method)))




;;;===============================================
;;;===============================================
;;; BUG FIX MCL ONLY :
;;; quand on passe vite sur un input
;;;===============================================
;;;===============================================
(defmethod window-update-cursor ((self EditorWindow) where)
   (if (editor-update-cursor (editor self) where)
     (call-next-method)))

(defmethod editor-update-cursor ((self EditorView) where) t)

(defmethod editor-update-cursor ((self patchEditor) where)
   (if (subviews self)
     (let ((container (panel self)))
       (if (and container (text-view self)
                (equal (class-name (class-of (text-view self))) 'text-enter-view)
                (view-contains-point-p (text-view self) where))
         (progn
           (exit-from-dialog (text-view self) (dialog-item-text (text-view self))) nil)
         t))
     t))
;;;===============================================
;;;===============================================
;;;===============================================