;OpenMusic
; 
;Copyright (C) 1997, 1998, 1999, 2000 by IRCAM-Centre Georges Pompidou, Paris, France.
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
;
;Authors: Gerard Assayag and Augusto Agon


;DocFile
;Package  editor definition.
;Last Modifications :
;18/10/97 first date.
;DocFile
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :om)


;---------------------------------------------

(omg-defclass Mult-scrollerEditor (nonrelationEditor) 
   ((current-scroll :initform 0  :accessor current-scroll)
    (scroll-list :initform nil  :accessor scroll-list))
   (:documentation "This is the class of windows containing one or more scrollers.#enddoc#
#seealso# (OMPackage Package-WinFrame PackagePanel) #seealso#
#presentation# 0 show normal icons, 1 show by list and 2 by type. #presentation#
#hardcopy-in-progress# A flag used for printing. #hardcopy-in-progress#"))


(defmethod get-editor-panel-class ((self Mult-scrollerEditor))  'Mult-scrollerPanel)


;---initialization
(defmethod initialize-instance :after ((self Mult-scrollerEditor) &key controls)
   (declare (ignore controls))
   (setf (scroll-list self) (list (slot-value self 'panel)))
   (setf (presentation self) 1))


(defmethod panel ((self Mult-scrollerEditor))
   "Return the current scroller of the window 'self'."
   (nth (current-scroll self) (scroll-list self)))


;;; enleve *15 au interior size
(defmethod UpDateScrollers ((self Mult-scrollerEditor))
   (let* ((scroll-list (scroll-list self))
          (size (om-view-size (nth 0 scroll-list))))
     (om-set-interior-size (window self) (om-make-point (* (om-point-h size) (length scroll-list)) 
                                              (om-point-v (om-view-size self))))
     (om-invalidate-view self t))
   )


(defmethod get-clipboard ((self Mult-scrollerEditor)) nil)

(defmethod editor-change-presentation ((self Mult-scrollerEditor) presentation)
   (declare (ignore presentation))
   (om-beep))

(defmethod close-editor-before ((self Mult-scrollerEditor))
   (call-next-method)
   (setf (current-scroll (editor self)) 0))


;;; enlev 15 aux tailles
(defmethod om-set-view-size ((self Mult-scrollerEditor) size-point)
   (declare (ignore size-point))
   (call-next-method)  
   (let* ((scroll-list (scroll-list self))
          (length (length scroll-list))
          size (i -1))
     (setf size (round (om-point-h (om-view-size self)) length))
     (mapc #'(lambda (scroll)
                       (om-set-view-size scroll (om-make-point size (om-point-v (panel-size self))))
                       (om-set-view-position scroll (om-make-point (* (incf i) size) (om-point-v (panel-position self))))
                       (when (nth i (titles self)) 
                           (om-set-view-position (nth i (titles self)) (om-make-point (+ 10 (* i size)) 4))
                           (om-set-view-size (nth i (titles self)) (om-make-point (- size 20) 32)))
                       ) 
         scroll-list)))


;------------------------------------------------------
;Package browser
;------------------------------------------------------


;------------
;EDITOR
;------------

(omg-defclass PackageEditor (Mult-scrollerEditor)  
   ((panel-title :initarg :panel-title :initform "" :accessor panel-title)
    (titles :initarg :titles :initform nil :accessor titles))
   (:documentation "This is the class of windows containing one or more scrollers.#enddoc#
#seealso# (OMPackage Package-WinFrame PackagePanel) #seealso#
#presentation# 0 show normal icons, 1 show by list and 2 by type. #presentation#
#hardcopy-in-progress# A flag used for printing. #hardcopy-in-progress#"))

(defmethod get-editor-panel-class ((self PackageEditor))  'PackagePanel)

(defmethod initialize-instance :after ((self PackageEditor)&key controls)
   (declare (ignore controls))
   (setf (panel-title self) (om-make-view 'om-view 
                                                                       :position (om-make-point 0 0) :size (om-make-point (om-point-h (om-view-size self)) 40)
                                                                       :bg-color  *om-window-def-color*
                                                                       :owner self))
   (setf (titles self) (list (make-panel-title self)))
   (om-add-subviews (panel-title self) (car (titles self)))
   )


;;; the left main title of the editor
;;; the other panel titles are defined with the corresponding icon frames (like make-new-scroll)
(defmethod make-panel-title ((self PackageEditor)) 
   (om-make-dialog-item 'om-static-text (om-make-point 10 4) (om-make-point 160 36)
                                             (string+ (format nil "Packages/Classes/Slots ~& of package ") (name (object self)))
                                     :font *om-default-font2b*
                                     :bg-color  *om-window-def-color*
                                     )
   )

(defmethod panel-position ((self PackageEditor)) (om-make-point 0 40))

(defmethod panel-size ((self PackageEditor)) 
   (om-make-point (w self) (- (h self) 40)))

(defmethod editor-make-new-icon-window ((self PackageEditor) &optional type)
   (let* ((scrollframe (nth 0 (scroll-list self)))
          new-object new-frame)
     (if (add-subpackages-p (object scrollframe))
       (when (equal type 'p) 
         (setf new-object (make-instance 'OMPackage :name (mk-unique-name  scrollframe "new package") :icon 159))
         (setf new-frame (make-icon-from-object  new-object 10 10 1 1))
           (omg-add-element scrollframe new-frame)
           ;;; test 
           (set-field-size scrollframe)
           ;;; 
           )
       (dialog-message "You can create subpackages only in the USER package or in its subpackages"))))
    
(defmethod make-new-scroll ((self t)) 
   "Open a new editor containing the simple frames elements of 'self'."
   nil)

(defmethod make-panel-title ((self t)) 
   "Creates a dialog item title for the editor of self"
   nil)

;------------
;PANEL
;------------

(omg-defclass PackagePanel (nonrelationPanel) ()
   ;;;(:default-initargs :retain-scrollbars t)
   (:documentation "This is the class for package's editors.
Packages panels contains icons of sub-packages, classes and sometimes slots.#enddoc#
#seealso# (OMPackage package-icon-frame class-icon-frame  slot-icon-frame) #seealso#"))

(defmethod metaobj-scrollbars-params ((self packageEditor))  '(t t))

(defmethod sort-subframes ((self PackagePanel) elements) elements)

(defmethod update-subviews ((self PackageEditor))
   (om-set-view-size (panel-title self) (om-make-point (w self) 40))
   (om-set-view-size (panel self) (panel-size self))
   (set-field-size (panel self )))


(defmethod om-view-cursor ((self PackagePanel))
   *om-arrow-cursor*)

(defmethod remove-scrolls ((editor packageEditor))      
   (mapc #'(lambda (scroll)
                     ;;;(apply 'om-remove-subviews editor (list (v-scroller scroll) (h-scroller scroll) scroll))
                     (apply 'om-remove-subviews editor (list scroll)))
       (subseq (scroll-list editor) (+ 1 (current-scroll editor))))
   (mapc #'(lambda (title) (om-remove-subviews (panel-title editor) title)) (subseq (titles editor) (+ 1 (current-scroll editor))))
   
   (setf (scroll-list editor) (subseq (scroll-list editor) 0 (+ 1 (current-scroll editor))))
   (setf (titles editor) (subseq (titles editor) 0 (+ 1 (current-scroll editor))))
   )

(defmethod om-view-click-handler ((view PackagePanel) where)
   (declare (ignore where))
   (when (equal view (call-next-method))
       (let ((win (om-view-container view))
               begin-remove)
         (setf (current-scroll win) (position view (scroll-list win) :test 'equal))
         (remove-scrolls win)     
         (UpDateScrollers win)
    )))

  