;OpenMusic
;
;Copyright (C) 1997, 1998, 1999, 2000 by IRCAM-Centre Georges Pompidou, Paris, France.
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
;
;Authors: Gerard Assayag and Augusto Agon


;DocFile
;Define a general class to PLAY.
;Last Modifications :
;17/11/97 first date.
;DocFile
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :om)

(defclass general-player ()
   ((playing-etat :initform "Idle" :accessor playing-etat)
    (list-to-play :initform nil :accessor list-to-play)
    (playing-list :initform nil :accessor playing-list)
    (cursor-pos :initform 0 :accessor cursor-pos)
    (delta-cursor-pos :initform 0 :accessor delta-cursor-pos)
    (recording-view :initform nil :accessor recording-view)
    (pausetime :initform 0 :accessor pausetime)
    (loopplay? :initform nil :accessor loopplay?)
    (forlooplist :initform nil :accessor forlooplist)
    (start-time :initform 0 :accessor start-time)
    (playedobjdur :initform 0 :accessor playedobjdur)
    (assoc-players :initform nil :accessor assoc-players)))

;etats "Idle" "Playing" "Pause"

(defvar *general-player* (make-instance 'general-player))

(defmethod Idle-p ((self general-player))
  (and (string-equal (playing-etat self) "Idle")
       (Idle-p (assoc-players self))))


(defmethod reset-player ((self t)  &optional view) nil)

(defmethod reset-player ((self general-player) &optional view)
   (setf (playing-etat self) "Idle")
   (setf (playing-list self) nil)
   (setf (forlooplist self) nil)
   (setf (list-to-play self) nil)
   (loop for player in (assoc-players self) do
         (reset-player player))
   (if (equal view (recording-view *general-player*))
     (setf (recording-view self) nil))
   (setf (cursor-pos self) 0)
   (setf (delta-cursor-pos self) 0)
   (setf (playedobjdur self) 0)
   (setf (pausetime self) 0)
   (reset-scheduler))

(defmethod get-player-etat ((self general-player))
   (playing-etat self))

(defmethod set-player-etat ((self general-player) etat)
   (setf (playing-etat self) etat)) 

(defmethod add-assoc-player ((self general-player) newplayer)
  (unless (member newplayer (assoc-players self))
    (pushr newplayer (assoc-players self))))

(defmethod remove-assoc-player ((self general-player) player)
  (remove player (assoc-players self)))

(defmethod Play-player ((self general-player))
   (when (or (list-to-play self) (assoc-players self))
     (setf (playing-etat self) "Playing")
     (setf (cursor-pos self) 0)
     (setf (pausetime self) 0)
     (setf (forlooplist self) (copy-list (list-to-play self)))
     (loop for player in (assoc-players self) do
           (play-player player))
     (without-interrupts
      (start
        (loop for item in (list-to-play self) do
              (let ((start (or (car (third item)) 0))
                    (end (or (second (third item)) (extent (first item)))))
                (dfuncall (second item) 'DoPlay (first item) start end)
                (dfuncall (+ (second item) end) 'DoStop (first item) )))
        (dfuncall (playedobjdur *general-player*) 'FinalPlay)))
     (setf (start-time self) (real-time))))

(defmethod Continue-Player ((self general-player))
   (setf (playing-etat self) "Playing")
   (setf (start-time self) (real-time))
   (loop for item in (playing-list self) do
         (DoContinue item))
   (loop for player in (assoc-players self) do
           (Continue-player player))
   (start
     (loop for item in (list-to-play self) do
           (dfuncall (- (second item) (pausetime self)) 'DoPlay (first item) 0 (extent (first item))))
     (dfuncall (- (playedobjdur self) (pausetime self)) 'FinalPlay)))

(defmethod Pause-Player ((self general-player))
   (setf (playing-etat self) "Pause")
   (setf (pausetime self) (+ (pausetime self) (- (real-time) (start-time self))))
   (loop for item in (playing-list self) do
         (DoPause item))
   (loop for player in (assoc-players self) do
           (Pause-player player))
   (reset-scheduler))

(defmethod Stop-Player ((self general-player) &optional view)
   (if (and view (equal view (recording-view self)))
     (doStopRecord (recording-view self))
     (without-interrupts 
      (loop for item in (playing-list self) do
            (DoStop item))
      (loop for player in (assoc-players self) do
            (Stop-player player view))))
   (reset-player self view))


(defmethod incf-cursor-pos ((self general-player))
   (setf (cursor-pos self) (round (+ (pausetime self) (- (real-time) (start-time self)) (delta-cursor-pos self)))))

(defun Finalplay ()
   (if (loopplay? *general-player*)
     (when (or (forlooplist *general-player*) (assoc-players *general-player*))
       (setf (playing-etat *general-player*) "Playing")
       (setf (cursor-pos *general-player*) 0)
       (setf (pausetime *general-player*) 0)
       (setf (list-to-play *general-player*) (copy-list (forlooplist *general-player*)))
       (loop for player in (assoc-players *general-player*) do
             (play-player player))
       (without-interrupts
        (loop for item in (list-to-play *general-player*) do
                (let ((start (or (car (third item)) 0))
                      (end (or (second (third item)) (extent (first item)))))
                  (dfuncall (second item) 'DoPlay (first item) start end)))
          (dfuncall (playedobjdur *general-player*) 'FinalPlay))
       (setf (start-time *general-player*) (real-time)))
     (progn
       (setf (playing-etat *general-player*) "Idle")
       (loop for player in (assoc-players *general-player*) do
             (stop-player player))
       )))

(defmethod Idle-p ((self list))
   (let ((rep t))
     (loop for item in self
           while rep do
           (unless (isidle? item)
             (setf rep nil)))
     rep))

(defmethod DoStopRecord ((self t)) t)

(defmethod isidle? ((self t)) t)

(defmethod DoStop ((self t)) t)

(defmethod DoPause ((self t)) t)

(defmethod DoContinue ((self t)) t)

(defmethod DoPlay ((self t) start end) 
   (when (string-equal (playing-etat *general-player*) "Playing")
     (setf (list-to-play *general-player*) (remove self (list-to-play *general-player*) :test 'equal :key 'car))
     (push self (playing-list *general-player*))
     t))

(defmethod PlayAny ((player t) (object t) &key (approx 2) (port nil) (interval nil))
   (if (Idle-p *general-player*) 
     (let ((objdur (get-obj-dur object))
           (avance (real-duration  object 0)))
       (when (second interval)
         (setf objdur (- (second interval) (first interval)))) 
       (setf avance (if (< avance 0) (abs avance) 0)) 
       (reset-player *general-player*)
       (setf (playedobjdur *general-player*) objdur)
       (PrepareToPlay player object avance :approx approx :interval interval :port port)
       (play-player *general-player*))
     (om-beep-msg  "Wait end of previous play!")))

;==================================================
;Play box
;==================================================

(defmethod* Play ((self t)   &key (approx 2) interval port)
   :initvals '(nil 2 nil nil) 
   :indoc '("object" "approx" "port" "selection") 
   :icon 207
   :doc "Plays any OM Musical object.
<approx> sets the temperament (2, 4, 8) in the case of Midi."
   (declare (ignore approx port))
   nil)
   

(defmethod* Play ((self simple-container) &key (player t) (approx 2) interval port)
   (setf port (verify-port port))
   (PlayAny player self :approx approx :port port :interval interval))

(defmethod* Play ((self list)   &key (player t) (approx 2) interval port)
   (setf port (verify-port port))
   (PlayAny player self :approx approx :port port :interval interval))


;==================================================
;Prepare to play
;==================================================

(defmethod* PrepareToPlay ((player t) (self t) at &key  approx port interval voice)
   :initvals '(nil 0 2 nil nil) 
   :indoc '("object" "start time" "approx" "port" "selection") 
   :icon 207
   :doc "use to redifine Play for new classes, see the manual"
   (declare (ignore approx seq port))
   (if interval
     (let ((newinterval (interval-intersec interval (list at (+ at (get-obj-dur self))))))
       (when newinterval
         (push (list self (- at (first interval)) (loop for pos in newinterval collect (- pos at)))
               (list-to-play *general-player*))))
     (push (list self at interval) (list-to-play *general-player*))) t)

(defmethod* PrepareToPlay ((player t) (self maquette-obj) at &key  approx port interval voice)
   (declare (ignore approx port))
   (loop for object in (inside self)
       for param in (param-list self) do
         (let ((objstart (+ at (offset->ms object))))
           (if interval
             (let ((newinterval (interval-intersec interval 
                                                   (list objstart 
                                                         (+ objstart (get-obj-dur object))))))
               (when newinterval 
                 (PrepareToPlay player object objstart
                                :approx (cdr (assoc 'approx param))
                                :port (strig2numport (cdr (assoc 'outport param)))
                                :interval interval
                                :voice voice)))
             (PrepareToPlay player object objstart
                            :approx (cdr (assoc 'approx param))
                            :port (strig2numport (cdr (assoc 'outport param)))
                            :voice voice)))))

(defmethod* PrepareToPlay ((player t) (list list) at &key  approx port interval voice)
   (setf port (verify-port port))
   (loop for object in list
         do (PrepareToPlay player object at 
                           :approx approx 
                           :port port
                           :interval interval
                           :voice voice)))

;=== general containers play all sub components

(defmethod* PrepareToPlay ((player t) (self container) at &key approx port interval voice)
   (setf port (verify-port port))
   (loop for sub in (inside self) do
         (let ((objstart (+ at (offset->ms sub))))
           (if interval
             (let ((newinterval (interval-intersec interval 
                                                   (list objstart (+ objstart (get-obj-dur sub))))))
               (when newinterval
                 (PrepareToPlay player sub objstart 
                                :approx approx 
                                :port port
                                :interval interval
                                :voice voice)))
             (PrepareToPlay player sub objstart 
                            :approx approx 
                            :port port
                            :voice voice)))))


(defmethod* PrepareToPlay ((player t) (self listtoplay) at &key  approx port interval voice)
   (declare (ignore approx))
   (loop for object in (thelist self)
         for param in (params self)
       do 
         (PrepareToPlay player object at 
                      :approx (cdr (assoc 'approx param)) 
                      :interval interval
                      :port (strig2numport (cdr (assoc 'outport param)))
                      :voice voice)))

(defun verify-port (port) t)

;===================
;Palette

(omg-defclass Playing-Palette (palette-win) 
   ((whoplay :initform nil :accessor whoplay)
    (pause-push-p :initform nil :accessor pause-push-p))
   (:default-initargs :delta-inpix (om-make-point 175 0)
     ;;;:back-color 15921906
     ))

(defmethod om-draw-contents :before ((self PLaying-Palette))
  (if (equal (panel (editor-assoc self)) (recording-view *general-player*))
      (setf (whoplay self) 3))
  (om-draw-picture self *play-controls-pict*)
  (om-with-focused-view self
                        (when (whoplay self)
                          (push-button self (whoplay self)))
                        (when (pause-push-p self)
                          (push-button self 2))
                        (when (loopplay? (panel (editor-assoc self)))
                          (push-button self 5))))

(defmethod click-button-inpal ((self Playing-Palette) x)
  (unless (equal x 6)
    (call-next-method)))

(defmethod palette-act ((self Playing-Palette) x)
   (let ((ed-view (panel (editor-assoc self))))
   (when (<= x 5) 
     (case x
       ;PLAY
       (0 (unless (whoplay self)
            (setf (whoplay self) 0)
            (play-from-palette ed-view)))
       ;STOP
       (1 
        (setf (whoplay self) nil
              (pause-push-p self) nil)
        (Stop-from-palette ed-view))
       ;PAUSE
       (2 
        (when (whoplay self)
          (if (string-equal (get-player-etat *general-player*) "Playing")
            (setf (pause-push-p self) t)
            (setf (pause-push-p self) nil))
          (pause-from-palette ed-view)))
       ;RECORD
       (3 (unless (whoplay self)
            (setf (whoplay self) 3)
            (unless (record-from-palette  ed-view)
              (setf (whoplay self) nil))))
       ;PLAY SELECTION
       (4 (when (selection-to-play-? ed-view)
            (unless (whoplay self)
              (setf (whoplay self) 4)
              (play-selection-from-palette ed-view))))
       ;LOOP
       (5 (push-loop-button ed-view)))
     (om-invalidate-view self t)
     t)))


;============Views which play, show cursor and allow selection

(defclass cursor-play-view-mixin (om-view-cursor-play) 
   ((loopplay? :initform nil :accessor loopplay?)
    (recording? :initform nil :accessor recording?)
    (cursor-interval :initform '(0 0) :accessor cursor-interval)
    (cursor-p :initform nil :accessor cursor-p)
    ))


(defmethod new-interval-cursor ((self cursor-play-view-mixin) where)
  (om-init-motion-functions self 'interval-select-action 'release-interval-select)
  (om-new-movable-object self (om-point-h where) 0 4 (h self) 'om-selection-rectangle))

(defmethod interval-select-action ((self cursor-play-view-mixin) pos)
 (let ((rect  (om-get-rect-movable-object self (om-point-h pos) (om-point-v pos))))
    (when rect
      (om-update-movable-object self (first rect) 0 (max 4 (third rect)) (h self)))))

(defmethod release-interval-select ((self cursor-play-view-mixin) pos)  
  (let ((rect  (om-get-rect-movable-object self (om-point-h pos) (om-point-v pos)))
         (minpixel 2)  position)
    (when rect
      (om-erase-movable-object self)
      (setf position (if (>  (third rect)  minpixel)
                         (list (car  rect) (+ (car rect) (third rect) ) )
                       (car rect)))
       (setf (cursor-interval self) position)
       (setf (cursor-interval self) (convert-interval self))
       (om-invalidate-view self t) )))

;--------------


(defmethod convert-interval ((self cursor-play-view-mixin))
  ;;; attention ici on fait des copies de tous les sound files de la maquette...
   (let* ((obj (car (get-obj-to-play self)))
          (dur (get-obj-dur obj))
          (int (cursor-interval self))  
          x x1
          rep)
     
     (if (listp int)
       (setf x (max 0 (om-point-h (pixel2point self (om-make-point (car int) 0))))
             x1 (min dur (om-point-h (pixel2point self (om-make-point (second int) 0)))))
       (setf x (max 0 (om-point-h (pixel2point self (om-make-point int 0)))) 
             x1 (get-obj-dur obj)))
     
       (setf rep (if (<= x x1)
                   (list x x1)
                   (list x1 x)))
       (if (< (car rep) dur)
         rep
         '(0 0))
       ))


(defmethod draw-interval-cursor ((self cursor-play-view-mixin))
   (let* ((sys-etat (get-system-etat self))
          (interval (cursor-interval self))
          (pixel-interval (list (om-point-h (point2pixel  self (om-make-big-point (car interval) 0) sys-etat))
                                (om-point-h (point2pixel  self (om-make-big-point (second interval) 0) sys-etat)))))
     (om-with-focused-view self
       (draw-h-rectangle (list (car pixel-interval) 0 (second pixel-interval) (h self)) t))))

(defmethod set-unset-cursor ((self cursor-play-view-mixin))
   (setf (cursor-p self) (not (cursor-p self)))
   (om-invalidate-view self t))



;;; new pixel = position-x ou il faut dessiner (defaut = current pos)
;;; si draw? dessine une ligne verticale en xor
(defmethod draw-line-cursor ((self cursor-play-view-mixin) &key newpixel (draw? t))
  (unless newpixel
    (setf newpixel (om-point-h (point2pixel self (om-make-big-point (cursor-pos *general-player*) 0) (get-system-etat self)))))
  (when draw? 
    (om-update-movable-cursor self  newpixel 0 4 (h self)))
   newpixel)


;;; play from palette : draw? = nil
(defmethod draw-play-cursor ((self cursor-play-view-mixin) st end &optional (draw? t))
 (when (and (om-view-window self) (om-window-open-p (om-view-window self)))
    (let* ((change-win t) (newpixel 0))
      (setf newpixel (draw-line-cursor self :draw? draw?))
      (when  (> newpixel (w self))
        (setf newpixel 0 
              change-win nil)
        (scroll-play-window self))
      (incf-cursor-pos *general-player*)
      (if (Idle-p *general-player*)
          (let ((palette (palette *global-palette*)))
            (when palette
              (setf (whoplay palette) nil)
              (om-invalidate-view palette t))
            (om-erase-movable-cursor self ) )
        (dfuncall 100  'draw-play-cursor self st end change-win)))))


(defmethod scroll-play-window ((self cursor-play-view-mixin)) 
  (without-interrupts
   (setf (rangex self) (list (cursor-pos *general-player*) 
                             (+ (cursor-pos *general-player*) 
                                (- (second (rangex self)) (first (rangex self))))))
  (change-view-ranges self)
  (om-redraw-view self)))

(defmethod get-selection-to-play ((self cursor-play-view-mixin))
   (let ((interval (cursor-interval self)))
     (values  (list (object (om-view-container self))
                    :interval interval)
              (first interval)
              (second interval))))

(defmethod get-obj-to-play ((self cursor-play-view-mixin))
   (list (object (om-view-container self))))



(defmethod pause-from-palette ((self cursor-play-view-mixin))
   (setf (loopplay? *general-player*) (loopplay? self))
   (cond ((string-equal (get-player-etat *general-player*) "Playing")
          (Pause-Player *general-player*))
         ((string-equal (get-player-etat *general-player*) "Pause")
          (Continue-Player *general-player*)
          (om-erase-movable-cursor self )
          (om-new-movable-cursor self 0 0 4 (h self) 'om-cursor-line)
          (start (draw-play-cursor self 0 100000)))))

(defmethod play-from-palette ((self cursor-play-view-mixin))
  (when  (Idle-p *general-player*)
    (setf (loopplay? *general-player*) (loopplay? self))
    (let ((obj (get-obj-to-play self)))
      (om-set-scroll-position self (om-make-point 0 (om-v-scroll-position self)))
      (apply 'PlayAny (cons (get-score-player self) obj))
      (om-new-movable-cursor self 0 0 4 (h self) 'om-cursor-line)
      (start (draw-play-cursor self 0 (get-obj-dur obj) t))
      )
    )
  )

(defmethod stop-from-palette ((self cursor-play-view-mixin))
  (om-erase-movable-cursor self )
  (Stop-Player *general-player* self))

(defmethod play-selection-from-palette ((self cursor-play-view-mixin))
   (setf (loopplay? *general-player*) (loopplay? self))
   (multiple-value-bind (obj start end) (get-selection-to-play self)
     (when obj
       (apply 'PlayAny (cons (get-score-player self) obj))
       (setf (delta-cursor-pos *general-player*) start)
       (om-new-movable-cursor self 0 0 4 (h self) 'om-cursor-line)
       (start (draw-play-cursor self start end  nil)))))


(defmethod push-loop-button ((self cursor-play-view-mixin))
   (setf (loopplay? self) (not (loopplay? self)))
   (setf (loopplay? *general-player*) (loopplay? self)))




(defmethod record-from-palette ((self cursor-play-view-mixin)) 
  (om-beep-msg "This editor does not allow recording."))

(defmethod selection-to-play-? ((self cursor-play-view-mixin))
  (and (cursor-p self) 
       (cursor-interval self) 
       (not (= (car (cursor-interval self)) (cadr (cursor-interval self))))
       ))



(AddGenFun2Pack '(play PrepareToPlay) *maquette-package*)





