(in-package :om)
                   

;-----------------return a list with the workspace parametres
(defun get-pathname-wsparams (pathname)
  "Return a keyword indicating the type of pathname"
  (let (line)
    (with-open-file (file pathname :direction :input :if-does-not-exist nil)
      (om-read-line file)
      (setf line (read-from-string (subseq (om-read-line file) 1))))
    line))

(defun write-header (self file)
  (write-line ";fileheader" file)
  (write-line (header-comment-from-obj self) file)
  (write-line ";endfileheader" file))

(defmethod header-comment-from-obj ((self t)) 
  (let* ((icon (icon self))
        (ftype (obj-file-type self))
        (version  *om-version*)
        (wspar (om-save-point-list (wsparams self)))
        (params (list version ftype (first wspar) (second wspar) (third wspar)  (str-without-nl (doc self)) (save-icon icon)) ))
    (string+ ";" (format nil " ~S" params ))))


;search par tout mais aussi wsparmas
;(version filetype position ed-position ed-size icon documentation)
;=========================create-file
(defun om-create-file (self pathname &key (file-type :TEXT))
  "Creates an empty file named pathname and its documentation file, and returns the truename of the created file"
  (when (directory-pathname-p pathname)
    (setq pathname (make-pathname :directory (pathname-directory pathname) :name ".finderinfo")))
  (with-open-file (file pathname :direction :output :if-does-not-exist :create) ;;; :external-format file-type )
    (write-header self  file)
    (om-set-mac-file-type file file-type))
  (om-set-file-format pathname)
  (truename pathname))

;==============get-finder-comment
;search par tout 
(defun get-finder-comment (pathname)
  "Returns the finder comment of a file or nil if it doesn't have one"
  (let (wsparams resources)
    (when (and (not (equal (format nil "~A" (file-namestring pathname)) ".finderinfo"))
               (not (equal (format nil "~A" (file-namestring pathname)) "/.finderinfo"))
              (not (equal (format nil "~A" (file-namestring pathname)) ".DS_Store")))
      (if (directory-pathname-p pathname)
        (let ((folder-info-path (make-pathname :directory (pathname-directory pathname) :name  ".finderinfo")))
          (unless (probe-file folder-info-path)
            (with-open-file (file folder-info-path :direction :output :if-does-not-exist :create) ;;; :external-format file-type )
              (write-line ";fileheader" file)
              (write-line (header-comment-from-obj (make-instance 'omfolder :icon 186)) file)
              (write-line ";endfileheader" file))
            (om-set-file-format folder-info-path))
          (setf wsparams (get-pathname-wsparams folder-info-path)))
        (cond
         ((not (file-has-comments pathname))
          (setf wsparams (list *om-version* (file-type pathname) (om-make-point 24 24) (om-make-point 50 50) (om-make-point 500 400) "No documentation " nil)))
         ((file-has-old-comments pathname)
          ;(setf resources (comp-get-resourcespathname pathname))
          (setf wsparams (list *om-version* (read-old-type pathname) (om-make-point 24 24) (om-make-point 50 50) (om-make-point 500 400) "No documentation " nil))
          ;(remove-old-comments pathname wsparams)
          ;(comp-set-resourcespathname pathname resources)
          )
         (t
          (setf wsparams (get-pathname-wsparams pathname))))))
    wsparams))


;search 
(defun get-init-wsparams (path)
  (let ((wsparams (get-finder-comment path)))
    (unless wsparams
      (setf wsparams (list *om-version* :fold (om-make-point 24 24) (om-make-point 50 50) (om-make-point 500 400) "No documentation " 186)))
    wsparams))


;==============get-finder-comment

(defun set-finder-comment (pathname self)
  "Sets the finder comment of a file"
  (when (om-persistant-p pathname)
    (let ( line (linelist nil) folderp)
      (when (setf folderp (directory-pathname-p pathname))
        (setq pathname (make-pathname :directory (pathname-directory pathname) :name ".finderinfo")))
      (when (probe-file pathname)
        (with-open-file (file pathname :direction :input)
          (om-read-line file)
          (om-read-line file)
          (om-read-line file)
          (loop while (not (equal line :eof)) do
                (setf line (read-line file nil :eof))
                (setf linelist (append linelist (list line)))
                ))
        (delete-file-protection pathname)
        (with-open-file (file pathname :direction :output :if-exists :supersede)
          (write-header self file)
          (setf line (pop linelist))
          (loop while (and line (not (equal line :eof))) do
                (write-line line file)
                (setf line (pop linelist))
                )
          (unless folderp
            (om-set-mac-file-type file (obj-file-type self))))
        (om-set-file-format pathname)))))



;======================file-type

;in api return nil for allegro


(defun file-type (pathname)
  "Return a keyword indicating the type of pathname"
  (or (good-om-file? pathname)
      (let ((wsparams (get-pathname-wsparams pathname)))
        (second wsparams)))) 

(defun file-has-comments (path)
  (let (rep)
    (with-open-file (in path :direction :input  )
      (let ((char (read-char in nil :eof)))
        (when (and (not (eq char :eof)) (equal char #\;))
          (setf rep t))))
    rep))

(defun file-has-old-comments (path)
  (let (rep)
    (with-open-file (in path :direction :input  )
      (let ((char (read-char in nil :eof)))
        (setf char (read-char in nil :eof))
        (setf char (read-char in nil :eof))
        (when (and (not (eq char :eof)) (equal char #\*))
          (setf rep t))))
    rep))


(defun remove-old-comments (pathname params)
  (let ((data nil)
        oldheader line)
    (with-open-file (file pathname :direction :input)
      (loop for i from 1 to 7 do (read-line file nil :eof))
      (setq line (read-line file nil :eof))
      (loop while (not (eq line :eof)) do
            (push line data)
            (setq line (read-line file nil :eof))))
    (setf data (reverse data))
    (delete-file-protection pathname)
    (with-open-file (file pathname :direction :output :if-exists :supersede)
      (write-line ";fileheader" file)
      (write-line (format nil "; (~D :~D ~D ~D ~D ~S ~D)" (first params) (second params)
                          (om-save-point (third params)) (om-save-point (fourth params)) (om-save-point (fifth params))
                          (sixth params) (seventh params)) file)
      (write-line ";endfileheader" file)
      (loop for item in data do
            (write-line item file)))
    (om-set-file-format pathname)))


(defun read-old-type (pathname)
  (let (line)
    (with-open-file (file pathname :direction :input)
      (read-line file)
      (read-line file)
      (setq line (read-from-string (subseq (read-line file) 2))))
    line))


(defun om-put-path-extension (path type)
  (make-pathname :directory (pathname-directory path) 
                 :name (pathname-name  path)
                 :type type))