;OpenMusic
;
;Copyright (C) 1997, 1998, 1999, 2000 by IRCAM-Centre Georges Pompidou, Paris, France.
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
;
;Authors: Gerard Assayag and Augusto Agon

(in-package :om)


;-------------
;TEXTFILE CLASS

(defclass* TextFile () 
   ((ed-mode :initform "supersede" :initarg :ed-mode :accessor ed-mode)
    (file-name :initform nil :accessor file-name)
    (buffer-text :initform nil :accessor buffer-text))
   (:icon 203))

(defmethod get-super-default-value ((type (eql 'TextFile)))
  (let ((tf (make-instance 'textfile)))
    ;(setf (buffer-text tf) (om-make-buffer))
    tf))


 
(defmethod get-type-of-ed-box ((self TextFile))  'OMTextFilebox)
(defmethod editor ((self t)) self)


;;; c'est trop long avec exp-list...
(defmethod draw-obj-in-rect ((self textfile) x x1 y y1 edparams view)
   (let ((fontsize 14) 
         (fontface (om-font-face (om-get-font view)))
         (elist (exp-list self)))
     (om-with-focused-view view
       (om-with-font (om-make-font fontface fontsize)  
                     (if (and (not elist) (not (equal (om-buffer-text (buffer-text self)) "")))
                         (om-draw-string 0 20 (om-buffer-text (buffer-text self)))
                       (loop for item in elist
                             for i = 1 then (+ i 1)
                             while (< (+ (* i 21) y) y1) do
                             (om-draw-string (+ x 5) (+ (* i 20) y) (format nil "~D " item))
                             )
                       )))))

(defmethod get-slot-in-out-names ((self TextFile))
   (values '("self" "exp-list" "ed-mode")
           '(nil nil "supersede")
           '("object or object list" "text to put in the file" "append or supersede mode")
           '(nil nil (( 2 (("append" "append")  ("supersede" "supersede")))))))


(defmethod* objfromobjs ((self string) (type textfile))
   (load-textfile self 'textfile "supersede"))

(defmethod* objfromobjs ((self pathname) (type textfile))
   (load-textfile self 'textfile "supersede"))

(defmethod* objfromobjs ((self null) (type textfile))
   type)

;;;==================================

;;; put a list of text in the textfile buffer
;;; mode supersede : reinitialize the buffer
;;; mode append : append text at the end
(defmethod add/replace-to-buffer ((self TextFile) list)
  (unless (buffer-text self)
    (setf (buffer-text self) (om-make-buffer)))
  (when (string-equal (ed-mode self) "supersede")
    (om-buffer-delete (buffer-text self)))
  (let ((pos (om-buffer-size (buffer-text self)))
        (str (format nil "~D" (car list))))
    (om-buffer-insert (buffer-text self) str pos)
    (setf pos (+ pos (length str)))
    (loop for item in (cdr list) do
          (om-buffer-insert-newline (buffer-text self) pos)
          (setf str (format nil "~D" item))
          (om-buffer-insert (buffer-text self) (format nil "~D" item) (+ 1 pos))
          (setf pos (+ pos (length str) 1))
          )
    (om-buffer-insert-newline (buffer-text self) pos)
    (buffer-text self)))

(defmethod (setf exp-list) ((exp-list list) (self TextFile))
   (setf exp-list (list! exp-list))
   (add/replace-to-buffer self exp-list))

(defmethod (setf buffer-text) :before ((buffer t) (self TextFile))
  (when (buffer-text self)
     (om-kill-buffer (buffer-text self))))


;;; new
(defmethod exp-list ((self TextFile))
   (let ((lista (list-of-lines (buffer-text self))))
     ;;; print un warning si exp-list est nulle alors que le buffer contient du texte
     (when (and (not lista) (not (string-equal (om-buffer-text (buffer-text self)) "")))
         (print (format nil "Warning : ~%An error occured while evaluating exp-list of the textfile")))
     lista))

(defun list-of-lines (buffer)
  (when buffer
    (let ((numlines (om-lines-in-buffer buffer))
          (pos 0) rep)
      (loop for i from 1 to numlines do
            (let ((start (om-buffer-line-start buffer pos))
                  (end (om-buffer-line-end buffer pos)))
              (push (om-buffer-substring buffer start end) rep)
              (setf pos (+ 1 end))))
      (reverse rep))))


;--------------Copy & Save----------------

(defmethod omng-copy ((self TextFile))
   `(let ((newtextfile (make-instance ',(type-of self)
                           :ed-mode ,(ed-mode self))))
        (setf (buffer-text newtextfile) ,(om-copy-buffer (buffer-text self)))
        (setf (file-name newtextfile) ,(file-name self))
        newtextfile))
    
(defmethod omNG-save ((self TextFile) &optional (values? nil))
  (print (list-of-lines (buffer-text self)))
   (declare (ignore values?))
   (if (file-name self)
       `(load-textfile ,(om-save-pathname-relative (file-name self)) ',(type-of self) ,(ed-mode self) )
     `(load-buffer-textfile ',(list-of-lines (buffer-text self)) ',(type-of self) ,(ed-mode self) )))
  
(defun load-textfile (filename class edmode)
  (let ((newtextfile (make-instance class
                       :ed-mode edmode)))
    (setf (buffer-text newtextfile) (om-make-buffer))
    (if (probe-file filename)
      (progn
        (om-buffer-insert-file (buffer-text newtextfile) filename)
        (setf (file-name newtextfile) filename))
      (om-beep-msg (string+ "I didn't open the file " (namestring filename))))
    newtextfile))

;;; !!! les lignes..
(defun load-buffer-textfile (listline class edmode)
  (let ((newtextfile (make-instance class
                                    :ed-mode edmode)))
    (setf (buffer-text newtextfile) (om-make-buffer))
    (loop for item in listline do
          (om-buffer-insert (buffer-text newtextfile) item)
          (om-buffer-insert-newline (buffer-text newtextfile))
          )
    newtextfile))




;==================EDITOR===============
;;; TEXT EDITOR IS A SPECIAL CASE OF EDITOR 
;;; DIRECTLY MANAGED BY OM-API

(omg-defclass TextEditorWindow (om-text-edit-window) 
   ((object :initform nil :initarg :object :accessor object)
    (ref :initform nil :initarg :ref :accessor ref)))

(defmethod Class-has-editor-p ((self TextFile)) t)

;;; no EDITOR slot
;;; editorframe is the window itself
(defmethod open-new-textedit ((self TextFile) box) 
  (setf (editorframe box) (om-make-window 'TextEditorWindow
                                          :object self
                                          :ref box
                                          :font (om-make-font "courier" 20)
                                          :window-title "TextFile"
                                          :size (om-make-point 500 400)
                                          :position :centered
                                          :ask-for-save nil
                                          :editor-buffer (buffer-text self)
                                          :editor-file (file-name self)
                                           )))


;; (defmethod get-editor-class ((self TextFile)) 'textfileEditor)

;(defmethod make-editor-window ((class (eql 'textfileEditor)) object name ref &key 
;                               winsize winpos (close-p t) (winshow t) (resize t) (retain-scroll nil))
;   (om-make-window 'TextEditorWindow
;                   :object object
;                   :ref ref
;                   :position :centered
;                  :size winsize
;                  :textbuffer (buffer-text object)
;                   :ask-for-save t))

(defmethod om-window-close-event :after ((self TextEditorWindow)) 
   (cond ((null (ref self)) nil)
         ((slot-initform-p (ref self))
          (change-initform-ed (ref self) (object self)) )
         ((EditorView-p (ref self))
          (setf (attached-editors (ref self)) 
                (remove self (attached-editors (ref self)) :test 'equal))
          (om-invalidate-view (ref self) t))
         ((ominstance-p (ref self))
          (setf (Editorframe (ref self)) nil))
         ((is-boxpatch-p  (ref self))
          (setf (Editorframe (ref self)) nil)
          (om-invalidate-view (car (frames (ref self)))))))

(defmethod update-editor-after-eval ((self TextEditorWindow) val)
   (setf (object self) val)
   (when (buffer-text val)
     (om-set-text-buffer self (buffer-text val) t)))


;==================BOX EDITOR================
(defclass OMTextFilebox (OMBoxEditCall) ())

(defmethod OpenEditorframe ((self OMTextFilebox))
   (or (editorframe self) (open-new-textedit (value self) self)))

 (defmethod OpenObjectEditor ((self OMTextFilebox)) 
   "If there is a EditorFrame open for SELF select the window of EditorFrame, 
else create a new Editor frame, and select its window."
   (setf (EditorFrame self) (OpenEditorframe self))
   (when (EditorFrame self)
     (om-select-window (Editorframe self))))

(defmethod numouts ((self OMTextFilebox)) 3)

(defmethod remove-extra ((self OMPatch) (box OMTextFilebox))
  (when (and (value box) (buffer-text (value box)))
    (om-kill-buffer (buffer-text (value box)))))

(defmethod (setf value) :before ((val TextFile) (self OMTextFilebox)) nil)
;  (when (and (value self) (buffer-text (value self)))
;    (om-kill-buffer (buffer-text (value self)))))

(defmethod fill-buffer-textfile ((self OMTextFilebox))
   (om-buffer-insert-file (buffer-text (value self)) (file-name (value self))))

;-----------------Evaluation&code generation-------


(defmethod omNG-box-value ((self OMTextFilebox) &optional (numout 0))
   ;(handler-bind ((error #'(lambda (c)
   ;                          (when *msg-error-label-on*
   ;                            (om-message-dialog (string+ "Error while evaluating the box " (string (name self)) " " 
   ;                                                     (om-report-condition c))
   ;                                            :size (om-make-point 300 200))
   ;                            (clear-after-error self)
   ;                            (om-abort)))))
      (let ((intype (omNG-box-value (third (inputs self)))))
       (setf (ed-mode (value self)) intype) 
       (cond
        ((equal (allow-lock self) "o") self)
        ((equal (allow-lock self) "l") "sorry not lambda mode for textfile boxes")
        ((and (equal (allow-lock self) "x") (value self)) (rep-editor (value self) numout))
        ((and (equal (allow-lock self) "&") (ev-once-p self)) (rep-editor (value self) numout))
        (t (let ((val (omNG-box-value  (second (inputs self))))
                  (newtextfile (make-instance (type-of (value self))
                                         :ed-mode intype
                                         )))
             (if (connected? (first (inputs self)))
                 (setf (value self) (objfromobjs (omNG-box-value (first (inputs self))) newtextfile))
               (progn
                  (setf (file-name newtextfile) (file-name (value self)))
                  (setf val (list! val))
                  (add/replace-to-buffer (value self) val)
                  (if (buffer-text (value self))
                      (setf (buffer-text newtextfile) (om-copy-buffer (buffer-text (value self))))
                    (setf (buffer-text newtextfile) (om-make-buffer)))
                  (unless (editorFrame self)
                    (om-kill-buffer (buffer-text (value self))))
                  (setf (value self) newtextfile)
                  (update-if-editor self)
                  )))
             (rep-editor (value self) numout))))) ;)

(defmethod gen-code ((self OMTextFilebox) numout)
   (cond
    ((equal (allow-lock self) "&") 
     (gen-code-for-ev-once self numout))
    ((equal (allow-lock self) "x") 
     `(rep-editor ,(value self) , numout))
    ((equal (allow-lock self) "o") 
     `(find-class 'Textfile nil)) 
    ((equal (allow-lock self) "l") 
     (om-beep-msg "Sorry no lambda mode for textfile boxes"))
    (t  (call-gen-code self numout))))

(defmethod gen-code-for-ev-once ((self OMTextFilebox) numout)
   (let ((varname (read-from-string (gen-box-string self))))
     (if (not (member varname *let-list* :test 'equal)) 
       (progn
         (push varname  *let-list*)
         (if *start-repeat-generation*
           (progn
             (push `(setf ,varname ,(call-gen-code self numout)) *repeat-ev-once-list*)
             varname)
           `(progn
              (setf ,varname ,(call-gen-code self numout))
              ,varname)))
       varname)))

(defmethod gen-new-val ((self TextFile) val mode numout)
   (setf (ed-mode self) mode)
   (setf val (list! val))
   (add/replace-to-buffer self val)
   (rep-editor self numout))

(defmethod call-gen-code ((self OMTextFilebox) numout)
   (if (connected? (first (inputs self)))
      `(rep-editor (objfromobjs ,(gen-code (first (inputs self)) 0) ,(value self)) ,numout) 
   `(gen-new-val ,(value self) ,(gen-code (second (inputs self)) 0) ,(gen-code (third (inputs self)) 0) ,numout)))

;---------------------Menu Contextual--------


(omg-defclass textEditorFrame (boxEditorFrame) ()
   (:documentation "Simple frame for OMTextFilebox meta objects. #enddoc#
#seealso# (OMTextFilebox) #seealso#"))

(defmethod get-frame-class ((self OMTextFilebox)) 'textEditorFrame)

(defmethod om-get-menu-context ((object texteditorframe))
   "Uhmmm If this method had been specialized by the class and not by the frame ?"
   (list+  (list (list 
                  (om-new-leafmenu "Import Contents From File" #'(lambda () (import-box-editor (object object))))
                  (om-new-leafmenu "Export Contents To File" #'(lambda () (export-box-editor (object object))))
                  )
                 (om-new-leafmenu "Import and Attach File" #'(lambda () (load-editor-frommenu (object object))))
                 (om-new-leafmenu "Export and Attach File" #'(lambda () (save-editor-frommenu (object object)))))
           (if (file-name (value (object object)))
               (list (om-new-leafmenu "Free Attached File" #'(lambda () (free-editor-from-file (object object)))))) 
           ))


; miniview
;;;======================================

(defmethod import-box-editor ((self OMTextFilebox))
   (catch-cancel
    (let ((name (om-choose-file-dialog)))
      (when name
        (unless (buffer-text (value self))
          (setf (buffer-text (value self)) (om-make-buffer)))
        (if (string-equal (ed-mode (value self)) "supersede")
          (om-buffer-delete (buffer-text (value self))))
        (om-buffer-insert-file (buffer-text (value self)) name (om-buffer-size (buffer-text (value self))))
        ;;;(when (editorframe self) (update-editor-after-eval (editorframe self) (value self)))
      ))))


(defmethod export-box-editor ((self OMTextFilebox))
   (catch-cancel
    (when (value self)
        (let ((name (om-choose-new-file-dialog    
                     :prompt "Save the instance of this editor")))
          (when name
            (delete-file-protection name)
            (when (buffer-text (value self))
              (om-buffer-write-file (buffer-text (value self)) name :if-exists :supersede)))))))
  

(defmethod load-editor-frommenu ((self OMTextFilebox))
   (catch-cancel
    (let ((name (om-choose-file-dialog)))
      (unless (buffer-text (value self))
        (setf (buffer-text (value self)) (om-make-buffer)))
      (om-buffer-delete (buffer-text (value self)))
      (om-buffer-insert-file (buffer-text (value self)) name)
      (setf (file-name (value self)) name))
    ))

(defmethod save-editor-frommenu ((self OMTextFilebox))
   (catch-cancel
   (when (value self)
     (let ((name (or (file-name (value self))
                     (om-choose-new-file-dialog    
                      :prompt "Save the instance of this editor"))))
       (when name
         (delete-file-protection name)
         (om-buffer-write-file (buffer-text (value self)) name :if-exists :supersede)
         (setf (file-name (value self)) name)))))
   )
      
(defmethod free-editor-from-file ((self OMTextFilebox))
   (when (value self)
     (if (file-name (value self))
       (setf (file-name (value self)) nil)
       (om-beep-msg "The text file has no file attached."))))

(defmethod load-editor-frommenu ((self t))
   (om-beep-msg "Use import in this editor box"))

(defmethod save-editor-frommenu ((self t))
   (om-beep-msg "Use export in this editor box"))


;;;(omNG-add-element *data-package* (omNG-protect-object (find-class 'textfile)))
(addClass2Pack 'TextFile *data-package*)




      









