;; jb 28.03.05

(in-package :om)



;;;===================================
;;; INTERFACE SPLINES dans BPF EDITOR
;;;===================================

(defmethod spline-p ((self bpfeditor)) t)
(defmethod spline-p ((self t)) nil)


(defclass editor-spline-manager ()
  ((editor :accessor editor :initform nil :initarg :editor)
   (active :accessor active :initform nil :initarg :active)
   (degre :accessor degre :initform 3 :initarg :degre)
   (s-resolution :accessor s-resolution :initform 100 :initarg :s-resolution)
   (opt-items :accessor opt-items :initform nil :initarg :opt-items)
   (outspline :accessor outspline :initform nil :initarg :outspline)))


(defmethod init-spline-manager ((self bpfeditor))
  (let ((spl (make-instance 'editor-spline-manager :editor self)))
    (setf (spline self) spl)
    (om-add-subviews (control self)
                     ;(om-make-dialog-item 'om-static-text (om-make-point 220 6) (om-make-point 80 12) "Spline"
                     ;                     :font *controls-font*
                     ;                     :bg-color *controls-color*
                     ;                     )
                     (om-make-dialog-item 'om-check-box (om-make-point 200 7) (om-make-point 60 12) "Spline"
                                          :di-action  (om-dialog-item-act item
                                                        (set-spline-preview (spline (om-view-container (om-view-container item)))
                                                                            (om-checked-p item))
                                                        (om-invalidate-view (panel (om-view-container (om-view-container  item))) t))
                                          :checked-p (active spl)
                                          :font *controls-font*
                                          :bg-color *controls-color*
                                          ))
    (when (active spl)
      (add-opt-items spl))
    ))

(defmethod add-opt-items ((self editor-spline-manager))
  (let ((x0 260))
  (setf (opt-items self)
        (list       (om-make-dialog-item 'om-static-text
                                         (om-make-point x0 6) 
                                         (om-make-point 70 16) "- Degre" 
                                         :font *controls-font*
                                         :bg-color *controls-color*
                                          )
                    (om-make-dialog-item 'numBox (om-make-point (+ x0 46) 6) (om-make-point 14 14) 
                                         (format nil " ~D" (degre self))
                                         :min-val 2
                                            :max-val 5
                                            :value (degre self)
                                            :afterfun #'(lambda (item)
                                                          (setf (degre self) (value item))
                                                          (compute-spline self)
                                                          (om-invalidate-view (panel (om-view-container (om-view-container item))) t))
                                            :font *controls-font*
                                            )
                    (om-make-dialog-item 'om-static-text
                                            (om-make-point (+ x0 70) 6) 
                                            (om-make-point 70 16) "Resolution" 
                                            :font *controls-font*
                                            :bg-color *controls-color*
                                            )
                    (om-make-dialog-item 'numBox (om-make-point (+ x0 132) 6) (om-make-point 26 14) 
                                        (format nil " ~D" (s-resolution self))
                                        :min-val 10
                                        :max-val 999
                                        :value (s-resolution self)
                                        :afterfun #'(lambda (item)
                                                      (setf (s-resolution self) (value item))
                                                      (compute-spline self)
                                                      (om-invalidate-view (panel (om-view-container (om-view-container item))) t))
                                            :font *controls-font*
                                            )
                    ))
  (loop for item in (opt-items self) do 
        (om-add-subviews (control (editor self)) item))
  ))

(defmethod remove-opt-items ((self editor-spline-manager))
  (loop while (opt-items self) do 
        (om-remove-subviews (control (editor self)) (pop (opt-items self))))
  )


(defmethod set-spline-preview ((self editor-spline-manager) t-or-nil)
  (setf (active self) t-or-nil)
  (if t-or-nil
    (progn
      (add-opt-items self)
      (compute-spline self))
    (remove-opt-items self))
  (om-invalidate-view (panel (editor self)))
  )

(defmethod compute-spline ((self editor-spline-manager))
  (let* ((bpf (currentbpf (panel (editor self))))
         (points (point-list bpf))
         (N (- (length points) 1))
         knots inpts)
    (if (< N 100)
      (progn
        (setf knots (SplineKnots N (degre self)))
        (setf inpts (mat-trans (list (x-points bpf) (y-points bpf))))
        (setf (outspline self) (SplineCurve2D inpts N knots (degre self) (s-resolution self)))
      )
      (om-print "spline could not be calculated because of too many points")
      )))




(defmethod draw-bpf :after ((Self Bpfpanel) (Bpf Bpf) Minx Maxx Miny Maxy &optional (Deltax 0) (Deltay 0))
  (when (spline-p (editor self))
    (let* ((spline (spline (editor self)))
           (points (outspline spline))
           (system-etat (get-system-etat self))
           (bpf-selected? (and (equal (selection? self) t) (equal bpf (currentbpf self))))
           (fact (expt 10.0 (decimals bpf))))
      (when (and (active spline) points); bpf-selected?)
        (om-with-fg-color self *om-red-color*
          (loop for thepoint in points do
                (let* ((pix-point (point2pixel self (om-make-big-point (round (* fact (car thepoint))) (round (* fact (cadr thepoint)))) system-etat)))
                  ;(om-draw-point (om-point-h pix-point) (om-point-v pix-point))
                  (om-fill-rect (om-point-h pix-point) (om-point-v pix-point) 2 2)
                  )
                )))
      )))


(defmethod spline ((self t)) nil)

(defmethod scroll-point :after ((Self Bpfpanel) Point)
  (when (and (spline (editor self)) (active (spline (editor self))))
    (compute-spline (spline (editor self)))))

(defmethod scroll-bpf :after ((Self Bpfpanel) Where)
  (when (and (spline (editor self)) (active (spline (editor self))))
    (compute-spline (spline (editor self)))))


;;; OLD

#|
(defclass! b-spline (bpc)
  ((degree :accessor degree :initarg :degree :initform 3)
   (s-resolution :accessor s-resolution :initarg s-resolution :initform 100)
   (outcurve :accessor outcurve :initform nil)))

(defmethod cons-spline ((self b-spline))
  (let* ((points (point-list self))
         (N (- (length points) 1))
         (knots (SplineKnots N (degree self)))
         (inpts (mat-trans (list (x-points self) (y-points self)))))
    (setf (outcurve self) (SplineCurve2D inpts 
                                         N knots (degree self) (s-resolution self)))
    ))

(defmethod cons-bpf ((self b-spline) points)
  (call-next-method)
  (cons-spline self))

(defmethod move-points-in-bpf :after ((self b-spline) points deltax deltay) 
  (cons-spline self))

(defmethod Class-has-editor-p ((self b-spline)) t)
(defmethod get-editor-class ((self b-spline)) 'splineEditor)

(defclass splineeditor (bpceditor) ())

(defmethod get-panel-class ((self splineeditor)) 'splinePanel)
(defmethod get-control-class ((self splineeditor)) 'control-spline)

(defclass splinePanel (bpcPanel) ())
(defclass control-spline (control-bpc) ())

(defmethod draw-bpf ((self splinePanel) (bpc b-spline) minx maxx miny maxy &optional (deltax 0) (deltay 0))
   (declare (ignore minx maxx miny maxy))
   (call-next-method)
   (draw-spline self bpc deltax deltay))

(defmethod draw-spline ((self splinePanel) (bpf b-spline) deltax deltay)
   (let* ((system-etat (get-system-etat self))
          (bpf-selected? (and (equal (selection? self) t) (equal bpf (currentbpf self))))
          (points (outcurve bpf))
          (fact (expt 10.0 (decimals bpf))))
     (om-with-fg-color self *om-red-color*
       (loop for thepoint in points do
             (let* ((pix-point (point2pixel self (om-make-point (round (* fact (car thepoint))) (round (* fact (cadr thepoint)))) system-etat)))
               (om-draw-line (om-point-h pix-point) (om-point-v pix-point) (om-point-h pix-point) (om-point-v pix-point))
               t)
))))
                 


(defmethod objfromobjs ((self b-spline) (type bpc))
  (let ((reslist (outcurve self)))
        (rep (make-instance 'bpc 
               :decimals (decimals self)
               ))
    (setf (x-points rep) (car reslists))
    (setf (y-points rep) (cadr reslists))
    rep))
|#