;OpenMusic
;
;Copyright (C) 1997, 1998, 1999, 2000 by IRCAM-Centre Georges Pompidou, Paris, France.
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
;
;Authors: Gerard Assayag and Augusto Agon

(in-package :om)

;-------SORT---------
(defmethod* sort. ((lst list)  &optional (test '<) (key nil))
  :initvals '(nil < nil)
  :indoc '("the list" "test" "key")
  :doc "Sorts  a list. 
<test> a  binary function name or function object indicating how to compare elements.
<key> a  binary function name or function object that will be applied to elements.

a function icon or subpatch icon in 'lambda' mode can be connected to <test> and key."
  :icon 176
  (sort (copy-list lst) (or test #'<) :key key))

;-------CIRCULAR PERMUTATION---------

(defmethod permut-circn ((list list) &optional (nth 1))
  (when list
    (let ((length (length list)) n-1thcdr)
      (setq nth (mod nth length))
      (if (zerop nth) list
          (prog1
            (cdr (nconc (setq n-1thcdr (nthcdr (1- nth) list)) list))
            (rplacd n-1thcdr ()))))))

(defmethod* rotate ((list list) &optional (nth 1))
  :initvals '(nil 1)
  :indoc '("the list" "nth")
  :doc "Returns a circular permutation of a copy of <list> starting from its <nth> 
element"
  :icon 176
  (permut-circn (copy-list list) nth))

;-------NTH-RANDOM---------

(defmethod* nth-random ((list list))
  :initvals '(nil)
  :indoc '("the list")
  :doc "returns a randomly chosen element from  <list>."
  :icon 176
  (nth (om-random-value (length list)) list))

;-------PERMUT-RANDOM---------

(defmethod npermut-random ((list list))
  "Returns a destructive random permutation of <list>."
   (let ((result ()) 
           (length (length list)) 
           (list (copy-list list)))
    (nconc list list)
    (loop for i from 0 to length do
              (setq list (nthcdr (om-random-value length) list)
            result (rplacd (prog1 (cdr list) (rplacd list (cddr list))) result)
            length (1- length)))
    result))


(defmethod* permut-random ((list list))
  :initvals '(nil)
  :indoc '("the list")
  :doc " Returns a random permutation of list."
  :icon 176
  (if (or (null list) (= (length list) 1)) list
      (npermut-random (copy-list list))))



;-------MAT-TRANS---------

(defmethod* posn-order ((list list)  (test symbol))
  :initvals '(nil '<)
  :indoc '("the list" "test")
  :doc  "Returns a list of indexes according to a sort function.
The indexes (from 0 to length-1) will be sorted according to <test>.
<test> may be a symbol for a binary comparison function (default '<)
or a function icon or subpatch icon in 'lambda' mode."
  :icon 176
  (do-posn-order list test))


(defmethod* posn-order ((list list)  (test function))
  (do-posn-order list test))


(defun do-posn-order (list test)
  (let* ((thelist (sort. list test))  rep)
    (mapc #'(lambda (x)
              (let* ((repons (position x thelist)) (i 0))
                (loop while (position repons rep) do
                      (incf i)
                      (setf repons (position  x thelist :start (+ (position x thelist) i))))
                (setf rep (cons repons rep)))) list)
    (reverse rep)))

;-------PERMUTATIONS---------

(defmethod* permutations ((bag list))
  :initvals '(nil ) 
  :indoc '("a list" )
  :icon 176
  :doc "Return a list of all the permutations of the input."
  
  (if (null bag)
    '(())
    ;; Otherwise, take an element, e, out of the bag.
    ;; Generate all permutations of the remaining elements,
    ;; And add e to the front of each of these.
    ;; Do this for all possible e to generate all permutations.
    (mapcan #'(lambda (e)
                (mapcar #'(lambda (p) (cons e p))
                        (permutations (remove e bag :count 1 :test #'eq))))
            bag)))

 
(defvar *comb-package* (omNG-protect-object (omNG-make-new-package "Combinatorial")))

(AddGenFun2Pack (list 'sort. 'rotate 'nth-random 'permut-random 'posn-order 'permutations) *comb-package*)

(AddPackage2Pack *comb-package* *basic-package*)