;OpenMusic
;
;Copyright (C) 1997, 1998, 1999, 2000 by IRCAM-Centre Georges Pompidou, Paris, France.
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
;
;Authors: Gerard Assayag and Augusto Agon


(in-package :om)

;-------UNION---------

(defun noRep-union (lists oper test key)
  (let ((the-union (list! (car lists))))
  (dolist (one-in (cdr lists))
      (setq the-union (funcall oper the-union (list!  one-in)  :test test :key key)))
  the-union))

(defmethod* x-union ((l1? list) (l2? list) 
                     &optional (test 'equal) (key 'identity)
                     &rest list)
  :initvals '(nil nil equal identity)
  :indoc '("first list" "second list" "test" "key" "optional list")
  :doc "Merges 2  lists, <l1?> and <l2?>, into a single list, with no repetitions. 
<test> is a name for a binary comparison function (default 'equal).
<key> is a name for a function that is applied to elements before comparison.
<test> and <key> may also be connected function or subpatch icons in 'lambda' mode."
  :icon 191
  (noRep-union  (list* l1? l2? (and list (list! list))) 'union test key))

;-------INTERSECTION---------

(defmethod* x-intersect ((l1? list) (l2? list)
                         &optional (test 'equal) (key 'identity)
                         &rest list)
  :initvals '(nil nil equal identity)
  :indoc '("first list" "second list" "test" "key" "optional list")
  :doc "Intersects 2  lists, <l1?> and <l2?>, into a single list.
<test> is a name for a binary comparison function (default 'equal).
<key> is a name for a function that is applied to elements before comparison.
<test> and <key> may also be connected function or subpatch icons in 'lambda' mode."
  :icon 191
  (nreverse (noRep-union  (list* l1? l2? (and list (list! list))) 'intersection test key)))

;-------XOR---------

(defmethod* x-Xor ((l1? list) (l2? list)
                   &optional (test 'equal) (key 'identity)
                   &rest list) 
  :initvals '(nil nil equal identity)
  :indoc '("first list" "second list" "test" "key" "optional list")
  :doc "XOR's 2 lists, into a single list (elements presents in one list and not in the other).
<test> is a name for a binary comparison function (default 'equal).
<key> is a name for a function that is applied to elements before comparison.
<test> and <key> may also be connected function or subpatch icons in 'lambda' mode."
  :icon 191
  (nreverse (noRep-union  (list* l1? l2? (and list (list! list))) 'set-exclusive-or test key)))

;-------XDIFF---------

(defmethod* x-diff ((l1? list) (l2? list)
                    &optional (test 'equal) (key 'identity)
                    &rest list) 
  :initvals '(nil nil equal identity)
  :indoc '("first list" "second list" "test" "key" "optional list")
  :doc "Returns the list of elements present in l1? but not in l2?.
<test> is a name for a binary comparison function (default 'equal).
<test> may also be a connected function or subpatch icon in 'lambda' mode."
  :icon 191
  (nreverse (noRep-union  (list* l1? l2? (and list (list! list))) 'set-difference test key)))

;-------INCLUDED?---------

(defmethod* included? ((lst1 list) (lst2 list) &optional (test 'equal))
  :initvals '(nil nil equal)
  :indoc '("first list" "second list" "test")
  :doc "Tests if <lst1> is included in <lst2>.
This box compares two lists, returning true if all the elements 
in the first are also elements of the second. If the optional <test> argument
 is added, it is used as a predicate to detect equality between elements.
Default value for <test> is the function 'equal. "
  :icon 191
  (subsetp lst1 lst2 :test test))

 
(defvar *set-package* (omNG-protect-object (omNG-make-new-package "Sets")))


(AddGenFun2Pack  (list 'x-union 'x-intersect 'x-Xor 'x-diff 'included?) *set-package*)

(AddPackage2Pack *set-package* *basic-package*)