;OpenMusic
;
;Copyright (C) 1997, 1998, 1999, 2000 by IRCAM-Centre Georges Pompidou, Paris, France.
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
;
;Authors: Gerard Assayag and Augusto Agon

(in-package :om) 


;===========================================================
;EDITOR
;===========================================================

;;;==== PALETTE =====
(omg-defclass sound-Palette (PLaying-Palette)  () 
   (:default-initargs :source-name "musicpalette"))

(defmethod palette-act ((self sound-Palette) x)
   (unless (call-next-method)
     (let ((ed-view (panel (editor-assoc self))))
       (case x
         (8 (setf (cursor-p ed-view) t) 
          (setf (mode (editor-assoc self)) x))
         (7 (setf (cursor-p ed-view) nil)
          (setf (mode (editor-assoc self)) x)))
       (om-invalidate-view ed-view t)
       (om-invalidate-view self t)
       )))


     
;============= CONTROLS ===========
(omg-defclass Aiff-control (3dBorder-view) 
  ((dyn-ctrls-list :initform nil :accessor dyn-ctrl-list)))



(defmethod make-snd-ctrl-list ((self Aiff-control))
  (list 
     (om-make-dialog-item 'numBox
                                        (om-make-point 280 10)
                                        (om-make-point 22 14) (format () " ~D" (tracknum (object (om-view-container self))))
                                        :min-val 1
                                        :max-val 16
                                        :font *controls-font*
                                        :value (tracknum (object (om-view-container self)))
                                        :afterfun #'(lambda (item)
                                                      (setf (tracknum (object (om-view-container self))) (value item))
                                                      (report-modifications (om-view-container self)))
                          )
     (om-make-dialog-item 'numBox
                          (om-make-point 350 10)
                          (om-make-point 28 14) (format () " ~D" (vol (object (om-view-container self))))
                          :min-val 0
                          :max-val 100
                          :font *controls-font*
                                        :value (vol (object (om-view-container self)))
                                        :afterfun #'(lambda (item)
                                                      (setf (vol (object (om-view-container self))) (value item))
                                                      (report-modifications (om-view-container self)))
                          )
     (om-make-dialog-item 'numBox
                                        (om-make-point 435 10)
                                        (om-make-point 28 14) (format () " ~D" (pan (object (om-view-container self))))
                                        :min-val -100
                                        :max-val 100
                                        :font *controls-font*
                                        :value (pan (object (om-view-container self)))
                                        :afterfun #'(lambda (item)
                                                      (setf (pan (object (om-view-container self))) (value item))
                                                      (report-modifications (om-view-container self)))
                          )
     ))

(defmethod add-sound-params ((self Aiff-control))
  (setf (dyn-ctrl-list self) (make-snd-ctrl-list self))
  (om-add-subviews self
                   (om-make-dialog-item 'om-static-text (om-make-point 240 10) (om-make-point 40 12)
                                        "Track" :font *controls-font* :bg-color *controls-color*)
                   (om-make-dialog-item 'om-static-text (om-make-point 320 10) (om-make-point 40 12)
                                        "Vol" :font *controls-font* :bg-color *controls-color*)
                   (om-make-dialog-item 'om-static-text (om-make-point 396 10) (om-make-point 40 12)
                                        "Pan" :font *controls-font* :bg-color *controls-color*)
                   (first (dyn-ctrl-list self))
                   (second (dyn-ctrl-list self))
                   (third (dyn-ctrl-list self))
                   )
  t)

(defmethod update-controls ((self Aiff-control))
  (loop for item in (dyn-ctrl-list self) do (om-remove-subviews self item))
  (setf (dyn-ctrl-list self) (make-snd-ctrl-list self))
  (loop for item in (dyn-ctrl-list self) do (om-add-subviews self item))  
  t)
 

;;;=========== BAR ===========

(omg-defclass sound-titlebar (editor-titlebar) ())



(defmethod init-titlebar ((self sound-titlebar))
  (om-add-subviews self
                   (om-make-dialog-item 'om-static-text (om-make-point 10 2) (om-make-point 
                                                                              (+ 10 (om-string-size (format nil "File: ~s" (namestring (om-sound-file-name (object (om-view-container self))))) *om-default-font1b*))
                                                                              14)
                                        (format nil "File: ~s" (namestring (om-sound-file-name (object (om-view-container self)))))
                                        :bg-color *editor-bar-color*
                                        :font *om-default-font1b*
                                        )
                   (om-make-dialog-item 'om-static-text (om-make-point 10 13) (om-make-point 80 11)
                                        (format nil "SR: ~D" (round (om-sound-sample-rate (object (om-view-container self)))))
                                        :bg-color *editor-bar-color*
                                        :font *om-default-font1b*
                                        )
                   (om-make-dialog-item 'om-static-text (om-make-point 100 13) (om-make-point 80 11)
                                        (format nil "SS: ~D" (om-sound-sample-size (object (om-view-container self))))
                                        :bg-color *editor-bar-color*
                                        :font *om-default-font1b*
                                        )
                   ))


;;;====== RULER ==========
(omg-defclass static-ruler (ruler) () )

(defmethod strech-ruler-motion ((self static-ruler) pos) t)

(defmethod strech-ruler-release ((view static-ruler) pos) t)

(defmethod om-view-cursor ((self static-ruler)) nil)

;=======================EDITOR====================
   
(omg-defclass soundEditor (EditorView object-editor) 
   ((rulerx :initform nil :accessor rulerx)
    (mode :initform nil :accessor mode)
    (control :initform nil :accessor control)))

(defmethod get-score-class-ctrls ((self soundeditor)) 'Aiff-control)
(defmethod get-panel-class ((self soundEditor)) 'soundpanel)
(defmethod get-control-h ((self soundEditor)) 36)
(defmethod get-titlebar-class ((self soundeditor)) 'sound-titlebar)


;;; *mouse-window-event*
(defmethod editor-null-event-handler :after ((self soundEditor))  ;chercher *mouse-window-event* par tout et enlever 
   (do-editor-null-event self))


(defmethod editor-has-palette-p ((self soundEditor)) 'sound-palette)

(defmethod initialize-instance :after ((self soundEditor) &rest l)
   (declare (ignore l))
   (let* ((ed-view (om-make-view (get-panel-class self) 
                     :owner self
                     :scrollbars nil
                     :position (om-make-point 0 (get-control-h self)) 
                     :size (om-make-point (w self) (- (h self) (+ 25 (get-control-h self))))))
          (rulerx (om-make-view 'static-ruler  ; ruler ?...
                    :owner self
                    :axe 'x
                    :assoc-view ed-view
                    :zoom 1000
                    :minzoom 1
                    :position (om-make-point 0 (- (h self) (get-control-h self) 25)) 
                    :size (om-make-point (w self) 25)))
          (control (om-make-view (get-score-class-ctrls self) 
                     :owner self
                     :position (om-make-point 0 (- (h self) (get-control-h self))) 
                     :size (om-make-point (w self) (get-control-h self)))))
     
     (add-sound-params control)
     (setf (mode self) 7)
     (setf (control self) control)
     (setf (panel self) ed-view)
     (om-set-bg-color (control self) *controls-color*)
     (setf (rulerx ed-view) rulerx)
     (setf (rangex ed-view) (list 0 (get-obj-dur (object self))))
     (set-units-ruler ed-view rulerx)))

(defmethod update-editor-after-eval ((self soundEditor) val)
  (call-next-method)
  (update-controls (control self))
  )

(defmethod do-editor-null-event ((self soundEditor)) 
   (when (om-view-contains-point-p (panel self) (om-mouse-position self))
     (om-with-focused-view 
       (panel self) ;;(control self) 
       (let* ((pixel (om-mouse-position (panel self)))
              (point (pixel2point (panel self) pixel)))
         (om-erase-rect-content 0 0 80 13)
         (om-with-font (om-make-font *om-score-font* 12 :style '(:bold))
                       (om-draw-string 6 12 (format () "t: ~D ms" (om-point-h point))))))))

      

(defmethod update-subviews ((self soundeditor))
   (when (title-bar self)
     (om-set-view-size  (title-bar self) (om-make-point (w self) *titlebars-h*)))
   (om-set-view-size  (panel self ) (om-make-point (w self) (- (h self) (get-control-h self) *titlebars-h* 25)))
   (om-set-view-position (panel self ) (om-make-point 0 *titlebars-h*))
   (om-set-view-size  (control self) (om-make-point (w self) (get-control-h self)))
   (om-set-view-position (control self) (om-make-point 0 (- (h self) (get-control-h self))))
   (om-set-view-position (rulerx (panel self)) (om-make-point 0 (- (h self) 25 (get-control-h self))))
   (om-set-view-size (rulerx (panel self)) (om-make-point (w self) 25))
   (om-invalidate-view self))

(defmethod om-get-menu-context ((self soundEditor))
  (let* ((thesound (object self)))
    (when (pict-spectre thesound)
      (if (pict-spectre? thesound)
        (list (om-new-leafmenu "show waveform" #'(lambda () (change-sound-pict (panel self))))
              )
        (list (om-new-leafmenu "show spectre" #'(lambda () (change-sound-pict (panel self))))
              )))))


;;;======= PANEL =======
(omg-defclass soundPanel (om-scroller view-with-ruler-x cursor-play-view-mixin om-view-drag) 
  ((mode :initform 0 :accessor mode)
   (selection? :initform nil :accessor selection?)))

(defmethod editor ((self soundpanel)) (om-view-container self)) 

(defmethod selection-to-play-? ((self soundpanel))
  (and (cursor-interval self) 
       (not (= (car (cursor-interval self)) (cadr (cursor-interval self))))
       ))


;------------------------------------
;Events
;------------------------------------

(defmethod om-view-cursor ((self soundPanel))
   (if (cursor-p self) 
     (if (om-option-key-p)
       *om-hand-cursor*
       *om-i-beam-cursor*)
     *om-arrow-cursor*))

(defmethod handle-key-event ((self soundPanel) char)
   (case char
     (#\g (setf (grille-p self) (not (grille-p self)))
          (draw-grille self))
     ;;;(#\z (set-unset-cursor self))
     (:om-key-delete (delete-sound-marker self))
     (#\SPACE (if (Idle-p *general-player*)
                  (palette-act (palette *global-palette*) (if (selection-to-play-? self) 4 0))
                  (palette-act (palette *global-palette*) 1)))
     (otherwise (call-next-method))))


(defmethod mrk-before ((self sound) t-s)
  (let ((b 0))
    (loop for m in (markers self) do
          (when (and (> m b) (<= m t-s)) (setf b m)))
    b))

(defmethod mrk-after ((self sound) t-s)
  (let ((a (/ (get-obj-dur self) 1000.0)))
    (loop for m in (markers self) do
          (when (and (< m a) (>= m t-s)) (setf a m)))
    a))
 
(defmethod special-move-marker ((Self soundPanel) Point)
  (let* ((thesound (object (om-view-container self)))
         (dur (/ (om-sound-n-samples  thesound) (om-sound-sample-rate  thesound)))
         (dec 6)
         (Xsize 80)
         (Mydialog (om-make-window 'om-dialog
                                   :size (om-make-point (+ xsize 60) 40)
                                   :window-title "Onset"
                                   :maximize nil :minimize nil :resizable nil
                                   :bg-color *om-window-def-color*
                                   :position (om-add-points (om-view-position (window self)) (om-mouse-position self))))
         (Xed (om-make-dialog-item 'om-editable-text (om-make-point 10 8) (om-get-text-edit-size (om-make-point (- xsize 25) 16))
                                   (format nil "~F" (nth point (markers thesound)))
                                   :font *controls-font*)))
    (om-add-subviews mydialog 
                     xed
                     ;(om-make-dialog-item 'om-static-text (om-make-point 20 30) (om-make-point 50 16) "Onset"
                     ;                     :font *om-default-font2b*)
                                          
                     (om-make-dialog-item 'om-button (om-make-point xsize 4) (om-make-point 40 20) "ok"
                                          :di-action (om-dialog-item-act item 
                                                       (declare (ignore item))
                                                       (let ((val (read-from-string (om-dialog-item-text xed))))
                                                         (when (and (numberp val)
                                                                    (>= val 0)
                                                                    (<= val dur))
                                                           (setf (nth point (markers thesound)) val)
                                                           (setf (markers thesound)
                                                                 (remove-duplicates (sort (markers thesound) '<)))
                                                           (setf (selection? self) (list (position val (markers thesound))))))
                                                       
                                                       (om-return-from-modal-dialog mydialog ()))
                                          :default-button t))
    (om-modal-dialog mydialog)))

(defmethod change-sound-pict ((self soundPanel))
  (let* ((thesound (object (om-view-container self))))
  (setf (pict-spectre? thesound) (not (pict-spectre? thesound)))
  (om-invalidate-view self t)))

(defmethod om-view-click-handler ((self soundPanel) where)
  (if (cursor-p self)
    (if (equal (cursor-interval self) '(0 0))
      (new-interval-cursor self where)
      (unless (om-option-key-p) 
        (setf (cursor-interval self) '(0 0))
        (om-invalidate-view self t)
        ))
  (let* ((graph-obj (click-in-sound-marker-p self where)))
    (cond 
     ((om-option-key-p) (add-sound-marker self where))
     (graph-obj 
      (if (om-shift-key-p) (omselect-with-shift self graph-obj )
        (when (not (member graph-obj (selection? self) :test 'equal))
          (setf (selection? self) (list graph-obj)))))
     (t (unless (om-drag-selection-p self where)
          (setf (cursor-interval self) '(0 0))
          (control-actives self where)
          )))
    (om-invalidate-view self t)))
  )

(defmethod om-view-doubleclick-handler ((Self soundPanel) Where)
  (let ((Position-Obj (click-in-sound-marker-p self where)))
    (if Position-Obj
      (special-move-marker self position-obj)
      (let* ((time (/ (om-point-h (pixel2point self where)) 1000.0))
             (t1 (mrk-before (object (editor self)) time))
             (t2 (mrk-after (object (editor self)) time)))
        (setf (cursor-interval self) (list (round (* t1 1000)) (round (* t2 1000))))
        ))
    (om-invalidate-view self t)
    ))

(defmethod control-actives ((self soundPanel) where)
  (let* ((thesound (object (om-view-container self)))
         (dur (/ (om-sound-n-samples  thesound) (om-sound-sample-rate  thesound))) minx maxx)
    (unless (om-shift-key-p)
      (setf (selection? self) nil))
    (om-init-motion-functions self 'make-selection-rectangle 'release-selection-rectangle)
    (om-new-movable-object self (om-point-h where) (om-point-v where) 4 4 'om-selection-rectangle)))

(defmethod make-selection-rectangle ((self soundPanel) pos)
  (let ((rect  (om-get-rect-movable-object self (om-point-h pos) (om-point-v pos))))
    (when rect
      (om-update-movable-object self (first rect) (second rect) (max 4 (third rect)) (max 4 (fourth rect))))))

(defmethod release-selection-rectangle ((self soundPanel) pos)   
  (let* ((rect  (om-get-rect-movable-object self (om-point-h pos) (om-point-v pos)))
        (thesound (object (om-view-container self)))
        (dur (/ (om-sound-n-samples  thesound) (om-sound-sample-rate  thesound))) minx maxx)
    (when rect
      (om-erase-movable-object self)
      (let (user-rect )
        (setf user-rect (om-make-rect (first rect) (second rect) (+ (first rect) (third rect)) (+ (second rect) (fourth rect))))
        (setf minx (min (om-rect-left user-rect) (om-rect-right user-rect)))
        (setf maxx (max (om-rect-left user-rect) (om-rect-right user-rect)))
        (loop for item in (markers thesound)
              for k = 0 then (+ k 1) do
            (let ((marker-pix (round (* (w self) item) dur)))
              (when (and (>= marker-pix minx) (<= marker-pix maxx))
                (push k (selection? self )))))))))



(defmethod omselect-with-shift ((self soundPanel) graph-obj)
  (if (member graph-obj (selection? self) :test 'equal)
    (setf (selection? self) (remove  graph-obj (selection? self) :test 'equal))
    (push graph-obj (selection? self))))


(defmethod click-in-sound-marker-p ((self soundPanel) where &optional (approx 3))
  (let* ((thesound (object (om-view-container self)))
         (dur (/ (om-sound-n-samples  thesound) (om-sound-sample-rate  thesound)))
         (x (om-point-h where))
         rep)
    (loop for item in (markers thesound)
          for i = 0 then (+ i 1)
          while (not rep) do
          (let ((pix (round (* (w self) item) dur)))
            (when (and (> x (- pix approx) ) (< x (+ pix approx)) )
              (setf rep i))))
    rep))

(defmethod add-sound-marker ((Self soundPanel) Where)
  (let* ((thesound (object (om-view-container self)))
         (point (pixel2point self where))
         (newpoint (/ (om-point-h point) 1000.0)))
    (setf (markers thesound) (remove-duplicates (sort (cons newpoint (markers thesound)) '<)))
    (setf (selection? self) (list (position newpoint (markers thesound))))
    (om-invalidate-view self t)))

(defmethod delete-sound-marker ((Self soundPanel))
  (let* ((thesound (object (om-view-container self)))
         (copy (copy-list (markers thesound))))
    (loop for item in (selection? self) do
          (let ((list (remove (nth item copy) (markers thesound))))
            (setf (markers thesound) list)))
    (setf (selection? self) nil)
    (om-invalidate-view self t)))

(defmethod editor-select-all ((self soundeditor)) 
  (let ((thesound (object  self)))
    (setf (selection? (panel self) )
          (loop for item in  (markers thesound)
                for k = 0 then (+ k 1) collect k))
    (om-invalidate-view self t)))

;------------------------------------
;DRAW
;------------------------------------

(defmethod om-draw-contents ((self soundPanel))
  (call-next-method)   
  (let* ((thesound (object (om-view-container self)))
         (dur (/ (om-sound-n-samples  thesound) (om-sound-sample-rate  thesound)))
         (thepicture (pic-to-draw thesound)))
    (om-with-focused-view self
      (when (and thesound thepicture)
        (om-with-fg-color self *om-dark-gray-color*
          (om-draw-picture self thepicture (om-make-point 0 0) (om-view-size self))                  
          ;(om-draw-picture-part self thepicture (om-make-point 0 0) (om-view-size self
          ;                      (om-make-point (round (* (cadr (rangex self)) 
          ;                      (om-point-h (om-get-picture-size thepicture))) 
          ;                      (get-obj-dur thesound)) (om-point-v (om-get-picture-size thepicture)))
          ;                      (om-make-point 0 0) (om-make-point (w self) (h self)))
          )
        (om-with-fg-color self (om-make-color 1 0.2 0.2) ;*om-red-color*
          (loop for item in (markers thesound)
                for k = 0 then (+ k 1) do
                (om-with-line-size (if (member k (selection? self)) 3 1)
                  (om-draw-line (round (* (w self) item) dur) 0 (round (* (w self) item) dur) (h self)))))))
    (when (grille-p self)
      (draw-grille self))
    ;(when  (cursor-p self)
    (draw-interval-cursor self)
    ;  )
    ))


(defmethod draw-interval-cursor ((self soundPanel))
  (let* ((sys-etat (get-system-etat self))
         (interval (cursor-interval self))
         (pixel-interval (list (om-point-h (point2pixel self (om-make-big-point (car interval) 0) sys-etat))
                               (om-point-h (point2pixel self (om-make-big-point (second interval) 0) sys-etat)))))
    (om-with-focused-view self
      (draw-h-rectangle (list (car pixel-interval) 0 (second pixel-interval) (h self)) t t))))

(defmethod scroll-play-window ((self soundPanel)) t)


;;;------------------------------
;;; drag sound

(defmethod om-drag-selection-p ((self soundpanel) position) 
  (or (click-in-sound-marker-p self position)
      (and
        (<= (om-point-h (pixel2point self position)) (cadr (cursor-interval self)))
        (>= (om-point-h (pixel2point self position)) (car (cursor-interval self))))))

(defmethod om-drag-start ((self soundpanel))
  (setf (dragged-view *OM-drag&drop-handler*) self
        (dragged-list-objs *OM-drag&drop-handler*) nil
        (container-view *OM-drag&drop-handler*)  self
        (true-dragged-view *OM-drag&drop-handler*) self
        (drag-flavor *OM-drag&drop-handler*) :omvw)
  (let ((r (om-new-region))
        (marks (markers (object (editor self))))
        (m (click-in-sound-marker-p self (om-mouse-position self) 10)))
    (if m
      (setf (selection? self) (list m)
            (dragged-list-objs *OM-drag&drop-handler*) m
            r (om-set-rect-region r
                                  (- (om-point-h (point2pixel self (om-make-big-point (round (* (nth m marks) 1000)) 0) (get-system-etat self))) 2)
                                  0 
                                  (+ 2 (om-point-h (point2pixel self (om-make-big-point (round (* (nth m marks) 1000)) 0) (get-system-etat self))))
                                  (h self))
            )
      (when (cursor-interval self)
        (setf r (om-set-rect-region r
                                    (om-point-h (point2pixel self (om-make-big-point (car (cursor-interval self)) 0) (get-system-etat self)))
                                    0 
                                    (om-point-h (point2pixel self (om-make-big-point (cadr (cursor-interval self)) 0) (get-system-etat self)))
                                    (h self))))
      )
    (unwind-protect
      (om-drag-create-item-bounds self r)
      (om-dispose-region r)))
  )
  

(defmethod extract-sound-selection ((self soundpanel) &optional filename)
  (let ((file (or filename (om-choose-new-file-dialog :prompt "Choose a New File" :directory *om-outfiles-folder*))))
    (when file 
      (save-sound (sound-cut (object (editor self)) (car (cursor-interval self)) (cadr (cursor-interval self)))
                  file))
    ))



(defmethod om-drag-receive ((target soundpanel) (dragged-ref t))
  (let ((dragged (dragged-view *OM-drag&drop-handler*)))
    (if (equal dragged target)
      (let* ((newpos (om-mouse-position target))
             (draggedobj (dragged-list-objs *OM-drag&drop-handler*)))
        (when (integerp draggedobj)
          (setf (nth draggedobj (markers (object (editor target))))
                (/ (om-point-h (pixel2point target newpos)) 1000.0))
          (om-invalidate-view target)
          t
          ))
      )))

