
;OpenMusic
;
;Copyright (C) 1997, 1998, 1999, 2000 by IRCAM-Centre Georges Pompidou, Paris, France.
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
;
;Authors: Gerard Assayag and Augusto Agon

(in-package :om)

;===========================================================
;EDITOR
;===========================================================

;Palette
(omg-defclass midi-Palette (PLaying-Palette)  () 
   (:default-initargs :source-name "midi-palette"))

(defmethod palette-act ((self midi-Palette) x)
   (unless (call-next-method)
     (let ((ed-view (panel (editor-assoc self))))
       (case x
         (9 (grille-on-off ed-view))
         (10 (om-beep-msg "Not implemented yet"))
         (otherwise
          (unless (= x 6)
            (setf (mode (editor-assoc self)) x)
            (om-invalidate-view self t)))))))

;=============The control===========--
(omg-defclass Midi-control (om-view)  ())

(defmethod om-draw-contents :before ((self Midi-control))
   (om-with-focused-view self
     (om-draw-line 0 24 (w self) 24)
     (om-draw-string 25 8 "Pitch :")
     (om-draw-string 25 20 "X-time:")
     (om-draw-string 120 10 "file:")
     (om-draw-string 160 10 (namestring (MidiFileName (object (om-view-container self)))))))

(defmethod initialize-instance :after ((self Midi-control) &rest l)
   (declare (ignore l))
   (additional-port-menu self :pos (om-make-point 120 0) :in nil))
          

;=============The ruler y===========
(omg-defclass piano-Palette (om-view) 
   ((thepal :initform (om-load-and-store-picture 'piano-pict 'kernel) :accessor thepal)))

(defmethod om-draw-contents :before ((self piano-Palette))
  (om-draw-picture self (thepal self) (om-make-point 0 0) (om-make-point (w self) (h self))))

(defmethod om-view-click-handler ((self piano-Palette) where)
   (om-with-focused-view (panel (om-view-container self))
    (om-draw-line 0 (om-point-v where) (w (panel (om-view-container self))) (om-point-v where))))

(defmethod ruler-print-draw-grille ((self piano-Palette) axe offx offy system-etat)
   (declare (ignore axe offx offy system-etat)) t)
        

;------------------------------------
(omg-defclass MidiEditor (EditorView) 
   ((control :initform nil :accessor control)
    (mode :initform 7 :accessor mode)))

(defmethod editor-has-palette-p ((self MidiEditor)) 'midi-palette)

(defmethod editor-null-event-handler :after ((self MidiEditor))
  (do-editor-null-event self))

(defmethod do-editor-null-event ((self MidiEditor))
  (when (om-view-contains-point-p (panel self) (om-mouse-position self))
    (om-with-focused-view (control self) 
      (let* ((pixel (om-mouse-position (panel self)))
             (point (pixel2point (panel self) pixel)))
        (om-erase-rect-content 70 0 50 23)
        (om-draw-string 70 21 (format () "~D" (om-point-h point)))
        (om-draw-string 70 9 (format () "~D" (om-point-v point)))
        point))))


(defmethod update-subviews ((self MidiEditor))
  (om-set-view-size (panel self ) (om-make-point (- (w self) 25) (- (h self) 50)))
  (om-set-view-size (control self ) (om-make-point (w self) 25))
  (om-set-view-size (rulery (panel self )) (om-make-point 25 (- (h self) 50)))
  (om-set-view-size (rulerx (panel self )) (om-make-point (w self) 25))
  (om-set-view-position (rulerx (panel self )) (om-make-point 25 (- (h self) 25)))
  (om-invalidate-view self t))

(defmethod initialize-instance :after ((self MidiEditor) &rest l)
   (declare (ignore l))
   (let* ((tracklist (tracks (object self)))
          (ed-view (om-make-view 'midiPanel 
                     :owner self
                     :scrollbars nil
                     :position (om-make-point 25 25) 
                     :size (om-make-point (- (w self) 25) (- (h self) 50))))
          (control (om-make-view 'Midi-control 
                     :owner self
                     :position (om-make-point 0 0)
                     :size (om-make-point (w self) 25)))
          (rulery (om-make-view 'piano-Palette 
                    :owner self
                    :position (om-make-point 0 25) 
                    :size (om-make-point 25 (- (h self) 50))))
          (rulerx (om-make-view 'ruler 
                    :owner self
                    :axe 'x
                    :assoc-view ed-view
                    :zoom 1000
                    :minzoom 0.001
                    :position (om-make-point 25 (- (h self) 25)) 
                    :size (om-make-point (w self) 25))))
     (setf (rulerx ed-view) rulerx)
     (setf (control self) control)
     (setf (rulery ed-view) rulery)
     (setf (panel self) ed-view)
     (setf (rangex ed-view) (list 0 10000))
     (set-units-ruler ed-view rulerx)))


;------------------------------------
(omg-defclass midiPanel (om-scroller view-with-ruler-xy cursor-play-view-mixin) ())

(defmethod set-units-ruler  ((self midiPanel) (ruler ruler)) nil)

(defmethod om-view-cursor ((self midiPanel))
   (declare (ignore where))
   (cond
    ((= 8 (mode (om-view-container self))) *om-hand-cursor*)
    ((cursor-p self) *om-i-beam-cursor*) 
    (t *om-arrow-cursor*)
    ))

(defvar *midi-panel-last-click* nil)
(defmethod om-view-click-handler ((self midiPanel) where)
   (when (=  (mode (om-view-container self)) 8) 
     (om-init-motion-functions self 'scroll-system-motion nil)
     (setf *midi-panel-last-click* where)
     (setf (grille-p self) nil)))

(defmethod scroll-system-motion ((self midiPanel) pos)
  (let* ((initmouse *midi-panel-last-click*)
         (initrangex (rangex self))
         (x (om-point-h pos))
         (y (om-point-v pos))
         deltax)
      (setf deltax (* 100 (- (om-point-h initmouse) x)))
      (if (minusp (+ (first initrangex) deltax))
          (setf (rangex self) (list 0 (- (second initrangex) (first initrangex))))
        (setf (rangex self) (list (+ (first initrangex) deltax)
                                  (+ (second initrangex) deltax))))
      (om-invalidate-view (om-view-container self) t)
      (setf *midi-panel-last-click* (om-make-point x y))))

(defmethod select-all ((self midiPanel)) nil)


;=====================DRAW


;=== optimization of midi main panel drawing method
(defmethod om-draw-contents ((self midiPanel))
   (call-next-method)
   (om-draw-view-outline self)
   (when (grille-p self)
     (draw-grille self))
   (when  (cursor-p self)
       (draw-interval-cursor self))
   (om-with-focused-view self
     (loop for item in (tracks (object (om-view-container self)))
           for i = 0 then (+ i 1) do
           (om-with-fg-color nil (nth (mod i 16) *16-color-list*)
             (draw-track self item 
                         (first (rangex self)) (second (rangex self)) 
                         24 96 t)))))
    

(defmethod draw-track ((self midiPanel) (track MidiTrack) minx maxx miny maxy mode)
  (declare (ignore mode))
  (let* ((x-notes (give-notes-in-x-range track minx maxx))
         (notes (sort (give-notes-iny-range x-notes miny maxy) '< :key 'second))
         (ysize (round (h self) (- maxy miny))))
    (draw-track-notes self track notes ysize)))
    

(defmethod draw-track-notes ((self midiPanel) (track MidiTrack) notes ysize)
  (let ((sys-etat (get-system-etat self)))
    (loop for note in notes do
              (let* ((topleft (point2pixel self (om-make-big-point (second note) (first note)) sys-etat))
                     (rigth (round  (* (third note) (first sys-etat)))))
                (om-fill-rect (om-point-h topleft) (om-point-v topleft) rigth ysize)))))
                     

(defmethod get-obj-to-play ((self midiPanel))
   (list (object (om-view-container self)) 
         :port (get-edit-param (editor self) 'outport)))

(defmethod editor ((self midiPanel))
   (om-view-container self))

(defmethod window ((self midiPanel))
   (om-view-window self))

(defmethod handle-key-event ((self midiPanel) char)
   (case char
     (#\z (set-unset-cursor self))
     (#\SPACE (if (Idle-p *general-player*)
                  (palette-act (palette *global-palette*) 0)
                  (palette-act (palette *global-palette*) 1)))
     (otherwise (call-next-method))))

(defmethod get-selection-to-play ((self midiPanel))
   (let ((interval (cursor-interval self)))
     (values  (list (object (om-view-container self))
                    :interval interval
                    :port (get-edit-param (editor self) 'outport))
              (first interval )
              (second interval))))
         
;------------------------------------
;POINT CONVERSION AND RULERS
;------------------------------------
(defmethod get-system-etat ((self midiPanel)) 
  (let* ((durrangex  (abs (- (second (rangex self)) (first (rangex self)))))
         (sizex (w self))
         (sizey (h self))
         (factx (/ sizex durrangex))
         (facty (/ sizey 72))
         (offsetx (round (* (first (rangex self)) factx)))
         (offsety (round (* 24 facty))))
    (list factx facty offsetx offsety)))

(defmethod point2pixel ((self midiPanel) point sys-etat)
   (let* ((x (om-point-h point))
          (y (om-point-v point))
          (x1 (round (* x (first sys-etat))))
          (y1 (round (* y (second sys-etat)))))
     (om-make-point (- x1 (third sys-etat))  (+ (- (h self) y1) (fourth sys-etat)))))

(defmethod pixel2point ((self midiPanel) pixel)
   (let* ((x (om-point-h pixel))
          (y (om-point-v pixel))
          (sizeview (om-view-size self))
          (sizex (om-point-h sizeview))
          (sizey (om-point-v sizeview))
          (durpointx  (abs (- (second (rangex self)) (first (rangex self)))))
          (durpointy  72)
          (x1 (round (* x durpointx) sizex))
          (y1 (round (* y durpointy) sizey)))
     (om-make-big-point (+  x1 (first (rangex self)))  
                 (+  (- durpointy y1) 24))))

;=====================================================================
