(in-package :om)

(defmethod drop-allow-p ((D&DHandler omdrag-drop) (dragged OMPatch) (target score-element)) t)

(defmethod perform-drop ((D&DHandler omdrag-drop) (dragged patch-icon-frame) 
                           (target scoreeditor) position)
   (declare (ignore position))
   (let ((panel (panel target)))
     (unless (in-patch-mode? panel)
       (apply-patch panel (object dragged)))))

(defun apply-patch (panel patch)
  (handler-bind ((error #'(lambda (c)
                            (when *msg-error-label-on*
                              (om-beep-msg (string+ "Error while apply the dragged patch to the score " " " 
                                                (om-report-condition c)))
                              nil))))
    (when (selection? panel)
      (unless (compiled? patch)
        (if (and (lisp-exp-p patch) (editorframe patch))
          (compile-without-close (editorframe patch))
          (compile-patch patch)))
      (if (not (= 1 (length (get-patch-inputs patch))))
        (om-beep-msg "The patch has changed It has not exactly one input")
        (loop for item in (selection? panel) do
              (apply  (intern (string (code patch)) :om) (list item)))))
    (update-panel panel t) t))


;==============================================
;Drag-and-drop
;==============================================

(defvar *score-drag-click* (om-make-point 0 0))
(defmethod om-drag-selection-p ((self scorePanel) where) 
   (setf *score-drag-click* where)
   t)

(defmethod big-icon-p ((self scoreEditor)) t)
(defmethod get-drag-object ((self scorePanel)) 
   (if (= 0 (score-mode self)) (om-view-container self) (call-next-method)))

(defmethod get-pos-in-object ((self scorePanel) where)  (if (= 0 (score-mode self)) where (call-next-method) ))
(defmethod allow-move-element ((self scorePanel)) (if (= 0 (score-mode self)) t (call-next-method)))

;;; formerly drag-tracking-enter-view
(defmethod om-drag-enter-view ((view scorePanel))
   (declare (special *OM-drag&drop-handler*))
   (unless (or (= 0 (score-mode view)) (equal (target-view *OM-drag&drop-handler*) view))
     (call-next-method)))

(defmethod make-drag-region ((self scorePanel) region x0 y0 view)
   (if (= 0 (score-mode self))
     (let* ((reg1 (om-new-region))
            (mode-obj (grap-class-from-type  (obj-mode self))))
       (loop for item in (get-graph-selection? self mode-obj) do
             (let ((rect (rectangle item)))
               (om-set-rect-region reg1 (first rect)
                                (second rect)
                                (third rect)
                                (fourth rect))
               (setf region (om-union-region reg1 region))))
       (om-dispose-region reg1)
       region) 
     (call-next-method)))


(defmethod om-drag-start ((view scorePanel))
  (if (and 
       (= 0 (score-mode view))
       (= 7 (mode (editor view))))
       (let* ((theview (get-drag-object view))
             (click-in-view *score-drag-click*)
             (mode-obj (grap-class-from-type (obj-mode view)))
             (graph-obj 
              (without-interrupts (click-in-obj (graphic-obj view) mode-obj click-in-view)))
             )
        (when graph-obj
          (if (om-shift-key-p) (omselect-with-shift view graph-obj)
            (when (not (member (reference graph-obj) (selection? view) :test 'equal))
              (off-selection view)
              (select-note view graph-obj)))
          (om-invalidate-view view t)
          (let* ((region (om-new-region)))
            (setf region (make-drag-region view region 0 0 view))
            (setf (dragged-view *OM-drag&drop-handler*)  theview
                  (dragged-list-objs *OM-drag&drop-handler*)  (selection? view)
                  (container-view *OM-drag&drop-handler*)  theview 
                  (initial-mouse-pos *OM-drag&drop-handler*) (get-pos-in-object view click-in-view)
                  (drag-flavor *OM-drag&drop-handler*) :omsc)
            (unwind-protect
                (progn
                  (om-drag-create-item-bounds view region))
              (om-dispose-region region)))
          t))))



(defmethod om-drag-receive ((view scorePanel)
                            (dragged-view t))
  (if (equal (drag-flavor *OM-drag&drop-handler*) :omsc)
    (cond ((= 0 (score-mode view)) 
          (let* ((drop-pos (om-mouse-position view)))
            (setf (opt-key-p *OM-drag&drop-handler*) (om-option-key-p)
                  (target-view  *OM-drag&drop-handler*) (get-drag-object view)
                  (drop-mouse-pos *OM-drag&drop-handler*)  drop-pos) 
            (score-drag&drop *OM-drag&drop-handler*)))
         (t (call-next-method)))
    (call-next-method)))

(defmethod score-drag&drop ((D&DHandler omdrag-drop))
   (cond 
    ;;; pb : sur mac ca rentre jamais  et si ca rentre ca plante : (object <note>) ;;;
    ;;;((opt-key-p *OM-drag&drop-handler*) (score-duplicate-drop D&DHandler))
    ((eq (target-view  D&DHandler) (dragged-view  D&DHandler))
     (score-move-inside D&DHandler))
    (t (score-change-view D&DHandler))))

(defmethod test-receptor ((self scorepanel) where)
   (click-in-obj (graphic-obj self) 'grap-note where))
     
(defmethod score-move-inside ((D&DHandler omdrag-drop))
   (let* ((editor (target-view  D&DHandler))
          (panel (panel editor))
          (pos0 (drop-mouse-pos D&DHandler))
          (selection (selection? panel))
          (receptor (test-receptor panel pos0))
          (receptor (if receptor (reference receptor)))
          (some-item-used nil))
     (loop for item in selection do
           (setf some-item-used 
                 (or some-item-used
                     (do-score-reception panel D&DHandler receptor item))))
     some-item-used))

(defmethod score-change-view ((D&DHandler omdrag-drop))
  (let* ((editor-t (target-view  D&DHandler))
          (panel-t (panel editor-t))
          (editor-s (dragged-view  D&DHandler))
          (panel-s (panel editor-s))
          (pos0 (drop-mouse-pos D&DHandler))
          (selection (selection? panel-s))
          (receptor (test-receptor panel-t pos0)) 
          (some-item-used nil)
          ;; jb..
          (firstoffset (loop for item in selection minimize (offset->ms item (object editor-s))))
          )
     (loop for item in selection do
           (setf some-item-used 
                 (or some-item-used
                     (do-score-change-view panel-t panel-s (if receptor (reference receptor) pos0) item 
                                           (- (offset->ms item (object editor-s)) firstoffset) 
                                           ;;; nouveau parametre optionel : offset par rapport au drag-pos (en ms)
                                           ))))
     some-item-used))

(defmethod score-duplicate-drop ((D&DHandler omdrag-drop))
   (let ((target-frame (get-drag-object (target-view  D&DHandler)))
         (pos0 (get-relative-position (get-drag-object (dragged-view D&DHandler))))
         (some-item-used t)
         nameerr)
     (when (allow-move-element (target-view  D&DHandler))
       (mapc #'(lambda (dragged-frame)
                 (when (or (no-allow-copy-p (object dragged-frame))
                           (not (drop-allow-p D&DHandler (object dragged-frame) (object target-frame)))
                           (null (can-copy (object target-frame) (object dragged-frame))))
                   (setf nameerr (name (object dragged-frame)))
                   (setf some-item-used nil))) (dragged-list-objs D&DHandler))
       (if some-item-used
         (let* ((subframes (mapcar #'(lambda (frame)
                                       (object frame)) (dragged-list-objs D&DHandler)))
                (copies (mapcar #'(lambda (frame)
                                    (eval (omNG-copy frame))) subframes)))
           (setf some-item-used (perform-duplicate-list D&DHandler (object target-frame) target-frame subframes copies pos0)))
         (om-beep-msg (string+ "The object " nameerr " can not be copied to this window"))))
     some-item-used))

(defmethod drop-allow-p ((D&DHandler omdrag-drop) (dragged score-element) (target OMPatch)) t)
(defmethod drop-allow-p ((D&DHandler omdrag-drop) (dragged score-element) (target score-element)) t)

(defmethod vertical-move? ((D&DHandler omdrag-drop))
   (let* ((p0 (initial-mouse-pos D&DHandler))
          (p1 (drop-mouse-pos D&DHandler))
          (m (m-line p0 p1)))
     (> (abs m) 1)))
     

;=========================================================
;
;=========================================================


(defmethod do-score-reception ((self scorePanel) (D&DHandler omdrag-drop) (receptor t) (dragged t))
   t)

;-----------transpose or move

(defmethod do-score-reception ((self scorePanel) (D&DHandler omdrag-drop) 
                                 (receptor null) (dragged simple-container))
  (transpose-drag self (dragged-list-objs D&DHandler) (initial-mouse-pos D&DHandler)))

(defmethod do-score-reception ((self chordseqPanel) (D&DHandler omdrag-drop) 
                                 (receptor null) (dragged chord))
   (if (vertical-move? D&DHandler)
     (transpose-drag self (dragged-list-objs D&DHandler) (initial-mouse-pos D&DHandler))
     (when (translate-chords-p self)
       (move-chords-in-x self (initial-mouse-pos D&DHandler) (drop-mouse-pos D&DHandler) 
                         (dragged-list-objs D&DHandler) t))))

(defmethod do-score-reception ((self chordseqPanel) (D&DHandler omdrag-drop) 
                                 (receptor null) (dragged chord-seq))
   (if (vertical-move? D&DHandler)
     (transpose-drag self (dragged-list-objs D&DHandler) (initial-mouse-pos D&DHandler))
     (when (translate-chords-p self)
       (move-chords-in-x self (initial-mouse-pos D&DHandler) (drop-mouse-pos D&DHandler) 
                         (inside (car (dragged-list-objs D&DHandler))) nil))))


;=========================================================
;
;=========================================================
(defmethod do-score-change-view ((target scorePanel) (source scorePanel) (receptor t) (dragged t) &optional (offset 0))
  t)

(defmethod do-score-change-view ((target notePanel) (source scorePanel) (receptor t) (dragged t) &optional (offset 0))
   (om-beep))


;;; cf add-new-object
(defmethod add-object-in-obj ((chordseq chord-seq) (new-chord chord) time)
  (let (new-chord-list)
    (setf (offset new-chord) time)
    (loop for item in (chords chordseq) do
          (setf (offset item) (offset->ms item)))
    (setf new-chord-list (cons new-chord (chords chordseq)))
    (setf (inside chordseq) (sort new-chord-list '< :key 'offset)) 
    (setf (Qvalue chordseq) 1000)
    (adjust-extent chordseq)
    (QNormalize chordseq)
    ))

(defmethod add-object-in-obj ((chordseq chord-seq) (new-chord chord-seq) time) 
  (loop for chord in (chords new-chord) do
        (add-object-in-obj chordseq chord (+ time (offset->ms chord)))))


(defmethod do-score-change-view ((target chordseqPanel) (source scorePanel) (receptor t) (dragged chord) &optional (offset 0))
  (let ((pos (pixel-toms target receptor))
        (cseq (reference (graphic-obj target)))
        (new-chord (clone dragged)))
    (setf pos (if (and pos (> pos 0)) pos 0))
    (add-object-in-obj cseq new-chord (+ pos offset))
    (update-panel target t)
    
    nil   ;;; return some-item-used??
    ))

(defmethod do-score-change-view ((target chordseqPanel) (source scorePanel) (receptor t) (dragged note) &optional (offset 0))
  (let ((pos (pixel-toms target receptor))
        (cseq (reference (graphic-obj target)))
        (new-chord (mki 'chord  :Lmidic (list (midic dragged)))))
    
    (setf pos (if (and pos (> pos 0)) pos 0))
    (add-object-in-obj cseq new-chord (+ pos offset))
    (update-panel target t)
    
    nil   ;;; return some-item-used??
    ))

(defmethod do-score-change-view ((target chordseqPanel) (source scorePanel) (receptor t) (dragged chord-seq) &optional (offset 0))
  (let ((pos (pixel-toms target receptor))
        (cseq (reference (graphic-obj target))))
    (setf pos (if (and pos (> pos 0)) pos 0))
    (add-object-in-obj cseq (clone dragged) (+ pos offset))
    (update-panel target t)
    nil   ;;; return some-item-used??
    )) 


;=========================================================
;PATCH AND MAQ RECEIVE
;=========================================================


(defmethod score-drag-receive ((view om-view-drop ) 
                               (dragged-view t))
  (let* ((drop-pos (om-mouse-position view))   ;;;(drag-mouse-drop-position view))
         (*receiving-in-drag* t))
    (setf (opt-key-p *OM-drag&drop-handler*)  (om-option-key-p)      ;;; (drag-option-key-p view)
          (target-view  *OM-drag&drop-handler*)      (get-drag-object view)
          (initial-mouse-pos *OM-drag&drop-handler*) (om-get-current-drag-initpos (dragged-view *OM-drag&drop-handler*)) 
          (true-target-view *OM-drag&drop-handler*)  view 
          (drop-mouse-pos *OM-drag&drop-handler*)    (get-pos-in-object view  drop-pos)) 
    (score-finalize-drag&drop *OM-drag&drop-handler*)))

(defmethod score-finalize-drag&drop ((D&DHandler omdrag-drop))
  (when (drop-allow-p D&DHandler (object (dragged-view *OM-drag&drop-handler*))
                      (object (true-target-view *OM-drag&drop-handler*)))
    (score-to-patch (true-target-view *OM-drag&drop-handler*)
                    (panel (dragged-view *OM-drag&drop-handler*))
                    (car (dragged-list-objs D&DHandler))
                    (dragged-list-objs D&DHandler)
                    (drop-mouse-pos D&DHandler))))

;---------note
(defmethod make-and-add-box ((self patchpanel) obj position)
  (let* ((newcall (omNG-make-new-boxcall (class-of obj) position (mk-unique-name self "dragged"))))
     (setf (value newcall) obj)
     (setf (showpict newcall) t)
     (omG-add-element self (make-frame-from-callobj  newcall)) t))

(defmethod make-and-add-box ((self maquettepanel) obj position)
  (let* ((maqpos (get-offset/posy-from-pixel self position))
         (newcall (omNG-make-tempobj (omNG-make-new-instance  obj "dragged") maqpos 
                                     (mk-unique-name self "dragged"))))
     (setf (offset newcall) (om-point-h  maqpos))
     (setf (posy newcall) (om-point-v  maqpos))
     (setf (showpict newcall) t)
     (omG-add-element self (make-frame-from-callobj  newcall)) t))


(defmethod score-to-patch ((self patchpanel) (source-view t) (type-obj t) (obj-list t) (position t)) nil)

(defmethod score-to-patch ((self patchpanel) (source-view scorepanel) (type note) 
                             (obj-list list) (position t))
   (let* ((newobj (if (= (length obj-list) 1) (clone (car obj-list))
                      (objfromobjs obj-list (make-instance 'chord)))))
     (make-and-add-box self newobj position) t))

;;; modif jb 10-04
;;;(defmethod* mk-obj-from-drag-list ((Self list) (Type Chord-seq))
;;;  (cond
;;;   ((list-subtypep self '(chord note))
;;;    (let ((chord-seq (make-instance (type-of type) :empty t)))
;;;      (setQValue chord-seq 1000 :recursive nil)
;;;      
;;;      (setf (inside chord-seq) 
;;;            (mapcar #'(lambda (object) (ObjfromObjs object (mki (type-of object))))
;;;              self))
;;;      
;;;      (let (min)
;;;        (loop for chord in (inside chord-seq) do
;;;                (when (or (not min) (< (offset chord) min)) (setf min (offset chord))))
;;;        (loop for chord in (inside chord-seq) do
;;;                (when (setf (offset chord) (- (offset chord) min)))))
;;;      ;;;(loop for chord in (inside chord-seq) 
;;;      ;;;      for onset from 0 by 1000
;;;      ;;;      do  (setf (slot-value chord 'offset)  onset)
;;;      ;;;        (InContext chord-seq (setf (extent chord) 1000)))
;;;      (QNormalize chord-seq)
;;;      (adjust-extent chord-seq)
;;;      chord-seq))    
;;;   (t nil)))

(defmethod* mk-obj-from-drag-list ((Self list) (Type Chord-seq))
  (cond
   ((list-subtypep self '(chord note))
    (let* ((clist (sort self '< :key 'offset->ms))
          (minoffset (offset->ms (car clist)))
          offset-list onset-list midic-list vel-list dur-list chan-list
          chord-seq)
      (loop for chord in clist do
          (pushr (- (offset->ms chord) minoffset) onset-list)
          (pushr (loffset chord)  offset-list)
          (pushr (ldur chord)  dur-list)
          (pushr (lmidic chord)  midic-list)
          (pushr (lvel chord)  vel-list)
          (pushr (lchan chord)  chan-list)
            )
      (setf chord-seq (make-instance (type-of type) :lonset onset-list 
                        :lmidic midic-list :ldur dur-list :loffset offset-list :lvel vel-list :lchan chan-list))
      ;;; new tonalite
      (loop for c in clist
            for i = 0 then (+ i 1) do
            (when (tonalite c) (set-tonalite (nth i (inside chord-seq)) (tonalite c)))) 
      (when (get-tonalite (parent (car clist))) (set-tonalite chord-seq (get-tonalite (parent (car clist)))))
      ;;;
      
      chord-seq))    
   (t nil)))

;;; modif jb 10-04
(defmethod score-to-patch ((self patchpanel) (source-view scorepanel) (type chord) 
                             (obj-list list) (position t))
   (let* ((newobj (if (= (length obj-list) 1) (let ((c (clone (car obj-list))))
                                                (set-tonalite c (get-tonalite (car obj-list)))
                                                c)
                    (mk-obj-from-drag-list obj-list (make-instance 'chord-seq)))))
     (if newobj
         (progn
           (make-and-add-box self newobj position)
           t)
       nil)))


(defmethod score-to-patch ((self patchpanel) (source-view scorepanel) (type chord-seq) 
                             (obj-list list) (position t))
   (let* ((newobj (if (= (length obj-list) 1) (clone (car obj-list))
                      (make-instance 'multi-seq :chord-seqs (clone obj-list)))))
     (make-and-add-box self newobj position) t))

(defmethod score-to-patch ((self patchpanel) (source-view scorepanel) (type multi-seq) 
                             (obj-list list) (position t))
   (let* ((newobj (clone (car obj-list))))
     (make-and-add-box self newobj position) t))

(defmethod score-to-patch ((self patchpanel) (source-view scorepanel) (type rest) 
                           (obj-list list) (position t))
  (let* ((newobj (if (= (length obj-list) 1) (clone (car obj-list))
                     (objfromobjs (loop for item in obj-list append (cons-chord-list item))
                                  (make-instance 'chord-seq)))))
    (make-and-add-box self newobj position) t))



;;;============================
;;; SPECIAL AUDIO EDITOR TO PATCHPANEL


(defmethod om-drag-receive ((target patchpanel) (dragged-ref t))
  (let ((dragged (dragged-view *OM-drag&drop-handler*))
        (posi (om-mouse-position target)))
    (if (subtypep (class-name (class-of dragged)) 'soundpanel)
        (unless (integerp (car (selection? dragged)))
      (let* ((newsoundfile (extract-sound-selection dragged))
             newsound newbox)
        (when newsoundfile (setf newsound (load-sound-file newsoundfile)))
        (when newsound
          (setf newbox (make-instance 'OMBoxEditCall
                         :name "sound extract"
                         :reference (class-of newsound) 
                         :icon (icon (class-of newsound))
                         :inputs (get-inputs-from-inst newsound)))
          (setf (value newbox) newsound)
          (set-edition-params (value newbox) newbox)
          (push newbox (attached-objs (class-of newsound)))
          (setf (numouts newbox) (length (get-outs-name (value newbox))))
          (setf (frame-position newbox) 
                (borne-position posi))
          (loop for item in (get-actives target) do
               (omG-unselect item))
          (let ((new-frame (make-frame-from-callobj newbox)))
            (omG-add-element target new-frame)
            (omG-select new-frame))
          )
        (om-invalidate-view target)
        t
        ))
      (call-next-method))))
