;OpenMusic
;
;Copyright (C) 1997, 1998, 1999, 2000 by IRCAM-Centre Georges Pompidou, Paris, France.
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
;
;Authors: Gerard Assayag and Augusto Agon


;DocFile
;This File containts Midi initialization, termination and management.
;Last Modifications :
;17/11/97 A temporal scheduler is implemented, we will like to change
;         this scheduler for another one more performant.
;DocFile

(in-package :om)


(defvar *player* nil      "refnum of the player")
(defvar *recorder* nil    "refnum of the recorder")
(defvar *filter* nil      "allow filter midievents in the recorder")
(defvar *Outmidiport* 0      "default output port number")
(defvar *Inmidiport* 0      "default input port number")
 

;;;Open MidiShare and connections
;;;
;;;=== MidiOpen : redefinition 
;;;== modif : connection du recorder
(defun midi-open ()
  "Check if MidiShare is present, if this is the case open the player and
the recorder, this function is called by a def-load-pointers"
  (setf *player* nil)
  (om-midi-startup)
  (if *MIDI*
      (om-without-interrupts  
        ;;; open player
        (open-ms-players)   
        (add-assoc-player *general-player* 'midishare)
        ;;; set scheduler time to midi time
        (om-stop-scheduler)
        (defun clock-time () (om-midi-get-time))
        (init-scheduler)
        )
    (om-message-dialog "MidiShare not present. OM won't play Midi."))
  t)


;;;Close MidiShare and off the scheduler
(defun midi-close ()
   "If MidiShare is present, close the player and the recorder before quit the application,
this function push in the CCL:*lisp-cleanup-functions* variable"
   (om-stop-scheduler)
   (when *MIDI*
     (close-ms-players)
     (om-midi-exit)
     ))

(defun open-ms-players ()
  (setq *player* (om-midi-open-player "OMPlayer"))
  (setf *recorder* (om-midi-open-player "OMRecorder"))
  )

(defun close-ms-players ()
  (when *player* (om-midi-close-player *player*))
  (when *recorder* (om-midi-close-player *recorder*))
  )

(defun make-port-menu (list posi) (declare (ignore list posi)))

;;;===============================
(om-add-init-func 'midi-open)  
(om-add-exit-cleanup-func 'midi-close t)
;;;===============================



;;;(ms::midishare)                                           	; <== EVALUATE THIS EXPRESSION.
;;;(defparameter *refnum* (ms::midiopen "Common Lisp"))     	; <== EVALUATE THIS EXPRESSION.
;;;
;;;
;;;(ms::MidiConnect *refnum* 0 -1)
;;;
;;;(defun send-note (pitch)
;;;  (let ((event (ms::MidiNewEv ms::typeNote)))	; ask for a new note event
;;;    (ms::chan event 0)			; set the midi channel to 0 (means channel 1)
;;;    (ms::port event 0)			; set the destination port to 0
;;;    (ms::field event 0 pitch)		; set the pitch field
;;;    (ms::field event 1 100)		; set the velocity field
;;;    (ms::field event 2 1000)		; set the duration field to 1 second
;;;    (ms::MidiSendIm *refnum* event))	; send the note immediatly
;;;  )							; <== EVALUATE THIS DEFINITION
;;;
;;;
;;;
;;;(send-note 60)	