;OpenMusic
;
;Copyright (C) 1997, 1998, 1999, 2000 by IRCAM-Centre Georges Pompidou, Paris, France.
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
;
;Authors: Gerard Assayag and Augusto Agon

(in-package :om)

;=================================================
;Midi File Object
;=================================================

(defclass Midi-Score-Element (simple-score-element) ())


(defclass InternalMidiFile () 
  ((MidiFileName :initform nil :initarg :MidiFileName :accessor MidiFileName)
   (fileseq :initform nil :accessor fileseq)
   (tracks :initform nil :initarg :tracks :accessor tracks)
))


(defclass* MidiFile (midi-score-element InternalMidiFile) () 
  (:icon 904)
  (:documentation "
Inputs (left to right)

a midifile object

Outputs (left to right)

a midifile object

Midifile factories are generally obtained by dragging a midifile from 
the finder to a patch window. After doing that, lock immediatly the Midifile 
icon, otherwise OM will complain if you try to evaluate it.

Play the Midifile by selecting the icon and pressing 'p' on the keyboard.

To convert a midifile to symbolic data, connect a mf-info function 
(package music:score) to the output of Midifile. 

You may also obtain a MidiFile factory in a patch by dragging the icon 
of the MidiFile Class from the Package music:score to your patch. 
The MidiFile factory will come empty.

When you evaluate a Midifile factory which is not locked and with 
nothing connected to its input, OM issues a file dialog that lets you 
choose a Midi file to be loaded in the factory.

Dbl-clicking on a Midifile icon opens a Midi displayer.")
  )


(defmethod get-type-of-ed-box ((self MidiFile))  'OMMidiFilebox)

(defmethod real-dur ((self midifile)) (round (extent->ms self)))

(defmethod real-duration ((self midifile) time) 
  (values time (+ time (round (extent->ms self)))))

(defmethod copy-container ((self midifile) &optional (pere ()))
  (let ((copy (make-instance 'MidiFile))
        (slots  (class-instance-slots (find-class 'simple-container))))
    (when (MidiFileName self)
      
      (setf (MidiFileName copy) (MidiFileName self))
      (setf (fileseq copy) (copy-midiseq (fileseq self)))
      (setf (tracks copy) (loop for track in (tracks self) 
                                collect  (make-instance 'MidiTrack
                                           :midinotes  (midinotes track))))
      (setf (slot-value copy 'parent) pere)
      (loop for slot in slots
            when (not (eq (slot-definition-name slot) 'parent))
            do (setf (slot-value  copy  (slot-definition-name slot))
                     (copy-container (slot-value self  (slot-definition-name slot)) copy))))
    copy))

(defmethod execption-save-p ((self midifile)) 'midifile)

(defmethod save-exepcion ((self midifile)) 
   (and (MidiFileName self) `(load-midi ,(om-save-pathname-relative (MidiFileName self)))))

;=================================================
;Midi track Object
;=================================================

(defclass MidiTrack () 
  ((midinotes :initform nil :initarg :midinotes :accessor midinotes)))


;--------------------set notes to Track ------------------
(defmethod cons-midi-track ((self MidiTrack) notes)
  (setf (midinotes self) notes))


;-------------------get the notes fall in the interval (x1 x2) ------------------------------
(defmethod give-notes-in-x-range ((self MidiTrack) x1 x2)
  (let ((notes (midinotes self))
        res-notes)
    (loop while (and notes (> x1 (second (car notes)))) do
          (pop notes))
    (loop while (and notes (> x2 (second (car notes)))) do
          (push (pop notes) res-notes))
    (nreverse res-notes)))

;-------------------get the notes fall in the interval (y1 y2) ------------------------------
(defun give-notes-iny-range (notes y1 y2)
  (let (res-notes)
    (loop for note in notes do
          (when (and (>= y2 (first note)) (<= y1 (first note)))
            (push note res-notes)))
    (nreverse res-notes)))


;---------------------
(defmethod make-one-instance ((self midifile) &rest slots-vals)
   (get-midifile))

(defmethod good-val-p? ((self midifile))
   (midifilename self))

(defmethod Class-has-editor-p ((self MidiFile)) t)

(defmethod get-initval ((self MidiFile)) (make-instance 'midifile))

(defmethod print-object ((self MidiFile) x) 
  (format x "MidiFile"))

(defmethod* objfromObjs ((self t) (type MidiFile))
   (when self
       (objFromObjs (save-as-midi self) type)))

(defmethod* objfromObjs ((self string) (type MidiFile))
   (load-midi-file self))

(defmethod* objfromObjs ((self pathname) (type MidiFile))
   (load-midi-file self))



(defmethod get-boxsize ((self MidiFile)) (om-make-point 50 72))

(defmethod get-editor-class ((self MidiFile)) 'MidiEditor)

        
 ;=======EDITOR

(defclass OMMidiFilebox (OMBoxEditCall) ())

(defmethod get-frame-class ((self OMMidiFilebox)) 'boxmidiframe)

(omg-defclass boxmidiframe (boxEditorFrame) ())

(defmethod om-get-menu-context ((object boxmidiframe))
  ;(list+  
  ; (list 
  ;  (list (om-new-leafmenu (if (view-of-patch (object object)) "Free View of Patch" "Set View of Patch")
  ;                         #'(lambda () (change-view-of-patch object)))
  ;        )
   ; ) 
  (boxframe-default-list object))

(defmethod remove-extra ((self OMPatch) (box OMMidiFilebox))
   (when (and (value box) (fileseq (value box)))
      (om-midi-free-seq (fileseq (value box)))))

    
(defun copy-midiseq (seq)
  (om-midi-copy-seq seq))

(defun copy-midiseq-without-tempoevents (seq)
   (om-midi-copy-seq seq '(:type 144)))

;ojo deberia ser standard
;notalist (midi ? vel ? ttime ?)



;======================================================================================
;=== Loading a midifile :                                                          
;=== - dates of midievents are converted in ms                                  
;=== (tempo = 60 --> 1 qter note = 1 s --> 1 mclck = 1ms if 1000 mck/qter note)     
;=== - tracks containing list of midinotes (pitch date dur vel channel) are created
;=== !!! tempo change events are still in the sequence
;======================================================================================

             
(defun load-midi-file (name) 
  (let ((themidiFile (make-instance 'MidiFile))
        rep track-list err)
    (multiple-value-bind (err recording-seq nbtracks clicks format timedef)
        (om-midi-load-file (om-path2cmdpath name) (om-midi-new-seq))
      (unless (and err (zerop err)) (om-beep-msg "error loading a midi file") (om-abort))
      (when recording-seq
        (setf  (fileseq themidiFile) recording-seq)
        (setf  (fileseq themidiFile) (convert-tempo-info recording-seq clicks))
        (setf track-list (make-list nbtracks :initial-element nil))
        (setf rep (mievents2midilist (fileseq themidiFile)))
        (loop for note in rep do
              (if (plusp (third note))
                  (push (list (first note) (second note) (third note) (fourth note) (fifth note)) (nth (sixth note) track-list))))
        (setf (MidiFileName themidiFile) name)
        (setf (extent themidiFile) (loop for track in track-list 
                                         if track maximize (+ (third (car track)) (second (car track)))))
        (setf (Qvalue themidiFile)  1000)
        (setf (tracks themidiFile) (loop for track in track-list 
                                         if track collect  (make-instance 'MidiTrack
                                                                          :midinotes  (reverse track))))
        themidifile))))

(defmethod* get-midifile () 
            :initvals nil :indoc nil :icon 148
            (let ((name (om-choose-file-dialog)))
              (when name
                (load-midi-file name))))

(defun load-midi (name)
  (om-load-if name 'load-midi-file))




;=====================================
;==== Midi file analysis functions ===
;=====================================

;=================
; FUNCTIONS
;=================
(defmethod* mf-info ((self midifile) &optional (tracknum nil))
  :initvals (list nil nil ) :indoc '("a Midifile object" "a track number or nil")
   :doc "Converts a Midifile object into a symbolic description.
The result of mf-info is a list of tracks. Each track is a list of notes. 
Each note is a list of parameters in the form :

(midi-number (pitch) , onset-time(ms), duration(ms), velocity, channel)

optional <tracknum> (a number in 0-15) allows to choose a single track."
  :icon 148
  (if tracknum
    (midinotes (nth tracknum (tracks self)))
    (loop for item in (tracks self)
          collect (midinotes item))))


;=== Creates MidiEvent list with all Midi events 
;=== optionnaly filtered with a test function
(defmethod! get-midievents ((self midifile) &optional test)
  :initvals '(nil nil) 
  :indoc '("midifile" "lambda test function for midi events to extract") 
  :icon 902
  :doc "Extracts a list of midi events from a Midi file"
   (let ((seq (fileseq self))
         msevent event (rep nil))
     (when seq
       (setf msevent (om-midi-seq-first-evt seq))
       (loop while msevent do
             (setf event (make-instance 'MidiEvent   
                                        :ev-type (om-midi-evt-get msevent :type) 
                                        :ev-date (om-midi-evt-get msevent :date) 
                                        :ev-ref (om-midi-evt-get msevent :ref) 
                                        :ev-port (om-midi-evt-get msevent :port)
                                        :ev-chan (+ (om-midi-evt-get msevent :chan) 1)
                                        :ev-fields (om-midi-evt-get msevent :fields)))
             (if (= (om-midi-evt-get msevent :type) (om-midi-get-num-from-type "Tempo")) 
                 (setf (ev-fields event) (list (mstempo2bpm (car (om-midi-evt-get msevent :fields))))))
             (if (or (not test) (funcall test event))  
                 (setf rep (cons event rep)))
             (setf msevent (om-midi-next-evt msevent))
             )
       (reverse rep))))

;=== Creates a string with Lyric events from Midi file
(defmethod! get-mf-lyrics ((self midifile))
  :initvals '(nil) 
  :indoc '("midifile") 
  :doc "Extracts lyrics from a Midi file"
  :icon 908
  (let ((seq (fileseq self))
         event str rep)
     (when seq
       (setf event (om-midi-seq-first-evt seq))
       (loop while event do
             (if (= (om-midi-evt-get event :type) (om-midi-get-num-from-type "Lyric"))
                   (setf rep (cons (om-midi-get-evt-text event) rep)))
               ;(setf rep (cons (list2string (ms::fieldsList event)) rep)))
               (setf event (om-midi-next-evt event))
               ))
     (strlist2string (reverse rep))))

(defmethod! get-midi-notes ((self midifile))
  :initvals '(nil) 
  :indoc '("midifile") 
  :doc "Extracts notes from a Midi file"
  :icon 909
  (mf-info self))


;========== conversions

(defmethod* objFromObjs ((self midifile) (type chord-seq))
  (let ((newcs (make-instance (type-of type)))
        (midilist (sort (loop for item in (mf-info self)
                              append item) '< :key 'second))) 
    (setQValue newcs 1000 :recursive nil)
    (setf (inside newcs) nil)
    (setf (inside newcs)  (make-quanti-chords midilist *global-deltachords*))
    (adjust-extent newcs)
    (QNormalize newcs)
    newcs
    ))

(defmethod* objFromObjs ((self midifile) (type multi-seq))
  (let ((midilist (mf-info self)) rep)
    (loop for item in midilist do
          (let ((newcs (make-instance 'chord-seq)))
            (setQValue newcs 1000 :recursive nil)
            (setf (inside newcs) nil)
            (setf (inside newcs)  (make-quanti-chords item *global-deltachords*))
            (QNormalize newcs)
            (push newcs rep)))
    (make-instance 'multi-seq
      :chord-seqs (reverse rep))))

