;OpenMusic
;
;Copyright (C) 1997, 1998, 1999, 2000 by IRCAM-Centre Georges Pompidou, Paris, France.
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
;
;Authors: Gerard Assayag and Augusto Agon
(in-package :om)


(defmethod add-ruler-metric ((self MaquetteEditor))
  (let* ((panel (panel self))
         (editor (editor self))
         (themaq (object panel))
         (params (metricparam (params themaq))))
    (setf (rulermetric editor) (om-make-view (maq-metric-ruler-class self) 
                                             :owner editor
                                             :axe 'x
                                  :zoom 1000
                                  :minzoom 0.001
                                  :tempo (first params)
                                  :meslist (second params)
                                  :maxsub (third params)
                                  :loop-mes-p (fourth params)
                                  :assoc-view panel
                                  :position (om-make-point 25 0) 
                                  :size (om-make-point (- (w self) 25) 25)))
     (om-set-bg-color (rulermetric editor) (om-get-bg-color panel))
     (maq-show-ruler themaq "on")
     (update-all-offsets editor)
    (update-subviews self)))

(defmethod get-metric-values ((self OMmaquette))
   "Get a list (tempo metric minval loopflag)."
   (metricparam (params self)))

(defmethod set-metric-values ((self OMmaquette) (list list))
   "Set a list (tempo metric minval loopflag)."
   (setf (metricparam (params self)) list))

(defmethod maq-changeport ((self OMmaquette) type val)
  (loop for item in (boxes self) do
        (maq-changeport item type val)))

(defmethod maq-changeport ((self temporalbox) type val)
  (rplacd (assoc type (edition-params self)) val) )

(defmethod maq-changeport ((self t) type val) t)

(defun change-maq-port (obj inport outport viewpos) 
  (let* ((changein nil)
         (changeout nil)
         (Mydialog (om-make-window 'om-dialog
                                  :size (om-make-point 100 70)
                                  :window-title "Change Midi Port"
                                  :close nil
                                  :font *om-default-font1*
                                  :position viewpos))
         (pos (om-make-point 5 8))
         (inport (om-make-dialog-item 'numbox (om-add-points pos (om-make-point 50 12)) (om-make-point 32 12)
                                                                (format nil "~D" (strig2numport inport)) 
                                                                :di-action (om-dialog-item-act item (setf changein t))
                                      :font *om-default-font1*
                                      :value (strig2numport inport)
                                      :min-val 0
                                      :max-val 255))
         (outport (om-make-dialog-item 'numbox (om-add-points pos (om-make-point 50 -1)) (om-make-point 32 12)
                                                                   (format nil "~D" (strig2numport outport) nil) 
                                                                   :di-action (om-dialog-item-act item (setf changeout t))
                                       
                                       :font *om-default-font1*
                                       :value (strig2numport outport nil)
                                       :min-val 0
                                       :max-val 255)))
    (om-add-subviews Mydialog inport outport
                     (om-make-dialog-item 'om-static-text (om-add-points pos (om-make-point 0 12)) (om-make-point 43 12) "InPort"
                                                               :font *om-default-font2*
                                                               :bg-color *om-window-def-color*)
                     
                     (om-make-dialog-item 'om-static-text pos (om-make-point 43 12) "OutPort" '
                                                               :font *om-default-font2*
                                                               :bg-color *om-window-def-color*)
                     
                     (om-make-dialog-item 'om-button (om-make-point 5 45) (om-make-point 45 15)
                                          "cancel"
                                          :di-action (om-dialog-item-act item
                                           (declare (ignore item)) (om-return-from-modal-dialog mydialog ())))
                     (om-make-dialog-item  'om-button (om-make-point 55 45) (om-make-point 45 15)
                                          "ok"
                                          :di-action (om-dialog-item-act item
                                           (declare (ignore item))
                                           (when changeout
                                             (maq-changeport obj 'outport (value outport)))
                                           (when changein (maq-changeport obj 'inport (value inport)))
                                           (om-return-from-modal-dialog mydialog ()))
                                          :default-button t))
    (om-modal-dialog mydialog)))



(defmethod om-get-menu-context ((self MaquettePanel))
  (let (items
        (pos (om-mouse-position self)))
    (setf items (list (om-new-leafmenu "Eval" #'(lambda () (eval-maquette self)))
                      (om-new-leafmenu "Play" #'(lambda () (play-from-palette self)))
                      (om-new-leafmenu "-" #'(lambda () nil))
                      (om-new-leafmenu (if (show-conect (params (object self))) "hide connections" "show connections") 
                                       #'(lambda () 
                                           (setf (show-conect (params (object self))) (not (show-conect (params (object self)))))
                                           (om-invalidate-view (editor self) t)))
                      (om-new-leafmenu (if (rulermetric (editor self)) "hide metric ruler" "show metric ruler") 
                                       #'(lambda () 
                                           (if (rulermetric (editor self))
                                               (delete-ruler-metric (editor self))
                                             (add-ruler-metric (editor self)))))
                      (om-new-leafmenu "-" #'(lambda () nil))
                      ))
    (setf items (list+ items (list 
                              (om-new-leafmenu "Add INPUT"
                                                      #'(lambda () (add-input self pos)))
                              (om-new-leafmenu "Add OUTPUT"
                                                      #'(lambda () (add-output self pos)))
                              (om-new-leafmenu "-" #'(lambda () nil)))))
    
    (if (thepict (pictu (object self)))
      (setf items (list+ items (list (om-new-leafmenu "kill picture"
                                                      #'(lambda () 
                                                          (setf (thepict (pictu (object self))) nil)
                                                          (setf (name (pictu (object  self))) nil)
                                                          (om-invalidate-view self t))))))
      (setf items (list+ items (list (om-new-leafmenu "load picture"
                                                      #'(lambda () 
                                                          (let ((pic (get-picture-file)))
                                                            (when pic
                                                              (setf (pictu (object self)) pic)
                                                              (om-invalidate-view self t)))
                                                          ))))))
    
    (when *MIDI*
      (setf items (list+ items (list 
                                (om-new-leafmenu "-" #'(lambda () nil))
                                (om-new-leafmenu "Midi Ports"
                                                      #'(lambda () (change-maq-port (object self) 0 0 
                                                                                    (om-subtract-points 
                                                                                     (om-add-points (om-mouse-position (om-view-window self))
                                                                                                 (om-view-position (om-view-window self)))
                                                                                     (om-make-point 80 80))
                                                                                    )))))))
    (unless (om-maquette-abs-p (object (editor self)))
      (setf items (list+ items (list 
                                (om-new-leafmenu "-" #'(lambda () nil))
                                (om-new-leafmenu "Last saved" #'(lambda () (window-last-saved (editor self)))))))
      )
    items))


(defmethod om-get-menu-context ((self tempobjframe))
  (let (items)
    (setf items (list (om-new-leafmenu "Open" #'(lambda () (OpenObjectEditor (object  self))))
                      (om-new-leafmenu "Get info" #'(lambda () (get-info-window (om-view-window self))))
                      (om-new-leafmenu "-" #'(lambda () nil))))
    (if (thepict (pictu (object  self)))
      (setf items (list+ items (list (om-new-leafmenu "kill picture"
                                                      #'(lambda () 
                                                          (setf (thepict (pictu (object  self))) nil)
                                                          (setf (name (pictu (object  self))) nil)
                                                          (om-invalidate-view self t))))))
      (setf items (list+ items (list (om-new-leafmenu "load picture"
                                                      #'(lambda () 
                                                          (let ((newpict (get-picture-file)))
                                                            (setf (thepict (pictu (object self))) (thepict newpict))
                                                            (setf (name (pictu (object self))) (name newpict))
                                                            (om-invalidate-view self t))))))))
    (when *midi*
      (setf items (list+ items (list (om-new-leafmenu "Midi Ports"
                                                      #'(lambda () (change-maq-port (object self) 
                                                                                    (cdr (assoc 'inport (edition-params (object self))))
                                                                                    (cdr (assoc 'outport (edition-params (object self))))
                                                                                    (om-subtract-points 
                                                                                     (om-add-points (om-mouse-position (om-front-window))
                                                                                                    (om-view-position (om-front-window)))
                                                                                     (om-make-point 80 80)))))))))
    
    (setf items (list+ items 
                       (list (om-new-leafmenu "-" #'(lambda () nil))
                             (if (mute (object self))
                               (om-new-leafmenu  "Mute Off" #'(lambda () 
                                                                (setf (mute (object self)) nil)
                                                                (om-invalidate-view self t)))
                               (om-new-leafmenu  "Mute" #'(lambda () 
                                                            (setf (mute (object self)) t)
                                                            (om-invalidate-view self t)))
                               )
                             (if (lock (object self))
                               (om-new-leafmenu  "Unlock" #'(lambda () 
                                                              (setf (lock (object self)) nil)
                                                              (om-invalidate-view self t)))
                               (om-new-leafmenu  "Lock" #'(lambda () 
                                                            (setf (lock (object self)) t)
                                                            (om-invalidate-view self t)))
                               ))))
    
    items))









