(in-package :om)
(export '(clip
	  beat->secs
	  interval
	  dB->lin
          lin->dB
          db-to-lin
	  load-files
	  nextl
	  printdate 
          stringdate
          sign 
          invert-sign) :om)


(defpackage "CHROMA" 
  (:use "COMMON-LISP" "OM" "OM-API")
  (:nicknames "CR"))

(in-package :cr)


(defun cons-cr-path (dir str)
  (make-pathname :directory (pathname-directory (cr::getenv dir)) :name str :type "lisp"))


; READ THE VARIABLES OF THE ENVIRONMENT
(defun getenv (var)
  (eval var))

(defvar *env-init* '())

(defmacro setenv (name n)
  `(progn (setf ,name ,n)
          (pushnew ,name *env-init*)
          ',name))


; MAIN ROOT DIRECTORIES:
; CROOT: ROOT DIRECTORY OF THE SYSTEM
; CRX : EXAMPLE FILES (USED BY TUTORIALS OR THE SYSTEM)
; CROUT: USER'S SPACE (WHERE OUTPUT FILES ARE TO BE WRITTEN)

; NB: ADDITIONAL PERSONAL SOFTWARE AND DATA BASES SHOULD BE PART OF A USER-DEFINED LIBRARY

;;;
;;; VARIABLES STATIQUES
;;;
(defun init-om-chroma ()
  
  ;(let* ((path-chroma  (make-pathname :device (pathname-device om::*om-root*)
  ;                                     :directory (append (pathname-directory om::*om-root*) 
  ;                                                        (list "code" "Projects" "6-AnalyseSynthese" "Chroma")))))
  ;   (setf (logical-pathname-translations "chroma")
       ;(list (list "**;*.*" path-chroma))
  ;     (list (list "**;*.*" "cl:code;projects;6-AnalyseSynthese;Chroma;**;"))
  ;     )
   ; )
  
   (setf  *env-init* nil)
  
   (setenv	CROOT (make-pathname :directory (pathname-directory (om::OMRoot "code;projects;6-omsounds;chroma;"))))
                                     ;(pathname-directory (truename "chroma:"))))
   
   ;-----------------------------------------------------------------------------
   ; OMCHROMA-SPECIFIC SYSTEM
   
   ; OMCHROMA ROOT DIRECTORY
   (setenv	CR	(make-pathname :directory (append
                                                   (pathname-directory CROOT)
                                                   (list "omchroma"))))
   ; CSOUND ROOT
   (setenv	CRCS	(make-pathname :directory (append
                                                   (pathname-directory CR)
                                                   (list "csound"))))

   ; CSOUND CLASSES
   (setenv	CScls	(make-pathname :directory (append
                                                   (pathname-directory CRCS)
                                                   (list "classes"))))

   (setenv	CSorc	(make-pathname :directory (pathname-directory CScls)))

   
   ;-----------------------------------------------------------------------------
   ; EXAMPLES
   
   (setenv	CRX (make-pathname :directory (pathname-directory (om::OMRoot "exampleFiles;chroma;"))))
                                     ;(pathname-directory (truename "chroma:"))))
   
   (setenv	CRXdata	(make-pathname :directory (append
                                                   (pathname-directory CRX)
                                                   (list "Data"))))
   
   (setenv	CRXsnd	(make-pathname :directory (append
                                                   (pathname-directory CRX)
                                                   (list "Snd"))))
   
   t)



(defun init-chroma-user ()
  
  (setenv       CRUSER      `om::*composer-name*)
  
  (setenv	CROUT       `om::*om-outfiles-folder*)
  
  (setenv	CSdata	`om::*om-infiles-folder*)

  ; CSOUND FUNCTIONS (FOR COMPATIBILITY'S REASONS)
  (setenv	CSfun	`om::*om-infiles-folder*)

   ;-----------------------------------------------------------------------------
   ; USER-WRITABLE DATA
   
   ; SCORES
   (setenv	CROUTsco	`(om::check-snd-folder (make-pathname :directory (append
                                                   (pathname-directory (getenv CROUT))
                                                   (list "Sco")))))
   ; INTERMEDIATE FILES
   (setenv	CROUTifls	`(om::check-snd-folder (make-pathname :directory (append
                                                   (pathname-directory (getenv CROUT))
                                                   (list "Inter-files")))))

   ; MIDI
   (setenv	CROUTmidi	`(om::check-snd-folder (make-pathname :directory (append
                                                   (pathname-directory (getenv CROUT))
                                                   (list "Midi")))))

   ; TABLES
   (setenv	CROUTtbls	`(om::check-snd-folder (make-pathname :directory (append
                                                   (pathname-directory (getenv CROUT))
                                                   (list "Tables")))))

   ; SOUNDS
   (setenv	CROUTsnd	`(om::check-snd-folder (make-pathname :directory (append
                                                   (pathname-directory (getenv CROUT))
                                                   (list "Snd")))))

    t)

(cr::init-om-chroma)
(om::om-add-init-func  'cr::init-om-chroma)

(om::add-init-user-func 'cr::init-chroma-user)


(defun not-used-anymore ()
  
  ;-----------------------------------------------------------------------------
  ;LISP-BASED KERNEL SYSTEM
  
  ; ROOT DIRECTORY FOR LISP-SYSTEM
  (setenv	LLsys	(make-pathname :directory (append
                                                   (pathname-directory CROOT)
                                                   (list "kernel"))))
  ; PLS TAGGED-ARCHITECTURE SYSTEM
  (setenv	LLpls	(make-pathname :directory (append
                                                   (pathname-directory LLsys)
                                                   (list "pls"))))
  
  ; SYMBOLIC DATA GENERATION AND PROCESSING
  (setenv	LLdg	(make-pathname :directory (append
                                                   (pathname-directory LLsys)
                                                   (list "dg"))))
  
  ; CTL1 LEVEL SYSTEM (BASIC CONTROL OF SYNTHESIS - REPLACED BY THE OMCHROMA'S ARRAY CLASS)
  
  ; CTL2 LEVEL SYSTEM (HIGH-LEVEL CONTROL OF SYNTHESIS)
  (setenv	LLctl2	(make-pathname :directory (append
                                                   (pathname-directory LLsys)
                                                   (list "ctl2"))))
  
  ; VPS-BASED SYSTEM (GENERATION AND ANALYSIS)
  (setenv	LLvps	(make-pathname :directory (append
                                                   (pathname-directory LLsys)
                                                   (list "vps"))))
  ; ANALYSIS SYNTHESIS MODELS
  (setenv	LLmdls	(make-pathname :directory (append
                                                   (pathname-directory LLsys)
                                                   (list "models"))))   

  ;-----------------------------------------------------------------------------
  ; OMCHROMA-SPECIFIC SYSTEM
  
  ; CSOUND SOURCES
  (setenv	CSsrc	CRCS)

  ; DIPHONE ROOT
  (setenv	CRdiph	(make-pathname :directory (append
                                                   (pathname-directory CR)
                                                   (list "DIPHONE"))))
  
  ; USER FUNCTIONS
  (setenv	CRusr	(make-pathname :directory (append
                                                   (pathname-directory CR)
                                                   (list "User-funs"))))
  
   ;-----------------------------------------------------------------------------
   ; EXTERNAL APPLICATIONS

   ; EXTERNALS
   (setenv	CRext	(make-pathname :directory (append
                                                   (pathname-directory CROOT)
                                                   (list "Externs"))))
  

  )