(in-package chroma)

(defclass chroma-model 
  ()
  ((sfname)
   (sfduration)
   (analysis-command)
   (total-duration :accessor total-duration
                   :reader get-total-dur
                   )
   (markers :type list
            :initform '(0.0)
            :accessor markers
            :initarg :markers
            :documentation "default marks")
   (structure :initform (make-hash-table)
              :documentation "list of marks objects")
   )
  (:documentation "Chroma Model"))


(defmethod nev ((x chroma-model))
  (length(fql-list x)))

(defmethod durtot ((x chroma-model))
  (-(nth  (nev x) (markers x))(first(markers x))))

(defmethod durlist ((x chroma-model))
  (markerstodur(markers x)))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defclass model-cseq 
  (chroma-model)
  ((max-amp :accessor max-amp)
   (min-amp :accessor min-amp)
   (max-bw :accessor max-bw)
   (min-bw :accessor min-bw)
   (fql-list :accessor fql-list
             :initform ()
             :initarg :fql-list 
             :documentation "list of FQL objects")
   )
  (:documentation "Chord Seq Model"))

(defmethod initialize-instance :after ((x model-cseq) &rest initargs)
  (declare (ignore initargs))
  (unless (equal (length (markers x)) (1+ (length (fql-list x))))
    (error "THERE SHOULD BE ONE MARKER MORE THAN FQLS~%      Markers = ~a,fqls = ~a~%"
           (length (markers x))(length (fql-list x)))))

(defmethod load-from-file ((x model-cseq) &optional file)
  (if(null file)(setf file (choose-file-dialog)))
  (format t "LOADING MODEL FROM ~a~%" file)
  (with-open-file (in-stream file :direction :input)
    (multiple-value-bind (m f) (chordseq-analysis-to-frq (read in-stream))
      (setf (markers x) m)
      (setf (fql-list x) f)
      (setf (total-duration x) (car(last m)))
      ))'loaded)

(defmethod load-from-formant-file ((x model-cseq) &optional file)
  (if(null file)(setf file (choose-file-dialog)))
  (format t "LOADING MODEL FROM ~a" file)
  (let ((truc nil) curr-list (markers-list ())
        (freqs ())(amps ())(bws ())(fql-list ()))
    (with-open-file (in-stream file :direction :input)
      (loop while (not truc) 
            do (multiple-value-bind(s tr)(read-line in-stream nil)
                 (setf truc tr)
                 (setf curr-list
                       (read-from-string (concatenate 'string "(" s ")")))
                 (case (length curr-list)
                   (1 (progn(push (car curr-list) markers-list)
                            (push (make-instance 'fql :the-list (nreverse freqs) 
                                                 :amplitudes (nreverse amps) 
                                                 :bwl (nreverse bws))
                                  fql-list))
                    (setf freqs ())
                    (setf amps ())
                    (setf bws ()))
                   (3 (progn (push (first curr-list) freqs)
                             (push (second curr-list) amps)
                             (push (third curr-list) bws)
                             )
                    )))))
    (setf(markers x) (reverse markers-list))
    (setf (fql-list x) (reverse (butlast fql-list)))
    (setf (total-duration x) (car(last markers-list)))
    )'loaded)

(defmethod get-max-amp ((x model-cseq))
  (let ((l (mapcar #'(lambda (y)(get-max-amp y))(fql-list x))))
    (max- l)))
(defmethod get-min-amp ((x model-cseq))
  (let ((l (mapcar #'(lambda (y)(get-min-amp y))(fql-list x))))
    (min- l)))
(defmethod get-max-bw ((x model-cseq))
  (let ((l (mapcar #'(lambda (y)(get-max-bw y))(fql-list x))))
    (max- l)))
(defmethod get-min-bw ((x model-cseq))
  (let ((l (mapcar #'(lambda (y)(get-min-bw y))(fql-list x))))
   (min- l)))
(defmethod get-max-fq ((x model-cseq) &key  &allow-other-keys)
  (let ((l (mapcar #'(lambda (y)(get-max-fq y))(fql-list x))))
    (max- l)))
(defmethod get-min-fq ((x model-cseq) &key  &allow-other-keys)
  (let ((l (mapcar #'(lambda (y)(get-min-fq y))(fql-list x))))
    (min- l)))

(defmethod get-norm-amp ((x model-cseq) rang &optional (scaling-factor 1))
"retourne les amplitudes de la vps de rang rang
 normalises par rapport au modle entier"
(mapcar #'(lambda (y) (/ y (/ (get-max-amp x)scaling-factor )))
                 (get-amp(nth rang (fql-list x)))))

;si time > dernier marker get-fql-from-time retourne nil
(defmethod get-fql-from-time ((x model-cseq) time)
  (let ((vps-number (length(member-if (lambda (y) (< time y)) (markers x)))))
    (car (last (fql-list x) vps-number))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;mars 2000 :
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmethod load-from-markers-file ((x model-cseq) &optional file)
  (if(null file)(setf file (choose-file-dialog)))
  (format t "LOADING MODEL FROM markers ~a" file)
    (with-open-file (in-stream file :direction :input)
      (setf (markers x) (cddr (read in-stream nil)))
            ))

(defmethod load-from-add-file ((x model-cseq) &optional file)
  (if(null file)(setf file (choose-file-dialog)))
  (format t "LOADING MODEL FROM ~a" file)
  (let (debut fin npartials freq_fun amp_fun (fql-list nil))
    (loop for (a b) on (markers x)  by #'cdr
          when (not(null b))
          do
          (format t "markers->~a~%" (list a b))
          ;pour chaque paire de marker
          (setf debut (marker-apres a file))
          (setf fin (marker-apres b file))
          (setf npartials (nombre_de_partiels  file debut fin))
          ;pour chaque partiel
          (loop for i from 1 to npartials
                 do
                 (format t "partiel n ~a~%" i)
                 ;moyenne pondre frequence
                (setf freq_fun (get_freq_fun file i debut fin ))
                ;amplitude moyenne
                (setf amp_fun (get_amp_fun file i debut fin))
                collect (moyenne freq_fun) into freqs
                ;amplitude moyenne
                collect (moyenne amp_fun) into amps
                finally    (push (make-instance 'fql :the-list freqs :amplitudes amps)fql-list)))
    fql-list
    ))
;test
;(setf my-model (make-instance 'model-cseq))
;(load-from-markers-file my-model #P"PRO-094-User:Serge:Chroma2000:nah-1-a.mark")
;(load-from-add-file my-model #P"PRO-094-User:Serge:Chroma2000:small.ADD")
;(load-from-add-file my-model #P"PRO-094-User:Serge:Chroma2000:25/nah-1.ADD")


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defclass model-f0 
  (chroma-model)
  ((f0 :accessor f0
       :documentation "f0 : type fun"))
  (:documentation "Fundamental Frequency Model"))

(defmethod load-from-file ((x model-f0) &optional file)
  (if(null file)(setf file (choose-file-dialog)))
  (let (curr-list (truc nil) (funl nil))
    (with-open-file (in-stream file :direction :input)
      (loop while (not truc) 
            do (multiple-value-bind(s tr)(read-line in-stream nil)
                 (setf truc tr)
                 (setf curr-list
                       (read-from-string (concatenate 'string "(" s ")")))
                 (case (length curr-list)
                   (1(progn(push (car curr-list) funl))))))
    (setf (f0 x) (make_fun(xy-echange  funl)))
    (setf (total-duration x) (car(last funl)))
    )'loaded))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defclass model-partials 
  (chroma-model)
  ((max-npartials :accessor max-npartials)
   (partials-list :accessor partials-list
             :initform ()
             :initarg :partials-list 
             :documentation "list of partials objects")
   )
  (:documentation "Partials Model"))



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;MODELES - MISC FUNCTIONS

(defun markerstodur (l)
   (cond((null (second l))())
       (t( cons (-(second l)(first l))(markerstodur(cdr l))))))


(defun chordseq-analysis-to-frq (list)
  (let( time time-end (freqs ())(amps ())(fql-list nil)(markers-list nil))
  ; (setf size (1-(cadr list)))
    (setf list (cddr list))
    (setf time (third(car list)))
    (loop for curr-list in list 
          do(let ((new-time (third curr-list)))
              (if(equal  new-time time)
                (progn(push (db-to-lin (fifth curr-list)) amps)
                      (push (fourth curr-list) freqs))
                (progn (push (make-instance 'fql :the-list (nreverse freqs) :amplitudes (nreverse amps))fql-list)
                       (push time markers-list)
                       (setf time-end (sixth curr-list))
                       (setf freqs nil)
                       (push (fourth curr-list) freqs)
                       (setf amps nil)
                       (push (db-to-lin (fifth curr-list)) amps)))
              (setf time new-time)))
    (push time markers-list)
    (push time-end markers-list)
    (push (make-instance 'fql :the-list (nreverse freqs) :amplitudes (nreverse amps))fql-list)
    (values (nreverse markers-list) (nreverse fql-list))))


(defun partials-analysis-to-model (list)
  (let( time time-end (freqs ())(amps ())(fql-list nil)(markers-list nil))
  ; (setf size (1-(cadr list)))
    (setf list (cddr list))
    (setf time (third(car list)))
    (loop for curr-list in list 
          do(let ((new-time (third curr-list)))
              (if(equal  new-time time)
                (progn(push (db-to-lin (fifth curr-list)) amps)
                      (push (fourth curr-list) freqs))
                (progn (push (make-instance 'fql :the-list (nreverse freqs) :amplitudes (nreverse amps))fql-list)
                       (push time markers-list)
                       (setf time-end (sixth curr-list))
                       (setf freqs nil)
                       (push (fourth curr-list) freqs)
                       (setf amps nil)
                       (push (db-to-lin (fifth curr-list)) amps)))
              (setf time new-time)))
    (push time markers-list)
    (push time-end markers-list)
    (push (make-instance 'fql :the-list (nreverse freqs) :amplitudes (nreverse amps))fql-list)
    (values (nreverse markers-list) (nreverse fql-list))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

#|
(defun xy-echange0 (l)                  ; RECURSIVE VERSION 
  (cond ((null l)())
        (t (append(xy-echange (cddr l))
                  (list(car l) (second l))))))

|#

(defun xy-echange (l)
  (let (result(flag 0))
    (dolist (x (mapcon #'(lambda (x)(list(second x)(car x)))l) result) 
      (incf flag)
      (case flag
        ((1 2)(push  x result))
        (4 (setf flag 0))))))

  

(defun marker-apres (a file)
 (let ((truc nil) curr-list )
   (with-open-file (in-stream file :direction :input)
      (loop while (not truc) 
            do (multiple-value-bind(s tr)(read-line in-stream nil)
                 (setf truc tr)
                 (setf curr-list
                       (read-from-string (concatenate 'string "(" s ")")))
                 (case (length curr-list)
                   (2 (if (> (cadr curr-list) a) (return (cadr curr-list))))
 ))))))

(defun nombre_de_partiels (file debut fin)
  (let ((truc nil) curr-list (max 0)(min 100)(curr_time 0))
    (with-open-file (in-stream file :direction :input)
      (loop while (and (not truc) (< curr_time fin))
            do (multiple-value-bind(s tr)(read-line in-stream nil)
                 (setf truc tr)
                 (setf curr-list
                       (read-from-string (concatenate 'string "(" s ")")))
                 (if (eq (length curr-list) 2)
                   (if (> (cadr curr-list) debut)
                     (progn (setf curr_time (cadr curr-list))
                            (if (> (car curr-list) max) (setf max (car curr-list)))
                            (if (< (car curr-list) min) (setf min (car curr-list))))))))) 
    min))
 
(defun get_add_fun (file numero colonne debut fin)
  (let ((truc nil)(freq_fun nil) curr-list (curr_time 0))
    (with-open-file (in-stream file :direction :input)
      (loop while (and (not truc) (< curr_time fin))
            do (multiple-value-bind(s tr)(read-line in-stream nil)
                 (setf truc tr)
                 (setf curr-list
                       (read-from-string (concatenate 'string "(" s ")")))
                 (if (eq (length curr-list) 2)
                   (setf curr_time (cadr curr-list)))
                 (if (>= curr_time debut)
                   (if (eq (length curr-list) 4)
                     (if (eq numero (car curr-list))
                       (push  (list curr_time (nth colonne curr-list))freq_fun)))))))
    (nreverse freq_fun)))

(defun get_freq_fun (file numero debut fin)
  (get_add_fun file numero 1 debut fin))
(defun get_amp_fun (file numero debut fin)
  (get_add_fun file numero 2 debut fin))

(defun moyenne (fun)
 (/(apply #'+ (mapcar #'cadr fun))(length fun)))

;(marker-apres 25 #P"PRO-094-User:Serge:Chroma2000:small.ADD");
;(nombre_de_partiels  #P"PRO-094-User:Serge:Chroma2000:small.ADD" 5 10)
;(get_freq_fun  #P"PRO-094-User:Serge:Chroma2000:small.ADD" 1 5 20)
;(get_amp_fun  #P"PRO-094-User:Serge:Chroma2000:small.ADD" 1 5 20)
;(moyenne (get_freq_fun  #P"PRO-094-User:Serge:Chroma2000:small.ADD" 1 5 20))

