(in-package :om)

(defvar *all-application* '("Csound" "Modalys" "Chant" "Diph" "Fo" "mklisp" "modres" 
                            "pm" "reduce" "sdif" "sdiftotext" "snd" "supervp" "tosdif"))

;============================================================
;Non Graphic CLASSES
;============================================================
(defclass internSynthEvt ()
   ((parsing-fun :initform nil  :accessor parsing-fun)
    (action-time :initform 0  :accessor action-time)))

;============================================================
;============================================================
(defclass! SynthesisEvt (class-Array simple-container internSynthEvt) () (:icon 603))

(defmethod collect-synthesis-events ((self t)) nil)
(defmethod collect-synthesis-events ((self list)) 
   (remove-if 'null (loop for object in self collect (collect-synthesis-events object))))

(defmethod collect-synthesis-events ((self container)) 
   (remove-if 'null (loop for object in (inside self) collect (collect-synthesis-events object))))
                          
(defmethod collect-synthesis-events ((self SynthesisEvt)) self)

(defmethod collect-synthesis-events ((self string)) self)


(defmethod draw-editor-mode ((self SynthesisEvt) view) 
   (draw-mini-view view self)
   (draw-carre view))

(defmethod array-special-keyword ((self SynthesisEvt) (key (eql :parsing-fun))) t)

(defmethod array-special-keyword-action ((self SynthesisEvt) (key (eql :parsing-fun)) val)
   (setf (parsing-fun self) val))

;============================================================
(defmethod make-instrument-parameter-list (num)
   (loop for i from 1 to num collect
         (make-instance 'parameter 
           :name (format nil "Input~D" i)
           :obj (make-instance 'simple-num))))


(defmethod omNG-copy ((self SynthesisEvt))
   `(let ((copy ,(call-next-method)))
        (setf (action-time copy) ,(action-time self))
        (setf (parsing-fun copy) ,(parsing-fun self))
        copy))

(defun get-all-slots-instance-of-class (Class)
    (let* ((Class-List (omcpl (find-class class)))
               Theslotti )
       (mapcar #'(lambda (Superclass)
                              (setf theslotti (remove-duplicates (concatenate 'list theslotti (class-direct-instance-slots superclass))
                                                                                             :test #'(lambda (x y) (equal (slot-definition-name x) (slot-definition-name y)))))) 
                        (reverse class-list))
       (make-slots class nil theslotti)))

(defun get-init-instance-slots-of-class (Class)
    (mapcar #'(lambda (Slot)
                           (list (internp (name slot) (symbol-package class)) (thetype slot))) (get-all-slots-instance-of-class class)))

(defmethod omNG-save ((self SynthesisEvt) &optional (values? nil)) 
    (let* ((theclass (class-name (class-of self)))
              (i -1)
              (slots (mapcan #'(lambda (slot)
                                              (list `(setf (slot-value copy ',(first slot)) 
                                                               ,(save-slot-array self (slot-value self (first slot)) (incf i))))) 
                                         (get-init-instance-slots-of-class theclass))))
      `(if (find-class ',theclass nil)
         (let ((copy (make-instance ',theclass
                                 :numrows ,(numrows self)
                                 :numcols ,(numcols self))))
           ,.slots
           (setf (Lcontrols   copy)
                    (list ,.(loop for con in (Lcontrols self)
                                       for j = (numrows self) then (+ j 1)
                                       collect (save-control self con j))))
           (setf (action-time copy) ,(action-time self))
           (setf (Lprecision copy) ,(omng-save (Lprecision self)))
           copy))))






;------------------------------
;by serge 07082001

(defun mk-array (class numcols fixedargs  &rest argkeys) 
  (let* ((rep (make-instance class :numcols numcols))
         (initargs (get-all-initargs-of-class class))
         (precision 3) slot?)
    (loop for slot in (cdr (fixed-slots-list rep))
          for i = 0 then (+ i 1) do  
          (setf (slot-value rep slot) (nth i fixedargs)))
    (loop while argkeys do
          (cond
           ((setf slot? (car (member (string (car argkeys)) initargs :test 'string-equal :key 'name)))
            (pop argkeys)
            (setf (slot-value rep (internp (name slot?) (slot-package slot?))) (verify-list-exacte (pop argkeys) (thetype slot?) (numcols rep) rep)))
           ((string-equal (string (car argkeys)) "precision")
            (pop argkeys)
            (setf precision (pop argkeys)))
           ((array-special-keyword rep (car argkeys))
            (array-special-keyword-action rep (pop argkeys) (pop argkeys)))
           (t
            (pushr (list (pop argkeys) (verify-list-exacte (pop argkeys) 'number (numcols rep) rep)) (Lcontrols rep)))))
    (put-precision rep precision)
    rep))



(defun mk-modalys-env (class timepoints  &rest argkeys )
  (let* ((rep (make-instance class :numcols (length timepoints)))
            (precision-list 6))
       (setf (times rep) timepoints)
       (loop while argkeys do
             (cond
              ((string-equal (string (car argkeys)) "precision")
               (pop argkeys)
               (setf precision-list (pop argkeys)))
              ((array-special-keyword rep (car argkeys))
               (array-special-keyword-action rep (pop argkeys) (pop argkeys)))
              (t
               (pushr (list (pop argkeys) (verify-list-exacte (pop argkeys) 'number (numcols rep) rep)) (Lcontrols rep)))))
       (put-precision rep precision-list)
       rep))

;(setf menv (mk-modalys-env 'modalys-env '(0 7.5 9.2 11.3) :dim1 '(12 33 45 0) :dim2 #'(lambda (x) (/ (sin x) x)) ))
;;


