;=============================================================================
;			SNARE-1.ORC
; SIMPLE SIMULATION OF A SNARE DRUM (FROM ACCCI, 03_01_1.ORC) / MONO
;=============================================================================

; timbre:    drum and snare drum
; synthesis: additive different units(03)
;            units: noise/inharm/fundamental (01)
; source:    #400  Drum and Snare-drum like Sounds, Risset(1969)
; coded:     jpg 8/93, modified ms 9/02

; NB1: this instrument works better with short durations
; NB2: the apparently arbitrary amplitude range (0-1000, rather than 0-1)
;         avoids Lisp printing small values with exponential
;-----------------------------------------------------------------------------

;	p1	= instrument number
;	p2	= action time [sec]
;	p3	= duration [sec]
;	p4	= amplitude of sine tone [linear, 0.0 -> 1000.0]
;	p5	= fundamental frequency [Hz]
;	p6	= amplitude of pseudo inharmonic tone [linear, 0.0 -> 1000.0]
;	p7	= noise amplitude [[linear, 0.0 -> 1000.0]
;	p8	= center frequency for the noise band [Hz]
;	p9	= noise 1/2 bandwidth [Hz]

;-----------------------------------------------------------------------------
; COMPULSORY GEN FUNCTIONS :
;	f11	noise-modulated sine wave
;	f12	sine wave with only one high partial (10th)
;	f13	pseudo-inharmonic spectrum made of high partials

;	f21	slowly descending exponential envelope
;	f22	rapidly descending exponential envelope
;_____________________________________________________________________________

; GEN functions **********************************************************
; waveforms
f11  0  4097  9   1  1  0
f12  0  8193  9  10  1  0
f13  0  8193  9  10  1  0  16  1  0  22  1  0  23  1  0

; envelopes
f21  0  513  5   256  513  1
f22  0  513  5  4096  513  1				; steeper slope

; score ******************************************************************

; section 1                                   
;   a-time	dur	isin_amp if0	inh_amp	iran_amp iran_cfq iran_bw

i1    .4    .2	200		20 		75		250		4000	1500
i1    .8    .2
i1   1.1    .15
i1   1.2    .2
i1   1.6    .2
i1   1.9    .15
i1   2.0    .2
i1   2.4    .2
i1   2.8    .2
i1   3.1    .15
i1   3.2    .2
i1   3.6    .2
i1   3.9    .15
i1   4      .2
i1   4.4
i1   4.8
i1   5.2
i1   5.6
i1   6      .      1300


s
; section 2               the noise unit is inactive (iran_amp = 0)
;

;   a-time	dur	isin_amp if0	inh_amp	iran_amp iran_cfq iran_bw
i1    .4    .3	375		12		125		0       0   	2.5
i1    .8    .2       .    16
i1   1.07   .2       .    12
i1   1.2    .2       .    16
i1   1.6    .3       .    12
i1   2.0    .25      .    14
i1   2.4    .23      .    15
i1   2.6    .27
i1   3.07   .23
i1   3.2
i1   3.6
i1   4


s
; section 3
;   a-time	dur	isin_amp if0	inh_amp	iran_amp iran_cfq iran_bw
i1    .4    .15		200		20	175		250		4000	1500
i1    .6    .20
i1   1.07
i1   1.2
i1   1.6
i1   2.0
i1   2.4    .25
i1   2.9    .15
i1   3
i1   3.1
i1   3.2    .20
i1   3.55   .15   700
i1   3.6    .20
i1   4      .15   800
i1   4.06
i1   4.13
i1   4.20
i1   4.27
i1   4.33
i1   4.40   .22  1200

e
