;******************************************************************
;		     CLASS ADD-1
;******************************************************************

(in-package om)

; LISP-DEFINED CLASSES SHOULD RESIDE IN THE LIBRARY'S PACKAGE AND 
;   NOT IN THE USER PACKAGE, WHICH CONTAINS ALL THE CLASSES
;   GRAPHICALLY DEFINED


(defclass! add-1
  (cs-evt) 			; INHERIT FROM CS-EVT
  (

; GLOBAL SLOTS (LIGHT BLUE, ON THE LEFT OF THE CLASS):
;    THE METHOD BELOW TRANSFORMS THEM INTO GLOBAL SLOTS ("SHOW" UNCHEKED)
;    ATTENTION: A GLOBAL SLOT SHOULD NOT HAVE AN INITARG

   ( source-code :initform
                 (load-buffer-textfile
                  (get-orc-source (get-orc "add-1"))
                  'textfile "append")
                 :allocation :class
                 :type textfile
                 :accessor source-code)
   ( numchan :initform (or (get-orc-channels (get-orc "add-1")) 1)
             :allocation :class  :accessor numchan)
   ( globals-list :initform 
                  (get-orc-globals
                   (get-orc "add-1"))
                  :allocation :class
                  :type list
                  :accessor globals-list)
   (InstID :initform 1  :allocation :class  :accessor InstID)

; LOCAL SLOTS (RED, CORRESPONDING TO THE P-FIELDS)
;    ATTENTION: A GLOBAL SLOT SHOULD HAVE AN INITARG
  ( amp		:type number
		:initarg :amp 
  		:initform 500.0
		:accessor amp)
   ( freq	:type number
        	:initarg :freq 
        	:initform 440.0
        	:accessor freq)
   ( aenv	:type cs-table
		:initarg :aenv 
                                               ; x-points y-points decimals
  		:initform (make-cs-table  'Gen07  '(0 256 513) '(0 100 0) 1 "?" 513)
		:accessor aenv)
   )

  (:documentation
   "
;=============================================================================
;			ADD-1.ORC
; SIMPLE ADDITIVE SYNTHESIS (FROM ACCS, 02_01_3.ORC) / MONO
; AMPLITUDE ENVELOPE WITH OSCILI
;=============================================================================

; Timbre:    gong
; Synthesis: additive same units (02)
;            basic instrument( 01)
; Source:    #420, Gong-like Sounds, Risset(1969)
; Coded:     jpg 8/93, modified ms 9/02

;-----------------------------------------------------------------------------

;       p1      = instrument number
;       p2      = action time [sec]
;       p3      = duration [sec]
;       p4      = maximum amplitude [linear, 0.0 -> 1000.0]
;       p5      = frequency [Hz]
;       p6      = amplitude envelope [GEN number]

;-----------------------------------------------------------------------------
; COMPULSORY GEN FUNCTIONS :
;       f1      audio wave
;_____________________________________________________________________________

; CLASS: ADD-1

;  GLOBAL KEYWORDS (default values within parentheses):
;       NUMROWS	    : amount of rows (components) in the event (1)
;       ACTION-TIME : start time of the whole event [sec] (0.0)
;       USER-FUN    : user-defined parsing function (nil)

;  LOCAL KEYWORDS:
;       E-DELS  : entry delays [sec] (0.0)
;       DURS    : duration [sec] (1.0)
;       AMP     : amplitude [sec] (500.0)
;       FREQ    : frequency [Hz] (440.0)
;       AENV    : function number for the amplitude envelope [GEN] (triangle)

;*****************************************************************************

"
   )
  (:icon 618)
  )

;------------------------------------------------------------------
;	METHODS FOR THIS CLASS
;------------------------------------------------------------------
; INCLUDE THE COMPULSORY FUNCTIONS AT THE BEGINNING OF A SCORE FILE
;     THIS METHOD MUST BE DEFINED FOR THIS CLASS

(defmethod! orc-header ((self add-1))
  (list

   "; GEN functions **********************************************************"
   "; audio wave"
   "f1  0   4097   9  1  1  0"

   ))

;******************************************************************
