;******************************************************************
;		     CLASS ADD-2
;******************************************************************

(in-package om)

; LISP-DEFINED CLASSES SHOULD RESIDE IN THE LIBRARY'S PACKAGE AND 
;   NOT IN THE USER PACKAGE, WHICH CONTAINS ALL THE CLASSES
;   GRAPHICALLY DEFINED


(defclass! add-2
  (cs-evt) 			; INHERIT FROM CS-EVT
  (

; GLOBAL SLOTS (LIGHT BLUE, ON THE LEFT OF THE CLASS):
;    THE METHOD BELOW TRANSFORMS THEM INTO GLOBAL SLOTS ("SHOW" UNCHEKED)
;    ATTENTION: A GLOBAL SLOT SHOULD NOT HAVE AN INITARG

   ( source-code :initform
                 (load-buffer-textfile
                  (get-orc-source (get-orc "add-2"))
                  'textfile "append")
                 :allocation :class
                 :type textfile
                 :accessor source-code)
   ( numchan :initform (or (get-orc-channels (get-orc "add-2")) 1)
             :allocation :class  :accessor numchan)
   ( globals-list :initform 
                  (get-orc-globals
                   (get-orc "add-2"))
                  :allocation :class
                  :type list
                  :accessor globals-list)

   (InstID :initform 1  :allocation :class  :accessor InstID)

; LOCAL SLOTS (RED, CORRESPONDING TO THE P-FIELDS)
;    ATTENTION: A GLOBAL SLOT SHOULD HAVE AN INITARG
  ( amp		:type number
		:initarg :amp 
  		:initform 500.0
		:accessor amp)
   ( freq	:type number
        	:initarg :freq 
        	:initform 440.0
        	:accessor freq)
   ( atk	:type number
        	:initarg :atk 
        	:initform 0.003
        	:accessor atk)
   ( dec	:type number
        	:initarg :dec 
        	:initform 0.01
        	:accessor dec)
   ( scal	:type number
        	:initarg :scal 
        	:initform 1.0
        	:accessor scal)
   ( jtf	:type number
        	:initarg :jtf 
        	:initform 0.02
        	:accessor jtf)
   )

  (:documentation
   "
;=============================================================================
;			ADD2.ORC
; SIMPLE ADDITIVE SYNTHESIS (FROM ACCSI, 02_41_1.ORC) / MONO
; AMPLITUDE ENVELOPE WITH LINEN
;=============================================================================

; Timbre:    Brass
; Synthesis: Additive, same building blocks units
;            Basic instrument with added random frequency variation
; Source:    #200, Brass-like Sounds through Independent Control of
;               Harmonics, Risset (1969)
; Coded:     jpg 8/93, modified ms 9/02

; NB 1: this instrument works better with short notes.
; NB 2: the apparently arbitrary amplitude range (0-1000, rather than 0-1)
;         avoids Lisp printing small values with exponential
;-----------------------------------------------------------------------------

;	p1	= instrument number
;	p2	= action time [sec]
;	p3	= duration [sec]
;	p4	= maximum amplitude [linear, 0.0 -> 1000.0]
;	p5	= fundamental frequency [Hz]
;	p6	= attack time of the amplitude envelope [sec]
;	p7	= decay time of the amplitude envelope [sec]
;	p8	= spectral scaler for the audio frequency
;	p9	= % of jitter (low-frequency random frequency variation) [0-1]

;-----------------------------------------------------------------------------
; COMPULSORY GEN FUNCTIONS :
;	f1	audio wave
;_____________________________________________________________________________

; CLASS: ADD2

;  GLOBAL KEYWORDS (default values within parentheses):
;	NUMROWS		: amount of rows (components) in the event (1)
;	ACTION-TIME	: start time of the whole event [sec] (0.0)
;	USER-FUN	: user-defined parsing function (nil)

;  LOCAL KEYWORDS:
;   E-DELS	:	entry delays [sec] (0.0)
;	DURS	:	duration [sec] (1.0)
;	AMP	:	amplitude [sec] (500.0)
;	FREQ	:	frequency [Hz] (440.0)
;	ATK	:	attack time [sec] (0.003)
;	DEC	:	decay time [sec] (0.01)
;	SCAL	:	spectral scaler for the audio frequency (1.0)
;	JTF	:	frequency jitter [0-1, % of fundamental frequency] (0.02)


;*****************************************************************************

"
   )
  (:icon 618)
  )

;------------------------------------------------------------------
;	METHODS FOR THIS CLASS
;------------------------------------------------------------------
; INCLUDE THE COMPULSORY FUNCTIONS AT THE BEGINNING OF A SCORE FILE
;     THIS METHOD MUST BE DEFINED FOR THIS CLASS

(defmethod! orc-header ((self add-2))
  (list

   "; GEN functions **********************************************************"
   ""
   "; waveform"
   "f1 0 4097 9 1 1 0                ; sinus with amplitude 1, start phase 0"

   ))

;******************************************************************
