;=============================================================================
;			BUZZ-1.ORC
; DYNAMIC SPECTRUM OSCILLATOR (FROM ACCCI, 43_21_1.ORC) / MONO
; AMPLITUDE ENVELOPE WITH OSCILI
;=============================================================================

; Timbre:    Various controlled noise spectra
; Synthesis: (g)buzz
;            OSCILI envelopeS
; Coded:     jpg 8/92, modified ms 9/02

; NB: the apparently arbitrary amplitude range (0-1000, rather than 0-1)
;         avoids Lisp printing small values with exponential notation
;-----------------------------------------------------------------------------

;	p1	= instrument number
;	p2	= action time [sec]
;	p3	= duration [sec]
;	p4	= maximum amplitude [linear, 0.0 -> 1000.0]
;	p5	= frequency [Hz]
;	p6	= amplitude envelope [GEN number]
;	p7	= lowest harmonic present in the buzz
;	p8	= % of maximum possible harmonic present [0-1]
;	p9	= envelope for the multiplier in the series of amplitude coefficients
;			of the buzz

;-----------------------------------------------------------------------------
; COMPULSORY GEN FUNCTIONS
;	f5	large cosine
;_____________________________________________________________________________

; CLASS: BUZZ-1

;  GLOBAL KEYWORDS (default values within parentheses):
;	NUMROWS		: amount of rows (components) in the event (1)
;	ACTION-TIME	: start time of the whole event [sec] (0.0)
;	USER-FUN	: user-defined parsing function (nil)

;  LOCAL KEYWORDS:
;   E-DELS	:	entry delays [sec] (0.0)
;	DURS	:	duration [sec] (1.0)
;	AMP	:	amplitude [sec] (500.0)
;	FREQ	:	frequency [Hz] (220.0)
;	AENV	:	function number for the amplitude envelope [GEN] (triangle)
;	BZLH	:	lowest harmonic present in the buzz [integer] (1)
;	BZHH	:	% of maximum possible harmonic present [0-1] (1)
;	BZENV	:	function number for the buzz envelope [GEN] (triangle)

;*****************************************************************************



sr	= 44100
kr	= 44100
ksmps	= 1
nchnls	= 1

gimaxamp init 32767 			; 16 bits


instr 1; *****************************************************************
idur	= p3
idurosc	= 1/p3
iamp	= p4 * gimaxamp * 0.001	; amplitude range 0-1000
ifq		= p5
iaenv	= p6
inn		= sr/2/ifq/4		; total possible number of harmonics present
inn		= int (inn * p8)	; % of possible total
ilh		= p7			; lowest harmonic present
ifn		= 5			; stored cosine function
ibzenv	= p9				; envelope for the buzz

   kenv		oscili	iamp, idurosc, iaenv		; amp envelope
   kratio	oscili	1, idurosc, ibzenv		; kratio envelope

   asrc    gbuzz   kenv,ifq,inn,ilh,kratio,ifn
           out     asrc

endin
