;******************************************************************
;		     CLASS FM-1
;******************************************************************

(in-package om)

; LISP-DEFINED CLASSES SHOULD RESIDE IN THE LIBRARY'S PACKAGE AND 
;   NOT IN THE USER PACKAGE, WHICH CONTAINS ALL THE CLASSES
;   GRAPHICALLY DEFINED


(defclass! fm-1
  (cs-evt) 			; INHERIT FROM CS-EVT
  (

; GLOBAL SLOTS (LIGHT BLUE, ON THE LEFT OF THE CLASS):
;    THE METHOD BELOW TRANSFORMS THEM INTO GLOBAL SLOTS ("SHOW" UNCHEKED)
;    ATTENTION: A GLOBAL SLOT SHOULD NOT HAVE AN INITARG

   ( source-code :initform
                 (load-buffer-textfile
                  (get-orc-source (get-orc "fm-1"))
                  'textfile "append")
                 :allocation :class
                 :type textfile
                 :accessor source-code)
   ( numchan :initform (or (get-orc-channels (get-orc "fm-1")) 1)
             :allocation :class  :accessor numchan)
   ( globals-list :initform 
                  (get-orc-globals
                   (get-orc "fm-1"))
                  :allocation :class
                  :type list
                  :accessor globals-list)
   (InstID :initform 1  :allocation :class  :accessor InstID)

; LOCAL SLOTS (RED, CORRESPONDING TO THE P-FIELDS)
;    ATTENTION: A GLOBAL SLOT SHOULD HAVE AN INITARG
  ( amp		:type number
		:initarg :amp 
  		:initform 500.0
		:accessor amp)
   ( f0		:type number
        	:initarg :f0 
        	:initform 55.0
        	:accessor f0)
   ( n1		:type number
        	:initarg :n1 
        	:initform 5
        	:accessor n1)
   ( n2		:type number
        	:initarg :n2 
        	:initform 7
        	:accessor n2)
   ( imax	:type number
        	:initarg :imax 
        	:initform 10
        	:accessor imax)
   ( imin	:type number
        	:initarg :imin 
        	:initform 0
        	:accessor imin)
   ( aenv	:type cs-table
		:initarg :aenv 
  		:initform (make-cs-table 'Gen05  '(0 513) '(1 0.0001) 4 "?" 513)
		:accessor aenv)
   ( ienv	:type cs-table
		:initarg :ienv 
  		:initform (make-cs-table 'Gen05  '(0 513) '(1 0.001) 3 "?" 513)
		:accessor ienv)

   )

  (:documentation
   "
;=============================================================================
;			FM-1.ORC
; FREQUENCY MODULATION (FROM ACCSI, 20_10_1.ORC) / MONO
; AMPLITUDE AND INDEX ENVELOPES WITH OSCILI
;=============================================================================

; Synthesis: FM with dynamic spectral evolution
;            Bell settings
; Source:    Chowning (1973)
; Coded:     jpg 8/92, modified ms 9/02

; NB: the apparently arbitrary amplitude range (0-1000, rather than 0-1)
;         avoids Lisp printing small values with exponential notation
;-----------------------------------------------------------------------------

;	p1	= instrument number
;	p2	= action time [sec]
;	p3	= duration [sec]
;	p4	= maximum amplitude [linear, 0.0 -> 1000.0]
;	p5	= fundamental frequency [Hz]
;	p6	= N1
;	p7	= N2
;	p8	= maximum index
;	p9	= minimum index
;	p10	= amplitude envelope [GEN number]
;	p11	= index envelope [GEN number]

;-----------------------------------------------------------------------------
; COMPULSORY GEN FUNCTIONS
;	f1	carrier audio wave (sine tone)
;	f2	modulating audio wave (sine tone)
;_____________________________________________________________________________

; CLASS: FM-1
;   Default settings for a bell-like sound

;  GLOBAL KEYWORDS (default values within parentheses):
;	NUMROWS		: amount of rows (components) in the event (1)
;	ACTION-TIME	: start time of the whole event [sec] (0.0)
;	USER-FUN	: user-defined parsing function (nil)

;  LOCAL KEYWORDS:
;   E-DELS	:	entry delays [sec] (0.0)
;	DURS	:	duration [sec] (10.0)
;	AMP	:	amplitude [sec] (500.0)
;	F0	:	fundamental frequency [Hz] (55.0)
;	N1	:	N1 (5)
;	N2	:	N2 (7)
;	IMAX	:	maximum index (10)
;	IMIN	:	minimum index (0)
;	AENV	:	function number for the amplitude envelope [GEN] (exponential)
;	IENV	:	function number for the index envelope [GEN] (exponential)

;*****************************************************************************

"
   )
  (:icon 618)
  )

;------------------------------------------------------------------
;	METHODS FOR THIS CLASS
;------------------------------------------------------------------
; INCLUDE THE COMPULSORY FUNCTIONS AT THE BEGINNING OF A SCORE FILE
;     THIS METHOD MUST BE DEFINED FOR THIS CLASS

(defmethod! orc-header ((self fm-1))
  (list

   "; GEN functions **********************************************************"
   "; carrier"
   "f1  0  4097  10  1"
   ""
   "; modulating"
   "f2  0  4097  10  1"

   ))

;******************************************************************
