;=============================================================================
;		FM-1.ORC
; FREQUENCY MODULATION (FROM ACCCI, 20_10_1.ORC) / MONO
; AMPLITUDE AND INDEX ENVELOPES WITH OSCILI
;=============================================================================

; Synthesis: FM with dynamic spectral evolution
;            Bell settings
; Source:    Chowning (1973)
; Coded:     jpg 8/92, modified ms 9/02

; NB: the apparently arbitrary amplitude range (0-1000, rather than 0-1)
;         avoids Lisp printing small values with exponential notation
;-----------------------------------------------------------------------------

;	p1	= instrument number
;	p2	= action time [sec]
;	p3	= duration [sec]
;	p4	= maximum amplitude [linear, 0.0 -> 1000.0]
;	p5	= fundamental frequency [Hz]
;	p6	= N1
;	p7	= N2
;	p8	= maximum index
;	p9	= minimum index
;	p10	= amplitude envelope [GEN number]
;	p11	= index envelope [GEN number]

;-----------------------------------------------------------------------------
; COMPULSORY GEN FUNCTIONS
;	f1	carrier audio wave (sine tone)
;	f2	modulating audio wave (sine tone)
;_____________________________________________________________________________

; CLASS: FM-1
;   Default settings for a bell-like sound

;  GLOBAL KEYWORDS (default values within parentheses):
;	NUMROWS		: amount of rows (components) in the event (1)
;	ACTION-TIME	: start time of the whole event [sec] (0.0)
;	USER-FUN	: user-defined parsing function (nil)

;  LOCAL KEYWORDS:
;   E-DELS	:	entry delays [sec] (0.0)
;	DURS	:	duration [sec] (10.0)
;	AMP	:	amplitude [sec] (500.0)
;	F0	:	fundamental frequency [Hz] (55.0)
;	N1	:	N1 (5)
;	N2	:	N2 (7)
;	IMAX	:	maximum index (10)
;	IMIN	:	minimum index (0)
;	AENV	:	function number for the amplitude envelope [GEN] (exponential)
;	IENV	:	function number for the index envelope [GEN] (exponential)

;*****************************************************************************

sr	= 44100
kr	= 44100
ksmps	= 1
nchnls	= 1

gimaxamp init 32767 				; 16 bits


instr 1; *****************************************************************
idur		= p3
idurosc		= 1/idur
iamp		= p4 * gimaxamp * 0.001	; amplitude range 0-1000
if0			= p5
in1			= p6
in2			= p7
icar		= if0*in1		; carrier
imod		= if0*in2		; modulating

imin		= p8
imax		= p9
idelta		= p9-p8

iaenv		= p10
ienv		= p11
icarfun		= 1
imodfun		= 2

ieps		= 0.1
						; short fadeout (avoid clicks in exp envelopes)


   ken	oscili  iamp, idurosc, iaenv            ; amplitude envelope
   k1	linseg 1, idur-ieps, 1,  ieps, 0	; avoid clicks
   kenv	=	ken * k1
   
   ki	oscili  imod*idelta, idurosc, ienv	; dynamic modulator
   kin	=	ki + imin
   kind	=	kin * k1
   
   amod  oscili  kind, imod, imodfun

   a1    oscili  kenv, icar+amod, icarfun	; carrier
         out     a1

endin

