;=============================================================================
;		FOF-2.ORC
; FORMANTIC WAVEFORM (FROM ACCCI, 45_01_2.ORC) / MONO
; AMPLITUDE ENVELOPE WITH OSCILI
; CONTROL OF THE DETAILS OF THE FOF FROM THE SCORE
;=============================================================================

; Timbre:    Granular synthesis with fof module, voice-like tones
; Synthesis: FOF (Forme d'Onde Formatique)
;            OSCILI envelopes
; Coded:     jpg 8/92, modified ms 9/04

; NB1: this implementation works better with audio and sub-audio f0's
;         and allows for an independent control of tex, debatt and atten
; NB2: the apparently arbitrary amplitude range (0-1000, rather than 0-1)
;         avoids Lisp printing small values with exponential notation
;-----------------------------------------------------------------------------

;	p1	= instrument number
;	p2	= action time [sec]
;	p3	= duration [sec]
;	p4	= maximum amplitude [linear, 0.0 -> 1000.0]
;	p5	= fundamental frequency [Hz]
;	p6	= formant frequency [Hz]
;	p7	= bandwidth [Hz]
;	p8	= amplitude envelope [GEN number]
;	p9	= tex [sec]
;	p10	= krise [sec]
;	p11	= kdur [sec]

;-----------------------------------------------------------------------------
; COMPULSORY GEN FUNCTIONS
;	f1	large sine tone
;	f19	sigmoid rise/decay shape
;_____________________________________________________________________________

; CLASS: FOF-2

;  GLOBAL KEYWORDS (default values within parentheses):
;	NUMROWS		: amount of rows (components) in the event (1)
;	ACTION-TIME	: start time of the whole event [sec] (0.0)
;	USER-FUN	: user-defined parsing function (nil)

;  LOCAL KEYWORDS:
;	E-DEL		: entry delays [sec] (0.0)
;	DURS		: duration [sec] (1.0)
;	AMP		: amplitude [absolute 0-1000] (500.0)
;	F0		: fundamental frequency [Hz] (100.0)
;	FREQ		: center frequency [Hz] (609.0)
;	BW		: bandwidth [Hz] (77.0)
;	AENV		: fun number for the amplitude envelope [GEN] (fast up/ slow down)
;	TEX		: excitation (rise) time of the FOF [sec]
;	DEBATT		: total duration of the FOF [sec]
;	ATTEN		: decay of the FOF [sec]
;
;*****************************************************************************



sr	= 44100
kr	= 44100
ksmps	= 1
nchnls	= 1

gimaxamp init 32767 			; 16 bits


instr 1; *****************************************************************
idur   	= p3
idurosc	= 1/p3
iamp	= p4 * gimaxamp * 0.001	; amplitude range 0-1000
if0	= p5
ifreq	= p6
ibw	= p7
iaenv	= p8
itex	= p9
idebatt	= p10
iatten	= p11

iolaps	= 1000	; how many simultaneous FOFs can be played (takes little memory if not used)

   kenv	oscili	iamp, idurosc, iaenv

   ;                           koct                       		ifna    idur
   ;          xamp  xfund xform     kband kris   kdur    kdec    iolaps    ifnb
   a1 fof     kenv, if0,  ifreq, 0,  ibw, itex, idebatt, iatten, iolaps, 1, 19, idur
      out     a1

endin
