;******************************************************************
;		     CLASS PLUCK-1
;******************************************************************

(in-package om)

; LISP-DEFINED CLASSES SHOULD RESIDE IN THE LIBRARY'S PACKAGE AND 
;   NOT IN THE USER PACKAGE, WHICH CONTAINS ALL THE CLASSES
;   GRAPHICALLY DEFINED


(defclass! pluck-1
  (cs-evt) 			; INHERIT FROM CS-EVT
  (
   
   ; GLOBAL SLOTS (LIGHT BLUE, ON THE LEFT OF THE CLASS):
   ;    THE METHOD BELOW TRANSFORMS THEM INTO GLOBAL SLOTS ("SHOW" UNCHEKED)
   ;    ATTENTION: A GLOBAL SLOT SHOULD NOT HAVE AN INITARG
   
   ( source-code :initform
                 (load-buffer-textfile
                  (get-orc-source (get-orc "pluck-1"))
                  'textfile "append")
                 :allocation :class
                 :type textfile
                 :accessor source-code)
   ( numchan :initform (or (get-orc-channels (get-orc "pluck-1")) 1)
             :allocation :class  :accessor numchan)
   ( globals-list :initform 
                  (get-orc-globals
                   (get-orc "pluck-1"))
                  :allocation :class
                  :type list
                  :accessor globals-list)
   (InstID :initform 1  :allocation :class  :accessor InstID)
   ; LOCAL SLOTS (RED, CORRESPONDING TO THE P-FIELDS)
   ;    ATTENTION: A GLOBAL SLOT SHOULD HAVE AN INITARG
   ( amp	:type number
		:initarg :amp 
  		:initform 500.0
		:accessor amp)
   ( freq	:type number
        	:initarg :freq 
        	:initform 440.0
        	:accessor freq)
   ( buf	:type number
        	:initarg :buf 
        	:initform 128
        	:accessor buf)
   ( ranfn	:type number
        	:initarg :ranfn 
        	:initform 71
        	:accessor ranfn)
   ( dec	:type number
        	:initarg :dec 
        	:initform 0.2
        	:accessor dec)
   )
  
  
  (:documentation
   "
;=============================================================================
;		PLUCK-1.ORC
; PLUCKED STRING USING THE KARPLUS-STRONG MODEL
;    (FROM ACCCI, 15_01_3.ORC) / MONO
; AMPLITUDE ENVELOPE WITH LINSEG, CONTROL OF NOISE SOURCE FROM THE SCORE
;=============================================================================

; Timbre:       Plucked string
; Synthesis:    Karplus-Strong algorithm
;               PLUCK
;               LINSEG envelope,
;               Varying parameter ranfun: 10 different sources of random
;                  numbers, two different buffer sizes
; Coded:     	jpg 8/93, modified ms 9/02

; NB: the apparently arbitrary amplitude range (0-1000, rather than 0-1)
;         avoids Lisp printing small values with exponential notation
;-----------------------------------------------------------------------------

;	p1	= instrument number
;	p2	= action time [sec]
;	p3	= duration [sec]
;	p4	= maximum amplitude [linear, 0.0 -> 1000.0]
;	p5	= frequency [Hz]
;	p6	= buffer size [power of 2]
;	p7	= function number of the random source [GEN number]
;	p8	= percentage of the total duration used for the last decay [0-1]

;-----------------------------------------------------------------------------
; COMPULSORY GEN FUNCTIONS

; The file "pluck-a.aiff" contains random noise with center frequency 'fqr'
; changing each second...


; !! BEWARE THAT THE DIRECTORY WHERE THIS FILE IS LOCATED IS KNOWN BY CSOUND !

; f71-80: 10 sources of random noise with center frequency changing every sec.

;_____________________________________________________________________________

; CLASS: PLUCK-1

;  GLOBAL KEYWORDS (default values within parentheses):
;	NUMROWS		: amount of rows (components) in the event (1)
;	ACTION-TIME	: start time of the whole event [sec] (0.0)
;	USER-FUN	: user-defined parsing function (nil)

;  LOCAL KEYWORDS:
;	E-DELS	:	entry delays [sec] (0.0)
;	DURS	:	duration [sec] (1.0)
;	AMP	:	amplitude [sec] (500.0)
;	FREQ	:	frequency [Hz] (440.0)
;	BUF	:	frequency of the noise [power of 2] (128)
;	RANFN	:	function number of the random source [GEN number or 0 for none] (0)
;	DEC	:	percentage of the total dur used for the last decay [0-1] (0.2)

;*****************************************************************************
"
   )
  (:icon 618)
  )

;------------------------------------------------------------------
;	METHODS FOR THIS CLASS
;------------------------------------------------------------------
; INCLUDE THE COMPULSORY FUNCTIONS AT THE BEGINNING OF A SCORE FILE
;     THIS METHOD MUST BE DEFINED FOR THIS CLASS


  

(defmethod! orc-header ((self pluck-1))
  (list
   
   "; GEN functions **********************************************************"
   ""
   "; pluck-a.aif contains random noise with center frequency 'fqr'"
   ";    changing each second..."
   "; NB: ignore csound's warning concerning the table's length"
   ""
   ";       size GEN  inputfile             start"
   (format nil "f71  0  1024  1   ~S    0.2    0  0         ; fqr = 10000" (om-path2cmdpath "cl:chroma;example-files;pluck-a.aif"))
   (format nil "f72  0  1024  1   ~S    1.2    0  0         ; fqr = 5000" (om-path2cmdpath "cl:chroma;example-files;pluck-a.aif"))
   (format nil "f73  0  1024  1   ~S    2.2    0  0         ; fqr = 2500" (om-path2cmdpath "cl:chroma;example-files;pluck-a.aif"))
   (format nil "f74  0  1024  1   ~S    3.2    0  0         ; fqr = 2000" (om-path2cmdpath "cl:chroma;example-files;pluck-a.aif"))
   (format nil "f75  0  1024  1   ~S    4.2    0  0         ; fqr = 1000" (om-path2cmdpath "cl:chroma;example-files;pluck-a.aif"))
   (format nil "f76  0  1024  1   ~S    5.2    0  0         ; fqr = 500" (om-path2cmdpath "cl:chroma;example-files;pluck-a.aif"))
   (format nil "f77  0  1024  1   ~S    6.2    0  0         ; fqr = 250" (om-path2cmdpath "cl:chroma;example-files;pluck-a.aif"))
   (format nil "f78  0  1024  1   ~S    7.2    0  0         ; fqr = 125" (om-path2cmdpath "cl:chroma;example-files;pluck-a.aif"))
   (format nil "f79  0  1024  1   ~S    8.2    0  0         ; fqr = 50" (om-path2cmdpath "cl:chroma;example-files;pluck-a.aif"))
   (format nil "f80  0  1024  1   ~S    9.2    0  0         ; fqr = 25" (om-path2cmdpath "cl:chroma;example-files;pluck-a.aif"))))

;******************************************************************
