;=============================================================================
;		PLUCK-1.ORC
; PLUCKED STRING USING THE KARPLUS-STRONG MODEL
;    (FROM ACCCI, 15_01_3.ORC) / MONO
; AMPLITUDE ENVELOPE WITH LINSEG, CONTROL OF NOISE SOURCE FROM THE SCORE
;=============================================================================

; Timbre:       Plucked string
; Synthesis:    Karplus-Strong algorithm
;               PLUCK
;               LINSEG envelope,
;               Varying parameter ranfun: 10 different sources of random
;                  numbers, two different buffer sizes
; Coded:     	jpg 8/93, modified ms 9/02

; NB: the apparently arbitrary amplitude range (0-1000, rather than 0-1)
;         avoids Lisp printing small values with exponential notation
;-----------------------------------------------------------------------------

;	p1	= instrument number
;	p2	= action time [sec]
;	p3	= duration [sec]
;	p4	= maximum amplitude [linear, 0.0 -> 1000.0]
;	p5	= frequency [Hz]
;	p6	= buffer size [power of 2]
;	p7	= function number of the random source [GEN number]
;	p8	= percentage of the total duration used for the last decay [0-1]

;-----------------------------------------------------------------------------
; COMPULSORY GEN FUNCTIONS

; The file "pluck-a.aiff" contains random noise with center frequency 'fqr'
; changing each second...


; !! BEWARE THAT THE DIRECTORY WHERE THIS FILE IS LOCATED IS KNOWN BY CSOUND !

; f71-80: 10 sources of random noise with center frequency changing every sec.

;_____________________________________________________________________________

; CLASS: PLUCK-1

;  GLOBAL KEYWORDS (default values within parentheses):
;	NUMROWS		: amount of rows (components) in the event (1)
;	ACTION-TIME	: start time of the whole event [sec] (0.0)
;	USER-FUN	: user-defined parsing function (nil)

;  LOCAL KEYWORDS:
;	E-DELS	:	entry delays [sec] (0.0)
;	DURS	:	duration [sec] (1.0)
;	AMP	:	amplitude [sec] (500.0)
;	FREQ	:	frequency [Hz] (440.0)
;	BUF	:	frequency of the noise [power of 2] (128)
;	RANFN	:	function number of the random source [GEN number or 0 for none] (0)
;	DEC	:	percentage of the total dur used for the last decay [0-1] (0.2)

;*****************************************************************************


sr	= 44100
kr	= 44100
ksmps	= 1
nchnls	= 1

gimaxamp init 32767 				; 16 bits


instr 1; *****************************************************************

idur		= p3
iamp		= p4 * gimaxamp * 0.001		; amplitude range 0-1000
ifq		= p5				; frequency
ibuf		= p6				; buffer size
iranfun		= p7				; various sources of random noise are called
idec		= p8				; percentage [0-1] of idur for the last decay
						; affect the nature of damping

imeth		= 1				; simple averaging

iseg1		= (1-idec)*idur
iseg2		= idec*idur

      kamp   linseg   iamp,iseg1,iamp,iseg2,0
      a1     pluck    kamp, ifq, ibuf, iranfun, imeth
             out      a1
endin
