;******************************************************************
;		     CLASS SNARE-1
;******************************************************************

(in-package om)

; LISP-DEFINED CLASSES SHOULD RESIDE IN THE LIBRARY'S PACKAGE AND 
;   NOT IN THE USER PACKAGE, WHICH CONTAINS ALL THE CLASSES
;   GRAPHICALLY DEFINED


(defclass! snare-1
  (cs-evt) 			; INHERIT FROM CS-EVT
  (

; GLOBAL SLOTS (LIGHT BLUE, ON THE LEFT OF THE CLASS):
;    THE METHOD BELOW TRANSFORMS THEM INTO GLOBAL SLOTS ("SHOW" UNCHEKED)
;    ATTENTION: A GLOBAL SLOT SHOULD NOT HAVE AN INITARG

   ( source-code :initform
                 (load-buffer-textfile
                  (get-orc-source (get-orc "snare-1"))
                  'textfile "append")
                 :allocation :class
                 :type textfile
                 :accessor source-code)
   ( numchan :initform (or (get-orc-channels (get-orc "snare-1")) 1)
             :allocation :class  :accessor numchan)
   ( globals-list :initform 
                  (get-orc-globals
                   (get-orc "snare-1"))
                  :allocation :class
                  :type list
                  :accessor globals-list)
   (InstID :initform 1  :allocation :class  :accessor InstID)

; LOCAL SLOTS (RED, CORRESPONDING TO THE P-FIELDS)
;    ATTENTION: A GLOBAL SLOT SHOULD HAVE AN INITARG
  ( amp		:type number
		:initarg :amp 
  		:initform 200.0
		:accessor amp)
   ( freq	:type number
        	:initarg :freq 
        	:initform 20.0
        	:accessor freq)
   ( inha	:type number
        	:initarg :inha 
        	:initform 75.0
        	:accessor inha)
   ( rana	:type number
        	:initarg :rana 
        	:initform 250.0
        	:accessor rana)
   ( rancf	:type number
        	:initarg :rancf 
        	:initform 4000.0
        	:accessor rancf)
   ( ranbw	:type number
        	:initarg :ranbw 
        	:initform 1500.0
        	:accessor ranbw)
   )

  (:documentation
   "
;=============================================================================
;				SNARE-1.ORC
; SIMPLE SIMULATION OF A SNARE DRUM (FROM ACCSI, 03_01_1.ORC) / MONO
;=============================================================================

; Timbre:    Drum and snare drum
; Synthesis: Additive different units
;            Units: noise / inharm / fundamental
; Source:    #400, Drum and Snare-drum like Sounds, Risset (1969)
; Coded:     jpg 8/93, modified ms 9/02

; NB1: this instrument works better with short durations
; NB2: the apparently arbitrary amplitude range (0-1000, rather than 0-1)
;         avoids Lisp printing small values with exponential
;-----------------------------------------------------------------------------

;	p1	= instrument number
;	p2	= action time [sec]
;	p3	= duration [sec]
;	p4	= amplitude of sine tone [linear, 0.0 -> 1000.0]
;	p5	= fundamental frequency [Hz]
;	p6	= amplitude of pseudo inharmonic tone [linear, 0.0 -> 1000.0]
;	p7	= noise amplitude [[linear, 0.0 -> 1000.0]
;	p8	= center frequency for the noise band [Hz]
;	p9	= noise 1/2 bandwidth [Hz]

;-----------------------------------------------------------------------------
; COMPULSORY GEN FUNCTIONS :
;	f11	noise-modulated sine wave
;	f12	sine wave with only one high partial (10th)
;	f13	pseudo-inharmonic spectrum made of high partials

;	f21	slowly descending exponential envelope
;	f22	rapidly descending exponential envelope
;_____________________________________________________________________________

; CLASS: SNARE-1

;  GLOBAL KEYWORDS (default values within parentheses):
;	NUMROWS		: amount of rows (components) in the event (1)
;	ACTION-TIME	: start time of the whole event [sec] (0.0)
;	USER-FUN	: user-defined parsing function (nil)

;  LOCAL KEYWORDS:
;   E-DELS	:	entry delays [sec] (0.0)
;	DURS	:	duration [sec] (1.0)
;	AMP	:	amplitude of sine tone [0.0 -> 1000.0] (200.0)
;	FREQ	:	fundamental frequency [Hz] (20.0)
;	INHA	:	amplitude of pseudo inharmonic tone [0.0 -> 1000.0] (75.0)
;	RANA	:	amplitude of pseudo inharmonic tone [0.0 -> 1000.0] (250.0)
;	RANCF	:	center frequency for the noise band [Hz] (4000.0)
;	RANBW	:	noise 1/2 bandwidth [Hz] (1500.0)


;*****************************************************************************

"
   )
  (:icon 618)
  )

;------------------------------------------------------------------
;	METHODS FOR THIS CLASS
;------------------------------------------------------------------
; INCLUDE THE COMPULSORY FUNCTIONS AT THE BEGINNING OF A SCORE FILE
;     THIS METHOD MUST BE DEFINED FOR THIS CLASS

(defmethod! orc-header ((self snare-1))
  (list
   "; GEN functions **********************************************************"
   "; waveforms"
   "f11  0  4097  9   1  1  0"
   "f12  0  8193  9  10  1  0"
   "f13  0  8193  9  10  1  0  16  1  0  22  1  0  23  1  0"
   ""
   "; envelopes"
   "f21  0  513  5   256  513  1"
   "f22  0  513  5  4096  513  1				; steeper slope"
   ))

;******************************************************************
