;=============================================================================
;			SNARE-1.ORC
; SIMPLE SIMULATION OF A SNARE DRUM (FROM ACCCI, 03_01_1.ORC) / MONO
;=============================================================================

; Timbre:    Drum and snare drum
; Synthesis: Additive different units
;            Units: noise / inharm / fundamental
; Source:    #400, Drum and Snare-drum like Sounds, Risset (1969)
; Coded:     jpg 8/93, modified ms 9/02

; NB1: this instrument works better with short durations
; NB2: the apparently arbitrary amplitude range (0-1000, rather than 0-1)
;         avoids Lisp printing small values with exponential notation
;-----------------------------------------------------------------------------

;	p1	= instrument number
;	p2	= action time [sec]
;	p3	= duration [sec]
;	p4	= amplitude of sine tone [linear, 0.0 -> 1000.0]
;	p5	= fundamental frequency [Hz]
;	p6	= amplitude of pseudo inharmonic tone [linear, 0.0 -> 1000.0]
;	p7	= noise amplitude [[linear, 0.0 -> 1000.0]
;	p8	= center frequency for the noise band [Hz]
;	p9	= noise 1/2 bandwidth [Hz]

;-----------------------------------------------------------------------------
; COMPULSORY GEN FUNCTIONS :
;	f11	noise-modulated sine wave
;	f12	sine wave with only one high partial (10th)
;	f13	pseudo-inharmonic spectrum made of high partials

;	f21	slowly descending exponential envelope
;	f22	rapidly descending exponential envelope
;_____________________________________________________________________________

; CLASS: SNARE-1

;  GLOBAL KEYWORDS (default values within parentheses):
;	NUMROWS		: amount of rows (components) in the event (1)
;	ACTION-TIME	: start time of the whole event [sec] (0.0)
;	USER-FUN	: user-defined parsing function (nil)

;  LOCAL KEYWORDS:
;   E-DELS	:	entry delays [sec] (0.0)
;	DURS	:	duration [sec] (0.5)
;	AMP	:	amplitude of sine tone [0.0 -> 1000.0] (200.0)
;	FREQ	:	fundamental frequency [Hz] (20.0)
;	INHA	:	amplitude of pseudo inharmonic tone [0.0 -> 1000.0] (75.0)
;	RANA	:	amplitude of the noise [0.0 -> 1000.0] (250.0)
;	RANCF	:	center frequency for the noise band [Hz] (4000.0)
;	RANBW	:	noise's 1/2 bandwidth [Hz] (1500.0)


;*****************************************************************************

sr	= 44100
kr	= 44100
ksmps	= 1
nchnls	= 1

gimaxamp init 32767 					; 16 bits


instr 1; --------------------------------------------------------------------

idur		=  p3
idurosc		= 1/idur
isin_amp	=  p4 * gimaxamp * 0.001	; amplitude range 0-1000
if0			=  p5

inh_amp		=  p6 * gimaxamp * 0.001
iran_amp	=  p7 * gimaxamp * 0.001
iran_cfq	=  p8
iran_bw		=  p9

iranfun		=  11
inhfun		=  13
iaudiofun	=  12
islowenv	=  21
ifastenv	=  22

; noise
         a5     rand    iran_amp, iran_bw
         a5     oscili  a5, idurosc, ifastenv
         a5     oscili  a5, iran_cfq, iranfun

; pseudo-inharmonic spectrum
         a3     oscili  iran_amp, idurosc, ifastenv
         a3     oscili  a3, if0, inhfun

; sine tone
         a1     oscili  isin_amp, idurosc, islowenv
         a1     oscili  a1, if0, iaudiofun

                out     a1+a3+a5
endin
