;=============================================================================
;			SUB-1.ORC
; SUBTRACTIVE SYNTHESIS OF RANDOM NOISE (FROM ACCCI, 50_01_1.ORC) / MONO
; AMPLITUDE ENVELOPE WITH OSCILI
;=============================================================================

; Timbre:    Bands of noise
; Synthesis: Subtractive synthesis
;            Basic design
;            RAND source
; Coded:     jpg 8/92, modified ms 9/02

; NB: the apparently arbitrary amplitude range (0-1000, rather than 0-1)
;         avoids Lisp printing small values with exponential notation
;-----------------------------------------------------------------------------

;	p1	= instrument number
;	p2	= action time [sec]
;	p3	= duration [sec]
;	p4	= maximum amplitude [linear, 0.0 -> 1000.0]
;	p5	= center frequency [Hz]
;	p6	= bandwidth [% of center frequency, 0->1]
;	p7	= amplitude envelope [GEN]

;-----------------------------------------------------------------------------
; COMPULSORY GEN FUNCTIONS
;	none
;_____________________________________________________________________________

; CLASS: SUB-1

;  GLOBAL KEYWORDS (default values within parentheses):
;	NUMROWS		: amount of rows (components) in the event (1)
;	ACTION-TIME	: start time of the whole event [sec] (0.0)
;	USER-FUN	: user-defined parsing function (nil)

;  LOCAL KEYWORDS:
;	E-DELS	:	entry delays [sec] (0.0)
;	DURS	:	duration [sec] (1.0)
;	AMP	:	amplitude [sec] (500.0)
;	FREQ	:	center frequency [Hz] (2000.0)
;	BW	:	bandwidth [% of FQ, 0->1] (0.05)
;	AENV	:	function number for the amplitude envelope [GEN] (fast up/down)

;*****************************************************************************



sr	= 44100
kr	= 44100
ksmps	= 1
nchnls	= 1 

gimaxamp init 32767 			; 16 bits


instr 1; *****************************************************************
idur	= p3
idurosc	= 1/p3
iamp	= p4 * gimaxamp * 0.001	; amplitude range 0-1000
icfq	= p5
ibw		= p6*icfq
iaenv	= p7

  
   anoise  rand   iamp                        ; white noise
   a1      reson  anoise,  icfq, ibw, 2       ; filter, iskip = 2
   a1      oscili a1, idurosc, iaenv
           out    a1

endin
