(in-package :om)




;============================================================
;CSOUND CLASSES
;============================================================

#|
(defclass internCSEvt ()
   ((parsing-fun :initform nil  :accessor parsing-fun)
    (action-time :initform 0  :accessor action-time)
    (InstID :initform 1 :initarg :instid :allocation :class  :accessor InstID)))
|#

(defclass internCSEvt ()
  ((parsing-fun :initform nil  :accessor parsing-fun)
   (action-time :initform 0  :accessor action-time)
   (user-fun :initform nil  :accessor user-fun)
   (numchan :initform 1  :initarg :numchan :accessor numchan)

   ))
   

(defclass! CS-Evt (SynthesisEvt internCSEvt) 
   ((source-code :initform (load-buffer-textfile nil 'textfile "append") :allocation :class :type textfile :accessor source-code)
    (globals-list :initform nil :allocation :class :type list :accessor globals-list)
    (macros-list :initform nil :allocation :class :type list :accessor macros-list)
    (E-dels :initform 0 :initarg :E-dels :type number :accessor E-dels)
    (Durs :initform 1 :initarg :Durs :type number :accessor Durs)
    (InstID :initform 1  :allocation :class  :accessor InstID)) 
   (:icon 601))

(defmethod get-obj-dur ((self CS-Evt)) 
   (loop for onset in (get-array-row self 0)
         for dur in (get-array-row self 1)
         minimize onset into start
         maximize  (+ onset dur) into dura
         finally (return (round (* 1000 (- dura start))))))


(defmethod prepare-tables ((self CS-Evt))
  (let* ((initargs (get-all-initargs-of-class (type-of self))))
    (loop for slot? in initargs do
          (when (subtypep (thetype slot?) 'cs-table)
            (prepare-table (slot-value self (internp (name slot?) (slot-package slot?))))))))

(defmethod prepare-tables ((self CS-Evt))
   (when (work-vector self)
     (let* ((initargs (get-all-initargs-of-class (type-of self))))
       (loop for slot? in initargs
             for i = 0 then (+ i 1) do
             (when (subtypep (thetype slot?) 'cs-table)
               (prepare-table (aref (work-vector self) i)))))))

(defmethod erase-temp-id-tables ((self CS-Evt))
  (let* ((initargs (get-all-initargs-of-class (type-of self))))
    (loop for slot? in initargs do
          (when (subtypep (thetype slot?) 'cs-table)
            (erase-temp-id-table (slot-value self (internp (name slot?) (slot-package slot?))))))))

(defmethod erase-temp-id-tables ((self CS-Evt))
   (when (work-vector self)
     (let* ((initargs (get-all-initargs-of-class (type-of self))))
       (loop for slot? in initargs 
             for i = 0 then (+ i 1)do
             (when (subtypep (thetype slot?) 'cs-table)
               (erase-temp-id-table (aref (work-vector self) i)))))))


(defmethod prepare-table ((self t)) t)

(defmethod prepare-table ((self list)) 
   (loop for item in self do
         (prepare-table item)))

(defmethod erase-temp-id-table ((self t)) t)

(defmethod erase-temp-id-table ((self list)) 
   (loop for item in self do
         (erase-temp-id-table item)))

(defmethod! orc-header ((self CS-Evt))  nil)

            

;-----------------------------------------------------------
;TABLES
;-----------------------------------------------------------


(defvar *dynamic-cs-tbale-table* (make-hash-table :test 'equal))

(defvar *table-counter* 0)

(defvar *globals-csound-tables* nil)

(defclass! CS-table () 
   ((id :initform "?" :initarg :id :accessor id)
    (size :initform *def-table-size* :initarg :size :type integer :accessor size)
    (stime :initform 0 :initarg :stime :type number :accessor stime))
   (:icon 605))

#|
(defmethod cs-table-p ((self CS-table)) t)
(defmethod cs-table-p ((self t)) nil)
|#

(defmethod prepare-table ((self CS-table))
   (cond 
    ((equal (id self) "?") 
     (let ((i (+ *auto-init-table-id* *table-counter*)))
       (setf *table-counter* (+ *table-counter* 1))
       (setf (id self) i)
       (setf (gethash i *dynamic-cs-tbale-table*) (cs-table-string self))
       (setf (id self) (list i))))
    ((listp (id self)) t)
    ((integerp (id self))
     (setf (gethash (id self) *dynamic-cs-tbale-table*) (cs-table-string self)))))

(defmethod erase-temp-id-table ((self CS-table))
   (when (listp (id self))
     (setf (id self) "?")))

(defmethod get-table-id ((self CS-table)) 
   (car (list! (id self))))
     

#|
(defmethod cs-table-format ((self CS-table))
  (format nil "f ~~D ~~D ~~D ~~D ~~{ ~~~DF~~}" (decimals self)))
|#

(defclass! gen-cs-table (CS-table) 
  ((gen-num :initform 1 :initarg :gen-num :type integer :accessor gen-num)
   (param-list :initform nil :initarg :param-list :type list :accessor param-list))
  (:icon 605))




(defmethod cs-table-string ((self gen-cs-table))
  (format nil "f ~D ~D ~D ~D ~{ ~6F~}" (round (id self)) (stime self) 
          (size self) (gen-num self) (param-list self)))


(defclass! bpf-cs-table (CS-table bpf) ()
   (:icon 605)
   (:default-initargs 
     :x-points '(0 256 512)
     :y-points '(0 100 0)))


(defmethod get-initval ((self bpf-cs-table))
  (make-instance (class-of self) :point-list (list (om-make-point 0 0)  (om-make-point 256 1) (om-make-point 512 0))))

(defmethod make-one-instance ((self bpf-cs-table) &rest slots-vals)
   (let ((table (call-next-method)))
     (setf (id table) (fourth slots-vals)
           (size table) (fifth slots-vals)
           (stime table) (sixth slots-vals))
     table))

(defun make-cs-table (class x-points y-points &optional (decimals 0) (id "?") (size 256) (stime 0))
   (let ((table (simple-bpf-from-list x-points y-points class decimals)))
     (setf (id table) id
           (size table) size
           (stime table) stime)
     table))



(defmethod cs-bpf-points ((self bpf-cs-table))
   (let* ((size (size self))
          (pointx (x-points self))
          (pointy (y-points self))
          )
     (setf pointx (cdr (mapcar 'round (om-scale pointx 0 size  ))))
     
     (append (loop for y in pointy
                   for last = 0 then x
                   for x in pointx
                   append (list y (- x last))) (last pointy)) ))

(defmethod cs-table-string ((self bpf-cs-table))
  (let* ((points (cs-bpf-points self)))
    (format nil "f ~D ~D ~D ~D ~{ ~6F~}" (id self) (stime self)  (size self) (tnum self) points)))


(defclass! Gen07 (bpf-cs-table) () 
   (:icon 605))

(defmethod tnum ((self Gen07)) 7)


(defclass! Gen-07 (bpf-cs-table) () 
   (:icon 605))

(defmethod tnum ((self Gen-07)) -7)

(defclass! Gen05 (bpf-cs-table) () (:icon 605))

(defmethod tnum ((self Gen05)) 5)

(defmethod get-super-default-value ((type (eql 'CS-table)))
  (make-instance 'Gen07 :point-list (list (om-make-point 0 0)  (om-make-point 256 1) (om-make-point 512 0)))  )


(defclass file-cs-table ()
  ((id :initform nil :initarg :id :accessor id)
   (csstring :initform nil :initarg :vect :accessor csstring)))

(defmethod cs-table-string ((self file-cs-table)) (csstring self))

;
(defun SCsT (string)
  (let ((table (make-instance 'file-cs-table
                 :id (gen-table-id-from-str  string))))
    (setf (csstring table) string)
    (push table *globals-csound-tables*)
    table))

(defun CsT (id vect &key (size *def-table-size*) (gennum 7))
  (let ((table (make-instance 'file-cs-table
                 :id id)))
    (setf (csstring table) (format nil "f ~D 0 ~D ~D ~{~S ~}" id size gennum vect))
    (push table *globals-csound-tables*)
    table))



(defmethod get-table-id ((self integer)) self)

;strings
(defmethod prepare-table ((self string))
   (setf (gethash (gen-table-id-from-str self) *dynamic-cs-tbale-table*) self))

(defmethod get-table-id ((self string)) (gen-table-id-from-str self))

(defun gen-table-id-from-str (string)
  (let ((rep (delete-spaces string)))
    (if (equal (elt rep 0) #\f)
      (setf rep (string-until-space (delete-spaces (subseq rep 1))))
      (error "Table definition strings must start by f"))
    (when rep
      (read-from-string rep))))

;================================================
;Coerce
;================================================

(defmethod* Objfromobjs ((Self bpf) (Type CS-table))
   (Objfromobjs (Objfromobjs self (make-instance *bpf-cs-coherce*)) type ))


(defmethod* Objfromobjs ((Self bpf-cs-table) (Type CS-table))
   (let* ((points (cs-bpf-points self))
          (size (size self)))
     (make-instance 'gen-cs-table
       :id (id self)
       :stime (stime self)
       :decimals (decimals self)
       :size (size self)
       :gen-num (tnum self)
       :param-list points)))

(defmethod* Objfromobjs ((Self bpf) (Type bpf-cs-table))
   (let ((table (make-instance (type-of type)
                  :point-list (point-list self)
                  :decimals (decimals self))))
     (setf (bpfcolor table) (bpfcolor self))
     table))


;================================================
;Coerce in class-array
;================================================
(defmethod verify-list-exacte ((self list) (type (eql 'CS-table)) num (array CS-evt))
   (loop for i from 0 to (- (length self) 1)
         collect (get-ieme self i array type)))

(defmethod verify-list-exacte ((self bpf-lib) (type (eql 'CS-table)) num (array CS-evt))
   (verify-list-exacte (bpf-list self) type num array))

(defmethod verify-list-exacte ((self t) (type (eql 'CS-table)) num (array CS-evt))
   (if (get-ieme self 0 array type) 
     self (error (format nil "can not coerce ~D in a cs-table" self))))

#|
(defmethod verify-list-exacte ((self t) (type (eql 'CS-table)) num (array CS-evt))
   (let ((ieme (get-ieme self 0 array type)))
     (if  ieme ieme (error (format nil "can not coerce ~D in a cs-table" self)))))
|#

;integer

(defmethod get-ieme ((self integer)  i (array CS-evt) (type (eql 'CS-table))) self)

(defmethod get-points ((self integer)  (type (eql 'CS-table)) (array CS-evt) )
   (make-list  (numcols array) :initial-element self))

;integer

(defmethod get-ieme ((self string)  i (array CS-evt) (type (eql 'CS-table))) self)

(defmethod get-points ((self string)  (type (eql 'CS-table))  (array CS-evt) )
   (make-list  (numcols array) :initial-element self))

;bpf

(defmethod get-ieme ((self bpf)  i (array CS-evt) (type (eql 'CS-table)))
   (let ((table (make-instance *bpf-cs-coherce*
                  :point-list (point-list self)
                  :decimals (decimals self))))
     (setf (bpfcolor table) (bpfcolor self))
     table))

(defmethod get-points ((self bpf)  (type (eql 'CS-table)) (array CS-evt))
   (make-list  (numcols array) :initial-element (get-ieme self 0 array type)))

;bpf-cs-table

(defmethod get-ieme ((self cs-table)  i (array CS-evt) (type (eql 'CS-table)))
   self)

(defmethod get-points ((self cs-table)  (type (eql 'CS-table)) (array CS-evt))
   (make-list  (numcols array) :initial-element self))



;bpf-lib
(defmethod get-ieme ((self bpf-lib) i  (array CS-evt) (type (eql 'CS-table)))
   (get-ieme (bpf-list self) i array type))

(defmethod get-points ((self bpf-lib)  (type (eql 'CS-table)) (array CS-evt))
   (get-points (bpf-list self) type array))

;list

(defmethod get-ieme ((self list) i  (array CS-evt) (type (eql 'CS-table)))
   (get-ieme (nth (mod i (length self)) self) 0 array type))

(defmethod get-points ((self list)  (type (eql 'CS-table)) (array CS-evt))
   (loop for i from 0 to (- (numcols array) 1)
             collect (get-ieme self i array type)))


(defmethod table-for-draw ((self t)) self)

(defmethod table-for-draw ((self cs-table)) (format nil "f ~D" (id self)))

(defmethod table-for-draw ((self bpf-cs-table)) self)

(defmethod table-for-draw ((self string)) (format nil "f ~D" (gen-table-id-from-str  self)))

(defmethod table-for-draw ((self integer)) (format nil "f ~D" self))

      
(defmethod get-array-i-j-for-eval ((self CS-evt) i j)
   (if (subtypep (row-type self i) 'cs-table)
     (table-for-draw (get-ieme (valor-from-index self i) j self (row-type self i)))
     ;(get-ieme (valor-from-index self i) j self (row-type self i))
     (call-next-method)
     ))

(defmethod get-array-i-j ((self CS-evt) i j)
   (if (subtypep (row-type self i) 'cs-table)
     (get-table-id (get-ieme (valor-from-index self i) j self (row-type self i)))
     (get-ieme (valor-from-index self i) j self (row-type self i))))


(defmethod get-array-row ((self CS-evt) num)
   (let ((rep (call-next-method)))
     (if (subtypep (row-type self num) 'cs-table)
       (loop for item in rep collect 
             (let (id)
               (prepare-table item)
               (setf id (get-table-id item))
               (erase-temp-id-table item)
               id))
       rep)))

(defmethod get-bfp-for-draw ((self CS-table) (array CS-evt) preci type) nil)

(defmethod get-bfp-for-draw ((self integer) (array CS-evt) preci type) 
   (if (subtypep type 'cs-table) nil (call-next-method)))

(defmethod get-bfp-for-draw ((self list) (array CS-evt) preci type) 
   (if (subtypep type 'cs-table) nil (call-next-method)))

(defmethod get-bfp-for-draw ((self string) (array CS-evt) preci type) 
   (if (subtypep type 'cs-table) nil (call-next-method)))


(defmethod get-bfp-all-points ((self number) (array CS-evt) preci type) 
   (if (subtypep type 'cs-table) nil (call-next-method)))


;=====================
;action-time
;=====================

#|
(defmethod get-slot-in-out-names ((self CS-evt))
   (values '("self" "numrows" "action-time") '(nil 1 0)
           '("object or object list" "How many vertical components for init ?" "local event offset (in seconds)")
           '(nil nil nil)))
|#

(defmethod get-slot-in-out-names ((self CS-evt))
   (values '("self" "numcols" "action-time" "user-fun") '(nil 1 0 nil)
           '("object or object list" "How many vertical components for init ?" "local event offset (in seconds)"
             "doc for user-fun")
           '(nil nil nil nil)))

;(defmethod fixed-slots-list ((self CS-evt)) '(numcols action-time))

(defmethod fixed-slots-list ((self CS-evt)) '(numcols action-time user-fun))

;NEW


(defclass! gen01 (CS-table)
  ((file :accessor file :initform "" :initarg :file)
   (skiptime :accessor skiptime :initform 0 :initarg :skiptime)
   )
  (:icon 605))

(defmethod initialize-instance ((self gen01) &rest initargs)
  (call-next-method)
  (when 
    (and (file self) (equal 'sound (type-of (file self))))
    (setf (file self) (om-sound-file-name (file self)))))

(defmethod tnum ((self Gen01)) 1)

(defmethod cs-table-string ((self gen01))
  (format nil "f~D ~D ~D ~D ~s ~D 0 0" (id self) (stime self) (size self) (tnum self) (om-path2cmdpath (file self)) (skiptime self)))