(in-package :om)

(defvar *perf-is-running* nil)


;===============================================================
;PARSE AN ORC FILE AND DEFINE A CLASS FOR EACH CSOUND INSTRUMENT
;===============================================================


(defmethod! get-instrument () 
   :initvals nil :indoc nil :icon 602
   :doc "For the time being parse only csound.orc files."
   (let ((name (om-CHOOSE-FILE-DIALOG)))
     (when name
       (load-instruments name))))


;;;===================================================

(defmethod! make-instrument ((input t)) 
   :initvals nil :indoc nil :icon 602
   :doc "For the time being parse only csound.orc files"
   nil)

(defmethod! make-instrument ((input pathname)) 
  (if (probe-file input)
    (load-instruments input)
    (format nil "~s not found" (namestring input))))

(defmethod! make-instrument ((input string)) 
  (make-instrument (pathname input)))


;;;===================================================

(defun delete-equal (string)
  (let ((pos  (position-if #'(lambda (x) (not (equal x #\=))) string)))
    (when pos
      (subseq string pos))))
     
(defun is-a-global? (string)
  (let ((substring (delete-spaces string))
        rep)
    (when substring
      (if (equal (elt substring 0) #\g)
        (let ((index (position-if #'delimator-char-p substring)))
          (when index
            (setf rep
                  (list (subseq substring 0 index) 
                        (om-correct-line (subseq substring index))))))))
    rep))


(defun is-a-channel? (string)
  (let ((substring (delete-spaces string))
        rep)
    (when substring
      (if (and (> (length substring) 6) (search "nchnls" substring :start2 0 :end2 7))
        (setf rep (read-from-string (subseq  (delete-spaces (subseq (delete-spaces (subseq substring 6)) 1)) 0 1)))
        ))
    rep))

(defun load-instrument-from-head (file name)
   (loop while (not (stream-eofp file)) do
         (let* ((rep (look-for-line-head file "instr"))
                (j (car rep))
                (globals (second rep))
                (chans (third rep)))
           (when (integerp j) (parse-csound-inst file j (pathname-name name) globals chans)))))

;;; !!!! file type !!!
(defun load-instruments (name)
   (cond
    ((string-equal (pathname-type name) "orc")
     (with-open-file (file name)
       ;;;(if (equal (mac-file-type file) :text)
         (load-instrument-from-head file name)
     ;;;  (om-beep-msg "Sorry only TEXT files can be parsed."))
     ))
    ((string-equal (pathname-type name) "csd")
     (with-open-file (file name)
       ;;;(if (equal (mac-file-type file) :text)
         (progn 
           (print (look-for-line file "<CsInstruments>"))
           (print 1)
           (load-instrument-from-head file name))
         ;;;(om-beep-msg "Sorry only TEXT files can be parsed."))
       ))
    (t  (om-beep-msg "Sorry only .ORC or .CSD files can be parsed."))))



(defun look-for-line (file name)
   (let ((rep nil) 
         (size (length name)))
     (loop while (and (not rep) (not (stream-eofp file))) do
           (let* ((newl (delete-spaces (read-line file))))
             (if (and (>= (length newl) size) (search name newl :start2 0 :end2 (length name)))
                 (setf rep t))))
     rep))



;search a line begining by "INST" (name)
(defun look-for-line-head (file name)
   (let ((rep nil) 
         (size (length name))
         globals chans)
     (loop while (and (not rep) (not (stream-eofp file))) do
           (let* ((newl (delete-spaces (read-line file)))
                  (global? (is-a-global? newl)))
             (unless chans (setf chans (is-a-channel? newl)))
             (cond 
              (global? (push global? globals))
              (t  
               (if (and (> (length newl) size) (search name newl :start2 0 :end2 (length name)))
                   (setf rep (read-from-string (subseq newl 5)))))))
           )
     (list rep globals chans)))


(defun parse-csound-inst (file instnumber namefile globals chans)
  (let ((oldpos (file-position file))
        (numins 0) end?)
    (loop while (and  (not end?) (not (stream-eofp file))) do
          (let ((tokem-rep (get-rep-char file 'valid "")))
            (cond
             ((integerp tokem-rep)
              (setf numins (max numins tokem-rep)))
             ((stringp tokem-rep)
              (def-csound-inst namefile instnumber (fill-linescode file oldpos) numins globals chans)
              (setf end? t)))))))

(defun get-rep-char (file etat str)
  (unless (stream-eofp file)
    (let ((newchar (om-correct-line (read-char file))))
      (cond
       ((equal etat 'e) (get-endin file newchar #\n 'en))
       ((equal etat 'en) (get-endin file newchar #\d 'end))
       ((equal etat 'end) (get-endin file newchar #\i 'endi))
       ((equal etat 'endi)
        (cond
         ((char-equal newchar #\n) "endin")
         ((delimator-char-p newchar) 
          (get-rep-char file 'valid ""))
         (t (get-rep-char file 'noway ""))))
       ((equal etat 'p)
        (cond
         ((digit-char-p newchar)
          (get-rep-char file 'p (string+ str (format () "~C" newchar))))
         ((or (delimator-char-p newchar)
              (char-equal newchar #\,) (char-equal newchar #\)))
          (read-from-string (string+ str (format () "~C" newchar))))
         (t (get-rep-char file 'noway ""))))
       ((equal etat 'noway)
        (loop while (and (not (stream-eofp file)) (not (delimator-char-p newchar))) do
              (setf newchar (read-char file)))
              (get-rep-char file 'valid ""))
       ((equal etat 'valid) 
        (cond
         ((char-equal newchar #\p) (get-rep-char file 'p ""))
         ((char-equal newchar #\e) (get-rep-char file 'e ""))
         ((delimator-char-p newchar)
          (get-rep-char file 'valid ""))
         (t (get-rep-char file 'noway ""))))
       ))))

(defun delimator-char-p (char)
  (or (char-equal char #\Space) 
      (char-equal char #\Newline) (char-equal char #\Tab) (char-equal char #\,) (char-equal char #\() (char-equal char #\))
      (char-equal char #\*) (char-equal char #\+) (char-equal char #\-) (char-equal char #\/) (char-equal char #\=)))

(defun get-endin (file newchar char next-etat)
  (cond
   ((char-equal newchar char) (get-rep-char file next-etat ""))
   ((delimator-char-p newchar) 
    (get-rep-char file 'valid ""))
   (t (get-rep-char file 'noway ""))))

(defun fill-linescode (file oldpos) 
  (file-position file oldpos)
  (let (rep final)
    (loop while (and (not final) (not (stream-eofp file))) do
          (let ((newl (om-correct-line (read-line file))))
            (if (string-equal newl "endin")
              (setf final t)
              (push newl rep))))
    (when final (reverse rep))))
                       
 
(defun def-csound-inst (name num lines numins globals chans)
   (setf chans (or chans 1))
   (let* ((newclassname (interne (format nil "~D~D" name num)))
          (exist? (find-class newclassname nil))  
          (define t) 
          (historique (format nil ";Instrument ~D from file ~D" num name))
          newins )
     (setf lines (cons historique lines))
     (when exist? 
       (setf define (y-or-n-dialog (format nil "The class ~D exists already, do you want redefine it ?" newclassname))))
     (when define
       (setf newins (eval  `(defclass* ,newclassname (CS-Evt)
                               ((source-code :initform nil :allocation :class :type textfile :accessor source-code)
                                (globals-list :initform nil :allocation :class :type list :accessor globals-list)
                                (numchan :initform ,chans :allocation :class  :accessor numchan)
                                ,.(loop for i from 4 to numins
                                        collect (list (interne (format nil "P~D" i))
                                                      :initform 0 :initarg (string2initarg  (format nil "P~D" i))
                                                      :type 'number
                                                      :accessor (interne (format nil "P~D" i))))
                                )
                               (:icon 601))))
       (when newins
         (let ((defval (get-super-default-value (class-name newins))))
           
           (setf (source-code defval) (load-buffer-textfile lines 'textfile "append"))
           
           (setf (globals-list defval) (reverse globals)))
         
         (resave-class newins)
         )
       (if newins
         (unless exist?
           (AddClass2Pack (class-name newins) *package-user* :protect nil)
           (om-beep-msg (format nil "A class called ~D~D  was defined." name num)))
         (om-beep-msg (format nil "Error defining a class called  ~D~D." name num))))))



;===============================================================
;Tables
;===============================================================

(defmethod csound-load-tables ((self null)) nil)

(defmethod csound-load-tables ((self textfile)) nil)

(defmethod csound-load-tables ((self list))
   (loop for item in self do 
         (csound-load-tables item)))

(defmethod csound-load-tables ((self string)) 
   (let ((rep (delete-spaces self)))
    (cond
     ((equal (elt rep 0) #\f) (SCsT self))
     (t (load (truename self))))))

(defmethod csound-load-tables ((self cs-table))
   (when (integerp (id self))
     (push self *globals-csound-tables*)))

(defmethod csound-load-tables ((self pathname)) (load self))

;===============================================================
;Building an orc file from an array
;===============================================================



(defmethod nullslots ((self class-array)) 0)


(defmethod csound-lines ((self SynthesisEvt))
   (let ((offset (+ (action-time self) (/ (offset self) 1000.0)))
         (rows (- (numrows self) (nullslots self)))
         (id (instid self)))
     (loop for i from 0 to (- (numcols self) 1) append 
           (csound-strings self (if (Parsing-fun self)
                                  (funcall (Parsing-fun self) self i)
                                  (list (aref (work-vector self) i))) rows offset id))))


(defun str-format (num)
  (format nil "i~~D ~~{~~,~DF ~~}" num))

(defmethod csound-strings ((self SynthesisEvt) (list list) rows offset id)
   (let ((format-str (str-format (car (lprecision self)))))
     (loop for item in list collect (real-csound-line item id rows offset format-str))))


(defmethod real-csound-line ((self component) id max offset str)
   (let ((list (first-n (val-list self) max)))
     (setf (nth 0 list) (+ (nth 0 list) offset))
     (format nil str id list)))


(defmethod real-csound-line ((self list) id max offset str)
   (let ((list (first-n self max)))
     (setf (nth 0 list) (+ (nth 0 list) offset))
     (format nil str id list)))

(defmethod real-csound-line ((self string) id max offset str) 
   (declare (ignore id max offset))
   (format nil ";~D" self))




;======================================



(defun unique-pathname (dir name &optional (ext ""))
  (let ((pathname (make-pathname :device (pathname-device dir) :directory (pathname-directory dir) :name name :type ext)))
    (loop while (probe-file pathname)
          for i = 1 then (+ i 1) do
          (setf pathname (make-pathname :device (pathname-device dir) :directory (pathname-directory dir) :name (string+ name (format nil "~D" i)) :type ext)))
    pathname))

(defun get-event-list (list)
  (remove-duplicates 
   (remove-if 'stringp list) 
    :test 'equal :key 'type-of)
  )


(defun dialogue-dup (globlist)
  (let* ((thedialog (om-make-window 'om-dialog  
                      :window-title "C-sound globals conflict" 
                      :size (om-make-point 300 248) :close nil :resizable nil :maximize nil))
         (scrol (om-make-view 'om-scroller
                  :owner thedialog
                  :size (om-make-point  280 200)
                  :scrollbars :v
                  :position (om-make-point  3 3))) tablelist)
    (loop for item in globlist
          for i = 0 then (+ 1 i) do
          (let ((table (om-make-dialog-item 'om-single-item-list (om-make-point 59 (+ 6 (* 52 i))) 
                                         (om-make-point  220 48) ""
                                         :font *om-default-font1*
                                         :range (loop for cr in item collect (cadr cr)))))
            (push table tablelist)
            (om-select-item-index table 0)
            (om-add-subviews scrol (om-make-dialog-item 'om-static-text
                                                  (om-make-point 5 (+ 18 (* 52 i))) (om-make-point  70 15)
                                                  (caar item)
                                                  :font *om-default-font1*) 
                          table)))
    (reverse tablelist)
    (om-add-subviews thedialog
      (om-make-dialog-item 'om-button (om-make-point 50 210) (om-make-point  70 24) "cancel" 
                        :di-action (om-dialog-item-act item
                                     (declare (ignore item)) 
                                     (om-return-from-modal-dialog thedialog ())))
      (om-make-dialog-item 'om-button (om-make-point 180 210) (om-make-point  70 24) "OK" 
                        :di-action (om-dialog-item-act item
                                     (declare (ignore item)) 
                                     (om-return-from-modal-dialog thedialog (loop for item in globlist
                                                                     for cel in tablelist
                                                                     collect (nth (om-get-selected-item-index cel) item)))) 
                        :default-button t ))
    (om-modal-dialog thedialog)))


    

(defun resoudre-duplicatas (globlist)
  (let (original)
    (setf globlist (remove-duplicates globlist :test 'equal))
    (setf original (copy-list globlist))
    (let (duplicatas)
      (loop while globlist do
            (let ((first (pop globlist)) dup )
              (loop for item in globlist do
                    (when (and (string-equal (car first) (car item))
                               (not (string-equal (second first) (second item))))
                      (push item dup)
                      (setf globlist (remove item globlist))))
              (when dup 
                (push first dup)
                (push dup duplicatas))))
      (setf original (reverse original))
      (when duplicatas
        (let ((finalista (dialogue-dup duplicatas)))
          (loop for item in finalista do
                (setf original (remove (car item) original :key 'car :test 'string-equal)))
          (setf original (append original finalista)))) 
      original)))



(defun write-globals-variables (file instlist)
  (let (globallist)
    (setf globallist (loop for item in instlist append (globals-list item)))
    (setf globallist (resoudre-duplicatas (copy-list globallist)))
    (format file ";Global variables~%~%")
    (loop for gvar in (reverse globallist) do
          (format file "~D  ~D~%" (car gvar) (second gvar)))
    (format file "~%~%;INSTRUMENTS~%~%")))


(defun write-macros (file instlist)
  (let ((maclist (loop for item in instlist append (macros-list item))))
    ;;;(setf globallist (resoudre-duplicatas (copy-list globallist)))
    (format file ";Macros~%~%")
    (loop for mac in maclist do
          (format file "~D ~%" mac))
    (format file "~%~%")
    ))

(defun replace-if-exist (name extlist)
  (let ((exist? nil)
        (rep t))
    (loop for item in extlist do
          (when (probe-file (make-pathname :directory (om-namestring *om-tmpfiles-folder*) 
                                           :name (string+ name item)))
            (push item exist?)))
    (if exist?
      ;;;(setf rep (y-or-n-dialog (format nil "Intermediate files ~D~D already exist, do you want replace them ?" name exist?)))
      (setf rep t))
    (when rep
      (loop for item in exist? do (delete-file (make-pathname :directory (om-namestring *om-tmpfiles-folder*) 
                                           :name (string+ name item)))))
    rep))


(defun all-channels-equal? (list)
  (let ((chan (numchan (car list)))
        (rep t))
  (loop for item in list 
        when rep do
        (setf rep (= chan (numchan item))))
  (if rep chan)))

(defmethod filtre-events-list ((ev string) funlist) t)

(defmethod filtre-events-list (ev funlist)
  (let ((rep t))
    (loop for fun in funlist
          while rep do
          (setf rep (funcall fun ev)))
    rep))

(defun filtre-events (list funlist) 
  (loop for item in list
        when (filtre-events-list item funlist) collect item))

(defun write-instrument-headers (file instlist)
  (loop for item in instlist do
        (loop for line in (orc-header item) do
              (write-line  line file)))
  (format file ";End of Instrument's Header~%~%"))



(defmethod cs-Synthesize ((csound-events t) &key  
                            tables
                            (name "my-synt") 
                            (sr 44100) (kr  4410) 
                            (nchnls  1) (rescale 0.0) (run t) (evt-test) (resolution 16))
  
  (unless (maq-obj-p csound-events)
    (setf csound-events (list! csound-events)))
  
  (let* ((elements (filtre-events (collect-synthesis-events csound-events) evt-test))
         (ksmps (/ sr kr))
         (instlist (get-event-list (copy-list elements)))
         (continue? (if *automatic-rename* t (replace-if-exist name '(".sco" ".orc"))))
         path-orc path-sco path-aiff aiff-temp
         rep)
    
    (when continue?
      
      (if *automatic-rename*
        (setf path-orc (unique-pathname *om-tmpfiles-folder* name "orc")
              path-sco (unique-pathname *om-tmpfiles-folder* name "sco")
              aiff-temp (unique-pathname *om-tmpfiles-folder* name "temp2"))
        (setf path-orc (make-pathname :device (pathname-device *om-tmpfiles-folder*) :directory (pathname-directory *om-tmpfiles-folder*) :name name :type "orc")
              path-sco (make-pathname :device (pathname-device *om-tmpfiles-folder*) :directory (pathname-directory *om-tmpfiles-folder*) :name name :type "sco")       
              aiff-temp (make-pathname :device (pathname-device *om-tmpfiles-folder*) :directory (pathname-directory *om-tmpfiles-folder*) :name name :type "temp2")))
      (if *automatic-rename*
        (setf path-aiff (unique-pathname (om-namestring *om-outfiles-folder*) name "aiff"))
        (setf path-aiff (make-pathname :device (pathname-device *om-outfiles-folder*) :directory (pathname-directory *om-outfiles-folder*) :name name :type "aiff"))
        )
      (unless elements (om-beep-msg "No synthesis events were found."))
      (when elements
        (when *verbose-matrix*
          (print "Start synthesis"))
        (setf *globals-csound-tables* nil)
        (csound-load-tables (list! tables))
        (if (setf nchnls (all-channels-equal? instlist))
           (progn
             (loop for item in instlist
                   for i = 1 then (+ i 1) do
                   (setf (instID  item) i)) 
             (WITH-OPEN-FILE (out path-orc :direction :output  :if-does-not-exist :create :if-exists :supersede)
               (format out ";Header ~%~%sr = ~D~%kr = ~D~%ksmps = ~D~%nchnls = ~D~%~%~%~%" sr kr ksmps nchnls)
               
               (write-macros out instlist)
               (write-globals-variables out instlist)
               
               (loop for item in instlist
                     for i = 1 then (+ i 1) do
                     (setf (instID  item) i))
               (loop for item in instlist 
                     for i = 1 then (+ i 1) do
                     (format out ";==============~%;==============~%instr ~D~%" i)
                     ;;; !!!!
                     (loop for lin in (list-of-lines (buffer-text (source-code item))) do
                           (write-line lin out))
                     ;;; !!!!
                     (format out "endin~%~%")))
             (WITH-OPEN-FILE (out path-sco :direction :output :if-does-not-exist :create :if-exists :supersede)
               (format out ";This synthesis process called ~D started on " name)
               (printdate out)
               (format out "~%~%")
               (format out ";Global Variables: sr = ~D, kr = ~D, ksmps = ~D, nchnls = ~D~%~%~%" sr kr ksmps nchnls)
               (format out ";Instrument's Header~%")
               (write-instrument-headers out instlist)
               (format out "~%;Loaded tables~%")
               (loop for item in *globals-csound-tables* do
                     (format out "~D~%" (cs-table-string item)))
               (format out "~%;Generated tables~%~%")
               
               (setf *table-counter* 0)
               (loop for item in elements
                     for k = 1 then (+ k 1) do
                     (if (stringp item)
                       (write-line item out)
                       (progn
                         (when (and *verbose-matrix* (>= *verbose-matrix* 1)
                                    (zerop (second (multiple-value-list (floor k *verbose-matrix* )))))
                         (print (format nil "------ Matrix n. ~D --------" k)))
                         (clrhash *dynamic-cs-tbale-table*)
                         
                         (cons-work-vector item)
                         (format out "~%;------ Lines for the event n. ~D --------~%~%" k)
                         
                     (maphash #'(lambda (key val) (declare (ignore key))
                                 (write-line val out)) *dynamic-cs-tbale-table*)
                     
                     (loop for lin in (csound-lines item)
                           for l = 1 then (+ l 1) do
                           (when (and *verbose-matrix* (<= *verbose-matrix* -1)
                                      (zerop (second (multiple-value-list (floor l *verbose-matrix* )))))
                             (print (format nil "------ Line n. ~D --------" l)))
                           (write-line lin out))
                     (free-work-vector item)
                     
                     (clrhash *dynamic-cs-tbale-table*)
                     )))
               (format out "~%~%"))
             (when run
               (print (format nil "Running CSound... writing ~D" (om-path2cmdpath (om-namestring aiff-temp)))) 

           
           (om-cmd-line
            (print (string+ "'" (om-path2cmdpath *CSOUND-PATH*) "' "
                     *csound-defflags* " '" (om-path2cmdpath path-orc) "' '" (om-path2cmdpath path-sco)
                     "' -o '" (om-path2cmdpath (om-namestring aiff-temp))  "'" 
                     ))
            t)
               
               (print "CSOUND Synthesis OK")
               
           (if (probe-file (om-namestring aiff-temp))
               (if (probe-file *SVP-PATH*)
                   (progn
                     (print (format nil "Rescaling ~D to ~D dB and converting to ~D bits" (om-path2cmdpath (om-namestring aiff-temp)) rescale resolution))
                     (om-cmd-line (print (format nil "'~D' -S~S -norm ~D -Osa~D ~S" 
                                           (om-path2cmdpath *SVP-PATH*)
                                           (om-path2cmdpath (om-namestring aiff-temp)) 
                                           (* -1 rescale) resolution
                                           (om-path2cmdpath path-aiff))))
                     (if (probe-file path-aiff)
                         (setf rep path-aiff)
                       (progn 
                         (print "Error in SVP processing.")
                         (setf rep nil))
                       ))
                 (progn
                   (print "SuperVP unavailable...")
                   (om-copy-file aiff-temp path-aiff)
                   (when (probe-file path-aiff) (setf rep path-aiff))
                   (print (list "REP :::::::::::: " rep))
                   ))
             (progn
               (om-message-dialog "Error in CSound syntheis.")
               (setf rep nil)))
           
           
           ;(when *delete-inter-file*
           ;  (delete-intermediates (list path-sco path-orc aiff-temp)))
           
           (when *verbose-matrix*
             (print "End synthesis")))
             (print (om-namestring path-aiff)))
           (progn
             (om-beep-msg "all instruments must have the same number of channels")
             (setf rep nil)))
         rep
         ))))



(defmethod! Synthesize ((synthetiser (eql 'csound)) elements &key
                          (name "my_synt")
                          (sr 44100)
                          (rescale 0.0)
                          (run t) (evt-test  nil) (resolution 16)
                          ;For Csound
                          (kr  4410) tables (nchnls  1)
                          ;For Chant
                          (Patch 0) sound  duration)
   :doc "Call the selected synthesizer passing it the data in its arguments."
   ;;;(verify-def-folders)
   (cs-Synthesize elements :name name :sr sr :kr kr :tables tables :nchnls nchnls :rescale rescale :run run :evt-test evt-test :resolution resolution))


;======================
(defmethod component-p ((self component)) t)
(defmethod component-p ((self t)) nil)


(defmethod! general-parsing ((self component) (predicates list) (modifiers list) &key (remove-duplicatas t) (remove-fields nil))
   :initvals '(nil nil nil t)
   (let ((comp-list nil)
         (current-comp self))
     (loop for item in predicates
           while (component-p current-comp) do
           (let ((temp-rep (funcall item current-comp)))
             (cond
              ((component-p temp-rep) 
               (setf current-comp temp-rep))
              ((stringp temp-rep)
               (setf comp-list (append comp-list (list temp-rep)))
               (setf current-comp nil))
              ((listp  temp-rep)
               (setf current-comp (car temp-rep))
               (setf comp-list (append comp-list (cdr temp-rep))))
              (t (setf current-comp nil)))))
     (when current-comp
       (loop for item in modifiers do
             (setf comp-list (append comp-list (list! (funcall item current-comp))))))
     (when remove-duplicatas
       (setf comp-list (remove-duplicates comp-list :test 'equal)))
     (make-remove-fields comp-list remove-fields)
     comp-list))


;=======================
