(in-package :om)


;;;===============================================================
;;; CR-CONTROL
;;; a structure for the specification of a matching between the CR-MODEL data with some classes of class-array
(defclass! cr-control ()
  ((ctl :accessor ctl :initform (make-instance 'ctrl)))
  (:icon 655))

(defclass ctrl (ompatchabs) ())

(defmethod initialize-instance :after ((self cr-control) &rest initargs &key from-file) 
  (when from-file
    (omng-add-element (ctl self) (make-modeldatabox "MODEL DATA" (om-make-point 100 40)))))


(defmethod omNG-copy ((self cr-control))
  (let ((obj (call-next-method)))
     `(let ((copy ,obj))
        (setf (ctl copy) ,(omng-copy (ctl self)))
        (setf (compiled? (ctl copy)) nil)
        (compile-patch (ctl copy))
        copy)))

(defmethod omNG-save ((self cr-control) &optional (values? nil))
  (let ((obj (call-next-method)))
     `(let ((ctrl ,obj))
        (setf (ctl ctrl) ,(om-save (ctl self)))
        (change-class (ctl ctrl) (find-class 'ctrl))
        (setf (compiled? (ctl ctrl)) nil)
        (compile-patch (ctl ctrl))
        ctrl)))



;;;;--------------------------
;;; BOX & EDITOR

(defmethod get-type-of-ed-box ((self cr-control))  'ctrleditbox)
(defclass ctrleditbox (OMBoxEditCall) ())

(defmethod omNG-box-value ((self ctrleditbox) &optional (numout 0))
  (value self))

(defmethod OpenObjectEditor ((self ctrleditbox))
  (openobjecteditor (ctl (value self)))) 
  
(defmethod Class-has-editor-p ((self cr-control)) t)
(defmethod get-editor-class ((self ctrl)) 'ctrlEditor)

(omg-defclass ctrlEditor (patchEditor)  ())
(defmethod get-editor-panel-class ((self ctrlEditor))  'ctrlPanel)

(omg-defclass ctrlPanel (patchPanel) ())

(defmethod EditorFrame ((box ctrleditbox)) (editorframe (ctl (value box))))

;;;;--------------------------
;;; OUTSLOTS
(defclass outslot (omout) ())
(omg-defclass outslotframe (outFrame) ())
(defmethod get-frame-class ((self outslot)) 'outslotframe)
(defmethod get-icon-box-class ((self outslot)) 'icon-box)

(defmethod draw-in-inout-box ((container t) object)
  (om-with-focused-view container
    (om-with-font *om-default-font4b*
                  (om-draw-string (- (round (w container) 2) 5)  26 (format () "~D" (indice object))))
    ))

(defmethod omG-rename ((self outslotframe) new-name)
  (call-next-method)
  (omng-rename (object self) new-name))

(defmethod omNG-save ((self outslot) &optional (values? nil))
  (let* ((inputs (mapcar #'(lambda (input) (omNG-save input values?)) (inputs self))))
    `(om-load-slotout ,(name self) ,(indice self)  ,(om-save-point (frame-position self))
                      ',inputs ,(frame-name self) ,(om-save-point (frame-size self)))))

(defun om-load-slotout (name indice position inputs &optional fname fsize)
  (let ((newbox (make-new-slotout name indice (om-correct-point position))))
    (setf (frame-name newbox) fname)
    (setf (inputs newbox) (mapcar #'(lambda (input) (eval input)) inputs))
    (when fsize
      (setf (frame-size newbox) (om-correct-point fsize)))
    newbox))

(defun make-new-slotout (name indice posi)
  (let* ((icon 665)
         (theout (make-instance 'outslot
                   :name name
                   :icon icon
                   :reference nil
                   :indice indice)))
    (setf (frame-position theout) posi)
    (setf (inputs theout) (list (make-instance 'input-funbox
                                  :name "out"
                                  :value nil
                                  :doc-string "out")))
    theout))

(defmethod add-window-buttons  ((self ctrlPanel))
  (om-add-subviews self 
                   (om-make-view 'button-icon
                                            :iconID 665
                                            :position (om-make-point 5 5)
                                            :size (om-make-point 24 24)
                                            :action
                                            #'(lambda(item) (declare (ignore item))
                                               (let* ((boxes (get-subframes self)) 
                                                      (i (length (list+ (find-class-boxes boxes 'tempOutFrame) 
                                                                        (find-class-boxes boxes 'outSlotFrame)
                                                                        (find-class-boxes boxes 'outFrame))))
                                                      (pos (om-make-point (+ 5 (* i 50)) 240)))
                                                 (omG-add-element self
                                                                  (make-frame-from-callobj 
                                                                   (make-new-output (mk-unique-name self "SLOT")
                                                                                    i pos
                                                                                    665
                                                                                    'outslot)))
                                                 (set-field-size self)
                                                 )))
    #|
(om-make-view 'button-icon
       :iconID (list 156 (get-csound-lib))
       :position (om-make-point 60 5)
       :size (om-make-point 24 24)
       :action
       #'(lambda(item) (declare (ignore item)) 
          (omG-add-element self
                           (make-frame-from-callobj 
                            (omNG-make-new-boxcall (fdefinition 'assign-val)
                                                   (om-make-point 50 80)
                                                   (mk-unique-name self "assign-val"))))))
|#
    ))


(defmethod gen-code ((self outslot) numout)
  `(list ,(name self) ,(call-next-method)))
   
;;;;--------------------------
;;; MODELDATA
;;; une boite vrtuelle pour recuperer les donnees du modele

(defclass! virtual-modeldata () 
  ((global-model :accessor global-model :initform nil)
   (local-time :accessor local-time :initarg :local-time :initform nil)
   (local-dur :accessor local-dur :initarg :local-dur :initform nil)
   (local-rank :accessor local-rank :initarg :local-rank :initform nil)
   (local-data :accessor local-data :initarg :local-data :initform nil))
  )

(defclass modeldatabox (OMTypedIn) 
  ((in-symbol :initform nil :initarg :in-symbol :accessor in-symbol)))

(defmethod OpenEditorframe ((self modeldatabox)) (not (dialog-message "This is a vrtual model accessor. It has no real value")))

(omg-defclass modeldataframe (TypedInFrame) ())
(defmethod get-frame-class ((self modeldatabox)) 'modeldataframe)

(defmethod draw-typed-special ((self modeldataframe)) t)

(defun make-modeldatabox (name posi)
  (let ((rep (make-instance 'modeldatabox
                :name name
                :icon 667
                :reference 'virtual-modeldata
                :indice 0)))
     (setf (frame-position rep) posi)
     (setf (defval rep) (make-instance 'virtual-modeldata))
     rep))

(defmethod omg-remove-element ((self patchPanel) (box modeldataframe)) nil)

(defmethod omNG-copy ((self modeldatabox))
  `(let* ((thein (make-instance ',(class-name (class-of self))
                   :name ,(name self)
                   :icon ,(copy-icon (icon self))
                   :reference 'virtual-modeldata
                   :indice 0)))
          (setf (frame-position thein) ,(om-copy-point (frame-position self)))
          (setf (frame-size thein) ,(om-copy-point (frame-size self)))
          (setf (defval thein) (make-instance 'virtual-modeldata))
     thein))

(defmethod omNG-save ((self modeldatabox) &optional (values? nil))
  (declare (ignore values?))
  `(om-load-modeldatabox  ,(name self)  ,(om-save-point (frame-position self)) ,(om-save-point (frame-size self))))

(defun om-load-modeldatabox (name  position  &optional fsize)
  (let ((newbox (make-modeldatabox name (om-correct-point position))))
    (when fsize
      (setf (frame-size newbox) (om-correct-point fsize)))
    (setf (defval newbox) (make-instance 'virtual-modeldata))
    newbox))

(defmethod gen-code ((self modeldatabox) numout)
   (if (= numout 0)
     `(global-model ,(in-symbol self))
     `(,(internp (nth numout (get-outs-name (defval self))) (symbol-package (reference self))) ,(in-symbol self))))

(defmethod rep-editor ((self virtual-modeldata) num)
   (let ((outs (get-outs-name self)))
     (if (= num 0)
       (global-model self)
       (eval `(,(internp (nth num outs) (symbol-package (type-of self))) ,self)))))


(defmethod compile-patch ((self ctrl)) 
  "Generation of lisp code from the graphic boxes."
  (unless (compiled? self)
    ;; (print "compiling control")
    (let* ((boxes (boxes self))
             (out-slots (find-class-boxes boxes 'outslot))
             (in-boxes (find-class-boxes boxes 'modeldatabox))
             (out-symb (code self))
             (oldletlist *let-list*) symbols body)
        (setf symbols (mapcar #'(lambda (thein) (setf (in-symbol thein) (gensym))) in-boxes))
        (setf *let-list* nil)
        (setf body `(values ,.(mapcar #'(lambda (theout)
                                          (gen-code theout 0)) out-slots)))
        (eval  `(defun ,(intern (string out-symb) :om)  (,.symbols)
                  (let* ,*let-list* ,body)))
        
        (setf *let-list* oldletlist))
    (setf (compiled? self) t)
    ))

;;;------------------------
;;; MAKE MATRICES
;;;------------------------
(defmethod! expand-model ((my-model cr-model) ctl-model (target-class symbol))
  :icon 655
  (expand-model my-model ctl-model (make-instance target-class)))

(defmethod! expand-model ((my-model cr-model) (ctl-model cr-control) (target-class SynthesisEvt))
  :icon 655
  (let ((n (length (data my-model))))
    (compile-patch (ctl ctl-model))
    (remove nil 
            (loop for i from 0 to (- n 1)
                  collect (make-ctl-synthesis-event ctl-model target-class i my-model)
                  ))
    ))

(defmethod! expand-model ((my-model cr-model) (ctl-model null) (target-class SynthesisEvt))
  :icon 655
  (let ((n (length (data my-model))))
    (remove nil 
            (loop for i from 0 to (- n 1)
                  collect (make-simple-synthesis-event target-class (nth i (time-struct my-model)) (nth i (data my-model)))))
    ))


(defmethod! expand-model ((my-model cr-model) (ctl-model textfile) (target-class SynthesisEvt))
  :icon 655
  (expand-model my-model (exp-list ctl-model) target-class))


;;; Chroma-style
(defmethod! expand-model ((my-model cr-model) (ctl-model list) (target-class SynthesisEvt))
  :icon 655
  (let ((n (length (data my-model))))
    (remove nil 
            (cr::make-ctl2-list my-model ctl-model target-class))))


(in-package chroma)

(defmethod make-ctl2-list ((my-model om::cr-model) ctl-model target-class
                           &key (outfile "yaka-out") (args-yaka-user nil)
                           (by-time nil))     
  (setf outfile  (make-pathname :directory (pathname-directory (cr::getenv cr::CROUTifls))
                                :name outfile :type "ctl2")) 
  (let ((n-fql (1- (om::nb-evts my-model)))      ; n-fql - 1 , nmarkers - 2
        resc-min resc-max gblamp-val )
    ; ? si outfile nil : stream pas vers fichier mais vers "load" ?
    (with-open-file  (outstream outfile :direction :output :if-exists :supersede)
      (format  outstream "(in-package :chroma)(defun cr::ctl2-result() (list ~%")
      (if by-time
        (setf resc-min (car (om::time-struct my-model)) resc-max (last-elem (om::time-struct my-model)))
        (setf resc-min 0 resc-max n-fql))
      ; FQL LOOP     
      (loop for i from 0 to n-fql
            do (cr::om-CTL2 target-class my-model (cr-interne-list ctl-model :chroma) i outstream args-yaka-user))
      (format outstream "))~%"))
    (load outfile))
  (cr::ctl2-result))

;; bof bof
(defmethod chroma::offset ((self om::cr-model)) 0)


(defun cr-interne-list (list package)
  (loop for elt in list 
        ;do (print elt)
        collect
        (cond ((listp elt) (cr-interne-list elt package))
              ((symbolp elt) (intern (symbol-name elt) package))
              (t elt))))

(defmethod om-CTL2 ((self om::SynthesisEvt) (my-model om::cr-model) ctl-model my-rank outstream 
                    args-yaka-user)
  (declare (special outstream my-model my-rank))
  ;(print ctl-model)
  (let*((my-time (nth my-rank (om::time-struct my-model)))
        (my-dur (- (nth (+ 1 my-rank) (om::time-struct my-model)) (nth my-rank (om::time-struct my-model))))
        (n-fql (1- (om::nb-evts my-model)))      ; n-fql - 1 , nmarkers - 2
        (my-fql (nth my-rank (om::data my-model)))
        (my-ptl my-fql)
        (my-nev (length (om::get-vps-freqs my-fql)))
        string1)
    (declare (special n-fql my-time my-dur my-nev my-fql my-ptl))
    (when my-fql
      (when args-yaka-user (apply #'yaka-user args-yaka-user))
      (format outstream"(om::mk-array 'om::~a ~a ~%~%"
              (type-of self) my-nev) 
      (if (cr::get-gbl 'cr::ctl2-print) (format t "~a~%" my-time))
      ; A FAIRE : GLOBALSLOTS loop sur (first (multiple-value-list (om::get-slot-in-out-names self)))
      (cr::CTL2_global self ctl-model)
      ; KEYWORDS LOOP
      (cr::CTL2_keywords_loop self ctl-model)
      (format  outstream ") ~%~%")
      ))
  )


(in-package :om)

(defmethod make-simple-synthesis-event (target-class onset data)
  (when data 
    (let ((rep (make-instance (class-name (class-of target-class))
                 :numcols (length (get-vps-freqs data)))))
      (setf (action-time rep) onset)
      (setf (freq rep) (get-vps-freqs data))
      (setf (amp rep) (get-vps-amps data))
      rep)))


;;; text ctl
(defmethod make-ctl-synthesis-event ((ctl list) target-class i model)
  (let* ((t1 (nth i (time-struct model)))
         (t2 (nth (+ i 1) (time-struct model)))
         (data (nth i (data model)))
        (rep (make-instance (class-name (class-of target-class))
               :numcols (length (get-vps-freqs data)))))
      (setf (action-time rep) t1)
      rep))
    
;;; visual ctl
(defmethod make-ctl-synthesis-event ((cr-ctrl cr-control) target-class i model)
  (let* ((rep nil)
         (t1 (nth i (time-struct model)))
         (t2 (nth (+ i 1) (time-struct model)))
         (ncols nil)
         (fun nil)
         (data (nth i (data model)))
         (modeldata (make-instance 'virtual-modeldata :local-time t1 
                                   :local-dur (- t2 t1)
                                   :local-data data
                                   :local-rank i))
         (realslots (mapcar 'name (get-all-initargs-of-class (type-of target-class))))
         slotslist
         argslist)
    (setf (global-model modeldata) model) 
    (setf slotslist (multiple-value-list (apply (intern (string (code (ctl cr-ctrl))) :om) (list modeldata))))
    
    ;(print slotslist)

    (let ((pos (position "action-time" slotslist :key 'car :test 'string-equal)))
      (when pos 
        (setf t1 (cadr (nth pos slotslist)))
        (setf slotslist (append (first-n slotslist pos) (nthcdr (+ pos 1) slotslist)))))
    (let ((pos (position "numcols" slotslist :key 'car :test 'string-equal)))
      (when pos 
          (setf ncols (cadr (nth pos slotslist)))
          (setf slotslist (append (first-n slotslist pos) (nthcdr (+ pos 1) slotslist)))))
    (let ((pos (position "user-fun" slotslist :key 'car :test 'string-equal)))
      (when pos 
        (setf fun (cadr (nth pos slotslist)))
        (setf slotslist (append (first-n slotslist pos) (nthcdr (+ pos 1) slotslist)))))
    
         
    (loop for oneslot in slotslist do
          (when (member (car oneslot) realslots :test 'string-equal)
            (setf argslist (append argslist (list (string2initarg (car oneslot)) (cadr oneslot))))
            ;(setf (slot-value rep (intern (car oneslot) :om)) (cadr oneslot))
            ))
    (setf rep (apply 'mk-array (append 
                                (list (class-name (class-of target-class)) 
                                      (or ncols (max 1 (length (get-vps-freqs data))))
                                      (list t1 fun))
                                argslist)))
    rep
    ))


;;;;-------------------------------------------
; Une methode pour interpoler entre deux trucs

(defmethod! interpol-value ((list1 list) (list2 list) (nbsteps integer) i &optional profil)
  :icon 657
  (nth i (interpole-ypoints list1 list2 nbsteps profil)))



;;;;------------------------------------------------------
; Compat chroma CTL2 : des methodes d'inspection du modle

(defmethod cr::get-norm-amp ((self cr-model) rang &optional (scaling-factor 1))
  (let ((thefql (nth rang (data self))))
    (if thefql
      (mapcar #'(lambda (y) (/ y (/ (max-amp self) scaling-factor ))) (get-vps-amps thefql))
      nil)))

(defmethod cr::begin-time ((self cr-model))
  (car (time-struct self)))

(defmethod cr::get-nth-time ((self cr-model) i)
  (nth i (time-struct self)))

