(in-package :om)


;;;==================
;;; UTILS
(defmethod! pch->fq (note &optional diapason)
  :icon 141
  (if diapason
    (cr::pch->fq note diapason)
    (cr::pch->fq note)))

(defmethod! pch->fq (note &optional diapason)
  :icon 141
  (if diapason
    (cr::pch->fq note diapason)
    (cr::pch->fq note)))

(defmethod! make-cr-fun ((self list))
  :icon 657
  :initvals '((0 0 1 1))
  (cr::make_fun self))

(defmethod! make-cr-fun ((self bpf))
  :icon 657
  (let ((list (loop for x in (x-points self)
                    for y in (y-points self)
                    append (list y x))))
    (cr::make_fun list)))

(defmethod! fun-to-bpf ((self list))
  :icon 657
  (simple-bpf-from-list (cr::x-list_fun self) (cr::y-list_fun self) 'bpf 3))

(defmethod! add-random ((self list) rate)
  :icon 656
  :indoc '("a list of values" "a random rate (0.0->1.0)")
  :initvals '(nil 0.1)
  (mapcar (lambda (x) (cr::ran x (* x rate))) self))


;;;==================
;;; MODEL INSPECT

(defmethod! max-amp ((self list))
  :icon 659
  (list-max (mapcar 'list-max (remove nil (mapcar 'get-vps-amps (remove nil self))))))

(defmethod! max-amp ((self cr-model))
  :icon 659
  (max-amp (data self)))

(defmethod! max-freq ((self list))
  :icon 659
  (list-max (mapcar 'list-max (remove nil (mapcar 'get-vps-freqs (remove nil self))))))

(defmethod! max-freq ((self cr-model))
  :icon 659
  (max-freq (data self)))

(defmethod! min-freq ((self list))
  :icon 659
  (list-min (mapcar 'list-min (remove nil (mapcar 'get-vps-freqs (remove nil self))))))

(defmethod! min-freq ((self cr-model))
  :icon 659
  (min-freq (data self)))

(defmethod! nb-evts ((self cr-model))
  :icon 659
  (length (data self)))

;;;===============================================================
;;; MODEL PEROCESSING
;;; 
;;; BIBLIOTHEQUE DE FONCTIONS POUR LE TRAITEMENT DES MODELES
;;; i.e. s'appliquent sur les objets vps

(defmethod! high-pass-filter ((self cr::vps) freq)
  :icon 656
  :indoc '("model data" "filter freq (Hz)")
  :initvals '(nil 800.0)
  (cr::high-pass-filter self freq))

(defmethod! low-pass-filter ((self cr::vps) freq)
  :icon 656
  :indoc '("model data" "filter freq (Hz)")
  :initvals '(nil 800.0)
  (cr::low-pass-filter self freq))

(defmethod! band-pass-filter ((self cr::vps) freqlist)
  :icon 656
  :indoc '("model data" "list of freqs (Hz)")
  :initvals '(nil (400.0 800.0))
  (cr::band-pass-filter self (list-min freqlist) (list-max freqlist)))
  

(defmethod! main-partials ((self cr::vps) threshold)
  :icon 656
  :indoc '("model data" "threshold amp")
  :initvals '(nil 0.01)
  (cr::main-partials self :threshold threshold))
  
(defmethod! n-main-partials ((self cr::vps) n)
  :icon 656
  :indoc '("model data" "number of partials")
  :initvals '(nil 20)
  (cr::main-partials self :max-nn n))

(defmethod! stretch-vps ((self cr::vps) fact)
  :icon 656
  :indoc '("model data" "stretching factor")
  :initvals '(nil 2)
  (cr::stretch_vps self :stretching fact))

(defmethod! pitch-transpose ((self cr::vps) ratio)
  :icon 656
  :indoc '("model data" "stretching factor")
  :initvals '(nil 1)
  (cr::transpose self ratio))


;;;==============================
;;; TEMPORAL PROCESSING

(defmethod! time-map-fun ((markers list) fun)
  :icon 658
  (let ((fun2 (copy-list fun))
        (markers2 (copy-list markers))
        new-durs)
    (cr::x-resc_fun fun2 (first markers2) (car (last markers2)))
    (setf new-durs (mapcar
                    (lambda (time dur) (* dur (cr::y-val_fun fun2 time)))
                    markers2 (cr::markerstodur markers2)))
    (cr::durtotime (first markers2) new-durs)))