(in-package :om)


(defvar *CSOUND-PATH* "path to Csound folder")

(defun init-csound ()
  (setf *CSOUND-PATH* (om-external-app "csound")))

(om-add-init-func 'init-csound)




(defvar *automatic-rename* nil)
(defvar *auto-init-table-id* 500)
(defvar *csound-defflags* "-m7 -A -e -b8192 -B8192 -V50 -P128")
(setf *csound-defflags* "-f -m7 -A -N -g -b8192 -B8192")
(defvar *bpf-cs-coherce* 'gen07)
(defvar *def-table-size* 513)
(defvar *def-table-str*  "f 1 0 513 7  0 256 1 256 0")
(defvar *delete-inter-file*  t)
(defvar *verbose-matrix*  1)


(defvar *default-cs-pref-list*
   (list nil nil nil 
         1000 513 1 nil t
         "f1 0 513 7 0 256 1 256 0" "-f -m7 -A -N -g -b8192 -B8192"
         (om-external-app "csound")))

; orc-patch snd-path tablepath == om-interfiles om-outfiless om-infiles
;(nil nil nil strattables tablesize verbose? autorename? delete-interfiles? deftable defperfparams)

(defmethod get-external-def-vals ((module (eql 'csound))) *default-cs-pref-list*)

(defmethod save-external-prefs ((module (eql 'csound))) 
  `(list nil nil nil
        ,*auto-init-table-id*
        ,*def-table-size*
        ,*verbose-matrix* 
        ,*automatic-rename* 
        ,*delete-inter-file* 
        ,*def-table-str* 
        ,*csound-defflags*
        ,*CSOUND-PATH*))

(defmethod put-external-preferences ((module (eql 'csound)) list-prefs)
   (setf *auto-init-table-id* (nth 3 list-prefs)
         *def-table-size* (nth 4 list-prefs)
         *verbose-matrix* (nth 5 list-prefs)
         *automatic-rename* (nth 6 list-prefs)
         *delete-inter-file* (nth 7 list-prefs)
         *def-table-str* (nth 8 list-prefs)
         *csound-defflags* (nth 9 list-prefs)
         *CSOUND-PATH* (nth 10 list-prefs)))


(defmethod show-external-prefs-dialog ((module (eql 'csound)) prefs)
  (let* ((rep-list (make-list 11))
         ;(rep-list (copy-list prefs))
         (dialog (om-make-window 'om-dialog
                                 :window-title "Csound options"
                                 :size (om-make-point 280 370)
                                 :position :centered
                                 :resizable nil :maximize nil :close nil))
         (pos 20)

         (pathlabel (om-make-dialog-item 'om-static-text  (om-make-point 10 pos) (om-make-point 120 15) "Csound exe Location :" :font *om-default-font2b*))
         (pos (+ pos 20))
         (pathtext (om-make-dialog-item 'om-static-text  (om-make-point 10 pos) (om-make-point 200 45) (om-namestring *CSOUND-PATH*)
                                           :font *om-default-font1*))
         (pathbutton (om-make-dialog-item 'om-button (om-make-point 200 pos) (om-make-point 60 15) "Browse" 
                              :di-action (om-dialog-item-act item
                                           (declare (ignore item))
                                           (let ((newpath (om-choose-file-dialog)))
                                             (when newpath 
                                               (om-set-dialog-item-text pathtext (om-namestring newpath))
                                               )))))
         (pos (+ pos 40))
         (flaglabel (om-make-dialog-item 'om-static-text  
                                          (om-make-point 20 pos)  (om-make-point 147 17) "Csound default flags"
                                          ;:bg-color *om-window-def-color*
                                          :font *om-default-font2*))
         (pos (+ pos 20))
         (defflagsline (om-make-dialog-item 'om-editable-text
                                         (om-make-point 20 pos) (om-get-text-edit-size (om-make-point 240 16)) *csound-defflags* ;;(nth 9 prefs) 
                                         :font *om-default-font2*))
         (pos (+ pos 40))
         (pasverbose (om-make-dialog-item 'om-editable-text  
                                          (om-make-point 134 pos) (om-get-text-edit-size (om-make-point 26 13)) 
                                          (if *verbose-matrix* ;;(nth 5 prefs) 
                                            (format nil "~D" *verbose-matrix*) ;;(nth 5 prefs)) 
                                            "1") 
                                          :enable *verbose-matrix* ;;(nth 5 prefs)
                                          :font *om-default-font2*))
        
         (verbosemat (om-make-dialog-item 'om-check-box  
                                          (om-make-point 14 pos) (om-make-point 120 20) "verbose matrix" 
                                          :di-action (om-dialog-item-act item 
                                                       (om-enable-dialog-item pasverbose (om-checked-p item)))
                                          :font *om-default-font2*
                                          ;:bg-color *om-window-def-color*
                                          :checked-p *verbose-matrix* ;;(nth 5 prefs)
                                          ))
         (pos (+ pos 20))
         (autorename (om-make-dialog-item 'om-check-box  
                                          (om-make-point 14 pos) (om-make-point 137 20)  "Automatic rename" 
                                          :font *om-default-font2*
                                          ;:bg-color *om-window-def-color*
                                          :checked-p *automatic-rename* ;;(nth 6 prefs)
                                          ))
         (pos (+ pos 20))
         (intermediates (om-make-dialog-item 'om-check-box  
                                             (om-make-point 14 pos) (om-make-point 200 20) (format nil "Delete intermediate files") 
                                             :font *om-default-font2*
                                             ;:bg-color *om-window-def-color*
                                             :checked-p *delete-inter-file* ;;(nth 7 prefs)
                                             ))
         (pos (+ pos 30))
         (tablelabel (om-make-dialog-item 'om-static-text  
                                          (om-make-point 20 pos) (om-make-point 147 17) "Default table string"
                                          ;:bg-color *om-window-def-color*
                                          :font *om-default-font2*))
         (pos (+ pos 20))
         (deftablestr (om-make-dialog-item 'om-editable-text  
                                           (om-make-point 20 pos) (om-get-text-edit-size (om-make-point 240 16)) *def-table-str* ;;(nth 8 prefs) 
                                           :font *om-default-font2*))
         (pos (+ pos 30))
         (startlabel (om-make-dialog-item 'om-static-text  
                                          (om-make-point 15 pos) (om-make-point 174 18) "Automatic tables start"
                                          ;:bg-color *om-window-def-color*
                                          :font *om-default-font2*))

         (tablestart (om-make-dialog-item 'om-editable-text  
                                          (om-make-point 200 pos) (om-get-text-edit-size (om-make-point 37 13)) 
                                          (format nil "~D" *auto-init-table-id*) ;;(nth 3 prefs)) 
                                          :font *om-default-font2*))
         (pos (+ pos 20))
         (sizelabel (om-make-dialog-item 'om-static-text  
                                          (om-make-point 16 pos) (om-make-point 133 19) "Default table size"
                                          ;:bg-color *om-window-def-color*
                                          :font *om-default-font2*))
         (tablesize (om-make-dialog-item 'om-editable-text  
                                         (om-make-point 200 pos) (om-get-text-edit-size (om-make-point 37 16)) 
                                         (format nil "~D" *def-table-size*)  ;;(nth 4 prefs)) 
                                         :font *om-default-font2*))
         )
    (setf pos (+ pos 50))
    (om-add-subviews dialog
                     pathlabel pathtext pathbutton
                     defflagsline autorename  verbosemat deftablestr pasverbose
                     intermediates tablestart tablesize
                     flaglabel tablelabel startlabel sizelabel
                     
      ;;; boutons
      (om-make-dialog-item 'om-button (om-make-point 20 pos) (om-make-point 80 20) "OK"
                           :di-action (om-dialog-item-act item
                                        (let ((argerror nil)
                                              (apptxt (om-dialog-item-text pathtext))
                                              (starttxt (om-dialog-item-text tablestart))
                                              (sizetxt (om-dialog-item-text tablesize))
                                              (verbosetxt (om-dialog-item-text pasverbose))
                                              )
                                          ;;; table start
                                          (if (and (not (string= "" starttxt))
                                                   (integerp (read-from-string starttxt)))
                                            (setf (nth 3 rep-list) (read-from-string starttxt))
                                            (setf argerror t))
                                          ;;; table size
                                          (if (and (not (string= "" sizetxt))
                                                   (integerp (read-from-string sizetxt)))
                                            (setf (nth 4 rep-list) (read-from-string sizetxt))
                                            (setf argerror t))
                                          ;;;verbose
                                          (if (om-checked-p verbosemat)
                                            (if (and (not (string= "" verbosetxt))
                                                     (integerp (read-from-string verbosetxt)))
                                              (setf (nth 5 rep-list) (read-from-string verbosetxt))
                                              (setf argerror t))
                                            (setf (nth 5 rep-list) nil))
                                          ;;; auto-rename
                                          (setf (nth 6 rep-list) (om-checked-p autorename))
                                          ;;; delete intermediates
                                          (setf (nth 7 rep-list) (om-checked-p intermediates))
                                          ;;; def table
                                          (setf (nth 8 rep-list) (om-dialog-item-text deftablestr))
                                          ;;; csound flags
                                          (setf (nth 9 rep-list) (om-dialog-item-text defflagsline))
                                           (if (probe-file apptxt) 
                                            (setf (nth 10 rep-list) apptxt)
                                            (setf argerror t))
                                          (if argerror
                                            (om-message-dialog (format nil "Error in a csoud option.~% Preferences could not be recorded.")) 
                                            (om-return-from-modal-dialog dialog rep-list))
                                          ))
                           )

      (om-make-dialog-item 'om-button (om-make-point 130 pos) (om-make-point 80 20) "Cancel"
                           :di-action (om-dialog-item-act item
                                        (om-return-from-modal-dialog dialog nil)))
      
      )
    (om-modal-dialog dialog)))


