
(in-package :om)


;======================================================
;Synth Maquettes 
;======================================================

(defclass SynthMaquette (OMMaqAbs) 
  ((settings :accessor settings :initarg :settings :initform nil)
   (snd :accessor snd :initarg :snd :initform nil)
   ;(sndfile :accessor sndfile :initarg :sndfile :initform nil)
   (samplerate :accessor samplerate :initarg :samplerate :initform 44100.0))
   (:documentation "This is the class of the maquette references of red maquette boxes.
The difference with normal maquettes is that this maquette are not saved in a owner file.
These maquettes are saved in the patch file which contains the red boxes.
So red maquettes can not be sharing.#enddoc#
#seealso# (OMPatch OMTemporalPatch OMMaqAbs) #seealso#")
   (:metaclass omstandardclass))

(defmethod SynthMaquette-p ((self SynthMaquette)) t)
(defmethod SynthMaquette-p ((self t)) nil)

(defun get-absmaqclass () 
  (if (om-option-key-p) 'SynthMaquette 'OMMaqAbs))

(defmethod get-maq-obj-name ((class (eql 'SynthMaquette))) "synthesis-maquette")

(defmethod omNG-make-tempobj ((self SynthMaquette) posi name)
   (let ((newobj (make-instance *def-metaclass-box-tempo* 
                   :name name
                   :reference self
                   :extend (get-obj-dur self)
                   :posy (om-point-v posi)
                   :offset (om-point-h posi)
                   :icon nil
                   :colorframe (om-make-color 0.53333336 0.5 0.5)
                   :sizey 20)))
     (push newobj (attached-objs self))
     ;; ***
     (update-from-reference newobj)
     newobj))


(defmethod initialize-instance :after ((self SynthMaquette) &rest initargs)
  (setf (maq-color (params self)) (om-make-color 0.53333336 0.5 0.5))
  (setf (settings self) (make-instance 'audio-mix-console :nbtracks 8)))

(defmethod om-save ((self SynthMaquette) &optional (values? nil))
   "Generation of code to save the maquette 'self'."
   (let ((boxes (mapcar #'(lambda (box) (omNG-save box values?)) (boxes self)))
           (connectiones (mk-connection-list (boxes self)))
           (markers (mk-markers-list (boxes self))) fond-ec)
     (when (thepict (pictu self))
         (setf fond-ec (omng-save (pictu self))))
     `(om-load-maq-synth1 ,(name self) ',boxes ',connectiones ',(range (params self)) ',markers
                                            ,(om-save-color (maq-color (params self))) ',(metricparam (params self)) 
                                            ,(show-conect (params self)) ,*om-version* ,fond-ec ,(omng-save (settings self)))))

(defun om-load-maq-synth1 (name boxes connections range markers 
                              &optional (colormaq nil) (metricparam '((4 60) ((4 4)) 16 t)) (show-connect t) (version nil) fond-ec settings)
   (let ((newmaquette (make-instance 'SynthMaquette :name name :icon 265)))
     (setf (boxes newmaquette) nil)
     (mapc #'(lambda (box) (omNG-add-element newmaquette (eval box)))  boxes)
     (setf (boxes newmaquette) (reverse (boxes newmaquette)))
     (setf (connec newmaquette) connections)
     (setf (settings newmaquette) (eval settings))
     (remk-markers markers (boxes newmaquette))
     (when fond-ec (setf (pictu newmaquette) fond-ec))
     (when version
       (setf (omversion newmaquette) version))
     (setf (params newmaquette) (make-instance 'maquette-params 
                                  :range range
                                  :maq-color (om-correct-color colormaq)
                                  :metricparam metricparam
                                  :show-conect show-connect))
     (push newmaquette *loaading-stack*)
     newmaquette))

(defmethod allowed-in-synthmaq-p ((self t)) nil)
(defmethod allowed-in-synthmaq-p ((self sound)) t)
(defmethod allowed-in-synthmaq-p ((self sound-data)) t)
(defmethod allowed-in-synthmaq-p ((self CS-Evt)) t)

(defmethod cons-maquette-object ((self SynthMaquette) objs)
   "Cons a maquette-obj instance which contains the temporal elements in the maquette 'self'."
   (let (
         ;(maq-obj (call-next-method))
         )
     (setf (snd self) (cons-sound-maquette-object self objs))
     ;maq-obj
     (snd self)))

(defmethod cons-copy-maquette-object ((self SynthMaquette) objs)
   "Cons a maquette-obj instance which contains copies of the temporal elements in the maquette 'self'."
   (let (
         ;(maq-obj (call-next-method))
         )
     (setf (snd self) (cons-sound-maquette-object self objs))
     ;maq-obj
     (snd self)))


(defmethod cons-play-maquette-object ((self SynthMaquette) objs)
  (snd self))

(defmethod make-sound-obj ((self sound-data)) self)

(defmethod make-sound-obj ((self sound)) 
  (let ((effectlist (au::MakeAudioEffectList))
        (snd (au::MakeReadSound (om-path2cmdpath (om-sound-file-name self)))))
    (when (or (vol self) (pan self))
      (when (vol self)
        (setq effectlist (au::AddAudioEffect effectlist (au::MakeVolAudioEffect (float (/ (vol self) 100))))))
      (when (pan self)
        (setq effectlist (au::AddAudioEffect effectlist (au::MakePanAudioEffect (float (/ (+ (pan self) 100) 200))))))
      (setq snd (au::MakeTransformSound snd effectlist 50 50)))
    (make-instance 'sound-data :ptr snd)
    ))

(defmethod make-sound-obj ((self CS-Evt)) 
  (let* ((sndpath (synthesize 'csound self))
        (snd (au::MakeReadSound (om-path2cmdpath sndpath))))
    (make-instance 'sound-data :ptr snd)
    ))

(defmethod make-cs-sound-obj ((self list)) 
  (let* ((sndpath (synthesize 'csound self :tables (when (probe-file (infile "TRA.fun")) (infile "TRA.fun"))))
         (snd (au::MakeReadSound (om-path2cmdpath sndpath))))
    (make-instance 'sound-data :ptr snd)
    ))

(defmethod cs-evt-p ((self CS-Evt)) t)
(defmethod cs-evt-p ((self t)) nil)
      
(defmethod cons-sound-maquette-object ((self SynthMaquette) objs)
   "Cons a maquette-obj instance which contains the temporal elements in the maquette 'self'."
   (let* ((maxdur 0) sounds rep params snd 
          ;(name (or (sndfile self) 
          ;          (unique-pathname *om-tmpfiles-folder* "maqtemp" "wav")))
          cslist snd-return)
     (loop for tempobj in objs do
           (let ((val-of-tempob (get-mus-ob tempobj)))
             (when (allowed-in-synthmaq-p val-of-tempob)
               (setf (offset val-of-tempob) 
                     (slot-value tempobj 'offset))
               (if (cs-evt-p val-of-tempob)
                 (push val-of-tempob cslist)
                 (push val-of-tempob sounds)))))
     (when (or sounds cslist)
       (setf snd (au::MakeNullSound (round (* (get-obj-dur self) (samplerate self)) 1000)))
       
       (when sounds
         (loop for s in sounds do
               (let* ((sndsynth (make-sound-obj s))
                      (offset (round (* (offset s) 44.1))))
                 (setf snd 
                       (au::makeMixSound snd 
                                         (au::MakeSeqSound (au::MakeNullSound offset) 
                                                           (ptr sndsynth)
                                                           0)
                                         )
                       )
                 ))
         
         )
       
       (when cslist
         (let ((cssynth (make-cs-sound-obj cslist)))
           (setf snd (au::makeMixSound snd (ptr cssynth)))
           ))
       (setf snd-return (make-instance 'sound-data :ptr snd))
       (setf (pict-sound snd-return) (make-pict-sound snd-return))
       snd-return
       ))) 

(defmethod synthesize-maquette ((self SynthMaquette))
  (setf (snd self) (cons-sound-maquette-object self (boxes self))))

(defmethod maquette-extra-eval ((self synthmaquette)) 
  (synthesize-maquette self))

(defmethod get-maq-mus-obj ((self SynthMaquette)) (snd self))

(defmethod allow-strech-p ((self synthmaquette) (factor number)) nil)


;;;====== EDITOR =======

(defclass synthmaquetteeditor (maquetteeditor) 
  ((sndbutt :accessor sndbutt :initform nil :initarg :sndbutt)))

(defmethod get-editor-class ((self SynthMaquette)) 'synthmaquetteeditor)

(defun openaudioconsole (obj) 
  (openobjecteditor (omng-make-new-ml-boxcall-from-obj (settings obj) (om-make-point 0 0) "Audio Settings")))

(defmethod initialize-instance :after ((self synthmaquetteeditor) &rest initargs)
  (setf (sndbutt self) (om-make-view 'button-icon
                                      :iconID 922
                                      :position (om-make-point 0 0)
                                      :size (om-make-point 24 24)
                                      :action
                                      #'(lambda (item) 
                                          (openaudioconsole (object self))
                                          )
                                      ))
  (om-add-subviews self (sndbutt self)))

(defmethod update-subviews ((self synthmaquetteeditor))
  (call-next-method)
  (om-set-view-position (sndbutt self) (om-make-point 0 (- (h self) 24))))



(defmethod draw-editor-mode ((self SynthMaquette) view)
  ;(let ((maq-obj (cons-sound-maquette-object self (boxes self))))
    (om-with-focused-view view
      (om-with-fg-color nil (colorframe (object view))
        (let ((durmaq (get-obj-dur self))
              (rangey (max-y-item self)))
          (om-fill-rect 0 0 (w view) (h view))
          (loop for item in (boxes self) do
                (when (boxtempobj-p item)  ;;; and (allowed-in-synthmaq-p (get-mus-ob item)))
                  (draw-rect-in-rect item  (w view) (- (h view) 65)  0 0  durmaq (first rangey) (second rangey) 1 view))))
         
      (when (and (snd self) (pict-sound (snd self)))
          (om-draw-picture view (pict-sound (snd self)) (om-make-point 0 (- (h view) 60)) (om-make-point (w view) 60))
          )
      )
   (draw-carre view)
   )
;)
)
