; Chordseq to SDIF
; jb  23/02/05

(in-package :om)


(defun calc-pad (n)
  (if (zerop (mod n 8)) n
      (* 8 (+ (floor n 8) 1))))


(defun midivel->partialamp (vel)
  (/ vel 1270.0))

(defmethod! chordseq-to-datalist ((self chord-seq))
  (let* ((ind -1)
         (chord-data nil)
         (enddata nil)
         newendlist)
    ;;; recup les donnes
    (setf chord-data 
          (loop for chord in (inside self) 
                collect (list (offset->ms chord)
                              (loop for note in (inside chord) 
                                    do (pushr (list (+ (offset->ms chord) (dur note)) (incf ind)) enddata)
                                    collect (list ind (mc->f (midic note)) (midivel->partialamp (vel note)) 0)))))
    ;;; mettre tous les end simulatnes dans une mme frame
    (setf enddata (sort enddata '< :key 'car))
    (let* (tmptime)
      (setf newendlist (loop while enddata do
                             (setf tmptime (car (car enddata)))
                             collect (list tmptime (loop while (equal (car (car enddata)) tmptime) collect (second (pop enddata))))) 
            ))
    (sort (append chord-data newendlist) '< :key 'car)
    ))


(defun write-begin-frame (fileptr data)
  (without-interrupts  
   (let* ((time (/ (car data) 1000.0))
        (datatype 4)
        (numlines (length (second data)))
        (framesize (+ 32 (calc-pad (* 4 datatype numlines)) (calc-pad (* 1 datatype numlines))))
        (mrk-values (om-make-pointer (* 1 datatype numlines)))
        (trc-values (om-make-pointer (* 4 datatype numlines))))
    (sdif::SdifFSetCurrFrameHeader fileptr (sdif-string-to-signature "1MRK") framesize 2 0 (coerce time 'double-float))
    (sdif::SdifFWriteFrameHeader fileptr)
    (loop for i from 0 to (- numlines 1)
          for note in (second data) do
            (om-write-ptr mrk-values (* i datatype) :single-float (coerce (car note) 'single-float))
            
            (om-write-ptr trc-values (* (* i 4) datatype) :single-float (coerce (car note) 'single-float) )
            (om-write-ptr trc-values (* (+ (* i 4) 1) datatype) :single-float (coerce (second note) 'single-float) )
            (om-write-ptr trc-values (* (+ (* i 4) 2) datatype) :single-float (coerce (third note) 'single-float) )
            (om-write-ptr trc-values (* (+ (* i 4) 3) datatype) :single-float (coerce (fourth note) 'single-float) )
            )
      (sdif::SdifFWriteMatrix fileptr (sdif-string-to-signature "1BEG") datatype numlines 1 mrk-values)
      (sdif::SdifFWriteMatrix fileptr (sdif-string-to-signature "1TRC") datatype numlines 4 trc-values)
      ))
)

(defun write-end-frame (fileptr data)
  (without-interrupts  
  (let* ((time (/ (car data) 1000.0))
        (datatype 4)
        (numlines (length (second data)))
        (framesize (+ 32 (calc-pad (* 1 datatype numlines))))
        (mrk-values (om-make-pointer (* 1 datatype numlines))))
    (sdif::SdifFSetCurrFrameHeader  fileptr (sdif-string-to-signature "1MRK") framesize 1 0 (coerce time 'double-float))
    (sdif::SdifFWriteFrameHeader fileptr)
    (loop for i from 0 to (- numlines 1)
          for ind in (second data) do
          (om-write-ptr mrk-values (* i datatype) :single-float (coerce ind 'single-float))
          )
      (sdif::SdifFWriteMatrix fileptr (sdif-string-to-signature "1END") datatype numlines 1 mrk-values)
      )))

(defun write-mrk-frame (fileptr data)
  ;(without-interrupts  
   (let (beg end)
     (loop for elt in (second data) do 
           (if (consp elt) 
             (pushr elt beg)
             (pushr elt end)))
  (let* ((time (/ (car data) 1000.0))
         (datatype 4)
         (numlinesb (length beg))
         (numlinese (length end))
         (framesize (+ 32 (calc-pad (* 4 datatype numlinesb)) (calc-pad (* 1 datatype (+ numlinesb numlinese)))))
         (beg-values (om-make-pointer (* 1 datatype numlinesb)))
         (trc-values (om-make-pointer (* 4 datatype numlinesb)))
         (end-values (om-make-pointer (* 1 datatype numlinese)))
         (nbmat 0))
    (when beg (setf nbmat (+ nbmat 2))) 
    (when end (setf nbmat (+ nbmat 1)))
    (sdif::SdifFSetCurrFrameHeader fileptr (sdif-string-to-signature "1MRK") framesize nbmat 0 (coerce time 'double-float))
    (sdif::SdifFWriteFrameHeader fileptr)
    (loop for i from 0 to (- numlinesb 1)
          for note in beg do
          (om-write-ptr beg-values (* i datatype) :single-float (coerce (car note) 'single-float))
          (om-write-ptr trc-values (* (* i 4) datatype) :single-float (coerce (car note) 'single-float) )
          (om-write-ptr trc-values (* (+ (* i 4) 1) datatype) :single-float (coerce (second note) 'single-float) )
          (om-write-ptr trc-values (* (+ (* i 4) 2) datatype) :single-float (coerce (third note) 'single-float) )
          (om-write-ptr trc-values (* (+ (* i 4) 3) datatype) :single-float (coerce (fourth note) 'single-float) )
          )
    (loop for i from 0 to (- numlinese 1)
          for note in end do
          (om-write-ptr end-values (* i datatype) :single-float (coerce note 'single-float))
          )
    (when beg
      (sdif::SdifFWriteMatrix fileptr (sdif-string-to-signature "1BEG") datatype numlinesb 1 beg-values)
      (sdif::SdifFWriteMatrix fileptr (sdif-string-to-signature "1TRC") datatype numlinesb 4 trc-values))
    (when end
      (sdif::SdifFWriteMatrix fileptr (sdif-string-to-signature "1END") datatype numlinese 1 end-values))
    ))
   ;)
)
     
 
(defun write-sdif-ascii-chunks (fileptr authorname NVTstring)
  (let (str sstr)
    (when authorname
      (write-1nvt-table fileptr
                        (list "Author" )
                        (list authorname)))
    (when NVTstring
      (setf str (format nil NVTstring))
      (setf sstr (sdif::SdifStringNew)) 
      (Sdif-String-Append sstr str)
      (sdif::SdifStringGetC sstr)
      (sdif::SdifFGetAllTypefromSdifString fileptr sstr))
    (sdif::SdifFWriteAllASCIIChunks fileptr)
  ))




(defun collect-mrk-frames (datalist)
  (let ((last-time nil)
        (frameslist nil))
    (loop for data in datalist do
          (if (and last-time (= (car data) last-time))
            (setf (cadr (last-elem frameslist))
                  (append (cadr (last-elem frameslist)) (cadr data)))
            (pushr data frameslist))
          (setf last-time (car data)))
    frameslist
    ))

(defmethod! chordseq->sdif ((self chord-seq) &optional outpath)
  :icon 264
  (let* ((error nil) time
    (out-path (or outpath (om-choose-new-file-dialog)))
    (outfile (sdif-open-file (om-path2cmdpath out-path) 1)))
  (sdif::SdifFWriteGeneralHeader outfile)
  (write-sdif-ascii-chunks outfile "OM5.0" "{1FTD 1MRK {1TRC chord_seq_partials;}}")
  (let ((datalist (chordseq-to-datalist self)))
    (setf datalist (collect-mrk-frames datalist)) 
    (loop for data in datalist
          while (not error) do
          ;(cond 
          ; ((consp (car (second data))) (write-begin-frame outfile data))
          ; ((integerp (car (second data))) (write-end-frame outfile data))
          ; (t nil))
          (write-mrk-frame outfile data)
          ))
  (sdif-close-file outfile)
  (om-namestring out-path)
))



; get Chordseq from SDIF

(defmethod! get-chordseq-data ((self sdiffile))
   :doc "Return a list of chords data (pitch onset offset velocity) from an sdif file (using 1MRK frames)"
   :icon 639
   (let ((chords nil) 
         (mrk-partials (make-hash-table)) (trc-partials (make-hash-table))
          mlist bmat emat pmat time
          (ptrfile (dynamic-open self)))
     (sdif::SdifFReadGeneralHeader ptrfile)
     (sdif::SdifFReadAllASCIIChunks ptrfile)
     (loop for item in (framesdesc self) do
             (when (equal "1MRK" (car item))
                 (setf time (nth 1 item))
                 (sdif-set-pos ptrfile (nth 3 item))
                 (setf mlist (nth 4 item))
                 (loop for mat in mlist do
                         (cond ((equal "1BEG" (car mat)) (setf bmat mat))
                                  ((equal "1END" (car mat)) (setf emat mat))
                                  ((equal "1TRC" (car mat)) (setf pmat mat))
                                  (t nil)))
                 (when bmat 
                     ;;; a begin matrix :
                     ;;; ajoute (onset (note1 note2 ...)) dans tmplist 
                     ;;; avec des valeurs de amp (0) freq (0) et dur (1000) arbitraires pour l'instant
                     (sdif-read-headers ptrfile (nth 3 item) (fifth bmat))
                     (loop for i = 0 then (+ i 1) while (< i (second bmat)) do
                             (sdif::SdifFReadOneRow ptrfile)
                             (sethash mrk-partials (floor (sdif::SdifFCurrOneRowCol ptrfile 1)) (list 0 time time 0))
                     ))
                 (when pmat 
                     ;;; a parameter matrix :
                     ;;; cherche les notes dans tmplist et set pitch et velocity
                     (sdif-read-headers ptrfile (nth 3 item) (fifth pmat))
                     (loop for i = 0 then (+ i 1) while (< i (second pmat)) do
                             (sdif::SdifFReadOneRow ptrfile)
                             (let* ((ind (floor (sdif::SdifFCurrOneRowCol  ptrfile 1))) 
                                    (freq (sdif::SdifFCurrOneRowCol ptrfile 2))
                                    (amp (sdif::SdifFCurrOneRowCol ptrfile 3)) 
                                    (par (gethash ind mrk-partials)))
                               (when par
                                 (setf (car par) freq)
                                 (setf (fourth par) amp))
                               ))
                     )
                 (when emat 
                     ;;; a end matrix :
                     ;;; find the notes, set duration and put int the final notes list 
                     (sdif-read-headers ptrfile (nth 3 item) (fifth emat))
                     (loop for i = 0 then (+ i 1) while (< i (second emat)) do
                             (sdif::SdifFReadOneRow ptrfile)
                             (let* ((ind (floor (sdif::SdifFCurrOneRowCol ptrfile 1)))
                                   (par (gethash ind mrk-partials)))
                               (when par
                                 (setf (third par) time))
                               )))
                 (setf bmat nil)
                 (setf emat nil)
                 (setf pmat nil)
                 )

             
             (when (or (equal "1TRC" (car item)) (equal "1HRM" (car item)))
               (setf time (nth 1 item))
               (sdif-set-pos ptrfile (nth 3 item))
               (setf mlist (nth 4 item))
               (loop for m in mlist do
                     (when (or (equal "1TRC" (car m)) (equal "1HRM" (car m)))
                       (sdif-read-headers ptrfile (nth 3 item) (fifth m))
                       (loop for i = 0 then (+ i 1) while (< i (second m)) do
                             (sdif::SdifFReadOneRow ptrfile)
                             (let* ((ind (floor (sdif::SdifFCurrOneRowCol ptrfile 1)))
                                    (freq (sdif::SdifFCurrOneRowCol ptrfile 2))
                                    (amp (sdif::SdifFCurrOneRowCol ptrfile 3))
                                    (par (gethash ind trc-partials)))
                               (if par
                                 (progn 
                                   (setf (car par) (push freq (car par)))
                                   (setf (fourth par) (push amp (fourth par)))
                                   (setf (third par) time)
                                   )
                                 (sethash trc-partials ind (list (list freq) time time (list amp))))
                               )
                             ))
                     )
               )


             )
     
        (maphash #'(lambda (key val)
                  (push val chords)) 
              mrk-partials)

     (maphash #'(lambda (key val)
                  (push (list (om-mean (car val)) 
                              (second val) (third val) 
                              (om-mean (fourth val)))  
                        chords)) 
              trc-partials)
     
     (dynamic-close self ptrfile)
     (sort chords '< :key 'cadr)
     
     ))



;;; !!!!!!   A REFAIRE   !!!!!!!
(defmethod! get-chordseq-data ((self sdifstream))
   :doc "Return a list of chords data (pitch onset offset velocity) from an sdif file (using 1MRK frames)"
   :icon 639
   (let ((chords nil) (tmplist nil) 
          mlist bmat emat pmat time)
     (loop for item in (Lframes self) do
             (when (equal "1MRK" (signature item))
                 (setf time (Ftime item))
                 (loop for mat in (LMatrix item) do
                         (cond ((equal "1BEG" (signature mat)) (setf bmat mat))
                                  ((equal "1END" (signature mat)) (setf emat mat))
                                  ((equal "1TRC" (signature mat)) (setf pmat mat))
                                  (t nil)))
                 (when bmat 
                     ;;; a begin matrix :
                     ;;; ajoute (onset (note1 note2 ...)) dans tmplist 
                     ;;; avec des valeurs de amp (0) freq (0) et outset (time) arbitraires pour l'instant
                     (loop for i = 0 then (+ i 1) while (< i (numcols bmat)) do
                             (push (list (floor (get-array-i-j bmat 0 i)) 0 time time 0) tmplist)
                             )
                     )
                 (when pmat 
                     ;;; a parameter matrix :
                     ;;; cherche les notes dans tmplist et set pitch et velocity
                     (loop for i = 0 then (+ i 1) while (< i (numcols pmat)) do
                             (let ((found nil) ind freq amp)
                               (setf ind (floor (get-array-i-j pmat 0 i)))
                               (setf freq (get-array-i-j pmat 1 i))
                               (setf amp (get-array-i-j pmat 2 i))
                               (loop for note in tmplist while (not found) do
                                       (when (equal ind (car note))
                                           (setf found t)
                                           (setf (second note) freq)
                                           (setf (fifth note) amp)))
                               ))
                     )
                 (when emat 
                     ;;; a end matrix :
                     ;;; find the notes, set duration and put int the final notes list 
                     (loop for i = 0 then (+ i 1) while (< i (numcols emat)) do
                             (let ((found nil) ind)
                               (setf ind (floor (get-array-i-j emat 0 i)))
                               (loop for note in tmplist while (not found) do
                                       (when (equal ind (car note))
                                           (setf found t)
                                           (setf (fourth note) time)
                                           (push (cdr note) chords)
                                           (setf tmplist (remove note tmplist))
                                           ))
                               )))
             (setf bmat nil)
             (setf emat nil)
             (setf pmat nil)
             ))
     (sort chords '< :key 'cadr)
     ))

;;by JB 05/10/06

(defmethod load-audiosculpt-file (file)
  (format t "LOADING DATA FROM ~a~%" file)
  (with-open-file (in-stream file :direction :input)
    (read in-stream)))


(defmethod! as-to-datalist ((self list))
  (let* ((ind -1)
         (chorddata nil)
         (enddata nil)
         newendlist
         newbeglist)
    ;;; recup les donnes
    (loop for partial in (cddr self) do
          (when (nth 5 partial)
          (pushr (list (* (nth 2 partial) 1000.0) (incf ind) (nth 3 partial) (/ (db->lin (+ 90 (nth 4 partial)))) 32767.0) chorddata)
          (pushr (list (* (nth 5 partial) 1000.0) ind) enddata)
          ))
    ;;; mettre tous les end simulatnes dans une mme frame
    (setf enddata (sort enddata '< :key 'car))
    (setf chorddata (sort chorddata '< :key 'car))
    (loop for end in enddata do
            (if (equal (car end) (car (last-elem newendlist)))
              (pushr (cadr end) (cadr (last-elem newendlist)))
              (pushr (list (car end) (list (cadr end))) newendlist)))
    (loop for beg in chorddata do
          (if (equal (car beg) (car (last-elem newbeglist)))
              (pushr (list (second beg) (third beg) (fourth beg) 0) (cadr (last-elem newbeglist)))
              (pushr (list (car beg) (list (list (second beg) (third beg) (fourth beg) 0))) newbeglist)))
    
    (sort (append newbeglist newendlist) '< :key 'car)
    ))

(defmethod! asfile->sdif (asfile &optional outpath)
  :icon 264
  (let* ((error nil) time
    (in-path (or asfile (om-choose-new-file-dialog)))
    (out-path (or outpath (om-choose-new-file-dialog)))
    (outfile (sdif-open-file (om-path2cmdpath out-path) 1)))
  (sdif::SdifFWriteGeneralHeader outfile)
  (write-sdif-ascii-chunks outfile "OM5.0" "{1FTD 1MRK {1TRC chord_seq_partials;}}")
  (let ((datalist (as-to-datalist (load-audiosculpt-file in-path))))
    (setf datalist (collect-mrk-frames datalist)) 
    (loop for data in datalist
          while (not error) do
          (write-mrk-frame outfile data)
          ))
  (sdif-close-file outfile)
  (om-namestring out-path)
))
