(in-package :om)


;;;=========================
;;; SDIF MATRIX
;;;=========================

(defclass! SDIFMatrix (class-array) 
   ((signature :initform nil :accessor signature))
   (:icon 644)
   (:documentation "SDIF MATRIX"))

(defmethod get-slot-in-out-names ((self SDIFMatrix))
   (values '("self" "numcols" "signature") '(nil 1 "")
           '("object or object list" "How many elements in each field?" "matrix type")
           '(nil nil nil)))

(defmethod fixed-slots-list ((self SDIFMatrix)) '(numcols signature))


(defmethod omNG-copy ((self SDIFMatrix))
  (let* ((theclass (class-name (class-of self)))
            (slots (mapcan #'(lambda (slot)
                               (list (string2initarg (string (first slot))) 
                                     (omNG-copy (slot-value self (first slot))))) 
                           (get-initargs-of-class theclass))))
     `(let ((copy (make-instance ',theclass ,.slots)))
        (setf (numrows copy) ,(numrows self))
        (setf (numcols copy) ,(numcols self))
        (setf (Lcontrols   copy) ,(omng-copy (Lcontrols self)))
        (setf (Lprecision copy) ',(Lprecision self))
        (setf (signature copy) ',(signature self))
        copy)))


;;; SDIFMatrix = comme une class array
;;; les fields = les controls du array

(defclass SDIFMatrixBox (ArrayBox) ())
(defmethod get-type-of-ed-box ((self SDIFMatrix))  'SDIFMatrixBox)

(defmethod do-add-one-input ((self SDIFMatrixBox))
  (or (call-next-method)
      (let* ((usedkeywords (loop for item in (find-class-boxes  (inputs self) 'control-keyword)
                                 collect (value item)))
             (newval "K0"))
        (loop for i = 1 then (+ i 1)
              while (member newval usedkeywords :test 'string-equal :key 'string) do
              (setf newval (format nil "K~D" i)))
        (setf (inputs self) (list+ (inputs self)
                                   (list (make-instance 'control-keyword
                                           :name "Control input"
                                           :box-ref self
                                           :value (string2initarg newval) 
                                           :doc-string "Optional control keyword. There exists 2 special keywords :attributes and :parsingFun")))))
      ))


(defmethod do-delete-one-input ((self SDIFMatrixBox))
   (let* ((usedkeywords (find-class-boxes (inputs self) 'control-keyword)))
     (if usedkeywords
       (setf (inputs self) (remove (car (last (inputs self))) (inputs self) :test 'equal))
       (call-next-method))))

(defmethod do-add-one-keyword ((self SDIFMatrixBox))
   (om-beep))

(defmethod do-delete-one-keyword ((self SDIFMatrixBox))
   (om-beep))

(omg-defclass SdifArrayEditor (ArrayEditor) ())

(defmethod get-editor-class ((self sdifmatrix)) 'SdifArrayEditor)

(defmethod initialize-instance :after ((self sdifarrayeditor) &rest initargs)
   (setf (show-controls-p self) t)
   (update-ins-panel  (panel self) :removeall t))

(defmethod draw-obj-in-rect ((self  SDIFMatrix) x x1 y y1 edparams  view)
   (let* ((visibles (get-draw-visibles-list self edparams))
           (numvis (length visibles)) deltay deltax)

     (cond ((or (<= numvis 0) (<= (numcols self) 0)) 
               ;;;;;;;;;;;;;;; new pour dessiner les controls = les fields SDIF
               (when (and (> (numcols self) 0) (Lcontrols self))
                   (setf deltay (max 15 (round (- y1 y)  (length (Lcontrols self))))
                         deltax (max 10 (round (- x1 x) (numcols self))))
                   
               (loop for i = 0 then (+ i 1) while (< i (length (Lcontrols self)))
                     for posy = y then (+ posy deltay) do
                     (let ((bpf? (get-bfp-for-draw  (get-array-row self i) self (get-precision self) (row-type self i))))
                       (if bpf?
                         (draw-obj-in-rect  bpf? x x1  posy (+ posy deltay) nil view)
                         (loop for di = 1 then (+ di 1)
                               for posx = (+ x (* di deltax))
                               while (< posx x1)  do
                               (om-draw-line posx posy posx (+ posy deltay))))
                       (om-draw-line x (+ posy deltay) x1  (+ posy deltay))
                       (om-draw-line x (+ 1 posy deltay) x1  (+ posy 1 deltay))
                       )
                     )
               )
               ;;;;;;;;;;;;;;;;;
               )
              ((= (numcols self) 1)
               (setf deltay (max 15 (round (- y1 y)  numvis)))
               (loop for i in visibles
                     for posy = y then (+ posy deltay) do
                       (let ((obj (get-array-i-j-for-eval self  i 0)))
                         (when (bpf-p obj)
                             (setf (bpfcolor obj) (nth i (color-list edparams))))
                         (draw-obj-in-rect obj x x1  posy (+ posy deltay) (default-edition-params obj) view)
                         (om-draw-line x (+ posy deltay) x1  (+ posy deltay))
                         (om-draw-line x (+ 1 posy deltay) x1  (+ posy  1 deltay)))))
              (t
               (setf deltay (max 15 (round (- y1 y)  numvis))
                 deltax (max 10 (round (- x1 x) (numcols self))))
               (loop for i in visibles
                     for posy = y then (+ posy deltay) do
                       (let ((bpf? (get-bfp-for-draw  (get-array-row self i) self (get-precision self) (row-type self i)))) 
                         (if bpf?
                             (progn
                                (setf (bpfcolor bpf?) (nth i (color-list edparams)))
                                (draw-obj-in-rect  bpf? x x1  posy (+ posy deltay) nil view))
                           (loop for di = 1 then (+ di 1)
                                 for posx = (+ x (* di deltax))
                                 while (< posx x1)  do
                                   (om-draw-line posx posy posx (+ posy deltay))))
                         (om-draw-line x (+ posy deltay) x1  (+ posy deltay))
                         (om-draw-line x (+ 1 posy deltay) x1  (+ posy  1 deltay))))))))


;;;=========================
;;; SDIF FRAME
;;;=========================
;;; Ensemble de matrices correspondant a un instant d'echantillonnage 
;;; Unite minimum pour ecrire dans un fichier SDIF
(defclass! sdifframe ()
   ((signature :initform nil :initarg :signature :accessor signature)
    (FTime :initform 0.0 :initarg :FTime :accessor FTime)
    (StreamId :initform 0 :initarg :StreamId :accessor StreamId)
    (LMatrix :initform nil :initarg :LMatrix :accessor LMatrix))
   (:icon 643))

(defmethod sdifframep ((self t)) nil)
(defmethod sdifframep ((self sdifframe)) t)

(defmethod initialize-instance :after ((self sdifframe) &rest initargs)
   (setf (LMatrix self) (list! (LMatrix self))))






;;;=========================
;;; SDIF STREAM
;;;=========================
;;; Ensemble de Frames constituant un flux de donnes SDIF 
(defclass! sdifstream ()
   ((Id :initform 0 :initarg :Id :accessor Id)
    (LFrames :initform nil :initarg :LFrames :accessor LFrames))
   (:icon 641))

(defmethod sdifstreamp ((self t)) nil)
(defmethod sdifstreamp ((self sdifstream)) t)

(defmethod initialize-instance :after ((self sdifstream) &rest initargs)   
   (setf (LFrames self) (list! (LFrames self)))
   (when (LFrames self)
       (setf (LFrames self) (sort (remove nil (LFrames self)) '< :key 'FTime))
       (let ((err nil))
         (loop for fr in (LFrames self) do
                 (if (sdifframep fr)
                     (when (not (equal (StreamId fr) (ID self)))
                         (setf (StreamId fr) (ID self))
                         (setf err t))
                   ))
         (when err (om-message-dialog (format nil "Warning :~%Some Frames IDs have been modified to fit the Stream ID ~D" (ID self)))))
       t))

(defmethod objfromobjs ((self list) (type sdifstream))
   (let ((id nil) (flist nil))
     (loop for item in self do 
             (when (sdifframep item)
                 (unless id 
                    (setf id (StreamID item)))
                 (push item flist)))
     (make-instance 'sdifstream :ID id :LFrames flist)))

(defmethod nb-frames ((self sdifstream))
   (length (LFrames self)))


;;;=========================
;;; SDIF TYPE
;;;=========================
(defclass! SDIFType ()
   ((Struct :initform 'F :initarg :Struct :accessor struct)
    (Signature :initform "" :initarg :Signature :accessor Signature)
    (Description :initform nil :initarg :description :accessor description))
   (:icon 646))

(defmethod sdiftypep ((self t)) nil)
(defmethod sdiftypep ((self sdiftype)) t)


(defmethod get-slot-in-out-names ((self SDIFType))
   (values '("self" "struct" "signature" "description")
           '(nil 'F "" "")
           '("object" "Frame/Matrix" "signature (4 chars)" "description du type")
           '(nil ((1 (("Frame" 'F)  ("Matrix" 'M)))) nil nil)))



;;;=========================
;;; SDIF Buffer
;;;=========================
;;; Ensemble de frames et/ou streams
;;; + infos permettant de reconstruire un fichier SDIF complet

(defclass! SDIF-buffer ()
   ((Types :initform nil :initarg :Types :accessor Types)
    (LFrames :initform nil :initarg :LFrames :accessor LFrames))
   (:icon 645))

(defmethod initialize-instance :after ((self SDIF-buffer) &rest initargs)
   (setf (LFrames self) (list! (LFrames self)))
   (setf (LFrames self) (sort 
                         (remove nil (flat (loop for item in (flat (LFrames self)) collect (cond ((sdifframep item) item)
                                                                                                 ((sdifstreamp item) (LFrames item))
                                                                                                 (t nil)))))
                         '< :key 'FTime))
   (setf (Types self) (remove nil (list! (Types self)))))



