(in-package :om)

;==============================================
;writing sdif file
;==============================================

(defmethod save-sdif ((self t) ptr) 
   nil)

;;; MATRIX
;;; Attention entre OMArray et SDIF Matrix les ROWs-COLS siont inverss

(defmethod calcul-sdif-size ((self sdifmatrix))
   (+ (* 4 (numcols self) (+ (numrows self) (length (Lcontrols self)))) 16))

(defmethod save-sdif ((matrix sdifmatrix) ptr)
  (let* ((datatype 4)
         (numrow (length (Lcontrols matrix)))    ;;; (numcols matrix) 
         (rowsize (* datatype numrow))
         (values (om-make-pointer (* datatype numrow (numcols matrix)))))          
    (loop for i from 0 to (- (numcols matrix) 1) do
          (let* ((sdifrow (first-n (get-array-col matrix i) numrow)))
            (loop for j from 0 to (- numrow 1)
                  for val in sdifrow do
                  (om-write-ptr values (+ (* j datatype) (* i rowsize)) :single-float (coerce val 'single-float)))))
    (sdif::SdifFWriteMatrix  ptr (sdif-string-to-signature (signature matrix)) datatype
                             (numcols matrix) 
                             numrow values)))



;;; FRAME
(defmethod calcul-sdif-size ((self sdifframe))
   (let ((rep 0))
     (loop for item in (LMatrix self) do
           (setf rep (+ rep (calcul-sdif-size item))))
     (+ rep 16)))

(defmethod save-sdif ((self sdifframe) fileptr)
   (let ((framesize (calcul-sdif-size self)))
     (sdif::SdifFSetCurrFrameHeader fileptr (sdif-string-to-signature (signature self)) framesize (length (Lmatrix self)) 
                              (streamId self) (coerce (Ftime self) 'double-float))
     (sdif::SdifFWriteFrameHeader fileptr)
     (loop for item in (LMatrix self) do
             (save-sdif item fileptr))))



;;; STREAM
(defmethod save-sdif ((self sdifstream) fileptr)
   (loop for fr in (LFrames self) do (save-sdif fr fileptr)))


;;; TYPES, NVT , IDS
;;; A faire avant d'appeler SdifWriteAllASCIIChunks

;;; types
(defmethod format-string ((self sdiftype))
   (let ((str "") (desc ""))
     (cond ((equal 'F (struct self)) 
               (loop for item in (description self) do
                       (when (listp item) (setf desc (string+ desc (car item) " " (second item) "; "))))
               (setf desc (subseq desc 0 (- (length desc) 1)))
               (setf str (string+ str " 1FTD " (signature self) " {" desc "}")))
              ((equal 'M (struct self)) 
               (loop for item in (description self) do
                       (when (stringp item) (setf desc (string+ desc item ", "))))
               (setf desc (subseq desc 0 (- (length desc) 2)))
               (setf str (string+ str " 1MTD " (signature self) " {" desc "}")))
              (t nil))
     str))

;;(setf str (format nil "{1MTD EASM   {Onset, Frequency, Amplitude} 1FTD EASF {EASM  PartialsSet;}}"))

(defun write-types-table (thefile typeslist)
   (let ((str "{") sstr (FList nil) (MList nil))
     (loop for typedef in typeslist do
             (when (sdiftypep typedef) 
                 (cond ((equal 'F (struct typedef)) (push typedef Flist))
                          ((equal 'M (struct typedef)) (push typedef Mlist)))))
     (loop for mdef in Mlist do
             (setf str (string+ str (format-string mdef))))
     (loop for fdef in Flist do
             (setf str (string+ str (format-string fdef))))
     (setf str (string+ str "}"))
     (setf sstr (sdif::SdifStringNew)) 
     (Sdif-String-Append sstr str)
     (sdif::SdifStringGetC sstr)
     (sdif::SdifFGetAllTypefromSdifString thefile sstr)))

;;; nvt
(defun write-1nvt-table (thefile namelist vallist)
   (let ((nvt (sdif::SdifFNameValueList thefile)))
     (sdif::SdifNameValuesLNewTable nvt #xffff)
     (loop for name in namelist
           for val in vallist do
           (sdif-new-value-in-table nvt name val))))

;;; ids
(defun write-1IDS-table (file id str tree)
   (let ((idstable (sdif::SdifFStreamIDTable file)))
     (sdif-put-ids-table idstable id str tree)))



;;; SDIF Global Write
(defmethod! save-sdif-file ((self sdif-buffer) &optional pathname)
   :icon 639
   :doc "save as SDIF file"
   (let ((outfile (or pathname (om-choose-new-file-dialog))))
     (when (or (not (probe-file outfile))
                       (om-y-or-n-dialog (format nil "The file ~D already exists.~%Do you want replace it ?" 
                                               outfile)))
         (let ((thefile (sdif-open-file (om-path2cmdpath outfile) :eWriteFile)))
           (sdif::SdifFWriteGeneralHeader thefile)
           (write-1nvt-table thefile (list "Author" ) (list (string+ "OM " *version-string*)))
           (write-types-table thefile (list! (types self)))
           (sdif::SdifFWriteAllASCIIChunks thefile)
           
           (loop for item in (LFrames self) do
                 (save-sdif item thefile))
           (sdif-close-file thefile)))
     outfile))



;;; INTERFACE FILE-BOX
;;; interface file-box

(defmethod! sdif-write-frame (stream (self sdifframe))
  :icon 908
  (save-sdif self stream))

(defmethod! sdif-write-frame (stream (self t))
  :icon 908
  (print "only write sdif frames") 
  nil)

(defmethod! sdif-write-header (stream &optional (types nil) (nvtitems nil))
  :icon 908
  (sdif::SdifFWriteGeneralHeader stream)
  (let ((nvtnames (if nvtitems (loop for item in nvtitems collect (car item)) (list "Creator")))
        (nvtvals (if nvtitems (loop for item in nvtitems collect (cadr item)) (list (string+ "OM " *version-string*)))))
    (write-1nvt-table stream nvtnames nvtvals))
  (when types (write-types-table stream types))
  (sdif::SdifFWriteAllASCIIChunks stream))

