(in-package :om)


;-------------------Class definition-----------------------------
(defclass! sdifFile ()   
   ((filepathname  :initform nil :accessor filepathname)
    (assoc-sound  :initform nil :accessor assoc-sound)
    (framesdesc :initform nil :accessor framesdesc))
   (:icon 638)
   (:documentation "This object refer to a sdif file and allow read this."))


(defmethod* objfromobjs ((self string) (type sdifFile))
   (load-sdif-file self))

(defmethod* objfromobjs ((self pathname) (type sdifFile))
   (load-sdif-file self))

(defmethod objfromobjs ((self sdif-buffer) (type sdiffile))
   (objfromobjs (save-sdif-file self) type))

(defmethod objfromobjs ((self sdiffile) (type textfile))
   (objfromobjs (bin2ascii self) type))


(defmethod execption-save-p ((self sdifFile)) 'sdifFile)
(defmethod save-exepcion ((self sdifFile)) 
   (if (filepathname self)
     `(om-load-if ,(om-save-pathname-relative (filepathname self)) 'load-sdif-file)
     `(make-instance 'sdifFile)))

(defmethod omNG-copy ((self sdiffile))
   (let ((copy (make-instance 'sdiffile)))
     (setf (slot-value copy 'filepathname) (filepathname self))
     (setf (slot-value copy 'assoc-sound) (assoc-sound self))
     (setf (slot-value copy 'framesdesc) (framesdesc self))
     copy))


(defmethod dynamic-open ((self sdifFile))
   (let (rep)
     (if (filepathname self)
       (let ((fileptr (sdif-open-file (om-path2cmdpath (filepathname self)) :eReadFile)))
         (if (and fileptr (not (sdif-null-ptr-p fileptr)))
           (setf rep fileptr))))
     rep))

(defmethod dynamic-close ((self sdifFile) ptr)
   (sdif-close-file  ptr)
   t)

;-------------------Box-----------------------------




(defmethod get-type-of-ed-box ((self sdifFile))  'sdifFilebox)
(defmethod Class-has-editor-p ((self sdifFile)) t)

(defclass sdifFilebox (OMBoxEditCall) ())

(defmethod get-frame-class ((self sdifFilebox)) 'boxsdifframe)

(omg-defclass boxsdifframe (boxEditorFrame) ())

(defmethod om-get-menu-context ((object boxsdifframe))
  (list+  (list (om-new-leafmenu (if (view-of-patch (object object)) "free view of patch" "set view of patch")
                                 #'(lambda () (change-view-of-patch object)))
                (om-new-leafmenu "-" #'(lambda () nil))
                ) 
          (boxframe-default-list object)))

(defmethod OpenObjectEditor ((self sdifFilebox))
   (if (filepathname (value self))
       (let ((open? (open-sdif-box? self)))
         (when open? (om-beep))
         (if open?
             (om-select-sdifedit-window self)
           (open-sdif-box self)))
     (om-message-dialog "This SDIF box has no attached file!")))
 
(defmethod open-sdif-box? ((self sdifFilebox))
   (om-sdifedit-open-p self))

(defmethod open-sdif-box ((self sdifFilebox))
   ;;;(ccl::launch-SE-with-an-event (path2unixpath (filepathname (value self)))
   ;;;                              (when (assoc-sound (value self))
   ;;;                                (mac-namestring (assoc-sound (value self))))
   ;;;                                             self)
   (om-run-sdifedit self (om-path2cmdpath (filepathname (value self))) 
                            (when (assoc-sound (value self))
                                (om-namestring (assoc-sound (value self)))))
   ;;;(print (format nil "launch sdif edit with ~s" (filepathname (value self))))
   )

;------------------Load sdif file method---------------------------            
 
(defmethod* get-sdif () 
   :initvals nil
   :indoc nil
   :doc "load a sdif file"
   :icon 148
   (let ((name (om-CHOOSE-FILE-DIALOG)))
     (when name
       (load-sdif-file (om-namestring name))
       )))


(defun load-sdif-file (name)
   (when name
     (let ((rep (make-instance 'sdifFile))
           (unix-name (om-path2cmdpath name)))
       (if (probe-file name)
         (if (sdif-check-file unix-name)
           (let ((fileptr (sdif-open-file unix-name :eReadFile)))
             (om-print (string+ "Loading SDIF file : " (om-namestring name)))
             (if (and fileptr (not (sdif-null-ptr-p fileptr)))
                 (progn 
                   (init-description rep name fileptr)
                   rep)
               (om-beep-msg (format nil "error trying to open the file ~D" name))
               ))
           (om-beep-msg "this file is not a SDIF file !!"))
         (om-beep-msg "this file does not exist !!"))
       )))

;----------------FILL SLOTS
(defmethod init-description ((self sdifFile) name ptrfile)
   (setf (filepathname self) name)
   (let (Framelist nextFrame)
     (sdif::SdifFReadGeneralHeader ptrfile)
     (sdif::SdifFReadAllASCIIChunks ptrfile)
     (setf nextFrame (firstframep self ptrfile)) 
     (loop while nextframe do
           (let ((desc (description-frame ptrfile)))
             (push desc framelist) 
             (setf nextFrame (nextframep self ptrfile))))
     (setf (framesdesc self) (reverse framelist))
     (dynamic-close self ptrfile) t))

; modiJB
;19/12/03
; FRAMEDESC = (signature time ID pos (matrixDesc-list))
; MATRIXDESC = (signature nbRow nbCol datatype pos)

(defmethod description-frame (ptr)
   (let ((nummatrix 0) pos rep)
     (setf pos (- (sdif-get-pos ptr) 4))
     (sdif::SdifFReadFrameHeader ptr)
     (setf rep (list (sdif-signature-to-string (sdif::SdifFCurrFrameSignature ptr))
                     (sdif::SdifFCurrTime ptr) 
                     (sdif::SdifFCurrId ptr)
                     pos
                     ))
     (setf nummatrix (sdif::SdifFCurrNbMatrix ptr)) 
     (append rep (list (loop for i from 1 to nummatrix 
                             collect (description-matrix ptr))))))

(defmethod description-matrix (ptr)
   (let (rep pos)
     (setf pos (sdif-get-pos ptr))
     (sdif::SdifFReadMatrixHeader ptr)
     (setf rep (list  (sdif-signature-to-string (sdif::SdifFCurrMatrixSignature ptr)) 
                      (sdif::SdifFCurrNbRow ptr) 
                      (sdif::SdifFCurrNbCol ptr) 
                      8 ;(SdifFCurrDataType thefile)
                      pos
                      ))   
     (sdif::SdifFSkipMatrixData ptr)
     rep))

(defmethod how-many-mat ((self sdifFile) i)
   (length (fifth (nth i (framesdesc self)))))

(defmethod firstframep ((self sdifFile) ptr)
   (let ((signature (sdif::SdifFCurrSignature ptr)))
     ;;;(not  (equal :******  signature))
     (not (equal "" (sdif-signature-to-string signature)))
     ))

(defmethod nextframep ((self sdifFile) ptr)
   (let ((signature (sdif-get-signature ptr)))
     (not (equal "" (sdif-signature-to-string (sdif::SdifFCurrSignature ptr))))))


;--------------------------------------
;MOVING INTO A FILE
(defmethod goto-sdif-frame ((self sdifFile) i)
   (if (< i (numframes self))
     (let ((ptrfile (dynamic-open self)))
       (if (null ptrfile)
         (om-beep-msg (format nil "Error reading the sdif file ~D" (filepathname self)))
         (progn
           (sdif::SdifFReadGeneralHeader ptrfile)
           (sdif::SdifFReadAllASCIIChunks ptrfile)
           (loop for j from 1 to i do
                 (sdif::SdifFReadFrameHeader ptrfile)
                 (sdif::SdifFSkipFrameData ptrfile)
                 (sdif-get-signature ptrfile))
           ptrfile)))
     (om-beep-msg (format nil "the sdif file ~D has only ~D frames" (filepathname self) (numframes self)))))

(defmethod goto-sdif-frame-mat ((self sdifFile) i j)
   (let ((ptrfile (goto-sdif-frame self i)) matnum)
     (when ptrfile
       (setf matnum (how-many-mat self i))
       (if (< j matnum)
         (progn
           (sdif::SdifFReadFrameHeader ptrfile)
           (loop for k from 1 to j do
                 (sdif::SdifFReadMatrixHeader ptrfile)
                 (sdif::SdifFSkipMatrixData ptrfile) )
           ptrfile)
         (om-beep-msg (format nil "the ~D frame of the sdif file ~D has only ~D matrix" i (filepathname self) matnum))))))

(defmethod get-sdif-i-j-point ((self sdifFile) frame mat i j)
   (let ((ptrfile (goto-sdif-frame-mat self frame mat))
         col row rep)
     (when ptrfile
       (sdif::SdifFReadMatrixHeader ptrfile)
       (setf col (sdif::SdifFCurrNbCol ptrfile))
       (setf row (sdif::SdifFCurrNbRow ptrfile))
       (if (and (> col j) (> row i))
         (progn
           (loop for k from 0 to i do
                 (sdif::SdifFReadOneRow ptrfile))
           (loop for k from 1 to (+ j 1) do
                 (setf rep (sdif::SdifFCurrOneRowCol ptrfile k)))
           (dynamic-close self ptrfile)
           rep)
         (om-beep-msg (format nil "the point (~D, ~D) is out of range, the matrix dimension is (~D,~D)" i j row col))))))

(defmethod get-sdif-col ((self sdifFile) frame mat i)
   (let ((ptrfile (goto-sdif-frame-mat self frame mat))
         col row rep)
     (when ptrfile
       (sdif::SdifFReadMatrixHeader ptrfile)
       (setf col (sdif::SdifFCurrNbCol ptrfile))
       (setf row (sdif::SdifFCurrNbRow ptrfile))
       (if (> col i)
         (progn
           (setf rep (loop for k from 0 to (- row 1) 
                           collect (progn
                                     (sdif::SdifFReadOneRow ptrfile)
                                     (sdif::SdifFCurrOneRowCol ptrfile (+ i 1)))))
           (dynamic-close self ptrfile)
           rep)
         (om-beep-msg (format nil "Error the matrix dimension is (~D,~D)" row col))))))


(defmethod get-sdif-row ((self sdifFile) frame mat i)
   (let ((ptrfile (goto-sdif-frame-mat self frame mat))
         col row rep)
     (when ptrfile
       (sdif::SdifFReadMatrixHeader ptrfile)
       (setf col (sdif::SdifFCurrNbCol ptrfile))
       (setf row (sdif::SdifFCurrNbRow ptrfile))
       (if  (> row i)
         (progn
           (loop for k from 0 to i do
                 (sdif::SdifFReadOneRow ptrfile))
           (setf rep (loop for k from 1 to col collect
                           (sdif::SdifFCurrOneRowCol ptrfile k)))
           (dynamic-close self ptrfile)
           rep)
         (om-beep-msg (format nil "Error the matrix dimension is (~D,~D)" row col))))))


;=====================================================
;METHODS FOR EDITION
;=====================================================

(defmethod get-initval ((self sdiffile)) (make-instance (type-of self)))


(defmethod make-one-instance ((class sdiffile) &rest slots-vals) 
   (get-sdif))

(defmethod good-val-p? ((self sdifFile))
   (not (null (filepathname self))))

;----GUI----
(defmethod! Assoc-aiff ((self sdifFile) &optional aiff-path)
   :icon 639
   :doc "Assoc the aiff to the sdif."
   (unless aiff-path (setf aiff-path (om-choose-file-dialog)))
   (if (probe-file aiff-path)
     (setf (assoc-sound self) aiff-path)
     (om-beep-msg "this file does not exist !!")))




;; compat
(defmethod! bin2ascii ((self sdifFile) &optional targetname)
   :icon 639
   :doc "Convertit un fichier SDIF <self> en un fichier texte pseudo-SDIF in targetname."
   (let ((rep nil))
   (if (filepathname self)
       (let ((ptr (dynamic-open self)))
         (unless targetname
            (setf targetname (om-CHOOSE-new-FILE-DIALOG :prompt "Output Text File")))
         (if (and ptr targetname)
             (progn
                (sdif-to-text ptr (namestring targetname))
                (setf rep targetname))
           (om-message-dialog "Could not convert SDIF file to text..."))
         (when ptr (dynamic-close self ptr)))
     (om-message-dialog "SDIFFile box seems not to have attached SDIF file..."))
     rep))

(defmethod! sdif2text ((self sdifFile) &optional targetname)
   :icon 639
   :doc "Convertit un fichier SDIF <self> en un fichier texte pseudo-SDIF in targetname."
   (bin2ascii self targetname))


(defmethod! numFrames ((self sdifFile))
   :icon 639
   :doc "get the number of frames in <self>."
   (length (framesdesc self)))


;modif JB
;19/12/03
(defmethod! FrameInfo ((self sdifFile) i)
   :icon 639
   :numouts 5
   :doc "Return Signature Time StreamId Pos and number of matrix for the i-eme frame of <self>."
   (let ((desc (nth i (framesdesc self))))
     (when desc
       (values (first desc) (second desc) (third desc) (fourth desc) (length (fifth desc))))))

(defmethod! numMatrix ((self sdifFile) i)
   :icon 639
   :doc "get the number of matrix for the i-eme frame of <self>."
   (length (fifth (nth i (framesdesc self)))))

(defmethod! MatrixInfo ((self sdifFile) i j)
   :icon 639
   :numouts 4
   :doc "Return Signature Numrows Numcols date-type and pos of the j-ieme matrix of the i-eme frame of <self>."
   (let ((desc (nth j (fifth (nth i (framesdesc self))))))
     (when desc
       (values (first desc) (second desc) (third desc) (fourth desc) (fifth desc)))))


(defmethod! SdifInfo ((self sdifFile))
   :icon 639
   :doc "Print <self> information."
   (format *standard-output* "----------------------------------------------------------~%")
   (format *standard-output*  "Sdif file description for ~D~%"  (namestring (filepathname self)))
   (format *standard-output* "----------------------------------------------------------~%")
   (loop for item in (framesdesc self)
         for i = 1 then (+ i 1) do
         (format *standard-output*  "**********Begin Frame number ~D******************~%" i)
         (format *standard-output*  "Signature : ~D~%Time : ~D~%Stream number : ~D~%Matrix number : ~D~%" 
                 (first item) (second item)  (third item) (length (fifth item)))
         (loop for mat in (fifth item)
               for j = 1 then (+ j 1) do
               (format *standard-output*  "==========Matrix number ~D=========~%" j)
               (format *standard-output*  "Signature : ~D~%Row number : ~D~%Col number : ~D~%Data type : ~D bytes~%Position ~D~%~%" 
                       (first mat) (second mat)  (third mat) (fourth mat) (fifth mat)))
           (format *standard-output*  "**********End Frame number ~D******************~%" i)))


(defmethod! GetRow ((self sdifFile) fnum Mnum Rnum)
   :icon 639
   :doc "Get as a list the Rnum row of the aNum array of the fNum Frame of <self>."
   (get-sdif-row self fnum Mnum Rnum))

(defmethod! GetCol ((self sdifFile) fnum Mnum Cnum)
   :icon 639
   :doc "Get as a list the Cnum col of the aNum array of the fNum Frame of <self>."
   (get-sdif-col self fnum mnum Cnum))

(defmethod! GetVal ((self sdifFile) fnum Mnum Rnum Cnum)
   :icon 639
   :doc "Get as the cell (Rnum, Cnum) of the aNum array of the fNum Frame of <self>."
   (get-sdif-i-j-point self fnum Mnum Rnum Cnum))

;==================================
;news JB
;12/19/03

(defmethod! GetSdifData ((self sdifFile) sID frameType matType Cnum rmin rmax tmin tmax)
   :icon 639
   :indoc '("sdif file" "stream number" "frame type" "matrix type" "field number" "min row" "max row" "min time" "max time")
   :doc "Get as bi-dimensionnal data array (rmin->rmax, tmin->tmax) from the fieldNum field of the matType matrix from the sID Stream of frameType frame from <self>."
   :numouts 2
   (get-sdif-data self sID frameType matType Cnum rmin rmax tmin tmax))


(defmethod get-sdif-data ((self sdifFile) streamNum frameT matT colNum rmin rmax tmin tmax)
  (let ((error nil) (data nil) (onecol nil)
        ptrfile
        col row val r1 r2
        (times nil))
    (if (or (and rmin rmax (> rmin rmax)) (and tmin tmax (> tmin tmax))) 
      (setf error t) 
      (setf ptrfile (dynamic-open self)))
    (if (null ptrfile) (setf error t)
        (progn
          ;(om-print "extracting SDIF data...")
          (sdif::SdifFReadGeneralHeader ptrfile)
          (sdif::SdifFReadAllASCIIChunks ptrfile)
    (loop for item in (framesdesc self)
          while (not error) do
          (if (and (= streamNum (third item)) (string-equal frameT (string (first item))) 
                   (or (not tmin) (>= (second item) tmin)))
            (if (or (not tmax) (<= (second item) tmax))
            (progn
              (loop for mat in (fifth item) do
                    (if (string-equal matT (first mat))
                      (progn
                        (sdif-read-headers ptrfile (fourth item) (fifth mat))
                        (setf row (second mat))
                        (setf col (third mat))
                        (if (and (numberp colNum) (<= col colNum))
                           (progn 
                             (om-beep-msg (format nil "Error the matrix has ~D fields" col))
                             (setf error t))
                           (progn
                            (if (and rmin (> row rmin)) (setf r1 rmin) (setf r1 0))
                            (if (and rmax (> row rmax)) (setf r2 rmax) (setf r2 (- row 1)))
                            (setf onecol nil)
                             (loop for k from 0 to (- r1 1) do (sdif::SdifFReadOneRow ptrfile))
                             (loop for k from r1 to r2 do
                                     (sdif::SdifFReadOneRow ptrfile)
                                     (cond
                                      ((numberp colNum) (setf val (sdif::SdifFCurrOneRowCol ptrfile (+ colNum 1))))
                                      ((consp colNum) (setf val (loop for n in colNum collect (sdif::SdifFCurrOneRowCol ptrfile (+ n 1)))))
                                      ((null colNum) (setf val (loop for n from 1 to col collect (sdif::SdifFCurrOneRowCol ptrfile n))))
                                      )
                                      (push val onecol)
                                  )
                            (push (reverse onecol) data)
                            (push (second item) times)
                            )
                         )
                        ))
                    ))
            (setf error t))))
    (dynamic-close self ptrfile)))
    (if (not data) (om-print (format nil "No data found with t1=~D t2=~D r1=~D r2=~D " tmin tmax rmin rmax)) 
        ;(om-print "... done")
        )
    (values (reverse data) (reverse times))
    ))


(defun sdif-read-headers (ptrfile framepos matpos)
  (sdif-set-pos ptrfile framepos)
  (sdif-get-signature ptrfile)
  (sdif::SdifFReadFrameHeader ptrfile)
  (sdif-set-pos ptrfile matpos)
  (sdif::SdifFReadMatrixHeader ptrfile))
          

(defmethod! Get-Times ((self sdifFile) sID frameType matType tmin tmax)
   :icon 639
   :indoc '("sdif file" "stream number" "frame type" "matrix type" "min time" "max time")
   :doc "Get as list of times of frames containing the matType matrix from the sID Stream of frameType frame from <self>."
   (get-matrix-times self sID frameType matType tmin tmax))


(defmethod get-matrix-times ((self sdifFile) streamNum frameT matT tmin tmax)
  (let ((error nil) (repList nil) currtime)
    (if (and tmin tmax (> tmin tmax)) (setf error t))
    (loop for item in (framesdesc self)
          while (not error) do
          (setf currtime (second item))
          (if (and (= streamNum (third item)) (string-equal frameT (string (first item))) 
                   (or (not tmin) (>= currtime tmin)) (or (not tmax) (<= currtime tmax)))
            (if (stringp matT)
              (loop for mat in (fifth item) do
                    (if (string-equal matT (first mat))
                      (push currtime repList)
                      ))
              (push currtime repList))
            )
          finally (if repList (return (reverse repList)) 
                      (progn (om-beep-msg (format nil "No data found"))(return repList))
    ))))


(defmethod! GetSdifStream ((self sdifFile) sID tmin tmax &optional frameType matType )
   :icon 639
   :indoc '("sdif file" "stream number" "min time" "max time" "frame type" "matrix type" )
   :initvals '(nil 0 nil nil nil nil)
   :doc "Returns a SdifStream object from sdiffile data "
   (get-sdif-stream self sID tmin tmax frameType matType ))

(defmethod get-sdif-stream ((self sdifFile) streamNum tmin tmax frameT matT)
   (let ((error nil) (frameslist nil) (oneframe nil) (onemat nil)
          ptrfile data rep)
     (if  (or (null self) (and tmin tmax (> tmin tmax))) 
         (setf error t) 
       (setf ptrfile (dynamic-open self)))
     (if (null ptrfile) (setf error t)
        (progn
          (print "reading SDIF data...")
          (sdif::SdifFReadGeneralHeader ptrfile)
          (sdif::SdifFReadAllASCIIChunks ptrfile)
           (loop for item in (framesdesc self)
                 while (not error) do
                   (if (and (= streamNum (third item)) (or (null frameT) (string-equal frameT (string (first item)))) 
                               (or (not tmin) (>= (second item) tmin)))
                       (if (or (not tmax) (<= (second item) tmax))
                           (progn
                              (setf oneframe (make-instance 'sdifframe 
                                                         :streamID streamNum
                                                         :signature (string (first item))
                                                         :FTime (second item)))
                              (loop for mat in (fifth item) do
                                      (if (or (null matT) (string-equal matT (first mat)))
                                          (progn
                                             (sdif-read-headers ptrfile (fourth item) (fifth mat))
                                             (setf data (mat-trans 
                                                             (loop for i = 0 then (+ i 1) while (< i (second mat)) do
                                                                     (sdif::sdiffreadonerow ptrfile)
                                                                   collect (loop for j = 0 then (+ j 1) while (< j (third mat)) 
                                                                                    collect (sdif::SdifFCurrOneRowCol ptrfile (+ j 1))))))
                                             (let ((mtype (sdif::sdiftestmatrixtype ptrfile (sdif-string-to-signature (first mat)))))
                                               (setf onemat (make-instance 'sdifmatrix :signature (first mat)))
                                               (setf onemat (cons-new-array onemat 
                                                                                             (list nil (second mat) (first mat))
                                                                                             (loop for control in data
                                                                                                   for j = 0 then (+ j 1)
                                                                                                   append (list (intern (if (sdif-null-ptr-p mtype)
                                                                                                                                     (format nil "Field ~D" j)
                                                                                                                                     (sdif-matrixtype-get-column-name mtype (+ j 1))
                                                                                                                                     ))
                                                                                                                     control
                                                                                                                     ))
                                                                                             nil nil))
                                               (push onemat (LMatrix oneframe))
                                               )
                                             )
                                        ))
                              (push oneframe frameslist)
                              )
                         (setf error t))))
           (dynamic-close self ptrfile)))
     (if (not frameslist) (print (format nil "No data found")))
     (setf rep (make-instance 'sdifstream :id streamNum :LFrames (reverse frameslist)))
     rep))



