(in-package :om)


;======================================================
;SND Process boxes 
;======================================================
; j-b 05
;===================

(defclass sound-data (simple-container)
  ((ptr :accessor ptr :initform nil :initarg :ptr)
   (pict-sound :accessor pict-sound :initform nil :initarg :pict-sound)
   (tracknum :accessor tracknum :initform 1 :initarg :tracknum)))

(defmethod Preparetoplay ((player t) (s sound-data) at &key approx port interval voice)
  (when (ptr s)
    (when *audio-player*
      (om-add-sound-to-player *audio-player* (ptr s) at (first interval) (second interval) (tracknum s))
      )))

(defmethod get-obj-dur ((self sound-data)) (round (au::GetLengthSound (ptr self)) 44.1))

(defmethod extent->ms ((self sound-data) &key (parent nil)) (round (au::GetLengthSound (ptr self)) 44.1))


(defmethod allowed-in-maq-p ((self sound-data)) t)

;;(defmethod get-editor-class ((self sound-data)) 'soundEditor)
(defmethod Class-has-editor-p ((self sound-data)) nil)



(defmethod make-pict-sound ((self sound-data))
  (let* ((snd (ptr self))
         (pixnum 5000)
         (numdat (au::GetLengthSound snd))
         (numchan (au::GetChannelsSound snd))
         (buffer-size (floor numdat pixnum))
         (buffer (om-make-pointer (* 4  buffer-size numchan) t))
         (deltax (round 256 numchan))
         (mid (round deltax 2))
         (firstpict nil)
         (sndr (au::MakeRendererSound snd))
         (res buffer-size)
         (ref buffer-size)
         )
    (om-with-cursor *om-wait-cursor*
      (setf firstpict 
            (om-record-pict *om-default-font2* (om-make-point pixnum 256) 
              (loop for i from 0 to (- numchan 1) do 
                    (om-draw-line 0 (+ (* i deltax) (round deltax 2)) pixnum (+ (* i deltax) (round deltax 2))))
                
              (au::ResetSound snd)
              ;(print (list "REF" ref))
              (loop for i from 0 to (- pixnum 1) 
                    while (= res ref)
                    do 
                    (setf res (au::ReadSound snd buffer buffer-size numchan))
                    ;(print res)
                    (loop for k from 0 to (- numchan 1) do
                          (setf pixpoint (om-read-ptr buffer (* 4 k) :single-float))
                          (setf pixpoint (round (* mid pixpoint)))
                          (om-draw-line i (+ (* k deltax) (- mid (round pixpoint numchan))) i  
                                        (+ (* k deltax) (+ mid (round pixpoint numchan))))                                         
                          )
                    )
              )))
    
    (om-free-pointer buffer)
    firstpict))


;; en attendant...
(defmethod make-pict-sound ((self sound-data))
 (let ((name (unique-pathname *om-tmpfiles-folder* "maqtemp" "wav"))
             soundobj pict)
         (save-sound-in-file (ptr self) name)
         (setf sound-obj (load-sound-file name))
         (setf pict (om-sound-get-pict sound-obj))
         ;(delete-file name)
         pict
         ))



(defmethod! get-sound-data ((self sound))
  :icon 221
  (make-instance 'sound-data 
    :ptr (au::MakeReadSound (om-path2cmdpath (om-sound-file-name self)))))

(defun save-sound-in-file (snd filename &optional (format 'aiff))
  (let ((sndr (au::MakeRendererSound (au::MakeWriteSound (om-path2cmdpath filename) snd 
                                                          (if (equal format 'aiff)
                                                            (logior au::SF_FORMAT_AIFF au::SF_FORMAT_PCM_16)
                                                            (logior au::SF_FORMAT_WAV au::SF_FORMAT_PCM_16))))))
    (setf buffer-size 512)
    ;(setf buffer (om-make-pointer (* 4 buffer-size (au::GetChannelsSound sndr)) t))
    (setf buffer (om-make-pointer (* 4 buffer-size 2) t))
    (setf res 512)
    (au::ResetSound sndr)
    (loop while (= res 512) do
          ;(setf res (au::ReadSound sndr buffer buffer-size (au::GetChannelsSound sndr)))
          (setf res (au::ReadSound sndr buffer buffer-size 2))
          )
    (om-free-pointer buffer)
    )
  (when (probe-file filename) 
    filename
    ))

(defmethod! save-sound ((self sound-data) filename &optional (format 'aiff))
  :icon 221
  (let ((file (or filename (om-choose-new-file-dialog :directory *om-outfiles-folder*))))
    (save-sound-in-file (ptr self) file format)))


;;;=========================
;;; BOXES
;;;=========================

;;;---- MIX ----

(defmethod! sound-mix ((s1 sound-data) (s2 sound-data))
  :icon 231
  (make-instance 'sound-data 
    :ptr (au::MakeMixSound (ptr s1) (ptr s2))))

(defmethod! sound-mix ((s1 sound) (s2 sound))
  :icon 231
  (sound-mix (get-sound-data s1) (get-sound-data s2)))

(defmethod! sound-mix ((s1 sound-data) (s2 sound))
  :icon 231
  (sound-mix s1 (get-sound-data s2)))

(defmethod! sound-mix ((s1 sound) (s2 sound-data))
  :icon 231
  (sound-mix (get-sound-data s1) s2))


;;;---- SILENCE ----

(defmethod! sound-silence ((dur float))
  :icon 142
  (make-instance 'sound-data 
    :ptr (au::MakeNullSound (round (* dur 44100)))))

(defmethod! sound-silence ((dur integer))
  :icon 142
  (make-instance 'sound-data 
    :ptr (au::MakeNullSound (round (* dur 44.100)))))


;;;---- SEQ ----

(defmethod! sound-seq ((s1 sound-data) (s2 sound-data) &optional (crossfade 0))
  :icon 230
  (make-instance 'sound-data 
    :ptr (au::MakeSeqSound (ptr s1) (ptr s2) (round (* crossfade 44.100)))))

(defmethod! sound-seq ((s1 sound-data) (s2 sound) &optional (crossfade 0))
  :icon 230
  (sound-seq s1 (get-sound-data s2) crossfade))

(defmethod! sound-seq ((s1 sound) (s2 sound-data) &optional (crossfade 0))
  :icon 230
  (sound-seq (get-sound-data s1) s2 crossfade))

(defmethod! sound-seq ((s1 sound) (s2 sound) &optional (crossfade 0))
  :icon 230
  (sound-seq (get-sound-data s1) (get-sound-data s2) crossfade))


;;;---- FADE ----

(defmethod! sound-fade ((s sound-data) in out)
  :icon 231
  (make-instance 'sound-data 
    :ptr (au::MakeFadeSound (ptr s) (round (* in 44.100)) (round (* out 44.100)))))

(defmethod! sound-fade ((s sound) in out)
  :icon 231
  (sound-fade (get-sound-data s) in out))


;;;---- LOOp ----

(defmethod! sound-loop ((s sound-data) n)
  :icon 231
  (make-instance 'sound-data 
    :ptr (au::MakeLoopSound (ptr s) n)))

(defmethod! sound-loop ((s sound) n)
  :icon 231
  (sound-loop (get-sound-data s) n))


;;;---- CUT ----

(defmethod! sound-cut ((s sound-data) beg end)
  :icon 231
  (make-instance 'sound-data 
    :ptr (au::MakeCutSound (ptr s) (round (* beg 44.100)) (round (* end 44.100)))))

(defmethod! sound-cut ((s sound) beg end)
  :icon 231
  (sound-cut (get-sound-data s) beg end))


;;;---- EFFECTS ----

(defmethod! sound-vol ((s sound-data) gain &optional (in 1) (out 1))
  :icon 231
  (make-instance 'sound-data 
    :ptr (au::MakeTransformSound (ptr s) 
                                 (au::AddAudioEffect (au::MakeAudioEffectList) (au::MakeVolAudioEffect (float gain))) 
                                 (round (* in 44.100)) (round (* out 44.100)))))

(defmethod! sound-vol ((s sound) gain &optional (in 1) (out 1))
  :icon 231
  (sound-vol (get-sound-data s) gain in out))


;;; TEST PLAY
(defmethod! Sound-Play ((s sound-data))
  :icon 207
  (when (ptr s)
    (om-play-one-sound (ptr s) *audio-player*))
  )


;;;---- CONVERSION ----

(defmethod objfromobjs ((self sound) (type sound-data)) 
  (objfromobjs (save-sound self nil) type)) 


;======================================================
;FAUST Effects 
;======================================================

(defvar *faust-folder* nil)
(setf *faust-folder* *om-infiles-folder*)

(defmethod! set-faust-folder ((path string))
  :icon 186
  (setf *faust-folder* path))

(defmethod! set-faust-folder ((path pathname))
  :icon 186
  (setf *faust-folder* (namestring path)))

(defmethod! set-faust-folder ((path null))
  :icon 186
  (setf *faust-folder* (namestring (om-choose-directory-dialog))))

(defmethod! faust-file ((path string) &optional (unix nil))
  :icon 186
  (let ((pa (string+ (namestring *faust-folder*) path)))
    (if unix (om-path2cmdpath pa) pa)))

(defmethod! faust-file ((path null) &optional (unix nil))
  :icon 186
  (let ((pa (namestring *faust-folder*)))
    (if unix (om-path2cmdpath pa) pa)))


(defmethod! faust-effect ((s1 sound-data) effect &optional values)
  :icon 231
  (let ((efflist (au::MakeAudioEffectList))
        (fausteffect (au::MakeFaustAudioEffect effect)))
    (if (sdif-null-ptr-p fausteffect)
      (om-beep-msg "Bad effect file!")
      (let ((nparams (au::getcontrolcount fausteffect)))
        (print "FAUST Effect")
        (print effect)
        (print (format nil "~d Params" nparams))
        (loop for i from 0 to (- nparams 1) 
              for v in values do
              (print (multiple-value-list (au::getcontrolparam fausteffect i)))
              (au::SetControlValue fausteffect i (float v))
              (print (format nil "Set to ~d" (au::GetControlValue fausteffect i)))
              )
        (setq efflist (au::AddAudioEffect efflist fausteffect))
        (make-instance 'sound-data 
          :ptr (au::MakeTransformSound (ptr s1) efflist 100 100))
        ))
    ))

(defmethod! faust-effect ((s1 sound) effect &optional values)
  :icon 231
  (faust-effect (get-sound-data s1) effect values))



;;;---- PACK -----
(defvar *sndprocpackage* (omNG-protect-object (omNG-make-new-package "Processing")))
(addPackage2Pack *sndprocpackage* *audiopackage*)
(AddGenFun2Pack '(sound-silence sound-mix sound-seq sound-fade sound-loop sound-cut save-sound sound-vol) *sndprocpackage*)


;(setf efflist (au::MakeAudioEffectList))
;(setf fx (sdif-null-ptr-p (au::MakeFaustAudioEffect (om-path2cmdpath "HD:Users:jean:Desktop:moduledir:freeverb.so")))
;(om-path2cmdpath "/Users/jean/Desktop/moduledir/freeverb.so")
;(au::getcontrolcount fx) sdifgeninit