(in-package :om)




(defvar *om-outfiles-folder* nil)
(defvar *om-tmpfiles-folder* nil)
(defvar *om-infiles-folder* nil)


(defun check-snd-folder (path)
  (unless (probe-file path)
    (create-directory path :if-exists nil))
  path)

(defun init-user-folders ()
  (setf *om-outfiles-folder* 
    (check-snd-folder (make-pathname :device (pathname-device (mypathname *current-workspace*))
                                     :directory (append (pathname-directory (mypathname *current-workspace*)) (list "out-files")))))
  (setf *om-tmpfiles-folder* 
    (check-snd-folder (make-pathname 
                       :device (pathname-device (mypathname *current-workspace*))
                       :directory (append (pathname-directory (mypathname *current-workspace*)) (list "out-files")))))
  (setf *om-infiles-folder*
        (check-snd-folder (make-pathname 
                       :device (pathname-device (mypathname *current-workspace*))
                       :directory (append (pathname-directory (mypathname *current-workspace*)) (list "in-files")))))
  )

(add-init-user-func 'init-user-folders)


;;;=== IN FILE ==

(defmethod! set-infiles-folder ((path string))
  :icon 186
  (setf *om-infiles-folder* path))

(defmethod! set-infiles-folder ((path pathname))
  :icon 186
  (setf *om-infiles-folder* (namestring path)))

(defmethod! infile ((path string) &optional (unix nil))
  :icon 186
  (let ((pa (string+ (namestring *om-infiles-folder*) path)))
    (if unix (om-path2cmdpath pa) pa)))

(defmethod! infile ((path null) &optional (unix nil))
  :icon 186
  (let ((pa (namestring *om-infiles-folder*)))
    (if unix (om-path2cmdpath pa) pa)))



;;;=== OUT FILE ==

(defmethod! set-outfiles-folder ((path string))
  :icon 186
  (setf *om-outfiles-folder* path))

(defmethod! set-outfiles-folder ((path pathname))
  :icon 186
  (setf *om-outfiles-folder* (namestring path)))

(defmethod! outfile ((path string) &optional (unix nil))
  :icon 186
  (let ((pa (string+ (namestring *om-outfiles-folder*) path)))
    (if unix (om-path2cmdpath pa) pa)))

(defmethod! outfile ((path null) &optional (unix nil))
  :icon 186
  (let ((pa (namestring *om-outfiles-folder*)))
    (if unix (om-path2cmdpath pa) pa)))


;;;=== TMP FILE ==

(defmethod! set-tmpfiles-folder ((path string))
  :icon 186
  (setf *om-tmpfiles-folder* path))

(defmethod! set-tmpfiles-folder ((path pathname))
  :icon 186
  (setf *om-tmpfiles-folder* (namestring path)))

(defmethod! tmpfile ((path string) &optional (unix nil))
  :icon 186
  (let ((pa (string+ (namestring *om-tmpfiles-folder*) path)))
    (if unix (om-path2cmdpath pa) pa)))

(defmethod! tmpfile ((path null) &optional (unix nil))
  :icon 186
  (let ((pa (namestring *om-tmpfiles-folder*)))
    (if unix (om-path2cmdpath pa) pa)))





(defvar *soundfolderspackage* (omNG-protect-object (omNG-make-new-package "Folders")))
(addPackage2Pack *soundfolderspackage* *soundpackage*)
(AddGenFun2Pack '(infile outfile tmpfile) *soundfolderspackage*)



;;;======== FROM OM2CSOUND ===========

(defun deep-mapcar/1 (fun list? &rest args)
  (labels ((map-structure (str accum)
             (cond ((null str) (reverse accum))
                   ((not (consp str))
                    (if accum (reverse (cons (apply fun str args) accum)) (apply fun str args)))
                   (t (map-structure (cdr str) (cons (map-structure (car str) ()) accum))))))
    (map-structure list? nil)))

(defun LLdecimals (list nbdec)
  "Arrondit liste de profondeur quelconque avec <nbdec> decimales"
  (let ((ndec 
         (if (> nbdec 0 ) (float (expt 10 nbdec)) (expt 10 nbdec))))
    (deep-mapcar/1 '/  
                   (deep-mapcar/1 'round list (/ 1 ndec)) ndec )))