(in-package :om)

(defvar *cc-mode* nil)

(defun SetCCMode (mode) (if (zerop mode) (setf *cc-mode* nil) (setf *cc-mode* T)))


(defclass* CrossEvent-cc (CrossEvent)
  ((nb-coeff :initform 0 :initarg :nb-coeff  :accessor nb-coeff )
   (coeffs :initform () :initarg :coeffs :accessor coeffs)
   (energy :initform 1.0 :initarg :energy :accessor energy)
    ))


(defun osc-route-bundle (bundle)
  (format *stdout* "~S~%" bundle)
  (handler-case
    (loop for item in bundle collect
          (cond
           ((stringp (car item)) 
            (cond ((string-equal (car item) "/cc") (funcall #'handle-osc-cc-evt (cdr item)))
                  ((string-equal (car item) "/mo") (funcall #'handle-osc-midi-evt (cdr item)))
                  ((string-equal (car item) "/sp") (funcall #'handle-osc-midi-stop-evt (cdr item)))
                  ((string-equal (car item) "/ph") (funcall #'handle-osc-midi-phrase (cdr item)))
                  ((string-equal (car item) "/omsend") (osc-method-invoke item))
                  ((string-equal (car item) "/lisp") (print item) 
                   (execute-message item)
                   )
                  ((string-equal (car item) "/lisp-string") (execute-message-from-string item))
                  (t (format t "~D~%" item))))
           ((consp (car item)) (osc-route-bundle item))
           (t nil))
          )
    (error (any) (print "In bundle Error trying to route OSC message") (print any) NIL))
  )


;; /cc date[ms] phrase_nb nb_coeff[7] coeff0[= nergie], coeff1, coeff2, coeff3, ...coeff6

(defun handle-osc-cc-evt (cc-data)
  (if (boundp '*osc-midi-listener*)
      (osc-cc-arrived *osc-midi-listener* cc-data)
    (format *stdout* "< ")))


(defmethod osc-cc-arrived  ((self osc-midi-listener) cc-data)
  ;(format *stdout* "~S..."  (round (first cc-data) 1000))
  ; osc internal midi event have (pitch date dur vel chan  nphrase audio-date)
  (talk self (new-midi-cc-event 60 (- (first cc-data) 2)  0 0 1 (second cc-data) (- (first cc-data) 2) (cddr cc-data)))
  (talk self (new-midi-cc-event 60 (first cc-data) 0 80 1 (second cc-data) (first cc-data) (cddr cc-data)))
  ;(incf *count*)
  ;(when (zerop (mod *count* 20)) (format *stdout* "~S :: ~S~%" *count* (round *count* 20)))
)


(defun new-midi-cc-event (pitch onset dur vel chan  phrase audio-onset cc-data)
  ; cc-data (nb-coeff c1 ..cn)
  (list pitch onset dur vel chan phrase audio-onset cc-data))

(defmacro MENbCoeff (event)
  `(1- (first (eighth ,event))))

(defmacro MECoeffs (event)
  `(cddr (eighth ,event)))

(defmacro MEEnergy (event)
  `(second (eighth ,event)))

#|
;;; this should be in a new class

(defmethod output-slice1 ((self polySlicer))
  (let ((cross (mki 'crossevent-cc)) (phrasenb (or (MEPhrase (first (running-notes self))) 0))
        (dur (- (magnet2 self) (magnet1 self))))
    (setf (duration cross) dur
          (MidiSet cross)
          (loop for note in (running-notes self)
                collect 
                (new-midi-event  
                 (if (< (MEOnset note) (magnet1 self))
                   (- (MEPitch note))
                   (MEPitch note))
                 0 
                 dur
                 (MEVel note) 
                 (MEChannel note)
                 (or (MEPhrase note) 0)
                 (or (MEAUdioOnset note) 0)))
          (startPhrase cross)
          (/= phrasenb (prev-phrase-nb self))
          (phraseNumber cross)
          phrasenb
          (nb-coeff cross) (MENbCoeff (first (running-notes self)))
          (coeffs cross) (apply #'vector (MECoeffs (first (running-notes self)))))
    (setf (prev-phrase-nb self) phrasenb)
    ;(format t "~%Slice : ~D ~D~%" (duration cross) (MidiSet cross))
    ;(analyse-slice cross)
    (when (>= dur (mindur self)) (talk self cross))
    cross))

|#


(defmethod output-slice2 ((self polySlicer) StopPlayEvent)
  (let ((cross (mki (if *cc-mode*'crossevent-cc 'crossevent)))
        (phrasenb (or (MEPhrase (first (running-notes self))) 0))
        (dur (- (magnet2 self) (magnet1 self))))
    (setf (duration cross) 
          dur
          (EndSection cross)
          (and StopPlayEvent (if (zerop (lastSliceDur self)) dur (lastSliceDur self)))
          (MidiSet cross)
          (loop for note in  (running-notes self)
                do (when (>= (MEOnset note) (magnet1 self)) (setf phrasenb (or (MEPhrase note) 0)))
                collect 
                ; hack : MEDur encodes the date of the note-off
                (new-midi-event  
                 (if (< (MEOnset note) (magnet1 self))
                   (- (MEPitch note))
                   (MEPitch note))
                 (max 0 (- (MEOnset note) (magnet1 self))) ; onset from the slice start
                 (if (zerop (MEDur note)) 
                   ; note traverses the slice
                   (min dur (- (MAgnet2 self) (MEOnset note)))
                   ; note ends in the slice
                   (min (- (magnet2 self) (magnet1 self))
                        (min (- (MEDur note) (Magnet1 self)) (- (MEDur note) (MEOnset note)))))
                 (MEVel note) 
                 (MEChannel note)
                 (or (MEPhrase note) 0)
                 (or (MEAUdioOnset note) 0)))
          (startPhrase cross)
          (/= phrasenb (prev-phrase-nb self))
          (phraseNumber cross)
          phrasenb
          (prev-phrase-nb self) phrasenb)

    (when (typep cross 'crossevent-cc)
      (setf (nb-coeff cross) (MENbCoeff (first (running-notes self)))
            (coeffs cross) (apply #'vector (MECoeffs (first (running-notes self))))
            (energy cross) (MEEnergy (first (running-notes self)))))
    
    ; notes that come from before the slice must have an audio onset aligned
    ; with the note that begins the slice (the local onset 0)
    (loop for evt in  (MidiSet cross)
          with audioOnset = (CrossEvtAudioOnset cross)
          do (setf (MEAudioOnset evt) (max audioOnset (MEAudioOnset evt))))
    (when (>= dur (mindur self)) (talk self cross))
    cross))



#|
(defmethod CompareEvents ((Event1 CrossEvent-cc) (Event2 CrossEvent-cc))
  (< (/ (print (euclidean-dist (coeffs event1) (coeffs event2) (nb-coeff event1)))
        (nb-coeff event1))
     1.0))


(defmethod CompareEvents ((Event1 CrossEvent-cc) (Event2 CrossEvent-cc))
  (< (/  (euclidean-dist (coeffs event1) (coeffs event2) (nb-coeff event1))
        (nb-coeff event1))
     0.5))

|#

(defmethod CompareEvents ((Event1 CrossEvent-cc) (Event2 CrossEvent-cc))
  (zerop (loop for i from 0 to (1- (nb-coeff event1)) count (/= (elt (coeffs event1) i) (elt (coeffs event2) i)))))




#|
(defun euclidean-dist (e1 e2 l)
  (sqrt (loop for i from 0 to (1- l) sum (expt (- (elt e2 i) (elt e1 i)) 2))))

(defvar *SLength-distribution* #(1 2 3 25 50 80 130 210 340 550 900 1450 2000))

|#




#|
(defclass improviser (agent)
  ( (oracle :accessor oracle :initarg :oracle)
    (continuity :initform 0 :initarg :continuity :accessor continuity)
    (max-continuity :initform 8 :initarg :max-continuity :accessor max-continuity)
    (start-play :initform nil :accessor start-play :initarg :start-play )
    (stop :initform nil :accessor stop :initarg :stop)
    (gindex :initform 0  :accessor gindex :initarg :gindex :type integer)
    (durseg :initform 0  :accessor durseg :initarg :durseg :type integer)
    (blocmode :initform nil  :accessor blocmode :initarg :blocmode)t
    (startOnPhrase :initform nil  :accessor startOnPhrase :initarg :startOnPhrase)
    (blocStartPhrase :initform -1  :accessor blocStartPhrase :initarg :blocStartPhrase)
    
    ; en index sur l'oracle
    (CurRegion :initform (list 1 1) :accessor CurRegion :initarg :CurRegion)
    
    ; en pourcentage de la longueur de l'oracle
    (CurRegion% :initform (list 0 100) :accessor CurRegion% :initarg :CurRegion%) 
    (AbsoluteRegions :initform NIL :accessor AbsoluteRegions :initarg :AbsoluteRegions)

    (PoursuiteMode :initform NIL :accessor PoursuiteMode :initarg :PoursuiteMode)

    (Poursuite :initform 5000 :accessor Poursuite :initarg :Poursuite)

    (*impro-tabou-states* :initform NIL :accessor  *impro-tabou-states* :initarg :*impro-tabou-states*)
    (*suffix-quality-thresh* :initform 0.25 :accessor *suffix-quality-thresh* :initarg :*suffix-quality-thresh*)
    (*min-suffix-length* :initform *min-suffix-length* :accessor *min-suffix-length* :initarg :*min-suffix-length*)
    (*Tabou-list-length* :initform 25 :accessor *Tabou-list-length* :initarg :*Tabou-list-length*)
    (*proba-if-1-sol* :initform 0.5 :accessor *proba-if-1-sol* :initarg :*proba-if-1-sol*)
    (*Close-edge-dist* :initform 15 :accessor *Close-edge-dist* :initarg :*Close-edge-dist*)
    
    (*regional-impro* :initform NIL :accessor *regional-impro* :initarg :*regional-impro*)


    ;;; Recombination Candisate search

    (SLTEscapeTag :initform (list 'SLTEscapeTag)  :accessor SLTEscapeTag :initarg :SLTEscapeTag )

    ; solution list temp data

    (recsolutions :initform (cons nil nil)  :accessor recsolutions :initarg :recsolutions )
    (maxRecSolNb :initform 25  :accessor maxRecSolNb :initarg :maxRecSolNb )
    (recSolNb :initform 0  :accessor recSolNb :initarg :recSolNb )
    (RecSolImproved :initform nil  :accessor RecSolImproved :initarg :RecSolImproved )
    (MinSolRlength :initform 1000  :accessor MinSolRlength :initarg :MinSolRlength )

    ; Search critical state

    (RecCritical :initform nil  :accessor RecCritical :initarg :RecCritical )

    ; limitate 1-length suffix search
    (onecounter :initform 0  :accessor onecounter :initarg :onecounter )
    (maxonecount :initform 100  :accessor maxonecount :initarg :maxonecount )

    ; keep track of time take for slt exploration
    (exploretime :initform 0  :accessor exploretime :initarg :exploretime )


    )
  )
|#


#|
(defclass PolySlicer (agent)
 ((name :accessor name :initarg :name)
 (magnet1 :initform 0 :accessor magnet1 :initarg :magnet1 :type integer)
 (magnet2 :initform 0 :accessor magnet2 :initarg :magnet2 :type integer)
 (delta-t :initform 10 :accessor delta-t :initarg :delta-t :type integer)
 (running-notes :initform nil :accessor running-notes :initarg :running-notes :type list)
 (complete-notes :initform nil :accessor complete-notes :initarg :complete-notes :type list)
 (mindur :initform 10 :accessor mindur :initarg :mindur :type integer)
 (prev-phrase-nb :initform -1 :accessor prev-phrase-nb :initarg :prev-phrase-nb :type integer)
 (last-offset :initform 0 :accessor last-offset :initarg :last-offset :type integer)
 (lastSliceDur :initform 0 :accessor lastSliceDur :initarg :lastSliceDur )
 ))
|#


(defun new-PolySlicer (name)
  (if (not  *cc-mode*)
    (make-instance 'PolySlicer :name name)
    (progn (print "Instantiate Slicer in Spectral Mode")
           (make-instance 'PolySlicer :name name :delta-t 10 :mindur 10))))
    




(defun OMaxSetQuality (quality)
  ; 1 excellent 2 moyen (defaut) 3 low
  (let ((self (a-improvizer 0)))
    (setf (*impro-tabou-states* self) nil)
    (if (not  *cc-mode*)
      (case quality
        (1 
         (setf (*suffix-quality-thresh* self) 0.205)
         (setf (*min-suffix-length* self) 4)
         (setf (*Tabou-list-length* self) 8)
         (setf (*proba-if-1-sol* self) 0.5) )
        (2
         (setf (*suffix-quality-thresh* self) 0.25)
         (setf (*min-suffix-length* self) 3)
         (setf (*Tabou-list-length* self) 8)
         (setf (*proba-if-1-sol* self) 0.5) )
        (3
         (setf (*suffix-quality-thresh* self) 0.35)
         (setf (*min-suffix-length* self) 2)
         (setf (*Tabou-list-length* self) 8)
         (setf (*proba-if-1-sol* self) 0.75) )
        (4
         (setf (*suffix-quality-thresh* self) 0.5)
         (setf (*min-suffix-length* self) 1)
         (setf (*Tabou-list-length* self) 5)
         (setf (*proba-if-1-sol* self) 0.99) ))
      
      (case quality
        (1 
         (setf (*suffix-quality-thresh* self) 0.205)
         (setf (*min-suffix-length* self) 5)
         (setf (*Tabou-list-length* self) 10)
         (setf (*proba-if-1-sol* self) 0.5) )
        (2
         (setf (*suffix-quality-thresh* self) 0.25)
         (setf (*min-suffix-length* self) 4)
         (setf (*Tabou-list-length* self) 10)
         (setf (*proba-if-1-sol* self) 0.5) )
        (3
         (setf (*suffix-quality-thresh* self) 0.35)
         (setf (*min-suffix-length* self) 3)
         (setf (*Tabou-list-length* self) 10)
         (setf (*proba-if-1-sol* self) 0.75) )
        (4
         (setf (*suffix-quality-thresh* self) 0.5)
         (setf (*min-suffix-length* self) 1)
         (setf (*Tabou-list-length* self) 10)
         (setf (*proba-if-1-sol* self) 0.99) )
        
        ))))

