
(in-package :om)

;--------------------------------------------------------------------------------

(defclass* improvizer (pythie)
   (
    (context :initform () :initarg :context :accessor context)
    (continuity :initform 0 :initarg :continuity :accessor continuity)
    (max-continuity :initform 8 :initarg :max-continuity :accessor max-continuity)
    (start-region :initform '(0 100) :initarg :start-region :accessor start-region)
    (fwsuffix :initform T :initarg :fwsuffix :accessor fwsuffix)
    (bwsuffix :initform T :initarg :bwsuffix :accessor bwsuffix)
    (bestSuffixMode :initform T :initarg :bestSuffixMode :accessor bestSuffixMode)
    (useEstrada :initform nil :initarg :useEstrada :accessor useEstrada)
    (useHindemith :initform nil :initarg :useHindemith :accessor useHindemith)
    (RefHarmGrid :initform nil :initarg :RefHarmGrid :accessor RefHarmGrid)
    (HarmGridLength :initform 48 :initarg :HarmGridLength :accessor HarmGridLength)
    (Beats/Measure :initform 4 :initarg :Beats/Measure :accessor Beats/Measure)
    (RefTempo :initform 536 :initarg :RefTempo :accessor RefTempo) ; in ms
    ))


(defclass* event ()
  (
   (MidiSet :initform () :initarg :MidiSet :accessor MidiSet)
   (duration :initform 500 :initarg :duration :accessor duration)
   (data :initform nil :initarg :data :accessor data)
   ))

(defclass* beat (event)
  (
   (HarmLabel :initform () :initarg :HarmLabel :accessor HarmLabel)
   (RelHarmLabel :initform () :initarg :RelHarmLabel :accessor RelHarmLabel)
   (NumBeat :initform 1 :initarg :NumBeat :accessor NumBeat) ; in the measure
   (RefHarmLabel :initform 1 :initarg :RefHarmLabel :accessor RefHarmLabel) ; label before substitution
   (StartPhrase :initform () :initarg :StartPhrase :accessor StartPhrase) ; booleen
   (QMidiSet :initform () :initarg :QMidiSet :accessor QMidiSet) ; quantized midiset
   (Qdivision :initform 1 :initarg :Qdivision :accessor Qdivision)
   (Density :initform 1 :initarg :Density :accessor Density)
   ))


(defclass* CrossEvent (event)
  ((StartPhrase :initform () :initarg :StartPhrase :accessor StartPhrase)
   (EndPhrase :initform () :initarg :EndPhrase :accessor EndPhrase)
   (phraseNumber :initform  0 :initarg :phraseNumber :accessor phraseNumber)
   (virFun :initform  nil :initarg :virFun :accessor virFun)
   (endSection :initform  nil :initarg :endSection :accessor endSection)   
   ))



;--------------------------------------------------------------------------------

(defmethod set-start-region ((self improvizer) (region list))
  (setf (start-region self) region))

(defmethod set-suffix-law ((self improvizer) (suffixes list))
  (setf (bwsuffix self) (first suffixes)
        (fwsuffix self) (second suffixes)))
        
(defmethod set-best-suffix-mode ((self improvizer) (value symbol))
  (setf (bestSuffixMode self) value))

(defmethod set-Estrada-Hind ((self improvizer) (value list))
  (setf (useEstrada self) (first value)
        (useHindemith self) (second value)))



;--------------------------------------------------------------------------------


(defmacro MEPitch (event)
  `(first ,event))

(defmacro MEOnset (event)
  `(second ,event))

(defmacro MEDur (event)
  `(third ,event))

(defmacro MEVel (event)
  `(fourth ,event))

(defmacro MEChannel (event)
  `(fifth ,event))

;--------------------------------------------------------------------------------

(defvar *current-improvizer*)


(defun NewImprovizer ()
   (let ((improvizer (make-instance 'Improvizer
                       :comparateur 'CompareEvents
                       :lrsMode t)))
     improvizer))
            

(defmethod NbEvent? ((self improvizer))
  (maxetat self))

(defmethod SetmaxCont ((self improvizer) (cont integer))
  (setf (continuity self) 0)
  (setf (max-continuity self) cont))



;--------------------------------------------------------------------------------
;;; APPRENTISSAGE



(defmethod learn-event ((self improvizer) (event event))
   (setf *current-improvizer* self)
   (ajouter-objet self event)
   )

(defmethod learn-event-list ((self improvizer) (events list))
   (setf *current-improvizer* self)
   (loop for event in events 
         do (ajouter-objet self event) )   
   self)

(defmethod learn-beat-list ((self improvizer) (beats list) (StartBeatNb integer))
  ;(print StartBeatNb)
  (decf StartBeatNb)
  (let ((beat-list (make-beat-list beats)))
    (when beat-list (setf (StartPhrase (first beat-list)) T))
    (loop for beat in beat-list  do 
          (setf (Numbeat beat) (mod StartBeatNb (Beats/Measure self))
                (RefHarmLabel beat) (nth (mod StartBeatNb (HarmGridLength self)) (RefHarmGrid self))
                (duration beat) (RefTempo self) )
          (set-qMidiSet beat)
          (setf (density beat) (beat-density beat))
          (learn-event self beat)
          (incf StartBeatNb))  
    self))

(defmethod learn-beats-objects ((self improvizer) (beats list) (StartBeatNb integer))
  (print StartBeatNb)
  (decf StartBeatNb)
  (let ((beat-list beats))
    (when beat-list (setf (StartPhrase (first beat-list)) T))
    (loop for beat in beat-list  do 
          (setf (Numbeat beat) (mod StartBeatNb (Beats/Measure self))
                (RefHarmLabel beat) (nth (mod StartBeatNb (HarmGridLength self)) (RefHarmGrid self))
                (duration beat) (RefTempo self) )
          (set-qMidiSet beat)
          (setf (density beat) (beat-density beat))
          (learn-event self beat)
          (incf StartBeatNb))  
    self))


(defmethod Oraclize ( (events list) &optional (estrada nil) (hindemith nil))
   (let ((self (NewImprovizer)))
     (setf (useEstrada self) estrada
           (useHindemith self) hindemith)
     (loop for event in events do (learn-event self event))
     self))


(defmethod CompareEvents ((Event1 T) (event2 T))
  (equal event1 event2))

(defmethod CompareEvents ((Event1 beat) (event2 beat))
  (and ;(not (empty-beat? event1))
   ;(not (empty-beat? event2))
   ;(= (density event1) (density event2))
   (equal (RefHarmlabel event1) (RefHarmlabel event2)) ))

(defmethod CompareEvents ((Event1 CrossEvent) (Event2 CrossEvent))
  (or (and (= (length (MidiSet event1)) (length (MidiSet event2)))
           (loop for ME1 in (MidiSet event1)
                 for ME2 in (MidiSet event2)
                 always (= (mod (abs (MEPitch ME1)) 12) (mod (abs (MEPitch ME2)) 12))))
      (and 
       (> (length (MidiSet event1)) 2)
       (> (length (MidiSet event2)) 2)
       (or (useEstrada *current-improvizer*) (useHindemith *current-improvizer*))
       (let ((ch1 (mapcar #'(lambda (e) (abs (first e)))  (MidiSet event1)))
             (ch2 (mapcar #'(lambda (e) (abs (first e)))  (MidiSet event2))))
         (or
          (and (useEstrada *current-improvizer*)
               (<= (dist-estrada (om* ch1 100) (om* ch2 100)) 1))
          (and (useHindemith *current-improvizer*)
               (eq (hindem  ch1) (hindem  ch2))))))))



;--------------------------------------------------------------------------------
;;; GENERATION



(defmethod ImprovizeOnHarmGrid ((self improvizer) (length integer) (harmGrid list))
  (let* ((beatlist (Improvize self (1- length) harmgrid))
         (beat1 (first beatlist)))
    (when (or (null (Midiset beat1))
              (> (MEOnset (first (Midiset beat1))) 0))
      (setf (Midiset beat1) (cons '(60 0 100 1 11) (Midiset beat1))))
    beatlist))
    



(defmethod Improvize ((self improvizer) (length integer) &optional (harmgrid nil))
   (format t  "Region: ~a Suff. law: ~a Best Suff.: ~a Max Cont.: ~a Hindemith : ~a Estrada ~a~%" 
           (start-region self) (list (bwsuffix self) (fwsuffix self)) (bestSuffixMode self) (max-continuity self)
           (useHindemith self) (useEstrada self))

;(subseq  (oracle->beatlist self) 200 248) )
;(first-n (oracle->beatlist self) 48) )

   ;(print harmgrid)
   (setf harmgrid (refHarmGrid self))

   (when (null (veclrs self)) (setf (bestSuffixMode self) nil)
         (format t "Old Oracle, setting bestSuffix mode to nil~%"))

   (loop for i from 1 to length
         for label =  (pop harmgrid)
         with index = 0
         with previndex = -1
         with index2
         with links
         with suppleances
         with mode = 'continuity
         with tocollect = nil
         
         
         initially (setf (continuity self) 0)
         do         
         (setf previndex index)
         (cond
          ((and (zerop index) (setf index2 (find-beat-label-match self label)))
           (format t "Starting point : ~a ~%" index2)
           (setf tocollect (otext self index2)
                 index index2))
          (t (if (check-continuity self) 
               (setf mode 'continuity)  
               (if (available-suppleance self index) 
                 (setf mode 'suppleance)
                 (setf mode 'continuity)))
             (when (eq mode 'continuity)
               ;(when (= index (maxetat self)) (setf index 0)
               ;      (format t "zero~%" ))
               (setf links (flink self index)
                     index2 (choose-factor-link self links label))
               (if index2 
                 (progn
                   (format t "c : ~a ~%" index2)
                   (setf index index2
                         tocollect  (otext self index2)))
                 (if (available-suppleance self index) 
                   (setf mode 'suppleance)
                   (setf mode 'nothing))))
             (when (eq mode 'suppleance)
               (setf index2 (continuations-by-suppleance self index label))
               (if (and index2 (/= index2 previndex))
                 (progn 
                   (format t "--->s : ~a ~%" index2)
                   (setf index index2
                         tocollect (otext self index2) 
                         (continuity self) 0))
                 (print (setf mode 'nothing))))
             (when (eq mode 'nothing)
               (setf index2 (find-beat-label-match self label))
               (if index2 
                 (progn (setf tocollect (otext self index2)
                              index index2))
                 (progn               
                   (print 'empty)
                   (setf index 0
                         tocollect (null-beat)))))))
         when tocollect collect tocollect 
         ))


(defmethod available-suppleance ((self improvizer) (index integer))
  (or (and (bwsuffix self)
           (not (zerop (suppleance self index))))
      (and (fwsuffix self)
           (suppleance-> self index))))


; version avec test de continuit

#|
(defmethod ImprovizeOnHarmGrid ((self improvizer) (length integer) (harmGrid list))
  (loop for i from 1 to length
        for label in harmGrid
        with index = 0
        with index2
        with links
        with suppleances
        with mode = 'continuity
        with tocollect = nil
        
        ;for links = (flink self index)
        
        
        initially (setf (continuity self) 0)
        do 
        (cond
         ((and (zerop index) (setf index2 (find-beat-label-match self label)))
          (setf tocollect (otext self index2)
                index index2))
         (t (if (check-continuity self) (setf mode 'continuity)  (setf mode 'suppleance))
            (when (eq mode 'continuity)
              (setf links (flink self index)
                    index2 (findlabel self label links))
              (if index2 
                (progn ;(print 'cont)
                       ;(print (list index index2))
                       (setf index index2
                             tocollect  (otext self index2)))
                (setf mode 'suppleance)))
            (when (eq mode 'suppleance)
              (setf index2 (continuations-by-suppleance self index label))
              (if index2
                (progn ;(print 'supp)
                       (setf index index2
                             tocollect (otext self index2) ))
                (setf mode 'nothing)))
            (when (eq mode 'nothing)
              (setf index2 (find-beat-label-match self label))
              (if index2 
                (progn (setf tocollect (otext self index2)
                             index index2))
                (progn               
                  ;(print 'empty)
                  (setf index 0
                        tocollect (null-beat)))))))
        when tocollect collect tocollect 
        ;and do (print (list label tocollect))
        ))
|#



(defmethod ImprovizeByContinuation ((self Improvizer) (nbevents integer) (context list))
   (let ((start-etat
          (loop for suffixe on context
                for etat = (factor-p self context)
                while (null etat)
                finally return (if suffixe etat 0))))
     (Improvize self nbevents :start-etat start-etat)))
     
;--------------------------------------------------------------------------------


(defmethod check-continuity ((self improvizer))
  (if (<= (continuity self) (max-continuity self))
    (true (incf (continuity self)))
    (false (setf (continuity self) 0))))



(defun null-beat ()
  (make-instance 'beat :harmlabel 'empty :midiset nil))

(defmethod empty-beat? ((self beat))
  (null (midiset self)))

(defmethod eligible-beat? ((self beat) (label list))
  (and (not (empty-beat? self)) (or (null label) (equal label (RefHarmLabel self)))))

(defmethod eligible-beat? ((self crossevent) (label list))
  (or (null label) (equal label (RefHarmLabel self))))

(defmethod beat-density ((self beat))
  (length (remove-duplicates (mapcar #'second (qMidiSet self)))))

(setf *last-suppleance* nil)
(defmethod continuations-by-suppleance ((self improvizer) (index integer) (label list))
   (let (back-cont forw-cont (best-suffix-state -1) (max-suffix 0))
     (when (bwSuffix self)
       (setf back-cont
             (loop for supp = (suppleance self index) then (suppleance self supp)
                   with previous = index
                   with suffix-length = 0
                   while (and supp (> supp 0))
                   do 
                   ;(print (lrs self previous)) 
                   (when (bestSuffixMode self) (setf suffix-length (lrs self previous)  previous supp))
                   append (loop for cont in (flink self  supp)
                                if (and (not (memq  supp *last-suppleance*))  (eligible-beat? (otext self cont) label))
                                do (when (> suffix-length max-suffix) (setf max-suffix suffix-length))
                                and collect (list cont suffix-length supp )))))
     (when (fwSuffix self)
       (setf forw-cont 
             (loop for supp = (suppleance-> self index) then (suppleance-> self supp)
                   with suffix-length = 0
                   while supp
                   do
                   (when (bestSuffixMode self) (setf suffix-length (lrs self supp)))
                   ;(print (list '-> (lrs self supp)))
                   ;(when (> suffix-length max-suffix) (setf best-suffix-state supp max-suffix suffix-length))
                   append (loop for cont in (flink self supp)
                                if (and (not (memq  supp *last-suppleance*) ) (eligible-beat? (otext self cont) label))
                                do (when (> suffix-length max-suffix) (setf max-suffix suffix-length))
                                and collect (list cont suffix-length supp )))))
     (let ((cont (if (bestSuffixMode self) 
                   (nth-random (loop for cont in  (append back-cont forw-cont)
                                     if (= (second cont) max-suffix) collect  cont))
                   (nth-random (append back-cont forw-cont)))))
       ; dont use one of the 3 previously used suppleances
       (print (list "cont" cont))
       (unless cont (setf cont (list (max (1- (Maxetat self)) (1+ index)) 0 *last-suppleance*)))
       (push  (third cont) *last-suppleance* )
       (when (> (length *last-suppleance*) 3) (setf  (cddr *last-suppleance*) nil))
       (first cont))))


#|
(defmethod continuations-by-suppleance ((self improvizer) (index integer) (label list))
   (let (back-cont forw-cont (best-suffix-state -1) (max-suffix 0))
     (when (bwSuffix self)
       (setf back-cont
             (loop for supp = (suppleance self index) then (suppleance self supp)
                   with previous = index
                   with suffix-length = 0
                   while (and supp (> supp 0))
                   do 
                   ;(print (lrs self previous)) 
                   (when (bestSuffixMode self) (setf suffix-length (lrs self previous)  ppp previous previous supp))
                   append (loop for cont in (flink self  supp)
                                if (and (not (memq  supp *last-suppleance*))  (eligible-beat? (otext self cont) label))
                                do (when (> suffix-length max-suffix) (setf max-suffix suffix-length))
                                and collect (list cont suffix-length supp ppp)))))
     (when (fwSuffix self)
       (setf forw-cont 
             (loop for supp = (suppleance-> self index) then (suppleance-> self supp)
                   with suffix-length = 0
                   while supp
                   do
                   (when (bestSuffixMode self) (setf suffix-length (lrs self supp)))
                   ;(print (list '-> (lrs self supp)))
                   ;(when (> suffix-length max-suffix) (setf best-suffix-state supp max-suffix suffix-length))
                   append (loop for cont in (flink self supp)
                                if (and (not (memq  supp *last-suppleance*) ) (eligible-beat? (otext self cont) label))
                                do (when (> suffix-length max-suffix) (setf max-suffix suffix-length))
                                and collect (list cont suffix-length supp ppp)))))
     (let ((cont (if (bestSuffixMode self) 
                   (nth-random (loop for cont in (print (append back-cont forw-cont))
                                     if (= (second cont) max-suffix) collect  cont))
                   (nth-random (append back-cont forw-cont)))))
       ; dont use one of the 3 previously used suppleances
       (print (list "cont" cont))
       (unless cont (setf cont (list (max (1- (Maxetat self)) (1+ index)) 0 *last-suppleance*)))
       (push  (third cont) *last-suppleance* )
       (when (> (length *last-suppleance*) 3) (setf  (cddr *last-suppleance*) nil))
       (first cont))))
|#


#|
(cond
 ((bestSuffixMode self)
   (let* ((best1 (loop for cont in  back-cont
                      if (= (second cont) max-suffix) collect  cont))
         (best2 (best1 (loop for cont in  forw-cont
                      if (= (second cont) max-suffix) collect  cont)))
         (win1 (nth-random best1) )
         (win2 (nth-random best2) )
         (win (if (and win1 win2) 
                (if (zerop (random 2)) win1 win2)
                (if win1 win1 win2))))

|#     




(defmethod find-beat-label-match ((self improvizer) (label list))    
  (let* ((left  (round (* (/ (first (start-region self)) 100) (1- (maxetat self)))))
         (width (round (* (/ (- (second (start-region self)) (first (start-region self))) 100)  (1- (maxetat self)))))
         (start (max 1 (min (+ (random width) left) (1- (maxetat self))))))
    (catch
      'tag-fblm
      (when (= (maxetat self) 1) (throw 'tag-fblm 1))
      (loop for i from start to (1- (maxetat self))
            ;do (print (list i ( eligible-beat? (otext self i) label) label (RefHarmLabel (otext self i))))
            if ( eligible-beat? (otext self i) label)  
            do (throw 'tag-fblm i))
      (setf  left 1 width (1- (maxetat self))
             start (min (+ (random width) left) (1- (maxetat self))) )
      (loop for i from start to (1- (maxetat self))
            if ( eligible-beat? (otext self i) label)  
            do (throw 'tag-fblm i))
      nil)))






#|
(defmethod findlabel ((self improvizer) (label list) (indexes list))
  (let ((succeed (loop for index in indexes 
                       if (equal label (HarmLabel (otext self index))) collect index)))
    (nth-random succeed)))

|#    

(defmethod choose-factor-link ((self improvizer) (indexes list) (label list) )
  (if (null label)
    (nth-random indexes)
     (let ((succeed (loop for index in indexes 
                          if (eligible-beat? (otext self index) label)  collect index)))
       ;(setf succeed (sort succeed #'(lambda (i j) (< (length (midiset (otext self i))) (length (midiset (otext self j)))))))
       ;(first succeed) )))

       (nth-random succeed))))
    


          
(defmethod factor-p ((self improvizer) (factor list))
       (loop for etat = 0 then (transition self etat (pop factor))
             while (and factor (not (null etat)))
             finally
             (if (null etat) (return nil)  (return etat))))


(defun make-beat-list (list-of-beats)
  (mapcar 'make-beat list-of-beats))

(defun make-beat (beat-list)
  (make-instance 'beat :HarmLabel (first beat-list) :MidiSet (second beat-list)))

(defmethod oracle->beatlist ((self oracle))
  (loop for i from 1 to (maxetat self)
        collect (otext self i)))


(defun beats-check-sustain (beatlist beatvalue delta-onset)
   (loop for beat1 in beatlist
         for beat2 in (rest beatlist)
         for evtstack = nil
         do (loop for (pitch onset dur vel channel) in (Midiset beat2)
                  for event in (Midiset beat2)
                  with prefix = nil

                  do (setf prefix nil)

                  when (<= onset delta-onset)
                  do (setf prefix 
                           (some #'(lambda (evt)
                                     (and (= pitch (MEPitch  evt))
                                          (= channel (MEChannel evt))
                                          ;(>= (MEOnset evt) (- (/ beatvalue 2) delta-onset))
                                          (<= (- beatvalue (+ (MEonset evt) (MEDur evt)) ) delta-onset)
                                          evt))
                                 (reverse (MidiSet  beat1))))
                  end
                  if prefix 
                  do 
                  ;(print event)
                  ;(print (MidiSet  beat1))
                  (setf (MEdur prefix) (+ (- beatvalue (MEOnset prefix)) dur)) 
                  ;(print prefix)
                  else collect event into good-events
                  finally do 
                  ;(print (list (MidiSet beat1) (MidiSet beat2))) 
                  (setf (MidiSet beat2) good-events)))
   beatlist)


(defun beats->chseq (beatlist beatvalue deltachords)
   (let (chords 
         lonset 
         last-note 
         (beats (loop for beat in beatlist
                      for onset = 0 then (+ onset beatvalue)
                      for label = (HarmLabel beat)
                      for eventlist = (MidiSet beat)
                      append (loop for event in eventlist
                                   collect (list (first event) (+ onset (second event)) (third event) (fourth event) (1+ (fifth event)))))))

     (setf chords (make-quanti-chords beats deltachords)
         lonset (mapcar 'offset chords))
    (make-instance 'chord-seq
      :lmidic chords
      :lonset lonset )))

(defun beats->5list (beatlist beatvalue deltachords)
   (let* ((chseq (beats->chseq beatlist beatvalue deltachords)))
     (list (om/ (Lmidic  chseq) 100)
           (butlast (LOnset chseq))
           (Ldur chseq)
           (Lvel chseq)
           (om- (LChan chseq) 1))))
        
(defun events->5list (eventlist  deltachords)
   (let* ((chseq (CrossEvents->ChordSeq eventlist)))
     (list (om/ (Lmidic  chseq) 100)
           (butlast (LOnset chseq))
           (Ldur chseq)
           (Lvel chseq)
           (om- (LChan chseq) 1))))
        


(defmethod save-improvizer-as ((self improvizer))
  (catch-cancel
     (let* ((name (choose-new-file-dialog :prompt "Save Improvizer As...")))
       (when name
         (save-improvizer self name)))))

(defmethod save-improvizer ((self improvizer) (name t))
  (WITH-OPEN-FILE (out name :direction :output  :if-does-not-exist :create :if-exists :supersede
                       :external-format :|TEXT|)
    (prin1 '(in-package :om) out)
    (prin1 (omng-save  self) out))
  t)


(defmethod load-improvizer-from ()
  (catch-cancel
     (let ((name (choose-file-dialog)))
       (when name
         (load-improvizer name)))))

(defmethod load-improvizer ((name t))
  (WITH-OPEN-FILE (in name :direction :input  :if-does-not-exist :nil)
    (eval (read in)) (eval (read in))))



(defmethod omNG-save ((self array) &optional (values? nil))
   `(make-array ,(length self) :initial-contents (list ,.(map 'list #'(lambda (elt) (omNG-save elt values?)) self))))


;--------------------------------------------------------------------------------
; Cross alphabet
;---------------

(defmethod mf->crossEvents ((mf-list list))
  (cross->events  (midi->cross  mf-list nil 100 nil nil 32)))


#|
(defmethod mf->crossEvents ((mf-list list))
  (cross->events  
   (if ms::*ofon-mode*
     (midi->cross  mf-list nil 0 nil nil 0)
     (if ms::*Prolifere-mode*
       (midi->cross  mf-list nil 50 100 nil 32)
       (midi->cross  mf-list nil 50 nil nil 20)))))
|#



;;; dernier 60=tendance a quantifier (donc hacher)

(defmethod clone-object ((self CrossEvent))
  (make-instance 'CrossEvent :duration (duration self) :MidiSet (copy-tree (MidiSet  self))))


;;; deep-clone of beat object
;;; one do not wants to modify beats in the Oracle

(defmethod clone-object ((self beat))
  (let ((cbeat (clone self)))
    (setf (MidiSet cbeat) 
          (copy-tree (MidiSet  self)))
    cbeat))


;;; clone improvized beats and scale their MidiSet (onset+durs) w/regards
;;; to actual (sent by max) beat dur

(defun clone-and-scale-beats (beats dureebeat)
  (loop for beat in beats
        for cbeat = (clone-object beat)
        for beatratio = (/ dureebeat (float (duration cbeat)))
        do (loop for ms  in (MidiSet cbeat)
                 do (setf (MEOnset ms) (round (* (MEOnset ms) beatratio))
                          (MEDur ms) (round (* (MEdur ms) beatratio))))
        collect cbeat))





(defmethod cross->events ((self list) &optional (channel 1))
   (loop for slice in self
         for duration = (second slice)
         collect (make-instance 'CrossEvent 
                   :duration duration 
                   :Midiset
                   (sort (loop for track in (first slice)
                               for chan = channel then (1+ chan)
                               append (loop for (pitch velocity) in track 
                                            collect (list pitch 0 duration velocity chan)))
                         #'< :key #'(lambda (evt) (abs (first evt)))))))
  


(defmethod thread-crossEvents ((self list))
  (setf self (mapcar #'clone-object self))
  (loop for event in  self
        for delay = (duration event)
        with new-waiting-list
        for waiting-list = nil then new-waiting-list
        do (loop for ME in (midiset event)
                 with new-midiset = nil
                 initially  (setf new-waiting-list nil)
                 do (cond ((plusp (MEPitch ME))
                           (push ME new-waiting-list)
                           (push ME new-midiset))
                          ((minusp (MEPitch ME))
                           (let ((waiting (find ME waiting-list 
                                                :test #'(lambda (m1 m2) 
                                                          (and (= (abs (MEPitch m1)) (abs (MEPitch m2))) 
                                                               (= (MEChannel m1) (MEChannel m2)))))))
                             (cond 
                              (waiting
                               (setf (MEDur waiting) (+ (MEDur waiting) (MEDur ME)))
                               (push waiting new-waiting-list))
                              (t (setf (MEPitch ME) (abs (MEPitch ME)))
                                 (push ME new-waiting-list)
                                 (push ME new-midiset))))))
                 finally (setf (Midiset event) new-midiSet)))
  self)

                 
(defmethod CrossEvents->MF ((self list))
  (loop for event in self
        with delay = 0
        for onset = 0 then (+ onset delay)
        do (setf delay (duration event))
        append (loop for (pitch monset dur vel chan) in (MidiSet event)
                     collect (list pitch onset dur vel chan))))


#|
(defmethod CrossEvents->ChordSeq ((self list))
  (if ms::*ofon-mode*
    ; we take no risk of changing the number of cross-evts by threading
    (mf-info->chord-seq (crossevents->mf self))
    (mf-info->chord-seq (crossevents->mf (thread-crossevents self)))
    ))
|#

;;; ------------
;;; UTILITAIRES
;;; ------------



(defun beatlist->chordseq (beatlist beatvalue channel)
  (let ((beats (loop for beat in beatlist
                     for onset = 0 then (+ onset beatvalue)
                     for label = (first beat)
                     for eventlist = (second beat)
                     append (loop for event in eventlist
                                  if (or (eq channel t) (= channel (fifth event)))
                                  collect (list (first event) (+ onset (second event)) (third event) (fourth event) (1+ (fifth event)))))))
        (mf-info->chord-seq  beats)))


(defun filter-beatlist (beatlist channels)
  (loop for beat in beatlist
        for label = (first beat)
        for eventlist = (second beat)
        collect (list label
                      (loop for event in eventlist
                            if (member (fifth event) channels)
                            collect event))))

(defun enhance-channel (beatlist channel)
  (loop for beat in beatlist
        for label = (first beat)
        for eventlist = (second beat)
        collect (list label
                      (loop for event in eventlist
                            for velocity = (fourth event)
                            if (= (fifth event) channel) do (setf velocity (min 127 (* 3 velocity))) end
                            collect (list (first event) (second event) (third event) velocity (fifth event))))))

       
(defmethod mf-info->chord-seq ((self list))
  (let* ((chords (make-quanti-chords self *global-deltachords*))
         (lonset (mapcar 'offset chords))
         (last-note (first (inside (first (last chords))))))
    (setf lonset (append lonset (list (+ (extent->ms last-note) (first (last lonset)))))) 
    (make-instance 'chord-seq
      :lmidic chords
      :lonset lonset 
      ;:legato 200
)))



;;; bug sort chord in LZ.lisp

#|
(defun continuation (poly legatime arpegtime releastime staccatime toltime)
  (let ((continuation (quantize-voice
                       (unstaccate-voices
                        (synchro-release
                         (unarpeggiate-voices
                          (unlegate-voices
                           (apply #'thread-voices
                                  (merge-voices
                                   (channelize
                                    (prepare-voices (copy-tree poly)))));)
                           legatime)
                          arpegtime)
                         releastime)
                        staccatime)
                       toltime)))
    (mapc #'(lambda (slice) (rplaca  slice (sortchord (first slice))))
             continuation)
    continuation))
|#





#|

(setf beat-list '(((b 7) ((35 0 237 97 1) (68 0 479 69 0) (63 0 479 69 0) (61 0 479 69 0) (57 0 479 69 0) (92 44 244 100 2) (69 68 67 40 2) (64 79 81 26 2) (58 79 56 21 2) (47 242 237 97 1) (93 288 102 70 2) (92 365 109 48 2) (90 435 49 56 2))) ((f 7) ((29 0 237 97 1) (62 0 479 69 0) (57 0 479 69 0) (55 0 479 69 0) (51 0 479 69 0) (90 0 56 56 2) (87 56 220 61 2) (41 242 237 97 1) (85 315 126 62 2))) ((bb m7) ((34 0 237 97 1) (65 0 479 69 0) (61 0 479 69 0) (60 0 479 69 0) (56 0 479 69 0) (46 242 237 97 1))) ((a) ((33 0 237 97 1) (73 0 479 69 0) (68 0 479 69 0) (64 0 479 69 0) (61 0 479 69 0) (57 0 479 69 0) (63 21 67 21 2) (57 28 60 21 2) (68 32 56 14 2) (45 242 237 97 1))) ((g# m7) ((32 0 479 97 1) (63 0 484 69 0) (59 0 484 69 0) (58 0 484 69 0) (54 0 484 69 0) (80 24 265 64 2) (81 303 91 36 2) (80 372 112 25 2) (78 424 60 25 2))) ((g# m7) ((44 0 237 97 1) (58 0 479 69 0) (59 0 479 69 0) (63 0 479 69 0) (44 0 237 97 1) (78 0 50 25 2) (80 0 15 25 2) (75 39 231 30 2) (32 242 116 97 1) (73 315 108 26 2) (44 363 116 97 1))) ((e m7) ((40 0 237 97 1) (66 0 484 69 0) (62 0 484 69 0) (59 0 484 69 0) (55 0 484 69 0) (40 242 242 97 1) (75 321 125 47 2) (56 345 104 18 2) (62 351 64 22 2) (67 356 70 13 2))) ((e m7) ((40 0 237 97 1) (55 0 479 69 0) (59 0 479 69 0) (62 0 479 69 0) (66 0 479 69 0) (40 242 237 97 1) (70 307 110 28 2))) ((a 7) ((33 0 237 97 1) (66 0 479 69 0) (61 0 484 69 0) (59 0 484 69 0) (55 0 484 69 0) (61 3 129 21 2) (66 3 105 15 2) (55 13 115 20 2) (40 242 237 97 1) (68 468 16 24 2))) ((a 7) ((45 0 237 97 1) (59 0 479 69 0) (61 0 479 69 0) (45 0 237 97 1) (68 0 169 24 2) (67 121 245 69 0) (71 363 116 69 0) (68 469 15 27 2))) ((g# 7) ((32 0 237 97 1) (65 0 484 69 0) (62 0 484 69 0) (60 0 484 69 0) (58 0 484 69 0) (54 0 484 69 0) (68 0 310 27 2) (44 242 237 97 1) (70 310 125 24 2))) ((g# 7) ((32 0 237 97 1) (58 0 479 69 0) (60 0 479 69 0) (62 0 479 69 0) (65 0 479 69 0) (32 0 237 97 1) (44 242 237 97 1) (56 316 115 21 2) (51 326 115 18 2) (61 326 95 16 2))) ((f m7) ((29 0 479 97 1) (60 0 484 69 0) (56 0 484 69 0) (55 0 484 69 0) (51 0 484 69 0) (63 332 150 25 2))) ((f m7) ((41 0 237 97 1) (55 0 479 69 0) (56 0 479 69 0) (60 0 479 69 0) (41 0 237 97 1) (29 242 116 97 1) (63 298 175 29 2) (41 363 116 97 1))) ((bb 7) ((34 0 237 97 1) (67 0 484 69 0) (64 0 484 69 0) (62 0 484 69 0) (60 0 484 69 0) (56 0 484 69 0) (65 67 49 36 2) (66 162 80 35 2) (46 242 237 97 1) (67 267 45 34 2) (68 357 60 32 2) (69 447 37 60 2))) ((bb 7) ((34 0 237 97 1) (60 0 479 69 0) (62 0 479 69 0) (64 0 479 69 0) (67 0 479 69 0) (34 0 237 97 1) (69 0 27 60 2) (70 50 103 33 2) (71 163 60 59 2) (46 242 237 97 1) (72 258 80 37 2) (73 338 35 33 2) (74 398 86 46 2))) ((c m7) ((36 0 479 97 1) (62 0 479 69 0) (58 0 479 69 0) (55 0 479 69 0) (51 0 479 69 0) (74 0 7 46 2) (75 29 105 32 2) (76 144 45 41 2) (77 214 105 27 2) (75 309 100 35 2) (74 419 65 50 2))) ((b) ((35 0 237 97 1) (75 0 479 69 0) (70 0 479 69 0) (66 0 479 69 0) (63 0 479 69 0) (59 0 479 69 0) (74 0 10 50 2) (75 20 95 25 2) (76 135 60 43 2) (77 215 85 51 2) (47 242 237 97 1) (78 300 45 39 2) (79 380 95 43 2))) ((bb m7) ((34 0 237 97 1) (65 0 479 69 0) (61 0 479 69 0) (60 0 479 69 0) (56 0 479 69 0) (80 14 92 31 2) (81 106 55 45 2) (82 165 101 41 2) (46 242 237 97 1) (81 291 35 14 2) (80 301 85 35 2) (79 381 70 33 2) (80 466 18 23 2))) ((a) ((33 0 237 97 1) (73 0 479 69 0) (68 0 479 69 0) (64 0 479 69 0) (61 0 479 69 0) (57 0 479 69 0) (80 0 87 23 2) (81 117 63 23 2) (82 192 100 31 2) (45 242 237 97 1) (83 307 45 53 2) (84 387 35 41 2) (85 447 37 20 2))) ((g# m7) ((32 0 479 97 1) (63 0 484 69 0) (59 0 484 69 0) (58 0 484 69 0) (54 0 484 69 0) (85 0 18 20 2) (86 43 60 58 2) (87 158 73 22 2) (88 228 35 51 2) (89 288 80 44 2) (87 388 95 40 2))) ((g# m7) ((44 0 237 97 1) (58 0 479 69 0) (59 0 479 69 0) (63 0 479 69 0) (44 0 237 97 1) (86 0 44 35 2) (87 69 105 27 2) (88 199 35 35 2) (32 242 116 97 1) (89 279 60 32 2) (44 363 116 97 1) (90 385 23 20 2) (91 419 65 51 2))) ((c#) ((37 0 237 97 1) (68 0 479 69 0) (65 0 479 69 0) (63 0 479 69 0) (60 0 479 69 0) (91 0 15 51 2) (92 50 105 26 2) (49 242 237 97 1))) ((g) ((31 0 237 97 1) (71 0 479 69 0) (66 0 479 69 0) (62 0 479 69 0) (59 0 479 69 0) (55 0 479 69 0) (43 242 237 97 1))) ((g 7) ((31 0 237 97 1) (64 0 479 69 0) (59 0 484 69 0) (57 0 484 69 0) (53 0 484 69 0) (68 12 440 18 2) (63 36 418 16 2) (73 57 427 11 2) (38 242 237 97 1))) ((g 7) ((43 0 237 97 1) (57 0 479 69 0) (59 0 479 69 0) (43 0 237 97 1) (73 0 108 11 2) (65 121 245 69 0) (69 363 116 69 0))) ((c 7) ((36 0 237 97 1) (69 0 479 69 0) (62 0 484 69 0) (58 0 484 69 0) (57 0 484 69 0) (52 0 484 69 0) (75 20 464 11 2) (65 29 325 18 2) (70 29 363 16 2) (36 242 242 97 1))) ((c 7) ((36 0 237 97 1) (52 0 479 69 0) (57 0 479 69 0) (58 0 479 69 0) (62 0 479 69 0) (75 0 10 11 2) (64 121 228 69 0) (36 242 237 97 1) (67 363 116 69 0))) ((f 7) ((29 0 237 97 1) (62 0 484 69 0) (59 0 484 69 0) (57 0 484 69 0) (55 0 484 69 0) (51 0 484 69 0) (68 16 128 18 2) (73 26 129 20 2) (78 26 105 16 2) (41 242 237 97 1))) ((f 7) ((29 0 237 97 1) (55 0 479 69 0) (57 0 479 69 0) (59 0 479 69 0) (62 0 479 69 0) (29 0 237 97 1) (97 7 230 84 2) (41 242 237 97 1) (96 310 137 98 2))) ((bb 7) ((34 0 237 97 1) (67 0 479 69 0) (62 0 484 69 0) (60 0 484 69 0) (56 0 484 69 0) (41 242 237 97 1) (92 298 105 48 2) (73 473 11 23 2))) ((bb 7) ((46 0 237 97 1) (60 0 479 69 0) (62 0 479 69 0) (46 0 237 97 1) (63 0 59 24 2) (68 0 73 23 2) (73 0 108 23 2) (94 3 140 72 2) (68 121 245 69 0) (72 363 116 69 0))) ((b m7) ((35 0 479 97 1) (66 0 484 69 0) (62 0 484 69 0) (61 0 484 69 0) (57 0 484 69 0) (70 0 170 17 2) (90 0 158 48 2) (65 15 165 16 2) (75 19 136 9 2))) ((b m7) ((47 0 237 97 1) (61 0 479 69 0) (62 0 479 69 0) (66 0 479 69 0) (47 0 237 97 1) (87 0 206 35 2) (35 242 116 97 1) (47 363 116 97 1))) ((e 7) ((40 0 479 97 1) (70 0 479 69 0) (66 0 479 69 0) (62 0 479 69 0) (56 0 479 69 0) (85 2 265 24 2) (82 302 154 25 2) (56 302 115 20 2) (66 316 116 16 2) (61 327 70 19 2))) ((bb 7) ((34 0 237 97 1) (67 0 479 69 0) (62 0 479 69 0) (60 0 479 69 0) (56 0 479 69 0) (46 242 237 97 1) (80 293 140 24 2))) ((eb 7) ((39 0 479 97 1) (69 0 479 69 0) (65 0 479 69 0) (61 0 479 69 0) (55 0 479 69 0) (78 9 164 22 2))) ((g# m7) ((32 0 237 97 1) (63 0 479 69 0) (59 0 479 69 0) (58 0 479 69 0) (54 0 479 69 0) (75 0 210 21 2) (44 242 237 97 1) (78 465 19 18 2))) ((c# 7) ((37 0 237 97 1) (70 0 479 69 0) (63 0 484 69 0) (59 0 484 69 0) (58 0 484 69 0) (53 0 484 69 0) (78 0 284 18 2) (37 242 242 97 1) (75 284 102 18 2))) ((c# 7) ((37 0 237 97 1) (53 0 479 69 0) (58 0 479 69 0) (59 0 479 69 0) (63 0 479 69 0) (65 121 228 69 0) (37 242 237 97 1) (80 252 150 25 2) (68 363 116 69 0))) ((f#) ((30 0 237 97 1) (70 0 484 69 0) (65 0 484 69 0) (61 0 484 69 0) (58 0 484 69 0) (54 0 484 69 0) (42 242 242 97 1) (75 243 140 21 2))) ((f#) ((42 0 237 97 1) (54 0 479 69 0) (58 0 479 69 0) (61 0 479 69 0) (65 0 479 69 0) (70 0 479 69 0) (37 242 237 97 1) (78 294 140 20 2))) ((f# m7) ((30 0 479 97 1) (61 0 484 69 0) (57 0 484 69 0) (56 0 484 69 0) (52 0 484 69 0) (63 229 46 22 2) (73 229 111 20 2) (68 240 59 18 2) (75 250 188 19 2))) ((f# m7) ((42 0 237 97 1) (56 0 479 69 0) (57 0 479 69 0) (61 0 479 69 0) (42 0 237 97 1) (80 206 85 35 2) (75 221 70 18 2) (30 242 116 97 1) (70 244 57 15 2) (81 291 139 33 2) (42 363 116 97 1))) ((f m7) ((29 0 479 97 1) (60 0 484 69 0) (56 0 484 69 0) (55 0 484 69 0) (51 0 484 69 0) (80 2 210 30 2) (78 397 87 22 2))) ((f m7) ((41 0 237 97 1) (55 0 479 69 0) (56 0 479 69 0) (60 0 479 69 0) (41 0 237 97 1) (78 0 133 22 2) (68 240 103 15 2) (29 242 116 97 1) (63 248 70 18 2) (75 252 231 24 2) (41 363 116 97 1))) ((bb 7) ((34 0 237 97 1) (67 0 479 69 0) (62 0 484 69 0) (60 0 484 69 0) (56 0 484 69 0) (73 239 167 26 2) (41 242 237 97 1))) ((bb 7) ((46 0 237 97 1) (60 0 479 69 0) (62 0 479 69 0) (46 0 237 97 1) (90 50 434 70 2) (68 121 245 69 0) (72 363 116 69 0))) ((b 7) ((35 0 237 97 1) (68 0 479 69 0) (63 0 479 69 0) (61 0 479 69 0) (57 0 479 69 0) (90 0 356 70 2) (47 242 237 97 1) (87 321 115 55 2))) ((f 7) ((29 0 237 97 1) (62 0 479 69 0) (57 0 479 69 0) (55 0 479 69 0) (51 0 479 69 0) (41 242 237 97 1) (85 347 80 32 2))) ((bb 7) ((34 0 237 97 1) (67 0 479 69 0) (62 0 479 69 0) (60 0 479 69 0) (56 0 479 69 0) (46 242 237 97 1) (82 338 80 27 2))) ((eb m7) ((39 0 479 97 1) (65 0 479 69 0) (61 0 479 69 0) (58 0 479 69 0) (54 0 479 69 0) (80 109 235 14 2))) ((g# 7) ((32 0 237 97 1) (65 0 484 69 0) (62 0 484 69 0) (60 0 484 69 0) (58 0 484 69 0) (54 0 484 69 0) (80 45 185 21 2) (44 242 237 97 1))) ((g# 7) ((32 0 237 97 1) (58 0 479 69 0) (60 0 479 69 0) (62 0 479 69 0) (65 0 479 69 0) (32 0 237 97 1) (78 21 284 16 2) (44 242 237 97 1))) ((c# 7) ((37 0 237 97 1) (70 0 479 69 0) (63 0 484 69 0) (59 0 484 69 0) (58 0 484 69 0) (53 0 484 69 0) (37 242 242 97 1))) ((c# 7) ((37 0 237 97 1) (53 0 479 69 0) (58 0 479 69 0) (59 0 479 69 0) (63 0 479 69 0) (73 68 416 22 2) (65 121 228 69 0) (37 242 237 97 1) (68 363 116 69 0))) ((eb m7) ((39 0 237 97 1) (65 0 484 69 0) (61 0 484 69 0) (58 0 484 69 0) (54 0 484 69 0) (66 0 49 20 2) (61 0 69 26 2) (73 0 314 22 2) (56 14 45 16 2) (39 242 242 97 1) (75 304 93 33 2))) ((eb m7) ((39 0 237 97 1) (54 0 479 69 0) (58 0 479 69 0) (61 0 479 69 0) (65 0 479 69 0) (63 35 90 20 2) (68 49 81 18 2) (39 242 237 97 1))) ((f#) ((30 0 237 97 1) (70 0 484 69 0) (65 0 484 69 0) (61 0 484 69 0) (58 0 484 69 0) (54 0 484 69 0) (87 16 130 28 2) (42 242 242 97 1) (65 281 130 29 2) (75 281 105 20 2) (70 296 140 19 2))) ((f#) ((42 0 237 97 1) (54 0 479 69 0) (58 0 479 69 0) (61 0 479 69 0) (65 0 479 69 0) (70 0 479 69 0) (37 242 237 97 1) (82 297 45 27 2))) ((f m7) ((29 0 479 97 1) (60 0 484 69 0) (56 0 484 69 0) (55 0 484 69 0) (51 0 484 69 0) (85 278 80 30 2))) ((f m7) ((41 0 237 97 1) (55 0 479 69 0) (56 0 479 69 0) (60 0 479 69 0) (41 0 237 97 1) (63 4 80 22 2) (68 14 60 22 2) (73 29 55 15 2) (85 39 245 34 2) (29 242 116 97 1) (41 363 116 97 1))) ((bb 7) ((34 0 237 97 1) (67 0 479 69 0) (62 0 484 69 0) (60 0 484 69 0) (56 0 484 69 0) (84 50 120 33 2) (41 242 237 97 1))) ((bb 7) ((46 0 237 97 1) (60 0 479 69 0) (62 0 479 69 0) (46 0 237 97 1) (80 36 448 27 2) (68 121 245 69 0) (70 301 115 16 2) (75 301 80 12 2) (65 301 105 18 2) (72 363 116 69 0))) ((c 7) ((36 0 479 97 1) (66 0 479 69 0) (62 0 479 69 0) (58 0 479 69 0) (52 0 479 69 0) (80 0 282 27 2) (82 292 105 45 2))) ((f 7) ((29 0 237 97 1) (62 0 479 69 0) (57 0 479 69 0) (55 0 479 69 0) (51 0 479 69 0) (41 242 237 97 1) (63 287 66 28 2) (73 298 59 16 2) (68 308 49 17 2))) ((bb m7) ((34 0 237 97 1) (65 0 479 69 0) (61 0 479 69 0) (60 0 479 69 0) (56 0 479 69 0) (87 9 140 47 2) (46 242 237 97 1))) ((a) ((33 0 237 97 1) (73 0 479 69 0) (68 0 479 69 0) (64 0 479 69 0) (61 0 479 69 0) (57 0 479 69 0) (65 15 160 20 2) (70 25 165 18 2) (75 25 130 17 2) (45 242 237 97 1) (82 315 140 35 2))) ((g# m7) ((32 0 479 97 1) (63 0 484 69 0) (59 0 484 69 0) (58 0 484 69 0) (54 0 484 69 0) (85 286 115 58 2))) ((g# m7) ((44 0 237 97 1) (58 0 479 69 0) (59 0 479 69 0) (63 0 479 69 0) (44 0 237 97 1) (85 32 185 34 2) (32 242 116 97 1) (73 287 85 13 2) (68 287 60 18 2) (63 297 50 13 2) (44 363 116 97 1) (84 462 22 45 2))) ((g) ((31 0 237 97 1) (71 0 484 69 0) (66 0 484 69 0) (62 0 484 69 0) (59 0 484 69 0) (55 0 484 69 0) (84 0 118 45 2) (43 242 242 97 1))) ((g) ((43 0 237 97 1) (55 0 479 69 0) (59 0 479 69 0) (62 0 479 69 0) (66 0 479 69 0) (71 0 479 69 0) (80 0 260 27 2) (38 242 237 97 1) (78 469 15 27 2))) ((f#) ((30 0 237 97 1) (70 0 484 69 0) (65 0 484 69 0) (61 0 484 69 0) (58 0 484 69 0) (54 0 484 69 0) (78 0 160 27 2) (42 242 242 97 1) (70 310 140 17 2) (75 310 130 14 2) (65 320 105 18 2))) ((f#) ((42 0 237 97 1) (54 0 479 69 0) (58 0 479 69 0) (61 0 479 69 0) (65 0 479 69 0) (70 0 479 69 0) (37 242 237 97 1) (75 276 203 25 2))) ((f# m7) ((30 0 479 97 1) (61 0 484 69 0) (57 0 484 69 0) (56 0 484 69 0) (52 0 484 69 0) (78 262 150 50 2))) ((f# m7) ((42 0 237 97 1) (56 0 479 69 0) (57 0 479 69 0) (61 0 479 69 0) (42 0 237 97 1) (77 0 277 25 2) (30 242 116 97 1) (66 248 140 24 2) (56 253 115 23 2) (73 263 154 29 2) (61 263 105 25 2) (42 363 116 97 1))) ((f m7) ((29 0 479 97 1) (60 0 484 69 0) (56 0 484 69 0) (55 0 484 69 0) (51 0 484 69 0) (75 239 215 31 2) (68 244 153 18 2) (58 254 178 20 2) (63 258 136 21 2))) ((f m7) ((41 0 237 97 1) (55 0 479 69 0) (56 0 479 69 0) (60 0 479 69 0) (41 0 237 97 1) (29 242 116 97 1) (70 250 170 28 2) (41 363 116 97 1))) ((bb 7) ((34 0 237 97 1) (67 0 479 69 0) (62 0 484 69 0) (60 0 484 69 0) (56 0 484 69 0) (41 242 237 97 1) (73 251 140 61 2))) ((bb 7) ((46 0 237 97 1) (60 0 479 69 0) (62 0 479 69 0) (46 0 237 97 1) (72 1 241 21 2) (68 121 245 69 0) (68 257 220 28 2) (72 363 116 69 0))) ((b m7) ((35 0 237 97 1) (66 0 479 69 0) (62 0 484 69 0) (61 0 484 69 0) (57 0 479 69 0) (51 237 186 18 2) (47 242 237 97 1) (70 243 241 26 2) (56 248 150 20 2) (61 258 119 18 2))) ((e 7) ((56 0 479 69 0) (62 0 479 69 0) (34 0 237 97 1) (73 0 479 69 0) (56 0 479 69 0) (70 0 218 26 2) (63 241 173 28 2) (46 242 237 97 1))) ((e 7) ((40 0 479 97 1) (70 0 479 69 0) (66 0 479 69 0) (62 0 479 69 0) (56 0 479 69 0) (63 198 77 27 2) (66 430 54 42 2))) ((eb 7) ((39 0 479 97 1) (69 0 479 69 0) (65 0 479 69 0) (61 0 479 69 0) (55 0 479 69 0) (66 0 151 42 2) (66 421 63 22 2))) ((eb 7) ((66 0 147 22 2) (66 237 95 27 2) (51 237 60 30 2) (61 252 55 20 2) (67 367 117 23 2))) ((g# m7) ((32 0 237 97 1) (63 0 479 69 0) (59 0 479 69 0) (58 0 479 69 0) (54 0 479 69 0) (67 0 103 23 2) (68 218 235 43 2) (44 242 237 97 1))) ((g) ((31 0 237 97 1) (71 0 484 69 0) (66 0 484 69 0) (62 0 484 69 0) (59 0 484 69 0) (55 0 484 69 0) (68 119 135 34 2) (43 242 242 97 1) (68 409 75 47 2))) ((g) ((43 0 237 97 1) (55 0 479 69 0) (59 0 479 69 0) (62 0 479 69 0) (66 0 479 69 0) (71 0 479 69 0) (68 0 40 47 2) (69 50 70 48 2) (38 242 237 97 1) (68 263 221 43 2))) ((f#) ((30 0 237 97 1) (70 0 484 69 0) (65 0 484 69 0) (61 0 484 69 0) (58 0 484 69 0) (54 0 484 69 0) (68 0 484 43 2) (42 242 242 97 1))) ((f#) ((42 0 237 97 1) (54 0 479 69 0) (58 0 479 69 0) (61 0 479 69 0) (65 0 479 69 0) (70 0 479 69 0) (68 0 484 43 2) (37 242 237 97 1) (63 427 57 22 2))) ((f# m7) ((30 0 479 97 1) (61 0 484 69 0) (57 0 484 69 0) (56 0 484 69 0) (52 0 484 69 0) (63 0 63 22 2) (68 0 443 43 2) (66 433 51 28 2))) ((f# m7) ((42 0 237 97 1) (56 0 479 69 0) (57 0 479 69 0) (61 0 479 69 0) (42 0 237 97 1) (66 0 274 28 2) (30 242 116 97 1) (63 274 95 21 2) (42 363 116 97 1))) ((f# 7) ((30 0 237 97 1) (63 0 479 69 0) (58 0 479 69 0) (56 0 479 69 0) (52 0 479 69 0) (42 242 237 97 1) (51 245 80 22 2) (56 255 60 19 2) (61 255 84 16 2))))

)

(setf labels (mapcar 'car beat-list))
(setf beats (make-beat-list beat-list))
(setf charlie (NewImprovizer))
(loop for i from 0 to (1- (length beats)) do (learn-event charlie (nth i beats)))
(setf impro (ImprovizeOnHarmGrid charlie 50 labels))


(SetMaxCont charlie 2)
(play (beats->chseq impro 484 0))

(beats-check-sustain impro 484 5)

(inspect impro)
(inspect charlie)


setf impro (ImprovizeByContinuation charlie 50 (improvize charlie 5 :start-etat 3)))
(setf impro (ImprovizeOnHarmGrid charlie 150 labels))
(inspect impro)
(beats-check-sustain impro 484 5)


(play (beats->chseq impro 484 0))
(save-improvizer-as charlie)
(setf charlie (load-improvizer-from))


(play ( beats->chseq  (ImprovizeByContinuation charlie 50 (improvize charlie 5 :start-etat 3)) 484 0))
(play ( beats->chseq  (ImprovizeOnHarmGrid charlie 100 labels) 484 0))

(flink charlie 2)

(setf impro (ImprovizeByContinuation charlie 50 (improvize charlie 5 :start-etat 3)))
(setf impro (ImprovizeOnHarmGrid charlie 10 labels))
(inspect impro)
(beats-check-sustain impro 484 5)


(play (beats->chseq impro 484 0))
(save-improvizer-as charlie)
(setf charlie (load-improvizer-from))

(mapcar 'length (beats->5list  impro 484 0))

(setf l '(1 2 3 4))
(loop for i in l for j in l collect (list i j))

|#