(in-package :cl-user)


(defun set-omax-dirs ()
  (let* ((root-path (make-pathname :device (pathname-device *load-pathname*)
                                  :directory (pathname-directory *load-pathname*)))
        (om-path (make-pathname :device (pathname-device root-path)
                                :directory (append (pathname-directory root-path) (list "OM"))))
        (cclos-path (make-pathname :device (pathname-device root-path)
                                   :directory (append (pathname-directory root-path) (list "CCLOS"))))
        (omax-path (make-pathname :device (pathname-device root-path)
                                  :directory (append (pathname-directory root-path) (list "OMAX"))))
        (osc-path (make-pathname :device (pathname-device root-path)
                                 :directory (append (pathname-directory root-path) (list "OSC"))))
        (system-path (make-pathname :device (pathname-device root-path)
                                 :directory (append (pathname-directory root-path) (list "SYSTEMLIBS")))))
  (setf (logical-pathname-translations "ROOT")
        (list (list "**;*.*.*" (format () "~A**/*.*" (namestring root-path))))
        (logical-pathname-translations "OM")
        (list (list "**;*.*.*" (format () "~A**/*.*" (namestring om-path))))
        (logical-pathname-translations "CCLOS")
        (list (list "**;*.*.*" (format () "~A**/*.*" (namestring cclos-path))))
        (logical-pathname-translations "OMAX")
        (list (list "**;*.*.*" (format () "~A**/*.*" (namestring omax-path))))
        (logical-pathname-translations "OSC")
        (list (list "**;*.*.*" (format () "~A**/*.*" (namestring osc-path))))
        (logical-pathname-translations "SYSTEM")
        (list (list "**;*.*.*" (format () "~A**/*.*" (namestring system-path)))))))

;;  don't bug me while I redefine LispWorks !
(setf LISPWORKS:*HANDLE-WARN-ON-REDEFINITION* :warn)

(defmethod clos::find-method-combination ((generic-function standard-generic-function)
                                          method-combination-type
                                          method-combination-options)
  (CLOS::FIND-A-METHOD-COMBINATION-TYPE method-combination-type))






(export 'clos::find-method-combination :clos)


(set-omax-dirs)

(defvar *stdout* *standard-output*)
(export '*stdout*)


(load "system:load")
(load "om:buildimage;buildlw;build-om")
(load "cclos:load")

(defun copy-instance (instance)
  (let* ((class (class-of instance))
         (slots (class-slots class))
         (new-instance (make-instance class)))
    (loop for slot in slots do
          (setf (slot-value new-instance (slot-definition-name slot))
                (slot-value instance (slot-definition-name slot))))
    new-instance))

(defun truncate-cdr (n list value)
  (and (plusp n)
       (let ((prev (nthcdr  (1- n) list)))
         (and prev 
              (setf (cdr prev) value))
       ))
       list)

(export '(copy-instance truncate-cdr))



(shadowing-import '(:nc nc:task nc:connect nc:reset nc:disable) :om)
(use-package :nc :om)
(use-package :acl-MP :om)

(load "osc:oscoverudp.lisp")
(load "omax:load")



(defun omax-init-func (&rest dummy)
  (oa::init-listener)
  (oa::om-make-new-listener :initial-lambda #'(lambda () (in-package :om)))
  (setf  *stdout* oa::*om-stream*)
  (setf CLOS::*CHECK-MAKE-INSTANCE-INITARGS* nil)
  (setf nc::*god* (make-instance 'nc::god))
  (om::omaxon))


(defun omax-quit-func (&rest dummy)
  (when nc::*god* (nc::kill nc::*god*))
  (when (boundp 'om::*osc-listener-msg*)
    (om::omaxoff))
  (sleep 1)
  t)


(in-package :om)


;;; setting different udp port sets for different images

(defvar *osc-in-info-port* 3005)
(defvar *osc-in-midi-port* 3006)

(setf *osc-out-time-port* 3007)
(setf *osc-out-midi-port* 3008)
(setf *Bosc-out-midi-port* 3009)
(setf *osc-out-info-port* 3010)


#|
(defvar *osc-in-info-port* 3105)
(defvar *osc-in-midi-port* 3106)

(setf *osc-out-time-port* 3107)
(setf *osc-out-midi-port* 3108)
(setf *Bosc-out-midi-port* 3109)
(setf *osc-out-info-port* 3110)

(defvar *udp-out* 1915)


(defvar *osc-in-info-port* 3205)
(defvar *osc-in-midi-port* 3206)

(setf *osc-out-time-port* 3207)
(setf *osc-out-midi-port* 3208)
(setf *Bosc-out-midi-port* 3209)
(setf *osc-out-info-port* 3210)

(defvar *udp-out* 1919)
|#




(defun omaxon () 
  (setf *osc-listener-msg* (new-osc-listener  *osc-in-host* *osc-in-info-port* (format nil "osc-listener-~a" *osc-in-info-port*)))
  (setf *osc-listener-midi* (new-osc-listener *osc-in-host* *osc-in-midi-port*  (format nil "osc-listener-~a" *osc-in-midi-port*)))
  (setf *osc-time-end-point* (osc:open-osc-out-stream *osc-out-host* *osc-out-time-port*))
  (setf *osc-midi-end-point* (osc:open-osc-out-stream *osc-out-host* *osc-out-midi-port*))
  (setf *Bosc-midi-end-point* (osc:open-osc-out-stream *osc-out-host* *Bosc-out-midi-port*))
  (setf *osc-info-end-point* (osc:open-osc-out-stream *osc-out-host* *osc-out-info-port*))
  )


